Package: inTextSummaryTable
Type: Package
Title: Creation of in-Text Summary Table
Version: 3.4.0
Date: 2026-05-04
Authors@R: c(
  person(given = "Laure", family = "Cougnaud", role = c("aut", "cre"),
  email = "laure.cougnaud@openanalytics.eu"),
  person(given = "Michela", family = "Pasetto", role = "aut",
  email = "michela.pasetto@openanalytics.eu"),
  person(given = "Margaux", family = "Faes", 
  role = "rev", comment = "tests"),
  person(given = "Open Analytics", role = "cph")
  )
Description: Creation of tables of summary statistics or counts for clinical data (for 'TLFs'). 
  These tables can be exported as in-text table (with the 'flextable' package) for a Clinical Study Report 
  (Word format) or a 'topline' presentation (PowerPoint format), 
  or as interactive table (with the 'DT' package) to an html document for clinical data review.
Imports: clinUtils (>= 0.1.0), cowplot, flextable (>= 0.5.5), ggplot2,
        ggrepel, magrittr, methods, officer, plyr, reshape2 (>= 1.4),
        scales, stats, utils
Suggests: htmltools, knitr, rmarkdown, testthat, xml2, tools, tibble,
        extrafont
URL: https://github.com/openanalytics/inTextSummaryTable
BugReports: https://github.com/openanalytics/inTextSummaryTable/issues
License: MIT + file LICENSE
VignetteBuilder: knitr
SystemRequirements: pandoc (to export an interactive summary table to a
        file)
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-04 14:29:24 UTC; root
Author: Laure Cougnaud [aut, cre],
  Michela Pasetto [aut],
  Margaux Faes [rev] (tests),
  Open Analytics [cph]
Maintainer: Laure Cougnaud <laure.cougnaud@openanalytics.eu>
Repository: CRAN
Date/Publication: 2026-05-04 15:50:02 UTC
Built: R 4.6.0; ; 2026-05-04 17:23:57 UTC; unix

Package: inTextSummaryTable
Type: Package
Title: Creation of in-Text Summary Table
Version: 3.4.0
Date: 2026-05-04
Authors@R: c(
  person(given = "Laure", family = "Cougnaud", role = c("aut", "cre"),
  email = "laure.cougnaud@openanalytics.eu"),
  person(given = "Michela", family = "Pasetto", role = "aut",
  email = "michela.pasetto@openanalytics.eu"),
  person(given = "Margaux", family = "Faes", 
  role = "rev", comment = "tests"),
  person(given = "Open Analytics", role = "cph")
  )
Description: Creation of tables of summary statistics or counts for clinical data (for 'TLFs'). 
  These tables can be exported as in-text table (with the 'flextable' package) for a Clinical Study Report 
  (Word format) or a 'topline' presentation (PowerPoint format), 
  or as interactive table (with the 'DT' package) to an html document for clinical data review.
Imports: clinUtils (>= 0.1.0), cowplot, flextable (>= 0.5.5), ggplot2,
        ggrepel, magrittr, methods, officer, plyr, reshape2 (>= 1.4),
        scales, stats, utils
Suggests: htmltools, knitr, rmarkdown, testthat, xml2, tools, tibble,
        extrafont
URL: https://github.com/openanalytics/inTextSummaryTable
BugReports: https://github.com/openanalytics/inTextSummaryTable/issues
License: MIT + file LICENSE
VignetteBuilder: knitr
SystemRequirements: pandoc (to export an interactive summary table to a
        file)
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-04 14:29:24 UTC; root
Author: Laure Cougnaud [aut, cre],
  Michela Pasetto [aut],
  Margaux Faes [rev] (tests),
  Open Analytics [cph]
Maintainer: Laure Cougnaud <laure.cougnaud@openanalytics.eu>
Repository: CRAN
Date/Publication: 2026-05-04 15:50:02 UTC
Built: R 4.5.2; ; 2026-05-04 23:21:45 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(combine,summaryTable)
S3method(export,summaryTable)
export(combine)
export(combineVariables)
export(computeSummaryStatistics)
export(computeSummaryStatisticsTable)
export(convertVectToBinary)
export(createFlextableWithHeader)
export(cv)
export(export)
export(exportFlextableToDocx)
export(exportSummaryStatisticsTable)
export(formatPercentage)
export(geomCV)
export(geomMean)
export(geomSD)
export(geomSE)
export(getColorPaletteTable)
export(getDimPage)
export(getListing)
export(getMaxNDecimals)
export(getMaxNDecimalsData)
export(getNDecimals)
export(getNDecimalsData)
export(getNDecimalsRule)
export(getStats)
export(getStatsData)
export(getSummaryStatisticsTable)
export(pageDimPresentation)
export(se)
export(subjectProfileSummaryPlot)
export(subjectProfileSummaryTable)
export(tableColorsPresentation)
export(tableColorsReport)
import(cowplot)
import(flextable)
import(ggplot2)
importFrom(clinUtils,clinColors)
importFrom(clinUtils,clinLinetypes)
importFrom(clinUtils,clinShapes)
importFrom(clinUtils,getClinDT)
importFrom(clinUtils,getColorPalette)
importFrom(clinUtils,getLabelVar)
importFrom(clinUtils,getLinetypePalette)
importFrom(clinUtils,getShapePalette)
importFrom(clinUtils,roundHalfUpTextFormat)
importFrom(flextable,as_b)
importFrom(flextable,as_paragraph)
importFrom(flextable,as_sub)
importFrom(flextable,as_sup)
importFrom(flextable,body_add_flextable)
importFrom(flextable,compose)
importFrom(flextable,set_header_df)
importFrom(ggrepel,geom_text_repel)
importFrom(magrittr,"%>%")
importFrom(methods,formalArgs)
importFrom(officer,body_add_break)
importFrom(officer,body_set_default_section)
importFrom(officer,fp_border)
importFrom(officer,fp_text)
importFrom(officer,page_size)
importFrom(officer,prop_section)
importFrom(plyr,colwise)
importFrom(plyr,daply)
importFrom(plyr,ddply)
importFrom(plyr,dlply)
importFrom(plyr,rbind.fill)
importFrom(reshape2,dcast)
importFrom(reshape2,melt)
importFrom(scales,trans_new)
importFrom(stats,as.formula)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(tools,file_ext)
importFrom(tools,file_path_sans_ext)
importFrom(utils,capture.output)
importFrom(utils,hasName)
importFrom(utils,head)
importFrom(utils,packageVersion)
importFrom(utils,write.table)

## ----options, echo = FALSE----------------------------------------------------------------------------------------------------------------------------------------------
	
	library(knitr)
	opts_chunk$set(
		echo = TRUE, results = 'markup', warning = FALSE, 
		# stop document execution if error (not the default)
		error = FALSE, 
		message = FALSE, cache = FALSE,
		fig.width = 8, fig.height = 7,
		fig.path = "./figures_vignette/",
		fig.align = 'center')
	options(width = 170)
	# instead of warn = 0 by default
	# include warnings when they occur in the document
	options(warn = 1)
	

## ----loadPackages-------------------------------------------------------------------------------------------------------------------------------------------------------

	library(inTextSummaryTable)
	library(tools) # toTitleCase


## ----loadData-----------------------------------------------------------------------------------------------------------------------------------------------------------
	
	library(clinUtils)

	# load example data
    data(dataADaMCDISCP01)
	
	dataAll <- dataADaMCDISCP01
    labelVars <- attr(dataAll, "labelVars")
	

## ----data-SL------------------------------------------------------------------------------------------------------------------------------------------------------------

	dataSL <- dataAll$ADSL


## ----getHelp, eval = FALSE----------------------------------------------------------------------------------------------------------------------------------------------
# 
# ? `inTextSummaryTable-stats`
# 

## ----count-simple-------------------------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(data = dataSL)


## ----count-categories---------------------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(data = dataSL, var = "SEX")


## ----count-categories-order---------------------------------------------------------------------------------------------------------------------------------------------

	# specify manually the order of the categories
	dataSL$SEX <- factor(dataSL$SEX, levels = c("M", "F"))
	getSummaryStatisticsTable(data = dataSL, var = "SEX")
	
	# order categories based on a numeric variable
    dataSL$SEXN <- ifelse(dataSL$SEX == "M", 2, 1)
	dataSL$SEX <- reorder(dataSL$SEX, dataSL$SEXN)
	getSummaryStatisticsTable(data = dataSL, var = "SEX")
	

## ----count-categories-empty-1-------------------------------------------------------------------------------------------------------------------------------------------

	dataSLExample <- dataSL
	
	# 'SEX' formatted as character with only male
	dataSLExample$SEX <- "M" # only male
	getSummaryStatisticsTable(data = dataSLExample, var = "SEX")
	

## ----count-categories-empty-2-------------------------------------------------------------------------------------------------------------------------------------------
	
	# 'SEX' formatted as factor, to include also female in the table
	# (even if not available in the data)
	dataSLExample$SEX <- factor("M", levels = c("F", "M"))
	getSummaryStatisticsTable(data = dataSLExample, var = "SEX", varInclude0 = TRUE)
	# or:
    getSummaryStatisticsTable(data = dataSLExample, var = "SEX", varInclude0 = "SEX")


## ----flag-variables-----------------------------------------------------------------------------------------------------------------------------------------------------

    labelVars[grep("FL$", colnames(dataSL), value = TRUE)]
    # has the subject discontinued from the study?
    dataSL$DISCONFL


## ----count-flag-var-----------------------------------------------------------------------------------------------------------------------------------------------------

    getSummaryStatisticsTable(
	    data = dataSL,
	    var = "SAFFL"
    )


## ----count-flag-varFlag-------------------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataSL,
		var = "SAFFL",
		varFlag = "SAFFL"
	)


## ----count-varTotalInclude----------------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataSL, 
		var = "SEX", 
		varTotalInclude = TRUE
	)


## ----numeric------------------------------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(data = dataSL, var = "AGE")


## ----mixedTable---------------------------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataSL, 
		var = c("AGE", "SEX")
	)


## ----getHelpStats, eval = FALSE-----------------------------------------------------------------------------------------------------------------------------------------
# 
#     ? `inTextSummaryTable-stats`
# 

## ----stats-count--------------------------------------------------------------------------------------------------------------------------------------------------------

	# count: n, '%' and m
	getSummaryStatisticsTable(
		data = dataSL,
		var = "SEX",
		stats = "count"
	)

	# n (%)
	getSummaryStatisticsTable(
		data = dataSL,
		var = "SEX",
		stats = "n (%)"
	)
	
	# n/N (%)
	getSummaryStatisticsTable(
		data = dataSL,
		var = "SEX",
		stats = "n/N (%)"
	)
	

## ----stats-numeric------------------------------------------------------------------------------------------------------------------------------------------------------
	
	## continuous variable
	
	# all summary stats
	getSummaryStatisticsTable(
		data = dataSL,
		var = "AGE",
		stats = "summary"
	)
	
	# median (range)
	getSummaryStatisticsTable(
		data = dataSL,
		var = "AGE",
		stats = "median (range)"
	)
	
	# median and (range) in a different line:
	getSummaryStatisticsTable(
		data = dataSL,
		var = "AGE",
		stats = "median\n(range)"
	)
	
	# mean (se)
	getSummaryStatisticsTable(
		data = dataSL,
		var = "AGE",
		stats = "mean (se)"
	)
	
	# mean (sd)
	getSummaryStatisticsTable(
		data = dataSL,
		var = "AGE",
		stats = "mean (sd)"
	)
	
	# mean and sd in separated rows
	getSummaryStatisticsTable(
		data = dataSL,
		var = "AGE",
		stats = c("Mean", "SD")
	)

## ----getHelpStats2, eval = FALSE----------------------------------------------------------------------------------------------------------------------------------------
# 
# ? `inTextSummaryTable-stats`
# 

## ----stats-N------------------------------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataSL,
		var = c("RACE", "SEX"),
		stats = list(N = expression(statN))
	)


## ----stats-meanSE-------------------------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataSL,
		var = c("AGE", "HEIGHTBL", "WEIGHTBL", "BMIBL"),
		varGeneralLab = "Parameter", statsGeneralLab = "",
		colVar = "TRT01P",
		stats = list(
			`median` = expression(statMedian),
			`(min, max)` = expression(paste0("(", statMin, ",", statMax, ")"))
		)
	)


## ----getStatsData-------------------------------------------------------------------------------------------------------------------------------------------------------

    # this count table:
    getSummaryStatisticsTable(
	    data = dataSL,
	    var = "SEX",
	    stats = "count"
    )
    # ... is equivalent to:
    getSummaryStatisticsTable(
	    data = dataSL,
	    var = "SEX",
	    stats = getStats(type = "count")
    )

    # this summary table...
    getSummaryStatisticsTable(
	    data = dataSL,
	    var = "AGE",
	    stats = "mean (se)"
    )
    # ... is equivalent to:
    getSummaryStatisticsTable(
	    data = dataSL,
	    var = "AGE",
	    stats = getStatsData(type = "mean (se)", var = "AGE", data = dataSL)[["AGE"]]
    )

## ----statExtra-EachVariable---------------------------------------------------------------------------------------------------------------------------------------------
		
	getSummaryStatisticsTable(
		data = dataSL, 
		var = c("AGE", "RACE"),
		stats = list(
			AGE = getStats("median (range)"),
			RACE = getStats("n (%)")
		)
	)


## ----statsUtilityFct----------------------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataSL,
		var = "HEIGHTBL",
		# specify extra stats to compute
		statsExtra = list(
			statCV = cv,
			statGeomMean = geomMean,
			statGeomSD = geomSD,
			statsGeomCV = geomCV
		)
	)


## ----statsExtra---------------------------------------------------------------------------------------------------------------------------------------------------------

	# include the coefficient of variation via the 'statsExtra' parameter
	getSummaryStatisticsTable(
		data = dataSL,
		var = "HEIGHTBL",
		statsExtra = list(statCVPerc = function(x) sd(x)/mean(x)*100)
	)


## ----statsExtra-stats---------------------------------------------------------------------------------------------------------------------------------------------------

	# format the statistics with the 'stats' parameter
	getSummaryStatisticsTable(
		data = dataSL,
		var = "HEIGHTBL",
		statsExtra = list(statCVPerc = function(x) sd(x)/mean(x)*100),
		stats = list(Mean = expression(statMean), 'CV%' = expression(statCVPerc))
	)



## ----getHelpStats3, eval = FALSE----------------------------------------------------------------------------------------------------------------------------------------
# 
#     ? `inTextSummaryTable-stats`
# 

## ----nDecimals-catVar, echo = FALSE, fig.cap = "Standard Layout for Frequency Tabulations of Categorical Variables<br>"-------------------------------------------------

include_graphics("./images/nDecimals_catVar.png")


## ----getStats-count-----------------------------------------------------------------------------------------------------------------------------------------------------

	# Internal rule for the number of decimals for the percentage
	formatPercentage(c(NA, 0, 100, 99.95, 0.012, 34.768))
	
	# Used by default in the 'getStats' function
	getStats(type = "count")


## ----nDecimals-numVar, echo = FALSE, fig.cap = "Standard Layout for Descriptive Statistics of Continuous Variables<br>"-------------------------------------------------

include_graphics("./images/nDecimals_numVar.png")


## ----getMaxNDecimals----------------------------------------------------------------------------------------------------------------------------------------------------

	# Duration of Disease (Months)
	print(dataSL$DURDIS)
	
	## Extract the number of decimals for each value:
	
	# based on pre-defined rule, this metric should be displayed with 1 decimal:
	getNDecimalsRule(x = dataSL$DURDIS)
	
	# but available in the data only with 0 decimals
	getNDecimalsData(x = dataSL$DURDIS)
	
	# The minimum of the #decimals based on the data and pre-defined rule is:
	getNDecimals(x = dataSL$DURDIS)
	
	## Take the maximum number of decimals 
	getMaxNDecimals(x = dataSL$DURDIS)
	
	## Custom set of statistics are extracted when x is specified:
	getStats(x = dataSL$DURDIS)
	
	# To fix the number of decimals:
	getStats(type = "summary", nDecCont = 1)
	
	## Create summary statistics table
	getSummaryStatisticsTable(
		data = dataSL,
		var = c("AGE", "DURDIS"),
		stats = list(
			AGE = getStats(type = "median (range)", x = dataSL$AGE),
            DURDIS = getStats(type = "median (range)", x = dataSL$DURDIS)
		)
	)


## ----stats-digits-------------------------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataSL,
		var = c("AGE", "HEIGHTBL"),
		stats = list(
			AGE = list(Median = expression(roundHalfUpTextFormat(statMedian, 1))),
            HEIGHTBL = list(Median = expression(roundHalfUpTextFormat(statMedian, 2)))
		)
	)


## ----stats-digits-complex-----------------------------------------------------------------------------------------------------------------------------------------------

	# wrapper function to include median with specific number of digits
	# and min/max with specified number of digits - 1
	statsDMNum <- function(digitsMin)
		list('Median (range)' = 
			bquote(paste0(
				roundHalfUpTextFormat(statMedian, .(digitsMin+1)), 
				" (", roundHalfUpTextFormat(statMin, .(digitsMin)), ",", 
				roundHalfUpTextFormat(statMax, .(digitsMin)),
				")"
			))
	)

	getSummaryStatisticsTable(
		data = dataSL,
		var = c("AGE", "HEIGHTBL", "WEIGHTBL", "BMIBL", "RACE", "SEX"),
		stats = list(
			AGE = statsDMNum(0),
			HEIGHTBL = statsDMNum(1),
			WEIGHTBL = statsDMNum(1),
			BMIBL = statsDMNum(1),
			RACE = getStats("n (%)"),
			SEX = getStats("n (%)")
		)
	)


## ----statsLayoutRow-----------------------------------------------------------------------------------------------------------------------------------------------------

	# statsLayout = 'row'
	getSummaryStatisticsTable(
		data = dataSL,
		var = c("AGE", "HEIGHTBL"),
		stats = list(Mean = expression(statMean), 'SE' = expression(statSE))
	)
	

## ----statsLayoutCol-----------------------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataSL,
		var = c("AGE", "HEIGHTBL"),
		stats = list(Mean = expression(statMean), 'SE' = expression(statSE)),
		statsLayout = "col"
	)


## ----summaryTable-PP-medianMinMax-statsLayoutRowVarInSepCol-------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataSL,
		var = c("AGE", "HEIGHTBL"),
		stats = list(Mean = expression(statMean), 'SE' = expression(statSE)),
		statsLayout = "rowInSepCol"
	)


## ----summaryTable-statsLayout-onlyOneStat-------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataSL,
		var = c("AGE", "HEIGHTBL"),
		stats = list(Mean = expression(statMean))
	)


## ----summaryTable-statsLayout-onlyOneStat-statsLabInclude---------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataSL,
		var = c("AGE", "HEIGHTBL"),
		stats = list(Mean = expression(statMean)),
		statsLabInclude = TRUE
	)


## ----countTable-AE-data-------------------------------------------------------------------------------------------------------------------------------------------------
				
	dataAE <-  subset(dataAll$ADAE, SAFFL == "Y" & TRTEMFL == "Y")
	
	# ensure that order of elements is the one specified in 
	# the corresponding numeric variable
	dataAE$TRTA <- with(dataAE, reorder(TRTA, TRTAN))
	dataAE$AESEV <- factor(
		dataAE$AESEV, 
		levels = c("MILD", "MODERATE", "SEVERE")
	)
	
	dataAEInterest <- subset(dataAE, AESOC %in% c(
		"INFECTIONS AND INFESTATIONS",
        "GENERAL DISORDERS AND ADMINISTRATION SITE CONDITIONS"
       )
    )


## ----rowVarColVar-------------------------------------------------------------------------------------------------------------------------------------------------------

	# unique row variable
	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = "AEDECOD",
		stats = getStats("n (%)"),
		labelVars = labelVars
	)
	
	# multiple nested row variables
	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		stats = getStats("n (%)"),
		labelVars = labelVars
	)
	
	# unique column variable
	getSummaryStatisticsTable(
		data = dataAEInterest,
		colVar = "TRTA",
		stats = getStats("n (%)"),
		labelVars = labelVars
	)
	
	# combination of rows and columns
	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		stats = getStats("n (%)"),
		labelVars = labelVars,
		colHeaderTotalInclude = FALSE
	)


## ----rowVarInSepCol-----------------------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD", "AESEV"),
		rowVarInSepCol = "AESEV",
		colVar = "TRTA",
		stats = getStats("n (%)"),
		labelVars = labelVars
	)
	

## ----rowOrder-common----------------------------------------------------------------------------------------------------------------------------------------------------

	# 'auto':

	# set order of SOC to reverse alphabetical order
	dataAEInterest$AESOC <- factor(
		dataAEInterest$AESOC, 
		levels = rev(sort(unique(as.character(dataAEInterest$AESOC))))
	)
	# AEDECOD is not a factor -> sort alphabetically by default
	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarLab = labelVars[c("AEDECOD")],
		rowVarTotalInclude = c("AESOC", "AEDECOD"),
		colVar = "TRTA", colTotalInclude = TRUE,
		stats = getStats("n (%)"),
		labelVars = labelVars
	)
	
	# total counts
	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarLab = labelVars[c("AEDECOD")],
		rowVarTotalInclude = c("AESOC", "AEDECOD"),
		colVar = "TRTA", colTotalInclude = TRUE, colTotalLab = "Number of subjects",
		rowOrder = "total",
		stats = getStats("n (%)"),
		labelVars = labelVars
	)
	# same order even if the 'total' column is not specified
	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarLab = labelVars[c("AEDECOD")],
		rowVarTotalInclude = c("AESOC", "AEDECOD"),
		colVar = "TRTA", 
		rowOrder = "total", 
		stats = getStats("n (%)"),
		labelVars = labelVars
	)


## ----rowOrder-specific--------------------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarLab = labelVars[c("AEDECOD")],
		rowVarTotalInclude = c("AESOC", "AEDECOD"),
		colVar = "TRTA", #colTotalInclude = TRUE,
		rowOrder = c(AESOC = "alphabetical", AEDECOD = "total"),
		stats = getStats("n (%)"),
		labelVars = labelVars
	)


## ----rowOrder-rowOrderTotalFilterFct------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarLab = labelVars[c("AEDECOD")],
		rowVarTotalInclude = c("AESOC", "AEDECOD"),
		colVar = "TRTA", colTotalInclude = TRUE,
		rowOrder = "total",
		stats = getStats("n (%)"),
		labelVars = labelVars,
		# consider only the counts of the treated patients to order the rows
		rowOrderTotalFilterFct = function(x) subset(x, TRTA == "Xanomeline High Dose")
	)


## ----rowOrder-functionExample1------------------------------------------------------------------------------------------------------------------------------------------

	library(plyr)
	getSummaryStatisticsTable(
		data = dataAEInterest,
		type = "count",
		rowVar = "AEHLT",
		rowOrder = function(x){
			x <- subset(x, !isTotal)
			totalAcrossTreatments <- subset(x, TRTA == "Total")
			# counts across treated patients
			totalForTreatmentOnly <- subset(x, TRTA == "Xanomeline High Dose")
			dataCounts <- merge(totalAcrossTreatments, totalForTreatmentOnly, by = "AEHLT", suffixes = c(".all", ".treat"))
			# sort first based on overall count, then counts of treated patients
			dataCounts[with(dataCounts, order(`statN.all`, `statN.treat`, decreasing = TRUE)), "AEHLT"]
		},
		colVar = "TRTA", colTotalInclude = TRUE,
		labelVars = labelVars,
		title = "Table: Adverse Events ordered based on total counts",
		stats = list(expression(paste0(statN, " (", round(statPercN, 1), ")"))),
		footer = "Statistics: n (%)"
	)


## ----rowOrder-functionExample2------------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarLab = labelVars[c("AEDECOD")],
		rowVarTotalInclude = c("AESOC", "AEDECOD"),
		colVar = "TRTA", colTotalInclude = TRUE,
		rowOrder = list(
			AESOC = function(table) {
				# records with total for each AESOC
				nAESOCPlacebo <- subset(table, !isTotal & grepl("placebo", TRTA) & AEDECOD == "Total")
				nAESOCTreat <- subset(table, !isTotal & grepl("High Dose", TRTA) & AEDECOD == "Total")
				nAESOCDf <- merge(nAESOCPlacebo, nAESOCTreat, by = "AESOC", suffixes = c(".placebo", ".treatment"))
				nAESOCDf[with(nAESOCDf, order(`statN.placebo`, `statN.treatment`, decreasing = TRUE)), "AESOC"]
			},
			AEDECOD = function(table) {
				# records with counts for each AEDECOD
				nAEDECODPlacebo <- subset(table, !isTotal & grepl("placebo", TRTA) & AEDECOD != "Total")
				nAEDECODTreat <- subset(table, !isTotal & grepl("High Dose", TRTA) & AEDECOD != "Total")
				nAEDECODDf <- merge(nAEDECODPlacebo, nAEDECODTreat, by = "AEDECOD", suffixes = c(".placebo", ".treatment"))
				nAEDECODDf[with(nAEDECODDf, order(`statN.placebo`, `statN.treatment`, decreasing = TRUE)), "AEDECOD"]
			}
		),
		stats = getStats("n (%)"),
		labelVars = labelVars
	)


## ----summaryTable-PP-rowVarWithLabel-labelVars--------------------------------------------------------------------------------------------------------------------------

	# combination of rows and columns
	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		stats = getStats("n (%)"),
		labelVars = labelVars
	)
	

## ----summaryTable-PP-rowVarWithLabel-rowVarLab--------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		stats = getStats("n (%)"),
		rowVarLab = c(
			'AESOC' = "TEAE by SOC and Preferred Term\nn (%)"
		),
		labelVars = labelVars
	)
	

## ----countTable-emptyVars-----------------------------------------------------------------------------------------------------------------------------------------------

	## only consider a subset of adverse events
	dataAESubset <- subset(dataAE, AEHLT == "HLT_0617")
	
	## create dummy categories for:
	# treatment
	dataAESubset$TRTA <- with(dataAESubset, 
		factor(TRTA, levels = c(unique(as.character(TRTA)), "Treatment B"))
	)
	# low-level term category
	dataAESubset$AELLT <- with(dataAESubset, 
		factor(AELLT, levels = c(unique(as.character(AELLT)), "Lymphocyte percentage increased"))
	)
	
	# create summary statistics table
	getSummaryStatisticsTable(
		data = dataAESubset,
		type = "count",
		rowVar = c("AEHLT", "AELLT"),
		rowInclude0 = TRUE, colInclude0 = TRUE,
		colVar = "TRTA",
		labelVars = labelVars,
		title = "Table: Adverse Events: white blood cell analyses",
		stats = getStats("n (%)"),
		footer = "Statistics: n (%)"
	)


## ----summaryTable-layout-var--------------------------------------------------------------------------------------------------------------------------------------------

    dataDIABP <- subset(dataAll$ADVS, 
		SAFFL == "Y" & ANL01FL == "Y" &
		PARAMCD == "DIABP" & 
		AVISIT %in% c("Baseline", "Week 8") &
		ATPT == "AFTER LYING DOWN FOR 5 MINUTES"
	)
    dataDIABP$TRTA <- reorder(dataDIABP$TRTA, dataDIABP$TRTAN)
    dataDIABP$AVISIT <- reorder(dataDIABP$AVISIT, dataDIABP$AVISITN)
    
	getSummaryStatisticsTable(
		data = dataDIABP,
		var = c("AVAL", "CHG"),
		colVar = "TRTA",
		rowVar = "AVISIT",
		labelVars = labelVars,
		stats = getStats("summary-default")
	)


## ----summaryTable-layout-varInColumn------------------------------------------------------------------------------------------------------------------------------------
    
getSummaryStatisticsTable(
	data = dataDIABP,
	var = c("AVAL", "CHG"),
	colVar = c("variable", "TRTA"),
	rowVar = "AVISIT",
	labelVars = labelVars,
	stats = getStats("summary-default")
)


## ----summaryTable-layout-var-oneLabel-default---------------------------------------------------------------------------------------------------------------------------
		
	getSummaryStatisticsTable(data = dataSL, var = "AGE", colVar = "TRT01P")


## ----summaryTable-layout-var-oneLabel-varLabInclude---------------------------------------------------------------------------------------------------------------------
		
	getSummaryStatisticsTable(
		data = dataSL, 
		var = "AGE", 
		varLabInclude = TRUE,
		colVar = "TRT01P"
	)


## ----summaryTable-layout-var-empty--------------------------------------------------------------------------------------------------------------------------------------

    dataAEInterest$AESEVN <- ifelse(dataAEInterest$AESEV == "MILD", 1, 2)
	dataAEInterestWC <- ddply(dataAEInterest, c("AEDECOD", "USUBJID", "TRTA"), function(x) {
		x[which.max(x$AESEVN), ]
	})
	dataAEInterestWC[1, "AESEV"] <- NA
    getSummaryStatisticsTable(
        data = dataAEInterestWC,
        colVar = "TRTA",
        rowVar = "AEBODSYS",
		stats = getStats("n (%)"),
        var = c("AESEV", "all"),
		labelVars = labelVars
    )


## ----header-total-default-----------------------------------------------------------------------------------------------------------------------------------------------
	
	# by default, total number of subjects extracted from data
	getSummaryStatisticsTable(
		data = subset(dataAEInterest, AESOC == "INFECTIONS AND INFESTATIONS"),
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		stats = getStats("n (%)"),
		rowVarLab = c(
			'AESOC' = "TEAE by SOC and Preferred Term\nn (%)"
		),
		labelVars = labelVars
	)
	

## ----header-total-dataTotal---------------------------------------------------------------------------------------------------------------------------------------------
	
	# dataset used to extract the 'Total'
	dataTotalAE <- subset(dataAll$ADSL, SAFFL == "Y")
	# should contain columns specified in 'colVar'
	dataTotalAE$TRTA <- dataTotalAE$TRT01A 

	getSummaryStatisticsTable(
		data = subset(dataAEInterest, AESOC == "INFECTIONS AND INFESTATIONS"),
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		stats = getStats("n (%)"),
		rowVarLab = c(
			'AESOC' = "TEAE by SOC and Preferred Term\nn (%)"
		),
		dataTotal = dataTotalAE,
		labelVars = labelVars
	)


## ----header-total-colHeaderTotalInclude---------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = subset(dataAEInterest, AESOC == "INFECTIONS AND INFESTATIONS"),
		rowVar = c("AESOC", "AEDECOD"),
		rowVarTotalInclude = "AEDECOD",
		rowVarTotalInSepRow = "AEDECOD",
		colVar = "TRTA",
		stats = getStats("n (%)"),
		rowVarLab = c(
			'AESOC' = "TEAE by SOC and Preferred Term\nn (%)"
		),
		dataTotal = dataTotalAE,
		labelVars = labelVars,
		colHeaderTotalInclude = FALSE
	)


## ----dataTotalPerc------------------------------------------------------------------------------------------------------------------------------------------------------

    getSummaryStatisticsTable(
	    data = subset(dataAEInterest, AESOC == "INFECTIONS AND INFESTATIONS"),
	    rowVar = c("AESOC", "AEDECOD"),
	    colVar = "TRTA",
	    stats = getStats("n (%)"),
	    rowVarLab = c(
		    'AESOC' = "TEAE by SOC and Preferred Term\nn (%)"
	    ),
	    dataTotalPerc = dataTotalAE,
	    labelVars = labelVars
    )


## ----rowVarTotalPerc----------------------------------------------------------------------------------------------------------------------------------------------------

	dataLB <- subset(dataAll$ADLBC, 
		SAFFL == "Y" & 
		PARAMCD %in% c("K", "CHOL") &
		grepl("(Baseline)|(Week 20)", AVISIT)
	)
	dataLB$AVISIT <- with(dataLB, reorder(trimws(AVISIT), AVISITN))
	
	# counts versus the total per actual treatment arm
	getSummaryStatisticsTable(
		data = dataLB,
		colVar = "TRTA", 
		rowVar = c("PARAM", "AVISIT"), 
		var = "LBNRIND",
		stats = getStats("n (%)"),
		rowAutoMerge = FALSE, emptyValue = "0",
	)
	
	# percentage based on total number of subjects with available
	# measurement at specific visit for each parameter
	getSummaryStatisticsTable(
		data = dataLB,
		colVar = "TRTA", 
		rowVar = c("PARAM", "AVISIT"), 
		rowVarTotalPerc = c("PARAM", "AVISIT"),
		var = "LBNRIND",
		stats = getStats("n (%)"),
		rowAutoMerge = FALSE, emptyValue = "0",
	)	
	

## ----statsPerc----------------------------------------------------------------------------------------------------------------------------------------------------------

getSummaryStatisticsTable(
	data = dataLB,
	colVar = "TRTA", 
	rowVar = c("PARAM", "AVISIT"), 
	rowVarTotalPerc = c("PARAM", "AVISIT"),
	var = "LBNRIND", 
	stats = getStats("m (%)"),
	statsPerc = "statm",
	rowAutoMerge = FALSE, emptyValue = "0",
)


## ----colTotalInclude----------------------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		colTotalInclude = TRUE, 
		stats = getStats("n (%)"),
		rowVarLab = c(
			'AESOC' = "TEAE by SOC and Preferred Term\nn (%)"
		),
		dataTotal = dataTotalAE,
		labelVars = labelVars
	)


## ----colTotalLab--------------------------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		colTotalInclude = TRUE, colTotalLab = "All subjects",
		stats = getStats("n (%)"),
		rowVarLab = c(
			'AESOC' = "TEAE by SOC and Preferred Term\nn (%)"
		),
		dataTotal = dataTotalAE,
		labelVars = labelVars
	)


## ----dataTotalCol-------------------------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = subset(dataAEInterest, grepl("High Dose", TRTA)),
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		colTotalInclude = TRUE, colTotalLab = "Placebo and treatment arm",
		dataTotalCol = dataAEInterest,
		stats = getStats("n (%)"), emptyValue = "0",
		rowVarLab = c(
			'AESOC' = "TEAE by SOC and Preferred Term\nn (%)"
		),
		dataTotal = dataTotalAE,
		labelVars = labelVars
	)


## ----rowVarTotalInclude-------------------------------------------------------------------------------------------------------------------------------------------------

	# total reported across AESOC
	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarTotalInclude = "AESOC", 
		colVar = "TRTA",
		stats = getStats("n (%)"),
		labelVars = labelVars
	)
	
	# total reported across AESOC and across AEDECOD
	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarTotalInclude = c("AESOC", "AEDECOD"), 
		colVar = "TRTA",
		stats = getStats("n (%)"),
		labelVars = labelVars
	)


## ----rowTotalLab--------------------------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarTotalInclude = "AESOC", rowTotalLab = "Any AE", 
		colVar = "TRTA",
		stats = getStats("n (%)"),
		labelVars = labelVars
	)


## ----rowVarTotalInclude-rowVarTotalInSepRow-----------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarTotalInclude = "AEDECOD",
		rowVarTotalInSepRow = "AEDECOD",
		colVar = "TRTA",
		stats = getStats("n (%)"),
		labelVars = labelVars
	)


## ----dataTotalRow-------------------------------------------------------------------------------------------------------------------------------------------------------

	dataAEInterest$AESEVN <- as.numeric(dataAEInterest$AESEV)
	
	# compute worst-case scenario per subject*AE term*treatment
	dataAEInterestWC <- ddply(dataAEInterest, c("AESOC", "AEDECOD", "USUBJID", "TRTA"), function(x){
		x[which.max(x$AESEVN), ]
	})

	## datasets used for the total: 
	# for total: compute worst-case across SOC and across AE term
	# (otherwise patient counted in multiple categories if present different categories for different AEs)
	dataTotalRow <- list(
		# within visit (across AEDECOD)
		'AEDECOD' = ddply(dataAEInterest, c("AESOC", "USUBJID", "TRTA"), function(x){	
			x[which.max(x$AESEVN), ]
		}),
		# across visits
		'AESOC' = ddply(dataAEInterest, c("USUBJID", "TRTA"), function(x){	
			x[which.max(x$AESEVN), ]
		})
	)

	getSummaryStatisticsTable(
		data = dataAEInterestWC,
		## row variables:
		rowVar = c("AESOC", "AEDECOD", "AESEV"), 
		rowVarInSepCol = "AESEV",
		# total for column header and denominator
		dataTotal = dataTotalAE, 
		# include total across SOC and across AEDECOD
		rowVarTotalInclude = c("AESOC", "AEDECOD"), 
		# data for total row
		dataTotalRow = dataTotalRow, 
		# count for each severity category for the total
		rowVarTotalByVar = "AESEV", 
		rowTotalLab = "Any TEAE", 
		rowVarLab = c(AESOC = "Subjects with, n(%):", AESEV = "Worst-case scenario"),
		# sort per total in the total column
		rowOrder = "total", 
		## column variables
		colVar = "TRTA", 
		stats = getStats("n (%)"),
		emptyValue = "0",
		labelVars = labelVars
	)


## ----titleAndFooter-----------------------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		stats = getStats("n (%)"),
		dataTotal = dataTotalAE,
		labelVars = labelVars,
		title = toTitleCase("MOR106-CL-102: Adverse Events by System Organ Class and Preferred Term (Safety Analysis Set, Part 1)"),
		footer = c(
			"N=number of subjects with data; n=number of subjects with this observation",
			"Denominator for percentage calculations = the total number of subjects per treatment group in the safety population"
		)
	)


## ----includeSessionInfo, echo = FALSE, results = "asis"-----------------------------------------------------------------------------------------------------------------
print(sessionInfo())

context("Compute summary statistics table with totals")

test_that("The totals are correctly extracted from the data by default", {
			
	# data with different number of subjects/records per treatment
	data <- data.frame(
		USUBJID = c(1, 1, 2, 3, 4, 4, 5),
		TRT = rep(c("A", "B"), times = c(4, 3)),
		ADECOD = c("a", "b", "a", "c", "d", "a", "b"),
		stringsAsFactors = FALSE
	)
	
	sumTable <- computeSummaryStatisticsTable(
		data = data,
		var = "ADECOD",
		colVar = "TRT"
	)
	# by default data total extracted from the dataset directly
	sumTableTotal <- subset(sumTable, isTotal)
	sumTableTotal <- sumTableTotal[match(c("A", "B"), sumTableTotal$TRT), ]
	
	# # subjects/records for total is correct
	expect_equal(sumTableTotal$statN, c(3, 2))
	expect_equal(sumTableTotal$statm, c(4, 3))
	
	# number of subjects in 'total' same as total used for percentage by default
	expect_equal(
		sumTable$statPercTotalN, 
		sumTableTotal[match(sumTable$TRT, sumTableTotal$TRT), "statN"]
	)
			
})

test_that("The totals are correctly extracted from a specified dataset", {
			
	# data with different number of subjects/records per treatment
	data <- data.frame(
		USUBJID = c(1, 1, 2, 3, 4, 4, 5),
		TRT = rep(c("A", "B"), times = c(4, 3)),
		ADECOD = c("a", "b", "a", "c", "d", "a", "b"),
		stringsAsFactors = FALSE
	)
	
	## correct spec
	
	# data with different number of subjects/records
	dataTotal <- data.frame(
		TRT = rep(c("A", "B"), c(6, 4)),
		USUBJID = c(c(1, seq.int(5)), c(1, seq.int(3)))
	)
			
	sumTable <- computeSummaryStatisticsTable(
		data = data,
		var = "ADECOD",
		colVar = "TRT",
		dataTotal = dataTotal
	)
			
	# data total extracted from a specified dataset directly
	sumTableTotal <- subset(sumTable, isTotal)
	sumTableTotal <- sumTableTotal[match(c("A", "B"), sumTableTotal$TRT), ]
			
	# # subjects/records for total is correct
	expect_equal(sumTableTotal$statN, c(5, 3))
	expect_equal(sumTableTotal$statm, c(6, 4))
			
	# number of subjects in 'total' same as total used for percentage by default
	expect_equal(
		sumTable$statPercTotalN, 
		sumTableTotal[match(sumTable$TRT, sumTableTotal$TRT), "statN"]
	)
	
})

test_that("Warnings are generated if the column variable is not available in the dataset for totals", {
			
	# data with different number of subjects/records per treatment
	data <- data.frame(
		USUBJID = c(1, 1, 2, 3, 4, 4, 5),
		TRT = rep(c("A", "B"), times = c(4, 3)),
		ADECOD = c("a", "b", "a", "c", "d", "a", "b"),
		stringsAsFactors = FALSE
	)

	dataTotal <- data.frame(
		USUBJID = c(c(1, seq.int(5)), c(1, seq.int(3)))
	)
	w <- capture_warnings(
		sumTable <- computeSummaryStatisticsTable(
			data = data,
			var = "ADECOD",
			colVar = "TRT",
			dataTotal = dataTotal
		)
	)
	expect_match(w, ".*colVarTotal are ignored because they are not available in: total dataset.", all = FALSE)
	expect_match(w, ".*colVarTotalPerc are ignored because they are not available in: total dataset for percentage.", all = FALSE)
	expect_equal(sum(sumTable$isTotal), 1) # total computed across columns
	
})

test_that("The percentages are correctly extracted from a specified dataset", {

	# data with different number of subjects/records per treatment
	data <- data.frame(
		USUBJID = c(1, 1, 2, 3, 4, 4, 5),
		TRT = rep(c("A", "B"), times = c(4, 3)),
		ADECOD = c("a", "b", "a", "c", "d", "a", "b"),
		stringsAsFactors = FALSE
	)	
	
	dataTotalPerc <- data.frame(
		TRT = rep(c("A", "B"), c(6, 4)),
		USUBJID = c(c(1, seq.int(5)), c(1, seq.int(3)))
	)
	
	sumTable <- computeSummaryStatisticsTable(
		data = data,
		var = "ADECOD",
		colVar = "TRT",
		dataTotalPerc = dataTotalPerc
	)
	
	# percentage is computed from a specified dataset
	expect_true(all(subset(sumTable, TRT == "A")$statPercTotalN == 5))
	expect_true(all(subset(sumTable, TRT == "B")$statPercTotalN == 3))
	
	expect_equal(sumTable$statPercN, with(sumTable, statN/statPercTotalN)*100)
	
	# data total is still extracted from the dataset directly
	sumTableTotal <- subset(sumTable, isTotal)
	sumTableTotal <- sumTableTotal[match(c("A", "B"), sumTableTotal$TRT), ]
	
	# # subjects/records for total is correct
	expect_equal(sumTableTotal$statN, c(3, 2))
	expect_equal(sumTableTotal$statm, c(4, 3))
	
})

test_that("A warning is generated if the column variable for percentages is not available in the dataset for percentages", {
			
	data <- data.frame(
		USUBJID = c(1, 1, 2, 3, 4, 4, 5),
		TRT = rep(c("A", "B"), times = c(4, 3)),
		ADECOD = c("a", "b", "a", "c", "d", "a", "b"),
		stringsAsFactors = FALSE
	)

	dataTotalPerc <- data.frame(
		USUBJID = c(c(1, seq.int(5)), c(1, seq.int(3)))
	)
	expect_warning(
		sumTable <- computeSummaryStatisticsTable(
			data = data,
			var = "ADECOD",
			colVar = "TRT",
			dataTotalPerc = dataTotalPerc
		),
		".*colVarTotalPerc are ignored because they are not available in: total dataset for percentage."
	)
})

test_that("The row totals are correctly extracted from a specified dataset", {
			
	# data
	data <- data.frame(
		USUBJID = c(1, 2, 4, 5),
		TRT = c("A", "A", "B", "B"),
		ABODSYS = c("AB", "AB", "CD", "AB"),
		ADECOD = c("a", "b", "d", "a")
	)
			
	# row total dataset: 
	# add one subject for TRT: B - ABODSYS: CD
	dataTotalRow <- data.frame(
		USUBJID = c(1, 2, 4, 5, 6),
		TRT = c("A", "A", "B", "B", "B"),
		ABODSYS = c("AB", "AB", "CD", "CD", "AB"),
		ADECOD = c("a", "b", "d", "e", "a")
	)
	
	# summary table
	summaryTable <- computeSummaryStatisticsTable(
		data = data,
		colVar = "TRT",
		rowVar = c("ABODSYS", "ADECOD"),
		rowVarTotalInclude = c("ABODSYS", "ADECOD"),
		dataTotalRow = dataTotalRow
	)
	
	# row totals across AEBODYS (by column)
	expect_equal(
		object = subset(summaryTable, 
			subset = (ADECOD == "Total" & ABODSYS == "Total"), 
			select = c("TRT", "statN", "statm"),
		),
		expected = subset(
			computeSummaryStatisticsTable(
				data = dataTotalRow,
				colVar = "TRT"
			),
			subset = !isTotal,
			select = c("TRT", "statN", "statm")
		),
		check.attributes = FALSE
	)
	
	# row totals across AEDECOD (by column)	
	expect_equal(
		object = subset(summaryTable, 
			subset = (ADECOD == "Total" & ABODSYS != "Total"), 
			select = c("TRT", "ABODSYS", "statN", "statm")
		),
		expected = subset(
			computeSummaryStatisticsTable(
				data = dataTotalRow,
				colVar = "TRT",
				rowVar = "ABODSYS"
			),
			subset = !isTotal,
			select = c("TRT", "ABODSYS", "statN", "statm")
		),
		check.attributes = FALSE
	)
	
})

test_that("An error is generated if the row or column variables are not available in the dataset for percentages", {
	
	# data with different number of subjects/records per treatment
	data <- data.frame(
		USUBJID = c(1, 1, 2, 3, 4, 4, 5),
		TRT = rep(c("A", "B"), times = c(4, 3)),
		ABODSYS = c("AB", "AB", "AB", "CD", "CD", "AB", "AB"),
		ADECOD = c("a", "b", "a", "c", "d", "a", "b"),
		stringsAsFactors = FALSE
	)
			
	# variables (columns, rows) missing in dataTotalRow
	expect_error(
		computeSummaryStatisticsTable(
			data = data,
			colVar = "TRT",
			rowVar = c("ABODSYS", "ADECOD"),
			rowVarTotalInclude = c("ABODSYS", "ADECOD"),
			dataTotalRow = data[, "USUBJID", drop = FALSE]
		),
		"*not available.*dataset for row total"
	)
	
})

test_that("An error is generated if the dataset for percentages is specified only for a subset of the row variables", {
			
	# data with different number of subjects/records per treatment
	data <- data.frame(
		USUBJID = c(1, 1, 2, 3, 4, 4, 5),
		TRT = rep(c("A", "B"), times = c(4, 3)),
		ABODSYS = c("AB", "AB", "AB", "CD", "CD", "AB", "AB"),
		ADECOD = c("a", "b", "a", "c", "d", "a", "b"),
		stringsAsFactors = FALSE
	)
			
	
	# dataset specified as a list but not for all variables
	expect_error(
		computeSummaryStatisticsTable(
			data = data,
			colVar = "TRT",
			rowVar = c("ABODSYS", "ADECOD"),
			rowVarTotalInclude = c("ABODSYS", "ADECOD"),
			dataTotalRow = list(ABODSYS = data)
		),
		"Dataset for row total missing for:.*ADECOD.*"
	)
	
})

## ----options, echo = FALSE----------------------------------------------------------------------------------------------------------------------------------------------

library(knitr)
opts_chunk$set(
    echo = TRUE, 
    results = 'markup', warning = FALSE, 
    # stop document execution if error (not the default)
    error = FALSE, 
    message = FALSE, cache = FALSE,
    fig.width = 8, fig.height = 7,
    fig.path = "./figures_vignette/",
    fig.align = 'center')
options(width = 170)
# instead of warn = 0 by default
# include warnings when they occur in the document
options(warn = 1)


## ----loadPackages-------------------------------------------------------------------------------------------------------------------------------------------------------
library(inTextSummaryTable)
library(clinUtils)

## ----loadData-----------------------------------------------------------------------------------------------------------------------------------------------------------

# load example data
data(dataADaMCDISCP01)

dataAll <- dataADaMCDISCP01
labelVars <- attr(dataAll, "labelVars")


## ----visualization-extractData------------------------------------------------------------------------------------------------------------------------------------------

dataLB <- subset(dataAll$ADLBC, grepl("Baseline|Week", AVISIT))
dataLB$AVISIT <- with(dataLB, reorder(trimws(AVISIT), AVISITN))
dataLB$TRTA <- with(dataLB, reorder(TRTA, TRTAN))

summaryTableDf <- computeSummaryStatisticsTable(
    data = dataLB,
    var = "AVAL",
    rowVar = c("PARCAT1", "PARAM"),
    colVar = c("TRTA", "AVISIT")
)


## ----visualization-a4document-------------------------------------------------------------------------------------------------------------------------------------------

# create the plot
dataPlot <- subset(
    summaryTableDf, 
    !isTotal &
     PARAM == "Alanine Aminotransferase (U/L)"
)

subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    colorVar = "TRTA",
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)


## ----visualization-presentation, eval = requireNamespace("extrafont", quietly = TRUE) && "Tahoma" %in% extrafont::fonts()-----------------------------------------------
# # 'Tahoma' font should be registered upfront to create plots with: 'presentation' style
# subjectProfileSummaryPlot(
#     data = dataPlot,
#     xVar = "AVISIT",
#     colorVar = "TRTA",
#     labelVars = labelVars,
#     useLinetype = TRUE,
#     tableText = "statN",
#     style = "presentation"
# )
# 

## ----getVignetteAesthetics, eval = FALSE--------------------------------------------------------------------------------------------------------------------------------
# 
# vignette("inTextSummaryTable-aesthetics", "inTextSummaryTable")
# 

## ----getVignetteClinUtils, eval = FALSE---------------------------------------------------------------------------------------------------------------------------------
# 
# vignette("clinUtils-vignette", "clinUtils")
# 

## ----visualization-customColorFromArgument------------------------------------------------------------------------------------------------------------------------------

# custom color palette by setting a named vector of colors
customColorPalette <- c(
    `Xanomeline Low Dose` = "green",
    `Xanomeline High Dose` = "red",
	`Placebo` = "blue"
)
subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    colorVar = "TRTA",
    colorPalette = customColorPalette,
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)


## ----visualization-customShapeFromArgument------------------------------------------------------------------------------------------------------------------------------

# custom shape palette
customShapePalette <- c(15, 17, 19)
subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    colorVar = "TRTA",
    shapePalette = customShapePalette,
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)

# custom linetype palette
customLinetypePalette <- c("twodash", "dotted", "longdash")
subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    colorVar = "TRTA",
    linetypePalette = customLinetypePalette,
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)


## ----exampleLoadingPackageAndOptions, eval = FALSE----------------------------------------------------------------------------------------------------------------------
# 
# # this is OK
# library(inTextSummaryTable)
# options(inTextSummaryTable.colors.plot = customColorPalette)
# 
# # this does NOT set your custom palette
# options(inTextSummaryTable.colors.plot = customColorPalette)
# library(inTextSummaryTable)
# 

## ----visualization-customColorFromOptions-------------------------------------------------------------------------------------------------------------------------------

# custom color palette by setting a simple vector of colors
customColorPalette <- c("blue", "green", "orange")
options(inTextSummaryTable.colors.plot = customColorPalette)

subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    colorVar = "TRTA",
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)

# custom color palette by setting a named vector of colors
customColorPalette <- c(
    `Xanomeline Low Dose` = "green",
    `Xanomeline High Dose` = "red",
	`Placebo` = "purple"
)
options(inTextSummaryTable.colors.plot = customColorPalette)

subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    colorVar = "TRTA",
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)


## ----visualization-customShapeFromOptions-------------------------------------------------------------------------------------------------------------------------------

# custom shape palette
customShapePalette <- c(17, 19, 22)
options(inTextSummaryTable.shapes.plot = customShapePalette)

subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    colorVar = "TRTA",
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)

# custom linetype palette
customLinetypePalette <- c("dotted", "longdash", "solid")
options(inTextSummaryTable.linetypes.plot = customLinetypePalette)

subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    colorVar = "TRTA",
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)


## ----visualization-backDefaultPalettes----------------------------------------------------------------------------------------------------------------------------------

options(inTextSummaryTable.colors.plot = clinColors)
options(inTextSummaryTable.shapes.plot = clinShapes)
options(inTextSummaryTable.linetypes.plot = clinLinetypes)


## ----visualization-table------------------------------------------------------------------------------------------------------------------------------------------------

subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    colorVar = "TRTA",
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = NULL
)


## ----visualization-facets, out.width = "100%", fig.height = 9, fig.width = 9--------------------------------------------------------------------------------------------

# create the plot
dataPlotFacets <- subset(
    summaryTableDf, 
    !isTotal &
        PARAM %in% c(
            "Alanine Aminotransferase (U/L)",
            "Albumin (g/L)",
            "Bilirubin (umol/L)",
            "Calcium (mmol/L)"
        )
)

subjectProfileSummaryPlot(
    data = dataPlotFacets,
    xVar = "AVISIT",
    colorVar = "TRTA",
    labelVars = labelVars,
    facetVar = c("PARCAT1", "PARAM"),
    useLinetype = TRUE
)


## ----visualization-otherInputs------------------------------------------------------------------------------------------------------------------------------------------

# plot the median instead of mean
# no error bars
subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    meanVar = "statMedian",
    seVar = NULL,
    colorVar = "TRTA",
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)

# plot the mean with standard deviation
subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    seVar = "statSD",
    colorVar = "TRTA",
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)

# change labels
subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    xLab = "Time points",
    yLab = "Mean and Standard Errors",
    title = "Title of the plot",
    colorVar = "TRTA",
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)


## ----includeSessionInfo, echo = FALSE, results = "asis"-----------------------------------------------------------------------------------------------------------------
print(sessionInfo())

## ----options, echo = FALSE----------------------------------------------------------------------------------------------------------------------------------------------

library(knitr)
opts_chunk$set(
    echo = TRUE, results = 'markup', warning = FALSE, 
    # stop document execution if error (not the default)
    error = FALSE, 
    message = FALSE, cache = FALSE,
    fig.width = 8, fig.height = 7,
    fig.path = "./figures_vignette/",
    fig.align = 'center')
options(width = 170)
# instead of warn = 0 by default
# include warnings when they occur in the document
options(warn = 1)


## ----loadPackages-------------------------------------------------------------------------------------------------------------------------------------------------------

library(inTextSummaryTable)
library(clinUtils)
library(tools) # toTitleCase


## ----loadData-----------------------------------------------------------------------------------------------------------------------------------------------------------

# load example data
data(dataADaMCDISCP01)

dataAll <- dataADaMCDISCP01
labelVars <- attr(dataAll, "labelVars")

dataADSL <- dataADaMCDISCP01$ADSL


## ----formatExampleData--------------------------------------------------------------------------------------------------------------------------------------------------

dataAE <-  subset(dataAll$ADAE, SAFFL == "Y" & TRTEMFL == "Y")
dataAEInterest <- subset(dataAE, AESOC %in% c(
        "INFECTIONS AND INFESTATIONS",
        "GENERAL DISORDERS AND ADMINISTRATION SITE CONDITIONS"
    )
)

# ensure that order of elements is the one specified in 
# the corresponding numeric variable
dataAEInterest$TRTA <- reorder(dataAEInterest$TRTA, dataAEInterest$TRTAN)
dataAEInterest$AESEV <- factor(dataAEInterest$AESEV, levels = c("MILD", "MODERATE"))

dataTotalAE <- subset(dataAll$ADSL, TRT01A != "Placebo")
# should contain columns specified in 'colVar'
dataTotalAE$TRTA <- dataTotalAE$TRT01A 


## ----computeSummaryStatisticsTable--------------------------------------------------------------------------------------------------------------------------------------

summaryTable <- computeSummaryStatisticsTable(
    data = dataAEInterest,
    rowVar = c("AESOC", "AEDECOD"),
    rowVarTotalInclude = c("AESOC", "AEDECOD"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    dataTotal = dataTotalAE,
    labelVars = labelVars,
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)")
)

print(head(summaryTable, 3))

## ----computeSummaryStatisticsTable-isTotal------------------------------------------------------------------------------------------------------------------------------
print(subset(summaryTable, isTotal))

## ----export-flextable---------------------------------------------------------------------------------------------------------------------------------------------------

export(
	summaryTable = summaryTable,
	outputType = "flextable"
)


## ----combine------------------------------------------------------------------------------------------------------------------------------------------------------------
tableDemoCat <- computeSummaryStatisticsTable(
	data = dataADSL,
	var = c("SEX", "AGE"), varInclude0 = TRUE,
	colVar = "TRT01P",
	stats = getStats("n (%)", includeName = FALSE),
	labelVars = labelVars
)
tableDemoCont <- computeSummaryStatisticsTable(
	data = dataADSL,
	var = c("HEIGHTBL", "WEIGHTBL"),
	colVar = "TRT01P",
	stats = getStats(c("n", "Mean")),
	labelVars = labelVars
)
tableDemo <- combine(tableDemoCat, tableDemoCont)
export(tableDemo)

## ----combine-manually---------------------------------------------------------------------------------------------------------------------------------------------------
dataADSL$TRT01P <- with(dataADSL, reorder(TRT01P, TRT01PN))

# check format of table created with the package:
descTable <- tableDemoCont
descTable[, c("variable", "TRT01P", "isTotal", "n", "Mean")]

## ----combine-manually-rows----------------------------------------------------------------------------------------------------------------------------------------------
# add p-values in an extra row
infTable <- unique(subset(descTable, !isTotal)[, c("variable", "TRT01P"), drop = FALSE])
infTable[which(infTable$variable == "Baseline Height (cm)"), "pValue"] <- 1e-10
infTable[which(infTable$variable == "Baseline Weight (kg)"), "pValue"] <- 1e-9
summaryTable <- plyr::rbind.fill(descTable, infTable)

exportSummaryStatisticsTable(
	summaryTable = summaryTable, 
	rowVar = "variable", 
	colVar = "TRT01P", 
	statsVar = c("n", "Mean", "pValue") 
)

## ----combine-manually-columns-------------------------------------------------------------------------------------------------------------------------------------------
compLab <- "Comparison between treatments (p-value)"

# add p-values in a new column - in an extra row
infTable <- unique(subset(descTable, !isTotal)[, "variable", drop = FALSE])
infTable$TRT01P <- compLab
infTable[which(infTable$variable == "Baseline Height (cm)"), "pValue"] <- 1e-10
infTable[which(infTable$variable == "Baseline Weight (kg)"), "pValue"] <- 1e-9 
summaryTable <- plyr::rbind.fill(descTable, infTable) 

# order columns to have comparison column as last 
summaryTable$TRT01P <- factor(summaryTable$TRT01P, levels = c(levels(dataADSL$TRT01P), compLab))
exportSummaryStatisticsTable(
	summaryTable = summaryTable,  
	rowVar = "variable",  
	colVar = "TRT01P",  
	statsVar = c("n", "Mean", "pValue")
)

## ----combine-manually-columns-rows--------------------------------------------------------------------------------------------------------------------------------------
infTable <- unique(subset(descTable, !isTotal)[, "variable", drop = FALSE])
infTable$TRT01P <- compLab
infTable[which(infTable$variable == "Baseline Height (cm)"), "Mean"] <- 1e-10
infTable[which(infTable$variable == "Baseline Weight (kg)"), "Mean"] <- 1e-9

summaryTable <- plyr::rbind.fill(descTable, infTable) 

# order columns to have comparison column as last 
summaryTable$TRT01P <- factor(summaryTable$TRT01P, levels = c(levels(dataADSL$TRT01P), compLab)) 

exportSummaryStatisticsTable(
	summaryTable = summaryTable,
	rowVar = "variable",
	colVar = "TRT01P",
	statsVar = c("n", "Mean")
)

## ----combineVariables---------------------------------------------------------------------------------------------------------------------------------------------------

# prepare the data: create grouping of interest
dataAEGroup <- combineVariables(
    data = dataAEInterest,
    newVar = "AEGRP",
    paramsList = list(
        # for all screened patients
        list(var = "TRTA", value = "Xanomeline High Dose"),
        # for moderate severity
        list(var = "AESEV", value = "MODERATE", labelExtra = "Moderate"),
        list(var = "AENDY", label = paste("With adverse events ending date"))
    ),
    # include also counts for all records
    includeAll = TRUE,
    labelAll = "All Adverse events", 
    labelVars = labelVars
)
labelVars["AEGRP"] <- "Patient groups of interest"

# create the table
getSummaryStatisticsTable(
    data = dataAEGroup,
    colVar = "TRTA", 
    rowVar = "AEGRP", 
    labelVars = labelVars,
    dataTotal = dataTotalAE,
    stats = list(expression(paste0(statN, " (", round(statPercN, 1), ")"))),
    title = "Table: Adverse events: counts for groups of interest",
    footer = "Statistics: n (%)"
)


## ----includeSessionInfo, echo = FALSE, results = "asis"-----------------------------------------------------------------------------------------------------------------
print(sessionInfo())

## ----getVignette, eval = FALSE------------------------------------------------
# 
# vignette("inTextSummaryTable-createTables", "inTextSummaryTable")
# vignette("inTextSummaryTable-exportTables", "inTextSummaryTable")
# 

## ----options, echo = FALSE----------------------------------------------------------------------------------------------------------------------------------------------

library(knitr)
opts_chunk$set(
    echo = TRUE, results = 'markup', warning = FALSE, 
    # stop document execution if error (not the default)
    error = FALSE, 
    message = FALSE, cache = FALSE,
    fig.width = 8, fig.height = 7,
    fig.path = "./figures_vignette/",
    fig.align = 'center')
options(width = 170)
# instead of warn = 0 by default
# include warnings when they occur in the document
options(warn = 1)


## ----loadPackages-------------------------------------------------------------------------------------------------------------------------------------------------------

library(inTextSummaryTable)
library(clinUtils)
library(tools) # toTitleCase


## ----loadData-----------------------------------------------------------------------------------------------------------------------------------------------------------

# load example data
data(dataADaMCDISCP01)

dataAll <- dataADaMCDISCP01
labelVars <- attr(dataAll, "labelVars")

dataADSL <- dataADaMCDISCP01$ADSL


## ----formatExampleData--------------------------------------------------------------------------------------------------------------------------------------------------

dataAE <-  subset(dataAll$ADAE, SAFFL == "Y" & TRTEMFL == "Y")
dataAEInterest <- subset(dataAE, AESOC %in% c(
        "INFECTIONS AND INFESTATIONS",
        "GENERAL DISORDERS AND ADMINISTRATION SITE CONDITIONS"
    )
)

# ensure that order of elements is the one specified in 
# the corresponding numeric variable
dataAEInterest$TRTA <- reorder(dataAEInterest$TRTA, dataAEInterest$TRTAN)
dataAEInterest$AESEV <- factor(dataAEInterest$AESEV, levels = c("MILD", "MODERATE"))

dataTotalAE <- subset(dataAll$ADSL, TRT01A != "Placebo")
# should contain columns specified in 'colVar'
dataTotalAE$TRTA <- dataTotalAE$TRT01A 


## ----computeSummaryStatisticsTable--------------------------------------------------------------------------------------------------------------------------------------

summaryTable <- computeSummaryStatisticsTable(
    data = dataAEInterest,
    rowVar = c("AESOC", "AEDECOD"),
    rowVarTotalInclude = c("AESOC", "AEDECOD"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    dataTotal = dataTotalAE,
    labelVars = labelVars,
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)")
)

print(head(summaryTable, 3))

## ----computeSummaryStatisticsTable-isTotal------------------------------------------------------------------------------------------------------------------------------
print(subset(summaryTable, isTotal))

## ----export-flextable---------------------------------------------------------------------------------------------------------------------------------------------------

export(
	summaryTable = summaryTable,
	outputType = "flextable"
)


## ----combine------------------------------------------------------------------------------------------------------------------------------------------------------------
tableDemoCat <- computeSummaryStatisticsTable(
	data = dataADSL,
	var = c("SEX", "AGE"), varInclude0 = TRUE,
	colVar = "TRT01P",
	stats = getStats("n (%)", includeName = FALSE),
	labelVars = labelVars
)
tableDemoCont <- computeSummaryStatisticsTable(
	data = dataADSL,
	var = c("HEIGHTBL", "WEIGHTBL"),
	colVar = "TRT01P",
	stats = getStats(c("n", "Mean")),
	labelVars = labelVars
)
tableDemo <- combine(tableDemoCat, tableDemoCont)
export(tableDemo)

## ----combine-manually---------------------------------------------------------------------------------------------------------------------------------------------------
dataADSL$TRT01P <- with(dataADSL, reorder(TRT01P, TRT01PN))

# check format of table created with the package:
descTable <- tableDemoCont
descTable[, c("variable", "TRT01P", "isTotal", "n", "Mean")]

## ----combine-manually-rows----------------------------------------------------------------------------------------------------------------------------------------------
# add p-values in an extra row
infTable <- unique(subset(descTable, !isTotal)[, c("variable", "TRT01P"), drop = FALSE])
infTable[which(infTable$variable == "Baseline Height (cm)"), "pValue"] <- 1e-10
infTable[which(infTable$variable == "Baseline Weight (kg)"), "pValue"] <- 1e-9
summaryTable <- plyr::rbind.fill(descTable, infTable)

exportSummaryStatisticsTable(
	summaryTable = summaryTable, 
	rowVar = "variable", 
	colVar = "TRT01P", 
	statsVar = c("n", "Mean", "pValue") 
)

## ----combine-manually-columns-------------------------------------------------------------------------------------------------------------------------------------------
compLab <- "Comparison between treatments (p-value)"

# add p-values in a new column - in an extra row
infTable <- unique(subset(descTable, !isTotal)[, "variable", drop = FALSE])
infTable$TRT01P <- compLab
infTable[which(infTable$variable == "Baseline Height (cm)"), "pValue"] <- 1e-10
infTable[which(infTable$variable == "Baseline Weight (kg)"), "pValue"] <- 1e-9 
summaryTable <- plyr::rbind.fill(descTable, infTable) 

# order columns to have comparison column as last 
summaryTable$TRT01P <- factor(summaryTable$TRT01P, levels = c(levels(dataADSL$TRT01P), compLab))
exportSummaryStatisticsTable(
	summaryTable = summaryTable,  
	rowVar = "variable",  
	colVar = "TRT01P",  
	statsVar = c("n", "Mean", "pValue")
)

## ----combine-manually-columns-rows--------------------------------------------------------------------------------------------------------------------------------------
infTable <- unique(subset(descTable, !isTotal)[, "variable", drop = FALSE])
infTable$TRT01P <- compLab
infTable[which(infTable$variable == "Baseline Height (cm)"), "Mean"] <- 1e-10
infTable[which(infTable$variable == "Baseline Weight (kg)"), "Mean"] <- 1e-9

summaryTable <- plyr::rbind.fill(descTable, infTable) 

# order columns to have comparison column as last 
summaryTable$TRT01P <- factor(summaryTable$TRT01P, levels = c(levels(dataADSL$TRT01P), compLab)) 

exportSummaryStatisticsTable(
	summaryTable = summaryTable,
	rowVar = "variable",
	colVar = "TRT01P",
	statsVar = c("n", "Mean")
)

## ----combineVariables---------------------------------------------------------------------------------------------------------------------------------------------------

# prepare the data: create grouping of interest
dataAEGroup <- combineVariables(
    data = dataAEInterest,
    newVar = "AEGRP",
    paramsList = list(
        # for all screened patients
        list(var = "TRTA", value = "Xanomeline High Dose"),
        # for moderate severity
        list(var = "AESEV", value = "MODERATE", labelExtra = "Moderate"),
        list(var = "AENDY", label = paste("With adverse events ending date"))
    ),
    # include also counts for all records
    includeAll = TRUE,
    labelAll = "All Adverse events", 
    labelVars = labelVars
)
labelVars["AEGRP"] <- "Patient groups of interest"

# create the table
getSummaryStatisticsTable(
    data = dataAEGroup,
    colVar = "TRTA", 
    rowVar = "AEGRP", 
    labelVars = labelVars,
    dataTotal = dataTotalAE,
    stats = list(expression(paste0(statN, " (", round(statPercN, 1), ")"))),
    title = "Table: Adverse events: counts for groups of interest",
    footer = "Statistics: n (%)"
)


## ----includeSessionInfo, echo = FALSE, results = "asis"-----------------------------------------------------------------------------------------------------------------
print(sessionInfo())

## ----options, echo = FALSE----------------------------------------------------------------------------------------------------------------------------------------------
	
	library(knitr)
	opts_chunk$set(
		echo = TRUE, results = 'markup', warning = FALSE, 
		# stop document execution if error (not the default)
		error = FALSE, 
		message = FALSE, cache = FALSE,
		fig.width = 8, fig.height = 7,
		fig.path = "./figures_vignette/",
		fig.align = 'center')
	options(width = 170)
	# instead of warn = 0 by default
	# include warnings when they occur in the document
	options(warn = 1)
	

## ----loadPackages-------------------------------------------------------------------------------------------------------------------------------------------------------

	library(inTextSummaryTable)
	library(tools) # toTitleCase


## ----loadData-----------------------------------------------------------------------------------------------------------------------------------------------------------
	
	library(clinUtils)

	# load example data
    data(dataADaMCDISCP01)
	
	dataAll <- dataADaMCDISCP01
    labelVars <- attr(dataAll, "labelVars")
	
	# dataAll contains a list
	class(dataAll)
	# ... of ADaM datasets
	names(dataAll)
	# access a specific dataset
	head(dataAll$ADSL, 1)
	
	# check label of a subset of the variable(s)
	head(labelVars)
	# or for a specific variable:
	labelVars["USUBJID"]
	# or:
	getLabelVar(var = "USUBJID", labelVars = labelVars)


## ----includeSessionInfo, echo = FALSE, results = "asis"-----------------------------------------------------------------------------------------------------------------
print(sessionInfo())

## ----options, echo = FALSE----------------------------------------------------------------------------------------------------------------------------------------------
	
	library(knitr)
	opts_chunk$set(
		echo = TRUE, results = 'markup', warning = FALSE, 
		# stop document execution if error (not the default)
		error = FALSE, 
		message = FALSE, cache = FALSE,
		fig.width = 8, fig.height = 7,
		fig.path = "./figures_vignette/",
		fig.align = 'center')
	options(width = 170)
	# instead of warn = 0 by default
	# include warnings when they occur in the document
	options(warn = 1)
	

## ----loadPackages-------------------------------------------------------------------------------------------------------------------------------------------------------

	library(inTextSummaryTable)
	library(tools) # toTitleCase


## ----loadData-----------------------------------------------------------------------------------------------------------------------------------------------------------
	
	library(clinUtils)

	# load example data
    data(dataADaMCDISCP01)
	
	dataAll <- dataADaMCDISCP01
    labelVars <- attr(dataAll, "labelVars")
	

## ----data-SL------------------------------------------------------------------------------------------------------------------------------------------------------------

	dataSL <- dataAll$ADSL


## ----getHelp, eval = FALSE----------------------------------------------------------------------------------------------------------------------------------------------
# 
# ? `inTextSummaryTable-stats`
# 

## ----count-simple-------------------------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(data = dataSL)


## ----count-categories---------------------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(data = dataSL, var = "SEX")


## ----count-categories-order---------------------------------------------------------------------------------------------------------------------------------------------

	# specify manually the order of the categories
	dataSL$SEX <- factor(dataSL$SEX, levels = c("M", "F"))
	getSummaryStatisticsTable(data = dataSL, var = "SEX")
	
	# order categories based on a numeric variable
    dataSL$SEXN <- ifelse(dataSL$SEX == "M", 2, 1)
	dataSL$SEX <- reorder(dataSL$SEX, dataSL$SEXN)
	getSummaryStatisticsTable(data = dataSL, var = "SEX")
	

## ----count-categories-empty-1-------------------------------------------------------------------------------------------------------------------------------------------

	dataSLExample <- dataSL
	
	# 'SEX' formatted as character with only male
	dataSLExample$SEX <- "M" # only male
	getSummaryStatisticsTable(data = dataSLExample, var = "SEX")
	

## ----count-categories-empty-2-------------------------------------------------------------------------------------------------------------------------------------------
	
	# 'SEX' formatted as factor, to include also female in the table
	# (even if not available in the data)
	dataSLExample$SEX <- factor("M", levels = c("F", "M"))
	getSummaryStatisticsTable(data = dataSLExample, var = "SEX", varInclude0 = TRUE)
	# or:
    getSummaryStatisticsTable(data = dataSLExample, var = "SEX", varInclude0 = "SEX")


## ----flag-variables-----------------------------------------------------------------------------------------------------------------------------------------------------

    labelVars[grep("FL$", colnames(dataSL), value = TRUE)]
    # has the subject discontinued from the study?
    dataSL$DISCONFL


## ----count-flag-var-----------------------------------------------------------------------------------------------------------------------------------------------------

    getSummaryStatisticsTable(
	    data = dataSL,
	    var = "SAFFL"
    )


## ----count-flag-varFlag-------------------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataSL,
		var = "SAFFL",
		varFlag = "SAFFL"
	)


## ----count-varTotalInclude----------------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataSL, 
		var = "SEX", 
		varTotalInclude = TRUE
	)


## ----numeric------------------------------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(data = dataSL, var = "AGE")


## ----mixedTable---------------------------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataSL, 
		var = c("AGE", "SEX")
	)


## ----getHelpStats, eval = FALSE-----------------------------------------------------------------------------------------------------------------------------------------
# 
#     ? `inTextSummaryTable-stats`
# 

## ----stats-count--------------------------------------------------------------------------------------------------------------------------------------------------------

	# count: n, '%' and m
	getSummaryStatisticsTable(
		data = dataSL,
		var = "SEX",
		stats = "count"
	)

	# n (%)
	getSummaryStatisticsTable(
		data = dataSL,
		var = "SEX",
		stats = "n (%)"
	)
	
	# n/N (%)
	getSummaryStatisticsTable(
		data = dataSL,
		var = "SEX",
		stats = "n/N (%)"
	)
	

## ----stats-numeric------------------------------------------------------------------------------------------------------------------------------------------------------
	
	## continuous variable
	
	# all summary stats
	getSummaryStatisticsTable(
		data = dataSL,
		var = "AGE",
		stats = "summary"
	)
	
	# median (range)
	getSummaryStatisticsTable(
		data = dataSL,
		var = "AGE",
		stats = "median (range)"
	)
	
	# median and (range) in a different line:
	getSummaryStatisticsTable(
		data = dataSL,
		var = "AGE",
		stats = "median\n(range)"
	)
	
	# mean (se)
	getSummaryStatisticsTable(
		data = dataSL,
		var = "AGE",
		stats = "mean (se)"
	)
	
	# mean (sd)
	getSummaryStatisticsTable(
		data = dataSL,
		var = "AGE",
		stats = "mean (sd)"
	)


## ----getHelpStats2, eval = FALSE----------------------------------------------------------------------------------------------------------------------------------------
# 
# ? `inTextSummaryTable-stats`
# 

## ----stats-N------------------------------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataSL,
		var = c("RACE", "SEX"),
		stats = list(N = expression(statN))
	)


## ----stats-meanSE-------------------------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataSL,
		var = c("AGE", "HEIGHTBL", "WEIGHTBL", "BMIBL"),
		varGeneralLab = "Parameter", statsGeneralLab = "",
		colVar = "TRT01P",
		stats = list(
			`median` = expression(statMedian),
			`(min, max)` = expression(paste0("(", statMin, ",", statMax, ")"))
		)
	)


## ----getStatsData-------------------------------------------------------------------------------------------------------------------------------------------------------

    # this count table:
    getSummaryStatisticsTable(
	    data = dataSL,
	    var = "SEX",
	    stats = "count"
    )
    # ... is equivalent to:
    getSummaryStatisticsTable(
	    data = dataSL,
	    var = "SEX",
	    stats = getStats(type = "count")
    )

    # this summary table...
    getSummaryStatisticsTable(
	    data = dataSL,
	    var = "AGE",
	    stats = "mean (se)"
    )
    # ... is equivalent to:
    getSummaryStatisticsTable(
	    data = dataSL,
	    var = "AGE",
	    stats = getStatsData(type = "mean (se)", var = "AGE", data = dataSL)[["AGE"]]
    )

## ----statExtra-EachVariable---------------------------------------------------------------------------------------------------------------------------------------------
		
	getSummaryStatisticsTable(
		data = dataSL, 
		var = c("AGE", "RACE"),
		stats = list(
			AGE = getStats("median (range)"),
			RACE = getStats("n (%)")
		)
	)


## ----statsUtilityFct----------------------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataSL,
		var = "HEIGHTBL",
		# specify extra stats to compute
		statsExtra = list(
			statCV = cv,
			statGeomMean = geomMean,
			statGeomSD = geomSD,
			statsGeomCV = geomCV
		)
	)


## ----statsExtra---------------------------------------------------------------------------------------------------------------------------------------------------------

	# include the coefficient of variation via the 'statsExtra' parameter
	getSummaryStatisticsTable(
		data = dataSL,
		var = "HEIGHTBL",
		statsExtra = list(statCVPerc = function(x) sd(x)/mean(x)*100)
	)


## ----statsExtra-stats---------------------------------------------------------------------------------------------------------------------------------------------------

	# format the statistics with the 'stats' parameter
	getSummaryStatisticsTable(
		data = dataSL,
		var = "HEIGHTBL",
		statsExtra = list(statCVPerc = function(x) sd(x)/mean(x)*100),
		stats = list(Mean = expression(statMean), 'CV%' = expression(statCVPerc))
	)



## ----getHelpStats3, eval = FALSE----------------------------------------------------------------------------------------------------------------------------------------
# 
#     ? `inTextSummaryTable-stats`
# 

## ----nDecimals-catVar, echo = FALSE, fig.cap = "Standard Layout for Frequency Tabulations of Categorical Variables<br>"-------------------------------------------------

include_graphics("./images/nDecimals_catVar.png")


## ----getStats-count-----------------------------------------------------------------------------------------------------------------------------------------------------

	# Internal rule for the number of decimals for the percentage
	formatPercentage(c(NA, 0, 100, 99.95, 0.012, 34.768))
	
	# Used by default in the 'getStats' function
	getStats(type = "count")


## ----nDecimals-numVar, echo = FALSE, fig.cap = "Standard Layout for Descriptive Statistics of Continuous Variables<br>"-------------------------------------------------

include_graphics("./images/nDecimals_numVar.png")


## ----getMaxNDecimals----------------------------------------------------------------------------------------------------------------------------------------------------

	# Duration of Disease (Months)
	print(dataSL$DURDIS)
	
	## Extract the number of decimals for each value:
	
	# based on pre-defined rule, this metric should be displayed with 1 decimal:
	getNDecimalsRule(x = dataSL$DURDIS)
	
	# but available in the data only with 0 decimals
	getNDecimalsData(x = dataSL$DURDIS)
	
	# The minimum of the #decimals based on the data and pre-defined rule is:
	getNDecimals(x = dataSL$DURDIS)
	
	## Take the maximum number of decimals 
	getMaxNDecimals(x = dataSL$DURDIS)
	
	## Custom set of statistics are extracted when x is specified:
	getStats(x = dataSL$DURDIS)
	
	# To fix the number of decimals:
	getStats(type = "summary", nDecCont = 1)
	
	## Create summary statistics table
	getSummaryStatisticsTable(
		data = dataSL,
		var = c("AGE", "DURDIS"),
		stats = list(
			AGE = getStats(type = "median (range)", x = dataSL$AGE),
            DURDIS = getStats(type = "median (range)", x = dataSL$DURDIS)
		)
	)


## ----stats-digits-------------------------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataSL,
		var = c("AGE", "HEIGHTBL"),
		stats = list(
			AGE = list(Median = expression(roundHalfUpTextFormat(statMedian, 1))),
            HEIGHTBL = list(Median = expression(roundHalfUpTextFormat(statMedian, 2)))
		)
	)


## ----stats-digits-complex-----------------------------------------------------------------------------------------------------------------------------------------------

	# wrapper function to include median with specific number of digits
	# and min/max with specified number of digits - 1
	statsDMNum <- function(digitsMin)
		list('Median (range)' = 
			bquote(paste0(
				roundHalfUpTextFormat(statMedian, .(digitsMin+1)), 
				" (", roundHalfUpTextFormat(statMin, .(digitsMin)), ",", 
				roundHalfUpTextFormat(statMax, .(digitsMin)),
				")"
			))
	)

	getSummaryStatisticsTable(
		data = dataSL,
		var = c("AGE", "HEIGHTBL", "WEIGHTBL", "BMIBL", "RACE", "SEX"),
		stats = list(
			AGE = statsDMNum(0),
			HEIGHTBL = statsDMNum(1),
			WEIGHTBL = statsDMNum(1),
			BMIBL = statsDMNum(1),
			RACE = getStats("n (%)"),
			SEX = getStats("n (%)")
		)
	)


## ----statsLayoutRow-----------------------------------------------------------------------------------------------------------------------------------------------------

	# statsLayout = 'row'
	getSummaryStatisticsTable(
		data = dataSL,
		var = c("AGE", "HEIGHTBL"),
		stats = list(Mean = expression(statMean), 'SE' = expression(statSE))
	)
	

## ----statsLayoutCol-----------------------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataSL,
		var = c("AGE", "HEIGHTBL"),
		stats = list(Mean = expression(statMean), 'SE' = expression(statSE)),
		statsLayout = "col"
	)


## ----summaryTable-PP-medianMinMax-statsLayoutRowVarInSepCol-------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataSL,
		var = c("AGE", "HEIGHTBL"),
		stats = list(Mean = expression(statMean), 'SE' = expression(statSE)),
		statsLayout = "rowInSepCol"
	)


## ----summaryTable-statsLayout-onlyOneStat-------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataSL,
		var = c("AGE", "HEIGHTBL"),
		stats = list(Mean = expression(statMean))
	)


## ----summaryTable-statsLayout-onlyOneStat-statsLabInclude---------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataSL,
		var = c("AGE", "HEIGHTBL"),
		stats = list(Mean = expression(statMean)),
		statsLabInclude = TRUE
	)


## ----countTable-AE-data-------------------------------------------------------------------------------------------------------------------------------------------------
				
	dataAE <-  subset(dataAll$ADAE, SAFFL == "Y" & TRTEMFL == "Y")
	
	# ensure that order of elements is the one specified in 
	# the corresponding numeric variable
	dataAE$TRTA <- with(dataAE, reorder(TRTA, TRTAN))
	dataAE$AESEV <- factor(
		dataAE$AESEV, 
		levels = c("MILD", "MODERATE", "SEVERE")
	)
	
	dataAEInterest <- subset(dataAE, AESOC %in% c(
		"INFECTIONS AND INFESTATIONS",
        "GENERAL DISORDERS AND ADMINISTRATION SITE CONDITIONS"
       )
    )


## ----rowVarColVar-------------------------------------------------------------------------------------------------------------------------------------------------------

	# unique row variable
	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = "AEDECOD",
		stats = getStats("n (%)"),
		labelVars = labelVars
	)
	
	# multiple nested row variables
	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		stats = getStats("n (%)"),
		labelVars = labelVars
	)
	
	# unique column variable
	getSummaryStatisticsTable(
		data = dataAEInterest,
		colVar = "TRTA",
		stats = getStats("n (%)"),
		labelVars = labelVars
	)
	
	# combination of rows and columns
	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		stats = getStats("n (%)"),
		labelVars = labelVars,
		colHeaderTotalInclude = FALSE
	)


## ----rowVarInSepCol-----------------------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD", "AESEV"),
		rowVarInSepCol = "AESEV",
		colVar = "TRTA",
		stats = getStats("n (%)"),
		labelVars = labelVars
	)
	

## ----rowOrder-common----------------------------------------------------------------------------------------------------------------------------------------------------

	# 'auto':

	# set order of SOC to reverse alphabetical order
	dataAEInterest$AESOC <- factor(
		dataAEInterest$AESOC, 
		levels = rev(sort(unique(as.character(dataAEInterest$AESOC))))
	)
	# AEDECOD is not a factor -> sort alphabetically by default
	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarLab = labelVars[c("AEDECOD")],
		rowVarTotalInclude = c("AESOC", "AEDECOD"),
		colVar = "TRTA", colTotalInclude = TRUE,
		stats = getStats("n (%)"),
		labelVars = labelVars
	)
	
	# total counts
	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarLab = labelVars[c("AEDECOD")],
		rowVarTotalInclude = c("AESOC", "AEDECOD"),
		colVar = "TRTA", colTotalInclude = TRUE, colTotalLab = "Number of subjects",
		rowOrder = "total",
		stats = getStats("n (%)"),
		labelVars = labelVars
	)
	# same order even if the 'total' column is not specified
	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarLab = labelVars[c("AEDECOD")],
		rowVarTotalInclude = c("AESOC", "AEDECOD"),
		colVar = "TRTA", 
		rowOrder = "total", 
		stats = getStats("n (%)"),
		labelVars = labelVars
	)


## ----rowOrder-specific--------------------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarLab = labelVars[c("AEDECOD")],
		rowVarTotalInclude = c("AESOC", "AEDECOD"),
		colVar = "TRTA", #colTotalInclude = TRUE,
		rowOrder = c(AESOC = "alphabetical", AEDECOD = "total"),
		stats = getStats("n (%)"),
		labelVars = labelVars
	)


## ----rowOrder-rowOrderTotalFilterFct------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarLab = labelVars[c("AEDECOD")],
		rowVarTotalInclude = c("AESOC", "AEDECOD"),
		colVar = "TRTA", colTotalInclude = TRUE,
		rowOrder = "total",
		stats = getStats("n (%)"),
		labelVars = labelVars,
		# consider only the counts of the treated patients to order the rows
		rowOrderTotalFilterFct = function(x) subset(x, TRTA == "Xanomeline High Dose")
	)


## ----rowOrder-functionExample1------------------------------------------------------------------------------------------------------------------------------------------

	library(plyr)
	getSummaryStatisticsTable(
		data = dataAEInterest,
		type = "count",
		rowVar = "AEHLT",
		rowOrder = function(x){
			x <- subset(x, !isTotal)
			totalAcrossTreatments <- subset(x, TRTA == "Total")
			# counts across treated patients
			totalForTreatmentOnly <- subset(x, TRTA == "Xanomeline High Dose")
			dataCounts <- merge(totalAcrossTreatments, totalForTreatmentOnly, by = "AEHLT", suffixes = c(".all", ".treat"))
			# sort first based on overall count, then counts of treated patients
			dataCounts[with(dataCounts, order(`statN.all`, `statN.treat`, decreasing = TRUE)), "AEHLT"]
		},
		colVar = "TRTA", colTotalInclude = TRUE,
		labelVars = labelVars,
		title = "Table: Adverse Events ordered based on total counts",
		stats = list(expression(paste0(statN, " (", round(statPercN, 1), ")"))),
		footer = "Statistics: n (%)"
	)


## ----rowOrder-functionExample2------------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarLab = labelVars[c("AEDECOD")],
		rowVarTotalInclude = c("AESOC", "AEDECOD"),
		colVar = "TRTA", colTotalInclude = TRUE,
		rowOrder = list(
			AESOC = function(table) {
				# records with total for each AESOC
				nAESOCPlacebo <- subset(table, !isTotal & grepl("placebo", TRTA) & AEDECOD == "Total")
				nAESOCTreat <- subset(table, !isTotal & grepl("High Dose", TRTA) & AEDECOD == "Total")
				nAESOCDf <- merge(nAESOCPlacebo, nAESOCTreat, by = "AESOC", suffixes = c(".placebo", ".treatment"))
				nAESOCDf[with(nAESOCDf, order(`statN.placebo`, `statN.treatment`, decreasing = TRUE)), "AESOC"]
			},
			AEDECOD = function(table) {
				# records with counts for each AEDECOD
				nAEDECODPlacebo <- subset(table, !isTotal & grepl("placebo", TRTA) & AEDECOD != "Total")
				nAEDECODTreat <- subset(table, !isTotal & grepl("High Dose", TRTA) & AEDECOD != "Total")
				nAEDECODDf <- merge(nAEDECODPlacebo, nAEDECODTreat, by = "AEDECOD", suffixes = c(".placebo", ".treatment"))
				nAEDECODDf[with(nAEDECODDf, order(`statN.placebo`, `statN.treatment`, decreasing = TRUE)), "AEDECOD"]
			}
		),
		stats = getStats("n (%)"),
		labelVars = labelVars
	)


## ----summaryTable-PP-rowVarWithLabel-labelVars--------------------------------------------------------------------------------------------------------------------------

	# combination of rows and columns
	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		stats = getStats("n (%)"),
		labelVars = labelVars
	)
	

## ----summaryTable-PP-rowVarWithLabel-rowVarLab--------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		stats = getStats("n (%)"),
		rowVarLab = c(
			'AESOC' = "TEAE by SOC and Preferred Term\nn (%)"
		),
		labelVars = labelVars
	)
	

## ----countTable-emptyVars-----------------------------------------------------------------------------------------------------------------------------------------------

	## only consider a subset of adverse events
	dataAESubset <- subset(dataAE, AEHLT == "HLT_0617")
	
	## create dummy categories for:
	# treatment
	dataAESubset$TRTA <- with(dataAESubset, 
		factor(TRTA, levels = c(unique(as.character(TRTA)), "Treatment B"))
	)
	# low-level term category
	dataAESubset$AELLT <- with(dataAESubset, 
		factor(AELLT, levels = c(unique(as.character(AELLT)), "Lymphocyte percentage increased"))
	)
	
	# create summary statistics table
	getSummaryStatisticsTable(
		data = dataAESubset,
		type = "count",
		rowVar = c("AEHLT", "AELLT"),
		rowInclude0 = TRUE, colInclude0 = TRUE,
		colVar = "TRTA",
		labelVars = labelVars,
		title = "Table: Adverse Events: white blood cell analyses",
		stats = getStats("n (%)"),
		footer = "Statistics: n (%)"
	)


## ----summaryTable-layout-var--------------------------------------------------------------------------------------------------------------------------------------------

    dataDIABP <- subset(dataAll$ADVS, 
		SAFFL == "Y" & ANL01FL == "Y" &
		PARAMCD == "DIABP" & 
		AVISIT %in% c("Baseline", "Week 8") &
		ATPT == "AFTER LYING DOWN FOR 5 MINUTES"
	)
    dataDIABP$TRTA <- reorder(dataDIABP$TRTA, dataDIABP$TRTAN)
    dataDIABP$AVISIT <- reorder(dataDIABP$AVISIT, dataDIABP$AVISITN)
    
	getSummaryStatisticsTable(
		data = dataDIABP,
		var = c("AVAL", "CHG"),
		colVar = "TRTA",
		rowVar = "AVISIT",
		labelVars = labelVars,
		stats = getStats("summary-default")
	)


## ----summaryTable-layout-varInColumn------------------------------------------------------------------------------------------------------------------------------------
    
getSummaryStatisticsTable(
	data = dataDIABP,
	var = c("AVAL", "CHG"),
	colVar = c("variable", "TRTA"),
	rowVar = "AVISIT",
	labelVars = labelVars,
	stats = getStats("summary-default")
)


## ----summaryTable-layout-var-oneLabel-default---------------------------------------------------------------------------------------------------------------------------
		
	getSummaryStatisticsTable(data = dataSL, var = "AGE", colVar = "TRT01P")


## ----summaryTable-layout-var-oneLabel-varLabInclude---------------------------------------------------------------------------------------------------------------------
		
	getSummaryStatisticsTable(
		data = dataSL, 
		var = "AGE", 
		varLabInclude = TRUE,
		colVar = "TRT01P"
	)


## ----summaryTable-layout-var-empty--------------------------------------------------------------------------------------------------------------------------------------

    dataAEInterest$AESEVN <- ifelse(dataAEInterest$AESEV == "MILD", 1, 2)
	dataAEInterestWC <- ddply(dataAEInterest, c("AEDECOD", "USUBJID", "TRTA"), function(x) {
		x[which.max(x$AESEVN), ]
	})
	dataAEInterestWC[1, "AESEV"] <- NA
    getSummaryStatisticsTable(
        data = dataAEInterestWC,
        colVar = "TRTA",
        rowVar = "AEBODSYS",
		stats = getStats("n (%)"),
        var = c("AESEV", "all"),
		labelVars = labelVars
    )


## ----header-total-default-----------------------------------------------------------------------------------------------------------------------------------------------
	
	# by default, total number of subjects extracted from data
	getSummaryStatisticsTable(
		data = subset(dataAEInterest, AESOC == "INFECTIONS AND INFESTATIONS"),
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		stats = getStats("n (%)"),
		rowVarLab = c(
			'AESOC' = "TEAE by SOC and Preferred Term\nn (%)"
		),
		labelVars = labelVars
	)
	

## ----header-total-dataTotal---------------------------------------------------------------------------------------------------------------------------------------------
	
	# dataset used to extract the 'Total'
	dataTotalAE <- subset(dataAll$ADSL, SAFFL == "Y")
	# should contain columns specified in 'colVar'
	dataTotalAE$TRTA <- dataTotalAE$TRT01A 

	getSummaryStatisticsTable(
		data = subset(dataAEInterest, AESOC == "INFECTIONS AND INFESTATIONS"),
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		stats = getStats("n (%)"),
		rowVarLab = c(
			'AESOC' = "TEAE by SOC and Preferred Term\nn (%)"
		),
		dataTotal = dataTotalAE,
		labelVars = labelVars
	)


## ----header-total-colHeaderTotalInclude---------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = subset(dataAEInterest, AESOC == "INFECTIONS AND INFESTATIONS"),
		rowVar = c("AESOC", "AEDECOD"),
		rowVarTotalInclude = "AEDECOD",
		rowVarTotalInSepRow = "AEDECOD",
		colVar = "TRTA",
		stats = getStats("n (%)"),
		rowVarLab = c(
			'AESOC' = "TEAE by SOC and Preferred Term\nn (%)"
		),
		dataTotal = dataTotalAE,
		labelVars = labelVars,
		colHeaderTotalInclude = FALSE
	)


## ----dataTotalPerc------------------------------------------------------------------------------------------------------------------------------------------------------

    getSummaryStatisticsTable(
	    data = subset(dataAEInterest, AESOC == "INFECTIONS AND INFESTATIONS"),
	    rowVar = c("AESOC", "AEDECOD"),
	    colVar = "TRTA",
	    stats = getStats("n (%)"),
	    rowVarLab = c(
		    'AESOC' = "TEAE by SOC and Preferred Term\nn (%)"
	    ),
	    dataTotalPerc = dataTotalAE,
	    labelVars = labelVars
    )


## ----rowVarTotalPerc----------------------------------------------------------------------------------------------------------------------------------------------------

	dataLB <- subset(dataAll$ADLBC, 
		SAFFL == "Y" & 
		PARAMCD %in% c("K", "CHOL") &
		grepl("(Baseline)|(Week 20)", AVISIT)
	)
	dataLB$AVISIT <- with(dataLB, reorder(trimws(AVISIT), AVISITN))
	
	# counts versus the total per actual treatment arm
	getSummaryStatisticsTable(
		data = dataLB,
		colVar = "TRTA", 
		rowVar = c("PARAM", "AVISIT"), 
		var = "LBNRIND",
		stats = getStats("n (%)"),
		rowAutoMerge = FALSE, emptyValue = "0",
	)
	
	# percentage based on total number of subjects with available
	# measurement at specific visit for each parameter
	getSummaryStatisticsTable(
		data = dataLB,
		colVar = "TRTA", 
		rowVar = c("PARAM", "AVISIT"), 
		rowVarTotalPerc = c("PARAM", "AVISIT"),
		var = "LBNRIND",
		stats = getStats("n (%)"),
		rowAutoMerge = FALSE, emptyValue = "0",
	)	
	

## ----statsPerc----------------------------------------------------------------------------------------------------------------------------------------------------------

getSummaryStatisticsTable(
	data = dataLB,
	colVar = "TRTA", 
	rowVar = c("PARAM", "AVISIT"), 
	rowVarTotalPerc = c("PARAM", "AVISIT"),
	var = "LBNRIND", 
	stats = getStats("m (%)"),
	statsPerc = "statm",
	rowAutoMerge = FALSE, emptyValue = "0",
)


## ----colTotalInclude----------------------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		colTotalInclude = TRUE, 
		stats = getStats("n (%)"),
		rowVarLab = c(
			'AESOC' = "TEAE by SOC and Preferred Term\nn (%)"
		),
		dataTotal = dataTotalAE,
		labelVars = labelVars
	)


## ----colTotalLab--------------------------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		colTotalInclude = TRUE, colTotalLab = "All subjects",
		stats = getStats("n (%)"),
		rowVarLab = c(
			'AESOC' = "TEAE by SOC and Preferred Term\nn (%)"
		),
		dataTotal = dataTotalAE,
		labelVars = labelVars
	)


## ----dataTotalCol-------------------------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = subset(dataAEInterest, grepl("High Dose", TRTA)),
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		colTotalInclude = TRUE, colTotalLab = "Placebo and treatment arm",
		dataTotalCol = dataAEInterest,
		stats = getStats("n (%)"), emptyValue = "0",
		rowVarLab = c(
			'AESOC' = "TEAE by SOC and Preferred Term\nn (%)"
		),
		dataTotal = dataTotalAE,
		labelVars = labelVars
	)


## ----rowVarTotalInclude-------------------------------------------------------------------------------------------------------------------------------------------------

	# total reported across AESOC
	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarTotalInclude = "AESOC", 
		colVar = "TRTA",
		stats = getStats("n (%)"),
		labelVars = labelVars
	)
	
	# total reported across AESOC and across AEDECOD
	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarTotalInclude = c("AESOC", "AEDECOD"), 
		colVar = "TRTA",
		stats = getStats("n (%)"),
		labelVars = labelVars
	)


## ----rowTotalLab--------------------------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarTotalInclude = "AESOC", rowTotalLab = "Any AE", 
		colVar = "TRTA",
		stats = getStats("n (%)"),
		labelVars = labelVars
	)


## ----rowVarTotalInclude-rowVarTotalInSepRow-----------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarTotalInclude = "AEDECOD",
		rowVarTotalInSepRow = "AEDECOD",
		colVar = "TRTA",
		stats = getStats("n (%)"),
		labelVars = labelVars
	)


## ----dataTotalRow-------------------------------------------------------------------------------------------------------------------------------------------------------

	dataAEInterest$AESEVN <- as.numeric(dataAEInterest$AESEV)
	
	# compute worst-case scenario per subject*AE term*treatment
	dataAEInterestWC <- ddply(dataAEInterest, c("AESOC", "AEDECOD", "USUBJID", "TRTA"), function(x){
		x[which.max(x$AESEVN), ]
	})

	## datasets used for the total: 
	# for total: compute worst-case across SOC and across AE term
	# (otherwise patient counted in multiple categories if present different categories for different AEs)
	dataTotalRow <- list(
		# within visit (across AEDECOD)
		'AEDECOD' = ddply(dataAEInterest, c("AESOC", "USUBJID", "TRTA"), function(x){	
			x[which.max(x$AESEVN), ]
		}),
		# across visits
		'AESOC' = ddply(dataAEInterest, c("USUBJID", "TRTA"), function(x){	
			x[which.max(x$AESEVN), ]
		})
	)

	getSummaryStatisticsTable(
		data = dataAEInterestWC,
		## row variables:
		rowVar = c("AESOC", "AEDECOD", "AESEV"), 
		rowVarInSepCol = "AESEV",
		# total for column header and denominator
		dataTotal = dataTotalAE, 
		# include total across SOC and across AEDECOD
		rowVarTotalInclude = c("AESOC", "AEDECOD"), 
		# data for total row
		dataTotalRow = dataTotalRow, 
		# count for each severity category for the total
		rowVarTotalByVar = "AESEV", 
		rowTotalLab = "Any TEAE", 
		rowVarLab = c(AESOC = "Subjects with, n(%):", AESEV = "Worst-case scenario"),
		# sort per total in the total column
		rowOrder = "total", 
		## column variables
		colVar = "TRTA", 
		stats = getStats("n (%)"),
		emptyValue = "0",
		labelVars = labelVars
	)


## ----titleAndFooter-----------------------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		stats = getStats("n (%)"),
		dataTotal = dataTotalAE,
		labelVars = labelVars,
		title = toTitleCase("MOR106-CL-102: Adverse Events by System Organ Class and Preferred Term (Safety Analysis Set, Part 1)"),
		footer = c(
			"N=number of subjects with data; n=number of subjects with this observation",
			"Denominator for percentage calculations = the total number of subjects per treatment group in the safety population"
		)
	)


## ----includeSessionInfo, echo = FALSE, results = "asis"-----------------------------------------------------------------------------------------------------------------
print(sessionInfo())

## ----options, echo = FALSE----------------------------------------------------------------------------------------------------------------------------------------------

library(knitr)
opts_chunk$set(
    echo = TRUE, results = 'markup', warning = FALSE, 
    # stop document execution if error (not the default)
    error = FALSE, 
    message = FALSE, cache = FALSE,
    fig.width = 4, fig.height = 4,
    fig.path = "./figures_vignette/",
    fig.align = 'center')
options(width = 170)
# instead of warn = 0 by default
# include warnings when they occur in the document
options(warn = 1)


## ----aesthetics-showOptions---------------------------------------------------------------------------------------------------------------------------------------------

library(inTextSummaryTable)

# options for color scheme in in presentations
options("inTextSummaryTable.colors.table.presentation")

# options for colors in plots: viridis palette
options("inTextSummaryTable.colors.plot")

# options for shapes in plots
options("inTextSummaryTable.shapes.plot")

# options for linetypes in plots
options("inTextSummaryTable.linetypes.plot")


## ----aesthetics-defaultsReporting---------------------------------------------------------------------------------------------------------------------------------------

# default report style
getSummaryStatisticsTable(
    data = data.frame(USUBJID = c(1, 2)),
    style = "report" 
)

# default presentation style
getSummaryStatisticsTable(
    data = data.frame(USUBJID = c(1, 2)),
    style = "presentation" 
)


## ----aesthetics-changeReportingOptions----------------------------------------------------------------------------------------------------------------------------------

# create named vector
customColorTable <- c(
    # black text in the header
    'header' = "#000000",
    # green background in the header
    'headerBackground' = "#74D055FF",
    # black text in the body
    'body' = "#000000", 
    # yellow background for all rows
    'bodyBackground1' = "#FDE725FF",
    'bodyBackground2' = "#FDE725FF",
    # black footer
    'footer' = "#000000",
    # white footer background
    'footerBackground' = "#FFFFFF",
    # black line for footer
    'line' = "#000000"
)
# set options
options(inTextSummaryTable.colors.table.presentation = customColorTable)
# create the table
getSummaryStatisticsTable(
    data = data.frame(USUBJID = c(1, 2)),
    style = "presentation" 
)


## ----aesthetics-changeDimPageOptions------------------------------------------------------------------------------------------------------------------------------------

# default page dimension of a powerpoint created with Rmd
getOption("inTextSummaryTable.pageDim.presentation")

# set custom dimension of page for presentation
# in this example, the dimension is the widescreen size
pageDimCustom <- c(7.5, 13.32)
options(inTextSummaryTable.pageDim.presentation = pageDimCustom)
getOption("inTextSummaryTable.pageDim.presentation")


## ----getVignetteClinUtils, eval = FALSE---------------------------------------------------------------------------------------------------------------------------------
# 
# vignette("clinUtils-vignette", "clinUtils")
# 

## ----aesthetics-defaultsVisualization-----------------------------------------------------------------------------------------------------------------------------------

# default colors, shapes and linetypes
summaryTable <- data.frame(
    visit = c(1, 2, 1, 2), 
    TRT = c("A", "A", "B", "B"),
    statMean = rnorm(4)
)    
subjectProfileSummaryPlot(
    data = summaryTable,
    xVar = "visit", 
    colorVar = "TRT" 
)


## ----aesthetics-changeVisualizationOptions------------------------------------------------------------------------------------------------------------------------------

# specify colors
options(inTextSummaryTable.colors.plot = c("red", "green"))
subjectProfileSummaryPlot(
    data = summaryTable,
    xVar = "visit", 
    colorVar = "TRT" 
)

# specify shape
options(inTextSummaryTable.shapes.plot = c("circle", "square"))
subjectProfileSummaryPlot(
    data = summaryTable,
    xVar = "visit", 
    colorVar = "TRT" 
)

# specify linetypes
options(inTextSummaryTable.linetypes.plot = c("dotdash", "longdash"))
subjectProfileSummaryPlot(
    data = summaryTable,
    xVar = "visit", 
    colorVar = "TRT" 
)


## ----aesthetics-backDefaultPalettes-------------------------------------------------------------------------------------------------------------------------------------

options(inTextSummaryTable.colors.table.presentation = tableColorsPresentation)
options(inTextSummaryTable.colors.plot = clinUtils::clinColors)
options(inTextSummaryTable.shapes.plot = clinUtils::clinShapes)
options(inTextSummaryTable.linetypes.plot = clinUtils::clinLinetypes)


## ----aesthetics-getDimPage----------------------------------------------------------------------------------------------------------------------------------------------

# a4 format with one 1 inch margin
getDimPage(type = "width", style = "report")
getDimPage(type = "height", style = "report")

# Presentation format (according to template) with one 1 inch margin
getDimPage(type = "width", style = "presentation")
getDimPage(type = "height", style = "presentation")


## ----includeSessionInfo, echo = FALSE, results = "asis"-----------------------------------------------------------------------------------------------------------------
print(sessionInfo())

## ----options, echo = FALSE----------------------------------------------------------------------------------------------------------------------------------------------

library(knitr)
opts_chunk$set(
    echo = TRUE, results = 'markup', warning = FALSE, 
    # stop document execution if error (not the default)
    error = FALSE, 
    message = FALSE, cache = FALSE,
    fig.width = 8, fig.height = 7,
    fig.path = "./figures_vignette/",
    fig.align = 'center')
options(width = 170)
# instead of warn = 0 by default
# include warnings when they occur in the document
options(warn = 1)


## ----loadPackages-------------------------------------------------------------------------------------------------------------------------------------------------------

library(inTextSummaryTable)
library(clinUtils)
library(tools) # toTitleCase


## ----loadData-----------------------------------------------------------------------------------------------------------------------------------------------------------

# load example data
data(dataADaMCDISCP01)

dataAll <- dataADaMCDISCP01
labelVars <- attr(dataAll, "labelVars")


## ----createExampleData--------------------------------------------------------------------------------------------------------------------------------------------------

dataAE <-  subset(dataAll$ADAE, SAFFL == "Y" & TRTEMFL == "Y")
dataAEInterest <- subset(dataAE, AESOC %in% c(
        "INFECTIONS AND INFESTATIONS",
        "GENERAL DISORDERS AND ADMINISTRATION SITE CONDITIONS"
    )
)

# ensure that order of elements is the one specified in 
# the corresponding numeric variable
dataAEInterest$TRTA <- reorder(dataAEInterest$TRTA, dataAEInterest$TRTAN)
dataAEInterest$AESEV <- factor(dataAEInterest$AESEV, levels = c("MILD", "MODERATE"))

dataTotalAE <- subset(dataAll$ADSL, TRT01A != "Placebo")
# should contain columns specified in 'colVar'
dataTotalAE$TRTA <- dataTotalAE$TRT01A 



## ----outputType-flextable-----------------------------------------------------------------------------------------------------------------------------------------------

summaryTableFt <- getSummaryStatisticsTable(
    data = dataAEInterest,
    rowVar = c("AESOC", "AEDECOD"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    dataTotal = dataTotalAE,
    labelVars = labelVars,
    title = "Table: Adverse Events by System Organ Class and Preferred Term",
    footer = c(
        "N = number of subjects in the data; n = number of subjects with observation",
        paste("Denominator for percentage calculations is the total number of subjects",
            "per treatment group in the safety population"
        )
    ),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)"),
    outputType = "flextable"
)
class(summaryTableFt)
summaryTableFt


## ----style-report-------------------------------------------------------------------------------------------------------------------------------------------------------

getSummaryStatisticsTable(
    data = dataAEInterest,
    rowVar = c("AESOC", "AEDECOD"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    dataTotal = dataTotalAE,
    labelVars = labelVars,
    title = "Table: Adverse Events by System Organ Class and Preferred Term",
    footer = c(
        "N = number of subjects with data; n = number of subjects with this observation",
        paste("Denominator for percentage calculations is the total number of subjects",
            "per treatment group in the safety population"
        )
    ),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)")
)


## ----style-presentation-------------------------------------------------------------------------------------------------------------------------------------------------

getSummaryStatisticsTable(
    data = dataAEInterest,
    rowVar = c("AESOC", "AEDECOD"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    dataTotal = dataTotalAE,
    labelVars = labelVars,
    title = "Table: Adverse Events by System Organ Class and Preferred Term",
    footer = c(
        "N = number of subjects with data; n = number of subjects with this observation",
        paste("Denominator for percentage calculations is the total number of subjects",
            "per treatment group in the safety population"
        )
    ),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)"),
    style = "presentation"
)


## ----outputType-DT, eval = rmarkdown::pandoc_available()----------------------------------------------------------------------------------------------------------------

getSummaryStatisticsTable(
    data = dataAEInterest,
    rowVar = c("AESOC", "AEDECOD"),
    colVar = "TRTA",
    stats = getStats("n"),
    dataTotal = dataTotalAE,
    labelVars = labelVars,
    title = "Table: Adverse Events by System Organ Class and Preferred Term",
    footer = c(
        "N = number of subjects in the data; n = number of subjects with observation",
        paste("Denominator for percentage calculations is the total number of subjects",
            "per treatment group in the safety population"
        )
    ),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)"),
    outputType = "DT",
	## DT-specific options
	buttons = c() # remove all export buttons
)


## ----outputType-dataframe-----------------------------------------------------------------------------------------------------------------------------------------------

summaryTable <- getSummaryStatisticsTable(
    data = dataAEInterest,
    rowVar = c("AESOC", "AEDECOD"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    dataTotal = dataTotalAE,
    labelVars = labelVars,
    title = "Table: Adverse Events by System Organ Class and Preferred Term",
    footer = c(
        "N = number of subjects in the data; n = number of subjects with observation",
        paste("Denominator for percentage calculations is the total number of subjects",
            "per treatment group in the safety population"
        )
    ),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)"),
    style = "presentation",
    outputType = "data.frame"
)
print(summaryTable, split.table = Inf)

## ----outputType-dataframe-base------------------------------------------------------------------------------------------------------------------------------------------

summaryTableAll <- getSummaryStatisticsTable(
    data = dataAEInterest,
    rowVar = c("AESOC", "AEDECOD"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    dataTotal = dataTotalAE,
    labelVars = labelVars,
    title = "Table: Adverse Events by System Organ Class and Preferred Term",
    footer = c(
        "N = number of subjects in the data; n = number of subjects with observation",
        paste("Denominator for percentage calculations is the total number of subjects",
            "per treatment group in the safety population"
        )
    ),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)"),
    style = "presentation",
    outputType = "data.frame-base"
)
print(summaryTableAll, split.table = Inf)

## ----aesthetics-changeReportingOptions----------------------------------------------------------------------------------------------------------------------------------

# create named vector
customColorTable <- c(
    # black text in the header
    'header' = "#000000",
    # green background in the header
    'headerBackground' = "#74D055FF",
    # black text in the body
    'body' = "#000000", 
    # yellow background for all rows
    'bodyBackground1' = "#FDE725FF",
    'bodyBackground2' = "#FDE725FF",
    # black footer
    'footer' = "#000000",
    # white footer background
    'footerBackground' = "#FFFFFF",
    # black line for footer
    'line' = "#000000"
)
# set options
options(inTextSummaryTable.colors.table.presentation = customColorTable)
# create the table on a dummy data set
getSummaryStatisticsTable(
    data = data.frame(USUBJID = c(1, 2)),
    style = "presentation" 
)


## ----aesthetics-changeDimPageOptions------------------------------------------------------------------------------------------------------------------------------------

# set custom dimension of page for presentation
# in this example, the dimension is the widescreen size
pageDimCustom <- c(7.5, 13.32)
options(inTextSummaryTable.pageDim.presentation = pageDimCustom)
getOption("inTextSummaryTable.pageDim.presentation")


## ----aesthetics-backDefaultPalettes-------------------------------------------------------------------------------------------------------------------------------------

options(inTextSummaryTable.colors.table.presentation = tableColorsPresentation)


## ----file, eval = FALSE-------------------------------------------------------------------------------------------------------------------------------------------------
# 
# # export table to a Word document
# summaryTableFt <- getSummaryStatisticsTable(
#     data = dataAEInterest,
#     rowVar = c("AESOC", "AEDECOD"),
#     colVar = "TRTA",
#     stats = getStats("n (%)"),
#     dataTotal = dataTotalAE,
#     labelVars = labelVars,
#     file = file.path("tables_CSR", "summaryTable-AEs.docx")
# )
# 
# # export interactive table to a html document
# summaryTableFt <- getSummaryStatisticsTable(
#     data = dataAEInterest,
#     rowVar = c("AESOC", "AEDECOD"),
#     colVar = "TRTA",
#     stats = getStats("n (%)"),
#     dataTotal = dataTotalAE,
#     labelVars = labelVars,
#     file = file.path("tables_CSR", "summaryTable-AEs.html")
# )
# 
# # export table in raw format to a text file
# summaryTableFt <- getSummaryStatisticsTable(
#     data = dataAEInterest,
#     rowVar = c("AESOC", "AEDECOD"),
#     colVar = "TRTA",
#     stats = getStats("n (%)"),
#     dataTotal = dataTotalAE,
#     labelVars = labelVars,
#     file = file.path("tables_CSR", "summaryTable-AEs.txt")
# )
# 
# # export to multiple formats at once
# summaryTableFt <- getSummaryStatisticsTable(
#     data = dataAEInterest,
#     rowVar = c("AESOC", "AEDECOD"),
#     colVar = "TRTA",
#     stats = getStats("n (%)"),
#     dataTotal = dataTotalAE,
#     labelVars = labelVars,
#     file = file.path("tables_CSR",
#         c("summaryTable.txt", "summaryTable.docx", "summaryTable.html")
#     )
# )
# 

## ----export-------------------------------------------------------------------------------------------------------------------------------------------------------------

dataDIABP <- subset(
	dataAll$ADVS, 
	SAFFL == "Y" & ANL01FL == "Y" &
	PARAMCD == "DIABP" & 
	AVISIT %in% c("Baseline", "Week 6") &
	ATPT == "AFTER LYING DOWN FOR 5 MINUTES"
)

# create example of data.frame containing statistics of interest
statsEff <- sapply(c("AVAL", "CHG"), function(var) {
      
      getStats(
          type = c("n", "mean (se)", "median (range)"),
          x = dataDIABP[[var]]
      )
      
    }, simplify = FALSE)

summaryTable <- computeSummaryStatisticsTable(
    data = dataDIABP,
    colVar = c("TRTP", "AVISIT"),
    var = c("AVAL", "CHG"), varGeneralLab = "",
    stats = statsEff,
    labelVars = labelVars
)

# format df with statistics to in-text table format
export(
    summaryTable = summaryTable,
    statsVar = c("n", "Mean (SE)", "Median (range)"),
    rowVar = "variable", rowVarLab = "Statistic",
    colVar = c("TRTP", "AVISIT"),
    colHeaderTotalInclude = TRUE,
    labelVars = simpleCap(tolower(labelVars[c("AVAL", "CHG")])),
    title = toTitleCase(
		"Table: Diastolic Blood Pressure (mmHg) statistics"
    )
)


## ----summaryTable-PP-rowVarWithLabel-rowVarLab-superscript--------------------------------------------------------------------------------------------------------------

dataSL <- subset(dataAll$ADSL, SAFFL == "Y")

varsSL <- c("AGE", "WEIGHTBL", "BMIBL")
labelVars[varsSL] <- c(
    "Age",
    "Weight_{t}",
    "BMI (kg/m^{2})"
)

getSummaryStatisticsTable(
    data = dataSL, 
    var = varsSL,  
    stats = getStats("n (%"),
    labelVars = labelVars,
    fontsize = 16,
    title = toTitleCase("Demographic data (Safety Analysis Set)")
)


## ----summaryTable-PP-rowVarWithLabel-rowVarLab-bold---------------------------------------------------------------------------------------------------------------------

getSummaryStatisticsTable(
    data = dataSL, 
    var = varsSL,  
    stats = list(
		expression(paste0(
			ifelse(statMean > statMedian, 
				paste0("bold{", roundHalfUpTextFormat(statMean, 1), "}"), 
				roundHalfUpTextFormat(statMean, 1)
			), 
			"\n(", 
				roundHalfUpTextFormat(statSE, 2),")")
        )
    ),
    labelVars = labelVars,
    fontsize = 12,
    title = toTitleCase("Demographic data (Safety Analysis Set)")
)


## ----'objectsList1', results = 'asis', echo = FALSE---------------------------------------------------------------------------------------------------------------------
xList[[1]]

## ----'objectsList2', results = 'asis', echo = FALSE---------------------------------------------------------------------------------------------------------------------
xList[[2]]

## ----'objectsList3', results = 'asis', echo = FALSE---------------------------------------------------------------------------------------------------------------------
xList[[3]]

## ----byVar, results = "asis"--------------------------------------------------------------------------------------------------------------------------------------------

summaryTableList <- getSummaryStatisticsTable(
    data = subset(dataAE, TRTEMFL == "Y"),
    rowVar = c("AESOC", "AEDECOD"),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)"),
    colVar = "TRTA",
    byVar = "TRTA",
    stats = getStats("n (%)"),
    labelVars = labelVars,
    title = "Table: Treatment-Emergent adverse events"
)

# print the list of tables in the rmarkdown document
clinUtils::knitPrintListObjects(summaryTableList) 


## ----getListing---------------------------------------------------------------------------------------------------------------------------------------------------------

varsListing <- c(
	"USUBJID", "AEBODSYS", "AEDECOD", "TRTA", 
	"AESEV", "AESER", "ASTDY", "AENDY"
)

dataListing <- subset(dataAE, TRTEMFL == "Y" & AESEV == "SEVERE")
dataListing <- dataListing[, varsListing]
colnames(dataListing) <- getLabelVar(var = varsListing, labelVars = labelVars)

getListing(
	data = dataListing, 
	title = "Listing of treatment-emergent severe adverse events",
	includeRownames = FALSE
)


## ----countTable-AE-filterFct--------------------------------------------------------------------------------------------------------------------------------------------

library(plyr)

# SOC with AE terms with at least 2 subjects
getSummaryStatisticsTable(
    data = subset(dataAE, TRTEMFL == "Y"),
    rowVar = c("AESOC", "AEDECOD"),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    filterFct = function(x)
      ddply(x, "AESOC", function(y)
            if(any(y$statN >= 2))	y			
      ),
    labelVars = labelVars,
    title = paste(
        "Table: Adverse Events by System Organ Class and",
        "Preferred Term with at least 2 patients in System Organ Class"
    )
)

# AE term with at least one term with more than 70% in the treatment
getSummaryStatisticsTable(
    data = subset(dataAE, TRTEMFL == "Y"),
    rowVar = c("AESOC", "AEDECOD"),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    filterFct = function(x)
      ddply(x, "AEDECOD", function(xTerm) {
            if(any(xTerm$statPercN >= 70))	xTerm		
          }),
    labelVars = labelVars,
    title = "Table: Adverse Events by System Organ Class and Preferred Term with at least 70% patients"
)


## ----countTable-AE-filterFct- details-----------------------------------------------------------------------------------------------------------------------------------

x <- computeSummaryStatisticsTable(
    data = subset(dataAE, TRTEMFL == "Y"),
    rowVar = c("AESOC", "AEDECOD"),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    labelVars = labelVars
)
head(x)

# for specific AEDECOD
xTerm <- subset(x, AEDECOD == "MYOCARDIAL INFARCTION")
# identify the record for treated patient:
subset(xTerm, grepl("Placebo", TRTA))
# keep all records (placebo + treatment) if percentage if higher than 20%:
# if no 'else' condition, nothing (NULL) is returned:
if(subset(xTerm, grepl("Placebo", TRTA))$statPercN >= 20)	xTerm
# across all AE terms:
ddply(x, "AEDECOD", function(xTerm)
      if(subset(xTerm, grepl("Placebo", TRTA))$statPercN >= 20)	xTerm
)

# format it as a function and pass it to the 'filterFct' parameter

## ----includeSessionInfo, echo = FALSE, results = "asis"-----------------------------------------------------------------------------------------------------------------
print(sessionInfo())

## ----options, echo = FALSE----------------------------------------------------------------------------------------------------------------------------------------------

library(knitr)
opts_chunk$set(
    echo = TRUE, results = 'markup', warning = FALSE, 
    # stop document execution if error (not the default)
    error = FALSE, 
    message = FALSE, cache = FALSE,
    fig.width = 8, fig.height = 7,
    fig.path = "./figures_vignette/",
    fig.align = 'center')
options(width = 170)
# instead of warn = 0 by default
# include warnings when they occur in the document
options(warn = 1)


## ----getVignette, eval = FALSE------------------------------------------------------------------------------------------------------------------------------------------
# 
# vignette("inTextSummaryTable-createTables", "inTextSummaryTable")
# 

## ----loadPackages-------------------------------------------------------------------------------------------------------------------------------------------------------

library(inTextSummaryTable)
library(clinUtils)
library(tools) # toTitleCase


## ----loadData-----------------------------------------------------------------------------------------------------------------------------------------------------------

# load example data
data(dataADaMCDISCP01)

dataAll <- dataADaMCDISCP01
labelVars <- attr(dataAll, "labelVars")


## ----createExampleData--------------------------------------------------------------------------------------------------------------------------------------------------

dataAE <-  subset(dataAll$ADAE, SAFFL == "Y" & TRTEMFL == "Y")
dataAEInterest <- subset(dataAE, AESOC %in% c(
        "INFECTIONS AND INFESTATIONS",
        "GENERAL DISORDERS AND ADMINISTRATION SITE CONDITIONS"
    )
)

# ensure that order of elements is the one specified in 
# the corresponding numeric variable
dataAEInterest$TRTA <- reorder(dataAEInterest$TRTA, dataAEInterest$TRTAN)
dataAEInterest$AESEV <- factor(dataAEInterest$AESEV, levels = c("MILD", "MODERATE"))

dataTotalAE <- subset(dataAll$ADSL, TRT01A != "Placebo")
# should contain columns specified in 'colVar'
dataTotalAE$TRTA <- dataTotalAE$TRT01A 



## ----outputType-flextable-----------------------------------------------------------------------------------------------------------------------------------------------

summaryTableFt <- getSummaryStatisticsTable(
    data = dataAEInterest,
    rowVar = c("AESOC", "AEDECOD"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    dataTotal = dataTotalAE,
    labelVars = labelVars,
    title = "Table: Adverse Events by System Organ Class and Preferred Term",
    footer = c(
        "N = number of subjects in the data; n = number of subjects with observation",
        paste("Denominator for percentage calculations is the total number of subjects",
            "per treatment group in the safety population"
        )
    ),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)"),
    outputType = "flextable"
)
class(summaryTableFt)
summaryTableFt


## ----style-report-------------------------------------------------------------------------------------------------------------------------------------------------------

getSummaryStatisticsTable(
    data = dataAEInterest,
    rowVar = c("AESOC", "AEDECOD"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    dataTotal = dataTotalAE,
    labelVars = labelVars,
    title = "Table: Adverse Events by System Organ Class and Preferred Term",
    footer = c(
        "N = number of subjects with data; n = number of subjects with this observation",
        paste("Denominator for percentage calculations is the total number of subjects",
            "per treatment group in the safety population"
        )
    ),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)")
)


## ----style-presentation-------------------------------------------------------------------------------------------------------------------------------------------------

getSummaryStatisticsTable(
    data = dataAEInterest,
    rowVar = c("AESOC", "AEDECOD"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    dataTotal = dataTotalAE,
    labelVars = labelVars,
    title = "Table: Adverse Events by System Organ Class and Preferred Term",
    footer = c(
        "N = number of subjects with data; n = number of subjects with this observation",
        paste("Denominator for percentage calculations is the total number of subjects",
            "per treatment group in the safety population"
        )
    ),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)"),
    style = "presentation"
)


## ----outputType-DT, eval = rmarkdown::pandoc_available()----------------------------------------------------------------------------------------------------------------

getSummaryStatisticsTable(
    data = dataAEInterest,
    rowVar = c("AESOC", "AEDECOD"),
    colVar = "TRTA",
    stats = getStats("n"),
    dataTotal = dataTotalAE,
    labelVars = labelVars,
    title = "Table: Adverse Events by System Organ Class and Preferred Term",
    footer = c(
        "N = number of subjects in the data; n = number of subjects with observation",
        paste("Denominator for percentage calculations is the total number of subjects",
            "per treatment group in the safety population"
        )
    ),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)"),
    outputType = "DT",
	## DT-specific options
	buttons = c() # remove all export buttons
)


## ----outputType-dataframe-----------------------------------------------------------------------------------------------------------------------------------------------

summaryTable <- getSummaryStatisticsTable(
    data = dataAEInterest,
    rowVar = c("AESOC", "AEDECOD"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    dataTotal = dataTotalAE,
    labelVars = labelVars,
    title = "Table: Adverse Events by System Organ Class and Preferred Term",
    footer = c(
        "N = number of subjects in the data; n = number of subjects with observation",
        paste("Denominator for percentage calculations is the total number of subjects",
            "per treatment group in the safety population"
        )
    ),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)"),
    style = "presentation",
    outputType = "data.frame"
)
print(summaryTable, split.table = Inf)

## ----outputType-dataframe-base------------------------------------------------------------------------------------------------------------------------------------------

summaryTableAll <- getSummaryStatisticsTable(
    data = dataAEInterest,
    rowVar = c("AESOC", "AEDECOD"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    dataTotal = dataTotalAE,
    labelVars = labelVars,
    title = "Table: Adverse Events by System Organ Class and Preferred Term",
    footer = c(
        "N = number of subjects in the data; n = number of subjects with observation",
        paste("Denominator for percentage calculations is the total number of subjects",
            "per treatment group in the safety population"
        )
    ),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)"),
    style = "presentation",
    outputType = "data.frame-base"
)
print(summaryTableAll, split.table = Inf)

## ----aesthetics-changeReportingOptions----------------------------------------------------------------------------------------------------------------------------------

# create named vector
customColorTable <- c(
    # black text in the header
    'header' = "#000000",
    # green background in the header
    'headerBackground' = "#74D055FF",
    # black text in the body
    'body' = "#000000", 
    # yellow background for all rows
    'bodyBackground1' = "#FDE725FF",
    'bodyBackground2' = "#FDE725FF",
    # black footer
    'footer' = "#000000",
    # white footer background
    'footerBackground' = "#FFFFFF",
    # black line for footer
    'line' = "#000000"
)
# set options
options(inTextSummaryTable.colors.table.presentation = customColorTable)
# create the table on a dummy data set
getSummaryStatisticsTable(
    data = data.frame(USUBJID = c(1, 2)),
    style = "presentation" 
)


## ----aesthetics-changeDimPageOptions------------------------------------------------------------------------------------------------------------------------------------

# set custom dimension of page for presentation
# in this example, the dimension is the widescreen size
pageDimCustom <- c(7.5, 13.32)
options(inTextSummaryTable.pageDim.presentation = pageDimCustom)
getOption("inTextSummaryTable.pageDim.presentation")


## ----getVignetteAesthetics, eval = FALSE--------------------------------------------------------------------------------------------------------------------------------
# 
# vignette("inTextSummaryTable-aesthetics", "inTextSummaryTable")
# 

## ----aesthetics-backDefaultPalettes-------------------------------------------------------------------------------------------------------------------------------------

options(inTextSummaryTable.colors.table.presentation = tableColorsPresentation)


## ----file, eval = FALSE-------------------------------------------------------------------------------------------------------------------------------------------------
# 
# # export table to a Word document
# summaryTableFt <- getSummaryStatisticsTable(
#     data = dataAEInterest,
#     rowVar = c("AESOC", "AEDECOD"),
#     colVar = "TRTA",
#     stats = getStats("n (%)"),
#     dataTotal = dataTotalAE,
#     labelVars = labelVars,
#     file = file.path("tables_CSR", "summaryTable-AEs.docx")
# )
# 
# # export interactive table to a html document
# summaryTableFt <- getSummaryStatisticsTable(
#     data = dataAEInterest,
#     rowVar = c("AESOC", "AEDECOD"),
#     colVar = "TRTA",
#     stats = getStats("n (%)"),
#     dataTotal = dataTotalAE,
#     labelVars = labelVars,
#     file = file.path("tables_CSR", "summaryTable-AEs.html")
# )
# 
# # export table in raw format to a text file
# summaryTableFt <- getSummaryStatisticsTable(
#     data = dataAEInterest,
#     rowVar = c("AESOC", "AEDECOD"),
#     colVar = "TRTA",
#     stats = getStats("n (%)"),
#     dataTotal = dataTotalAE,
#     labelVars = labelVars,
#     file = file.path("tables_CSR", "summaryTable-AEs.txt")
# )
# 
# # export to multiple formats at once
# summaryTableFt <- getSummaryStatisticsTable(
#     data = dataAEInterest,
#     rowVar = c("AESOC", "AEDECOD"),
#     colVar = "TRTA",
#     stats = getStats("n (%)"),
#     dataTotal = dataTotalAE,
#     labelVars = labelVars,
#     file = file.path("tables_CSR",
#         c("summaryTable.txt", "summaryTable.docx", "summaryTable.html")
#     )
# )
# 

## ----export-------------------------------------------------------------------------------------------------------------------------------------------------------------

dataDIABP <- subset(
	dataAll$ADVS, 
	SAFFL == "Y" & ANL01FL == "Y" &
	PARAMCD == "DIABP" & 
	AVISIT %in% c("Baseline", "Week 6") &
	ATPT == "AFTER LYING DOWN FOR 5 MINUTES"
)

# create example of data.frame containing statistics of interest
statsEff <- sapply(c("AVAL", "CHG"), function(var) {
      
      getStats(
          type = c("n", "mean (se)", "median (range)"),
          x = dataDIABP[[var]]
      )
      
    }, simplify = FALSE)

summaryTable <- computeSummaryStatisticsTable(
    data = dataDIABP,
    colVar = c("TRTP", "AVISIT"),
    var = c("AVAL", "CHG"), varGeneralLab = "",
    stats = statsEff,
    labelVars = labelVars
)

# format df with statistics to in-text table format
export(
    summaryTable = summaryTable,
    statsVar = c("n", "Mean (SE)", "Median (range)"),
    rowVar = "variable", rowVarLab = "Statistic",
    colVar = c("TRTP", "AVISIT"),
    colHeaderTotalInclude = TRUE,
    labelVars = simpleCap(tolower(labelVars[c("AVAL", "CHG")])),
    title = toTitleCase(
		"Table: Diastolic Blood Pressure (mmHg) statistics"
    )
)


## ----summaryTable-PP-rowVarWithLabel-rowVarLab-superscript--------------------------------------------------------------------------------------------------------------

dataSL <- subset(dataAll$ADSL, SAFFL == "Y")

varsSL <- c("AGE", "WEIGHTBL", "BMIBL")
labelVars[varsSL] <- c(
    "Age",
    "Weight_{t}",
    "BMI (kg/m^{2})"
)

getSummaryStatisticsTable(
    data = dataSL, 
    var = varsSL,  
    stats = getStats("n (%"),
    labelVars = labelVars,
    fontsize = 16,
    title = toTitleCase("Demographic data (Safety Analysis Set)")
)


## ----summaryTable-PP-rowVarWithLabel-rowVarLab-bold---------------------------------------------------------------------------------------------------------------------

getSummaryStatisticsTable(
    data = dataSL, 
    var = varsSL,  
    stats = list(
		expression(paste0(
			ifelse(statMean > statMedian, 
				paste0("bold{", roundHalfUpTextFormat(statMean, 1), "}"), 
				roundHalfUpTextFormat(statMean, 1)
			), 
			"\n(", 
				roundHalfUpTextFormat(statSE, 2),")")
        )
    ),
    labelVars = labelVars,
    fontsize = 12,
    title = toTitleCase("Demographic data (Safety Analysis Set)")
)


## ----'objectsList1', results = 'asis', echo = FALSE---------------------------------------------------------------------------------------------------------------------
xList[[1]]

## ----'objectsList2', results = 'asis', echo = FALSE---------------------------------------------------------------------------------------------------------------------
xList[[2]]

## ----'objectsList3', results = 'asis', echo = FALSE---------------------------------------------------------------------------------------------------------------------
xList[[3]]

## ----byVar, results = "asis"--------------------------------------------------------------------------------------------------------------------------------------------

summaryTableList <- getSummaryStatisticsTable(
    data = subset(dataAE, TRTEMFL == "Y"),
    rowVar = c("AESOC", "AEDECOD"),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)"),
    colVar = "TRTA",
    byVar = "TRTA",
    stats = getStats("n (%)"),
    labelVars = labelVars,
    title = "Table: Treatment-Emergent adverse events"
)

# print the list of tables in the rmarkdown document
clinUtils::knitPrintListObjects(summaryTableList) 


## ----getListing---------------------------------------------------------------------------------------------------------------------------------------------------------

varsListing <- c(
	"USUBJID", "AEBODSYS", "AEDECOD", "TRTA", 
	"AESEV", "AESER", "ASTDY", "AENDY"
)

dataListing <- subset(dataAE, TRTEMFL == "Y" & AESEV == "SEVERE")
dataListing <- dataListing[, varsListing]
colnames(dataListing) <- getLabelVar(var = varsListing, labelVars = labelVars)

getListing(
	data = dataListing, 
	title = "Listing of treatment-emergent severe adverse events",
	includeRownames = FALSE
)


## ----countTable-AE-filterFct--------------------------------------------------------------------------------------------------------------------------------------------

library(plyr)

# SOC with AE terms with at least 2 subjects
getSummaryStatisticsTable(
    data = subset(dataAE, TRTEMFL == "Y"),
    rowVar = c("AESOC", "AEDECOD"),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    filterFct = function(x)
      ddply(x, "AESOC", function(y)
            if(any(y$statN >= 2))	y			
      ),
    labelVars = labelVars,
    title = paste(
        "Table: Adverse Events by System Organ Class and",
        "Preferred Term with at least 2 patients in System Organ Class"
    )
)

# AE term with at least one term with more than 70% in the treatment
getSummaryStatisticsTable(
    data = subset(dataAE, TRTEMFL == "Y"),
    rowVar = c("AESOC", "AEDECOD"),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    filterFct = function(x)
      ddply(x, "AEDECOD", function(xTerm) {
            if(any(xTerm$statPercN >= 70))	xTerm		
          }),
    labelVars = labelVars,
    title = "Table: Adverse Events by System Organ Class and Preferred Term with at least 70% patients"
)


## ----countTable-AE-filterFct- details-----------------------------------------------------------------------------------------------------------------------------------

x <- computeSummaryStatisticsTable(
    data = subset(dataAE, TRTEMFL == "Y"),
    rowVar = c("AESOC", "AEDECOD"),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    labelVars = labelVars
)
head(x)

# for specific AEDECOD
xTerm <- subset(x, AEDECOD == "MYOCARDIAL INFARCTION")
# identify the record for treated patient:
subset(xTerm, grepl("Placebo", TRTA))
# keep all records (placebo + treatment) if percentage if higher than 20%:
# if no 'else' condition, nothing (NULL) is returned:
if(subset(xTerm, grepl("Placebo", TRTA))$statPercN >= 20)	xTerm
# across all AE terms:
ddply(x, "AEDECOD", function(xTerm)
      if(subset(xTerm, grepl("Placebo", TRTA))$statPercN >= 20)	xTerm
)

# format it as a function and pass it to the 'filterFct' parameter

## ----includeSessionInfo, echo = FALSE, results = "asis"-----------------------------------------------------------------------------------------------------------------
print(sessionInfo())

## ----options, echo = FALSE----------------------------------------------------------------------------------------------------------------------------------------------

library(knitr)
opts_chunk$set(
    echo = TRUE, 
    results = 'markup', warning = FALSE, 
    # stop document execution if error (not the default)
    error = FALSE, 
    message = FALSE, cache = FALSE,
    fig.width = 8, fig.height = 7,
    fig.path = "./figures_vignette/",
    fig.align = 'center')
options(width = 170)
# instead of warn = 0 by default
# include warnings when they occur in the document
options(warn = 1)


## ----loadPackages-------------------------------------------------------------------------------------------------------------------------------------------------------
library(inTextSummaryTable)
library(clinUtils)

## ----loadData-----------------------------------------------------------------------------------------------------------------------------------------------------------

# load example data
data(dataADaMCDISCP01)

dataAll <- dataADaMCDISCP01
labelVars <- attr(dataAll, "labelVars")


## ----visualization-extractData------------------------------------------------------------------------------------------------------------------------------------------

dataLB <- subset(dataAll$ADLBC, grepl("Baseline|Week", AVISIT))
dataLB$AVISIT <- with(dataLB, reorder(trimws(AVISIT), AVISITN))
dataLB$TRTA <- with(dataLB, reorder(TRTA, TRTAN))

summaryTableDf <- computeSummaryStatisticsTable(
    data = dataLB,
    var = "AVAL",
    rowVar = c("PARCAT1", "PARAM"),
    colVar = c("TRTA", "AVISIT")
)


## ----visualization-a4document-------------------------------------------------------------------------------------------------------------------------------------------

# create the plot
dataPlot <- subset(
    summaryTableDf, 
    !isTotal &
     PARAM == "Alanine Aminotransferase (U/L)"
)

subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    colorVar = "TRTA",
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)


## ----visualization-presentation, eval = requireNamespace("extrafont", quietly = TRUE) && "Tahoma" %in% extrafont::fonts()-----------------------------------------------
# # 'Tahoma' font should be registered upfront to create plots with: 'presentation' style
# subjectProfileSummaryPlot(
#     data = dataPlot,
#     xVar = "AVISIT",
#     colorVar = "TRTA",
#     labelVars = labelVars,
#     useLinetype = TRUE,
#     tableText = "statN",
#     style = "presentation"
# )
# 

## ----getVignetteClinUtils, eval = FALSE---------------------------------------------------------------------------------------------------------------------------------
# 
# vignette("clinUtils-vignette", "clinUtils")
# 

## ----visualization-customColorFromArgument------------------------------------------------------------------------------------------------------------------------------

# custom color palette by setting a named vector of colors
customColorPalette <- c(
    `Xanomeline Low Dose` = "green",
    `Xanomeline High Dose` = "red",
	`Placebo` = "blue"
)
subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    colorVar = "TRTA",
    colorPalette = customColorPalette,
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)


## ----visualization-customShapeFromArgument------------------------------------------------------------------------------------------------------------------------------

# custom shape palette
customShapePalette <- c(15, 17, 19)
subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    colorVar = "TRTA",
    shapePalette = customShapePalette,
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)

# custom linetype palette
customLinetypePalette <- c("twodash", "dotted", "longdash")
subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    colorVar = "TRTA",
    linetypePalette = customLinetypePalette,
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)


## ----exampleLoadingPackageAndOptions, eval = FALSE----------------------------------------------------------------------------------------------------------------------
# 
# # this is OK
# library(inTextSummaryTable)
# options(inTextSummaryTable.colors.plot = customColorPalette)
# 
# # this does NOT set your custom palette
# options(inTextSummaryTable.colors.plot = customColorPalette)
# library(inTextSummaryTable)
# 

## ----visualization-customColorFromOptions-------------------------------------------------------------------------------------------------------------------------------

# custom color palette by setting a simple vector of colors
customColorPalette <- c("blue", "green", "orange")
options(inTextSummaryTable.colors.plot = customColorPalette)

subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    colorVar = "TRTA",
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)

# custom color palette by setting a named vector of colors
customColorPalette <- c(
    `Xanomeline Low Dose` = "green",
    `Xanomeline High Dose` = "red",
	`Placebo` = "purple"
)
options(inTextSummaryTable.colors.plot = customColorPalette)

subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    colorVar = "TRTA",
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)


## ----visualization-customShapeFromOptions-------------------------------------------------------------------------------------------------------------------------------

# custom shape palette
customShapePalette <- c(17, 19, 22)
options(inTextSummaryTable.shapes.plot = customShapePalette)

subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    colorVar = "TRTA",
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)

# custom linetype palette
customLinetypePalette <- c("dotted", "longdash", "solid")
options(inTextSummaryTable.linetypes.plot = customLinetypePalette)

subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    colorVar = "TRTA",
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)


## ----visualization-backDefaultPalettes----------------------------------------------------------------------------------------------------------------------------------

options(inTextSummaryTable.colors.plot = clinColors)
options(inTextSummaryTable.shapes.plot = clinShapes)
options(inTextSummaryTable.linetypes.plot = clinLinetypes)


## ----visualization-table------------------------------------------------------------------------------------------------------------------------------------------------

subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    colorVar = "TRTA",
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = NULL
)


## ----visualization-facets, out.width = "100%", fig.height = 9, fig.width = 9--------------------------------------------------------------------------------------------

# create the plot
dataPlotFacets <- subset(
    summaryTableDf, 
    !isTotal &
        PARAM %in% c(
            "Alanine Aminotransferase (U/L)",
            "Albumin (g/L)",
            "Bilirubin (umol/L)",
            "Calcium (mmol/L)"
        )
)

subjectProfileSummaryPlot(
    data = dataPlotFacets,
    xVar = "AVISIT",
    colorVar = "TRTA",
    labelVars = labelVars,
    facetVar = c("PARCAT1", "PARAM"),
    useLinetype = TRUE
)


## ----visualization-otherInputs------------------------------------------------------------------------------------------------------------------------------------------

# plot the median instead of mean
# no error bars
subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    meanVar = "statMedian",
    seVar = NULL,
    colorVar = "TRTA",
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)

# plot the mean with standard deviation
subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    seVar = "statSD",
    colorVar = "TRTA",
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)

# change labels
subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    xLab = "Time points",
    yLab = "Mean and Standard Errors",
    title = "Title of the plot",
    colorVar = "TRTA",
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)


## ----includeSessionInfo, echo = FALSE, results = "asis"-----------------------------------------------------------------------------------------------------------------
print(sessionInfo())

## ----options, echo = FALSE----------------------------------------------------------------------------------------------------------------------------------------------
	
	library(knitr)
	opts_chunk$set(
		echo = TRUE, results = 'asis', warning = FALSE, 
		# stop document execution if error (not the default)
		error = FALSE, message = FALSE, cache = FALSE,
		fig.width = 8, fig.height = 7,
		fig.path = "./figures_vignette/",
		fig.align = 'center')
	# instead of warn = 0 by default
	# include warnings when they occur in the document
	options(width = 170)
	

## ----loadPackages, warning = FALSE--------------------------------------------------------------------------------------------------------------------------------------

	library(clinUtils)
	library(tools)# toTitleCase
	library(plyr) # for ddply, rbind.fill
	library(inTextSummaryTable)


## ----loadData-----------------------------------------------------------------------------------------------------------------------------------------------------------

	# load example data
    library(clinUtils)

    # load example data
    data(dataADaMCDISCP01)

    dataAll <- dataADaMCDISCP01
    labelVars <- attr(dataAll, "labelVars")


## ----table-subjectDisposition-------------------------------------------------------------------------------------------------------------------------------------------
	
	# data of interest
	dataDM <- dataAll$ADSL
	
	varDMFL <- grep("FL$", colnames(dataDM), value = TRUE)
	varDMFLLabel <- sub(" Flag$", "", labelVars[varDMFL])
	
	getSummaryStatisticsTable(
		data = dataDM,
		var = varDMFL, varFlag = varDMFL, varGeneralLab = "Analysis Set, N", 
		varLab = varDMFLLabel,
		stats = getStats("n (%)"),
		colVar = "TRT01P",
		labelVars = labelVars,
		colTotalInclude = TRUE, colTotalLab = "All subjects",
		varInclude0 = TRUE,
		title = toTitleCase("Table: subject disposition"),
		file = file.path("tables_CSR", "Table_subjectDisposition.docx")
	)


## ----table-demography---------------------------------------------------------------------------------------------------------------------------------------------------

	# data of interest
	dataDM <- subset(dataAll$ADSL, SAFFL == "Y")
	
	# variables of interest
	# Note: if available: ethnicity is included
	varsDM <- c(
		"SEX", "AGE", "AGEGR1",
		"RACE", "ETHNIC",
		"HEIGHTBL", "WEIGHTBL", 
		"BMIBL", "BMIBLGR1"
	)

	# Sort variables according to corresponding numeric variable
	dataDM$AGEGR1 <- with(dataDM, reorder(AGEGR1, AGEGR1N))
	dataDM$RACE <- with(dataDM, reorder(RACE, RACEN))
	dataDM$TRT01P <- with(dataDM, reorder(TRT01P, TRT01PN))
	
	## Define set of statistics of interest:
	statsDM <- getStatsData(
		data = dataDM, var = varsDM,
		# different for continuous and categorical variable
		type = c(cont = "median (range)", cat = "n (%)"),
		# for categorical variable, statistic name (here: 'n (%)')
		# should not be included in the table
		args = list(cat = list(includeName = FALSE))
	)

	## create the table:
	
	getSummaryStatisticsTable(
		data = dataDM, 
		# variables to summarize
		var = varsDM, 
		varGeneralLab = "Parameter",
		# column
		colVar = "TRT01P", colTotalInclude = TRUE, colTotalLab = "All subjects",
		# statistics
		stats = statsDM,
		statsGeneralLab = "",
		labelVars = labelVars,
		# if only one category, should be included in separated row (e.g. RACE: White)
		rowAutoMerge = FALSE,
		rowInclude0 = FALSE, emptyValue = 0,
		title = toTitleCase("Table: Demographic Data (safety Analysis Set)"),
		file = file.path("tables_CSR", "Table_demographicData.docx")
	)
	

## ----table-baselineDiseaseCharacteristics-------------------------------------------------------------------------------------------------------------------------------

	# data of interest
	dataBDC <- subset(dataAll$ADSL, SAFFL == "Y")
	
	# create table
	getSummaryStatisticsTable(
		data = dataBDC,
		var = c("DURDIS", "EDUCLVL"), varGeneralLab = "Parameter", 
		colVar = "TRT01P", colTotalInclude = TRUE, colTotalLab = "All subjects",
		stats = getStats("median\n(range)"), statsGeneralLab = "",
		rowAutoMerge = FALSE,
		labelVars = labelVars,
		title = toTitleCase("Table: Baseline Disease Characteristics (safety analysis set)"),
		file = file.path("tables_CSR", "Table_BaselineCharacteristics.docx")
	)


## ----table-MH-----------------------------------------------------------------------------------------------------------------------------------------------------------

	dataCM <- subset(dataAll$ADCM, SAFFL == "Y")

	# sort variable according to corresponding numeric variables
	dataCM$TRTA <- with(dataCM, reorder(TRTA, TRTAN))
	
	# Terms should be in lower-case
	dataCM$CMDECOD <- simpleCap(tolower(dataCM$CMDECOD))
	dataCM$CMCLAS <- simpleCap(tolower(dataCM$CMCLAS))
			
	getSummaryStatisticsTable(
		data = dataCM,
		colVar = "TRTA", colTotalInclude = TRUE, colTotalLab = "All subjects",
		rowVar = c("CMCLAS", "CMDECOD"), 
		# include total across generic terms and across ATC4 classes
		rowVarTotalInclude = c("CMCLAS", "CMDECOD"), 
		rowTotalLab = "Any prior and concomitant medication",
		stats = getStats("n (%)"),
		# sort rows based on counts of subjects in the total column 
		rowOrder = "total",
		labelVars = labelVars,
		emptyValue = 0,
		title = toTitleCase(paste("Prior and concomitant therapies",
			"by medication class and generic term (safety analyis set)"
		)),
		file = file.path("tables_CSR", "Table_CM.docx")
	)
	

## ----table-efficacy-----------------------------------------------------------------------------------------------------------------------------------------------------

	dataAdasCog11 <- subset(dataAll$ADQSADAS, PARAMCD == "ACTOT")
	dataCIBIC <- subset(dataAll$ADQSCIBC, PARAMCD == "CIBICVAL")
	
	dataEfficacy <- plyr::rbind.fill(dataAdasCog11, dataCIBIC)
	
	dataEfficacy$TRTP <- with(dataEfficacy, reorder(TRTP, TRTPN))
	dataEfficacy$AVISIT <- with(dataEfficacy, reorder(AVISIT, AVISITN))
	
	stats <- getStatsData(
		data = dataEfficacy, 
		var = c("AVAL", "CHG"), 
		type = c("n", "mean (se)", "median (range)")
	)
	
	getSummaryStatisticsTable(
		data = dataEfficacy,
		rowVar = "PARAM",
		colVar = c("TRTP", "AVISIT"),
		var = c("AVAL", "CHG"), 
		stats = stats,
		labelVars = labelVars,
		title = paste("Table: efficacy endpoints", 
			toTitleCase("actual value and changes from baseline per time point"				
		)),
		file = file.path("tables_CSR", "Table_efficacy.docx")
	)
	

## ----table-summaryTable-------------------------------------------------------------------------------------------------------------------------------------------------

	## data of interest: safety analysis set and treatment-emergent
	dataTEAE <- subset(dataAll$ADAE, SAFFL == "Y" & TRTEMFL == "Y")
	
	# order treatment and severity categories
	dataTEAE$TRTA <- with(dataTEAE, reorder(TRTA, TRTAN))
	
	## data considered for the total
	dataTotalAE <- subset(dataAll$ADSL, SAFFL == "Y")
	dataTotalAE$TRTA <- with(dataTotalAE, reorder(TRT01A, TRT01AN))
	
	# TEAE with worst intensity
	# build worst-case scenario
	dataTEAE$AESEV <- factor(dataTEAE$AESEV, levels = c("MILD", "MODERATE", "SEVERE"))
	dataTEAE$AESEVN <- as.numeric(dataTEAE$AESEV)
	dataTEAE <- ddply(dataTEAE, c("USUBJID", "TRTA"), function(x)
		cbind.data.frame(x, 
			WORSTINT = with(x, ifelse(AESEVN == max(AESEVN), as.character(AESEV), NA_character_))
	))
	dataTEAE$WORSTINT <- factor(dataTEAE$WORSTINT, levels = levels(dataTEAE$AESEV))
	
	## specify labels for each variable:
	varsAE <- c("TRTEMFL", "AESER", "AESDTH", "AEREL")
	
	# create the table
	getSummaryStatisticsTable(
		data = dataTEAE,
		colVar = "TRTA",
		# define variables to compute statistics on
		var = c("TRTEMFL", "AESER", "WORSTINT", "AESDTH", "AEREL"), 
		varFlag = c("TRTEMFL", "AESER", "AESDTH"),
		varLab = c(TRTEMFL = "Treatment-Emergent", WORSTINT = "Worst-case severity:"),
		varGeneralLab = "Subjects with, n(%):",
		# force the inclusion of lines for variable without count:
		varInclude0 = TRUE,
		# include the total for the worst-case scenario
		varTotalInclude = "WORSTINT",
		# statistics:
		stats = getStats('n (%)'),
		emptyValue = "0",
		labelVars = labelVars,
		# dataset used for the total in the header column (and for percentage as default)
		dataTotal = dataTotalAE,
		# title/export
		title = toTitleCase("Table: Summary Table of Treatment-emergent Adverse Events (safety analysis set)"),
		file = file.path("tables_CSR", "Table_TEAE_summary.docx")
	)
	

## ----table-TEAE---------------------------------------------------------------------------------------------------------------------------------------------------------

	dataTEAE <- subset(dataAll$ADAE, SAFFL == "Y" & TRTEMFL == "Y")
	
	# order treatment and severity categories
	dataTEAE$TRTA <- with(dataTEAE, reorder(TRTA, TRTAN))
	
	## data considered for the total
	dataTotalAE <- subset(dataAll$ADSL, SAFFL == "Y")
	dataTotalAE$TRTA <- with(dataTotalAE, reorder(TRT01A, TRT01AN))
	
	getSummaryStatisticsTable(
		data = dataTEAE,
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		## total
		# data
		dataTotal = dataTotalAE,
		# row total
		rowVarTotalInclude = c("AESOC", "AEDECOD"), rowTotalLab = "Any TEAE",
		stats = getStats("n (%)"),
		labelVars = labelVars,
		rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term,\nn (%)"),
		# sort rows based on the total column:
		rowOrder = "total", 
		rowOrderTotalFilterFct = function(x) subset(x, TRTA == "Total"),
		title = paste("Table: Treatment-emergent Adverse Events by System Organ Class",
			"and Preferred Term (Safety Analysis Set)"
		),
		file = file.path("tables_CSR", "Table_TEAE_SOCPT_atLeast1Subject.docx")
	)


## ----table-TEAE-inAtLeast25Percent--------------------------------------------------------------------------------------------------------------------------------------

	getSummaryStatisticsTable(
		data = dataTEAE,
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		## total
		# data
		dataTotal = dataTotalAE, 
		# row total
		rowVarTotalInclude = c("AESOC", "AEDECOD"), rowTotalLab = "Any TEAE",
		stats = getStats("n (%)"),
		labelVars = labelVars,
		rowVarLab = c('AESOC' = "SOC and Preferred Term,\nn (%)"),
		# sort rows based on the total column:
		rowOrder = "total", 
		rowOrderTotalFilterFct = function(x) subset(x, TRTA == "Total"),
		title = paste("Table: Treatment-emergent Adverse Events by System Organ Class",
			"and Preferred Term reported in at least 25% of the subjects",
			"in any treatment group (Safety Analysis Set)"
		),
		file = file.path("tables_CSR", "Table_TEAE_SOCPT_atLeast25PercentsSubject.docx"),
		# include only events occuring in at least 25% for at least one preferred term:
		filterFct = function(x)
			ddply(x, "AESOC", function(x){ # per AESOC to include the total
				ddply(x, "AEDECOD", function(y){
					yTotal <- subset(y, grepl("Total", TRTA))
					if(any(yTotal$statPercN >= 25))	y
				})
			})
	)


## ----tableTEAE-worstCase------------------------------------------------------------------------------------------------------------------------------------------------

	dataTEAE <- subset(dataAll$ADAE, SAFFL == "Y" & TRTEMFL == "Y")
	
	# order treatment and severity categories
	dataTEAE$TRTA <- with(dataTEAE, reorder(TRTA, TRTAN))
	
	## data considered for the total
	dataTotalAE <- subset(dataAll$ADSL, SAFFL == "Y")
	dataTotalAE$TRTA <- with(dataTotalAE, reorder(TRT01A, TRT01AN))
	
	# TEAE with worst intensity
	dataTEAE$AESEV <- factor(dataTEAE$AESEV, levels = c("MILD", "MODERATE", "SEVERE"))
	dataTEAE$AESEVN <- as.numeric(dataTEAE$AESEV)
	
	# extract worst-case scenario data (only one record if multiple with same severity)
	dataAEWC <- ddply(dataTEAE, c("AESOC", "AEDECOD", "USUBJID", "TRTA"), function(x){
		x[which.max(x$AESEVN), ]
	})
	# worst-case scenario in lower case
	dataAEWC$WORSTINT <- simpleCap(tolower(dataAEWC$AESEV))
	labelVars["WORSTINT"] <- "Worst-case scenario"

	## datasets used for the total: 
	# for total: compute worst-case across SOC and across AE term
	# (otherwise patient counted in multiple categories if present different categories for different AEs)
	dataTotalRow <- list(
		# within SOC (across AEDECOD)
		'AEDECOD' = ddply(dataAEWC, c("AESOC", "USUBJID", "TRTA"), function(x){	
			x[which.max(x$AESEVN), ]
		}),
		# across SOC
		'AESOC' = ddply(dataAEWC, c("USUBJID", "TRTA"), function(x){	
			x[which.max(x$AESEVN), ]
		})
	)
	
	getSummaryStatisticsTable(
		data = dataAEWC,
		## row variables:
		rowVar = c("AESOC", "AEDECOD", "WORSTINT"), rowVarInSepCol = "WORSTINT",
		# include total across SOC and across AEDECOD
		rowVarTotalInclude = c("AESOC", "AEDECOD"), dataTotalRow = dataTotalRow, 
		rowVarTotalByVar = "WORSTINT", # count for each severity category for the total
		rowTotalLab = "Any TEAE", rowVarLab = c(AESOC = "Subjects with, n(%):", WORSTINT = "Worst-case scenario"),
		# sort per total in the total column
		rowOrder = "total", 
		## column variables
		colVar = "TRTA", 
		stats = getStats("n (%)"),
		emptyValue = "0",
		labelVars = labelVars,
		dataTotal = dataTotalAE,
		title = toTitleCase(paste("Table: Treatment-emergent Adverse",
			"Events by system organ",
			"and preferred term by worst-case (safety Analysis Set)"
		)),
		file = file.path("tables_CSR", "Table_TEAE_Severity.docx")
	)
	

## ----tableLab-----------------------------------------------------------------------------------------------------------------------------------------------------------

	dataLBAbn <- subset(dataAll$ADLBC, SAFFL == "Y" & LBNRIND != "NORMAL")
	
	dataLBAbn$PARAM <- with(dataLBAbn, reorder(PARAM, PARAMN))
	dataLBAbn$TRTA <- with(dataLBAbn, reorder(TRTA, TRTAN))
	dataLBAbn$LBNRIND <- factor(dataLBAbn$LBNRIND, levels = c("LOW", "HIGH"))

	dataLBAbnTotal <- subset(dataAll$ADSL, SAFFL == "Y")
	dataLBAbnTotal$TRTA <- with(dataLBAbnTotal, reorder(TRT01A, TRT01AN))
	
	getSummaryStatisticsTable(
		data = dataLBAbn,
		rowVar = c("PARCAT1", "PARAM"), 
		rowVarTotalInclude = c("PARCAT1", "PARAM"),
		colVar = "TRTA", 
		var = "LBNRIND", 
		rowVarInSepCol = "variableGroup", varSubgroupLab = "Abnormality",
		rowVarLab = c('PARCAT1' = "Laboratory Parameter\nn (%)"),
		stats = getStats("n (%)"),
		labelVars = labelVars,
		rowOrder = c("PARCAT1" = "total", "PARAM" = "total", "variableGroup" = "auto"),
		dataTotal = dataLBAbnTotal, 
		title = toTitleCase(paste("Table: Treatment-emergent",
			"Worst-case Laboratory Abnormalities (safety analysis set)"
		)),
		emptyValue = "0",
		file = file.path("tables_CSR", "Table_Lab_Severity.docx")
	)


## ----ECG-formatData, eval = FALSE---------------------------------------------------------------------------------------------------------------------------------------
# 
# 	# data of interest
# 	paramsECG <- c("QT", "QTCF", "QRS", "PR", "RR", "EGHR")
# 
# 	dataECG <- subset(dataAll$ADEG, SAFFL == "Y" & PARAMCD %in% paramsECG)
# 	dataECG$TRTA <- with(dataECG, reorder(TRTA, TRTAN))
# 	dataECG$PARAM <- with(dataECG, reorder(PARAM, PARAMN))
# 	
# 	# consider all non-missing post-baseline records
# 	dataECGPostBaseline <- subset(dataECG,
# 		AVISIT %in% c("Screening", "Baseline", "Worst-case post-baseline")
# 	)
# 	
# 	# worst-case scenario:
# 	dataECGWC <- subset(dataECG, AVISIT == "Worst-case post-baseline")
# 	# treatment-emergent
# 	dataECGWC$TRTEMFL <- with(dataECGWC, ifelse(BASECAT1 != CHGCAT1, "Y", "N"))
# 	dataECGWCTE <- subset(dataECGWC, TRTEMFL == "Y")
# 	dataECGWC <- convertVarToFactor(dataECGWC,
# 		var = c("AVALCAT1", "CHGCAT1"),
# 		varNum = c("AVALCA1N", "CHGCAT1N")
# 	)
# 	
# 	# create the table
# 	getSummaryStatisticsTable(
# 		data = dataECGWC,
# 		# layout:
# 		colVar = "TRTA",
# 		rowVar = "PARAM", rowVarLab = c('PARAM' = "ECG Parameter"),
# 		# metrics to compute statistics on
# 		var = c("AVALCAT1", "CHGCAT1"),
# 		# in a separated column
# 		rowVarInSepCol = c("variable", "variableGroup"),
# 		# labels
# 		varGeneralLab = "Abnormality",
# 		varSubgroupLab = "Worst-Case Post-Baseline",
# 		stats = getStats("n (%)"),
# 		labelVars = labelVars,
# 		# total: all post-baseline
# 		dataTotal = dataECGPostBaseline,
# 		emptyValue = "0",
# 		rowVarTotalPerc = "PARAM", # total per parameter
# 		# ensure that categories are below the type of abnormality
# 		rowAutoMerge = FALSE,
# 		# only retain abnormalities:
# 		filterFct = function(x){
# 			subset(x, !variableGroup %in% c("<= 450 msec", "<= 30 msec"))
# 		},
# 		title = toTitleCase(paste("Table: Treatment-emergent worst-case",
# 			"ECG abnormalities and change from baseline ECG abnormalities (safety analysis set)"
# 		)),
# 		file = file.path("tables_CSR", "Table_ECG.docx")
# 	)
# 	

## ----tableVitalSigns----------------------------------------------------------------------------------------------------------------------------------------------------

	# analyis set and parameters of interest
	dataVS <- subset(dataAll$ADVS, 
		SAFFL == "Y" & ANL01FL == "Y" & VISIT != "BASELINE"
	)
	
	dataVS$PARAM <- with(dataVS, reorder(PARAM, PARAMN))
	dataVS$ANRIND <- with(dataVS, reorder(PARAM, PARAMN))
	dataVS$TRTA <- with(dataVS, reorder(TRTA, TRTAN))
	dataVS$SHIFT1 <- with(dataVS, factor(ifelse(SHIFT1 == "", NA_character_, SHIFT1)))
			
	getSummaryStatisticsTable(
		data = dataVS,
		rowVar = "PARAM", 
		rowVarInSepCol = "variableGroup", 
		rowVarInclude0 = TRUE,
		colVar = "TRTA", 
		var = "SHIFT1", varTotalInclude = TRUE,
		emptyValue = 0,
		stats = getStats("n (%)"),
		rowVarTotalPerc = "PARAM",
		labelVars = labelVars,
		title = toTitleCase(paste("Table: Treatment-emergent Worst-case",
			"Vital Sign Abnormalities (Safety Analysis Set)"
		)),
		file = file.path("tables_CSR", "Table_VitalSigns_Severity.docx")
	)


## ----PK-----------------------------------------------------------------------------------------------------------------------------------------------------------------

	paramcdPK <- c("AUCINFO", "CMAX", "TMAX")
	dataPK <- subset(dataAll$ADPP, PKFL == "Y" & PARAMCD %in% paramcdPK)
	
	dataPK$PARCAT1 <- with(dataPK, reorder(PARCAT1, PARCAT1N))
	dataPK$PARAMCD <- with(dataPK, reorder(PARAMCD, PARAMN))
	dataPK$TRTA <- with(dataPK, reorder(TRTA, TRTAN))
	dataPK$PARAMCD <- with(dataPK, reorder(PARAMCD, PARAMN))
	
	# build pretty labels
	labelsPK <- c(
		AUCINFO = "AUC_{Inf,obs}\n(h*ng/mL)",
		CMAX = "C_{max}\n(ng/mL)",
		TMAX = "t_{max}\n(h)"
	)
	dataPK$PARAM <- factor(dataPK$PARAMCD, 
		levels = levels(dataPK$PARAMCD), 
		labels = labelsPK[levels(dataPK$PARAMCD)]
	)
	
	statsPK <- dlply(dataPK, "PARAM", function(dataParam){
		getStatsData(
			data = dataParam,
			var = "AVAL",
			type = "median\n(range)",
			includeName = FALSE
		)[[1]]
	})
	
	getSummaryStatisticsTable(
		data = dataPK,
		rowVar = c("PARCAT1", "PARAM"), colVar = "TRTA",
		var = "AVAL",
#		rowVarLab = c('PARCAT1' = "PK parameters"),
		stats = statsPK, statsVarBy = "PARAM",
		emptyValue = "-",
		title = toTitleCase("Table: Summary of PK parameters (pharmacokinetics analysis set)"),
		file = file.path("tables_CSR", "Table_PK_Parameters.docx"),
		labelVars = labelVars
	)


## ----includeSessionInfo, echo = FALSE, results = "asis"-----------------------------------------------------------------------------------------------------------------
print(sessionInfo())

#' Combine objects
#' @param ... Extra parameters for the corresponding
#' method.
#' @seealso \code{\link{combine.summaryTable}}
#' to combine \code{\link{summaryTable}} objects.
#' @export
combine <- function(...) UseMethod("combine")

#' Combine summary statistics table
#' 
#' \itemize{
#' \item{A new table is created, combining the tables by rows.}
#' \item{The attributes of the combined summary table are 
#' obtained by combining the attributes of all summary tables
#' (and removing duplicates).}
#' }
#' 
#' \itemize{
#' \item{In case only a set of tables contain categorical variable,
#' in nested rows, so the \code{variable} and \code{variableGroup}
#' variables, these variables are included as last in 
#' the 'row variables' attribute of the combined table.}
#' \item{Only one row is retained for the columns totals
#' per column variable (the first one in order of appearance).\cr
#' The column totals are not included if the column
#' variable(s) are not the same across tables, or
#' if the totals differ between tables.}
#' }
#' @param ... \code{\link{summaryTable}} objects.
#' @param summaryTables List of \code{\link{summaryTable}} objects.
#' @param combineVar (optional) String with name
#' of a new variable tracking from which table
#' each record originally come from.\cr
#' The label for each table is extracted from the names
#' of the \code{summaryTables} list, or 1:length(tables)
#' if the list is not named.\cr
#' If not specified (by default), the tables will be combined
#' but the information on which input table:
#' each record from the combined table belongs to will
#' not be retained.
#' @param combineDir String indicating
#' on which direction: 'row' or 'col' (a.k.a column)
#' the information on the table appartenance 
#' (\code{combineVar}) will
#' be displayed when the table is exported.
#' @return A combined \code{\link{summaryTable}}.
#' @author Laure Cougnaud
#' @importFrom plyr rbind.fill
#' @method combine summaryTable
#' @export
combine.summaryTable <- function(..., 
	summaryTables, 
	combineVar = NULL,
	combineDir = c("row", "col")){

	combineDir <- match.arg(combineDir)
	
	if(missing(summaryTables))
		summaryTables <- list(...)
	
	# get attributes
	attrs <- lapply(summaryTables, attr, 
		which = "summaryTable", exact = TRUE
	)
	
	# in case only one categorical variable is specified
	# the variable should be included as well
	
	# add label to identify the tables (if specified)
	if(!is.null(combineVar)){
		
		if(combineVar %in% unlist(lapply(summaryTables, colnames)))
			stop(shQuote(combineVar), 
				" is already available in the summary table, ",
				"please specify a different name for 'combineVar'.")
		
		if(is.null(names(summaryTables))){
			names(summaryTables) <- as.character(seq_along(summaryTables))
		}
		
		summaryTables <- sapply(names(summaryTables), function(name){
			summaryTable <- summaryTables[[name]]
			summaryTable[[combineVar]] <- factor(
				name, levels = names(summaryTables)
			)
			summaryTable
		}, simplify = FALSE)

	}
	
	# combine tables
	summaryTableCombine <- do.call(plyr::rbind.fill, summaryTables)
		
	# combine attributes
	attrNames <- lapply(attrs, names)
	hasEmptyAttrNames <-
		sapply(attrNames, is.null) | 
		sapply(attrNames, function(x) any(x == ""))
	if(any(hasEmptyAttrNames))
		stop("The summary table attribute should be named for all tables.")
	# add 'combine' variable as row or column variable
	if(!is.null(combineVar)){
		attrCombine <- switch(
			combineDir, 
			`row` = "rowVar", 
			`col` = "colVar"
		)
		attrs <- sapply(attrs, function(x){
			x[[attrCombine]] <- c(combineVar, x[[attrCombine]])	
			x
		}, simplify = FALSE)
	}
	attrsCombine <- tapply(
		X = unlist(attrs, use.names = FALSE, recursive = FALSE), 
		INDEX = unlist(lapply(attrs, names)),
		FUN = uniqueVarWithOrder
	)
	
	# combine the column total across tables
	# Note: attr should have been updated with combine variable
	# (in case combineVar is included in columns)
	summaryTableCombine <- combineColTotal(
		summaryTable = summaryTableCombine,
		attrs = attrs
	)
	
	# Check that tables have sufficient information to identify records uniquely
	rowColVars <- c(attrsCombine[["rowVar"]], attrsCombine[["colVar"]])
	idxTotal <- which(summaryTableCombine$isTotal)
	summaryTableNoTotal <- if(length(idxTotal) > 0){
		summaryTableCombine[-idxTotal, ]
	}else	summaryTableCombine
	rowColVarsCombine <- apply(summaryTableNoTotal[, rowColVars, drop = FALSE], 1, paste, collapse = ".")
	if(any(rowColVarsCombine == "NA") || any(duplicated(rowColVarsCombine)))
		stop("Summary tables cannot be combined.\n",
			"You might want to set combineVar or ",
			"create your table with varLabInclude to TRUE.")
		
	attr(summaryTableCombine, "summaryTable") <- attrsCombine
	class(summaryTableCombine) <- c("summaryTable", class(summaryTableCombine))
	
	return(summaryTableCombine)
	
}


#' Get unique variables with meaningful order.
#' 
#' The following framework is followed:
#' \enumerate{
#' \item{get the unique elements in the vectors}
#' \item{for each of this element: get the average order across
#' the different vectors}
#' \item{put \code{variable}, if present as second to last element}
#' \item{put \code{variableGroup}, if present, as last element}
#' \item{order the unique elements based on the extracted order}
#' }
#' @param ... Lists
#' @return Vector with unique and ordered elements.
#' @author Laure Cougnaud
uniqueVarWithOrder <- function(...){
	
	x <- c(...)
	if(!is.list(x))
		stop("Input should be lists.")
	
	# extract unique elements
	xUnique <- unique(unlist(x))
	
	# match per position
	xUniqueAll <- lapply(x, match, x = xUnique)
	# extract mean position
	xOrder <- do.call(mapply, c(xUniqueAll, list(FUN = function(...){mean(c(...), na.rm = TRUE)})))
	
	# set variable & variableGroup as last elements
	xOrder[which(xUnique == "variableGroup")] <- length(xUnique)
	xOrder[which(xUnique == "variable")] <- length(xUnique)-1
	
	# order vector
	xUniqueOrdered <- xUnique[order(xOrder, decreasing = FALSE)]
	
	return(xUniqueOrdered)
	
}

#' Combine the column total for a combined summary table
#' @param summaryTable Combined summary table
#' @param attrs Nested list with attributes of each
#' summary table.
#' @return summary table, with combined total columns
#' @importFrom plyr daply
#' @author Laure Cougnaud
combineColTotal <- function(summaryTable, attrs){
	
	# extract column variable(s) for each table
	attrsColVar <- lapply(attrs, function(x) 
		if(hasName(x, "colVar"))	x[["colVar"]])

	# are the column variable(s) identical for all tables?
	isIdenticalList <- function(x, y){
		if(identical(x, y)){
			x
		}else{ 
			warning("Column variable(s) are not identical across tables,",
				" so the column totals are discarded.")
		}
	}
	res <- tryCatch(
		Reduce(isIdenticalList, attrsColVar),
		warning = function(w) return(w)
	)
	# if column variable(s) are not identical, the column
	# totals are removed with a warning 
	if(inherits(res, "warning")){
		warning(res)
		summaryTable <- subset(summaryTable, !isTotal)
	}else{
		colVar <- unique(unlist(attrsColVar))
		nDiffSubjPerCol <- daply(
			subset(summaryTable, isTotal),
			colVar,
			function(x){
				length(unique(x[, "statN"]))
			}
		)
		isTotal <- which(summaryTable$isTotal)
		
		# if the total are the same for all columns
		# replicated rows with column total are removed
		if(all(nDiffSubjPerCol == 1, na.rm = TRUE)){
			isTotalDupl <- which(
				duplicated(summaryTable[isTotal, c(colVar, "statN")])
			)
			isTotalDupl <- isTotal[isTotalDupl]
			if(length(isTotalDupl) > 0){
				summaryTable <- summaryTable[-isTotalDupl, ]
			}
		# if the column total differ between tables
		# totals are removed with a warning 
		}else{
			warning("The column totals differ between the ",
				"tables, so these are discarded.")
			summaryTable <- subset(summaryTable, !isTotal)
		}
	}
	
	return(summaryTable)
	
}

context("Combine summary statistics tables")

library(plyr)

test_that("An error is generated if summary tables without variable names are combined", {
			
	# In case a single variable is included,
	# the variable name is not included in the table
	# so tables cannot be combined
	dataCont <- data.frame(
		USUBJID = c("1", "2"),
		AVAL = c(1, 2)
	)
	summaryTableCont <- computeSummaryStatisticsTable(
		data = dataCont, 
		var = "AVAL"
	)
	dataCat <- data.frame(
		USUBJID = c("1", "2"),
		GROUP = c("A", "B")
	)
	summaryTableCat <- computeSummaryStatisticsTable(
		data = dataCat, 
		var = "GROUP"
	)
	expect_error(
		combine(summaryTableCont, summaryTableCat),
		"Summary tables cannot be combined"
	)
			
})

test_that("Summary tables with only one variable and with similar layout are correctly combined", {
			
	data <- data.frame(
		USUBJID = c("1", "2"),
		AVAL = c(1, 2),
		AVAL2 = c(2, 3)
	)
	summaryTable1 <- computeSummaryStatisticsTable(
		data = data, 
		var = "AVAL",
		varLabInclude = TRUE
	)
	summaryTable2 <- computeSummaryStatisticsTable(
		data = data, 
		var = "AVAL2",
		varLabInclude = TRUE
	)
	summaryTableCombine <- combine(summaryTable1, summaryTable2)
	expect_s3_class(summaryTableCombine, "summaryTable")
			
	# check that content without total is correct
	expect_equal(
		subset(summaryTableCombine, !isTotal),
		rbind(
			subset(summaryTable1, !isTotal), 
			subset(summaryTable2, !isTotal)
		),
		check.attributes = FALSE
	)
			
	# only one row should be retained for the total
	expect_equal(
		subset(summaryTableCombine, isTotal),
		subset(summaryTable1, isTotal), 
		check.attributes = FALSE
	)	
	
	## check attributes
	expect_equal(
		attr(summaryTableCombine, "summaryTable")[["rowVar"]], 
		"variable"
	)
	expect_equal(
		attr(summaryTableCombine, "summaryTable")[["statsVar"]], 
		attr(summaryTable1, "summaryTable")[["statsVar"]]
	)	
	
})

test_that("Summary tables with a variable to summarize, row and column variables and with similar layout are correctly combined", {
			
	data <- data.frame(
		USUBJID = c("1", "2", "1", "2"),
		PARAM = c("a", "a", "b", "b"),
		TRT = c("A", "B", "A", "B"),
		AVAL = c(1, 2 ,3, 4),
		AVAL2 = c(4, 3 , 2, 1),
		stringsAsFactors = FALSE
	)
	summaryTable1 <- computeSummaryStatisticsTable(
		data = data, 
		var = "AVAL", varLabInclude = TRUE,
		colVar = "TRT",
		rowVar = "PARAM"
	)
	summaryTable2 <- computeSummaryStatisticsTable(
		data = data, 
		var = "AVAL2", varLabInclude = TRUE,
		colVar = "TRT",
		rowVar = "PARAM"
	)
	summaryTableCombine <- combine(summaryTable1, summaryTable2)
	expect_s3_class(summaryTableCombine, "summaryTable")
	
	# check that content without total is correct
	expect_equal(
		subset(summaryTableCombine, !isTotal),
		rbind.fill(subset(summaryTable1, !isTotal), 
			subset(summaryTable2, !isTotal)),
		check.attributes = FALSE
	)
	
	# only one row should be retained for the total
	expect_equal(
		subset(summaryTableCombine, isTotal),
		subset(summaryTable1, isTotal), 
		check.attributes = FALSE
	)
	
	## check attributes
	expect_equal(
		attr(summaryTableCombine, "summaryTable")[["rowVar"]], 
		c("PARAM", "variable")
	)
	expect_equal(
		attr(summaryTableCombine, "summaryTable")[["colVar"]], 
		"TRT"
	)
	expect_equal(
		attr(summaryTableCombine, "summaryTable")[["statsVar"]], 
		attr(summaryTable1, "summaryTable")[["statsVar"]]
	)	
			
})

test_that("Summary tables with a categorical and with a continuous variable are successfully combined", {
			
	dataCont <- data.frame(
		USUBJID = c("1", "2"),
		AVAL = c(1, 2)
	)
	summaryTableCont <- computeSummaryStatisticsTable(
		data = dataCont, 
		var = "AVAL",
		varLabInclude = TRUE
	)
	dataCat <- data.frame(
		USUBJID = c("1", "2"),
		GROUP = c("A", "B")
	)
	summaryTableCat <- computeSummaryStatisticsTable(
		data = dataCat, 
		var = "GROUP",
		varLabInclude = TRUE
	)
	summaryTableCombine <- combine(summaryTableCont, summaryTableCat)
	
	# check for the table with categorical variable
	# that initial content is retained
	summaryTableCCat <- subset(summaryTableCombine, !isTotal & variable == "GROUP")
	expect_equal(
		subset(summaryTableCCat, select = colnames(summaryTableCat)),
		subset(summaryTableCat, !isTotal),
		check.attributes = FALSE
	)
	# and extra columns for other table is filled with NA
	summaryTableCCatExtra <- subset(summaryTableCCat, 
		select = setdiff(colnames(summaryTableCCat), 
			colnames(summaryTableCat)
		))
	expect_true(all(is.na(summaryTableCCatExtra)))
	
	# check for the table with continuous variable
	# that initial content is retained
	summaryTableCCont <- subset(summaryTableCombine, !isTotal & variable == "AVAL")
	expect_equal(
		subset(summaryTableCCont, select = colnames(summaryTableCont)),
		subset(summaryTableCont, !isTotal),
		check.attributes = FALSE
	)
	# and extra columns for other table is filled with NA
	summaryTableCContExtra <- subset(summaryTableCCont, 
		select = setdiff(colnames(summaryTableCCont), 
			colnames(summaryTableCont)
	))
	expect_true(all(is.na(summaryTableCContExtra)))
	
	# only one row should be retained for the total
	expect_equal(
		subset(summaryTableCombine, isTotal)[, colnames(summaryTableCat)],
		subset(summaryTableCat, isTotal), 
		check.attributes = FALSE
	)	
	
	## check attributes
	expect_equal(
		attr(summaryTableCombine, "summaryTable")[["rowVar"]],
		c("variable", "variableGroup")
	)
	expect_setequal(
		attr(summaryTableCombine, "summaryTable")[["statsVar"]],
		c(
			attr(summaryTableCat, "summaryTable")[["statsVar"]],
			attr(summaryTableCont, "summaryTable")[["statsVar"]]
		)
	)
			
})

test_that("The origin of the table is correctly included in the combined table as row", {
			
	data <- data.frame(
		USUBJID = c("1", "2"),
		AVAL = c(1, 2),
		AVAL2 = c(2, 3)
	)
	summaryTable1 <- computeSummaryStatisticsTable(
		data = data, 
		var = "AVAL",
		varLabInclude = TRUE
	)
	summaryTable2 <- computeSummaryStatisticsTable(
		data = data, 
		var = "AVAL2",
		varLabInclude = TRUE
	)
	summaryTableCombine <- combine(summaryTable1, summaryTable2,
		combineVar = "Version", combineDir = "row")
	
	expect_true("Version" %in% colnames(summaryTableCombine))
	
	# check that input table are correctly saved in correct combine var
	summaryTableCombineWthtTotal <- subset(summaryTableCombine, !isTotal)
	expect_equal(
		subset(summaryTableCombineWthtTotal, Version == 1, select = -Version),
		subset(summaryTable1, !isTotal),
		check.attributes = FALSE
	)
	expect_equal(
		subset(summaryTableCombineWthtTotal, Version == 2, select = -Version),
		subset(summaryTable2, !isTotal),
		check.attributes = FALSE
	)
	
	## check attributes
	expect_equal(
		attr(summaryTableCombine, "summaryTable")$rowVar,
		c("Version", "variable")
	)
			
})

test_that("The table label is correctly included in the combined table", {
			
	data <- data.frame(
		USUBJID = c("1", "2"),
		AVAL = c(1, 2),
		AVAL2 = c(2, 3)
	)
	summaryTable1 <- computeSummaryStatisticsTable(
		data = data, 
		var = "AVAL",
		varLabInclude = TRUE
	)
	summaryTable2 <- computeSummaryStatisticsTable(
		data = data, 
		var = "AVAL2",
		varLabInclude = TRUE
	)
	summaryTableCombine <- combine(
		`old batch` = summaryTable1, 
		`new batch` = summaryTable2,
		combineVar = "Version", 
		combineDir = "row"
	)
	
	expect_true("Version" %in% colnames(summaryTableCombine))
			
	# check that input table are correctly saved in correct combine var
	summaryTableCombineWthtTotal <- subset(summaryTableCombine, !isTotal)
	expect_equal(
		subset(summaryTableCombineWthtTotal, Version == "old batch", select = -Version),
		subset(summaryTable1, !isTotal),
		check.attributes = FALSE
	)
	expect_equal(
		subset(summaryTableCombineWthtTotal, Version == "new batch", select = -Version),
		subset(summaryTable2, !isTotal),
		check.attributes = FALSE
	)
			
})


test_that("The origin of the table is correctly included in the combined table as column", {
			
	data <- data.frame(
		USUBJID = c("1", "2"),
		AVAL = c(1, 2),
		AVAL2 = c(2, 3)
	)
	summaryTable1 <- computeSummaryStatisticsTable(
		data = data, 
		var = "AVAL",
		varLabInclude = TRUE
	)
	summaryTable2 <- computeSummaryStatisticsTable(
		data = data, 
		var = "AVAL2",
		varLabInclude = TRUE
	)
	summaryTableCombine <- combine(summaryTable1, summaryTable2,
		combineVar = "Version", combineDir = "col")
			
	expect_true("Version" %in% colnames(summaryTableCombine))
			
	# check that input table are correctly saved in correct combine var
	summaryTableCombineWthtTotal <- subset(summaryTableCombine, !isTotal)
	expect_equal(
		subset(summaryTableCombineWthtTotal, Version == 1, select = -Version),
		subset(summaryTable1, !isTotal),
		check.attributes = FALSE
	)
	expect_equal(
		subset(summaryTableCombineWthtTotal, Version == 2, select = -Version),
		subset(summaryTable2, !isTotal),
		check.attributes = FALSE
	)
			
	## check attributes
	expect_equal(
		attr(summaryTableCombine, "summaryTable")$colVar,
		"Version"
	)
			
})

test_that("The column total is discarded in the combined table if it differs between tables", {
			
	data1 <- data.frame(
		USUBJID = c("1", "2"),
		AVAL = c(1, 2)
	)
	summaryTable1 <- computeSummaryStatisticsTable(
		data = data1, 
		var = "AVAL",
		varLabInclude = TRUE
	)
	data2 <- data.frame(
		USUBJID = c("1", "2", "3"),
		AVAL2 = c(1, 2, 3)
	)
	summaryTable2 <- computeSummaryStatisticsTable(
		data = data2, 
		var = "AVAL2",
		varLabInclude = TRUE
	)
	expect_warning(
		summaryTableCombine <- combine(summaryTable1, summaryTable2),
		"column totals differ"
	)
	expect_false(all(summaryTableCombine$isTotal))
	
})

test_that("The column total is discarded if the tables contain different column variables", {
			
	dataTRT <- data.frame(
		USUBJID = c("1", "2"),
		AVAL = c(1, 2),
		TRT = c("A", "B")
	)
	summaryTableTRT <- computeSummaryStatisticsTable(
		data = dataTRT, 
		var = "AVAL",
		colVar = "TRT",
		varLabInclude = TRUE
	)
	dataVISIT <- data.frame(
		USUBJID = c("1", "2"),
		AVAL = c(1, 2),
		VISIT = c("Baseline", "Week 4")
	)
	summaryTableVISIT <- computeSummaryStatisticsTable(
		data = dataVISIT, 
		var = "AVAL",
		colVar = "VISIT",
		varLabInclude = TRUE
	)
	expect_warning(
		summaryTableCombine <- combine(summaryTableTRT, summaryTableVISIT),
		"column totals are discarded"
	)
	expect_false(all(summaryTableCombine$isTotal))
			
})

#' Convert summary statistics table to flextable
#' @inheritParams inTextSummaryTable-common-args
#' @inheritParams inTextSummaryTable-flextable-args
#' @return \code{\link[flextable]{flextable}} object with summary table
#' If \code{summaryTable} is a list of summary tables,
#' returns a list of \code{\link[flextable]{flextable}}.
#' @import flextable
#' @importFrom officer fp_border
#' @importFrom stats setNames
#' @author Laure Cougnaud
convertSummaryStatisticsTableToFlextable <- function(
	summaryTable, 
	colHeaderMerge = TRUE,
	landscape = (style == "presentation"), 
	margin = 1, rowPadBase = 14.4,
	title = NULL, 
	footer = NULL,
	style = "report",
	colorTable = getColorPaletteTable(style = style),
	fontname = switch(style, 'report' = "Times", 'presentation' = "Tahoma"),
	fontsize = switch(style, 'report' = 8, 'presentation' = 10),
	file = NULL, 
	pageDim = NULL, columnsWidth = NULL
) {
	
	style <- match.arg(style, choices = c("report", "presentation"))
	
	if(is.null(summaryTable)) {
		return(invisible())
	}
	
	if(!is.data.frame(summaryTable)) {
		
		inputParams <- as.list(environment())
		res <- sapply(seq_along(summaryTable), function(i) {
			summaryTableI <- summaryTable[[i]]
			inputParamsBy <- inputParams
			inputParamsBy$summaryTable <- summaryTableI
			inputParamsBy$title <- if(length(title) > 1)
				inputParams$title[i]	else	c(
				inputParams$title, 
				strsplit(names(summaryTable)[i], split = "\n")[[1]]
			)
			inputParamsBy$file <- if(!is.null(file)) {
				paste0(file_path_sans_ext(file), "_", i, ".", file_ext(file))
			}
			do.call(convertSummaryStatisticsTableToFlextable, inputParamsBy)		
		}, simplify = FALSE)	
		if(!is.null(names(summaryTable)))
			names(res) <- names(summaryTable)
	
		return(res)
		
	}
	
	sumTableAttr <- attributes(summaryTable)$summaryTable
	
	# create base flextable with header
	headerDf <- sumTableAttr$header
	ftWithHeader <- createFlextableWithHeader(
		data = summaryTable, 
		headerDf = headerDf,
		title = title,
		includeRownames = FALSE,
		colHeaderMerge = colHeaderMerge
	)
	ft <- ftWithHeader$ft
	colsDataFt <- ftWithHeader$colsData
	nCols <- length(colsDataFt)
	getNewCol <- function(initCol)
		na.omit(names(colsDataFt)[match(initCol, colsDataFt)])
	
	rowVar <- sumTableAttr$rowVar
	if(is.null(rowVar))	rowVar <- colnames(summaryTable)[1]
	
	# is there some padding specified?
	padParams <- sumTableAttr$padParams
	hasPadding <- length(padParams) > 0
	
	# special formatting (e.g. bold)
	for(el in sumTableAttr$formatParams) {
		if("bold" %in% el$type)
			ft <- ft %>% bold(i = el$i, j = el$j, part = el$part)
	}
	
	# merge rows
	# important: merge rows before setting horizontal lines
	# otherwise might encounter issues
	rowVarToMerge <- c(rowVar, sumTableAttr$rowVarInSepCol)
	for(col in rowVarToMerge) {
		j <- match(col, colnames(summaryTable))
		# vector with # duplicates
		countDupl <- rle(x = summaryTable[, j])$lengths
		countDuplIdx <- which(countDupl > 1) # only duplicated
		for(idx in countDuplIdx) {
			# indices of duplicated rows
			i <- seq.int(
				from = ifelse(idx == 1, 1, cumsum(countDupl)[idx-1]+1), 
				length.out = countDupl[idx]
			)
			if(hasPadding) {
				# extract padding spec for this column:
				idxPadCol <- sapply(padParams, function(x)
					x$part == "body" & 
					x$j == j
				)
				padParamsCol <- padParams[idxPadCol]
				if(length(padParamsCol) > 0) {
					# extract padding for each row
					iPad <- sapply(i, function(iP) {
						padIP <- unlist(lapply(padParamsCol, function(pad)
							if(iP %in% pad$i)	pad$padding.left	
						))
						ifelse(is.null(padIP), 0, max(padIP))
					})
					# remove row(s) which have a different padding
					iKept <- unique(unlist(lapply(which(diff(iPad) == 0), function(x) x+c(0, 1))))
					i <- i[iKept]
				}
			}
			# merge rows
			if(length(i) > 1) {
				ft <- merge_at(ft, j = getNewCol(col), i = i)
			}
		}
	}
	
	if(!is.null(sumTableAttr$mergeParams)) {
		for(params in sumTableAttr$mergeParams)
			ft <- merge_at(ft, j = params$j, params$i, part = params$part)
	}
	
	# add footer
	if(!is.null(footer)) {
		for(iFoot in seq_along(footer)) {
			paramsFooter <- setNames(list(footer[iFoot]), names(colsDataFt[1]))
			ft <- do.call(add_footer, 
				c(list(x = ft, top = FALSE), paramsFooter)
			)
			ft <- ft %>% merge_at(
				i = iFoot,
				j = seq_len(ncol(summaryTable)), part = "footer"
			)
		}
	}

	# set correct alignments
	colsAlignLeft <- getNewCol(c("Statistic", rowVar))
	colsAlignCenter <- setdiff(names(colsDataFt), colsAlignLeft)
	ft <- align(ft, j = colsAlignLeft, align = "left", part = "all")
	ft <- align(ft, j = colsAlignCenter, align = "center", part = "all")
	
	## padding
	if(hasPadding)
		for(padParams in sumTableAttr$padParams) {
			padPars <- grep("^padding", names(padParams), value = TRUE)
			padParams[padPars] <- sapply(padPars, function(par) padParams[[par]] * rowPadBase, simplify = FALSE)
			# if title is specified, shift row coordinate of padding by 1
			if(!is.null(title) && padParams$part == "header" && "i" %in% names(padParams))
				padParams$i <- padParams$i + length(title)
			ft <- do.call(padding, c(list(x = ft), padParams))
		}
	
	## borders
	bd <- fp_border(color = colorTable["line"])
	ft <- border_remove(ft)
	# if no vertical lines, only horizontal line 
	# between header/stub, top header and bottom stub
	isVline <- sumTableAttr$vline
	if(!is.null(isVline) && isVline == "none") {
		ft <- ft %>% 
#			hline_top(border = bd, part = "header") %>%
			hline_bottom(border = bd, part = "body") %>%
			hline_bottom(border = bd, part = "header")
	}else ft <- border_outer(ft, border = bd, part = "all") # otherwise all border (stub + header)
	if(!is.null(title))
		ft <- ft %>% hline(i = length(title), border = bd, part = "header")
	
	# horizontal lines
	isHline <- sumTableAttr$hline
	if(!(!is.null(isHline) && isHline == "none")) {
		for(hlineParams in sumTableAttr$hlineParams)
			ft <- do.call(hline, c(list(x = ft, border = bd), hlineParams))
	}
	
	# vertical lines
	for(vlineParams in sumTableAttr$vlineParams) {
		if(!is.null(vlineParams$i))	vlineParams$i <- vlineParams$i + length(title)
		ft <- do.call(vline, c(list(x = ft, border = bd), vlineParams))
	}
	
	# important! in case cells are merged in a column, 
	# correct the position of horizontal lines
	ft <- ft %>% fix_border_issues()
	
	# Custom formatting (if any)
	# for body
	ft <- formatCustomFlextable(
		dataTable = summaryTable, 
		ft = ft, 
		fontname = fontname,
		fontsize = fontsize,
		part = "body"
	)
	# for footnote
	if(!is.null(footer))
		ft <- formatCustomFlextable(
			dataTable = data.frame(footer), 
			ft = ft, 
			fontname = fontname,
			fontsize = fontsize,
			part = "footer"
		)
	
	# for title
	if(!is.null(title))
		ft <- formatCustomFlextable(
			dataTable = data.frame(title), ft = ft, 
			part = "header",
			fontname = fontname,
			fontsize = fontsize,
			bold = TRUE
		)
	# for header
	if(!is.null(headerDf))
		ft <- formatCustomFlextable(
			dataTable = headerDf, ft = ft, 
			part = "header",
			fontname = fontname,
			fontsize = fontsize,
			iBase = length(title)
		)
	
	# set style
	ft <- getListing(
		data = summaryTable, ft = ft, 
		border = FALSE, adjustWidth = FALSE, align = FALSE,
		style = style,
		fontname = fontname,
		fontsize = fontsize,
		landscape = landscape,
		margin = margin,
		colorTable = colorTable,
		pageDim = pageDim
	)
	
	# set columns width
	if(!is.null(columnsWidth) && length(columnsWidth) > length(colsDataFt)){
	  warning("The width is specified for too many columns of the table,", 
	    "so the specified 'columnsWidth' is ignored.")
	  columnsWidth <- NULL
	}
	
	# adjust to fit in document:
	widthPage <- getDimPage(
	  type = "width", landscape = landscape, margin = margin,
	  pageDim = pageDim,
	  style = style
	)
	if(is.null(columnsWidth)){
	  columnsWidth <- rep(widthPage/nCols, nCols)
	}else if(sum(columnsWidth) <= 1){
	  columnsWidth <- columnsWidth * widthPage
	}
	# fill in if not specified for all columns
	if(length(columnsWidth) < nCols){
	  nColsMissing <- nCols - length(columnsWidth)
	  columnsWidth <- c(columnsWidth, 
      rep((widthPage-sum(columnsWidth))/nColsMissing, times = nColsMissing))
	}
	ft <- width(ft, j = names(colsDataFt), width = columnsWidth)
	
	if(!is.null(file))
		exportFlextableToDocx(object = ft, file = file, landscape = landscape)
	
	return(ft)
	
}

#' Format superscript/subscripts/bold cells in a flextable.
#' Superscript should be indicated as: '\code{a^{b}}',
#' subscript as '\code{a_{b}}' and bold as '\code{bold{a}}' in the
#'  the input summary table.
#' @param dataTable data.frame with data used in table,
#' summary table for body or header data.frame for the header.
#' @param ft Corresponding \code{\link[flextable]{flextable}}.
#' @param fontname String with font name, 'Times' by default.
#' @param fontsize Integer with font size, 8 by default.
#' @param part string with part of the table to consider, 
#' see \code{\link[flextable]{compose}}.
#' @param iBase Integer with base row index (if different than 0).
#' @param bold Logical (FALSE by default) should the superscript/subscript 
#' be indicated in bold?
#' @return \code{\link[flextable]{flextable}} with superscript/subscript.
#' @importFrom stats as.formula
#' @importFrom officer fp_text
#' @importFrom flextable compose as_b as_sup as_sub as_paragraph
#' @author Laure Cougnaud
#' @keywords internal
formatCustomFlextable <- function(
	dataTable, ft, 
	fontname = "Times",
	part = "body",
	fontsize = 8,
	iBase = 0,
	bold = FALSE) {

	startPatterns <- c(
		"bold" = "bold\\{",
		"superscript" = "\\^\\{", 
		"subscript" = "\\_\\{"
	)
	endPattern <- "\\}"
	
	# extract indices with pattern
	dataTableMat <- as.matrix(dataTable)
	startRegex <- paste(paste0("(", startPatterns, ")"), collapse = "|")
	pattern <- paste0(startRegex, ".+", endPattern)
	idxPatternMat <- grep(pattern, dataTableMat)
	
	if(length(idxPatternMat) > 0) {
		
		# for each cell with special formatting
		for(idx in seq_along(idxPatternMat)) {
			
			# extract text
			text <- dataTableMat[idxPatternMat[idx]]
			endText <- nchar(text)
			
			# get start/end positions of a match
			positions <- lapply(names(startPatterns), function(fmt)
				getPatternPosition(
					x = text, 
					startPattern = startPatterns[fmt], 
					endPattern = endPattern,
					format = fmt
				)
			)
			positions <- do.call(rbind, positions)
			positions <- positions[order(positions$start), ]
			
			nMatches <- nrow(positions)
			
			xPars <- list()
				
			for(iMatch in seq_len(nMatches)){
				
				positionMatch <- positions[iMatch, ]
				start <- positionMatch[, "start"]
				end <- positionMatch[, "end"]
				pattern <- positionMatch[, "pattern"]
				format <-  positionMatch[, "format"]
				
				startPattern <- startPatterns[format]
				
				# get flextable fct for the pattern
				fctFm <- switch(format,
					bold = flextable::as_b,
					superscript = flextable::as_sup, 
					subscript = flextable::as_sub
				)

				# include part before matched string (if any)
				if(iMatch == 1 && start != 1){
					x <- substr(x = text, start = 1, stop = start + 1)
					x <- sub(startPattern, "", x)
					xPars <- c(xPars, list(as_chunk(x)))
				}
					
				# include special formatting for matched string
				x <- substr(x = text, start = start, stop = end)
				x <- sub(pattern, "\\1", x)
				xPars <- c(xPars, list(fctFm(x)))
					
				# include part after matched string (if any)
				if(end != endText){
						
					startNextMatch <- positions[iMatch+1, "start"] - 1
					if(!(!is.na(startNextMatch) && end == startNextMatch)){
					
						if(iMatch == nMatches){
							endNext <- endText
						}else{
							endNext <- startNextMatch
						}
						startNext <- end + 1
						x <- substr(x = text, start = startNext, stop = endNext)
						xPars <- c(xPars, list(as_chunk(x)))
					}
				}
					
			}
				
			if(length(xPars) > 0){
					
				# convert matrix indices to [row, col]
				idxPatternAI <- arrayInd(idxPatternMat, .dim = dim(dataTableMat))
					
				# extract row/columns in input table
				iEl <- idxPatternAI[idx, 1] + iBase
				jEl <- idxPatternAI[idx, 2]
					
				ft <- ft %>% flextable::compose(
					i = iEl, j = jEl,
					value = do.call(flextable::as_paragraph, xPars),
					part = part
				)
			
			}
		}
	}
	
	return(ft)

}

#' Get position(s) (start, end) of a pattern in a string.
#' @param x String.
#' @param startPattern String with start pattern.
#' @param endPattern String with end pattern.
#' @param format String with type of formatting
#' @return Matrix with columns: 'start' and 'end' with
#' start and end position(x) of the pattern, 'format' with
#' the \code{format} and 'pattern' with the regex pattern for the full match. 
#' NULL if no match.
#' @author Laure Cougnaud
getPatternPosition <- function(x, startPattern, endPattern = "\\}", format){
	
	# split text before/after pattern
	pattern <-  paste0(startPattern, "([^}]+)", endPattern)
	matches <- gregexpr(pattern = pattern, text = x)[[1]]
	
	isNoMatch <- length(matches) == 1 && matches == (-1)
	res <- if(!isNoMatch){
	
		start <- as.integer(matches)
		end <- matches + attr(matches, "match.length") - 1
		
		idx <- data.frame(start = start, end = end)
		idx <- cbind(idx, format = format, pattern = pattern)
		idx
		
	}
		
	return(res)
	
}

#' Export flextable to docx file
#' @param object \code{\link[flextable]{flextable}} object, or list of such objects
#' @param breaksAfter In case \code{object} is list: 
#' integer vector with indices of list item after which a page break should 
#' be included in the final document.
#' @inheritParams inTextSummaryTable-flextable-args
#' @return no returned value, the \code{object} is exported to a docx file.
#' @importFrom officer prop_section page_size body_set_default_section body_add_break
#' @importFrom flextable body_add_flextable
#' @importFrom magrittr "%>%"
#' @author Laure Cougnaud
#' @export
exportFlextableToDocx <- function(
	object, file, landscape = FALSE,
	breaksAfter = if(!inherits(object, "flextable"))	seq_along(object)	else	1
	) {
	
	isListTables <- !inherits(object, "flextable")
	
	if(!dir.exists(dirname(file)))	dir.create(dirname(file), recursive = TRUE)
	
	doc <- officer::read_docx()
	
	if(landscape){
	  secLandscape <- officer::prop_section(
	    page_size = officer::page_size(orient = "landscape")
	  )
	  doc <- doc %>% officer::body_set_default_section(value = secLandscape)
	}
	
	if(isListTables) {
		for(i in seq_along(object)) {
			doc <- doc %>% flextable::body_add_flextable(value = object[[i]]) 
			if(i %in% breaksAfter)	
				doc <- doc %>% officer::body_add_break()
		}
	}else	doc <- doc %>% flextable::body_add_flextable(value = object)

	print(doc, target = file)
	
}

context("Export summary statistics table to a flextable")

library(flextable)
library(officer)

test_that("A summary table with a row variable is correctly exported to flextable", {
			
	summaryTable <- data.frame(
		PARAM = factor(c("A", "B"), levels = c("B", "A")),
		n = c(9, 10)
	)	
	expect_silent(
		ft <- exportSummaryStatisticsTable(
			summaryTable = summaryTable,
			rowVar = "PARAM"
		)
	)
	expect_s3_class(ft, "flextable")
	expect_identical(
		ft$body$dataset[, 1],
		c("B", "A")
	)
			
})

test_that("Multiple row variables are nested correctly in a flextable summary table", {
			
	summaryTable <- data.frame(
		PARAM = rep(c("Actual Value", "Change from baseline"), each = 3),
		COHORT = rep(c("I", "I", "II"), times = 2),
		TRT = factor(rep(c("A", "B", "A"), times = 2), levels = c("B", "A")),
		n = seq_len(6)
	)
			
	ft <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		rowVar = c("PARAM", "COHORT", "TRT")
	)
	
	# check that correct data is displayed
	dataRef <- data.frame(
		c(
			"Actual Value", "I", "B", "A", "II", "A",
			"Change from baseline", "I", "B", "A", "II", "A"
		),
		c(
			NA_character_, NA_character_, "2", "1", NA_character_, "3", 
			NA_character_, NA_character_, "5", "4", NA_character_, "6"
		),
		stringsAsFactors = FALSE
	)
	expect_equal(
		unname(ft$body$dataset),
		unname(dataRef),
		check.attributes = FALSE
	)
			
})		

test_that("Padding is correctly set for nested row variables in a flextable summary table", {
			
	summaryTable <- data.frame(
		PARAM = rep(c("Actual Value", "Change from baseline"), each = 3),
		COHORT = rep(c("I", "I", "II"), times = 2),
		TRT = factor(rep(c("A", "B", "A"), times = 2), levels = c("B", "A")),
		n = seq_len(6)
	)
			
	rowPadBase <- 50
	ft <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		rowVar = c("PARAM", "COHORT", "TRT"),
		rowPadBase = rowPadBase
	)
			
	# check that correct padding is set for the nested column
	# (Note: remove row headers because flextable default padding is used)
	expect_equal(
		ft$body$styles$pars$padding.left$data[-c(1, 7), 1],
		rowPadBase * rep(c(1, 2, 2, 1, 2), 2)
	)
	
})

test_that("Horizontal lines are correctly set for nested row variables in a flextable summary table", {
			
	summaryTable <- data.frame(
		PARAM = rep(c("Actual Value", "Change from baseline"), each = 3),
		COHORT = rep(c("I", "I", "II"), times = 2),
		TRT = factor(rep(c("A", "B", "A"), times = 2), levels = c("B", "A")),
		n = seq_len(6)
	)

	# specify all colors as requested
	colorTable <- getColorPaletteTable()
	colorTable["line"] <- "red"
	ft <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		rowVar = c("PARAM", "COHORT", "TRT"),
		colorTable = colorTable
	)
	
	# check that horizontal lines are properly set
	ftDataBd <- ft$body$styles$cells$border.color.top$data
	# separator between groups is set
	expect_setequal(
		object = ftDataBd[7, ], 
		expected = "red"
	)
	# other lines are not set
	expect_false(unique(c(ftDataBd[-7, ])) == "red") 
	
})
			
test_that("The specified labels of the row variables are correctly set in a flextable summary table", {
			
	summaryTable <- data.frame(
		PARAM = factor(c("A", "B"), levels = c("B", "A")),
		n = c(9, 10)
	)
	ft <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		rowVar = "PARAM",
		rowVarLab = c(PARAM = "Parameter")
	)
	expect_identical(
		object = ft$header$dataset[, 1], 
		expected = "Parameter"
	)
			
})

test_that("The labels of the row variables, extracted from the labels of all variables, are correctly set in a flextable summary table", {
			
	summaryTable <- data.frame(
		PARAM = factor(c("A", "B"), levels = c("B", "A")),
		n = c(9, 10)
	)
	expect_identical(
		object = {
			ft <- exportSummaryStatisticsTable(
				summaryTable = summaryTable,
				rowVar = "PARAM",
				labelVars = c(PARAM = "Parameter")
			)
			ft$header$dataset[, 1]
		}, 
		expected = "Parameter"
	)
	
})

test_that("Row variables in a separated column are merged correctly in a flextable summary table", {
			
	summaryTable <- data.frame(
		PARAM = rep(c("Actual Value", "Change from baseline"), each = 3),
		COHORT = rep(c("I", "I", "II"), times = 2),
		TRT = factor(rep(c("A", "B", "A"), times = 2), levels = c("B", "A")),
		n = seq_len(6)
	)
	ft <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		rowVar = c("PARAM", "COHORT", "TRT"),
		rowVarInSepCol = c("COHORT", "TRT")
	)
	
	# each variable is exported as separated column
	expect_identical(
		object = as.character(as.vector(ft$header$dataset[1, 1:3])),
		expected = c("PARAM", "COHORT", "TRT")
	)
	
	# the data is correct
	# Note: factor should have been re-ordered based on levels (e.g. TRT)
	dataRef <- data.frame(
		rep(c("Actual Value", "Change from baseline"), each = 3),
		rep(c("I", "I", "II"), times = 2),
		rep(c("B", "A", "A"), times = 2),
		c("2", "1", "3", "5", "4", "6"),
		stringsAsFactors = FALSE
	)
	expect_equal(
		object = unname(ft$body$dataset),
		expected = unname(dataRef),
		check.attributes = FALSE
	)
	
	# flextable specify 'merging' as the 'spans' in 'columns'
	# correct lines are merged
	expect_identical(
		ft$body$spans$columns[, 1:2],
		cbind(
			c(3, 0, 0, 3, 0, 0),
			c(2, 0, 1, 2, 0, 1)
		)
	)
	
	# other lines are not merged
	expect_setequal(
		c(ft$body$spans$columns[, -(1:2)]),
		1
	)
	
})

test_that("Nested and merged row variables are displayed correctly in a flextable summary table", {
			
	summaryTable <- data.frame(
		AESOC = c("A", "A", "A"),
		AEDECOD = factor(c("a", "b", "b"), levels = c("b", "a")),
		WORSTINT = factor(
			c("Moderate", "Severe", "Moderate"), 
			levels = c("Moderate", "Severe")
		),
		n = c(2, 4, 7)
	)
	
	rowPadBase <- 50
	ft <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		rowVar = c("AESOC", "AEDECOD", "WORSTINT"),
		rowVarInSepCol = "WORSTINT",
		rowPadBase = rowPadBase
	)
	
	# check if data is correct
	dataRef <- data.frame(
		c("A", "b", "b", "a"),
		c(NA_character_, "Moderate", "Severe", "Moderate"),
		c(NA_character_, "7", "4", "2"),
		stringsAsFactors = FALSE
	)
	expect_equal(
		object = unname(ft$body$dataset),
		expected = unname(dataRef),
		check.attributes = FALSE
	)
	
	# check that correct padding is set for the nested column
	# (Note: remove row headers because flextable default padding is used)
	expect_setequal(
		ft$body$styles$pars$padding.left$data[-1, 1],
		rowPadBase
	)
	
})

test_that("Horizontal lines are correctly set for multiple row variables in a separated column in a flextable summary table", {
	
	summaryTable <- data.frame(
		PARAM = rep(c("Actual Value", "Change from baseline"), each = 3),
		COHORT = rep(c("I", "I", "II"), times = 2),
		TRT = factor(rep(c("A", "B", "A"), times = 2), levels = c("B", "A")),
		n = seq_len(6)
	)
	
	# warning because not all table colors are set
	expect_warning(
		ft <- exportSummaryStatisticsTable(
			summaryTable = summaryTable,
			rowVar = c("PARAM", "COHORT", "TRT"),
			rowVarInSepCol = c("COHORT", "TRT"),
			colorTable = c(line = "red")
		)
	)
	
	# check that horizontal lines are properly set
	ftDataBd <- ft$body$styles$cells$border.color.top$data
	# separator between groups is set
	expect_setequal(
		object = ftDataBd[c(3, 4, 6), 2:4], 
		expected = "red"
	)
	# other lines are not set
	expect_false(unique(c(ftDataBd[-c(3, 4, 6), 2:4])) == "red") 
			
})

test_that("Custom formatting is correctly set to a row variable in a flextable summary table", {
			
	summaryTable <- data.frame(
		TRT = c("A", "B"),
		n = 1:2
	)
			
	ft <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		rowVar = "TRT",
		rowVarFormat = list(TRT = "bold")
	)
	expect_identical(
		object = unname(ft$body$styles$text$bold$data),
		expected = cbind(
			rep(TRUE, nrow(summaryTable)),
			rep(FALSE, nrow(summaryTable))
		)
	)	
		
})

test_that("Custom formatting is correctly set to a nested row variable in a flextable summary table", {
			
	summaryTable <- data.frame(
		TRT = c("A", "A", "B", "B"),
		PARAM = c("a", "b", "a", "b"),
		n = 1:4
	)
	ft <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		rowVar = c("TRT", "PARAM"),
		rowVarFormat = list(PARAM = "bold")
	)
	expect_identical(
		object = unname(ft$body$styles$text$bold$data),
		expected = cbind(
			c(FALSE, TRUE, TRUE, FALSE, TRUE, TRUE),
			rep(FALSE, 6)
		)
	)
			
})

test_that("Custom formatting is correctly set to a row variable in a different column in a flextable summary table", {
			
	summaryTable <- data.frame(
		TRT = c("A", "A", "B", "B"),
		PARAM = c("a", "b", "a", "b"),
		n = 1:4
	)
		
	# only entire second column is in bold
	ft <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		rowVar = c("TRT", "PARAM"),
		rowVarInSepCol = "PARAM",
		rowVarFormat = list(PARAM = "bold")
	)
	expect_identical(
		object = unname(ft$body$styles$text$bold$data),
		expected = cbind(
			rep(FALSE, nrow(summaryTable)),
			rep(TRUE, nrow(summaryTable)),
			rep(FALSE, nrow(summaryTable))
		)
	)
			
})

test_that("Row totals are correctly included in the header row in a flextable summary table", {
			
	summaryTable <- data.frame(
		TRT = c("A", "A", "B", "B"),
		PARAM = factor(c("a", "Total", "b", "Total"), levels = c("Total", "a", "b")),
		n = c("2", "1", "4", "3")
	)
		
	ft <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		rowVar = c("TRT", "PARAM"),
		rowVarTotalInclude = "PARAM"
	)
	expect_equal(
		object = ft$body$dataset,
		expected = data.frame(
			c("A", "a", "B", "b"),
			c("1", "2", "3", "4"),
			stringsAsFactors = FALSE
		),
		check.attributes = FALSE
	)
	
})

test_that("Row totals are correctly included in the header rows when requested for a subset of the row variables in a flextable summary table", {
  
  summaryTable <- data.frame(
    PARCAT1 = factor(c("A", "A", "Total", "A"), levels = c("Total", "A")),
    PARCAT2 = factor(c("A1", "A2", "Total", "Total"), levels = c("Total", "A1", "A2")),
    PARCAT3 = c("A2", "A3", "Total", "Total"),
    PARAM = c("a", "b", "Total", "Total"),
    n = c("1", "2", "3", "3")
  )
  
  ft <- exportSummaryStatisticsTable(
    summaryTable = summaryTable,
    rowVar = c("PARCAT1", "PARCAT2", "PARCAT3", "PARAM"),
    rowVarTotalInclude = c("PARCAT1", "PARCAT2")
  )
  expect_equal(
    object = ft$body$dataset,
    expected = data.frame(
      c("Any PARCAT1, PARCAT2, PARCAT3, PARAM", "A", "A1", "A2", "a", "A2", "A3", "b"),
      c("3", "3", NA_character_, NA_character_, "1", NA_character_, NA_character_, "2"),
      stringsAsFactors = FALSE
    ),
    check.attributes = FALSE
  )
  
  expect_setequal(
    object = ft$body$styles$cells$border.width.top$data[2, ],
    expected = 1
  )
  expect_setequal(
    object = ft$body$styles$cells$border.width.top$data[-2, ],
    expected = 0
  )
  
})

test_that("Row totals are correctly included in separated rows in a flextable summary table", {
			
	summaryTable <- data.frame(
		TRT = c("A", "A", "B", "B"),
		PARAM = factor(c("a", "Total", "b", "Total"), levels = c("Total", "a", "b")),
		n = c("2", "1", "4", "3")
	)
		
	ft <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		rowVar = c("TRT", "PARAM"),
		rowVarTotalInclude = "PARAM",
		rowVarTotalInSepRow = "PARAM"
	)
	expect_equal(
		object = ft$body$dataset,
		expected = data.frame(
			c("A", "Total", "a", "B", "Total", "b"),
			c(NA_character_, "1", "2", NA_character_, "3", "4"),
			stringsAsFactors = FALSE
		),
		check.attributes = FALSE
	)
	
})

test_that("The variable group and name are correctly merged in a flextable summary table in case of unique group", {
			
	summaryTable <- data.frame(
		variable = c("A", "B", "B"),
		variableGroup = c("a", "b1", "b2"),
		n = c("1", "2", "3")
	)
		
	# (by default) variable and variableGroup 
	# are automatically merged in one row if only one category
	ft <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		rowVar = c("variable", "variableGroup"),
		rowAutoMerge = TRUE
	)
	expect_equal(
		object = ft$body$dataset[, 1],
		expected = c("A a", "B", "b1", "b2"),
		check.attributes = FALSE
	)
	
})

test_that("The variable group and name are correctly included in different rows when requested in a flextable summary table in case of unique group", {
	
	summaryTable <- data.frame(
		variable = c("A", "B", "B"),
		variableGroup = c("a", "b1", "b2"),
		n = c("1", "2", "3")
	)
	
	ft <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		rowVar = c("variable", "variableGroup"),
		rowAutoMerge = FALSE
	)
	expect_equal(
		object = ft$body$dataset[, 1],
		expected = c("A", "a", "B", "b1", "b2"),
		check.attributes = FALSE
	)
	
})

test_that("The variable name and the statistic are correctly merged in a flextable summary table in case of unique statistic", {
			
	summaryTable <- data.frame(
		variable = c("A", "B"),
		n = c("1", "2"),
		Mean = c(NA_character_, "0.56"),
		stringsAsFactors = FALSE
	)
			
	# (by default) row var and statistic label
	# are automatically merged in the same row if only one stat
	ft <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		rowVar = "variable",
		statsVar = c("n", "Mean"),
		rowAutoMerge = TRUE
	)
	expect_equal(
		object = ft$body$dataset[, 1],
		expected = c("A n", "B", "n", "Mean"),
		check.attributes = FALSE
	)
	
})

test_that("The variable name and the statistic are correctly included in different rows when requested in a flextable summary table in case of unique statistic", {
			
	summaryTable <- data.frame(
		variable = c("A", "B"),
		n = c("1", "2"),
		Mean = c(NA_character_, "0.56"),
		stringsAsFactors = FALSE
	)		
	
	ft <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		rowVar = "variable",
		statsVar = c("n", "Mean"),
		rowAutoMerge = FALSE
	)
	expect_equal(
		object = ft$body$dataset[, 1],
		expected = c("A", "n", "B", "n", "Mean"),
		check.attributes = FALSE
	)
	
})

test_that("A summary table with a continuous and a categorical variables is correctly exported to flextable", {
			
	summaryTable <- data.frame(
		variable = factor(
			c("SEX", "SEX", "AGE"), 
			levels = c("SEX", "AGE")
		),
		variableGroup = factor(
			c("Female", "Male", NA_character_),
			levels = c("Male", "Female")
		),
		n = c(3, 4, NA_real_),
		mean = c(NA_real_, NA_real_, 3.33)
	)
			
	ft <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		rowVar = c("variable", "variableGroup"),
		statsVar = c("n", "mean")
	)
	
	dataRef <- data.frame(
		c("SEX", "Male n", "Female n", "AGE mean"),
		c(NA_character_, "4", "3", "3.33"),
		stringsAsFactors = FALSE
	)
	expect_equal(
		object = unname(ft$body$dataset),
		expected = unname(dataRef),
		check.attributes = FALSE	
	)
			
})	

test_that("A summary table with a continuous and a categorical variables is correctly exported without row merging to flextable", {
		
	# Edge-case: different levels of nestings by row variables
	# because variables are of different types (categorical/numeric)
	# Padding is extracted for each nested level
	# and smoothed afterwards to avoid double padding for cont var 
	# (check 'smoothPadding')
			
	summaryTable <- data.frame(
		variable = factor(
			c("SEX", "SEX", "AGE"), 
			levels = c("SEX", "AGE")
		),
		variableGroup = factor(
			c("Female", "Male", NA_character_),
			levels = c("Male", "Female")
		),
		n = c(3, 4, NA_real_),
		mean = c(NA_real_, NA_real_, 3.33)
	)
			
	rowPadBase <- 50
	ft <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		rowVar = c("variable", "variableGroup"),
		statsVar = c("n", "mean"),
		rowPadBase = rowPadBase,
		rowAutoMerge = FALSE
	)
		
	# check that data is correctly set
	dataRef <- data.frame(
		c("SEX", "Male", "n", "Female", "n", 
			"AGE", "mean"),
		c(NA_character_, NA_character_, "4", NA_character_, "3", 
			NA_character_, "3.33"),
		stringsAsFactors = FALSE
	)
	
	expect_equal(
		object = unname(ft$body$dataset),
		expected = unname(dataRef),
		check.attributes = FALSE	
	)
	
	# check that correct padding is set for the nested column
	# (Note: remove row headers because flextable default padding is used)
	expect_equal(
		ft$body$styles$pars$padding.left$data[-c(1, 6), 1],
		rowPadBase * c(1, 2, 1, 2, 1)
	)
	
})	

test_that("A summary table with a continuous and a categorical variable, with and without named statistic, is correctly exported to flextable", {
			
	# edge case to check if merging rows variableGroup/stats is correct
	summaryTable <- data.frame(
		variable = factor(
			c("SEX", "SEX", "AGE"), 
			levels = c("SEX", "AGE")
		),
		variableGroup = factor(
			c("Female", "Male", NA_character_),
			levels = c("Male", "Female")
		),
		Statistic = c(3, 4, NA_real_),
		Mean = c(NA_real_, NA_real_, 3.33)
	)
			
	rowPadBase <- 50
	ft <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		rowVar = c("variable", "variableGroup"),
		rowPadBase = rowPadBase
	)	
	
	# check that data is correctly set
	dataRef <- data.frame(
		c("SEX", "Male", "Female", "AGE Mean"),
		c(NA_character_, "4", "3", "3.33"),
		stringsAsFactors = FALSE
	)
	
	expect_equal(
		object = unname(ft$body$dataset),
		expected = unname(dataRef),
		check.attributes = FALSE	
	)
	
	# check that correct padding is set for the nested column
	# (Note: remove row headers because flextable default padding is used)
	expect_setequal(
		ft$body$styles$pars$padding.left$data[-c(1, 4), 1],
		rowPadBase
	)
			
})

test_that("A summary table with a column variable is correctly exported to flextable", {
			
	summaryTable <- data.frame(
		TRT = factor(c("A", "B"), levels = c("B", "A")),
		n = c(9, 10)
	)
	expect_silent(
		ft <- exportSummaryStatisticsTable(
			summaryTable = summaryTable,
			colVar = "TRT"
		)
	)
	expect_s3_class(ft, "flextable")
	expect_identical(
		unname(unlist(ft$header$dataset[1, ])),
		c("B", "A")
	)
	
})

test_that("An error is generated if a flextable summary table contains multiple records for the same variable", {
			
	summaryTable <- data.frame(
		TRT = c("A", "A", "B"), 
		n = c(9, 8, 10)
	)
	expect_error(
		exportSummaryStatisticsTable(
			summaryTable = summaryTable,
			colVar = "TRT"
		),
		"Table formatting to multiple columns failed because of duplicated records for each row/col"
	)
			
})

test_that("A summary table without totals in the header is correctly exported to flextable", {
	
	summaryTable <- data.frame(
		TRT = factor(c("A", "A", "B", "B"), levels = c("B", "A")),
		statN = c(1, 4, 2, 5),
		isTotal = c(FALSE, TRUE, FALSE, TRUE)
	)
	
	ft <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		colVar = "TRT",
		colHeaderTotalInclude = TRUE
	)
	expect_identical(
		object = unname(unlist(ft$header$dataset)),,
		expected = c("B\n(N=5)", "A\n(N=4)")
	)
	
})
	
test_that("A summary table without totals in the header is correctly exported to flextable", {
			
	summaryTable <- data.frame(
		TRT = factor(c("A", "A", "B", "B"), levels = c("B", "A")),
		statN = c(1, 4, 2, 5),
		isTotal = c(FALSE, TRUE, FALSE, TRUE)
	)
	ft <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		colVar = "TRT",
		colHeaderTotalInclude = FALSE
	)
	expect_identical(
		object = unname(unlist(ft$header$dataset)),
		expected = c("B", "A")
	)

})

test_that("A summary table with statistics in rows is correctly exported to flextable", {
			
	summaryTable <- data.frame(
		variable = c("A", "B"),
		n = c("1", "2"),
		Mean = c("0.34", "0.56"),
		stringsAsFactors = FALSE
	)
	
	ft <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		rowVar = "variable",
		statsVar = c("n", "Mean"),
		statsLayout = "row"
	)
	expect_equal(
		object = ft$body$dataset,
		expected = data.frame(
			c("A", "n", "Mean", "B", "n", 'Mean'),
			c(NA_character_, "1", "0.34", NA_character_, "2", "0.56"),
			stringsAsFactors = FALSE
		),
		check.attributes = FALSE
	)
	
})	
	
test_that("A summary table with statistics in columns is correctly exported to flextable", {
		
	summaryTable <- data.frame(
		variable = c("A", "B"),
		n = c("1", "2"),
		Mean = c("0.34", "0.56"),
		stringsAsFactors = FALSE
	)
				
	ft <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		rowVar = "variable",
		statsVar = c("n", "Mean"),
		statsLayout = "col"
	)
	expect_identical(
		object = unname(unlist(ft$header$dataset[1, ])),
		expected = c("variable", "n", "Mean")
	)
	expect_equal(
		object = ft$body$dataset,
		expected = data.frame(
			c("A", "B"),
			c("1", "2"),
			c("0.34", "0.56"),
			stringsAsFactors = FALSE
		),
		check.attributes = FALSE	
	)
	
})

test_that("A summary table with statistics in a separated column is correctly exported to flextable", {
	
	summaryTable <- data.frame(
		variable = c("A", "B"),
		n = c("1", "2"),
		Mean = c("0.34", "0.56"),
		stringsAsFactors = FALSE
	)		
	
	# stat in the row direction, but in a separated column
	ft <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		rowVar = "variable",
		statsVar = c("n", "Mean"),
		statsLayout = "rowInSepCol"
	)
	# header
	expect_identical(
		object = unname(unlist(ft$header$dataset[1:2])), 
		expected = c("variable", "Statistic")
	)
	# data
	expect_equal(
		object = as.data.frame(sapply(ft$body$dataset, as.character)),
		expected = data.frame(
			c("A", "A", "B", "B"),
			c("n", "Mean", "n", "Mean"),
			c("1", "0.34", "2", "0.56")
		),
		check.attributes = FALSE	
	)
	
})

test_that("The label for the statistic value is correctly set in a flextable summary table", {
			
	summaryTable <- data.frame(
		variable = c("A", "B"),
		Statistic = c("1", "2"),
		stringsAsFactors = FALSE
	)
	ft <- exportSummaryStatisticsTable(
		summaryTable, rowVar = "variable", 
		statsVar = "Statistic", 
		statsValueLab = "Number of subjects"
	)
	expect_match(
		object = unname(unlist(ft$header$dataset[, 2])),
		regexp = "Number of subjects.*"
	)
	
})

test_that("An error is generated if the label for the statistic value is set to the default name in a flextable summary table", {
			
	summaryTable <- data.frame(
		variable = c("A", "B"),
		Statistic = c("1", "2"),
		stringsAsFactors = FALSE
	)
	expect_error(
		exportSummaryStatisticsTable(summaryTable, rowVar = "variable", 
			statsVar = "Statistic", statsValueLab = "Statistic"
		),
		"'statsValueLab' should be different than 'Statistic'."
	)
			
})

test_that("A summary table with one statistic is correctly exported to flextable with the statistic name", {
			
	summaryTable <- data.frame(
		TRT = c("A", "A", "B", "B"),
		variable = c("a", "b", "a", "b"),
		n = c("1", "2", "3", "4"),
		stringsAsFactors = FALSE
	)
	
	ft <- exportSummaryStatisticsTable(
		summaryTable, 
		rowVar = "variable", colVar = "TRT",
		statsVar = "n", 
		statsLabInclude = TRUE
	)
	expect_identical(
		object = ft$body$dataset[, 1],
		expected = c("a", "n", "b", "n")
	)
	
})

test_that("A summary table with one statistic is correctly exported to flextable without the statistic name", {
		
	summaryTable <- data.frame(
		TRT = c("A", "A", "B", "B"),
		variable = c("a", "b", "a", "b"),
		n = c("1", "2", "3", "4"),
		stringsAsFactors = FALSE
	)		
	
	ft <- exportSummaryStatisticsTable(
		summaryTable, 
		rowVar = "variable", colVar = "TRT",
		statsVar = "n", 
		statsLabInclude = FALSE
	)
	
	expect_identical(
		object = ft$body$dataset[, 1],
		expected = c("a", "b")
	)
	
})
	
test_that("A warning is generated if a flextable summary table contain multiple statistics but the names are specified to be not included", {
	
	summaryTable <- data.frame(
		TRT = c("A", "A", "B", "B"),
		variable = c("a", "b", "a", "b"),
		n = c("1", "2", "3", "4"),
		Mean = c("0.34", "0.56"),
		stringsAsFactors = FALSE
	)		
			
	expect_warning(
		exportSummaryStatisticsTable(
			summaryTable, 
			rowVar = "variable", colVar = "TRT",
			statsVar = c("n", "Mean"),
			statsLabInclude = FALSE
		),
		"Statistic label is included.*because more than one statistic variable.*"
	)
	
})

test_that("A placeholder for empty value is correctly included by default in a flextable summary table", {
			
	summaryTable <- data.frame(
		TRT = c("A", "A", "B", "B"),
		PARAM = c("a", "b", "a", "b"),
		n = c("1", "2", "3", NA_character_),
		stringsAsFactors = FALSE
	)
	
	# by default, 'empty' value are set to '-'
	ft <- exportSummaryStatisticsTable(
		summaryTable,
		rowVar = "PARAM", colVar = "TRT",
		statsVar = "n"
	)
	expect_equal(
		object = ft$body$dataset,
		expected = data.frame(
			c("a", "b"),
			c("1", "2"),
			c("3", "-"),
			stringsAsFactors = FALSE
		),
		check.attributes = FALSE
	)
	
})

test_that("A specified placeholder for empty value is correctly included in a flextable summary table", {
			
	summaryTable <- data.frame(
		TRT = c("A", "A", "B", "B"),
		PARAM = c("a", "b", "a", "b"),
		n = c("1", "2", "3", NA_character_),
		stringsAsFactors = FALSE
	)
	
	ft <- exportSummaryStatisticsTable(
		summaryTable,
		rowVar = "PARAM", colVar = "TRT",
		statsVar = "n",
		emptyValue = "0"
	)
	expect_equal(
		object = ft$body$dataset,
		expected = data.frame(
			c("a", "b"),
			c("1", "2"),
			c("3", "0"),
			stringsAsFactors = FALSE
		),
		check.attributes = FALSE
	)
			
})

test_that("A list of summary tables is correctly exported to flextable", {
			
	summaryTables <- list(
		`PARAM 2` = data.frame(n = 10),
		`PARAM 1` = data.frame(n = 2)
	)
			
	fts <- exportSummaryStatisticsTable(
		summaryTables, 
		outputType = "flextable"
	)
	expect_type(fts, "list")
	expect_named(fts, names(summaryTables))
			
	for(group in names(summaryTables)){
			
		# table content is the same as if the table would have
		# been created directly
		expect_identical({
				ft <- exportSummaryStatisticsTable(summaryTables[[!!group]])
				ft$body$dataset
			},
			expected = fts[[!!group]]$body$dataset
		)
		
		# correct group is specified in the title
		expect_equal(fts[[!!group]]$header$dataset[1, ], !!group)

	}
			
})

test_that("A list of summary tables with different titles is correctly exported to flextable", {
			
	summaryTables <- list(
		`PAR2` = data.frame(n = 10),
		`PAR1` = data.frame(n = 2)
	)
			
	titles <- c("PARAMETER 2", "PARAMETER 1")
	fts <- exportSummaryStatisticsTable(
		summaryTables, 
		outputType = "flextable",
		title = titles
	)
	
	for(i in seq_along(summaryTables)){
		
		expect_equal(
			fts[[!!i]]$header$dataset[1, 1],
			titles[!!i]
		)
		
	}
		
})

test_that("A warning is generated if the variable group with totals is not formatted correctly in a flextable summary table", {
			
	summaryTable <- data.frame(
		variable = factor(c("RACE", "SEX", "SEX", "SEX")),
		variableGroup = factor(
			c("White", "Female", "Male", "Total"), 
			levels = c("White", "Female", "Male", "Total")
		),
		n = c("9", "3", "7", "10")
	)
			
	expect_warning(
		exportSummaryStatisticsTable(
			summaryTable = summaryTable,
			rowVar = c("variable", "variableGroup"),
			statsVar = "n",
			rowVarTotalInclude = "variableGroup"
		),
		"variable.*with total.*should be formatted.*with 'Total' as the first level"
	)
	
})

test_that("A flextable summary table is correctly formatted if totals are included only for a subset of the categorical variables", {
			
	summaryTable <- data.frame(
		variable = factor(c("RACE", "SEX", "SEX", "SEX")),
		variableGroup = factor(
			c("White", "Female", "Male", "Total"), 
			levels = c("Total", "White", "Female", "Male")
		),
		n = c("9", "3", "7", "10")
	)
	
	ft <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		rowVar = c("variable", "variableGroup"),
		statsVar = "n",
		rowVarTotalInclude = "variableGroup"
	)
	
	refData <- data.frame(
		c("RACE", "White", "SEX", "Female", "Male"),
		c(NA_character_, "9", "10", "3", "7"),
		stringsAsFactors = FALSE
	)
	expect_equal(
		unname(ft$body$dataset),
		unname(refData),
		check.attributes = FALSE
	)
	
})

test_that("A flextable summary table is correctly formatted if totals are included only for a subset of the categorical variables in separated rows", {
			
	summaryTable <- data.frame(
		variable = factor(c("RACE", "SEX", "SEX", "SEX")),
		variableGroup = factor(
			c("White", "Female", "Male", "Total"), 
			levels = c("Total", "White", "Female", "Male")
		),
		n = c("9", "3", "7", "10")
	)
			
	ft <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		rowVar = c("variable", "variableGroup"),
		statsVar = "n",
		rowVarTotalInclude = "variableGroup",
		rowVarTotalInSepRow = "variableGroup"
	)
			
	refData <- data.frame(
		c("RACE White", "SEX", "Total", "Female", "Male"),
		c("9", NA_character_, "10", "3", "7"),
		stringsAsFactors = FALSE
	)
			
	expect_equal(
		unname(ft$body$dataset),
		unname(refData),
		check.attributes = FALSE
	)
			
})

test_that("Missing values in nested row variables are correctly represented in a flextable summary table", {
			
	summaryTable <- data.frame(
		variable = factor(c("SEX", "SEX", "SEX")),
		variableGroup = factor(
			c(NA_character_, "Male", "Female"), 
			levels = c("Male", "Female")
		),
		n = c("3", "7", "10")
	)
			
	ft <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		rowVar = c("variable", "variableGroup"),
		statsVar = "n"
	)
			
	refData <- data.frame(
		c("SEX", "Male", "Female", NA_character_),
		c(NA_character_, "7", "10", "3"),
		stringsAsFactors = FALSE
	)
			
	expect_equal(
		unname(ft$body$dataset),
		unname(refData),
		check.attributes = FALSE
	)
			
})

test_that("A summary table with only one element in nested row variable in a row variable is correctly exported to flextable", {
			
	summaryTable <- data.frame(
		variable = "SEX",
		variableGroup = factor("Total"),
		n = "10"
	)
			
	ft <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		rowVar = c("variable", "variableGroup"),
		statsVar = "n",
		rowVarTotalInclude = "variableGroup"
	)
	
	expect_equal(
		unname(ft$body$dataset),
		unname(data.frame("SEX", "10", stringsAsFactors = FALSE)),
		check.attributes = FALSE
	)

})

test_that("Page dimension are correctly set in a flextable summary table", {
	
	summaryTable <- data.frame(n = 10)
	
	ft <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		pageDim = c(10, 3),
		margin = 0
	)
	
	expect_equal(
		object = ft$body$colwidths,
		expected = 10
	)
			
})

test_that("Margins are correctly set in a flextable summary table", {
			
	summaryTable <- data.frame(n = 10)
	
	ft <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		pageDim = c(10, 3),
		margin = 1
	)
	expect_equal(
		object = ft$body$colwidths,
		expected = 8
	)
	
})

test_that("A summary table is correctly exported to flextable in landscape mode", {
			
	summaryTable <- data.frame(n = 10)	
	
	# check that width of body in landscape mode > width in portrait mode
	ftLandscape <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		landscape = TRUE
	)
	ftPortrait <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		landscape = FALSE
	)
	
	# check that width in landscape mode is > than in portrait mode
	for(part in c("body", "header", "footer"))
		expect_gt(
			object = ftLandscape[[!!part]]$colwidths, 
			expected = ftPortrait[[!!part]]$colwidths
		)
	
})

test_that("Multiple titles are correctly included in a flextable summary table", {
		
	summaryTable <- data.frame(n = 10)
	
	# multiple titles
	titles <- c("Title A", "Title B")
	expect_identical(
		object = {
			ft <- exportSummaryStatisticsTable(
				summaryTable = summaryTable,
				title = titles
			)
			ft$header$dataset[seq_along(titles), 1]
		}, titles
	)
			
})

test_that("Footers are correctly included in a flextable summary table", {
			
	summaryTable <- data.frame(n = 10)
	
	footers <- c("Explanation 1", "Explanation 2")
	ft <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		footer = footers
	)
	expect_identical(
		object = ft$footer$dataset[seq_along(footers), 1],
		expected = footers
	)
	
})

test_that("Colors are correctly set in a flextable summary table", {
	
	summaryTable <- data.frame(n = 10)
	
	colorTable <- c(
		header = "#330000", headerBackground = "#CC3333",
		body = "#CCCC00", bodyBackground = "#FFFFCC", 
		footerBackground = "#CCFFFF", footer = "#000066",
		line = "#669900"
	)
	
	expect_silent(
		ft <- exportSummaryStatisticsTable(
			summaryTable, 
			colorTable = colorTable,
			footer = "test"
		)
	)
	# check colors in flextable object
	expect_setequal(ft$body$styles$text$color$data, colorTable["body"])
	expect_setequal(ft$body$styles$cells$background.color$data, colorTable["bodyBackground"])
	expect_setequal(ft$header$styles$text$color$data, colorTable["header"])
	expect_setequal(ft$header$styles$cells$background.color$data, colorTable["headerBackground"])
	expect_setequal(ft$footer$styles$text$color$data, colorTable["footer"])
	expect_setequal(ft$footer$styles$cells$background.color$data, colorTable["footerBackground"])
	expect_setequal(ft$body$styles$cells$border.color.bottom$data, colorTable["line"])
	
})

test_that("Fontsize is correctly set in a flextable summary table", {
			
	summaryTable <- data.frame(n = 10)
	
	ft <- exportSummaryStatisticsTable(
		summaryTable, 
		fontsize = 25
	)
	
	expect_identical(
		object = c(ft$body$styles$text$font.size$data), 
		expected = 25
	)
			
})

test_that("Fontname is correctly set in a flextable summary table", {
			
	summaryTable <- data.frame(n = 10)
	
	ft <- exportSummaryStatisticsTable(
		summaryTable, 
		fontname = "Arial"
	)
	
	expect_identical(
		object = c(ft$body$styles$text$font.family$data), 
		expected = "Arial"
	)
	
})

test_that("Vertical lines are not included in a flextable summary table when requested", {
				
	summaryTable <- data.frame(
		TRT = c("A", "A", "B", "B"),
		AVISIT = c("a", "b", "a", "b"),
		n = c("1", "2", "3", "4"),
		stringsAsFactors = FALSE
	)
	
	ft <- exportSummaryStatisticsTable(
		summaryTable, 
		colVar = c("TRT", "AVISIT"),
		statsVar = "n",
		vline = "none"
	)
	
	expect_setequal(
		object = ft$body$styles$cells$border.width.left$data, 
		expected = 0
	)
	
})

test_that("Vertical lines are correctly set in a flextable summary table", {
			
	summaryTable <- data.frame(
		TRT = c("A", "A", "B", "B"),
		AVISIT = c("a", "b", "a", "b"),
		n = c("1", "2", "3", "4"),
		stringsAsFactors = FALSE
	)
		
	ft <- exportSummaryStatisticsTable(
		summaryTable, 
		colVar = c("TRT", "AVISIT"),
		statsVar = "n",
		vline = "auto"
	)

	expect_setequal(
		object = ft$body$styles$cells$border.width.left$data[, c(1, 3)], 
		expected = 1
	)
			
})

test_that("Horizontal lines are not included in a flextable summary table when requested", {
			
	summaryTable <- data.frame(
		PARAM = c("a", "b"),
		n = c("1", "2"),
		Mean = c("1", "2"),
		stringsAsFactors = FALSE
	)
			
	ft <- exportSummaryStatisticsTable(
		summaryTable, 
		rowVar = "PARAM",
		statsVar = c("n", "Mean"),
		hline = "none"
	)
	expect_setequal(
		object = ft$body$styles$cells$border.width.top$data, 
		expected = 0
	)
	
})


test_that("Horizontal lines are correctly set in a flextable summary table", {
			
	summaryTable <- data.frame(
		PARAM = c("a", "b"),
		n = c("1", "2"),
		Mean = c("1", "2"),
		stringsAsFactors = FALSE
	)
	
	ft <- exportSummaryStatisticsTable(
		summaryTable, 
		rowVar = "PARAM",
		statsVar = c("n", "Mean"),
		hline = "auto"
	)
	expect_setequal(
		object = ft$body$styles$cells$border.width.top$data[4, ], 
		expected = 1
	)
	
})

test_that("A warning is generated if no data remain after filtering of the column totals in a flextable summary table", {
			
	data <- data.frame(
		isTotal = rep(TRUE, 2),
		n = c(1, 2)
	)
	expect_warning(
		exportSummaryStatisticsTable(data),
		regexp = "No data remain after filtering of total rows."
	)
			
})

test_that("An error is generated if a flextable summary table contains multiple values for the column total but no column variables are specified", {
			
	data <- data.frame(
		isTotal = c(FALSE, TRUE, TRUE),
		statN = c(1, 2, 3)
	)
	expect_error(
		exportSummaryStatisticsTable(data),
		regexp = "Multiple values for the header total .*"
	)
			
})

test_that("A superscript is correctly formatted in a flextable summary table", {
			
	# Example with row/col vars specification in specific cell
	xSps <- "<0.001^{*}"
	data <- data.frame(
		pValue = c("0.05", xSps, "1", "0.89"),
		TRT = rep(c("A", "B"), each = 2),
		PARAM = rep(c("Actual Value", "Change from Baseline"), times = 2)
	)
	ft <- exportSummaryStatisticsTable(
		data, 
		rowVar = "PARAM", colVar = "TRT",
		statsVar = "pValue"
	)
	
	if(packageVersion("flextable") >= "0.9.5"){
		
		ftDataChunk <- flextable::information_data_chunk(ft)
		ftDataTxtSuperscript <- subset(ftDataChunk, txt == "*")
		expect_equal(
			object = ftDataTxtSuperscript[, "vertical.align"], 
			expected = "superscript"
		)
		
	}else{
	
		idxSps <- which(ft$body$dataset == xSps)
		ftBodyCnt <- ft$body$content$content$data
		cntDataSps <- ftBodyCnt[idxSps][[1]]
		expect_equal(nrow(cntDataSps), 2)
		expect_equal(cntDataSps[, "txt"], c("<0.001", "*"))
		expect_equal(cntDataSps[, "vertical.align"], c(NA_character_, "superscript"))
		
		alignDataOther <- unlist(lapply(ftBodyCnt[-idxSps], "[", "vertical.align"))
		expect_setequal(alignDataOther, NA_character_)
		
	}
			
})

test_that("A subscript is correctly formatted in a flextable summary table", {
			
	# Example with row/col vars specification in specific cell
	xSbs <- "<0.001_{(significative)}"
	data <- data.frame(
		pValue = c("0.05", xSbs, "1", "0.89"),
		TRT = rep(c("A", "B"), each = 2),
		PARAM = rep(c("Actual Value", "Change from Baseline"), times = 2)
	)
	ft <- exportSummaryStatisticsTable(
		data, 
		rowVar = "PARAM", colVar = "TRT",
		statsVar = "pValue"
	)
	
	if(packageVersion("flextable") >= "0.9.5"){
		
		ftDataChunk <- flextable::information_data_chunk(ft)
		ftDataSubscript <- subset(ftDataChunk, txt == "(significative)")
		expect_equal(
			object = ftDataSubscript[, "vertical.align"], 
			expected = "subscript"
		)
		
	}else{
			
		idxSbs <- which(ft$body$dataset == xSbs)
		ftBodyCnt <- ft$body$content$content$data
		cntDataSbs <- ftBodyCnt[idxSbs][[1]]
		expect_equal(nrow(cntDataSbs), 2)
		expect_equal(cntDataSbs[, "txt"], c("<0.001", "(significative)"))
		expect_equal(cntDataSbs[, "vertical.align"], c(NA_character_, "subscript"))
				
		alignDataOther <- unlist(lapply(ftBodyCnt[-idxSbs], "[", "vertical.align"))
		expect_setequal(alignDataOther, NA_character_)
		
	}
			
})

test_that("A cell is correctly formatted in bold in a flextable summary table", {
			
	# Example with row/col vars specification in specific cell
	xBold <- "bold{<0.001}"
	data <- data.frame(
		pValue = c("0.05", xBold, "1", "0.89"),
		TRT = rep(c("A", "B"), each = 2),
		PARAM = rep(c("Actual Value", "Change from Baseline"), times = 2)
	)
	ft <- exportSummaryStatisticsTable(
		data, 
		rowVar = "PARAM", colVar = "TRT",
		statsVar = "pValue"
	)
	
	if(packageVersion("flextable") >= "0.9.5"){
		
		ftDataChunk <- flextable::information_data_chunk(ft)
		ftDataBold <- subset(ftDataChunk, txt == "<0.001")
		expect_true(object = ftDataBold[, "bold"])
		
	}else{
	
		isBold <- apply(
			ft$body$content$content$data, 
			2, function(x) sapply(x, `[[`, "bold")	
		)
		idxBold <- which(ft$body$dataset == xBold)
		expect_setequal(isBold[idxBold], TRUE)
		expect_setequal(isBold[-idxBold], NA)
		
	}
			
})

test_that("A cell is correctly formatted with multiple same text formatting in a flextable summary table", {
			
	# Example with row/col vars specification in specific cell
	data <- data.frame(
		pValue = c("0.05", "1", "1", "0.89"),
		TRT = rep(c("A", "B"), each = 2),
		PARAM = rep(c("Actual Value^{test} of the measurement^{test2} in data", 
			"Change from Baseline"), times = 2)
	)
	ft <- exportSummaryStatisticsTable(
		data, 
		rowVar = "PARAM", colVar = "TRT",
		statsVar = "pValue"
	)
	
	if(packageVersion("flextable") >= "0.9.5"){
		
		ftDataChunk <- flextable::information_data_chunk(ft)
		expect_equal(
			object = subset(ftDataChunk, txt == "test")[, "vertical.align"],
			expected = "superscript"
		)
		expect_equal(
			object = subset(ftDataChunk, txt == "test2")[, "vertical.align"],
			expected = "superscript"
		)
		
	}else{
	
		idxSps <- which(grepl("Actual Value", ft$body$dataset))
		ftBodyCnt <- ft$body$content$content$data
		cntDataSps <- ftBodyCnt[idxSps][[1]]
		
		expect_equal(
			object = cntDataSps[, "txt"], 
			expected = c("Actual Value", "test", " of the measurement", "test2",
				" in data")
		)
		expect_equal(
			object = cntDataSps[, "vertical.align"], 
			expected = c(NA_character_, "superscript", NA_character_, 
				"superscript", NA_character_)
		)
		
	}
	
})

test_that("A cell is correctly formatted with multiple and different text formatting in a flextable summary table", {
			
	# Example with row/col vars specification in specific cell
	x <- "^{test}bold{<0.001}"
	data <- data.frame(
		pValue = c("0.05", x, "1", "0.89"),
		TRT = rep(c("A", "B"), each = 2),
		PARAM = rep(c("Actual Value", "Change from Baseline"), times = 2)
	)
	ft <- exportSummaryStatisticsTable(
		data, 
		rowVar = "PARAM", colVar = "TRT",
		statsVar = "pValue"
	)
	
	if(packageVersion("flextable") >= "0.9.5"){
		
		ftDataChunk <- flextable::information_data_chunk(ft)
		expect_equal(
			object = subset(ftDataChunk, txt == "test")[, "vertical.align"],
			expected = "superscript"
		)
		expect_true(object = subset(ftDataChunk, txt == "<0.001")[, "bold"])
		
	}else{
		
		cntData <- ft$body$content$content$data[2, 2][[1]]
		
		expect_equal(object = cntData[, "txt"], expected = c("test", "<0.001"))
		expect_equal(object = cntData[1, "vertical.align"], expected = "superscript")
		expect_true(object = cntData[2, "bold"])
		
	}
	
})

test_that("A summary table is correctly exported to a docx file", {

	data <- data.frame(
		pValue = c("0.05", "<0.001", "1", "0.89"),
		TRT = rep(c("A", "B"), each = 2),
		PARAM = rep(c("Actual Value", "Change from Baseline"), times = 2)
	)
	
	file <- tempfile(pattern = "table", fileext = ".docx")	
	expect_silent(
		ft <- exportSummaryStatisticsTable(
			data, 
			rowVar = "PARAM", colVar = "TRT",
			statsVar = "pValue",
			file = file
		)
	)
	
	# has file been created?
	expect_true(file.exists(file))
	
	# check if content is correct
	# (Note: this is based on the 'officerverse' user documentation of officer)
	doc <- officer::read_docx(file)
	docTable <- subset(docx_summary(doc), `content_type` == "table cell")
	
	docTableData <- with(
		subset(docTable, !`is_header`),
		tapply(text, list(`row_id`, `cell_id`), I)
	)
	docTableHeader <- with(
		subset(docTable, is_header), 
		tapply(text, `cell_id`, I)
	)
	colnames(docTableData) <- docTableHeader
	
	dataRef <- cbind(
		PARAM = c("Actual Value", "Change from Baseline"),
		A = c("0.05", "<0.001"),
		B = c("1", "0.89")
	)
	expect_equal(object = docTableData, expected = dataRef, check.attributes = FALSE)
	
})

test_that("A summary table is correctly exported to a docx file in landscape format", {
		
	file <- tempfile(pattern = "table", fileext = ".docx")	
	ft <- exportSummaryStatisticsTable(
		summaryTable = data.frame(n = 10),
		file = file,
		landscape = TRUE
	)
	doc <- officer::read_docx(file)
	docCnt <- officer::docx_body_xml(doc)
	expect_match(
		object = as.character(docCnt), 
		regexp = 'orient=\"landscape\"', 
		fixed = TRUE
	)
			
})

test_that("A summary table is correctly exported to a docx file in landscape format without additional empty pages", {
  
  file <- tempfile(pattern = "table", fileext = ".docx")
  ft <- exportSummaryStatisticsTable(
    summaryTable = data.frame(n = 10),
    file = file,
    landscape = TRUE
  )
  
  # check that document doesn't contain any extra paragraph (i.e. empty page)
  doc <- officer::read_docx(file)
  docCntType <- officer::docx_summary(doc)[, "content_type"]
  expect_false("paragraph" %in% docCntType)
  
})

test_that("A list of summary tables is correctly exported to docx files", {
			
	summaryTables <- list(
		`PARAM 2` = data.frame(n = 10),
		`PARAM 1` = data.frame(n = 2)
	)
			
	file <- tempfile(pattern = "table", fileext = ".docx")		
	dts <- exportSummaryStatisticsTable(
		summaryTables, 
		file = file,
		colHeaderTotalInclude = FALSE
	)
			
	fileTableOutput <- paste0(
		tools::file_path_sans_ext(file),
		"_", c("1", "2"), ".docx"
	)
	expect_true(all(file.exists(fileTableOutput)))
	
	for(iTable in seq_along(summaryTables)){

		expect_equal(
			object = {
				doc <- officer::read_docx(fileTableOutput[!!iTable])
				docTables <- subset(officer::docx_summary(doc), `content_type` == "table cell")
				docTables$text
			}, 
			expected = {
				table <- summaryTables[[!!iTable]]
				tableCnt <- c(names(summaryTables)[!!iTable], colnames(table), unlist(table))
				unname(tableCnt)
			}
		)
	}
			
})

test_that("A list of summary tables is correctly exported to a single docx file", {
			
	# Note: this functionality is currently not available 
	# in exportSummaryStatisticsTable
	# only in exportFlextableToDocx
	summaryTables <- list(
		`PARAM 2` = flextable(data.frame(n = "10")),
		`PARAM 1` = flextable(data.frame(n = "2"))
	)
	
	file <- tempfile(pattern = "table", fileext = ".docx")	
	exportFlextableToDocx(object = summaryTables, file = file)		
	
	doc <- officer::read_docx(file)
	docTables <- subset(officer::docx_summary(doc), `content_type` == "table cell")
	expect_equal(docTables$text, c("n", "10", "n", "2"))
	
})

test_that("A flextable summary table is correctly styled for a report", {
	
	summaryTable <- data.frame(n = 9)	
			
	expect_identical(
		object = exportSummaryStatisticsTable(
			summaryTable = summaryTable, 
			style = "report"
		),
		expected = exportSummaryStatisticsTable(
			summaryTable = summaryTable, 
			fontname = "Times", fontsize = 8,
			landscape = FALSE,
			pageDim = getDimPage(style = "report", margin = 0),
			colorTable = getColorPaletteTable(style = "report")
		)
	)
	
})

test_that("A flextable summary table is correctly styled for a presentation", {
			
	summaryTable <- data.frame(n = 9)	
	
	expect_identical(
		object = exportSummaryStatisticsTable(
			summaryTable = summaryTable, 
			style = "presentation"
		),
		expected = exportSummaryStatisticsTable(
			summaryTable = summaryTable, 
			fontname = "Tahoma", fontsize = 10,
			landscape = TRUE,
			pageDim = getDimPage(
				style = "presentation", margin = 0,
				landscape = FALSE
			),
			colorTable = getColorPaletteTable(style = "presentation")
		)
	)
			
})

test_that("Column headers of a summary table with identical elements across consecutive columns or rows are correctly merged", {
  
  # Example with:
  # - column that should not be merged based on previous column header:
  # [TRT1, TRT1, Placebo] and [TRT2, Placebo, Placebo]
  # - nesting of > 2 cells: Placebo (rows) and TRT1 (columns)
  summaryTable <- data.frame(
    TRT01P = factor(c("TRT1", "TRT1", "TRT1", "TRT2", "TRT2"), 
      levels = c("TRT1", "TRT2")),
    TRT02P = factor(c("TRT1", "TRT1", "Placebo", "TRT1", "TRT1"), 
        levels = c("TRT1", "Placebo")),
    TRT03P = factor(
      c("TRT1", "Placebo", "Placebo", "TRT1", "TRT2"),
      levels = c("TRT1", "TRT2", "Placebo")
    ),
    variable = "A",
    n = as.character(1:5),
    stringsAsFactors = FALSE
  )		
  
  ft <- exportSummaryStatisticsTable(
    summaryTable = summaryTable,
    statsVar = "n",
    colVar = c("TRT01P", "TRT02P", "TRT03P"),
    rowVar = "variable",
    style = "presentation"
  )
  # columns are correctly merged
  expect_equal(
    ft$header$spans$rows,
    rbind(
      c(1, 3, 0, 0, 2, 0), # TRT01P
      c(1, 2, 0, 1, 2, 0), # TRT02P
      c(1, 1, 1, 1, 1, 1) # TRT03P
    )
  )
  # rows are correctly merged:
  expect_equal(
    ft$header$spans$columns,
    cbind(c(1, 1, 1), c(1, 1, 1), c(1, 1, 1), c(1, 2, 0), c(1, 1, 1), c(1, 1, 1))
  )
  
})

test_that("Column headers of a summary table with identical elements across consecutive columns and rows are correctly merged", {
  
  summaryTable <- data.frame(
    TRT01P = c("TRT1", "TRT1"),
    TRT02P = c("TRT1", "TRT1"),
    AVISIT = factor(c("Baseline", "Week 10"), levels = c("Baseline", "Week 10")),
    n = as.character(1:2),
    stringsAsFactors = FALSE
  )		
  
  ft <- exportSummaryStatisticsTable(
    summaryTable = summaryTable,
    statsVar = "n",
    colVar = c("TRT01P", "TRT02P", "AVISIT"),
  )
  # columns are correctly merged
  expect_equal(
    ft$header$spans$rows,
    rbind(
      c(2, 0), # TRT01P
      c(2, 0), # TRT02P
      c(1, 1) # AVISIT
    )
  )
  # rows are correctly merged:
  expect_setequal(
    ft$header$spans$columns,
    c(c(2, 2), c(2, 0), c(1, 2))
  )
  
})

test_that("Column headers of a summary table with less elements than row variables are not replicated", {
  
  summaryTable <- data.frame(
    TRT01P = c(rep("TRTX", 2), rep("Placebo", 4)),
    TRT02P = c(rep("TRTX", 4), rep("Placebo", 2)),
    PARCAT = "Z",
    PARAM = "X",
    variable = "A",
    variableGroup = rep(c("a", "b"), times = 3),
    n = c("1", "2", "3", "4", "5", "6"),
    stringsAsFactors = FALSE
  )		
  
  ft <- exportSummaryStatisticsTable(
    summaryTable = summaryTable,
    rowVar = c("PARCAT", "PARAM", "variable", "variableGroup"),
    statsVar = "n",
    colVar = c("TRT01P", "TRT02P"),
    vline = "auto"
  )
  
  expect_equal(
    ft$header$dataset,
    data.frame(
      c("PARCAT", "PARAM", "variable", "variableGroup"),
      c("Placebo", "Placebo", NA_character_, NA_character_),
      c("Placebo", "TRTX", NA_character_, NA_character_),
      c("TRTX", "TRTX", NA_character_, NA_character_)
    ),
    check.attributes = FALSE
  )
  
})

test_that("Column headers of a summary table are not merged if specified", {
  
  summaryTable <- data.frame(
    TRT01P = c("TRTX", "TRTX", "Placebo"),
    TRT02P = c("TRTX", "Placebo", "Placebo"),
    variable = "A",
    n = c("1", "2", "3"),
    stringsAsFactors = FALSE
  )		
  
  ft <- exportSummaryStatisticsTable(
    summaryTable = summaryTable,
    rowVar = "variable",
    statsVar = "n",
    colVar = c("TRT01P", "TRT02P"),
    colHeaderMerge = FALSE
  )
  # columns are correctly merged
  expect_setequal(object = ft$header$spans$rows, expected = 1)
  
  # rows are correctly merged:
  expect_setequal(object = ft$header$spans$columns, expected = 1)
  
})

#' Format or create flextable for listings.
#' 
#' Flextable version >= 0.4.7 and pandoc >= 2.4
#' is required to included such table in a Rmarkdown document.
#' @param data data.frame with data used in table.
#' @param ft Corresponding \code{\link[flextable]{flextable}}.
#' @param border Logical, if TRUE add a border.
#' @param highlight Integer vector with index(ices) of column(s) to highlight
#' (only applies for \code{style}: 'presentation').
#' 0 for rownames (if present).
#' Colors for:
#' \itemize{
#' \item{highlighted columns is specified in \code{colorTable["headerBackgroundHighlight"]}}
#' \item{non highlighted columns is specified in \code{colorTable["headerBackground"]}}
#' }
#' @param bgVar String with the column of the \code{data} used for
#' alternating the body background colors of the table. 
#' @param fontname String with font name, 'Times' by default.
#' @param fontsize Integer with font size, 8 by default.
#' @param adjustWidth Logical, if TRUE adjust column widths,
#' to comply to specification of \code{landscape},
#' \code{margin} and \code{pageDim}
#' (only set to \code{FALSE} if e.g. table dimensions 
#' are pre-set with the specified \code{ft}).
#' @param title Character vector with title(s) for the table.
#' Set to NULL (by default) if no title should be included.
#' Only available if \code{ft} is not specified.
#' @param align Logical, if TRUE (by default), 
#' default alignment is set ('center' in all table).
#' @param colorTable Named character vector with color for the table,
#' see output of \code{\link{getColorPaletteTable}} for required elements.
#' @examples
#' # style: report or presentation
#' getListing(data = head(mtcars), style = "report")
#' getListing(data = head(mtcars), style = "presentation")
#' # remove rownames (included by default)
#' getListing(data = head(mtcars), style = "presentation", includeRownames = FALSE)
#' # highlight:
#' #  all columns
#' getListing(data = head(mtcars), style = "presentation", highlight = seq_along(mtcars))
#' # rownames
#' getListing(data = head(mtcars), style = "presentation", highlight = 0)
#' # specific columns
#' getListing(data = head(mtcars), style = "presentation", highlight = c(2, 4))
#' @inheritParams getDimPage
#' @inheritParams createFlextableWithHeader
#' @return \code{\link[flextable]{flextable}} with style.
#' @import flextable
#' @importFrom officer fp_border
#' @importFrom magrittr "%>%"
#' @export
getListing <- function(
    data, ft, 
    border = TRUE,
    highlight = integer(),
    bgVar = NULL,
    fontname = switch(style, 'report' = "Times", 'presentation' = "Tahoma"),
    fontsize = switch(style, 'report' = 8, 'presentation' = 10),
    landscape = (style == "presentation"),
    style = "report",
    margin = 1,
    adjustWidth = TRUE,
    colorTable = getColorPaletteTable(style = style),
    align = TRUE,
    title = NULL,
    pageDim = NULL,
    includeRownames = TRUE
) {
  
  style <- match.arg(style, choices = c("report", "presentation"))
  
  if(missing(data) & missing(ft))
    stop("Input 'data' or flextable object (as 'ft') should be specified.")
  
  isColorElSpec <- function(el)
    !is.null(colorTable) && !is.na(colorTable[el])
  
  shiftCol <- if(missing(ft)) {
        ft <- createFlextableWithHeader(data = data, title = title, includeRownames = includeRownames)$ft
        hasRN <- includeRownames && !is.null(rownames(data))
        if(!hasRN & 0 %in% highlight){
          warning("Rownames are not highlighted because no rownames or 'includeRownames' is set to FALSE.")
          highlight <- setdiff(highlight, 0)
        }
        as.integer(hasRN)
      } else	0
  
  # by default, bottom and top padding are set to 2
  ft <- ft %>% padding(padding.top = 0, padding.bottom = 0)
  
  if(isColorElSpec("line")) {
    bd <- fp_border(color = colorTable["line"])
  } else	bd <- fp_border()
  
  # set fontsize
  ft <- fontsize(ft, size = fontsize, part = "all")
  
  # set header in bold
  ft <- bold(ft, part = "header")
  
  # set font
  ft <- ft %>% font(fontname = fontname, part = "all")
  
  # set border
  if(border) {
    ft <- border_remove(ft) %>%
        border_outer(border = bd, part = "all")%>% 
        vline(border = bd, part = "body") %>%
        vline(border = bd, part = "header")
    if(style == "presentation")
      ft <- ft %>% hline(border = bd, part = "body")
    if(!is.null(title))
      ft <- ft %>% hline(j = length(title), border = bd, part = "header") 
  }
  
  ## change color text + background
  # header text color:
  if(isColorElSpec("header"))
    ft <- ft %>% color(color = colorTable["header"], part = "header")
  # footer text color
  if(isColorElSpec("footer"))
    ft <- ft %>% color(color = colorTable["footer"], part = "footer")
  # footer background color
  if(isColorElSpec("footerBackground"))
    ft <- ft %>% bg(bg = colorTable["footerBackground"], part = "footer")
  # general text color
  if(isColorElSpec("body"))
    ft <- ft %>% color(color = colorTable["body"], part = "body")
  
  # highlight columns:
  nCol <- length(ft$body$dataset)
  idxHigh <- highlight + shiftCol
  idxNonHigh <- setdiff(seq_len(nCol), highlight + shiftCol) 
  if(length(idxHigh) > 0) {
    if(isColorElSpec("headerBackgroundHighlight")) {
      ft <- ft %>% 
          bg(
              bg = colorTable["headerBackgroundHighlight"], 
              part = "header", j = idxHigh
          )
    } else {
      warning(
          "Header background color for highlighted columns",
          " should be specified",
          " (in 'headerBackgroundHighlight' in colorTable),",
          " highlighting is ignored."
      )
    }
  }
  if(length(idxNonHigh) > 0) {
    if(isColorElSpec("headerBackground")) {
      ft <- ft %>% 
          bg(
              bg = colorTable["headerBackground"], 
              part = "header", j = idxNonHigh
          )
    } else {
      warning(
          "Header background color for non highlighted columns",
          " should be specified",
          " (in 'headerBackground' in colorTable),",
          " bg color for non-highlighted columns is ignored."
      )
    }
  }
  
  # alternate background between elements of either column 'bgVar' of first column
  if(!is.null(bgVar)) bgIdx <- which(colnames(data) %in% bgVar) else bgIdx <- 1
  xBg <- convertVectToBinary(x = data[, bgIdx])
  if(length(idxNonHigh) > 0) {
    if(isColorElSpec("bodyBackground1")) {
      ft <- ft %>%
          bg(
              bg = colorTable["bodyBackground1"], 
              i = which(xBg %% 2 == 0), 
              part = "body", 
              j = idxNonHigh
          )
    }
    if(isColorElSpec("bodyBackground2")) {
      ft <- ft %>%
          bg(
              bg = colorTable["bodyBackground2"], 
              i = which(xBg %% 2 == 1), 
              part = "body", 
              j = idxNonHigh
          )
    }
    
    if(length(idxHigh) > 0){
      if(isColorElSpec("bodyBackgroundHighlight1")) {
        ft <- ft %>%
            bg(
                bg = colorTable["bodyBackgroundHighlight1"], 
                i = which(xBg %% 2 == 0), 
                part = "body", 
                j = idxHigh
            )
      }
      if(isColorElSpec("bodyBackgroundHighlight2")) {
        ft <- ft %>%
            bg(
                bg = colorTable["bodyBackgroundHighlight2"], 
                i = which(xBg %% 2 == 1), 
                part = "body", 
                j = idxHigh
            )
      }
    }
    
    if(isColorElSpec("bodyBackground"))
      ft <- ft %>% bg(bg = colorTable["bodyBackground"], part = "body")
  }
  
  if(adjustWidth) {
    widthPage <- getDimPage(
        type = "width", landscape = landscape, margin = margin,
        style = style, pageDim = pageDim
    )
    width <- widthPage/nCol
    ft <- width(ft, width = width)
  }
  
  if(align)
    ft <- align(ft, align = "center", part = "all")
  
  # by default, height of each header/footer (excepted the first one) line is quite big
  ft <- height(ft, height = dim_pretty(ft, part = "header")$heights, part = "header")
  ft <- height(ft, height = dim_pretty(ft, part = "footer")$heights, part = "footer")
  
  return(ft)
  
}



#' Convert vector to a bincode of 0/1
#' based on consecutive values in the vector.
#' @param x Vector.
#' @return Integer vector of same length than \code{x}.
#' @author Laure Cougnaud
#' @examples 
#' x <- c("group1", "group1", "group1", "group2", "group2", "group3", "group4", "group4")
#' convertVectToBinary(x = x)
#' @export
convertVectToBinary <- function(x) {
  
  xBin <- rep(NA, length(x))
  idxChg <- c(1, which(diff(as.numeric(factor(x, exclude = FALSE))) != 0) + 1)
  xBin[idxChg] <- rep(c(0, 1), length.out = length(idxChg))
  for(i in seq_along(xBin)) {
    if(is.na(xBin[i]))	xBin[i] <- xBin[i-1]
  }
  return(xBin)
  
}

#' Create a flextable, setting the column names to syntactic names
#' if it is not the case.
#' @param data Data.frame with data.
#' @param headerDf (optional) Data.frame with header.
#' This should contain the same number of columns than \code{data}
#' (+ if \code{includeRownames} is TRUE) and optionally multiple rows. 
#' Neighbouring cells with same content
#' will be represented merged in the output.
#' @param title Character vector with title(s) for the table.
#' Set to NULL (by default) if no title should be included.
#' @param includeRownames Logical, if TRUE (by default)
#' rownames are included in the \code{\link[flextable]{flextable}} object.
#' @inheritParams inTextSummaryTable-flextable-args
#' @return list with:
#' \itemize{
#' \item{'ft': \code{\link[flextable]{flextable}}}
#' \item{'colsData': Named vector with original column names,
#' with names set to new syntactic names.}
#' }
#' @author Laure Cougnaud
#' @importFrom magrittr "%>%"
#' @import flextable
#' @importFrom stats setNames
#' @export
createFlextableWithHeader <- function(data, 
  headerDf = NULL, title = NULL,
  includeRownames = TRUE,
  colHeaderMerge = TRUE) {
  
  # bind rownames with data (not included in flextable by default)
  if(includeRownames) {
    
    colnamesInit <- colnames(data)
    data <- cbind.data.frame(rownames(data), data, stringsAsFactors = FALSE)
    colnames(data) <- c("", colnamesInit)
    
  }
  
  if(!is.null(headerDf) && ncol(headerDf) != ncol(data)) {
    stop(
        "Header should have the same number of columns than the data.",
        if(includeRownames)	" Did you forget to specify header for the rows?"
    )
  }
  
  # re-label the columns to avoid the error: 'invalid col_keys, flextable support only syntactic names'
  colsDataFt <- setNames(colnames(data), paste0("col", seq_len(ncol(data))))
  colnames(data) <- names(colsDataFt)
  
  if(!is.null(headerDf)) {
    colnames(headerDf) <- names(colsDataFt)
    # convert each column to character to avoid merging issue with 'rbind.data.frame'
    headerDf[, seq_len(ncol(headerDf))] <- lapply(headerDf, as.character)
  } else {
    headerDf <- as.data.frame(t(colsDataFt), stringsAsFactors = FALSE)
  }
  
  # add title
  if(!is.null(title) && !(length(title) == 1 && title == "")) {
    titleDf <- replicate(length(colsDataFt), title)
    if(is.matrix(titleDf))	colnames(titleDf) <- names(colsDataFt)
    headerDf <- rbind.data.frame(titleDf, headerDf, stringsAsFactors = FALSE)
  }
  
  mapping <- as.data.frame(t(headerDf), stringsAsFactors = FALSE)
  mapping$`col_keys` <- rownames(mapping)
  
  # base flextable
  ft <- flextable(data)
  
  ft <- set_header_df(x = ft, mapping = mapping)
  
  # merge cells with similar content in the header
  if(colHeaderMerge)
    ft <- mergeHeaderCols(x = ft, mapping = mapping)
  
  res <- list(ft = ft, colsData = colsDataFt)
  return(res)
  
}

#' Merge columns and rows in the header of a flextable object.
#' 
#' This:
#' \itemize{
#' \item{(1) for each row: checks that subsequent columns are identical (and corresponding
#' sub-header overlap), so should be merged}
#' \item{(2) for each column: checks that subsequent rows are identical, so should be
#' merged}
#' \item{filters cells planned to be merged across rows (2) if they are already
#' merged across columns (1)}
#' }
#' @inheritParams flextable::set_header_df
#' @return Updated flextable object with merged cells in the header.
#' @importFrom flextable set_header_df
#' @keywords internal
mergeHeaderCols <- function(x, mapping){
  
  headerCnt <- mapping[, setdiff(colnames(mapping), "col_keys"), drop = FALSE]
  headerCnt <- t(headerCnt)
  nRows <- nrow(headerCnt)
  nCols <- ncol(headerCnt)
  
  # get indices of replicated elements in a vector
  getDuplEl <- function(x){
    xRle <- rle(x)$lengths
    end <- cumsum(xRle)
    start <- end - xRle + 1
    idx <- mapply(FUN = c, start, end, SIMPLIFY = FALSE)
    idx <- idx[which(start != end)]
    if(length(x) > 0){
      # add intermediate indices
      idx <- lapply(idx, function(x) seq(from = x[1], to = x[2]))
    }
    return(idx)
  }
  
  ## column merging
  idxColToMerge <- lapply(seq_len(nRows), function(iRow){
    # consider the current and previous column headers
    xColHead <- apply(headerCnt[seq_len(iRow), , drop = FALSE], 2, paste, collapse = ".")
    lapply(getDuplEl(xColHead), function(x) list(row = iRow, col = x))
  })
  idxColToMerge <- unlist(idxColToMerge, recursive = FALSE)
  
  # if columns to be merged are identical across consecutive rows
  # merge the rows together into one single merging item
  colsToMerge <- sapply(idxColToMerge, function(x) paste(x[["col"]], collapse = "."))
  idxToRemove <- c()
  for(colComb in unique(colsToMerge)){
    idxCols <- which(colsToMerge == colComb)
    if(length(idxCols) > 1){
      rows <- unique(unlist(lapply(idxColToMerge[idxCols], `[[`, "row")))
      cols <- unique(unlist(lapply(idxColToMerge[idxCols], `[[`, "col")))
      for(row in rows){
        # if last row contains the same columns to merge...
        if( (row-1) %in% rows){
          # ... and the same elements
          if(length(unique(c(headerCnt[rows, cols]))) == 1){
            # combine the two mergings
            idx1 <- idxCols[which(rows == (row-1))]
            idx2 <- idxCols[which(rows == row)]
            idxColToMerge[[idx1]][["row"]] <- c(idxColToMerge[[idx1]][["row"]], idxColToMerge[[idx2]][["row"]])
            idxToRemove <- c(idxToRemove, idx2)
          }
        }
      }
    }
  }
  if(length(idxToRemove) > 0)
    idxColToMerge <- idxColToMerge[-idxToRemove]
  
  ## row merging
  idxRowToMerge <- lapply(seq_len(nCols), function(iCol){
    idxRowToMerge <- lapply(seq_len(nRows), function(iRow){
      if(iRow != 1){
        # consider the current and previous column headers
        xColHead <- c(headerCnt[seq_len(iRow), iCol, drop = FALSE])
        lapply(getDuplEl(xColHead), function(x) list(row = x, col = iCol))
      }
    })
    idxRowToMerge <- unlist(idxRowToMerge, recursive = FALSE)
  })
  idxRowToMerge <- unlist(idxRowToMerge, recursive = FALSE)
  
  ## remove row to merge which are already included in column merging
  
  # get cell index
  getCellIdx <- function(x){
    xGrid <- do.call(expand.grid, x)
    xInteract <- do.call(interaction, c(xGrid, list(drop = FALSE)))
    return(levels(xInteract))
  }
  idxColCells <- unlist(lapply(idxColToMerge, getCellIdx))
  idxRowToMerge <- lapply(idxRowToMerge, function(x){
    if(!any(getCellIdx(x) %in% idxColCells))
      x
  })
  
  # index of cells to be merged across rows or columns
  idxMerge <- c(idxColToMerge, idxRowToMerge)
  idxMerge <- idxMerge[!sapply(idxMerge, is.null)]
  
  # merge cells in flextable
  for(idx in idxMerge){
    x <- x %>% flextable::merge_at(i = idx[["row"]], j = idx[["col"]], part = "header")
  }

  return(x)
  
}

context("Create a flextable with header")

library(flextable)

test_that("A header is correctly set by default from the data header to a flextable", {		
			
	data <- data.frame(
		USUBJID = seq.int(2),
		x = c(1, 2)
	)
	res <- createFlextableWithHeader(data = data)
	expect_is(res, "list")
	expect_is(res$ft, "flextable")
	# table header
	expect_equal(
		object = as.character(as.vector(res$ft$header$dataset[1, ])),
		expected = c("", colnames(data))
	)
	# original column names
	expect_equal(
		object = unname(res$colsData),
		expected = c("", colnames(data))
	)
	
})

test_that("An error is generated if the flextable header is not specified for the rows when rownames are requested", {		
			
	data <- data.frame(
		USUBJID = seq.int(2),
		x = c(1, 2)
	)
	
	# in case ncol(headerDf) != ncol(data)
	headerDf <- data.frame(
		V1 = "subject ID", 
		V2 = "x variable"
	)
	expect_error(
		ft <- createFlextableWithHeader(
			data = data, header = headerDf, 
			includeRownames = TRUE
		)
	)
	
})

test_that("A header is correctly set from a specified dataset to a flextable", {		
			
	data <- data.frame(
		USUBJID = seq.int(2),
		x = c(1, 2)
	)
	headerDf <- data.frame(	
		V1 = "subject ID", 
		V2 = "x variable",  
		stringsAsFactors = FALSE
	)
	res <- createFlextableWithHeader(
		data = data, header = headerDf, 
		includeRownames = FALSE
	)
	# table header
	expect_equal(
		object = as.character(as.vector(res$ft$header$dataset[1, ])),
		expected = as.character(as.vector(headerDf[1, ]))
	)
	# original column names
	expect_equal(
		object = unname(res$colsData),
		expected = colnames(data)
	)

})

test_that("A header is correctly set from a specified dataset to a flextable with row names", {		
			
	data <- data.frame(
		USUBJID = seq.int(2),
		x = c(1, 2)
	)
	headerDf <- data.frame(
		V0 = "rownames", 
		V1 = "subject ID", 
		V2 = "x variable", 
		stringsAsFactors = FALSE
	)
	res <- createFlextableWithHeader(
		data = data, header = headerDf, 
		includeRownames = TRUE
	)
	# table header
	expect_equal(
		object = as.character(as.vector(res$ft$header$dataset[1, ])),
		expected = as.character(as.vector(headerDf[1, ]))
	)
	# original column names
	expect_equal(
		object = unname(res$colsData),
		expected = c("", colnames(data))
	)
			
})

test_that("A header and title are correctly set to a flextable", {

	data <- data.frame(
		USUBJID = seq.int(2),
		x = c(1, 2)
	)
	headerDf <- data.frame(
		V1 = "subject ID", 
		V2 = "x variable", 
		stringsAsFactors = FALSE
	)	
	titles <- c(
		"Table: content of the iris dataset", 
		"This is an informative subtitle"
	)
	res <- createFlextableWithHeader(
		data = data, header = headerDf, 
		title = titles,
		includeRownames = FALSE
	)
	expect_is(res$ft, "flextable")
	
	## table header
	
	# titles are inserted first in the header data
	for(i in seq_along(titles)){
		expect_setequal(
			object = res$ft$header$dataset[i, ], 
			expected = titles[i]
		)
	}
	
	# then header:
	ftHeader <- res$ft$header$dataset[length(titles)+1, ]
	expect_equal(
		as.character(as.vector(ftHeader)), 
		as.character(as.vector(headerDf)), 
		check.attributes = FALSE
	)

	## original column names
	expect_equal(
		object = unname(res$colsData),
		expected = colnames(data)
	)
	
})

test_that("A header spanning multiple rows and columns is correctly set to a flextable", {
	
	data <- data.frame(
		USUBJID = seq.int(2),
		x = c(1, 2),
		y = c(4, 5),
		z = c(0, 1)
	)
			
	# example table with multi-column and multi-row
	headerDf <- rbind.data.frame(
		c("subject ID", rep("Analysis values", 3)),
		c("subject ID", "x analyte", "y analyte", "z analyte"),
		stringsAsFactors = FALSE
	)
	colnames(headerDf) <- c("V1", "V2", "V3", "V4")
	
	res <- createFlextableWithHeader(
		data = data, 
		headerDf = headerDf, 
		includeRownames = FALSE
	)
	
	## table header
	expect_equal(
		object = unname(res$ft$header$dataset),
		expected = unname(headerDf)
	)
	
	## spanning header
	headerSpan <- res$ft$header$spans
	
	# (multi-row) first column span 2 rows
	expect_equal(
		object = headerSpan$columns[, 1],
		expected = c(2, 0)
	)
	expect_setequal(
		object = headerSpan$columns[, -1],
		expected = 1
	) # others
	# (multi-column) first row of second column span 3 columns
	expect_equal(
		object = headerSpan$rows[1, 2:4],
		expected = c(3, 0, 0)
	)
	expect_setequal(
		object = c(headerSpan$rows[-1, ], headerSpan$rows[1, 1]),
		expected = 1
	)# others
	
	## original column names
	expect_equal(
		object = unname(res$colsData),
		expected = colnames(data)
	)
	
})

context("Create a listing")

data <- head(mtcars)

library(flextable)

test_that("An error is generated if no data or flextable object is specified for a listing", {
	expect_error(ft <- getListing())
})

test_that("A listing is correctly styled for a report", {
	ft <- getListing(data = data, style = "report")
	expect_is(ft, "flextable")
	expect_setequal(ft$body$styles$text$font.size$data, 8)
	expect_setequal(ft$body$styles$text$font.family$data, "Times")
})

test_that("A listing is correctly styled for a presentation", {
      
	ftPres <- getListing(data = data, style = "presentation")
	expect_is(ftPres, "flextable")
	expect_setequal(ftPres$body$styles$text$font.size$data, 10)
	expect_setequal(ftPres$body$styles$text$font.family$data, "Tahoma")
      
})

test_that("A warning is generated if columns should be highlighted but no color is specified for the listing", {

	idxHighlight <- c(2, 3)
	expect_warning(
		ftHighlight <- getListing(
			data = data, 
			highlight = idxHighlight, 
			style = "presentation",
			colorTable = NULL
		)
	)
	
})

test_that("Specified columns are correctly highlighted in a listing", {
	
	idxHighlight <- c(2, 3)
	ftHighlight <- getListing(
		data = data, 
		highlight = idxHighlight, 
		style = "presentation",
		includeRownames = FALSE
	)
	# check if background colors in highlighted columns are different in non highlighted columns
	for(el in c("header", "body")){		
        
		bgColors <- ftHighlight[[el]]$styles$cells$background.color$data
		tmp <- apply(bgColors, 1, function(row){
			expect_false(unique(row[idxHighlight]) == unique(row[-idxHighlight]))
		})
        
	}
	
})

test_that("A warning is generated if rownames should be highlighted but rownames should not be included in a listing", {
      
	dataNoRn <- data
	rownames(dataNoRn) <- NULL
	expect_warning(
		ftHighlight <- getListing(
			data = dataNoRn, 
			highlight = 0, 
			style = "presentation",
			includeRownames = FALSE
		)
	)
      
})

test_that("Rownames are correctly highlighted in a listing", {

	dataWithRn <- data
	rownames(dataWithRn) <- seq_len(nrow(dataWithRn))
	ftHighlight <- getListing(
		data = dataWithRn, 
		highlight = 0, 
		style = "presentation",
		includeRownames = TRUE
	)
      
	for(el in c("header", "body")){		
        
		bgColors <- ftHighlight[[el]]$styles$cells$background.color$data
        
		# are rows included?
        expect_equal(ncol(bgColors), ncol(dataWithRn) + 1)
        tmp <- apply(bgColors, 1, function(row){
			expect_false(unique(row[1]) == unique(row[-1]))
		})
        
	}
      
})

test_that("A listing is correctly created in landscape format", {
      
	ftPortrait <- getListing(data = data, landscape = FALSE)
	ftLandscape <- getListing(data = data, landscape = TRUE)
      
	expect_gte(
		object = sum(ftLandscape$body$colwidths), 
		expected = sum(ftPortrait$body$colwidths)
	)
	expect_gte(
		object = sum(ftLandscape$header$colwidths), 
		expected = sum(ftPortrait$header$colwidths)
	)
      
})

test_that("Column widths are properly adjusted depending on page dimensions in a listing", {
      
	widths <- c(2, 50)
	tableWidths <- numeric()
	for(wid in widths){
        
		ft <- getListing(
			data = data, 
 			pageDim = c(wid, 50), margin = 0,
			adjustWidth = TRUE
		)
		expect_equal(sum(ft$body$colwidths), wid)
        
	}
      
})

test_that("A title is correctly included in a listing", {
      
	title <- "Subset of the cars dataset"
	ft <- getListing(data = data, title = title)
	expect_setequal(
		ft$header$dataset[1, ],
		title
	)
	# a border
	expect_setequal(
		ft$header$styles$cells$border.color.bottom$data[1, ],
		c("#000000", "black")
	)
      
})


test_that("A character vector is correctly converted to a binary vector", {
      
	x <- c("group1", "group1", "group1", "group2", "group2", "group3", "group4", "group4")			
	expect_equivalent(
		convertVectToBinary(x = x),
		c(0, 0, 0, 1, 1, 0, 1, 1)
	)
      
})

test_that("Background colors are correctly set based on a variable in a listing", {
      
	data <- data.frame(
		x = c("A", "B", "C", "C"),
		y = seq.int(4)
	)
	colorTable <- getColorPaletteTable(style = "presentation")
	colorTable["bodyBackground1"] <- "#FFFFFF"
	colorTable["bodyBackground2"] <- "#D9D9D9"
	ft <- getListing(
		data = data, bgVar = "x",
		colorTable = colorTable
	)
	
	expect_setequal(
		object = ft$body$styles$cells$background.color$data[c(1, 3, 4), ],
		expected = "#FFFFFF"
	)
	
	expect_setequal(
		object = ft$body$styles$cells$background.color$data[2, ],
		expected = "#D9D9D9"
	)
      
})

#' Get color palette for the tables
#' 
#' This function gets the color palettes for the tables 
#' specified as global options.
#' 
#' By default, the function returns the palette of the package. 
#' The user can specify a custom palette by setting the global options.
#' @param style String with style of report. Either 'report' or
#' 'presentation'. By default, the style is 'report'.
#' @return A named vector with hex colors.
#' @examples 
#' # report style (the default)
#' getColorPaletteTable()
#' # presentation style
#' getColorPaletteTable(style = "presentation")
#' # custom palette
#' customColorTable <- c('header' = "#FFFFFF",'headerBackground' = "#3F4788FF", 
#' 'body' = "#000000", 'bodyBackground1' = "#D9D9D9", 'bodyBackground2' = "#D9D9D9", 
#' 'footer' = "#000000", 'footerBackground' = "#FFFFFF",'line' = "#FFFFFF")
#' options(inTextSummaryTable.colors.table.presentation = customColorTable)
#' getColorPaletteTable("presentation")
#' @export 
getColorPaletteTable <- function(style = c("report", "presentation")) {
  
  style <- match.arg(style)
  
  switch(style,
      'report' = tableColorsReport,
      'presentation' = getOption("inTextSummaryTable.colors.table.presentation")
  )
  
  # old 'report' = getOption("inTextSummaryTable.colors.table.report")
  
}


#' Colors for tables in a report style
#' 
#' Default colors are black text on a white background.
#' @export
tableColorsReport <- c(    
    'header'           = "#000000", # black      
    'headerBackground' = "#FFFFFF", # white 
    'body' 			   = "#000000", 
    'bodyBackground'   = "#FFFFFF",
    'footer' 		   = "#000000",
    'footerBackground' = "#FFFFFF",
    'line' 			   = "#000000"
)

#' Colors for tables in a presentation style
#' 
#' Default colors are
#' \itemize{
#' \item{header: white text on a blue background}
#' \item{body: black text on a grey background}
#' \item{footer: black text on a white background.}
#' }
#' @export
tableColorsPresentation <- c(     
    # header:
    'header' 					= "#FFFFFF",    # white text
    'headerBackground' 		    = "#32648EFF",  # lightblue
    'headerBackgroundHighlight' = "#287D8EFF",  # lighter blue
    
    # body black with alternated background color
    'body' 					 	= "#000000",   # black text
    'bodyBackground1' 		 	= "#D9D9D9",   # grey
    'bodyBackground2' 		 	= "#D9D9D9",   # grey
    'bodyBackgroundHighlight1' 	= "#DCE318FF", # yellowish
    'bodyBackgroundHighlight2' 	= "#DCE318FF", # yellowish
    
    # footer: black on white
    'footer' 					= "#000000",   # black text
    'footerBackground' 		 	= "#FFFFFF",   # white background
    # line color
    'line' 					 	= "#FFFFFF"    # white
)

#' Export summary table to an interactive DT table,
#' e.g. to be exported into an html document.
#' @param summaryTable Summary table as provided by the 
#' \code{\link{formatSummaryStatisticsTable}}.
#' @param ... (DT output) Extra parameters passed to the 
#' \code{\link[clinUtils]{getClinDT}}
#' @inheritParams inTextSummaryTable-common-args
#' @inheritParams inTextSummaryTable-DT-args
#' @inherit clinUtils::getClinDT return
#' @author Laure Cougnaud
#' @importFrom utils head
#' @importFrom clinUtils getLabelVar getClinDT
#' @importFrom tools file_path_sans_ext file_ext
exportSummaryStatisticsTableToDT <- function(
	summaryTable, 
	rowVar = getAttribute(summaryTable, "rowVar"), 
	rowVarLab = getAttribute(summaryTable, "rowVarLab", default = getLabelVar(rowVar, labelVars = labelVars)),
	rowVarInSepCol = NULL, 
	statsVar = getAttribute(summaryTable, "statsVar"),
	statsLayout = getAttribute(summaryTable, "statsLayout", default = "col"),
	statsValueLab = "StatisticValue",
	title = NULL,
	expandVar = NULL, noEscapeVar = NULL, barVar = NULL,
	pageDim = NULL,
	labelVars = NULL,
	file = NULL,
	...){

	statsLayout <- match.arg(statsLayout, choices = c("row", "col", "rowInSepCol"))
	
	if(!is.data.frame(summaryTable)){
		
		inputParams <- as.list(environment())
		res <- sapply(seq_along(summaryTable), function(i){
			summaryTableI <- summaryTable[[i]]
			inputParamsBy <- inputParams
			inputParamsBy$summaryTable <- summaryTableI
			inputParamsBy$title <- if(length(title) > 1){
				inputParams$title[i]	
			}else{c(
					inputParams$title, 
					strsplit(names(summaryTable)[i], split = "\n")[[1]]
				)
			}
			inputParamsBy$file <- if(!is.null(file)){
				paste0(file_path_sans_ext(file), "_", i, ".", file_ext(file))
			}
			do.call(exportSummaryStatisticsTableToDT, inputParamsBy)		
		}, simplify = FALSE)	
		if(!is.null(names(summaryTable)))
			names(res) <- names(summaryTable)
		
		return(res)
		
	}
	
	# important: sort data.frame with specified row variables!
	if(!is.null(rowVar))
		summaryTable <- ddply(summaryTable, rowVar)
	
	# nesting in row variables specified via: 'rowGroup' parameter for DT
	rowVarInRow <- setdiff(rowVar, rowVarInSepCol)
	if(statsLayout == "row" & "Statistic" %in% colnames(summaryTable))
		rowVarInRow <- c(rowVarInRow, "Statistic")
	# consider all row variables, excepted the last one as row group
	rowGroupVar <- head(rowVarInRow, -1)
	if(length(rowGroupVar) == 0)	rowGroupVar <- NULL
	if(length(rowGroupVar) > 1){
		rowGroupVar <- head(rowGroupVar, 1)
		warning(paste0("Currently multi-level row grouping row variable is not available in the 'DT' package",
			", so the rows are grouped by ", rowGroupVar, " only."))
	}
	
	## extract column(s) containing statistics (for expand or escape)

	# if no colVar is specified, results are stored in the column: statsValueLab
	colStat <- colnames(summaryTable)[match(statsValueLab, 
		sub("(.+)(\n\\(N=\\d{1,}\\))", "\\1", colnames(summaryTable)))
	]
	# otherwise we should extract the groups of colVar
	if(is.na(colStat)){
		colStat <- setdiff(colnames(summaryTable), c(rowVar, statsVar, "Statistic"))
	}
		
	## format expandVar
	expandIdx <- expandVarDT <- NULL
	if(!is.null(expandVar)){
		
		# extract expandVar which are in the column names
		expandVarDT <- intersect(colnames(summaryTable), expandVar)
		
		# in case expandVar is one statistic,
		# extract indices if statistics are in rows
		expandVarStats <- intersect(statsVar, expandVar)
		if(length(expandVarStats) > 0){
			specifyExpand <- 
				statsLayout %in% c("row", "rowInSepCol") & 
				"Statistic" %in% colnames(summaryTable)
		
			if(specifyExpand){
			
				idxRow <- which(summaryTable$Statistic %in% expandVarStats)
				idxColStat <- which(colnames(summaryTable) %in% colStat)
				if(length(idxColStat) == 0)	
					stop("Issue with extraction statistic value column during the formatting 'expandVar'.")
				expandIdx <- as.matrix(expand.grid(idxRow, idxColStat))
				colnames(expandIdx) <- c("row", "col")
			}
		}
		
	}
	
	## format escape
	escape <- if(!is.null(noEscapeVar)){
		
		if(is.character(noEscapeVar)){
			
			if(statsLayout %in% c("row", "rowInSepCol") && noEscapeVar %in% statsVar){
				noEscapeVar <- colStat
			}
		
			escape <- which(!colnames(summaryTable) %in% noEscapeVar)
			if(length(escape) == 0)	{
				FALSE
			}else	escape
			
		}else	noEscapeVar
		
	}else	TRUE

	# convert 'numeric-like' column to numeric in order to have nice filters
	isColNum <- sapply(seq_len(ncol(summaryTable)), function(iCol){
		x <- summaryTable[, iCol]
		if(is.character(x)){
			tryCatchW <- tryCatch(expr = as.numeric(x), warning = function(w) w)
			!inherits(x = tryCatchW, what = "warning")
		}else	FALSE
	})
	idxColNum <- which(isColNum)
	summaryTable[, idxColNum] <- sapply(summaryTable[, idxColNum, drop = FALSE],
		as.numeric, simplify = FALSE)

	# set row variable labels
	colnamesDT <- colnames(summaryTable)
	rowVarLabs <- c(
		rowVarLab[setdiff(rowVar, "Statistic")], 
		if(statsLayout != "col" && "Statistic" %in% colnames(summaryTable))	rowVarLab["Statistic"]
	)
	rowVarLabs <- rowVarLabs[!is.na(rowVarLabs)]
	colnamesDT <- if(length(rowVarLabs) > 0)
		setNames(names(rowVarLabs), rowVarLabs)
	
	# page length
	pageLength <- if(!is.null(pageDim)){
		if(length(pageDim) != 2)
			stop("'pageDim' should be of length 2.") 
		pageDim[2]
	}
	
	# bar
	barVar <- intersect(barVar, colnames(summaryTable))
	if(length(barVar) == 0) barVar <- NULL
	
	if(!is.null(title) && is.character(title) && length(title) > 1)
		title <- paste(title, collapse = " ")
	
	argsDT <- c(
		list(
			data = summaryTable,
			rowGroupVar = rowGroupVar,
			caption = title,
			escape = escape,
			barVar = barVar,
			file = file
		),
		if(!is.null(pageLength))	list(pageLength = pageLength),
		if(length(expandIdx) > 0)	list(expandIdx = expandIdx),
		if(length(expandVarDT) > 0)	list(expandVar = expandVarDT),
		if(length(colnamesDT) > 0)	list(colnames = colnamesDT)
	)
	
	# check if extra parameters for the 'getClinDT' function 
	# specified by the user are not already specified in this function
	argsDTExtra <- list(...)
	isArgsDupl <- names(argsDTExtra) %in% names(argsDT)
	if(any(isArgsDupl)){
		warning(paste("Parameter(s):", toString(sQuote(names(argsDTExtra)[isArgsDupl])),
			"are already specified internally, you may consider using in-text table specific",
			"parameters instead.")
		)
	}
	argsDT <- c(argsDT, argsDTExtra[!isArgsDupl])
	
	# create DT
	res <- do.call(getClinDT, argsDT)
	
	return(res)
	
}

#' \code{summaryTable}
#' 
#' \enumerate{
#' \item{The summary statistics are computed in
#' \code{\link{computeSummaryStatisticsTable}},
#' which creates a \code{summaryTable} object.}
#' \item{This object is exported to diverse formats via 
#' \code{\link{export}}}
#' }
#' Multiple \code{summaryTable} objects are combined
#' together with \code{\link{combine.summaryTable}}.
#' 
#' The \code{summaryTable} is an intermediary 
#' object of the package. 
#' This contains the summary statistics as data.frame.
#' This object contains:
#' \itemize{
#' \item{the row and column variable(s)}
#' \item{the computed statistic(s): \cr
#' If \code{type} is:
#' \itemize{
#' \item{'summaryTable': 
#' \itemize{
#' \item{'statN': number of subjects}
#' \item{'statMean': mean of \code{var}}
#' \item{'statSD': standard deviation of \code{var}}
#' \item{'statSE': standard error of \code{var}}
#' \item{'statMedian': median of \code{var}}
#' \item{'statMin': minimum of \code{var}}
#' \item{'statMax': maximum of \code{var}}
#' \item{'statPerc': percentage of subjects}
#' \item{'statPercTotalN': total number of subjects based on \code{dataTotalPerc},
#' denominator of \code{statPerc}}
#' \item{'statm': number of records}
#' }
#' }
#' \item{'countTable': 
#' \itemize{
#' \item{'statN': number of subjects}
#' \item{'statPercN' (or 'statPercm'): percentage of subjects
#' (or records depending on \code{statsPerc})}
#' \item{'statPercTotalN' (or 'statPercTotalm'): total number of 
#' subjects (or records) based on \code{dataTotalPerc}, and used as
#' denominator of \code{statPercN} (or 'statPercm')}
#' \item{'statm': number of records}
#' }}}
#' }
#' \item{computed statistics.\cr
#' The statistics are stored in columns corresponding
#' to names of the \code{statsVar}.\cr
#' If the specified statistics are not named and of length
#' 1, the statistics are stored in a column called: 'Statistic'.}
#' \item{variables: 
#' \itemize{
#' \item{'variable': variable name in case \code{var} is of length > 1}
#' \item{'variableGroup': in case \code{var} is of length > 1 and for 
#' variable(s) used for count: elements of the variable}
#' }
#' }
#' \item{'isTotal': variable with logical flag, TRUE if the record contain 
#' the total by column}
#' }
#' Additionally, the output contains an extra attribute
#' 'summaryTable', which is a list composed of:
#' \itemize{
#' \item{'statsVar': column name(s) of summary table with computed statistics
#' included in the final table}
#' \item{'rowVar': column name(s) of summary table with row variable
#' included in the final table. This parameter should be mainly used for qualitative variables and 'nests' together different rows in the final output table.}
#' \item{'rowVarLab': labels corresponding to the 'rowVar' attribute}
#' \item{'rowVarTotalInclude': row variables whose total will be included:
#' \code{rowVarTotalInclude} and 'variableGroup' if the variable total should be included}
#' \item{'rowVarTotalInSepRow': row variables whose total will be included in a separated row:
#' \code{rowVarTotalInSepRow} and 'variableGroup' if \code{varTotalInSepRow}}
#' \item{'colVar': column name(s) of summary table with column variable
#' included in the final table}
#' \item{'colTotalLab': label for the total}
#' } 
#' @name summaryTable
#' @return Not relevant
NULL

#' Export summary table to a flextable object,
#' e.g. to be exported in Word or PowerPoint.
#' @param summaryTable Summary table as provided by the 
#' \code{\link{formatSummaryStatisticsTable}}
#' @inheritParams inTextSummaryTable-common-args
#' @inheritParams inTextSummaryTable-flextable-args
#' @inherit convertSummaryStatisticsTableToFlextable return
#' @author Laure Cougnaud
#' @importFrom clinUtils getLabelVar
exportSummaryStatisticsTableToFlextable <- function(
	# for 'format' function
	summaryTable,
	rowVar = getAttribute(summaryTable, "rowVar"), 
	rowVarInSepCol = NULL, 
	rowVarTotalInclude = getAttribute(summaryTable, "rowVarTotalInclude"),
	statsLayout = getAttribute(summaryTable, "statsLayout", default = "row"), 	
	statsVar = getAttribute(summaryTable, "statsVar"), 
	statsLabInclude = getAttribute(summaryTable, "statsLabInclude", default = length(statsVar) > 1),
	rowVarLab = getAttribute(summaryTable, "rowVarLab", default = getLabelVar(rowVar, labelVars = labelVars)),
	rowVarTotalInSepRow = NULL,
	vline = c("none", "auto"),
	hline = c("auto", "none"),
	rowAutoMerge = TRUE,
	rowVarFormat = NULL,
	rowTotalLab = NULL,
	# for 'convert' function
	colHeaderMerge = TRUE,
	landscape = (style == "presentation"), 
	margin = 1, rowPadBase = 14.4,
	title = NULL, 
	footer = NULL,
	style = "report",
	colorTable = getColorPaletteTable(style = style),
	fontname = switch(style, 'report' = "Times", 'presentation' = "Tahoma"),
	fontsize = switch(style, 'report' = 8, 'presentation' = 10),
	file = NULL, pageDim = NULL, columnsWidth = NULL,
	labelVars = NULL){

	statsLayout <- match.arg(statsLayout, choices = c("row", "col", "rowInSepCol"))
	vline <- match.arg(vline)
	hline <- match.arg(hline)

	# custom formatting for flextable
	summaryTableLong <- formatSummaryStatisticsTableFlextable(
		summaryTable = summaryTable,
		rowVar = rowVar, rowVarInSepCol = rowVarInSepCol, rowVarTotalInclude = rowVarTotalInclude,
		statsLayout = statsLayout, statsVar = statsVar, statsLabInclude = statsLabInclude,
		rowVarLab = rowVarLab,
		vline = vline,
		hline = hline,
		rowAutoMerge = rowAutoMerge, rowVarFormat = rowVarFormat,
		rowVarTotalInSepRow = rowVarTotalInSepRow,
		rowTotalLab = rowTotalLab,
		labelVars = labelVars
	)
	
	# create flextable only with header to extract dimensions header
	summaryTableFt <- convertSummaryStatisticsTableToFlextable(
		summaryTable = summaryTableLong,
		landscape = landscape, margin = margin, rowPadBase = rowPadBase,
		title = title, footer = footer,
		style = style, fontsize = fontsize,
		file = file,
		fontname = fontname,
		colorTable = colorTable,
		pageDim = pageDim, columnsWidth = columnsWidth,
		colHeaderMerge = colHeaderMerge
	)
	
	return(summaryTableFt)
	
}

#' Common arguments for the for the statistics utility functions
#' of the inTextSummaryTable package.
#' @param x Numeric vector.
#' @param na.rm Logical, should NA value(s) be removed (FALSE by default)?
#' @name inTextSummaryTable-stats-utility
#' @return No return value, used for the documentation of 
#' stat utility R functions
NULL

#' Compute standard error of the mean.
#' 
#' The standard error of the mean is computed as:
#' \eqn{\frac{\sigma(x)}{\sqrt{length(x)}}}, with:\cr
#' \eqn{\sigma(x)}: standard deviation of \code{x}
#' @inheritParams inTextSummaryTable-stats-utility
#' @return Numeric vector with standard error of the mean
#' @author Laure Cougnaud
#' @importFrom stats na.omit sd
#' @examples
#' se(rnorm(1000))
#' @family stats utility functions
#' @export
se <- function(x, na.rm = FALSE){
	if(na.rm)	 x <- na.omit(x)
	res <- sd(x)/sqrt(length(x))
	return(res)
}

#' Compute the percentage coefficient of variation,
#' (in a scale from 0 to 100).
#' 
#' The coefficient of variation is computed as:
#' \eqn{\frac{\sigma(x)}{\bar{x}}*100}, with:
#' \itemize{
#' \item{\eqn{\sigma(x)}: standard deviation of \code{x}}
#' \item{\eqn{\bar{x}}: arithmetic mean of \code{x}}
#' }
#' @inheritParams inTextSummaryTable-stats-utility
#' @return Numeric vector of length 1 with coefficient of variation.
#' @author Laure Cougnaud
#' @importFrom stats sd
#' @examples
#' # coefficient of variation of normal distribution tends to 100%
#' cv(rnorm(n = 1000, mean = 1, sd = 1))
#' @family stats utility functions
#' @export
cv <- function(x, na.rm = FALSE){
	
	res <- sd(x, na.rm = na.rm)/mean(x, na.rm = na.rm)*100
	return(res)
	
}

#' Compute geometric mean.
#' 
#' The geometric mean is computed as:
#' \eqn{\exp(\bar{log(x)})}, with:
#' \itemize{
#' \item{log: natural logarithm}
#' \item{\eqn{\bar{log(x)}}: arithmetic mean of log(x)}
#' }
#' @inheritParams inTextSummaryTable-stats-utility
#' @return Numeric vector of length 1 with geometric mean.
#' @author Laure Cougnaud
#' @examples
#' # geometric mean of a big sample from log normal distribution
#' # tends to the mean of the distribution:
#' geomMean(rlnorm(n = 1000, meanlog = 0, sdlog = 1))
#' @family stats utility functions
#' @export
geomMean <- function(x, na.rm = FALSE){
	res <- exp(mean(log(x), na.rm = na.rm))
	return(res)
}

#' Compute geometric standard deviation
#' 
#' The geometric standard deviation is computed as:
#' \eqn{\exp(\sigma(log(x)))}, with:
#' \itemize{
#' \item{log: natural logarithm}
#' \item{\eqn{\sigma}: standard deviation}
#' }
#' @inheritParams inTextSummaryTable-stats-utility
#' @return Numeric vector of length 1 with geometric mean.
#' @author Laure Cougnaud
#' @importFrom stats sd
#' @examples
#' # geometric standard deviation of a sample from a log normal distribution:
#' geomSD(rlnorm(n = 1000, meanlog = 0, sdlog = 1))
#' @family stats utility functions
#' @export
geomSD <- function(x, na.rm = FALSE){
	res <- exp(sd(log(x), na.rm = na.rm))
	return(res)
}

#' Compute geometric coefficient of variation 
#' (in a scale from 0 to 100).
#' 
#' The geometric coefficient of variation is computed as:
#' \eqn{\sqrt{\exp(\sigma(log(x))^2)-1}*100}, with:
#' \itemize{
#' \item{log: natural logarithm}
#' \item{\eqn{\sigma}: standard deviation}
#' }
#' @inheritParams inTextSummaryTable-stats-utility
#' @return Numeric vector of length 1 with 
#' geometric coefficient of variation.
#' @author Laure Cougnaud
#' @importFrom stats sd
#' @examples
#' # Geometric coefficient of variation of a sample from a log normal distribution:
#' geomCV(rlnorm(n = 1000, meanlog = 0, sdlog = 1))
#' @family stats utility functions
#' @export
geomCV <- function(x, na.rm = FALSE){
	res <- sqrt(exp(sd(log(x), na.rm = na.rm)^2)-1)*100
	return(res)
}

#' Compute geometric standard error of the mean.
#' 
#' The geometric standard error of the mean is computed as:
#' \eqn{\exp(se(log(x)}, with:
#' \itemize{
#' \item{log: natural logarithm}
#' \item{\eqn{se}: standard error of the mean, as computed with \code{\link{se}}}
#' }
#' @inheritParams inTextSummaryTable-stats-utility
#' @return Numeric vector of length 1 with 
#' geometric standard error of the mean.
#' @author Laure Cougnaud
#' @examples
#' # Geometric standard error of the mean of a sample from a log normal distribution:
#' geomSE(rlnorm(n = 1000, meanlog = 0, sdlog = 1))
#' @family stats utility functions
#' @export
geomSE <- function(x, na.rm = FALSE){
	exp(se(log(x), na.rm = na.rm))
}

context("Compute summary statistics table with custom statistics")

test_that("A unique statistic specified as an expression is correctly computed", {
		
	set.seed(123)
	data <- data.frame(AVAL = rnorm(5), USUBJID = seq.int(5))

	expect_silent(
		summaryTableExpr <- computeSummaryStatisticsTable(
			var = "AVAL",
			data = data,
			stats = expression(statMean)
		)
	)
	
	expect_identical(
		attr(summaryTableExpr, "summaryTable")$statsVar, 
		"Statistic"
	)
	expect_equal(
		subset(summaryTableExpr, !isTotal)$Statistic, 
		mean(data$AVAL)
	)
	
})

test_that("A unique statistic specified as a name is correctly compute", {
			
	set.seed(123)
	data <- data.frame(AVAL = rnorm(5), USUBJID = seq.int(5))
	
	# as a 'name' object
	expect_silent(
		summaryTableName <- computeSummaryStatisticsTable(
			var = "AVAL",
			data = data,
			stats = bquote(statMean)
		)
	)
	expect_identical(
		attr(summaryTableName, "summaryTable")$statsVar, 
		"Statistic"
	)
	expect_equal(
		subset(summaryTableName, !isTotal)$Statistic, 
		mean(data$AVAL)
	)

})

test_that("The statistic of interest is specified as a string from the default set", {
			
	set.seed(123)
	data <- data.frame(AVAL = rnorm(5), USUBJID = seq.int(5))
		
	expect_silent(
		summaryTableString <- computeSummaryStatisticsTable(
			var = "AVAL",
			data = data,
			stats = "default"
		)
	)
	
	# specification equivalent to:
	stats <- getStatsData("summary-default", data = data, var = "AVAL")$AVAL
	summaryTableGetStats <- computeSummaryStatisticsTable(
		var = "AVAL",
		data = data,
		stats = stats
	)
	expect_identical(summaryTableString, summaryTableGetStats)
	
})

test_that("The statistic of interest is specified as vector of character", {
  
  set.seed(123)
  data <- data.frame(AVAL = rnorm(5), USUBJID = seq.int(5))
  
  expect_silent(
    summaryTableString <- computeSummaryStatisticsTable(
      var = "AVAL",
      data = data,
      stats = c("Mean", "SD")
    )
  )
  
  # specification equivalent to:
  stats <- getStatsData(type =  c("Mean", "SD"), data = data, var = "AVAL")$AVAL
  summaryTableGetStats <- computeSummaryStatisticsTable(
    var = "AVAL",
    data = data,
    stats = stats
  )
  expect_identical(summaryTableString, summaryTableGetStats)
  
})


test_that("An error is generated if the statistic of interest is not available in the default set", {
			
	set.seed(123)
	data <- data.frame(AVAL = rnorm(5), USUBJID = seq.int(5))	
			
	expect_error(
		computeSummaryStatisticsTable(
			var = "AVAL",
			data = data,
			stats = "myStat"
		),
		"should be one of .*summary.*, .*count.*"
	)
			
})

test_that("The statistic of interest specified as a string: geometric mean, if requested, is correctly added to the default set", {
  
  data <- data.frame(AVAL = c(1, 3, 4, 10, 100), USUBJID = seq.int(5))
  
  expect_silent(
    summaryTableString <- computeSummaryStatisticsTable(
      var = "AVAL",
      data = data,
      stats = "Geometric Mean"
    )
  )
  
  # is geometric mean added to the default set?
  expect_equal(
    object = subset(summaryTableString, !isTotal)$statGeomMean,
    expected = inTextSummaryTable::geomMean(data$AVAL)
  )
  
  # specification equivalent to:
  summaryTableGetStats <- computeSummaryStatisticsTable(
    var = "AVAL",
    data = data,
    stats = getStatsData("Geometric Mean", data = data, var = "AVAL")$AVAL
  )
  expect_identical(summaryTableString, summaryTableGetStats)
  
})

test_that("The statistic of interest specified as a string: geometric standard deviation, if requested, is correctly added to the default set", {
  
  data <- data.frame(AVAL = c(1, 3, 4, 10, 100), USUBJID = seq.int(5))
  
  expect_silent(
    summaryTableString <- computeSummaryStatisticsTable(
      var = "AVAL",
      data = data,
      stats = "Geometric SD"
    )
  )
  
  # is geometric SD added to the default set?
  expect_equal(
    object = subset(summaryTableString, !isTotal)$statGeomSD,
    expected = inTextSummaryTable::geomSD(data$AVAL)
  )
  
  # specification equivalent to:
  summaryTableGetStats <- computeSummaryStatisticsTable(
    var = "AVAL",
    data = data,
    stats = getStatsData("Geometric SD", data = data, var = "AVAL")$AVAL
  )
  expect_identical(summaryTableString, summaryTableGetStats)
  
})

test_that("The statistic of interest as a copy of a default statistic is correctly computed	", {
			
	set.seed(123)
	data <- data.frame(AVAL = rnorm(5),	USUBJID = seq.int(5))	
	expect_silent(
		summaryTable <- computeSummaryStatisticsTable(
			data = data, var = "AVAL", 
			stats = list(statMean = expression(statMean))
		)
	)
	expect_true("statMean" %in% colnames(summaryTable))
	expect_equal(
		subset(summaryTable, !isTotal)$statMean,
		mean(data$AVAL)
	)
			
})

test_that("An error is generated if the statistics has the same name as a default statistic", {
			
	data <- data.frame(AVAL = rnorm(5),	USUBJID = seq.int(5))	
	expect_error(
		computeSummaryStatisticsTable(
			data = data, var = "AVAL", 
			stats = list(statMean = expression(statMean+statSD))
		),
		"statistic name.*is a default name used"
	)
			
})

test_that("A unique set of statistics is correctly computed by variable", {
		
	set.seed(123)
	data <- data.frame(
		AVAL = rnorm(10), 
		CHG = c(NA_real_, rnorm(9)), 
		USUBJID = seq.int(10)
	)	
	
	expect_silent(
		summaryTable <- computeSummaryStatisticsTable(
			var = c("AVAL", "CHG"),
			data = data,
			stats = list(
				AVAL = expression(statMean), 
				CHG = expression(statMedian)
			)
		)
	)
	expect_identical(
		attr(summaryTable, "summaryTable")$statsVar,
		"Statistic"
	)

	expect_equal(subset(summaryTable, variable == "AVAL")$Statistic, mean(data$AVAL))
	expect_equal(subset(summaryTable, variable == "CHG")$Statistic, median(data$CHG, na.rm = TRUE))	
			
})

test_that("Multiple sets of statistics are correctly computed by variable", {
		
	set.seed(123)
	data <- data.frame(
		AVAL = rnorm(10), 
		CHG = c(NA_real_, rnorm(9)), 
		USUBJID = seq.int(10)
	)		
		
	# variable to summarize
	stats <- list(
		AVAL = list(n = expression(statN), `Mean` = expression(statMean)), 
		CHG = list(Median = expression(round(statMedian, 3)))
	)
	expect_silent(
		summaryTable <- computeSummaryStatisticsTable(
			var = c("AVAL", "CHG"),
			data = data,
			stats = stats
		)
	)
	expect_identical(
		attr(summaryTable, "summaryTable")$statsVar,
		c("n", "Median", "Mean"),
		label = "Order of statistics is as specified by variable"
	)
	sumTableAVAL <- subset(summaryTable, variable == "AVAL")[, c("n", "Median", "Mean")]
	expect_equal(unlist(sumTableAVAL), c(n = 10, Median = NA_real_, Mean = mean(data$AVAL)))
	
	sumTableCHG <- subset(summaryTable, variable == "CHG")[, c("n", "Median", "Mean")]
	expect_equal(unlist(sumTableCHG), c(n = NA_integer_, Median = round(median(data$CHG, na.rm = TRUE), 3), Mean = NA_real_))
	
})

test_that("An error is generated if the statistics are not specified for all variables to summarize", {
		
	set.seed(123)
	data <- data.frame(
		AVAL = rnorm(10), 
		CHG = c(NA_real_, rnorm(9)), 
		USUBJID = seq.int(10)
	)	
				
	# in this case, stats should be specified for all variables
	expect_error(
		computeSummaryStatisticsTable(
			var = c("AVAL", "CHG"),
			data = data,
			stats = list(AVAL = list(Mean = expression(statMean)))
		),
		"'stats'.*should be specified for all variables specified in 'var'"
	)
	
})

test_that("An error is generated if the statistics are specified for a variable not available", {
	
	set.seed(123)
	data <- data.frame(
		AVAL = rnorm(10), 
		CHG = c(NA_real_, rnorm(9)), 
		USUBJID = seq.int(10)
	)	
	
	expect_error(
		computeSummaryStatisticsTable(
			var = c("AVAL", "CHG"),
			data = data,
			stats = list(AVAL1 = list(Mean = expression(statMean)))
		),
		"Statistics specified in 'stats' should be.*list named with variable.*"
	)
	
})

test_that("Statistics are correctly computed by each element of a row variable", {
			
	set.seed(123)
	data <- data.frame(
		PARAM = rep(c("ALB", "ALT"), length.out = 10),
		AVAL = rnorm(10), 
		USUBJID = seq.int(10)
	)
	
	# variable to summarize
	stats <- list(
		ALB = list(Mean = expression(statMean)), 
		ALT = list(Median = expression(statMedian))
	)
	
	expect_silent(
		summaryTable <- computeSummaryStatisticsTable(
			var = "AVAL",
			data = data,
			stats = stats,
			rowVar = "PARAM", statsVarBy = "PARAM"
		)
	)
	
	sumTableALT <- subset(summaryTable, PARAM == "ALT")[, c("Median", "Mean")]
	dataALT <- subset(data, PARAM == "ALT")
	expect_equal(
		unlist(sumTableALT), 
		c(Median = median(dataALT$AVAL), Mean = NA_real_)
	)
	
	sumTableALB <- subset(summaryTable, PARAM == "ALB")[, c("Median", "Mean")]
	dataALB <- subset(data, PARAM == "ALB")
	expect_equal(
		unlist(sumTableALB), 
		c(Median = NA_real_, Mean = mean(dataALB$AVAL))
	)
	
})

	
test_that("An error is generated if the variable to compute statistics by is not specified as row or column variable", {
				
	set.seed(123)
	data <- data.frame(
		PARAM = rep(c("ALB", "ALT"), length.out = 10),
		AVAL = rnorm(10), 
		USUBJID = seq.int(10)
	)
	
	stats <- list(
		ALB = list(Mean = expression(statMean)), 
		ALT = list(Median = expression(statMedian))
	)
			
	expect_error(
		computeSummaryStatisticsTable(
			var = "AVAL",
			data = data,
			stats = stats,
			statsVarBy = "PARAM"
		),
		".*not available in.*row or column variables.*"
	)
	
})

test_that("An error is generated if the variable to compute statistics by is not specified but the statistics are specified by group", {
		
	set.seed(123)
	data <- data.frame(
		PARAM = rep(c("ALB", "ALT"), length.out = 10),
		AVAL = rnorm(10), 
		USUBJID = seq.int(10)
	)
			
	stats <- list(
		ALB = list(Mean = expression(statMean)), 
		ALT = list(Median = expression(statMedian))
	)
	
	expect_error(
		computeSummaryStatisticsTable(
			var = "AVAL",
			data = data,
			stats = stats
		),
		"Statistics.*should be.*list named with.*statsVarBy elements.*"
	)
	
})

test_that("Statistics are correctly computed by a specified row/column variable and by each of its element", {
	
	set.seed(123)
	data <- data.frame(
		PARAM = rep(c("ALB", "ALT"), length.out = 10),
		AVAL = rnorm(10), 
		CHG = rnorm(10),
		USUBJID = seq.int(10)
	)
	
	# variable to summarize
	stats <- list(
		AVAL = list(ALB = list(Mean = expression(statMean))), 
		CHG = list(ALT = list(Median = expression(statMedian)))
	)
	
	expect_silent(
		summaryTable <- computeSummaryStatisticsTable(
			var = c("AVAL", "CHG"),
			data = data,
			stats = stats,
			rowVar = "PARAM", statsVarBy = "PARAM"
		)
	)
	
	expect_equal(
		unlist(subset(summaryTable, PARAM == "ALB" & variable == "AVAL", c("Mean", "Median"))), 
		c(Mean = mean(subset(data, PARAM == "ALB")$AVAL), Median = NA_real_)
	)
	expect_identical(
		unlist(subset(summaryTable, PARAM == "ALT" & variable == "AVAL", c("Mean", "Median"))), 
		c(Mean = NA_real_, Median = NA_real_)
	)
	expect_identical(
		unlist(subset(summaryTable, PARAM == "ALB" & variable == "CHG", c("Mean", "Median"))), 
		c(Mean = NA_real_, Median = NA_real_)
	)
	expect_identical(
		unlist(subset(summaryTable, PARAM == "ALT" & variable == "CHG", c("Mean", "Median"))), 
		c(Mean = NA_real_, Median = median(subset(data, PARAM == "ALT")$CHG))
	)
				
})
			

test_that("Custom statistics are correctly computed", {
			
	set.seed(123)
	data <- data.frame(
		AVAL = rnorm(10), 
		USUBJID = seq.int(10)
	)
	
	CV <- function(x) sd(x, na.rm = TRUE)/mean(x, na.rm = TRUE)*100
	
	summaryTable <- computeSummaryStatisticsTable(
		var = "AVAL",
		data = data,
		stats = list(CV = bquote(statCV)),
		statsExtra = list(statCV = CV)
	)
	expect_equal(subset(summaryTable, !isTotal)$CV, CV(data$AVAL))
	
	# format of statsExtra already checked in unit tests
	# for the 'computeSummaryStatistics' function
			
})

test_that("A general label is correctly set for the statistics", {

	set.seed(123)
	data <- data.frame(USUBJID = seq.int(10))	
	
	statLab <- "My custom statistic"
	
	summaryTable <- computeSummaryStatisticsTable(data = data, statsGeneralLab = statLab)
	# general label is stored in table attribute
	# (and set when the table is formatted)
	expect_identical(
		attr(summaryTable, "summaryTable")$rowVarLab,
		c(Statistic = statLab)
	)
			
})

test_that("Percentages are correctly computed on the number of subject by default", {
			
	set.seed(123)
	data <- data.frame(
		USUBJID = c("1", "1", "2", "3", "4"),
		AEDECOD = rep("A", 5)
	)
	
	expect_silent(
		summaryTableStatN <- computeSummaryStatisticsTable(
			var = "AEDECOD",
			data = data,
			statsPerc = "statN"
		)
	)
	summaryTableStatNGroup <- subset(summaryTableStatN, variableGroup == "A")
	expect_equal(summaryTableStatNGroup$statPercTotalN, 4)
	
})


test_that("Percentages are correctly computed on the number of records when requested", {
				
	set.seed(123)
	data <- data.frame(
		USUBJID = c("1", "1", "2", "3", "4"),
		AEDECOD = rep("A", 5)
	)
				
	expect_silent(
		summaryTableStatm <- computeSummaryStatisticsTable(
			var = "AEDECOD",
			data = data,
			statsPerc = "statm"
		)
	)
	summaryTableStatmGroup <- subset(summaryTableStatm , variableGroup == "A")
	expect_equal(summaryTableStatmGroup$statPercTotalm, 5)
	
})

test_that("An error is generated when the statistic specified for percentages is not available", {
			
	set.seed(123)
	data <- data.frame(
		USUBJID = c("1", "1", "2", "3", "4"),
		AEDECOD = rep("A", 5)
	)

	expect_error(
		computeSummaryStatisticsTable(
			var = "AEDECOD",
			data = data,
			statsPerc = "statTest"
		)
	)
			
})

context("Test utility statistical functionalities")

test_that("The standard error is correctly computed", {
      
	x <- c(1, 2, 3)	
	expect_equal(se(x = x), 1/sqrt(3))
      
})

test_that("The standard error of data with missing values is correctly computed", {
		
	x <- c(1, 2, 3, NA)	
	expect_equal(
		se(x = x, na.rm = FALSE), 
		NA_real_
	)
	
	expect_equal(
		se(x = x, na.rm = TRUE), 
		1/sqrt(3)
	)
			
})

test_that("The coefficient of variation is correctly computed", {
			
	x <- c(1, 2, 3)	
	expect_equal(cv(x = x), 50)
			
})

test_that("The coefficient of variation of data with missing values is correctly computed", {
			
	x <- c(1, 2, 3, NA)	
	expect_equal(
		cv(x = x, na.rm = FALSE), 
		NA_real_
	)
			
	expect_equal(
		cv(x = x, na.rm = TRUE), 
		50
	)
			
})


test_that("The geometric mean is correctly computed", {
      
	x <- exp(c(1, 2, 3))
	expect_equal(geomMean(x = x), exp(2))
      
})

test_that("The geometric mean of data with missing values is correctly computed", {
	
	x <- exp(c(1, 2, 3, NA))
	expect_equal(
		geomMean(x = x, na.rm = FALSE), 
		NA_real_
	)
			
	expect_equal(
		geomMean(x = x, na.rm = TRUE), 
		exp(2)
	)
			
})

test_that("The geometric mean of data with with negative values is correctly computed", {
			
	x <- c(-3, 2)
	expect_warning(gM <- geomMean(x = x))
	expect_true(is.nan(gM))
			
})


test_that("The geometric standard deviation is correctly computed", {
			
	x <- exp(c(1, 2, 3))
	expect_equal(geomSD(x = x), exp(1))
			
})

test_that("The geometric standard deviation of data with missing values is correctly computed", {
			
	x <- exp(c(1, 2, 3, NA))
	expect_equal(
		geomSD(x = x, na.rm = FALSE), 
		NA_real_
	)
			
	expect_equal(
		geomSD(x = x, na.rm = TRUE), 
		exp(1)
	)
			
})

test_that("The geometric standard deviation of data with negative values is correctly computed", {
			
	x <- c(-3, -2)
	expect_warning(gSD <- geomSD(x = x))
	expect_equal(gSD, NA_real_)
			
})

test_that("The geometric coefficient of variation is correctly computed", {
			
	x <- exp(c(1, 2, 3))
	expect_equal(geomCV(x = x), sqrt(exp(sd(log(x))^2)-1)*100)
	
})

test_that("The geometric coefficient of variation of data with missing values is correctly computed", {
			
	x <- c(1, 2, 3, NA)	
	expect_equal(
		geomCV(x = x, na.rm = FALSE), 
		NA_real_
	)
			
	expect_equal(
		geomCV(x = x, na.rm = TRUE), 
		geomCV(x = c(1, 2, 3))
	)
			
})

test_that("The geometric coefficient of variation of data with negative values is correctly computed", {
			
	x <- c(-3, -2)
	expect_warning(gCV <- geomCV(x = x))
	expect_equal(gCV, NA_real_)
			
})

test_that("The geometric standard error of the mean is correctly computed", {
			
	x <- exp(c(1, 2, 3))
	# geomSE is exp of SE of log (x)
	expect_equal(geomSE(x = x), exp(1/sqrt(3)))
			
})


test_that("The geometric standard error of the mean of data with missing values is correctly computed", {
			
	x <- exp(c(1, 2, 3, NA_real_))
	expect_equal(
		geomSE(x = x, na.rm = FALSE), 
		NA_real_
	)
			
	expect_equal(
		geomSE(x = x, na.rm = TRUE), 
		exp(1/sqrt(3))
	)
			
})

#' Statistics in the in-text table package.
#' 
#' In the in-text package, different set of statistics are available.
#' 
#' The statistics are first computed as numeric
#' ('Base statistics' section below),
#' then formatted to be displayed in the table
#' ('Formatted statistics' section below).
#' @template stats-name-base
#' @template stats-name-formatted
#' @template stats-formatting
#' @name inTextSummaryTable-stats
#' @return No return value, used for the documentation of 
#' the \code{stats} parameter
NULL

#' Export an object
#' @param ... Extra parameters for the corresponding
#' method.
#' @seealso \code{\link{export.summaryTable}}
#' to export \code{\link{summaryTable}} objects.
#' @export
export <- function(...) UseMethod("export")

#' @method export summaryTable
#' @describeIn exportSummaryStatisticsTable export \code{\link{summaryTable}} object
#' @export
export.summaryTable <- function(...){
	exportSummaryStatisticsTable(...)
}

#' Export a summary table to \code{docx}, \code{pptx} 
#' or \code{html} format (interactive table)
#' 
#' The use of \code{export} is recommended.\cr
#' \code{exportSummaryStatisticsTable} is retained for
#' back-compatibility.
#' @param outputType String with output type:
#' \itemize{
#' \item{'flextable' (by default): \code{\link[flextable]{flextable}} object, with format for
#' CSR, compatible with Word/PowerPoint export}
#' \item{'DT': \code{\link[DT]{datatable}} interactive table,
#' compatible with html export}
#' \item{'data.frame': data.frame in wide format (with elements in 
#' \code{colVar} in different columns)}
#' \item{'data.frame-base': data.frame in long format (with elements in 
#' \code{colVar} in different rows), useful for QC}
#' }
#' @param file (Optional) Name of the file the table should be exported to, 
#' either:
#' \itemize{
#' \item{string (of length 1). In this case, depending on the
#' file extension, the following is exported: 
#' \itemize{
#' \item{'txt': summary table in long format ('data.frame-base' \code{outputType})}
#' \item{'docx': summary table in final format is exported ('flextable' \code{outputType})}
#' \item{'html': interactive summary table is exported ('DT' \code{outputType})}
#' }}
#' \item{named character vector in case of multiple exports.
#' The names should correspond to the options in \code{outputType}:
#' \itemize{
#' \item{for 'data.frame-base' and 'data.frame': filename with 'txt' extension}
#' \item{for 'flextable': filename with 'docx' extension}
#' \item{for 'DT': filename with 'html' extension}
#' }}
#' }
#' If NULL (by default), the summary table is not exported but only returned as output.
#' If \code{byVar} is specified, each table is exported to a separated
#' file with the suffix: 'file_[i].[ext]' with i the index of the file
#' (and [ext] the file extension).
#' @inheritParams inTextSummaryTable-common-args
#' @inheritParams formatSummaryStatisticsTable
#' @inheritParams exportSummaryStatisticsTableToFlextable
#' @inheritParams exportSummaryStatisticsTableToDT
#' @return Depending on the \code{outputType}:
#' \itemize{
#' \item{'data.frame-base': input summary table in a long format with
#' all computed statistics}
#' \item{'data.frame': summary table in a wide format (
#' different columns for each \code{colVar}), with specified labels}
#' \item{'flextable' (by default): \code{\link[flextable]{flextable}}
#'  object with summary table}
#' \item{'DT': \code{\link[DT]{datatable}} object with summary table}
#' }
#' If multiple \code{outputType} are specified, a list of those objects, named
#' by \code{outputType}.\cr
#' If \code{byVar} is specified, each object consists of a list of tables,
#' one for each element in \code{byVar}.
#' @inherit convertSummaryStatisticsTableToFlextable return
#' @author Laure Cougnaud
#' @importFrom clinUtils getLabelVar
#' @importFrom tools file_ext
#' @export
exportSummaryStatisticsTable <- function(
	summaryTable, 
	# row
	rowVar = getAttribute(summaryTable, "rowVar"), 
	rowVarLab = getAttribute(summaryTable, "rowVarLab", default = getLabelVar(rowVar, labelVars = labelVars)),
	rowVarInSepCol = NULL, 
	rowVarFormat = NULL,
	rowVarTotalInclude = getAttribute(summaryTable, "rowVarTotalInclude"),
	rowTotalLab = NULL,
	rowVarTotalInSepRow = getAttribute(summaryTable, "rowVarTotalInSepRow"),
	rowAutoMerge = TRUE,
	# column
	colVar = getAttribute(summaryTable, "colVar"), 
	colTotalLab = getAttribute(summaryTable, "colTotalLab", default = "Total"),
	colHeaderTotalInclude = TRUE,
	colHeaderMerge = TRUE,
	# stats
	statsVar = getAttribute(summaryTable, "statsVar"),
	statsLayout = getAttribute(
		summaryTable, "statsLayout", 
		default = ifelse("DT" %in% outputType, "col", "row")
	),
	statsValueLab = "StatisticValue",
	statsLabInclude = NULL,
	emptyValue = "-",
	# extra
	labelVars = NULL, 
	file = NULL, 
	title = NULL,
	outputType = "flextable",
	pageDim = NULL,
	columnsWidth = NULL,
	# flextable-specific
	landscape = (style == "presentation"), 
	margin = 1, rowPadBase = 14.4,
	footer = NULL,
	style = "report",
    colorTable = getColorPaletteTable(style = style),
	fontsize = switch(style, 'report' = 8, 'presentation' = 10),
	fontname = switch(style, 'report' = "Times", 'presentation' = "Tahoma"),
	vline = "none", hline = "auto", 
	# DT-specific
	expandVar = NULL, noEscapeVar = NULL, barVar = NULL,
	...){

	outputType  <- match.arg(outputType, 
		choices = c("flextable", "DT", "data.frame", "data.frame-base"),
		several.ok = TRUE
	)
	
	# check if table should be exported for a specified output type
	getFileForOutputType <- function(outputType, fileExt = NULL){
		fileOutputType <- if(is.null(names(file)) & !is.null(fileExt)){
			idxFile <- which(file_ext(file) == fileExt)
			if(length(idxFile) > 0){
				file[idxFile[1]]
			}
		}else	if(!is.null(names(file)) & outputType %in% names(file)){
			file[outputType]
		}
		return(fileOutputType)
	}
	
	# export input summary table (if required)
	summaryTableFile <- getFileForOutputType("data.frame-base", fileExt = "txt")
	if(!is.null(summaryTableFile))
		writeTable(summaryTable, file = summaryTableFile)
	
	# convert to long format
	summaryTableLong <- formatSummaryStatisticsTable(
		summaryTable,
		# row
		rowVar = rowVar,
		# column
		colVar = colVar,
		colTotalLab = colTotalLab,
		colHeaderTotalInclude = colHeaderTotalInclude,
		# stats
		statsVar = statsVar,
		statsLabInclude = statsLabInclude,
		statsLayout = statsLayout,
		statsValueLab = statsValueLab,
		emptyValue = emptyValue
	)
	# export summary table in long format (if required)
	summaryTableLongFile <- getFileForOutputType("data.frame")
	if(!is.null(summaryTableLongFile)){
		writeTable(summaryTableLong, file = summaryTableLongFile)
	}
	
	summaryTableFtFile <- getFileForOutputType("flextable", fileExt = "docx")
	createFt <- "flextable" %in% outputType | !is.null(summaryTableFtFile)
	if(createFt){
		
		# create flextable only with header to extract dimensions header
		argsExport <- list(
			# for 'format' function
			summaryTable = summaryTableLong,
			rowVar = rowVar,
			rowVarInSepCol = rowVarInSepCol,
			rowVarLab = rowVarLab,
			rowVarTotalInSepRow = rowVarTotalInSepRow,
			rowVarTotalInclude = rowVarTotalInclude,
			statsVar = statsVar,
			vline = vline, hline = hline,
			rowAutoMerge = rowAutoMerge,
			rowVarFormat = rowVarFormat,
			rowTotalLab = rowTotalLab,
			# for 'convert' function
			colHeaderMerge = colHeaderMerge,
			landscape = landscape, margin = margin, rowPadBase = rowPadBase,
			title = title, footer = footer,
			style = style, fontsize = fontsize,
			file = summaryTableFtFile,
			fontname = fontname,
			colorTable = colorTable,
			pageDim = pageDim, columnsWidth = columnsWidth,
			labelVars = labelVars
		)
			
		summaryTableFt <- do.call(exportSummaryStatisticsTableToFlextable, argsExport)
		
	}
	
	summaryTableDTFile <- getFileForOutputType("DT", fileExt = "html")
	createDT <- "DT" %in% outputType | !is.null(summaryTableDTFile)
	if(createDT){
		
		summaryTableDT <- exportSummaryStatisticsTableToDT(
			summaryTable = summaryTableLong,
			rowVar = rowVar,
			rowVarLab = rowVarLab,
			rowVarInSepCol = rowVarInSepCol,
			statsVar = statsVar, 
			statsValueLab = statsValueLab,
			expandVar = expandVar,
			noEscapeVar = noEscapeVar,
			barVar = barVar,
			pageDim = pageDim,
			title = title,
			file = summaryTableDTFile,
			columnsWidth = columnsWidth,
			labelVars = labelVars,
			...
		)
		
	}
	
	result <- sapply(outputType, function(type)
		switch(type, 
			'data.frame-base' = summaryTable,
			'data.frame' = summaryTableLong,	
			'flextable' = summaryTableFt,
			'DT' = summaryTableDT
		)
	, simplify = FALSE)

	if(length(outputType) == 1)
		result <- result[[1]]
		
	return(result)
	
}

context("Export summary statistics table")

library(tools)
library(officer)
library(rmarkdown)

test_that("A summary table is correctly exported to multiple formats", {
			
	skip_if_not(
		condition = rmarkdown::pandoc_available(), 
		message = "pandoc is not available"
	)
			
	summaryTable <- data.frame(
		PARAM = c("A", "B"),
		n = c(9, 10)
	)	
	
	files <- c(
		`data.frame-base` = "table-base.txt",
		`data.frame` = "table.txt",
		`flextable` = "table.docx",
		`DT` = "table.html"
	)
	ft <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		rowVar = "PARAM", statsVar = "n",
		file = files
	)
	
	importTableFromFile <- function(file){
		switch(tools::file_ext(file),
			txt = readLines(file),
			docx = officer::docx_summary(officer::read_docx(file)),
			html = {
				x <- readLines(file)
				x <- gsub("htmlwidget-\\w+", "\\1", x)
				# fix for pandoc >= 2.19
				x <- paste(x, collapse = "")
				x <- gsub("\\s", "", x)
				x
			}
		)
	}
	
	# check that output is similar as output for single exportSummaryStatisticsTable
	for(outputType in names(files)){
		
		expect_equal(
			object = {
				importTableFromFile(file = files[[!!outputType]])
			}, 	
			expected = {
				fileTest <- sapply(files[!!outputType], function(x)
					sub("(.+)\\.(.+)", "\\1-test.\\2", x)
				)
				res <- exportSummaryStatisticsTable(
					summaryTable = summaryTable,
					rowVar = "PARAM", statsVar = "n",
					file = fileTest
				)
				importTableFromFile(fileTest)
			},
			check.attributes = FALSE,
			label = outputType
		)
		
	}
	
})
			

context("Get a summary statistics table")

test_that("Getting a summary table is identical to computing and exporting a summary table", {
		
	data <- data.frame(
		PARAM = c("a", "b", "a", "b"),
		TRT = c("A", "A", "B", "B"),
		AVAL = c(1, 2, 4, 5),
		USUBJID = seq.int(4)
	)		
	
	summaryTableOutput <- getSummaryStatisticsTable(
		data = data,	
		rowVar = "PARAM", colVar = "TRT",
		var = "AVAL"
	)
	
	summaryTable <- computeSummaryStatisticsTable(
		data = data,	
		rowVar = "PARAM", 
		colVar = "TRT",
		var = "AVAL"
	)
	summaryTableOutputSteps <- export(summaryTable)
			
	expect_identical(summaryTableOutput, summaryTableOutputSteps)
			
})

test_that("No output and a message is generated if the data is empty", {

	expect_message(
		res <- getSummaryStatisticsTable(data = data.frame())
	)
	expect_null(res)
	
})

test_that("A variable with subgroups and a flag variable are correctly combined when multiple statistics are set", {
			
	data <- data.frame(
		TRTEMFL = c("Y", "Y", "N", "N"),
		WORST = factor(
			c("Mild", "Moderate", "Mild", "Mild"),
			levels = c("Mild", "Moderate"),
		),
		TRT = c("A", "A", "A", "A"),
		USUBJID = c("1", "2", "3", "4"),
		stringsAsFactors = FALSE
	)
	ft <- getSummaryStatisticsTable(
		data = data,
		var = c("TRTEMFL", "WORST"),
		varFlag = "TRTEMFL",
		colVar = "TRT",
		stats = getStats(c("n", "%"))
	)
			
	dataRef <- 	data.frame(
		col1 = c(
			"TRTEMFL", "n", "%",
			"WORST", 
			"Mild", "n", "%",
			"Moderate", "n", "%"
		),
		col2 = c(
			NA_character_, "2", "50.0", 
			NA_character_, 
			NA_character_, "3", "75.0", 
			NA_character_, "1", "25.0"
		),
		stringsAsFactors = FALSE
	)
			
	expect_equal(
		object = ft$body$data, 
		expected = dataRef, 
		# rownames might differ
		check.attributes = FALSE
	)
			
})

test_that("The width of the columns is correctly set", {
			
	data <- data.frame(
		TRT = c("A", "B"),
		USUBJID = c("1", "2"),
		stringsAsFactors = FALSE
	)
	columnsWidth <- c(0.5, 2, 8)
	ft <- getSummaryStatisticsTable(data = data, colVar = "TRT", columnsWidth = columnsWidth)
			
	expect_equal(object = ft$body$colwidths, expected = columnsWidth)
			
})

test_that("The width of the columns specified as percentage is correctly set", {
  
  data <- data.frame(
    TRT = c("A", "B"),
    USUBJID = c("1", "2"),
    stringsAsFactors = FALSE
  )
  columnsWidth <- c(0.8, 0.1, 0.1)
  ft <- getSummaryStatisticsTable(data = data, colVar = "TRT", columnsWidth = columnsWidth)
  
  # widths are converted to inches for flextable
  expect_equal(
    object = ft$body$colwidths, 
    expected = columnsWidth * getDimPage(type = "width")
  )
  
})

test_that("The width of the columns specified for a subset of the columns is correctly set", {
  
  data <- data.frame(
    TRT = c("A", "B"),
    USUBJID = c("1", "2"),
    stringsAsFactors = FALSE
  )
  columnsWidth <- 2
  ft <- getSummaryStatisticsTable(data = data, colVar = "TRT", columnsWidth = columnsWidth)
  
  # width is set as specified for the first column
  expect_equal(object = ft$body$colwidths[1], expected = columnsWidth)
  # total width = page width
  expect_equal(object = sum(ft$body$colwidths), expected = getDimPage(type = "width"))
  
})

test_that("A warning is generated if the width of the columns is not correctly set", {
			
	data <- data.frame(
		TRT = c("A", "B"),
		USUBJID = c("1", "2"),
		stringsAsFactors = FALSE
	)
	columnsWidth <- c(2, 8, 4, 5)
	expect_warning(
		object = getSummaryStatisticsTable(
			data = data, 
			colVar = "TRT", 
			columnsWidth = columnsWidth
		)
	)

})
			

context("Export summary statistics table as data frame")

test_that("The summary table is correctly exported to a text file in base format by default", {
			
	summaryTable <- data.frame(PARAM = c("A", "B"), n = c(9, 10))	
	
	# by default, if a text file is specified,
	# format is set to 'data.frame-base'
	fileTable <- tempfile(pattern = "table", fileext = ".txt")
	ft <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		rowVar = "PARAM", statsVar = "n",
		file = fileTable
	)
	
	expect_true(file.exists(fileTable))
	fileCnt <- read.table(fileTable, header = TRUE, sep = "\t")
	expect_equal(fileCnt, summaryTable)
	
})

test_that("The summary table is correctly exported to a text file in base format as specified", {
			
	summaryTable <- data.frame(PARAM = c("A", "B"), n = c(9, 10))			
			
	fileTable <- tempfile(pattern = "table", fileext = ".txt")
	ft <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		rowVar = "PARAM", statsVar = "n",
		file = c(`data.frame-base` = fileTable)
	)
	expect_true(file.exists(fileTable))
	fileCnt <- read.table(fileTable, header = TRUE, sep = "\t")
	expect_equal(fileCnt, summaryTable)
	
})

test_that("A list of summary tables is successfully exported to text files in base format", {
  
  summaryTables <- list(
    A = data.frame(PARAM = "A", n = 10),
    B = data.frame(PARAM = "B", n = 9)
  )
  
  file <- tempfile(pattern = "table", fileext = ".txt")
  ft <- exportSummaryStatisticsTable(
    summaryTable = summaryTables,
    rowVar = "PARAM", statsVar = "n",
    file = c(`data.frame-base` = file)
  )
  for(i in seq_along(summaryTables)){
    expect_equal(
      object = read.table(
        file = paste0(
          tools::file_path_sans_ext(file), 
          "_", !!i, ".txt"
        ),
        header = TRUE, sep = "\t"
      ),
      expected = summaryTables[[!!i]]
    )
  }
  
})

test_that("The summary table is correctly exported to a text file in in-text format as specified", {
			
	summaryTable <- data.frame(PARAM = c("A", "B"),n = c(9, 10))	
	
	fileTable <- tempfile(pattern = "table", fileext = ".txt")
	ft <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		rowVar = "PARAM", statsVar = "n",
		file = c('data.frame' = fileTable)
	)
	
	expect_true(file.exists(fileTable))
	fileCnt <- readLines(fileTable)
	expect_equal(
		fileCnt, 
		c("PARAM\tn", "(N=NA)", "A\t9", "B\t10")
	)
	
})

test_that("A list of summary tables is successfully exported to text files in in-text format", {
  
  summaryTables <- list(
    A = data.frame(PARAM = "A", n = 10),
    B = data.frame(PARAM = "B", n = 9)
  )
  
  file <- tempfile(pattern = "table", fileext = ".txt")
  ft <- exportSummaryStatisticsTable(
    summaryTable = summaryTables,
    rowVar = "PARAM", statsVar = "n",
    file = c('data.frame' = file)
  )
  
  for(i in seq_along(summaryTables)){
    expect_equal(
      object = readLines(
        con = paste0(
          tools::file_path_sans_ext(file), 
          "_", !!i, ".txt"
        )
      ),
      expected = c(
        "PARAM\tn", 
        "(N=NA)", 
        paste(summaryTables[[!!i]], collapse = "\t")
      )
    )
  }
  
})

context("Export summary statistics table to a DataTables")

library(htmltools)
library(rmarkdown)
library(tools)

test_that("A summary table with a row variable is correctly exported to DataTables", {
			
	summaryTable <- data.frame(
		PARAM = factor(c("A", "B"), levels = c("B", "A")),
		n = c(9, 10)
	)	
	dt <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		rowVar = "PARAM",
		outputType = "DT"
	)
	expect_s3_class(dt, "datatables")
	expect_identical(
		as.character(dt$x$data[, 1]),
		c("B", "A")
	)
			
})

test_that("A warning is generated if a summary table is exported to DataTables with multiple nested row variables", {
			
	summaryTable <- data.frame(
		TRT = rep(c("A", "A", "B", "B"), times = 2),
		PARAM = rep(c("a", "b", "a", "b"), times = 2),
		metric = rep(c("Actual Value", "Change from Baseline"), each = 4),
		n = seq_len(8)
	)
	
	expect_warning(
		exportSummaryStatisticsTable(
			summaryTable = summaryTable,
			rowVar = c("TRT", "PARAM", "metric"),
			outputType = "DT"
		),
		"multi-level row grouping row variable is not available"
	)
	
})


test_that("The specified labels of the row variables are correctly set in a DataTables summary table", {
			
	summaryTable <- data.frame(
		PARAM = factor(c("A", "B"), levels = c("B", "A")),
		n = c(9, 10)
	)
	dt <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		rowVar = "PARAM",
		rowVarLab = c(PARAM = "Parameter"),
		outputType = "DT"
	)
	expect_identical(
		object = attr(dt$x, "colnames")[1], 
		expected = "Parameter"
	)
			
})

test_that("The labels of the row variables, extracted from the labels of all variables, are correctly set in a DataTables summary table", {
			
	summaryTable <- data.frame(
		PARAM = factor(c("A", "B"), levels = c("B", "A")),
		n = c(9, 10)
	)
	dt <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		rowVar = "PARAM",
		labelVars = c(PARAM = "Parameter"),
		outputType = "DT"
	)
	expect_identical(
		object = attr(dt$x, "colnames")[1], 
		expected = "Parameter"
	)
	
})


test_that("A summary table with a row variable in a separated column is correctly exported to DataTables", {
			
	summaryTable <- data.frame(
		TRT = c("A", "A", "B", "B"),
		PARAM = c("a", "b", "a", "b"),
		n = 1:4
	)
	dt <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		rowVar = c("TRT", "PARAM"),
		rowVarInSepCol = "PARAM",
		outputType = "DT"
	)
	expect_identical(
		object = attr(dt$x, "colnames")[1:2], 
		expected = c("TRT", "PARAM")
	)
	
})

test_that("A summary table with a column variable is correctly exported to DataTables", {
			
	summaryTable <- data.frame(
		TRT = factor(c("A", "B"), levels = c("B", "A")),
		n = c(9, 10)
	)
	dt <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		colVar = "TRT",
		outputType = "DT"
	)
	expect_s3_class(dt, "datatables")
	expect_identical(
		colnames(dt$x$data),
		c("B", "A")
	)
	
})

test_that("A summary table with totals in the header is correctly exported to DataTables", {
	
	summaryTable <- data.frame(
		TRT = factor(c("A", "A", "B", "B"), levels = c("B", "A")),
		statN = c(1, 4, 2, 5),
		isTotal = c(FALSE, TRUE, FALSE, TRUE)
	)
	
	dt <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		colVar = "TRT",
		outputType = "DT",
		colHeaderTotalInclude = TRUE
	)
	expect_identical(
		object = colnames(dt$x$data),
		expected = c("B\n(N=5)", "A\n(N=4)")
	)
	
})
	
test_that("A summary table without totals in the header is correctly exported to DataTables", {
		
	summaryTable <- data.frame(
		TRT = factor(c("A", "A", "B", "B"), levels = c("B", "A")),
		statN = c(1, 4, 2, 5),
		isTotal = c(FALSE, TRUE, FALSE, TRUE)
	)
	
	dt <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		colVar = "TRT",
		outputType = "DT",
		colHeaderTotalInclude = FALSE
	)
	expect_identical(
		object = colnames(dt$x$data),
		expected = c("B", "A")
	)

})

test_that("A summary table without column total is correctly exported to DataTables without totals in header", {
			
	# check that no records are filtered
	summaryTable <- data.frame(
		TRT = factor(c("A", "B"), levels = c("B", "A")),
		statN = c(1, 4),
		isTotal = c(FALSE, FALSE)
	)	

	dt <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		colVar = "TRT",
		outputType = "DT",
		colHeaderTotalInclude = FALSE
	)
	expect_equal(
		object = dt$x$data, 
		expected = data.frame(B = 4, A = 1), 
		check.attributes = FALSE
	)
			
})

test_that("A summary table with statistics in a separated column is correctly exported to DataTables", {
			
	summaryTable <- data.frame(
		variable = c("A", "B"),
		n = c("1", "2"),
		Mean = c("0.34", "0.56"),
		stringsAsFactors = FALSE
	)
	
	dtStatRowInSepCol <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		rowVar = "variable",
		statsVar = c("n", "Mean"),
		statsLayout = "rowInSepCol",
		outputType = "DT"
	)

	dataRefStatRowInSepCol <- data.frame(
		variable = c("A", "A", "B", "B"),
		Statistic = c("n", "Mean", "n", "Mean"),
		`StatisticValue\n(N=NA)` = c("1", "0.34", "2", "0.56"),
		check.names = FALSE, stringsAsFactors = FALSE
	)
	# class of single column differ
	dataDTStatRowInSepCol <- as.data.frame(sapply(dtStatRowInSepCol$x$data, as.character), stringsAsFactors = FALSE)
	expect_identical(dataDTStatRowInSepCol, dataRefStatRowInSepCol)
	
})

test_that("A summary table with statistics in rows is correctly exported to DataTables", {
	
	summaryTable <- data.frame(
		variable = c("A", "B"),
		n = c("1", "2"),
		Mean = c("0.34", "0.56"),
		stringsAsFactors = FALSE
	)
			
	dtStatInRow <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		rowVar = "variable",
		statsVar = c("n", "Mean"),
		statsLayout = "row",
		outputType = "DT"
	)

	# class of single column differ
	dataDTStatInRow <- as.data.frame(sapply(dtStatInRow$x$data, as.character), stringsAsFactors = FALSE)
	# if columns are nested, the internal dataset represents them as separated column:
	dataRefStatRowInSepCol <- data.frame(
		variable = c("A", "A", "B", "B"),
		Statistic = c("n", "Mean", "n", "Mean"),
		`StatisticValue\n(N=NA)` = c("1", "0.34", "2", "0.56"),
		check.names = FALSE, stringsAsFactors = FALSE
	)
	expect_identical(dataDTStatInRow, dataRefStatRowInSepCol)
	# but these are specified via the 'rowGroup' option
	# (in JS column index which starts at 0)
	expect_identical(dtStatInRow$x$options$rowGroup$dataSrc, 0)
	
})

test_that("A summary table with statistics in columns is correctly exported to DataTables", {
			
	summaryTable <- data.frame(
		variable = c("A", "B"),
		n = c("1", "2"),
		Mean = c("0.34", "0.56"),
		stringsAsFactors = FALSE
	)

	dtStatInCol <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		rowVar = "variable",
		statsVar = c("n", "Mean"),
		statsLayout = "col",
		outputType = "DT"
	)		
	dataRefStatInCol <-data.frame(
		variable = c("A", "B"),
		n = c("1", "2"),
		Mean = c("0.34", "0.56"),
		stringsAsFactors = FALSE
	)
	# class of single column differ
	dataDTStatInCol <- as.data.frame(sapply(dtStatInCol$x$data, as.character), stringsAsFactors = FALSE)
	expect_identical(object = dataDTStatInCol, expected = dataRefStatInCol)
	
})

test_that("The label for the statistic value is correctly set in a DataTables summary table", {
			
	summaryTable <- data.frame(
		variable = c("A", "B"),
		Statistic = c("1", "2"),
		stringsAsFactors = FALSE
	)
	dt <- exportSummaryStatisticsTable(
		summaryTable, rowVar = "variable", 
		statsVar = "Statistic", statsValueLab = "Number of subjects",
		outputType = "DT"
	)
	expect_match(
		object = colnames(dt$x$data)[2],
		regexp = "Number of subjects.*" 
	)
	
})

test_that("An error is generated if the label for the statistic value is set to the default name in a DataTables summary table", {
	
	summaryTable <- data.frame(
		variable = c("A", "B"),
		Statistic = c("1", "2"),
		stringsAsFactors = FALSE
	)		
	
	# error is label is set to 'Statistic' (used as default naming)
	expect_error(
		exportSummaryStatisticsTable(
			summaryTable, rowVar = "variable", 
			statsVar = "Statistic", statsValueLab = "Statistic",
			outputType = "DT"
		),
		"'statsValueLab' should be different than 'Statistic'."
	)
			
})

test_that("A summary table with one statistic is correctly exported to DataTables with the statistic name", {
			
	summaryTable <- data.frame(
		TRT = c("A", "A", "B", "B"),
		variable = c("a", "b", "a", "b"),
		n = c("1", "2", "3", "4"),
		stringsAsFactors = FALSE
	)
	
	dt <- exportSummaryStatisticsTable(
		summaryTable, 
		rowVar = "variable", colVar = "TRT",
		statsVar = "n", 
		statsLabInclude = TRUE,
		outputType = "DT"
	)
	expect_identical(
		object = attr(dt$x, "colnames")[2:3],
		expected = c("A_n", "B_n")
	)
	
})

test_that("A summary table with one statistic is correctly exported to DataTables without the statistic name", {
			
	summaryTable <- data.frame(
		TRT = c("A", "A", "B", "B"),
		variable = c("a", "b", "a", "b"),
		n = c("1", "2", "3", "4"),
		stringsAsFactors = FALSE
	)
	
	dt <- exportSummaryStatisticsTable(
		summaryTable, 
		rowVar = "variable", colVar = "TRT",
		statsVar = "n", 
		statsLabInclude = FALSE,
		outputType = "DT"
	)
	expect_identical(
		object = attr(dt$x, "colnames")[2:3],
		expected = c("A", "B")
	)
	
})

test_that("A warning is generated if a DataTables summary table contain multiple statistics but the names are specified to be not included", {
			
	summaryTable <- data.frame(
		TRT = c("A", "A", "B", "B"),
		variable = c("a", "b", "a", "b"),
		n = c("1", "2", "3", "4"),
		Mean = c("0.1", "1.3", "4.5", "6.7"),
		stringsAsFactors = FALSE
	)
	expect_warning(
		exportSummaryStatisticsTable(
			summaryTable, 
			rowVar = "variable", colVar = "TRT",
			statsVar = c("n", "Mean"),
			statsLabInclude = FALSE,
			outputType = "DT"
		),
		"Statistic label is included.*because more than one statistic variable.*"
	)
	
})

test_that("A placeholder for empty value is correctly included by default in a DataTables summary table", {
			
	summaryTable <- data.frame(
		TRT = c("A", "A", "B", "B"),
		PARAM = c("a", "b", "a", "b"),
		n = c("1", "2", "3", NA_character_),
		stringsAsFactors = FALSE
	)
	
	# by default, 'empty' value are set to '-'
	# Note that 'numeric-like' column to numeric 
	dt <- exportSummaryStatisticsTable(
		summaryTable,
		rowVar = "PARAM", colVar = "TRT",
		statsVar = "n",
		outputType = "DT"
	)
	expect_identical(
		object = dt$x$data,
		expected = data.frame(
			PARAM = c("a", "b"),
			A = c(1, 2),
			B = c("3", "-"),
			stringsAsFactors = FALSE
		)
	)
	
})

test_that("A specified placeholder for empty value is correctly included in a DataTables summary table", {
			
	summaryTable <- data.frame(
		TRT = c("A", "A", "B", "B"),
		PARAM = c("a", "b", "a", "b"),
		n = c("1", "2", "3", NA_character_),
		stringsAsFactors = FALSE
	)
		
	dt <- exportSummaryStatisticsTable(
		summaryTable,
		rowVar = "PARAM", colVar = "TRT",
		statsVar = "n",
		emptyValue = "0",
		outputType = "DT"
	)
	expect_identical(
		object = dt$x$data,
		expected = data.frame(
			PARAM = c("a", "b"),
			A = c(1, 2),
			B = c(3, 0),
			stringsAsFactors = FALSE
		)
	)
			
})

test_that("Multiple titles are correctly included in a DataTables summary table", {
		
	summaryTable <- data.frame(n = 10)
	
	titles <- c("Title A", "Title B")
	dt <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		title = titles,
		outputType = "DT"
	)
	expect_match(
		object = dt$x$caption, 
		regexp = paste(titles, collapse = " ")
	)
			
})

test_that("Multiple titles are correctly included in HTML format in a DataTables summary table", {
			
	summaryTable <- data.frame(n = 10)
			
	# multiple titles
	titles <- htmltools::tags$caption(
		htmltools::a("Formatted .docx table", target="_blank", href = "./myFile.docx"),
		htmltools::br(),
		"This is a test caption."
	)
	dt <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		title = titles,
		outputType = "DT"
	)
	expect_match(
		object = dt$x$caption, 
		regexp = "<caption>.*href=.*\\./myFile\\.docx.*This is a test caption.*</caption>"
	)
			
})

test_that("A warning is generated if no data remain after filtering of the column totals in a DataTables summary table", {
			
	data <- data.frame(
		isTotal = rep(TRUE, 2),
		n = c(1, 2)
	)
	expect_warning(
		exportSummaryStatisticsTable(data, outputType = "DT"),
		regexp = "No data remain after filtering of total rows."
	)
			
})

test_that("An error is generated if a DataTables summary table contains multiple values for the column total but no column variables are specified", {
			
	data <- data.frame(
		isTotal = c(FALSE, TRUE, TRUE),
		statN = c(1, 2, 3)
	)
	expect_error(
		exportSummaryStatisticsTable(data, outputType = "DT"),
		regexp = "Multiple values for the header total .*"
	)
			
})


test_that("A summary table is correctly exported to an html file", {
	
	skip_if_not(
		condition = rmarkdown::pandoc_available(), 
		message = "pandoc is not available"
	)
			
	summaryTable <- data.frame(n = 10)
	
	file <- tempfile(pattern = "table", fileext = ".html")
	expect_silent(
		exportSummaryStatisticsTable(
			summaryTable, 
			file = file
		)
	)
	expect_true(file.exists(file))

})

test_that("Variables are correctly expanded in a DataTables summary table", {
			
	summaryTable <- data.frame(
		patientProfileLink = "/path/to/patientProfile1.pdf",
		stringsAsFactors = FALSE
	)
			
	## stat in the row direction, but in a separated column
	dt <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		statsVar = "patientProfileLink",
		expandVar = "patientProfileLink",
		outputType = "DT",
		colHeaderTotalInclude = FALSE
	)

	
	# there is a button available:
	cDefs <- dt$x$options$columnDefs
	isControlPresent <- sapply(cDefs, function(x) 
		hasName(x, "className") && 
		x[["className"]] == "details-control"
	)
	expect_length(which(isControlPresent), 1)
	expect_equal(cDefs[[which(isControlPresent)]]$targets, 0)
	
	# there is a JS callback defined for this variable:
	expect_match(dt$x$callback, regexp = "patientProfileLink")
	
})

test_that("A subset of the variables are correctly expanded in a DataTables summary table", {
			
	summaryTable <- data.frame(
		patientProfileLink = "/path/to/patientProfile1.pdf",
		n = 1,
		stringsAsFactors = FALSE
	)
			
	# creation of expand for only one statistic works without error:
	expect_silent(
		dt <- exportSummaryStatisticsTable(
			summaryTable = summaryTable,
			statsVar = c("n", "patientProfileLink"),
			expandVar = "patientProfileLink",
			statsLayout = "row",
			outputType = "DT"
		)
	)
	
	# there is a button available:
	cDefs <- dt$x$options$columnDefs
	isControlPresent <- sapply(cDefs, function(x) 
		hasName(x, "className") && 
		x[["className"]] == "details-control"
	)
	expect_length(which(isControlPresent), 1)
	expect_equal(cDefs[[which(isControlPresent)]]$targets, 1)
	
	# there is a JS callback defined for this variable:
	expect_length(dt$x$callback, 1)
			
})

test_that("Page length is correctly set in a DataTables summary table", {
			
	summaryTable <- data.frame(n = 10)
	
	dt <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		pageDim = c(NA, 3),
		outputType = "DT"
	)
	expect_equal(
		object = dt$x$options$pageLength,
		expected = 3
	)
	
})

test_that("All variables of a summary table are correctly not (HTML) escaped when exported to DataTables", {
			
	summaryTable <- data.frame(
		patientProfileLink = '<a href="www.google.com">blabla</a>',
		n = 1,
		stringsAsFactors = FALSE
	)
	
	# entire table is escaped:
	dt <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		noEscapeVar = c("patientProfileLink", "n"),
		statsVar = c("patientProfileLink", "n"),
		outputType = "DT"
	)	
	expect_false(grepl("\\d", attr(dt$x$options, "escapeIdx")))
	
})

test_that("One variable of a summary table is correctly not (HTML) escaped when exported to DataTables", {
			
	summaryTable <- data.frame(
		patientProfileLink = '<a href="www.google.com">blabla</a>',
		n = 1,
		stringsAsFactors = FALSE
	)
	
	# escape only one column
	# This is checked with 'statsLayout' is 'row'
	# as in this case: the entire stat value column
	# should not be escaped
	dt <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		noEscapeVar = "patientProfileLink",
		statsVar = c("patientProfileLink", "n"),
		statsLayout = "row",
		outputType = "DT"
	)
	expect_match(
		object = attr(dt$x$options, "escapeIdx"), 
		regexp = "1"
	)
			
})

test_that("A specified parameter is correctly passed to datatable", {
	
	summaryTable <- data.frame(
		patientProfileLink = "www.google.com",
		n = 1,
		stringsAsFactors = FALSE
	)
	
	dt <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		outputType = "DT",
		width = 200
	)
	expect_equal(
		object = dt$width,
		expected = 200
	)
	
})

test_that("A warning is generated if a datatable parameter is already set via in-text specific parameters in a DataTables summary table", {
	
	summaryTable <- data.frame(
		patientProfileLink = "www.google.com",
		n = 1,
		stringsAsFactors = FALSE
	)
			
	# warning in case a parameter is already set
	# via the in-text table wrapper
	# e.g. 'noEscapeVar' is passed to datatable 'escape' parameter
	expect_warning(
		exportSummaryStatisticsTable(
			summaryTable = summaryTable,
			noEscapeVar = "patientProfileLink",
			escape = 1,
			outputType = "DT"
		),
		"Parameter.+escape.+ are already specified internally"
	)
	
})

test_that("An error is generated if a specified parameter is not a datatable parameter in a DataTables summary table", {
			
	summaryTable <- data.frame(
		patientProfileLink = "www.google.com",
		n = 1,
		stringsAsFactors = FALSE
	)
	
	# error if parameter not available
	expect_error(
		exportSummaryStatisticsTable(
			summaryTable = summaryTable,
			blabla = "test",
			outputType = "DT"
		),
		"unused argument"
	)
			
})

test_that("A list of summary tables is correctly exported to DataTables", {
			
	summaryTables <- list(
		`PARAM 2` = data.frame(n = 10),
		`PARAM 1` = data.frame(n = 2)
	)
			
	dts <- exportSummaryStatisticsTable(
		summaryTables, 
		outputType = "DT"
	)
	expect_type(dts, "list")
	expect_named(dts, names(summaryTables))
			
	for(group in names(summaryTables)){
				
		# table content is the same as if the table would have
		# been created directly
		expect_identical({
			dt <- exportSummaryStatisticsTable(summaryTables[[!!group]], outputType = "DT")
			dt$x$data
			},
			expected = dts[[!!group]]$x$data
		)
		
		# and group is specified in the title
		expect_true(grepl(!!group, dts[[!!group]]$x$caption,))
			
	}
	
})

test_that("A list of summary tables with different titles is correctly exported to DataTables", {
			
	summaryTables <- list(
		`PAR2` = data.frame(n = 10),
		`PAR1` = data.frame(n = 2)
	)
			
	titles <- c("PARAMETER 2", "PARAMETER 1")
	dts <- exportSummaryStatisticsTable(
		summaryTables, 
		outputType = "DT",
		title = titles
	)
			
	for(i in seq_along(summaryTables)){
		expect_true(grepl(titles[[!!i]], dts[[!!i]]$x$caption))
	}
			
})

test_that("A list of summary tables is successfully exported to html files", {
			
	skip_if_not(
		condition = rmarkdown::pandoc_available(), 
		message = "pandoc is not available"
	)
			
	summaryTables <- list(
		`PARAM 2` = data.frame(n = 10),
		`PARAM 1` = data.frame(n = 2)
	)
			
	file <- tempfile(pattern = "table", fileext = ".html")	
	dts <- exportSummaryStatisticsTable(
		summaryTables, 
		file = file
	)
	fileTableOutput <- paste0(
		tools::file_path_sans_ext(file),
		"_", c("1", "2"), ".html"
	)
	expect_true(all(file.exists(fileTableOutput)))

})


test_that("A variable is successfully visualized as a colored bar in a DataTables summary table", {
			
	summaryTable <- data.frame(
		n = c(1, 2, 4),
		PARAM = paste("PARAM", seq_len(3)),
		stringsAsFactors = FALSE
	)
			
	dt <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		barVar = "n",
		colHeaderTotalInclude = FALSE,
		rowVar = "PARAM", 
		outputType = "DT"
	)
	
	# check if a color gradient is specified in JS in the object:
	expect_match(dt$x$options$rowCallback, "linear-gradient")
			
})

#' Get number of decimals for a specific vector.
#' 
#' The number of decimals is extracted either:
#' \itemize{
#' \item{from specific implemented rule :
#' see \code{\link{getNDecimalsRule}} for further details}
#' \item{from the data itself: 
#' see \code{\link{getNDecimalsData}} for further details
#' }
#' \item{both criterias: in this case the minimum of 
#' the number of decimals for both criterias is used}
#' }
#' @param useRule Logical (TRUE by default), should the \code{rule} be applied?
#' @param useData Logical (TRUE by default), should the number of decimals
#' be extracted based on the input data \code{x}?
#' @inherit getNDecimalsRule params return
#' @author Laure Cougnaud
#' @examples 
#' x <- c(0.99, 5.679, 50.45, 1450)
#' # extract number of decimals based on data:
#' getNDecimals(x, useRule = FALSE, useData = TRUE)
#' # extract number of decimals based on pre-defined rule:
#' getNDecimals(x, useRule = TRUE, useData = FALSE)
#' # extract number of decimals based on both rules
#' # minimum of both is used (by default)
#' getNDecimals(x, useRule = TRUE, useData = TRUE)
#' @family decimals
#' @export
getNDecimals <- function(x, useRule = TRUE, rule = "1", useData = TRUE){
	
	if(!useRule & !useData)
		stop(paste("The number of decimals should be extracted based on the rule and/or the data:",
			"'useRule' and/or 'useData' should be set to TRUE."))
	
	if(useRule)	nDecRule <- getNDecimalsRule(x, rule = rule)
	if(useData)	nDecData <- getNDecimalsData(x)
	
	nDec <- if(useRule & useData){
		mapply(min, nDecRule, nDecData)
	}else	if(!useData){
		nDecRule
	}else if(!useRule)	nDecData
	
	return(nDec)
	
}

#' Get maximum number of decimals in a variable,
#' based on pre-defined rule and/or data.
#' 
#' The function \code{\link{getNDecimals}}) extracts the number
#' of decimals in a specific variable.
#' @param ... Any parameters for the \code{\link{getNDecimals}} function.
#' @inheritParams getNDecimals
#' @return Integer with maximum number of decimals in a character vector.
#' @author Laure Cougnaud
#' @examples 
#' x <- c(0.99, 5.679, 50.45, 1450)
#' # extract max number of decimals based on data:
#' getMaxNDecimals(x, useRule = FALSE, useData = TRUE)
#' # extract max number of decimals based on pre-defined rule:
#' getMaxNDecimals(x, useRule = TRUE, useData = FALSE)
#' # extract max number of decimals based on both rules
#' # minimum of both is used (by default)
#' getMaxNDecimals(x, useRule = TRUE, useData = TRUE)
#' @family decimals
#' @export
getMaxNDecimals <- function(x, ...)
	max(getNDecimals(x, ...), na.rm = TRUE)

#' Get number of decimals based pre-defined rule(s).
#' 
#' Note: NA is returned if the element is missing (NA).
#' @param rule Character vector with rule to use to derive 
#' the number of parameters.
#' Currently only: '1' is implemented.
#' \itemize{
#' \item{'1': standard rule for the number of 
#' decimals for individual values for a 
#' continuous variable:
#' \itemize{
#' \item{value < 1 ('very small values'): 3}
#' \item{value < 10: 2}
#' \item{value in [10, 1000[: 1}
#' \item{value >= 1000: 0}
#' }
#' }
#' }
#' @examples
#' x <- c(0.99, 5.679, 50.45, 1450)
#' getNDecimalsRule(x = x)
#' @inherit getNDecimalsData params return
#' @author Laure Cougnaud
#' @export
getNDecimalsRule <- function(x, rule = c("1")){
	
	if(!is.numeric(x))
		stop("'x' should be numeric.")
	
	nDecRule <- switch(rule,
		'1' = ifelse(x < 1, 3, ifelse(x < 10, 2, ifelse(x < 1000, 1, 0))),
		stop(paste("Rule:", shQuote(rule), "not yet implemented."))
	)
	return(nDecRule)
}

#' Get number of decimals based on the data in a numeric vector.
#' Note: NA is returned if the element is missing (NA).
#' @param x Numeric vector.
#' @return Numeric vector of same length than \code{x}
#' with the number of decimals.
#' @author Laure Cougnaud
#' @examples
#' x <- c(0.99, 5.679, 50.45, 1450)
#' getNDecimalsData(x)
#' @family decimals
#' @export
getNDecimalsData <- function(x){
	
	if(!is.numeric(x))
		stop("'x' should be numeric.")
	
	xNumber <- ifelse(
		is.na(x), 
		NA_character_,
		sub("-*[[:digit:]]+\\.*([[:digit:]]+)*", 
			"\\1", 
			as.character(sapply(x, format, scientific = FALSE)
			)
		)
	)
	nDec <- nchar(xNumber)
	return(nDec)
}

#' Get maximum number of decimals in a variable based on the data
#' (\code{\link{getNDecimalsData}})
#' @inheritParams getNDecimalsData
#' @return Integer with maximum number of decimals in a character vector.
#' @author Laure Cougnaud
#' x <- c(0.99, 5.679, 50.45, 1450)
#' # extract max number of decimals based on data:
#' getMaxNDecimalsData(x)
#' @family decimals
#' @export
getMaxNDecimalsData <- function(x)
	return(max(getNDecimalsData(x), na.rm = TRUE))

#' Format a percentage.
#' 
#' The following rules are used:
#' \itemize{
#' \item{percentage = 0\%: '0'}
#' \item{ 0\% < percentage < 0.1\%: '<0.1'}
#' \item{99.9\% < percentage < 100\%: '>99.9'}
#' \item{percentage = 100\%: '100'}
#' \item{missing value (NA) (class without valid data): '-'}
#' \item{other: 'x.x' (1 decimal)}
#' }
#' @param x Numeric vector with percentage(s)
#' @param nDec Integer of length 1, number
#' of decimals used to round the percentage,
#' 1 by default.
#' @return String with formatted percentage
#' @author Laure Cougnaud
#' @examples
#' xPerc <- c(NA, 0, 100, 99.95, 0.012, 34.768)
#' formatPercentage(x = xPerc)
#' @family decimals
#' @importFrom clinUtils roundHalfUpTextFormat
#' @export
formatPercentage <- function(x, nDec = 1){
	
	xRF <- ifelse(is.na(x),
		"-",
		ifelse(x == 0,
			"0", 
			ifelse(x == 100,
				"100",
				ifelse(x < 0.1,
					"<0.1",
					ifelse(x > 99.9,
						">99.9",
                        roundHalfUpTextFormat(x, digits = nDec)
					)
				)
			)
		)
	)
	return(xRF)
	
}

context("Format numbers")

test_that("Percentages are correctly formatted", {

	expect_equal(formatPercentage(x = 0.55, nDec = 2), "0.55")
	expect_equal(formatPercentage(x = 48.43, nDec = 1), "48.4")
	expect_equal(formatPercentage(x = 48.0, nDec = 1), "48.0")
	expect_equal(formatPercentage(x = 100, nDec = 2), "100")
	expect_equal(formatPercentage(x = NA), "-")
	expect_equal(formatPercentage(x = 0.01), "<0.1")
	expect_equal(formatPercentage(x = 99.9998), ">99.9")
	
})

test_that("The number of decimals is correctly extracted from the data", {
			
	# set of values above and below 0
	xOrdMag <- seq(-3, 3)
	x <- 10^(xOrdMag)
	expect_equal(
		object = getNDecimalsData(x), 
		expected = abs(ifelse(xOrdMag > 0, 0, xOrdMag))
	)

})

test_that("The number of decimals is set to missing is the value is missing", {
			
	expect_equal(getNDecimalsData(NA_real_), NA_real_)
	
})

test_that("An error is generated if the input data is not a numeric", {

	expect_error(getNDecimalsData("0.890"), "should be numeric")
				
})

test_that("The maximum number of decimals of a set of numbers is correctly extracted from the data", {

	x <- c(NA, 1e-2, 1e-3)
	expect_equal(getMaxNDecimalsData(x), 3)
			
})

test_that("The number of decimals is correctly extracted according to rule", {
		
	expect_equal(
		object = getNDecimalsRule(c(0.999, 1, 10, 1000), rule = "1"), 
		expected = c(3, 2, 1, 0)
	)
	
})

test_that("An error is generated if the specified rule for the number of decimals is not implemented", {
			
	expect_error(getNDecimalsRule(0.999, rule = "test"))
	
})

test_that("An error is generated if the input value for the extraction of the number of decimals according to rule is not a numeric", {
	
	expect_error(getNDecimalsRule("0.890"), "should be numeric")
			
})

test_that("The number of decimals is correctly extracted according to rule when specified", {
			
	nDec <- getNDecimals(
		x = 5.1, 
		useRule = TRUE, rule = "1", 
		useData = FALSE
	)
	expect_equal(
		object = nDec,
		expected = 2,
		label = "decimals from rule only"
	)
	
})

test_that("The number of decimals is correctly extracted from the data when specified", {
		
	nDec <- getNDecimals(
		x = 5.1, 
		useRule = FALSE, 
		useData = TRUE
	)
	expect_equal(
		object = nDec,
		expected = 1,
		label = "decimals from data only"
	)
	
})

test_that("The number of decimals is correctly extracted from the minimum of the data and rule when both are specified", {
			
	nDec <- getNDecimals(
		x = 5.1, 
		useRule = TRUE, rule = "1", 
		useData = TRUE
	)
	expect_equal(
		object = nDec,
		expected = 1,
		label = "decimals from rule and data"
	)
	
})

test_that("An error is generated if neither a rule or data are specified for the extraction of the number of decimals", {
			
	expect_error(
		getNDecimals(5.1, useRule = FALSE, useData = FALSE),
		"'useRule' and/or 'useData'"
	)
			
})

test_that("The maximum number of decimals of a set of numbers is correctly extracted from the data and rule", {
	
	x <- c(5.1, 6.8, 10.2, 80.0, NA_real_)
			
	# nDec based on rule is taken
	expect_equal(getMaxNDecimals(x), 1)
			
})

context("Get set of statistics")

library(clinUtils)

test_that("The output format of formatted base and combined statistics is correct", {	
			
	type <- c(
		"n", "m", "%", "%m", 
		"Mean", "SD", "Median", "SE", "Min", "Max",
		"median (range)", "median\n(range)", "mean (se)", "mean (range)", 
		"n (%)", "n/N (%)", "m (%)"
	)
	statName <- sapply(type, identity)
	statName[grep("^median|mean", type)] <- clinUtils::simpleCap(statName[grep("^median|mean", type)])
	statName["mean (se)"] <- "Mean (SE)"
	
	for(type in type){
				
		expect_type(getStats(type = !!type), "list")
		expect_length(getStats(type = !!type), 1)
		expect_equal(names(getStats(type = !!type)), statName[[!!type]])
		expect_type(getStats(type = !!type)[[ statName[[!!type]]]], "language")
		
	}
	
})

test_that("Base count and summary statistics are correctly formatted", {	
			
	typeBase <- c("n", "m", "Mean", "SD", "Median", "SE", "Min", "Max")
	statBase <- c(n = "statN", `%` = "statPercN", `%m` = "statPercm")
	statBase <- sapply(typeBase, function(type){
		if(type %in% names(statBase)){
			statBase[[type]]
		}else	paste0("stat", type)
	})
	for(type in names(statBase)){
		
		expect_equal({
				summaryTable <- setNames(
					data.frame(c(1, 2, NA_integer_)),
					statBase[!!type]
				)
				stat <- getStats(type = !!type)
				eval(expr = stat[[!!type]], envir = summaryTable)
			},
			formatC(x = summaryTable[[statBase[!!type]]])
		)
		
	}
})

test_that("Percentage of subjects and records are formatted correctly", {	
			
	statPerc <- c(`%` = "statPercN", `%m` = "statPercm")
	for(type in names(statPerc)){
		
		expect_equal({
			summaryTable <- setNames(
				data.frame(c(56.7, 0, 0.999, 1e-6, 99.99, NA_integer_)),
				statPerc[!!type]
			)
			stat <- getStats(type = !!type)
			eval(expr = stat[[!!type]], envir = summaryTable)
			},
			formatPercentage(x = summaryTable[[statPerc[!!type]]])
		)
	}
	
})

test_that("The number of subjects is correctly formatted with specified number of decimals as number", {
	
	stat <- getStats(type = "n", nDecN = 1)
	summaryTable <- data.frame(statN = c(13, 100.56))
	expect_equal(
		eval(expr = stat$n, envir = summaryTable),
		c("13.0", "100.6")
	)
			
})

test_that("The number of records is correctly formatted with specified number of decimals as number", {
			
	stat <- getStats(type = "m", nDecN = 2)
	summaryTable <- data.frame(statm = c(13, 100.567))
	expect_equal(
		eval(expr = stat$m, envir = summaryTable),
		c("13.00", "100.57")
	)
			
})

test_that("The mean is correctly formatted with specified number of decimals as number", {
			
	stat <- getStats(type = "Mean", nDecCont = 4)
	summaryTable <- data.frame(statMean = c(1098, 100.567567))
	expect_equal(
		eval(expr = stat$Mean, envir = summaryTable),
		c("1098.00000", "100.56757")
	)
			
})

test_that("The mean is correctly formatted with specified number of decimals as function", {
			
	stat <- getStats(
		type = "Mean", 
		x = seq.int(10),
		nDecCont = function(x) 2
	)
	
	summaryTable <- data.frame(statMean = c(1098, 100.567567))
		
	expect_equal(
		eval(expr = stat$Mean, envir = summaryTable),
		c("1098.000", "100.568")
	)
			
})

test_that("The geometric mean is correctly formatted with specified number of decimals as number", {
  
  stat <- getStats(type = "Geometric Mean", nDecCont = 4)
  summaryTable <- data.frame(statGeomMean = c(1098, 100.567567))
  expect_equal(
    eval(expr = stat$`Geometric Mean`, envir = summaryTable),
    c("1098.00000", "100.56757")
  )
  
})

test_that("The geometric mean is correctly formatted with specified number of decimals as function", {
  
  stat <- getStats(
    type = "Geometric Mean", 
    x = seq.int(10),
    nDecCont = function(x) 2
  )
  
  summaryTable <- data.frame(statGeomMean = c(1098, 100.567567))
  
  expect_equal(
    eval(expr = stat$`Geometric Mean`, envir = summaryTable),
    c("1098.000", "100.568")
  )
  
})

test_that("The median is correctly formatted with specified number of decimals as number", {
			
	stat <- getStats(type = "Median", nDecCont = 4)
	summaryTable <- data.frame(statMedian = c(1098, 100.567567))
	expect_equal(
		eval(expr = stat$Median, envir = summaryTable),
		c("1098.00000", "100.56757")
	)
			
})

test_that("The standard deviation is correctly formatted with specified number of decimals as number", {
			
	stat <- getStats(type = "SD", nDecCont = 4)
	summaryTable <- data.frame(statSD = c(1098, 100.567567))
	expect_equal(
		eval(expr = stat$SD, envir = summaryTable),
		c("1098.00000", "100.56757")
	)
			
})

test_that("The geometric standard deviation is correctly formatted with specified number of decimals as number", {
  
  stat <- getStats(type = "Geometric SD", nDecCont = 4)
  summaryTable <- data.frame(statGeomSD = c(1098, 100.567567))
  expect_equal(
    eval(expr = stat$`Geometric SD`, envir = summaryTable),
    c("1098.00000", "100.56757")
  )
  
})

test_that("The standard error is correctly formatted with specified number of decimals as number", {
			
	stat <- getStats(type = "SE", nDecCont = 4)
	summaryTable <- data.frame(statSE = c(1098, 100.56756767))
	expect_equal(
		eval(expr = stat$SE, envir = summaryTable),
		c("1098.000000", "100.567568")
	)
			
})

test_that("The minimum is correctly formatted with specified number of decimals as number", {
			
	stat <- getStats(type = "Min", nDecCont = 4)
	summaryTable <- data.frame(statMin = c(1098, 100.56756767))
	expect_equal(
		eval(expr = stat$Min, envir = summaryTable),
		c("1098.0000", "100.5676")
	)
			
})

test_that("The maximum is correctly formatted with specified number of decimals as number", {
			
	stat <- getStats(type = "Max", nDecCont = 4)
	summaryTable <- data.frame(statMax = c(1098, 100.56756767))
	expect_equal(
		eval(expr = stat$Max, envir = summaryTable),
		c("1098.0000", "100.5676")
	)
			
})

test_that("The default set of summary statistics is correctly extracted", {
			
	statType <- c("statN", "statMean", "statSD", "statSE", "statMedian", "statMin", "statMax")
	type <- ifelse(statType == "statN", "n", sub("stat", "", statType))
	
	expect_identical(
		getStats(type = "summary-default"),		
		getStats(type = type)
	)
			
})

test_that("The entire set of summary statistics is correctly extracted", {
			
	statType <- c("statN", "statMean", "statSD", "statSE", "statMedian", 
		"statMin", "statMax", "statPercN", "statm")
	type <- sapply(statType, sub, pattern = "stat", replacement = "")
	type["statN"] <- "n"
	type["statPercN"] <- "%"
			
	expect_identical(
		getStats(type = "summary"),		
		getStats(type = type)
	)
			
})

test_that("The default set of count statistics is correctly extracted", {
			
	statType <- c("statN", "statPercN")
	type <- c(statN = "n", statm = "%")
			
	expect_identical(
		getStats(type = "count-default"),		
		getStats(type = type)
	)
			
})

test_that("The entire set of count statistics is correctly extracted", {
			
	statType <- c("statN", "statPercN", "statm")
	type <- c(statN = "n", statm = "%", statm = "m")
			
	expect_identical(
		getStats(type = "count"),		
		getStats(type = type)
	)
			
})

test_that("The median (range) is correctly extracted", {

	stat <- getStats(type = "median (range)")
	
	statType <- c("statMin", "statMax", "statMedian")
	type <- sub("stat", "", statType)
	summaryTable <- setNames(
		as.data.frame(replicate(length(statType), rnorm(4))),
		statType
	)
	
	# check value
	statValue <- eval(expr = stat[[1]], envir = summaryTable)
	statBase <- getStats(type = type)
	statBaseValue <- as.data.frame(lapply(statBase, eval, envir = summaryTable))
	expect_equal(
		statValue,
		with(statBaseValue, paste0(Median, " (", Min, ",", Max, ")"))
	)
			
})

test_that("The median\n(range) is correctly extracted", {
			
	stat <- getStats(type = "median\n(range)")
			
	statType <- c("statMin", "statMax", "statMedian")
	type <- sub("stat", "", statType)
	summaryTable <- setNames(
		as.data.frame(replicate(length(statType), rnorm(4))),
		statType
	)
			
	# check value
	statValue <- eval(expr = stat[[1]], envir = summaryTable)
	statBase <- getStats(type = type)
	statBaseValue <- as.data.frame(lapply(statBase, eval, envir = summaryTable))
	expect_equal(
		statValue,
		with(statBaseValue, paste0(Median, "\n(", Min, ",", Max, ")"))
	)
			
})

test_that("The mean (se) is correctly extracted", {
			
	stat <- getStats(type = "mean (se)")
			
	statType <- c("statMean", "statSE")
	type <- sub("stat", "", statType)
	summaryTable <- setNames(
		as.data.frame(replicate(length(statType), rnorm(4))),
		statType
	)
			
	# check value
	statValue <- eval(expr = stat[[1]], envir = summaryTable)
	statBase <- getStats(type = type)
	statBaseValue <- as.data.frame(lapply(statBase, eval, envir = summaryTable))
	expect_equal(
		statValue,
		with(statBaseValue, paste0(Mean, " (", SE, ")"))
	)
	
})

test_that("The mean (range) is correctly extracted", {
			
	stat <- getStats(type = "mean (range)")
			
	statType <- c("statMean", "statMin", "statMax")
	type <- sub("stat", "", statType)
	summaryTable <- setNames(
		as.data.frame(replicate(length(statType), rnorm(4))),
		statType
	)
	
	# check value
	statValue <- eval(expr = stat[[1]], envir = summaryTable)
	statBase <- getStats(type = type)
	statBaseValue <- as.data.frame(lapply(statBase, eval, envir = summaryTable))
	expect_equal(
		statValue,
		with(statBaseValue, paste0(Mean, " (", Min, ",", Max, ")"))
	)
	
})

test_that("The n (%) is correctly extracted", {
			
	stat <- getStats(type = "n (%)")
			
	statType <- c("statN", "statPercN")
	type <- c(statN = "n", statPercN = "%")
	summaryTable <- setNames(
		as.data.frame(replicate(length(statType), rnorm(4))),
		statType
	)
			
	# check value
	statValue <- eval(expr = stat[[1]], envir = summaryTable)
	statBase <- getStats(type = type)
	statBaseValue <- as.data.frame(lapply(statBase, eval, envir = summaryTable), check.names = FALSE)
	expect_equal(
		statValue,
		with(statBaseValue, paste0(n, " (", `%`, ")"))
	)
			
})

test_that("If case of 0 counts, the percentage is not included in n (%)", {
			
	stat0 <- eval(
		expr = getStats(type = "n (%)")[[1]],
		envir = data.frame(statN = 0, statPercN = Inf)
	)
	expect_equal(stat0, "0")
	
})

test_that("The m (%) is correctly extracted", {
			
	stat <- getStats(type = "m (%)")
	
	statType <- c("statm", "statPercm")
	type <- c(statm = "m", statPercm = "%m")
	summaryTable <- setNames(
		as.data.frame(replicate(length(statType), rnorm(4))),
		statType
	)
	
	# check value
	statValue <- eval(expr = stat[[1]], envir = summaryTable)
	statBase <- getStats(type = type)
	statBaseValue <- as.data.frame(lapply(statBase, eval, envir = summaryTable), check.names = FALSE)
	expect_equal(
		statValue,
		with(statBaseValue, paste0(m, " (", `%m`, ")"))
	)
	
})

test_that("If case of 0 counts, the percentage is not included in m (%)", {
			
	stat0 <- eval(
		expr = getStats(type = "m (%)")[[1]],
		envir = data.frame(statm = 0, statPercm = Inf)
	)
	expect_equal(stat0, "0")
			
})

test_that("The n/N (%) is correctly extracted", {
			
	stat <- getStats(type = "n/N (%)")
			
	statType <- c("statN", "statPercN", "statPercTotalN")
	summaryTable <- setNames(
		as.data.frame(replicate(length(statType), rnorm(4))),
		statType
	)
			
	statValue <- eval(expr = stat[[1]], envir = summaryTable)
			
	# get base stats
	type <- c(statN = "n", statPercN = "%")
	statBase <- getStats(type = type)
	statBaseValue <- as.data.frame(lapply(statBase, eval, envir = summaryTable), check.names = FALSE)
	# + add statPercTotalN
	statBaseValue <- cbind(summaryTable, statBaseValue)		
	
	expect_equal(
		statValue,
		with(statBaseValue, paste0(n, "/", statPercTotalN, " (", `%`, ")"))
	)
			
})

test_that("If case of 0 counts, the percentage not included in n/N (%)", {
			
	stat0 <- eval(
		expr = getStats(type = "n/N (%)")[[1]],
		envir = data.frame(statN = 0, statPercN = Inf, statPercTotalN = NA)
	)
	expect_equal(stat0, "0")
			
})

test_that("A warning is generated if the statistics don't have unique nams", {
			
	expect_warning(
		getStats(type = c("count", "n")),
		"duplicated names"
	)
			
})

test_that("The statistic name is included if specified", {

	expect_named(
		getStats(type = "n", includeName = TRUE),
		"n"
	)
	
})

test_that("The statistic name is not included if specified", {
			
	expect_named(
		getStats(type = "n", includeName = FALSE),
		NULL
	)
			
})

test_that("The name is by default included if multiple statistics are requested", {
		
	type <- c("n", "m")
	expect_warning(
		stat <- getStats(type = type, includeName = FALSE),
		"The labels for the different types.*are retained.*"
	)
	
	expect_named(stat, type)
			
})

#' inTextSummaryTable: creation of in-text summary table
#' 
#' @description Tables of summary statistics or count tables are created. 
#' These tables can be exported as in-text table to a Clinical Study Report 
#' (Word format), a topline presentation (PowerPoint format),
#'  or as interactive table to an html document.
#'
#' @details
#' \itemize{
#' \item{To get started with the package, see:\cr
#' \code{vignette("inTextSummaryTable-01-introduction", package = "inTextSummaryTable")}}
#' \item{To get example code for standard in-text tables created with the package, 
#' see:\cr
#' \code{vignette("inTextSummaryTable-03-standardTables", package = "inTextSummaryTable")}}
#' \item{The main function: \code{\link{getSummaryStatisticsTable}} enables
#' to create ready in-text table.}
#' }
#' @docType package
#' @name inTextSummaryTable
#' @aliases inTextSummaryTable-package
NULL

#' Get dimension of the page available for content 
#' for standard Word report or PowerPoint presentation.
#' 
#' Report is in A4 and presentation dimensions extracted from
#' PowerPoint.
#' The returned dimensions are the page dimensions without the margins.
#' @param type Character vector with dimension of interest, among: 
#' 'width', 'height', multiple are possible.
#' By default: \code{c("width", "height")}
#' @param landscape Logical, if TRUE the table is presented in landscape
#' format. \cr
#' By default: TRUE for \code{style}: 'report',
#' FALSE for \code{style}: 'presentation'.
#' @param margin Margin in the document in inches, 1 by default.
#' @param style String with table style, either 'report' (by default, a4 format) 
#' or 'presentation'
#' @param pageDim (optional) Numeric vector of length 2 with 
#' page width and height in inches in portrait format,
#' in case page dimensions differ from the default
#' implemented report/presentation.
#' These dimensions should include the margins.
#' @examples 
#' ## get part of the page available for content
#' # report A4 portrait format:
#' getDimPage(type = "width")
#' getDimPage(type = "height")
#' # report A4 landscape format:
#' getDimPage(type = "width", landscape = TRUE)
#' getDimPage(type = "height", landscape = TRUE)
#' # Note that the layout is by default set to 'landscape'
#' getDimPage(type = "width", style = "presentation")
#' getDimPage(type = "height", style = "presentation")
#' # custom dimensions: A3 format
#' getDimPage(type = "width", pageDim = c(11.7, 16.5))
#' # increase margin
#' getDimPage(type = "width", margin = 1.5)
#' # get both dimensions at once
#' getDimPage(type = c("width", "height"))
#' # get dimensions of the full page (including margins)
#' getDimPage(type = c("width", "height"), style = "report", margin = 0)
#' getDimPage(type = c("width", "height"), style = "presentation", margin = 0)
#' @return numeric vector with dimension of interest,
#' in the same order as specified via the \code{type}
#' parameter.
#' @author Laure Cougnaud
#' @export
getDimPage <- function(
    type = c("width", "height"), 
    landscape = (style == "presentation"), 
    margin = 1,
    pageDim = NULL,
    style = "report")
{
  
  # landscape: 29.7 * 21 cm ~ 11 * 8 inches ~ 2138.4 * 1512 ptx
  type <- match.arg(type, several.ok = TRUE)
  
  style <- match.arg(style, choices = c("report", "presentation"))
  
  pageDimPortrait <- 	if(is.null(pageDim)){
        switch(style,
            'report' = c(21, 29.7)/2.54,
            'presentation' = getOption("inTextSummaryTable.pageDim.presentation")
        )
      } else {
        if(!is.numeric(pageDim))
          stop("'pageDim' should be a numeric vector.")
        if(length(pageDim) != 2)
          stop("'pageDim' should be of length 2.")
        pageDim
      }
  
  typeDim <- numeric()
  for(i in seq_along(type)) {
    typeDim[i] <- switch(type[i],
        'width' = ifelse(landscape, pageDimPortrait[2], pageDimPortrait[1]),
        'height' = ifelse(landscape, pageDimPortrait[1], pageDimPortrait[2])
    )
  }
  dimPage <- typeDim - 2 * margin
  
  return(dimPage)
}


#' Page dimension for powerpoints
#' @export 
pageDimPresentation <- c(7.5, 10)

#' Get specific attribute from a summaryTable or a list of summaryTables
#' @param name String with attribute name.
#' @param default Object with default value for attribute, 
#' NULL by default.
#' @return Object with attribute, or \code{default} is doesn't
#' exists in \code{summaryTable}
#' @author Laure Cougnaud
#' @keywords internal
getAttribute <- function(summaryTable, name, default = NULL){
	
	attribute <- if(is.data.frame(summaryTable)){
		attributes(summaryTable)$summaryTable[[name]]
	}else{
		attributeList <- lapply(summaryTable, function(x) attributes(x)$summaryTable[[name]])
		# only keep names within the list ('use.names' in unlist concatenate with higher-level element)
		res <- unlist(attributeList)
		names(res) <- unlist(lapply(attributeList, names))
		# use 'duplicated' because unique lose names
		res[!duplicated(res)]
	}
	
	if(is.null(attribute))
		attribute <- default

	return(attribute)
	
}

#' Compute the interaction between variable(s),
#' without propagating the missing values (if present in one of the variable)
#' unlike the behaviour with \code{\link{interaction}}.
#' This also ensure that the levels of the final interaction variables
#' are ordered similarly as the levels of the input \code{var} (if present).
#' @param data Data.frame with data.
#' @param var Character vector with variable(s) to consider.
#' @param varDataLevels (optional) Data.frame with data to consider
#' to define the levels of the variable.
#' If not specified, only the combinations of variable(s) available in the data are retained.
#' @return list with:
#' \itemize{
#' \item{'x': factor with interaction between the input \code{var}}
#' \item{'dataLevels': data.frame with mapping between the \code{var} 
#' variable(s) and the new factor levels}
#' }
#' @author Laure Cougnaud
#' @keywords internal
interactionCustom <- function(data, var, 
	varDataLevels = NULL){

	# use paste rather than 'interaction', otherwise the missing values are propagated
	varInteraction <- do.call(paste, data[, var, drop = FALSE])

	# extract the levels of the output factor 
	dataLevels <- (if(!is.null(varDataLevels))	varDataLevels	else	data)[, var, drop = FALSE]
	# ensure that the order from the input factors are preserved:
	dataLevels <- unique(dataLevels[do.call(order, dataLevels), , drop = FALSE])
	dataLevels$factorLevel <- do.call(paste, dataLevels)

	# build the final output factor
	if(!all(varInteraction %in% dataLevels$factorLevel))
		warning("Some variable records are not present in the data used for variable levels.")
	varInteractionFact <- factor(varInteraction, levels = dataLevels$factorLevel)
	
	res <- list(x = varInteractionFact, dataLevels = dataLevels)
	
	return(res)
	
}

#' Convert flag variable to a format such as only the flagged records are counted
#' in the summary table.
#' @param x Character or factor variable with flag variable,
#' should contain elements: 'Y' and 'N', or '' (for missing value).
#' @return Formatted factor variable with levels: 'Y' & 'N'.
#' Empty strings have been converted to NA.
#' @author Laure Cougnaud
convertVarFlag <- function(x){
	
	if(any(!x %in% c("", "Y", "N")))
		stop("Flag variable should only contain: '', 'Y' or 'N'.")
	x <- as.character(x)
	x[which(x == "")] <- NA_character_
	xRF <- factor(x, levels = c("Y", "N"))

	return(xRF)

}

#' Post-process the summary statistics table with variable flag.
#' 
#' This function is for internal use within
#' the \code{\link{computeSummaryStatisticsTable}}
#' function.
#' 
#' This includes:
#' \itemize{
#' \item{converting the records from a flag variable
#' for the 'variableGroup' variable 
#' from 'Y' to \code{NA_character_}
#' }
#' \item{filter records from a flag variable 
#' with variableGroup set as 'N'}
#' }
#' @param summaryTable Summary table as created internally in
#' \code{\link{computeSummaryStatisticsTable}}.
#' @inheritParams inTextSummaryTable-common-args
#' @return Summary table with
#' @author Laure Cougnaud
postProcessVarFlag <- function(summaryTable, varFlag){
	
	# which records refer to a flag variable?
	summaryTable$isVarFlag <- if("variableInit" %in% colnames(summaryTable)){
		summaryTable$variableInit %in% varFlag
	}else{TRUE}
	
	# filter the 'non' flagged counts:
	idxKept <- with(summaryTable, which(isTotal | !(isVarFlag & variableGroup == "N")))
	summaryTable <- summaryTable[idxKept, ]
	
	# convert '' to NA to indicate that there was no internal subgroup
	idxFlag <- with(summaryTable, which(isVarFlag & variableGroup == "Y"))
	summaryTable[idxFlag, "variableGroup"] <- NA_character_
	
	summaryTable$isVarFlag <- NULL
	
	return(summaryTable)
	
}

#' Convert \code{rowVar}, \code{colVar} and character \code{var} in \code{data} to factor
#' @return Updated \code{data}
#' @inheritParams inTextSummaryTable-common-args
#' @author Laure Cougnaud
convertVarRowVarColVarToFactor <- function(data, rowVar = NULL, colVar = NULL, var = NULL){
	
	# convert row and column variable to factor in the data
	# (if character, variables pre-defined as factor in one summary tables will be lost during rbind.fill)
	if(!is.null(rowVar)){
		rowVar <- intersect(rowVar, colnames(data))
		if(length(rowVar) > 0)
			data[, rowVar] <- lapply(data[, rowVar, drop = FALSE], function(x){
				levelsX <- if(is.factor(x))	levels(x)	else	sort(unique(x))
				factor(x, levels = levelsX)
			})
	}
	
	if(!is.null(colVar)){
		colVar <- intersect(colVar, colnames(data))
		if(length(colVar) > 0)
			data[, colVar] <- lapply(data[, colVar, drop = FALSE], function(x){
				levelsX <- if(is.factor(x))	levels(x)	else	sort(unique(x))
				factor(x, levels = levelsX)
			})
	}

	if(!is.null(var)){
		var <- intersect(var, colnames(data))
		if(length(var) > 0){
			isVarCharac <- names(which(sapply(data[, var, drop = FALSE], is.character)))
			if(length(isVarCharac) > 0)
				data[, isVarCharac] <- lapply(data[, isVarCharac, drop = FALSE], function(x){
					levelsX <- if(is.factor(x))	levels(x)	else	sort(unique(x))
					factor(x, levels = levelsX)
				})
		}
	}

	return(data)
	
}

#' Check the \code{varLabInclude} variable.
#' 
#' This function ensures that:
#' \itemize{
#' \item{variable name is included if more than one variable
#' are specified
#' }
#' \item{variable name is not included if no variable is specified}
#' }
#' @param var String with variable to check.
#' @inheritParams inTextSummaryTable-common-args
#' @return (Updated) \code{varLabInclude}
#' @author Laure Cougnaud
checkVarLabInclude <- function(var, varLabInclude = length(var) > 1){
	
	if(length(var) > 1 & !varLabInclude){
		warning(paste("Variable label is included in the table ('varLabInclude' is set to TRUE)",
			"because more than 1 variable is specified in the 'var' parameter."))
		varLabInclude <- TRUE
	}
	if(length(var) == 0 & varLabInclude){
		warning(paste("Variable label is not included ('varLabInclude' is set to FALSE)",
			"because no variable is specified in the 'var' parameter."))
		varLabInclude <- FALSE
	}
	
	return(varLabInclude)
	
}

#' Check if variable(s) are present in reference: either in columns in 
#' a dataset or in reference set.
#' 
#' Filter variables not present in the data or in reference set with a warning,
#' and only returned filtered vector, or NULL if empty.
#' @param var String with variable to check.
#' @param varLabel String with label for \code{var}, e.g.
#' name of associated parameter.
#' @param varRef (Named) character vector with set of reference variables.
#' @param data Data.frame with data.
#' @param refLabel String with label for the reference
#' @param varUncheck (Named) character vector with extra variables 
#' in \code{var} which shouldn't be checked.
#' @param msgType String with type of message returned, either a 'warning' (default)
#' or an error.
#' @return Depending on \code{msgType}:
#' \itemize{
#' \item{\code{warning}: warning is printed in the console, and a 
#' \code{var} filtered with element not in \code{data}
#' or in \code{refSet} is returned.
#' If filtered \code{var} is empty, NULL is returned.}
#' \item{\code{error}: an error is triggered.}
#' }
#' @author Laure Cougnaud
checkVar <- function(
	var, varLabel, varUncheck = NULL,
	varRef, 
	refLabel = ifelse(!missing(varRef), "reference variable", "data"),
	data,
	msgType = c("warning", "error")
){
	
	msgType <- match.arg(msgType)
	
	if(!missing(varRef) & !missing(data))
		stop("Either 'data' or 'varRef' should be specified.")
	
	varToFilter <- NULL
	if(!missing(varRef)){
		varToFilter <- var[!var %in% varRef]
	}else	if(!missing(data)){
		varToFilter <- var[!var %in% colnames(data)]
	}else	stop("'data' or 'varRef' should be specified.")
	
	# remove extra variables that shouldn't be checked
	varToFilter <- varToFilter[!varToFilter %in% varUncheck]
	
	if(length(varToFilter) > 0){
		msg <- paste0(
			"Variable(s): ",
			toString(shQuote(varToFilter)),
			" in ", varLabel, 
			if(msgType == "warning")	" are ignored because they",
			" are not available in: ", 
			refLabel, "."
		)
		switch(msgType, warning = warning(msg), error = stop(msg))
	}
	
	varFiltered <- var[!var %in% varToFilter]
	if(length(varFiltered) == 0)
		varFiltered <- NULL
	
	return(varFiltered)
	
}

#' Custom function to write table to a text file
#' 
#' This function is mainly a wrapper on \code{\link[utils]{write.table}},
#' with the specific options:
#' \itemize{
#' \item{no rownames}
#' \item{no quoting}
#' \item{tab separator}
#' }
#' @param x Data.frame to export to the table, or list of such tables.
#' @param file String with text file to export to.
#' @param ... Any parameters passed to the \code{\link[utils]{write.table}} function.
#' @return No returned value, the object \code{x} is exported to the specified \code{file}.
#' @importFrom utils write.table
#' @importFrom tools file_ext file_path_sans_ext
#' @author Laure Cougnaud
writeTable <- function(x, file, ...){
  
  if(length(file) > 1)
    stop("'file' should be of length 1.")
  if(file_ext(file) != "txt")
    stop("'file' should be of 'txt' extension.")
  
  if(!is.data.frame(x)) {
    tmp <- sapply(seq_along(x), function(i){
      fileI <- paste0(file_path_sans_ext(file), "_", i, ".", file_ext(file))
      writeTable(x = x[[i]], file = fileI, ...)
    })
  }else{
	  write.table(x, file, quote = FALSE, sep = "\t", row.names = FALSE, ...)
  }
	
}

context("Test miscellaneous functions")

test_that("A variable is correctly converted to a flag variable", {
      
	x <- c("Y", "N", '')	
	xFL <- inTextSummaryTable:::convertVarFlag(x)
	expect_is(xFL, "factor")
      
	# correct levels
	expect_equivalent(levels(xFL), c("Y", "N"))
      
	# correct conversions
	expect_equal(as.character(xFL), c("Y", "N", NA_character_))
	
})

test_that("An error is generated if a flag variable contains missing values", {
      
	x <- c("Y", "N", '')
	expect_error(
		inTextSummaryTable:::convertVarFlag(c(NA_character_, x)), 
		pattern = "*should only contain*"
	)
	
})

test_that("An error is generated if a flag variable contains unexpected elements", {
			
	expect_error(
		object = inTextSummaryTable:::convertVarFlag(c("blabla", x)), 
		pattern = "*should only contain*"
	)

})

test_that("An error is generated if the specified page dimensions are not a numeric", {
      
	expect_error(
		getDimPage(type = "width", pageDim = "ciao"),
		"'pageDim' should be a numeric vector."
	)
	
})
      
test_that("An error is generated if the specified page dimensions are not of correct length", {
			
	expect_error(
		getDimPage(type = "width", pageDim = c(10, 2, 3)),
		"'pageDim' should be of length 2."
	)
      
})

test_that("The page width is correctly extracted for a report", {
      
	expect_equal(
		object = getDimPage(type = "width", style = "report", margin = 0),
		expected = 21/2.54
	)
	
})

test_that("The page height is correctly extracted for a report", {
			
	expect_equal(
		object = getDimPage(type = "height", style = "report", margin = 0),
		expected = 29.7/2.54
	)
      
})

test_that("The page dimensions are correctly extracted (and in correct order) if multiple dimensions are specified", {
      
	expect_equal(
		object = getDimPage(type = c("height", "width"), style = "report"),
		expected = rev(getDimPage(type = c("width", "height"), style = "report"))
	)
      
})

test_that("The page width is correctly extracted for a presentation", {
      
	expect_equal(
		object = getDimPage(type = "width", style = "presentation", margin = 0),
		expected = 10
	)
	
})

test_that("The page height is correctly extracted for a presentation", {
			
	expect_equal(
		object = getDimPage(type = "height", style = "presentation", margin = 0),
		expected = 7.5
	)
      
})

test_that("The dimensions in portrait and landscape match", {
      
	expect_equal(
		object = getDimPage(type = "width", style = "report", margin = 0, landscape = FALSE),
		expected = getDimPage(type = "height", style = "report", margin = 0, landscape = TRUE)
	)
	expect_equal(
		object = getDimPage(type = "height", style = "report", margin = 0, landscape = FALSE),
		expected = getDimPage(type = "width", style = "report", margin = 0, landscape = TRUE)
	)
      
})

test_that("The page width is correctly set if a margin is specified", {
      
	widthPageNoMargin <- getDimPage(type = "width", style = "report", margin = 0)
	expect_equal(
		getDimPage(type = "width", style = "report", margin = 3),
		expected = widthPageNoMargin-3*2
	)
	
})

test_that("The page height is correctly set if a margin is specified", {
			
	heightPageNoMargin <- getDimPage(type = "height", style = "report", margin = 0)
	expect_equal(
		getDimPage(type = "height", style = "report", margin = 3),
		expected = heightPageNoMargin-3*2
	)
      
})

test_that("The page width is correctly set when page dimensions are specified", {
      
	pageDimSpecified <- c(10, 20)
	expect_equal(
		getDimPage(type = "width", style = "report", margin = 0, pageDim = pageDimSpecified),
		expected = pageDimSpecified[1]
	)
	
})

test_that("The page height is correctly set when page dimensions are specified", {
			
	pageDimSpecified <- c(10, 20)
	expect_equal(
		getDimPage(type = "height", style = "report", margin = 0, pageDim = pageDimSpecified),
		expected = pageDimSpecified[2]
	)	
      
})

#' Merge nested rows of a summary table
#' for a format compatible with \code{flextable} 
#' @inheritParams inTextSummaryTable-common-args
#' @inheritParams inTextSummaryTable-flextable-args
#' @return summaryTable reformatted in long format, with extra attributes:
#' \itemize{
#' \item{'header': data.frame with header for each column}
#' \item{'padParams': list of list of parameters to be passed to the 
#' \code{\link[flextable]{padding}} function}
#' \item{'rowVar': column of output with row variable}
#' \item{'rowVarInSepCol': column(s) of output with row variable in separated column(s)}
#' \item{'vlineParams' and 'hlineParams': 
#' list of list with correspondingly parameters for
#' vertical and horizontal lines
#' }
#' \item{'vline': \code{vline} parameter}
#' \item{'formatParams': list of list with special formatting for the table,
#' currently only used if \code{rowVarFormat} if specified.}
#' }
#' If \code{summaryTable} is a list of summary tables,
#' returns a list of corresponding summary tables in long format.
#' @importFrom clinUtils getLabelVar
#' @author Laure Cougnaud
formatSummaryStatisticsTableFlextable <- function(
	summaryTable,
	rowVar = getAttribute(summaryTable, "rowVar"), 
	rowVarInSepCol = NULL, 
	rowVarTotalInclude = getAttribute(summaryTable, "rowVarTotalInclude"),
	statsLayout = "row", 
	statsVar = getAttribute(summaryTable, "statsVar"), 	
	statsLabInclude = getAttribute(summaryTable, "statsLabInclude", default = length(statsVar) > 1),
	rowVarLab = getAttribute(summaryTable, "rowVarLab", default = getLabelVar(rowVar, labelVars = labelVars)),
	rowVarTotalInSepRow = NULL,
	vline = c("none", "auto"),
	hline = c("none", "auto"),
	rowAutoMerge = TRUE,
	rowVarFormat = NULL,
	rowTotalLab = NULL,
	labelVars = NULL){

	statsLayout <- match.arg(statsLayout, choices = c("row", "col", "rowInSepCol"))
		
	vline <- match.arg(vline)
	hline <- match.arg(hline)
	
	if(is.null(summaryTable)){
		return(invisible())
	}
	
	if(!is.data.frame(summaryTable)){
		
		inputParams <- as.list(environment())
		res <- sapply(summaryTable, function(summaryTableI){
			inputParamsBy <- inputParams
			inputParamsBy$summaryTable <- summaryTableI
			do.call(formatSummaryStatisticsTableFlextable, inputParamsBy)		
		}, simplify = FALSE)	
		return(res)
		
	}
	
	if(length(rowVarTotalInclude) > 0){
		rowVarTotalWrongFormat <- names(which(sapply(
			summaryTable[, rowVarTotalInclude, drop = FALSE], function(x){
			!(is.factor(x) && levels(x)[1] == "Total")
		})))
		if(length(rowVarTotalWrongFormat) > 0){
			
			warning(paste(
				"Row variable(s):", toString(shQuote(rowVarTotalWrongFormat)),
				"with total should be formatted",
				"as factor with 'Total' as the first level,",
				"these variable(s) are not considered for total formatting."
			))
			
			rowVarTotalInclude <- setdiff(rowVarTotalInclude, rowVarTotalWrongFormat)
		}
	}
	
	hlineParams <- mergeParams <- formatParams <- NULL
	
	mergeRows <- !is.null(rowVar) | (statsLayout != "rowInSepCol" & statsLabInclude)
	if(mergeRows){
	
		rowVarUsed <- c(rowVar, if(statsLayout == "row" & statsLabInclude)	"Statistic")
		
		# important: sort summaryTable.frame with specified row variables!
		summaryTable <- ddply(summaryTable, rowVar)
		if(".id" %in% colnames(summaryTable))	summaryTable$`.id` <- NULL
		
		# in case than rowVar are factor, can have issues to include additional rows (*invalid factor*), so convert them as character
		rowVarUsedFact <- names(which(sapply(summaryTable[, rowVarUsed, drop = FALSE], is.factor)))
		if(length(rowVarUsedFact) > 0)
			summaryTable[, rowVarUsedFact] <- colwise(.fun = as.character)(summaryTable[, rowVarUsedFact, drop = FALSE])
		
		# if more than one rowVar, convert them to different rows
		rowVarInRow <- setdiff(rowVarUsed, rowVarInSepCol) # row variables to merge
		rowVarFinal <- rowVarInRow[length(rowVarInRow)] # final column = more nested row variable
		rowVarToModify <- rowVarInRow[-length(rowVarInRow)] # variables to merge
		
		getTotalRow <- function(summaryTable){
			# if only variableGroup is specified, no total row
			rowVarForTotal <- setdiff(rowVar, c("variable", "variableGroup"))
			if(length(rowVarForTotal) > 0){
				which(rowSums(summaryTable[, rowVar, drop = FALSE] == "Total") == length(rowVar))
			}else	integer()
		}
		
		if(length(rowVarToModify) > 0){
			
			# save initial value of rowVarFinal for later
			summaryTable$rowVarFinal <- summaryTable[, rowVarFinal]
			
			summaryTable$rowPadding <- rowPadding <- length(rowVarInRow)-1
			idxTotalRow <- getTotalRow(summaryTable = summaryTable)
			summaryTable[idxTotalRow, "rowPadding"] <- 0
			
			# include the value in each rowVar in the column of the more nested variable
			# start by the second more nested variable
			for(i in rev(seq_along(rowVarToModify))){
				
				rowPadding <- rowPadding - 1
				
				var <- rowVarToModify[i]
				varX <- summaryTable[, var]
				
				## add new rows
				
				# get indices of rows to replicates
				dataVarI <- summaryTable[, rowVarToModify[seq_len(i)]]
				# fix in case value in one column is NA
				# -> interaction is set to NA (without concatenating other columns)
				dataVarI[is.na(dataVarI)] <- ""
				idxRowToMove <- which(!duplicated(interaction(dataVarI))) 
				idxRowToMove <- setdiff(idxRowToMove, idxTotalRow)
				
				# 1) the total should be included in the header row
				# ( == the sub-total for this variable is computed and not included in separated row)
				varNested <- rowVarInRow[match(var, rowVarInRow)+1]
				isVarTotalNotInSepRow <- varNested %in% rowVarTotalInclude & !varNested %in% rowVarTotalInSepRow
				if(isVarTotalNotInSepRow){
					
					# in case multiple records for total, add extra rows
					varNestedAll <- rowVarInRow[seq(from = match(var, rowVarInRow)+1, to = length(rowVarInRow))]
					idxTotalNested <- which(rowSums(summaryTable[, varNestedAll, drop = FALSE] == "Total") == length(varNestedAll))
					idxTotalNested <- idxTotalNested[which(diff(idxTotalNested) == 1) + 1]
					idxRowToMove <- sort(unique(c(idxRowToMove, idxTotalNested)))
					
					if(var == "variable" & varNested == "variableGroup"){
						
						# if variable total is included only for certain variables
						idxCorrect <- which(summaryTable[idxRowToMove, rowVarFinal] %in% c("Total", ""))
						idxRowToFill <- idxRowToMove[idxCorrect]
						idxRowToRepl <- setdiff(idxRowToMove[-idxCorrect], idxTotalNested)
						
					}else{
						if(any(summaryTable[idxRowToMove, rowVarFinal] != "Total"))
								stop("Missing total sub-category")
						idxRowToFill <- idxRowToMove
						idxRowToRepl <- integer()
					}
							
					summaryTable[idxRowToFill, rowVarFinal] <- summaryTable[idxRowToFill, var]
					summaryTable[idxRowToFill, "rowPadding"] <- rowPadding
					
				}else	idxRowToRepl <- idxRowToMove
				
				# 2) if total should not be included in the header: create new rows to include the header
				if(!isVarTotalNotInSepRow || length(idxRowToRepl) > 0){
					
					# new element:
					# convert to character in case is a factor
					val <- as.character(varX[idxRowToRepl])
					isNAVal <- which(is.na(val))
					if(length(isNAVal) > 0){
						val <- val[-isNAVal]
						idxRowToRepl <- idxRowToRepl[-isNAVal]
					}
					
					# if row to replicate is 'Total' and already replicated in previous iterations (== only first row var have total)
					idxTotal <- which((summaryTable[idxRowToRepl, rowVarFinal] == "Total") & (val == "Total"))
					if(length(idxTotal) > 0){
					  idxRowToRepl <- idxRowToRepl[-idxTotal]
					  val <- val[-idxTotal]
					}
					
					summaryTable <- summaryTable[sort(c(idxRowToRepl, seq_len(nrow(summaryTable)))), ]
					# fill columns
					idxRowToRepl <- idxRowToRepl + seq_along(idxRowToRepl)-1 # indices of replicates rows in new df
					# set var element in final row column
					summaryTable[idxRowToRepl, rowVarFinal] <- val #ifelse(is.na(val) | val == "", "Non specified", val)
					# and set to rest to NA
					summaryTable[idxRowToRepl, !colnames(summaryTable) %in% c(rowVarFinal, rowVarToModify)] <- NA
					# save the padding for flextable
					summaryTable[idxRowToRepl, "rowPadding"] <- rowPadding
					
				}
				
			}
			
			# if only one element in last nested rowVar (e.g. only one Statistic, or counts with categories)
			# merge with previous row (concatenate values)
			if(rowAutoMerge && statsLayout != "rowInSepCol" && (length(statsVar) > 1 | "variableGroup" %in% rowVar)){
				idxNARVF <- which(!is.na(summaryTable$rowVarFinal))
				idxMore2El <- which(diff(idxNARVF) == 1) # consecutive NAs
				if(length(idxMore2El) > 0)
					idxNARVF <- idxNARVF[-unique(c(idxMore2El, idxMore2El+1))]
				# doesn't consider the case if row with total is the first row already
				idxNARVF <- setdiff(idxNARVF, 1)
				if(length(idxNARVF) > 0){
					# get value of only element (next row)
					dataNARVF <- summaryTable[idxNARVF, ] 
					# for final rowVar, concatenate the different values
					dataNARVF[, rowVarFinal] <- paste(
						summaryTable[idxNARVF-1, rowVarFinal], 
						summaryTable[idxNARVF, rowVarFinal]
					)
					# get the padding value of the last row
					dataNARVF[, "rowPadding"] <- summaryTable[idxNARVF-1, "rowPadding"]
					summaryTable[idxNARVF-1, ] <- dataNARVF
					summaryTable <- summaryTable[-idxNARVF, ]
				}
			}
			
			# merge rows, e.g. unique statistic for a subgroup are fill in the sub-group directly
			idxIsNA <- which(is.na(summaryTable[, rowVarFinal]))
			if(length(idxIsNA) > 0){
				
				colContentTable <- setdiff(colnames(summaryTable), c(rowVar, rowVarFinal, "rowVarFinal", "rowPadding"))
				
				# first case: values are the same than in previous row
				# e.g. only total included (e.g. rest is NA)
				idxIsNaSameValueAsPreviousRow <- rowSums(
					summaryTable[idxIsNA-1, colContentTable, drop = FALSE] == 
						summaryTable[idxIsNA, colContentTable, drop = FALSE]
				) == length(colContentTable)
				
				# second case: table content to fill (previous row) is all NA
				idxIsNaButNotContent <- rowSums(is.na(summaryTable[idxIsNA-1, colContentTable, drop = FALSE])) == length(colContentTable)
				
				idxIsNaToRemove <- which(!idxIsNaButNotContent & !idxIsNaSameValueAsPreviousRow)
				
				# don't consider the case that it is NA, e.g. if subgroup in input summaryTable is NA
				if(length(idxIsNaToRemove) > 0)
					idxIsNA  <- idxIsNA[-idxIsNaToRemove]
				if(length(idxIsNA) > 0){
					dataIsNa <- summaryTable[idxIsNA, ]
					dataIsNa[, c("rowPadding", rowVarFinal)] <- summaryTable[idxIsNA-1,  c("rowPadding", rowVarFinal)]
					summaryTable[idxIsNA-1, ] <- dataIsNa
					summaryTable <- summaryTable[-idxIsNA, ] 
				}
			}
			
			summaryTable[, c(rowVarToModify, "rowVarFinal")] <- rownames(summaryTable) <- NULL
			
			# order columns (in case rowVarInSepCol and row stats)
			rowColsFinal <- c(rowVarFinal, rowVarInSepCol)
			summaryTable <- summaryTable[, c(rowColsFinal, setdiff(colnames(summaryTable), rowColsFinal))]
			
			# adjust padding:
			# in case missing element in one of the nested variable
			# padding should be reduced
			summaryTable$rowPadding <- smoothPadding(pad = summaryTable$rowPadding)
			
			# save indices of rows to set padding in flextable
			padParams <- lapply(setdiff(unique(summaryTable$rowPadding), 0), function(pad)
				list(i = which(summaryTable$rowPadding == pad), j = 1, part = "body", padding.left = pad)				
			)
			
		}else	padParams <- list()
		
		## extract extra parameters for flextable (including header)
		
		## extract horizontal lines
		
		# rows with indent set to 0 (and no lines if all rows have no indent)
		idxHLine <- if(length(rowVarToModify) > 0 & !all(summaryTable$rowPadding == 0))	which(summaryTable$rowPadding == 0)-1
		idxHLine <- unique(idxHLine[idxHLine > 0])
		
		# remove lines between rows with same indent (e.g. multiple total rows)
		if(length(rowVarInSepCol) > 0)
			idxHLine <- idxHLine[idxHLine %in% cumsum(rle(summaryTable[, rowVarFinal])$lengths)]
		
		# set the label for the total row (only if total is included for one of the original row total)
		if(!is.null(rowVarTotalInclude) && !all(rowVarTotalInclude %in% "variableGroup")){
			idxRowTotal <- which(summaryTable[, rowVarFinal] == "Total" & summaryTable$rowPadding == 0)
			idxHLine <- c(idxHLine, idxRowTotal[length(idxRowTotal)]) # + include horizontal line after row total
			if(is.null(rowTotalLab)){
				rowTotalLabVars <- rowVarLab[rowVarInRow]
				rowTotalLabVars <- rowTotalLabVars[!is.na(rowTotalLabVars)]
				rowTotalLab <- paste("Any", toString(rowTotalLabVars))
			}
			summaryTable[idxRowTotal, rowVarFinal] <- rowTotalLab
		}
		
		# extract special formatting
		# if rowVar to format has been merged, extract corresponding rows based on padding
		rowVarFormat <- rowVarFormat[names(rowVarFormat) %in% rowVar]
		for(var in names(rowVarFormat)){
			# if variable has been merged, first column and extract row indices based on padding
			if(var %in% rowVarInRow){
				if(length(rowVarToModify) > 0){
					pad <- match(var, rowVarInRow)-1
					i <- which(summaryTable$rowPadding == pad)
				}else{
					i <- seq_len(nrow(summaryTable))
				}
				j <- 1
			# otherwise, extract column idx and consider all rows
			}else{
				j <- match(var, colnames(summaryTable))
				i <- seq_len(nrow(summaryTable))
			}
			formatParams <- c(formatParams, 
				list(list(i = i, j = j, part = "body", type = unname(rowVarFormat[[var]])))
			)
		}	
		
		summaryTable$rowPadding <- NULL
		if(length(idxHLine) > 0)
			hlineParams <- c(hlineParams, list(
				list(i = idxHLine, part = "body", j = seq_len(ncol(summaryTable)))
			))
		for(var in rowVarInSepCol[-length(rowVarInSepCol)]){
			i <- which(diff(as.numeric(factor(summaryTable[, var], exclude = ""))) != 0)
			if(length(i) > 0)
				hlineParams <- c(hlineParams, list(
					list(
						i = i, 
						part = "body", 
						j = seq(from = which(colnames(summaryTable) == var), to = ncol(summaryTable))
					)
				))
		}
		
		# merge rows for rowVarInSepCol
		for(var in rowVarInSepCol){
			varPrev <- intersect(rowVar[seq_len(match(var, rowVar))], colnames(summaryTable))
			varPrevBin <- convertVectToBinary(interactionCustom(summaryTable[, varPrev, drop = FALSE])$x)
			idx <- which(diff(varPrevBin) == 0)
			if(length(idx) > 0){
				idxFact <- cut(seq_along(idx), breaks = c(-Inf, which(diff(idx) != 1), Inf))
				for(l in levels(idxFact)){
					i <- idx[idxFact == l]
					i <- c(i, max(i)+1)
					mergeParams <- c(mergeParams, list(list(
						i = i,
						j = match(var, colnames(summaryTable)),
						part = "body"
					)))
				}
			}
		}
	
		# label header for rows
		rowVarLabs <- c(
			rowVarLab[setdiff(rowVarInRow, "Statistic")], 
			if(statsLayout == "row")	rowVarLab["Statistic"]
		)
		colnames(summaryTable)[match(rowVarFinal, colnames(summaryTable))] <- headerRow <-
			paste(rowVarLabs, collapse = "_")
		colnames(summaryTable)[match(rowVarInSepCol, colnames(summaryTable))] <- headerRowVarInSepCol <- rowVarLab[rowVarInSepCol]
		
	}
	
	# extract header (in case multiple 'colVar' specified)
	header <- sapply(colnames(summaryTable), function(x){
		res <- strsplit(x, split = "_")[[1]]
		# remove empty header - if they are the last element(s) in the column
		# otherwise keep it for alignment of column groups in each header row
		isEmpty <- rev(res) %in% c("", "NA")
		if(all(isEmpty)){
		  res <- ""
		}else if(any(isEmpty)){
		  idxFirst <- min(which(!isEmpty))
		  res <- res[seq.int(from = length(res) - idxFirst + 1)]
		}
		res[which(res == "NA")] <- "" # convert NA -> empty in each column header
		return(res)
	}, simplify = FALSE)
	nRowsHeaderMax <- max(sapply(header, length))
	# if less elements in one column, add missing (header will be top vertically aligned)
	header <- lapply(header, function(x){
	  if(length(x) == 0) x <- ""
	  c(x, rep(NA_character_, nRowsHeaderMax - length(x)))
	})
	headerDf <- do.call(cbind.data.frame, header)
	colnames(headerDf) <- colnames(summaryTable)
	
	emptyRows <- rowSums(is.na(headerDf) | headerDf == "") == ncol(headerDf)
	headerDf <- headerDf[!emptyRows, , drop = FALSE]
	
	# attributes required when converting table to flextable:
	attributes(summaryTable)$summaryTable$header <- headerDf
	
	attributes(summaryTable)$summaryTable$hlineParams <- hlineParams
	attributes(summaryTable)$summaryTable$hline <- hline
	
	if(!is.null(mergeParams))
		attributes(summaryTable)$summaryTable$mergeParams <- mergeParams
	
	if(!is.null(formatParams))
		attributes(summaryTable)$summaryTable$formatParams <- formatParams
	
	# extract vertical lines (specified by the right border)
	if(vline == "auto"){
		vLineParams <- lapply(seq_len(nrow(headerDf)-1), function(i){
			idx <- diff(as.numeric(factor(unlist(headerDf[i, ]), exclude = "")))
			j <- which(idx != 0)
			if(!all(idx == 0, na.rm = TRUE))	list(i = i:nrow(headerDf), part = "header", j = j)
		})
		if(length(vLineParams) > 0)
			vLineParams <- c(vLineParams, list(list(j = vLineParams[[length(vLineParams)]]$j, part = "body")))
	}else	vLineParams <- NULL
	attributes(summaryTable)$summaryTable$vlineParams <- vLineParams
	attributes(summaryTable)$summaryTable$vline <- vline
	
	if(mergeRows){
		
		if(length(headerRow) > 0 && headerRow != ""){
			
			# save padding of header
			idxRowHeaderForPad <- which(headerDf[, headerRow] != "")[-1] # consider header for row column
			padParams <- c(
				padParams,
				lapply(seq_along(idxRowHeaderForPad), function(i)
					list(i = idxRowHeaderForPad[i], j = 1, part = "header", padding.left = i)
				)
			)
			
			attributes(summaryTable)$summaryTable$rowVar <- headerRow
			
		}
		
		if(length(padParams) > 0)
			attributes(summaryTable)$summaryTable$padParams <- padParams
		
	}
	
	
	return(summaryTable)
	
}

#' Smooth padding, e.g. remove padding bigger than 1
#' @param pad Integer vector with padding.
#' @return Integer vector with 'smooth' padding.
#' @author Laure Cougnaud
#' @keywords internal
smoothPadding <- function(pad){
	
	# extract indices in vector which have padding > 1 with previous row
	idxPaddingTooBig <- which(diff(pad) > 1)+1
	while(length(idxPaddingTooBig) > 0){
		# if next rows have same padding, include also their indices
		idxPad <- lapply(idxPaddingTooBig, function(i){
			idxExtra <- which(cumsum(abs(diff(pad[seq(from = i, to = length(pad))]))) == 0) + i
			idx <- c(i, idxExtra)
		})
		# set padding to padding of previous row
		pad[unlist(idxPad)] <- rep(pad[idxPaddingTooBig-1]+1, times = sapply(idxPad, length))
		idxPaddingTooBig <- which(diff(pad) > 1)
	}
	
	return(pad)
	
}

#' Get summary statistics table
#' @inheritParams computeSummaryStatisticsTable
#' @inheritParams exportSummaryStatisticsTable
#' @inheritParams inTextSummaryTable-common-args
#' @inherit exportSummaryStatisticsTable return
#' @author Laure Cougnaud
#' @export
getSummaryStatisticsTable <- function(
	data, 
	## parameters for summary statistics
	var = NULL, varFlag = NULL, varLab = NULL, 
	varLabInclude = length(var) > 1,
	varInclude0 = FALSE,
	varIgnore = NULL,
	varGeneralLab = "Variable", varSubgroupLab = "Variable group",
	varIncludeTotal = FALSE, varTotalInclude = FALSE,
	varTotalInSepRow = FALSE,
	## row variables
	rowVar = NULL, rowVarLab = NULL,
	rowVarDataLevels = NULL, 
	rowOrder = "auto", rowOrderTotalFilterFct = NULL, rowOrderCatLast = NULL,
	rowVarInSepCol = NULL,
	rowVarFormat = NULL,
	# total
	rowVarTotalInclude = NULL, 
	rowVarTotalByVar = NULL,
	rowVarTotalInSepRow = NULL,
	rowTotalLab = NULL,
	rowInclude0 = FALSE, 
	rowAutoMerge = TRUE,
	emptyValue = "-",
	rowVarTotalPerc = NULL,
	## column variable
	colVar = NULL, 
	colVarTotal = colVar, colVarTotalPerc = colVarTotal, 
	colInclude0 = FALSE,
	colVarDataLevels = NULL, 
	colTotalInclude = FALSE, colTotalLab = "Total",
	## stats
	stats = NULL, 
	statsExtra = NULL, 
	statsVarBy = NULL,
	statsPerc = c("statN", "statm"),
	statsGeneralLab = "Statistic",
	statsValueLab = "StatisticValue",
	statsLabInclude = NULL,
	## extra
	subjectVar = "USUBJID",
	filterFct = NULL,
	dataTotal = NULL, dataTotalPerc = dataTotal, 
	dataTotalRow = NULL, dataTotalCol = NULL,
	type = "auto",
	byVar = NULL, byVarLab = NULL,
	checkVarDiffBySubj = "error",
	labelVars = NULL, 
	outputType = "flextable",
	statsLayout = ifelse("DT" %in% outputType, "col", "row"),
	landscape = (style == "presentation"), 	margin = 1, rowPadBase = 14.4, 
	title = NULL, footer = NULL,
	file = NULL,
	style = "report", 
	colorTable = getColorPaletteTable(style = style),
	colHeaderTotalInclude = TRUE, colHeaderMerge = TRUE,
	fontsize = switch(style, 'report' = 8, 'presentation' = 10),
	fontname = switch(style, 'report' = "Times", 'presentation' = "Tahoma"),
	vline = "none", hline = "auto", 
	pageDim = NULL, columnsWidth = NULL,
	# DT
	expandVar = NULL, noEscapeVar = NULL, barVar = NULL,
	...){

	summaryTable <- computeSummaryStatisticsTable(
		data = data,  
		## variables
		var = var, varFlag = varFlag, varLab = varLab, 
		varLabInclude = varLabInclude, varInclude0 = varInclude0,
		varIgnore = varIgnore,
		varGeneralLab = varGeneralLab, varSubgroupLab = varSubgroupLab, 
		varIncludeTotal = varIncludeTotal,
		varTotalInclude = varTotalInclude,
		varTotalInSepRow = varTotalInSepRow,
		## columns
		colVar = colVar, 
		colVarTotal = colVarTotal, colVarTotalPerc = colVarTotalPerc, 
		colInclude0 = colInclude0,
		colVarDataLevels = colVarDataLevels,
		colTotalInclude = colTotalInclude, 
		colTotalLab = colTotalLab,
		## rows
		rowVar = rowVar, rowInclude0 = rowInclude0,
		rowVarDataLevels = rowVarDataLevels,
		rowVarLab = rowVarLab,
		rowOrder = rowOrder, 
		rowOrderTotalFilterFct = rowOrderTotalFilterFct,
		rowOrderCatLast = rowOrderCatLast,
		# total
		rowVarTotalInclude = rowVarTotalInclude,
		rowVarTotalByVar = rowVarTotalByVar,
		rowVarTotalPerc = rowVarTotalPerc,
		rowVarTotalInSepRow = rowVarTotalInSepRow,
		## extra
		type = type,
		subjectVar = subjectVar,
		stats = stats, statsExtra = statsExtra,
		statsPerc = statsPerc,
		statsVarBy = statsVarBy,
		statsGeneralLab = statsGeneralLab,
		filterFct = filterFct,
		dataTotal = dataTotal, dataTotalPerc = dataTotalPerc, 
		dataTotalRow = dataTotalRow, dataTotalCol = dataTotalCol,
		labelVars = labelVars,
		byVar = byVar, byVarLab = byVarLab,
		checkVarDiffBySubj = checkVarDiffBySubj
	)

	if(is.null(summaryTable))	return(invisible())
	ft <- exportSummaryStatisticsTable(
		summaryTable = summaryTable,
		rowVarFormat = rowVarFormat,
		# total
		rowTotalLab = rowTotalLab,
		rowVarInSepCol = rowVarInSepCol,
		rowAutoMerge = rowAutoMerge,
		colHeaderTotalInclude = colHeaderTotalInclude,
		colHeaderMerge = colHeaderMerge,
		statsValueLab = statsValueLab,
		statsLabInclude = statsLabInclude,
		title = title, footer = footer,
		labelVars = labelVars,
		emptyValue = emptyValue,
		file = file, landscape = landscape,
		margin = margin, rowPadBase = rowPadBase,
		outputType = outputType,
		statsLayout = statsLayout,
		expandVar = expandVar, noEscapeVar = noEscapeVar, barVar = barVar,
		style = style, colorTable = colorTable,
		fontsize = fontsize,
		fontname = fontname,
		vline = vline, hline = hline,
		pageDim = pageDim, columnsWidth = columnsWidth,
		...
	)
	
	return(ft)

}

#' Compute summary statistics of interest of an unique variable of interest.
#' 
#' Additionally, this function run extra checks on the data:
#' \itemize{
#' \item{an error message is triggered if any subject (identified by \code{subjectVar}) 
#' have different values in a continuous \code{var}
#' }
#' \item{an indicative message is triggered if multiple but identical records are available
#' for \code{subjectVar} and a continuous \code{var}}
#' } 
#' @param filterEmptyVar Logical, if TRUE doesn't return any results
#' if the variable is empty, otherwise return 0 for the counts and NA for summary 
#' statistics.
#' Criterias to consider a variable empty are:
#' \itemize{
#' \item{for a continuous variable: all missing (NA)}
#' \item{for a categorical variable: all missing or **category is included in the 
#' factor levels but not available in \code{data}**}
#' }
#' By default, an empty variable are filtered.
#' @param varTotalInclude Logical (FALSE by default)
#' Should the total across all categories of \code{var} 
#' be included for the count table?
#' Only used if \code{var} is a categorical variable.
#' @param msgLabel (optional) String with label for the data (NULL by default), 
#' included in the message/warning for checks.
#' @param msgVars (optional) Character vector with columns of \code{data}
#' containing extra variables (besides \code{var} and \code{subjectVar})
#' that should be included in the message/warning for checks.
#' @param checkVarDiffBySubj String, 'error' (default), 'warning',
#' or 'none'.  
#' Should an error, a warning, or nothing be produced
#' if a continuous variable (\code{var}) contains
#' different values for the same subject?
#' @inheritParams inTextSummaryTable-common-args
#' @return Data.frame with summary statistics in columns,
#' depending if \code{type} is:
#' \itemize{
#' \item{'summary': 
#' \itemize{
#' \item{'statN': number of subjects}
#' \item{'statm': number of records}
#' \item{'statMean': mean of \code{var}}
#' \item{'statSD': standard deviation of \code{var}}
#' \item{'statSE': standard error the mean of \code{var}}
#' \item{'statMedian': median of \code{var}}
#' \item{'statMin': minimum of \code{var}}
#' \item{'statMax': maximum of \code{var}}
#' }
#' }
#' \item{'count': 
#' \itemize{
#' \item{'variableGroup': factor with groups of \code{var} for which counts are reported}
#' \item{'statN': number of subjects}
#' \item{'statm': number of records}
#' }
#' }
#' }
#' @author Laure Cougnaud
#' @importFrom stats na.omit median sd
#' @importFrom methods formalArgs
#' @importFrom utils capture.output hasName
#' @export
computeSummaryStatistics <- function(data, 
	var = NULL, varTotalInclude = FALSE,
	statsExtra = NULL,
	subjectVar = "USUBJID",
	filterEmptyVar = TRUE,
	type = "auto",
	checkVarDiffBySubj = c("error", "warning", "none"),
	msgLabel = NULL, msgVars = NULL){
	
	checkVarDiffBySubj <- match.arg(checkVarDiffBySubj)
	
	if(class(data)[1] != "data.frame")
		data <- as.data.frame(data)

	## checks parameters
	
	type <- match.arg(type, choices = c("auto", "summaryTable", "countTable"))
	
	if(!is.null(var) && var != "all" && !var %in% colnames(data))
		stop(paste("Variable to summarize:", shQuote(var), "is not available in data."))
	
	if(!subjectVar %in% colnames(data))
		stop(paste("Subject variable:", shQuote(subjectVar), 
			"is not available for the computation of the number of subjects."))
	
	if(type == "auto")
		type <- ifelse(
			!is.null(var) &&  var != "all" && is.numeric(data[, var]), 
			"summaryTable", 
			"countTable"
		)
	
	if(type == "summaryTable"){
		if(is.null(var)){
			stop("Variable of interest should be specified via the 'var' parameter for a summary table.")
		}else if(var == "all"){
			stop("The 'type' should be set to 'countTable' in case 'var' is 'all'.")
		}else	if(!is.numeric(data[, var])){
			stop("Variable of interest: 'var' should be numeric in case type is set to 'summaryTable'.")
		}
	}
	
	isVarSpec <- (!is.null(var) && var != "all")
	
	if(isVarSpec)
		data <- data[!is.na(data[, var]), , drop = FALSE]
	
	getNSubjects <- function(x){
		if(isVarSpec && any(is.na(x[, subjectVar])))
			warning(paste(
				"Missing records (NA) in:", shQuote(subjectVar),
				"are not considered for subject counts,",
				"please note that this can results in discrepancies",
				"between summary statistics of", shQuote(var),
				"and subject counts."
			))
		as.integer(length(unique(na.omit(x[, subjectVar]))))
	}
	getNRecords <- function(x) nrow(x)
	
	# wrapper to add extra statistics
	statsExtraFct <- function(res, statsExtra, val = NULL, data){
		# Note: error with: 'cbind' if res is empty
		# so only add stats if some results are available
		if(!is.null(statsExtra) & nrow(res) > 0){
			
			if(!is.list(statsExtra))
				stop("'statsExtra' should be a list.")
			
			if(is.null(names(statsExtra)) || !all(names(statsExtra) != ""))
				stop("'statsExtra' should be named.")
			
			statsExtraCommon <- intersect(names(statsExtra), colnames(res))
			if(length(statsExtraCommon) > 0)
				stop("Name(s) in 'statsExtra': ", toString(statsExtraCommon), 
					" are already in default statistics, please choose a different name for your custom statistics.")
			resExtra <- sapply(statsExtra, function(fct){
				switch(
					formalArgs(fct)[1], 
					'x' = fct(val), 
					'data' = fct(data),
					stop("'statsExtra' should contain a parameter named 'x' or 'data'.")
				)
			}, simplify = FALSE)	
			
			res <- cbind(res, resExtra, stringsAsFactors = TRUE)
		}
		return(res)
	}
	
	switch(type,
			
		'summaryTable' = {
				
			val <- data[, var]
			emptyVar <- is.null(val) || length(val) == 0
			if(!(filterEmptyVar & emptyVar)){
				
				# check of multiple records per subject
				if(!emptyVar){
					
					# filter records with same values per subject:
					dataSubjVar <- data[, c(subjectVar, var)]
					isDuplSubjVar <- duplicated(dataSubjVar)
					if(any(isDuplSubjVar) > 0){
						dataDupl <- merge(data, data[isDuplSubjVar, c(subjectVar, var), drop = FALSE])
						dataDupl <- dataDupl[, unique(c(subjectVar, msgVars, var)), drop = FALSE]
						message(sum(isDuplSubjVar), " record(s) with duplicated values",
							" for ", var, 
							" are filtered before the computation of the statistics ", 
							if(!is.null(msgLabel))	paste("for the", msgLabel), ":\n",
							paste(capture.output(print(dataDupl)), collapse = "\n")
						)
						data <- data[!isDuplSubjVar, ]
						val <- data[, var]
					}
					
					isDupl <- duplicated(data[, subjectVar])
					if(any(isDupl) & checkVarDiffBySubj != "none"){
						dataDupl <- merge(data, unique(data[isDupl, subjectVar, drop = FALSE]))
						dataDupl <- dataDupl[, unique(c(subjectVar, msgVars, var)), drop = FALSE]
						msgDupl <- paste0(
							switch(checkVarDiffBySubj,
								`error` = "Extraction of statistics failed because multiple", 
								`warning` = "Multiple"
							),
							" different records of ", var, 
							if(!is.null(msgLabel))	paste(" for the", msgLabel), 
							" are available for ",
							"the same ", subjectVar, ":\n",
							paste(capture.output(print(dataDupl)), collapse = "\n")
						)
						switch(
							checkVarDiffBySubj,
							`error` = stop(msgDupl),
							`warning` = warning(msgDupl)
						)
					}
				}
				
				res <- data.frame(
					statN = getNSubjects(data),
					statm = getNRecords(data),
					statMean = ifelse(emptyVar, NA_real_, mean(val)),
					statSD = ifelse(emptyVar, NA_real_, sd(val)),
					statSE = ifelse(emptyVar, NA_real_, se(val)),
					statMedian = ifelse(emptyVar, NA_real_, median(val)),
					statMin = ifelse(emptyVar, NA_real_, min(val)),
					statMax = ifelse(emptyVar, NA_real_, max(val))
				)
				res <- statsExtraFct(
					res = res, statsExtra = statsExtra, 
					val = val, data = data
				)
				
			}else	res <- NULL
			
		},
		
		'countTable' = {
				
			if(isVarSpec){
				
				varLevels <- if(is.factor(data[, var]))	levels(data[, var])	else	unique(data[, var])
				resList <- lapply(varLevels, function(level){
					x <- data[which(data[, var] == level), ]	
					# compute stats in data or if filterEmptyVar is FALSE
					if(!(nrow(x) == 0 & filterEmptyVar)){
						res <- setNames(
							data.frame(level, getNSubjects(x), getNRecords(x), stringsAsFactors = FALSE),
							c(var, "statN", "statm")
						)
						res <- statsExtraFct(
							res = res, 
							statsExtra = statsExtra,
							data = x
						)
					}
				})
				resList <- resList[!sapply(resList, is.null)]
				if(length(resList) > 0){
					res <- do.call(rbind, c(resList, stringsAsFactors = FALSE))
					res[, var] <- factor(res[, var], levels = varLevels)
					rownames(res) <- NULL
				}else{
					res <- data.frame()
				}
				
			}else{
				res <- data.frame(statN = getNSubjects(data), statm = getNRecords(data))
				res <- statsExtraFct(res = res, statsExtra = statsExtra, data = data)
			}
				
			includeTotal <- varTotalInclude & 
				(is.null(var) || var != "all")
			if(includeTotal){
				
				listTotal <- c(
					setNames(list("Total"), var),
					list(
						statN = getNSubjects(data),
						statm = getNRecords(data)
					)
				)
				resTotal <- do.call(data.frame, listTotal)
				resTotal <- statsExtraFct(res = resTotal, statsExtra = statsExtra, data = data)
				res <- rbind.fill(res, resTotal)
				elVar <- if(is.factor(data[, var]))	levels(data[, var])	else	unique(data[, var])
				res[, var] <- factor(res[, var], levels = c(elVar, "Total"))
				
			}
			
			if(isVarSpec && var %in% colnames(res))
				colnames(res)[match(var, colnames(res))] <- "variableGroup"
			
		}
	
	)
	
	return(res)
	
}

context("Compute summary statistics")

test_that("The statistics are correctly computed for a continuous variable (including missing)", {
	
	dataCont <- data.frame(
		x = c(NA, 1, 3, 6, 10), 
		USUBJID = seq.int(5)
	)
	statsCont <- computeSummaryStatistics(data = dataCont, var = "x")
	
	expect_s3_class(statsCont, "data.frame")
	
	statsContNameInt <- c("statN", "statm")
	statsContNameNum <- c("statMean", "statSD", "statSE", "statMedian", "statMin", "statMax")
	expect_named(
		object = statsCont, 
		expected = c(statsContNameInt, statsContNameNum),
		ignore.order = TRUE
	)
	
	# check variable type
	for(col in statsContNameInt){
		expect_type(statsCont[, !!col], type = "integer")
	}
	for(col in statsContNameNum){
		expect_type(statsCont[, !!col], type = "double")
	}
	
	# check if computations are correct
	expect_equal(statsCont[, "statN"], 4)
	expect_equal(statsCont[, "statm"], 4)	
	expect_equal(statsCont[, "statMean"], 5)
	expect_equal(statsCont[, "statMedian"], 4.5)
	expect_equal(statsCont[, "statSD"], sd(dataCont$x, na.rm = TRUE))
	expect_equal(statsCont[, "statSE"], statsCont[, "statSD"]/sqrt(statsCont[, "statm"]))
	expect_equal(statsCont[, "statMin"], 1)
	expect_equal(statsCont[, "statMax"], 10)
	
})

test_that("The statistics are correctly computed for a categorical variable (including missing)", {
			
	dataCat <- data.frame(
		x = c(NA_character_, "B", "B", "B", "A"), 
		USUBJID = seq.int(5),
		stringsAsFactors = TRUE
	)
	statsCat <- computeSummaryStatistics(data = dataCat, var = "x")
			
	expect_s3_class(statsCat, "data.frame")
			
	expect_named(
		object = statsCat, 
		expected = c("variableGroup", "statN", "statm"),
		ignore.order = TRUE
	)
			
	expect_s3_class(statsCat[, "variableGroup"], "factor")
	expect_identical(statsCat[, "variableGroup"], factor(c("A", "B")))
	
	expect_equal(statsCat[, "statm"], c(1, 3))
	expect_equal(statsCat[, "statN"], c(1, 3))

})


test_that("Counts are computed on the entire dataset if the variable is not specified", {
			
	data <- data.frame(
		x = c("A", "B"), 
		USUBJID = c("1", "2")
	)
	expect_equal(
		computeSummaryStatistics(data = data, var = NULL), 
		data.frame(statN = 2, statm = 2)
	)
	
})

test_that("Counts are computed on the entire dataset if the variable is set to 'all'", {
			
	data <- data.frame(
		x = c("A", "B"), 
		USUBJID = c("1", "2")
	)
	expect_equal(
		computeSummaryStatistics(data = data, var = "all"), 
		data.frame(statN = 2, statm = 2)
	)
	
})

test_that("Counts are computed similarly for empty variable or variable set to 'all'", {
			
	data <- data.frame(
		x = c("A", "B"), 
		USUBJID = c("1", "2")
	)
	expect_equal(
		computeSummaryStatistics(data = data, var = "all"), 
		computeSummaryStatistics(data = data, var = NULL)
	)
	
})

test_that("An error is generated if a specified variable is not available in the data", {
			
	data <- data.frame(
		x = c("A", "B"), 
		USUBJID = c("1", "2")
	)
	expect_error(
		computeSummaryStatistics(data = data, var = "y"),
		"Variable to summarize.*not available in data."
	)
	
})


test_that("An error is generated if the subject variable is not available in the data", {
	expect_error(
		computeSummaryStatistics(
			data = data.frame(x = 1:10), 
			var = "x", 
			subjectVar = "blabla"
		),
		pattern = "Subject variable .* not available"
	)		
})

test_that("An error is generated if multiple records are available per subject", {
	dataCont <- data.frame(x = seq.int(5), USUBJID = c(1, 1, 2, 2, 2))
	expect_error(
		computeSummaryStatistics(data = dataCont, var = "x"),
		pattern = "Extraction of statistics failed .* because multiple different records are available"
	)		
})

test_that("A warning is generated if multiple records are available per subject when requested", {
	dataCont <- data.frame(x = seq.int(5), USUBJID = c(1, 1, 2, 2, 2))
	expect_warning(
		computeSummaryStatistics(data = dataCont, var = "x", checkVarDiffBySubj = "warning"),
		pattern = "multiple different records are available"
	)		
})

test_that("No check is run for multiple records available per subject if requested", {
	dataCont <- data.frame(x = seq.int(5), USUBJID = c(1, 1, 2, 2, 2))
	expect_silent(
		computeSummaryStatistics(data = dataCont, var = "x", checkVarDiffBySubj = "none")
	)		
})

test_that("The variable total is correctly included when requested", {
		
	dataCat <- data.frame(
		x = c(NA_character_, "B", "B", "B", "A"), 
		USUBJID = seq.int(5)
	)
	statsCatWithTotal <- computeSummaryStatistics(
		data = dataCat, var = "x", varTotalInclude = TRUE
	)
	statsCatWthtTotal <- computeSummaryStatistics(
		data = dataCat, var = "x", varTotalInclude = FALSE
	)
	statsCatIncludeTotal <- rbind(
		statsCatWthtTotal, 
		data.frame(variableGroup = "Total", statN = 4, statm = 4)
	)
	
	expect_equal(
		object = statsCatWithTotal, 
		expected = statsCatIncludeTotal
	)
			
})

test_that("An error is generated if the type of the extra statistic is not correct", {
			
	dataCont <- data.frame(
		x = c(NA, 1, 3, 6, 10), 
		USUBJID = seq.int(5)
	)
	expect_error(
		computeSummaryStatistics(
			data = dataCont, 
			var = "x", 
			statsExtra = function(x) x
		),
		"should be a list",
	)
	
})

test_that("An error is generated if the parameter of the extra statistic function is not correct", {
			
	dataCont <- data.frame(
		x = c(NA, 1, 3, 6, 10), 
		USUBJID = seq.int(5)
	)	
	expect_error(
		computeSummaryStatistics(
			data = dataCont, 
			var = "x", 
			statsExtra = list(A = function(y) length(y))
		),
		"'statsExtra' should contain a parameter named 'x' or 'data'"
	)
			
})

test_that("An error is generated if no names are specified for the extra statistics", {
			
	dataCont <- data.frame(
		x = c(NA, 1, 3, 6, 10), 
		USUBJID = seq.int(5)
	)
	
	# create example of custom function (coefficient of variation)
	statsFctX <- function(x) sd(x, na.rm = TRUE)/mean(x, na.rm = TRUE)*100	
	
	expect_error(
		computeSummaryStatistics(data = dataCont, var = "x", statsExtra = list(statsFctX)),
		"should be named"
	)
	expect_error(
		computeSummaryStatistics(data = dataCont, var = "x", statsExtra = list(A = statsFctX, 2)),
		"should be named"
	)
	
})
	
test_that("An error is generated if the name of the extra statistic is set to a default statistic", {
				
	dataCont <- data.frame(
		x = c(NA, 1, 3, 6, 10), 
		USUBJID = seq.int(5)
	)
	
	# create example of custom function (coefficient of variation)
	statsFctX <- function(x) sd(x, na.rm = TRUE)/mean(x, na.rm = TRUE)*100	
	
	expect_error(
		computeSummaryStatistics(
			data = dataCont, 
			var = "x", 
			statsExtra = list(statN = statsFctX)
		),
		"choose a different name"
	)
	
})
	
test_that("Extra statistic is correctly included when specified as a function of a vector", {
	
	dataCont <- data.frame(
		x = c(NA, 1, 3, 6, 10), 
		USUBJID = seq.int(5)
	)
	
	# create example of custom function (coefficient of variation)
	statsFctX <- function(x) sd(x, na.rm = TRUE)/mean(x, na.rm = TRUE)*100	
	
	statsExtraFctX <- computeSummaryStatistics(
		data = dataCont, 
		var = "x", 
		statsExtra = list(A = statsFctX)
	)
	expect_true("A" %in% names(statsExtraFctX))
	expect_equal(statsExtraFctX$A, statsFctX(dataCont$x))
	
})
	
test_that("Extra statistic is correctly included when specified as a function of the data", {
				
	dataCont <- data.frame(
		x = c(NA, 1, 3, 6, 10), 
		USUBJID = seq.int(5)
	)
	
	# create example of custom function (coefficient of variation)
	statsFctData <- function(data) sd(data$x, na.rm = TRUE)/mean(data$x, na.rm = TRUE)*100
	
	statsExtraFctData <- computeSummaryStatistics(
		data = dataCont, 
		var = "x", 
		statsExtra = list(B = statsFctData)
	)
	expect_true("B" %in% names(statsExtraFctData))
	expect_equal(statsExtraFctData$B, statsFctData(dataCont))
	
})

test_that("An error is generated if the type of table is continuous whereas the variable is categorical", {
		
	dataCat <- data.frame(
		x = c(NA_character_, "B", "B", "B", "A"), 
		USUBJID = seq.int(5)
	)
	expect_error(
		computeSummaryStatistics(
			data = dataCat, 
			var = "x", 
			type = "summaryTable"
		),
		"should be numeric"
	)
	
})

test_that("The type of table is correct for a categorical variable by default (or auto)", {
			
	dataCat <- data.frame(
		x = c(NA_character_, "B", "B", "B", "A"), 
		USUBJID = seq.int(5)
	)
	expect_identical(
		computeSummaryStatistics(data = dataCat, var = "x"),
		computeSummaryStatistics(data = dataCat, var = "x", type = "countTable")
	)
	expect_identical(
		computeSummaryStatistics(data = dataCat, var = "x", type = "auto"),
		computeSummaryStatistics(data = dataCat, var = "x", type = "countTable")
	)
	
})

test_that("The type of table is correct for a continuous variable by default (or auto)", {
			
	dataCont <- data.frame(
		x = c(NA, 1, 3, 6, 10), 
		USUBJID = seq.int(5)
	)
	expect_identical(
		computeSummaryStatistics(data = dataCont, var = "x"),
		computeSummaryStatistics(data = dataCont, var = "x", type = "summaryTable")
	)
	expect_identical(
		computeSummaryStatistics(data = dataCont, var = "x", type = "auto"),
		computeSummaryStatistics(data = dataCont, var = "x", type = "summaryTable")
	)
	
})

test_that("A count table is successfully extracted for a continuous variable", {
			
	dataCont <- data.frame(
		x = c(NA, 1, 3, 6, 10), 
		USUBJID = seq.int(5)
	)
	
	expect_error(
		computeSummaryStatistics(
			data = dataCont, 
			var = "x", 
			type = "countTable"
		), 
		NA
	)
			
})

test_that("An error is generated if the variable is not specified for a continuous table", {
			
	dataCont <- data.frame(
		x = c(1, 3), 
		USUBJID = c("a", "b")
	)
	expect_error(
		computeSummaryStatistics(
			data = dataCont, 
			var = NULL, 
			type = "summaryTable"
		),
		"Variable.*should be specified.*for a summary table"
	)
			
})

test_that("An error is generated if the table type is not count in case the variable is set to 'all'", {
			
	dataCont <- data.frame(
		x = c(1, 3), 
		USUBJID = c("a", "b")
	)
	expect_error(
		computeSummaryStatistics(
			data = dataCont, 
			var = "all", 
			type = "summaryTable"
		),
		"'type' should be set to 'countTable'"
	)
			
})

test_that("A continuous variable with only missing values is correctly filtered by default", {
	
	# variable is considered empty if all missing:
	emptyDataNA <- data.frame(
		x = rep(NA_real_, 5), 
		USUBJID = seq.int(5)
	)	
	expect_null(
		computeSummaryStatistics(data = emptyDataNA, var = "x")
	)
	
})

test_that("A continuous variable with only missing values is correctly included when requested", {
			
	# variable is considered empty if all missing:
	emptyDataNA <- data.frame(
		x = rep(NA_real_, 5), 
		USUBJID = seq.int(5)
	)	
	statsEmptyDataNA <- computeSummaryStatistics(
		data = emptyDataNA, 
		var = "x", 
		filterEmptyVar = FALSE
	)
	expect_equal(
		statsEmptyDataNA, 
		data.frame(
			statN = 0, statm = 0, statMean = NA_real_, 
			statSD = NA_real_, statSE = NA_real_, statMedian = NA_real_, 
			statMin = NA_real_, statMax = NA_real_
		)
	)

})	
	
test_that("An empty continuous variable is correctly included when requested", {

	emptyData <- data.frame(x = numeric(), USUBJID = character())
	statsEmptyData <- computeSummaryStatistics(
		data = emptyData, 
		var = "x", 
		filterEmptyVar = FALSE
	)
	expect_equal(
		statsEmptyData, 
		data.frame(
			statN = 0, statm = 0, statMean = NA_real_, 
			statSD = NA_real_, statSE = NA_real_, statMedian = NA_real_, 
			statMin = NA_real_, statMax = NA_real_
		)
	)
	
})	

test_that("Missing values are correctly filtered from a categorical variable with only missing values by default", {
			
	dataCat <- data.frame(
		x = factor(
			c(NA_character_, "B", "B", "B", "A"), 
			levels = c("A", "B", "C")
		),
		USUBJID = seq.int(5)
	)
	
	sumTable <- computeSummaryStatistics(
		data = dataCat, 
		var = "x", 
		filterEmptyVar = TRUE
	)
	expect_equal(sumTable$variableGroup, factor(c("A", "B"), levels = c("A", "B", "C")))
	
})

test_that("Missing values are correctly included from a categorical variable when requested", {

	dataCat <- data.frame(
		x = factor(
			c(NA_character_, "B", "B", "B", "A"), 
			levels = c("A", "B", "C")
		),
		USUBJID = seq.int(5)
	)
	statsCatNoFilter <- computeSummaryStatistics(data = dataCat, var = "x", filterEmptyVar = FALSE)
	expect_equal(nrow(statsCatNoFilter), 3)
	expect_true("C" %in% statsCatNoFilter$variableGroup)
	expect_equal(subset(statsCatNoFilter, variableGroup == "C")$statN, 0)
	expect_equal(subset(statsCatNoFilter, variableGroup == "C")$statm, 0)
	
})

test_that("An empty summary table is correctly returned by default if the input data is empty", {
		
	emptyData <- do.call(
		data.frame,
		list(
			x = factor(character(), levels = c("a", "b")),
			USUBJID = character()
		)
	)
	
	# by default, nothing is returned
	expect_equal(nrow(computeSummaryStatistics(data = emptyData, var = "x")), 0)
	
})	

test_that("A count table is correctly returned if the input data is empty when empty variable should be included", {
	
	# if variable should not be filtered, 0 counts are returned
	emptyData <- do.call(
		data.frame,
		list(
			x = factor(character(), levels = c("a", "b")),
			USUBJID = character()
		)
	)
	expect_equal(
		computeSummaryStatistics(data = emptyData, var = "x", filterEmptyVar = FALSE), 
		data.frame(variableGroup = c("a", "b"), statN = c(0, 0), statm = c(0, 0), stringsAsFactors = TRUE)
	)
	
})

test_that("A count table is correctly returned if the input data is empty when the empty variable and total of the variable are requested", {
			
	emptyData <- do.call(
		data.frame,
		list(
			x = factor(character(), levels = c("a", "b")),
			USUBJID = character()
		)
	)	
	expect_equal(
		computeSummaryStatistics(
			data = emptyData, 
			var = "x", 
			filterEmptyVar = FALSE, 
			varTotalInclude = TRUE
		), 
		data.frame(
			variableGroup = factor(
				c("a", "b", "Total"), 
				levels = c(c("a", "b", "Total"))
			),
			statN = c(0, 0, 0), 
			statm = c(0, 0, 0)
		)
	)
	
})
	
test_that("A count table is correctly returned if the input data is empty when total of the variable is requested", {
				
	emptyData <- do.call(
		data.frame,
		list(
			x = factor(character(), levels = c("a", "b")),
			USUBJID = character()
		)
	)	
	expect_equal(
		computeSummaryStatistics(
			data = emptyData, 
			var = "x", 
			varTotalInclude = TRUE
		), 
		data.frame(variableGroup = factor("Total", c("a", "b", "Total")),statN = 0, statm = 0)
	)
			
})

test_that("An error is generated if a continuous variable contains different values for the same subject", {
			
	dataCont <- data.frame(
		x = c(NA, 1, 3, 6, 10), 
		y = rep(2, 5),
		USUBJID = c("A", "B", "B", "C", "D")
	)
	expect_error(
		computeSummaryStatistics(data = dataCont, var = "x"),
		"multiple different records of x are available for the same USUBJID"
	)
	
})

test_that("An error is generated containing a specified variable if a continuous variable contains different values for the same subject", {
			
	dataCont <- data.frame(
		x = c(NA, 1, 3, 6, 10), 
		y = rep(2, 5),
		USUBJID = c("A", "B", "B", "C", "D")
	)
	
	expect_error(
		computeSummaryStatistics(data = dataCont, var = "x", msgVars = "y"),
		"multiple different records of x are available for the same USUBJID.* USUBJID y x"
	)
	
})

test_that("An error is generated containing a specified label if a continuous variable contains different values for the same subject", {
			
	dataCont <- data.frame(
		x = c(NA, 1, 3, 6, 10), 
		USUBJID = c("A", "B", "B", "C", "D")
	)
	
	expect_error(
		computeSummaryStatistics(
			data = dataCont, 
			var = "x", 
			msgLabel = "test dataset"
		),
		"multiple different records of x for the test dataset are available for the same USUBJID"
	)
			
})

test_that("A message is generated if a continuous variable contains duplicated values for the same subject", {
			
	dataCont <- data.frame(
		x = c(NA, 1, 1, 6, 10), 
		USUBJID = c("A", "B", "B", "C", "D")
	)
			
	expect_message(
		computeSummaryStatistics(data = dataCont, var = "x"),
		"duplicated values for x"
	)
	
})

test_that("A warning is generated if some subject IDs are missing and a variable is specified", {

	data <- data.frame(USUBJID = c(NA_character_, 1), x = 1:2)
	expect_warning(
		summaryTable <- computeSummaryStatistics(data = data, var = "x"), 
		regexp = "Missing records.*are not considered for subject counts"
	)
			
})

#' Create a data.frame combining a set of records from same or multiple variables.
#' 
#' This typically converts the data from a wide to a long format.
#' For each variable, a subset of interest based on a condition can be specified.
#' @param paramsList nested list of parameters, 
#' specifying how the records of interest should be selected.\cr
#' There are two ways to select a subset of interest:
#' \itemize{
#' \item{by specifying one unique variable of interest with: 
#' \itemize{
#' \item{\code{var}: string with column of \code{data} of interest}
#' \item{\code{value}: value of \code{var} of interest (only used if \code{var} is specified).\cr
#' If not specified only the values different than NA and '' are considered.}
#' \item{\code{fctTest}: string with name or directly comparison function
#' to apply on \code{var} to select subset of interest versus \code{value}.\cr
#' The function should take \code{var} as first parameter and \code{value} to compare to
#' as second parameter and returns a logical vector with TRUE or FALSE (of length \code{var})
#' if the condition is fullfilled.\cr
#' If not specified, the records with \code{var} equal to \code{value} are retained 
#' (\code{fctTest} is set to '==').} 
#' \item{label specification: 
#' \itemize{
#' \item{\code{label}: string with label for the condition,
#' includde in the new 'variable' column.\cr
#' If not specified and:
#' \itemize{
#' \item{\code{var} is specified: label is extracted from 
#' \code{labelVars} if available or set to \code{var} otherwise.}
#' \item{\code{var} is not specified: label should be specified.}
#' }}
#' \item{\code{labelExtra}: string with extra label, will be concatenated with \code{label}}
#' }}
#' }}
#' 
#' \item{by specifying a combination of variable of interest with: 
#' \itemize{
#' \item{\code{exprs}: string with expression of columns of \code{data} to select subset of interest}
#' \item{\code{label}: string with complete label for the group}
#' }}
#' 
#' }
#' @param newVar String with name of new variable to construct.
#' @param fctTest Default function to use to compare \code{var} and \code{value} specified
#' in each sublist of \code{paramsList}.\cr
#' This is only used if \code{fctTest} is not specified in each sublist.
#' @param includeAll Logical, if TRUE (FALSE by default) include also the entire data as an additional subgroup.
#' @param labelAll String of group label for the entire data in case \code{includeAll} is TRUE.
#' @inheritParams inTextSummaryTable-common-args
#' @return Data.frame with records from \code{data} extracted based on
#' the different conditions specified in \code{paramsList}.\cr
#' This data.frame contains an additional variable 
#' (labelled based on \code{newVar}) mentioning the
#' specific condition for which the record was extracted
#' (based \code{label}, \code{labelExtra}, \code{labelVars}).\cr
#' This variable is a factor whose levels are ordered based on the order
#' of the condition specified in \code{paramsList}.
#' @author Laure Cougnaud
#' @importFrom clinUtils getLabelVar
#' @export
combineVariables <- function(
	data, paramsList, newVar, 
	labelVars = NULL, fctTest = "==",
	includeAll = FALSE, labelAll = "Any"){

	if(class(data)[1] != "data.frame")
		data <- as.data.frame(data)
	
	if(!all(sapply(paramsList, is.list)))
		stop("'paramsList' should be a nested list.")

	# extract label used for new variable for each subset of interest
	paramsLabel <- sapply(seq_along(paramsList), function(i){
		x <- paramsList[[i]]
		label <- if(!is.null(x$var)){
			if(!is.null(x[["label"]]))	x[["label"]]	else	getLabelVar(x$var, data, labelVars)
		}else{
			if(!is.null(x[["label"]]))	x[["label"]]	else
				stop("Label should be specified for parameter: '", i, "'.")
		}
		if(!is.null(x[["labelExtra"]]))	label <- paste(label, x[["labelExtra"]])
		label
	})
	if(any(duplicated(paramsLabel)))
		stop("Duplicated labels in the specified parameter list.")
	
	# extract each subset of interest
	dataList <- lapply(seq_along(paramsList), function(i){
		x <- paramsList[[i]]
		dataRetained <- if(!is.null(x$exprs)){
			rowsRetained <- eval(expr = parse(text = x$exprs), envir = data)
			data[rowsRetained, ]
		}else{
			if(!is.null(x$var)){
				if(!is.null(x$value)){
					fct <- if(!is.null(x$fctTest))	x$fctTest	else	fctTest
					data[match.fun(fct)(data[, x$var], x$value), ]
				}else{
					xVar <- data[, x$var]
					data[!is.na(xVar) & xVar != "", ]
				}
			}else stop("Expression ('exprs') or a variable",
				" of interest ('var') should be specified",
				" for the parameter: ", i, ".")
		}
		if(nrow(dataRetained) > 0)
			dataRetained[, newVar] <- paramsLabel[i]
		dataRetained
	})

	# combine all data
	dataRetained <- do.call(rbind, dataList)

	if(includeAll){
		dataRetained <- rbind(
			cbind(data, setNames(list(labelAll), newVar), stringsAsFactors = TRUE), 
			dataRetained, stringsAsFactors = TRUE
		)
		paramsLabel <- c(labelAll, paramsLabel)
	}

	dataRetained[, newVar] <- factor(
		if(nrow(dataRetained) == 0)	character()	else	dataRetained[, newVar]
	, levels = paramsLabel)

	return(dataRetained)

}

context("Combine variables")

test_that("An error is generated if the variable is not correctly specified", {
			
	expect_error(
		combineVariables(
			data = data,
			paramsList = list(var = "AFL"),
			newVar = "variable"
		),
		".*paramsList.* should be a nested list.*"
	)
			
})

test_that("An error is generated if no variable or expression is specified", {
			
	expect_error(
		combineVariables(
			data = data,
			paramsList = list(list(label = "test")),
			newVar = "variable"
		),
		".*variable.* should be specified.*"
	)
			
})

test_that("The data is correctly extracted based on a specified variable", {
			
	data <- data.frame(
		ID = seq.int(4),
		AFL = c(NA_character_, "N", "", "N")
	)	
	
	sumTable <- combineVariables(
		data = data,
		paramsList = list(list(var = "AFL")),
		newVar = "variable"
	)
	expect_equal(
		sumTable,		
		cbind.data.frame(
			subset(data, !is.na(AFL) & AFL != ""),
			variable = factor("AFL", levels = "AFL")
		)
	)
	
})

test_that("The data is correctly extracted based on a value of a specified variable", {
	
	data <- data.frame(
		ID = seq.int(4),
		AFL = c("Y", "N", "Y", "N")
	)
	sumTable <- combineVariables(
		data = data,
		paramsList = list(list(var = "AFL", value = "Y")),
		newVar = "variable"
	)
	expect_equal(
		sumTable,		
		cbind.data.frame(
			subset(data, AFL == "Y"),
			variable = factor("AFL", levels = "AFL")
		)
	)
			
})

test_that("The data is extracted by default based on the equality if a value of a variable is specified", {
		
	data <- data.frame(
		ID = seq.int(4),
		AVAL = c(1, 2, 3, 4)
	)
	expect_identical(
		combineVariables(data = data,
			paramsList = list(list(var = "AVAL", value = 1, fctTest = "==")),
			newVar = "variable"
		),
		combineVariables(data = data,
			paramsList = list(list(var = "AVAL", value = 1)),
			newVar = "variable"
		)
	)
	
})

test_that("The data is extracted correctly based on a comparison function specified as a character", {
	
	data <- data.frame(
		ID = seq.int(4),
		AVAL = c(1, 2, 3, 4)
	)
	dataLongFct <- combineVariables(data = data,
		paramsList = list(list(var = "AVAL", value = 2, fctTest = ">")),
		newVar = "variable"
	)
	expect_identical(
		dataLongFct,
		cbind.data.frame(
			subset(data, AVAL > 2),
			variable = "AVAL",
			stringsAsFactors = TRUE
		)
	)

})

test_that("The data is extracted correctly based on a comparison function specified by variable", {

	data <- data.frame(
		ID = seq.int(4),
		AVAL = c(1, 2, 3, 4)
	)
	expect_identical(
		combineVariables(data = data,
			paramsList = list(list(var = "AVAL", value = 2, fctTest = function(x, value) x > value)),
			newVar = "variable"
		),
		cbind.data.frame(
			subset(data, AVAL > 2),
			variable = "AVAL",
			stringsAsFactors = TRUE
		)
	)
	
})

test_that("The data is extracted correctly based on a comparison function specified for all variables", {
	
	data <- data.frame(
		ID = seq.int(4),
		AVAL = c(1, 2, 3, 4)
	)
	expect_identical(
		combineVariables(data = data,
			paramsList = list(list(var = "AVAL", value = 2)),
			newVar = "variable", fctTest = ">"
		),
		cbind.data.frame(
			subset(data, AVAL > 2),
			variable = "AVAL",
			stringsAsFactors = TRUE
		)
	)
	
})

test_that("An error is generated if the comparison function is not a function", {
	
	data <- data.frame(
		ID = seq.int(4),
		AVAL = c(1, 2, 3, 4)
	)
	expect_error(
		combineVariables(data = data,
			paramsList = list(list(var = "AVAL", value = 2, fctTest = "blabla")),
			newVar = "variable"
		)
	)
	
})

test_that("An error is generated if the comparison function doesn't contain parameters to compare", {
			
	data <- data.frame(
		ID = seq.int(4),
		AVAL = c(1, 2, 3, 4)
	)	
	expect_error(
		combineVariables(data = data,
			paramsList = list(list(var = "AVAL", value = 2, fctTest = is.character)),
			newVar = "variable"
		)
	)
			
})

test_that("The data is correctly extracted based on a condition expression of multiple variables", {
			
	data <- data.frame(
		ID =  c(1, 2, 3, 4),
		AVAL = c(4, 3, 2, 1)
	)
	sumTable <- combineVariables(data = data,
		paramsList = list(list(var = "AVAL", exprs = "AVAL <= 3 & ID >= 2")),
		newVar = "variable"
	)
	expect_identical(sumTable[, c("ID", "AVAL")], subset(data, AVAL <= 3 & ID >= 2))
	
})	

test_that("An error is generated if the specification of the condition is incorrect", {
			
	data <- data.frame(
		ID =  c(1, 2, 3, 4),
		AVAL = c(4, 3, 2, 1)
	)
	expect_error(	
		combineVariables(data = data,
			paramsList = list(list(var = "AVAL", exprs = "AVAL is of specific type")),
			newVar = "variable"
		)
	)
			
})

test_that("The data is correctly extracted based on multiple conditions of variables", {
		
	data <- data.frame(
		ID = seq.int(4),
		AVAL = c(1, 2, 3, 4),
		CHG = c(-1, 0, 1, 2)
	)
	
	sumTable <- combineVariables(
		data = data,
		paramsList = list(
			list(var = "CHG", value = 0, label = "Change from baseline positive"),
			list(var = "AVAL", value = 2, label = "Actual value >= 2")
		),
		fctTest = ">=",
		newVar = "variable"
	)
	
	# build reference object
	sumTableCond1 <- cbind.data.frame(
		subset(data, CHG >= 0),
		variable = factor("Change from baseline positive")
	)
	sumTableCond2 <- cbind.data.frame(
		subset(data, AVAL >= 2),
		variable = factor("Actual value >= 2")
	)
	sumTableRef <- rbind(sumTableCond1, sumTableCond2)
	# levels are in the order specified in 'paramsList':
	sumTableRef$variable <- factor(sumTableRef$variable,
		levels = c("Change from baseline positive", "Actual value >= 2"))
	
	expect_equal(object = sumTable, expected = sumTableRef)
			
})

test_that("The variable label is correctly set to the variable name by default", {
			
	data <- data.frame(
		ID = seq.int(4),
		AFL = c("Y", "N", "Y", "N")
	)
	sumTable <- combineVariables(
		data = data,
		paramsList = list(list(var = "AFL", value = "Y")),
		newVar = "variable"
	)
	expect_equal(unique(as.character(sumTable$variable)), "AFL")
	
})

test_that("An error is generated if the variable and label are not specified", {
			
	data <- data.frame(
		ID = seq.int(4),
		AFL = c("Y", "N", "Y", "N")
	)
	expect_error(
		combineVariables(
			data = data,
			paramsList = list(list()),
			newVar = "variable"
		),
		".*Label should be specified"
	)
	
})

test_that("The label of the variable is correctly set when specified", {
			
	data <- data.frame(
		ID = seq.int(4),
		AFL = c("Y", "N", "Y", "N")
	)
	sumTableLabel <- combineVariables(
		data = data,
		paramsList = list(list(var = "AFL", value = "Y", label = "test")),
		newVar = "variable"
	)
	expect_identical(unique(as.character(sumTableLabel$variable)), "test")
	
})

test_that("The label of the variable is correctly extracted from the labels of all variables", {
			
	data <- data.frame(
		ID = seq.int(4),
		AFL = c("Y", "N", "Y", "N")
	)
	sumTableLabel <- combineVariables(
		data = data,
		paramsList = list(list(var = "AFL", value = "Y")),
		newVar = "variable",
		labelVars = c(AFL = "test")
	)
	expect_identical(unique(as.character(sumTableLabel$variable)), "test")
	
})

test_that("An error is generated if the variables have duplicated labels", {
			
	data <- data.frame(
		ID = seq.int(4),
		AFL = c("Y", "N", "Y", "N")
	)
	expect_error(
		combineVariables(
			data = data,
			paramsList = list(
				list(var = "AFL", value = "Y", label = "test 1"),
				list(var = "AFL", value = "N", label = "test 1")
			),
			newVar = "variable"
		),
		"Duplicated labels in the specified parameter list."
	)
			
})

test_that("The extra label of the variable is correctly set when specified", {
			
	data <- data.frame(
		ID = seq.int(4),
		AFL = c("Y", "N", "Y", "N")
	)
	sumTableLabelExtra <- combineVariables(
		data = data,
		paramsList = list(list(var = "AFL", value = "Y", 
			label = "test", labelExtra = "blabla")),
		newVar = "variable"
	)
	expect_identical(unique(as.character(sumTableLabelExtra$variable)), "test blabla")
			
})

test_that("All records are correctly included when requested", {
				
	data <- data.frame(
		ID = seq.int(4),
		AFL = c("Y", "N", "Y", "N")
	)		
	sumTableIncludeAll <- combineVariables(
		data = data,
		paramsList = list(
			list(var = "AFL", value = "Y")
		),
		includeAll = TRUE, newVar = "variable"
	)
	
	sumTableRef <- rbind(
		# entire dataset is returned with 'variable' set to: 'Any'
		cbind.data.frame(data, variable = factor("Any")),
		# the rest of the dataset is based on specified condition in 'paramsList':
		cbind.data.frame(
			subset(data, AFL == "Y"),
			variable = factor("AFL")
		)	
	)
	
	expect_equal(
		object = sumTableIncludeAll, 
		expected = sumTableRef,
		check.attributes = FALSE
	)
			
})

test_that("A label is correctly set to all records when specified", {

	data <- data.frame(
		ID = seq.int(4),
		AFL = c("Y", "N", "Y", "N")
	)		
	sumTableLabelAll <- combineVariables(
		data = data,
		paramsList = list(
			list(var = "AFL", value = "Y")
		),
		newVar = "variable",
		includeAll = TRUE, labelAll = "Entire dataset"
	)
	expect_equal(
		object = subset(sumTableLabelAll, variable == "Entire dataset", select = -variable), 
		expected = data,
		check.attributes = FALSE
	)
			
})
			

#' Arguments used across the functions of the inTextSummaryTable package.
#' @param data Data.frame with dataset to consider for the summary table.
#' @param summaryTable A \code{\link{summaryTable}} object.
#' @param var Character vector with variable(s) of \code{data}, 
#' to compute statistics on.\cr
#' If NULL (by default), counts by row/column variable(s) are computed.\cr
#' To also return counts of the \code{rowVar} in case other \code{var}
#' are specified, you can include: 'all' in the \code{var}.\cr
#' Missing values, if present, are filtered 
#' (also for the report of number of subjects/records).
#' @param varFlag Character vector, subset of \code{var} with variable(s) 
#' of type 'flag' (with 'Y', 'N' or '' for empty/non specified value).
#' Only the counts for records flagged (with 'Y') are retained.
#' @param varLabInclude Logical, if TRUE
#' the name of the summary statistic variable(s) (\code{var})
#' are included in the table.
#' This is automatically set to TRUE if more than one variable(s) 
#' and is specified, and FALSE if only one variable is specified.
#' @param rowVar Character vector with variable(s)
#' to be included in the rows.
#' If multiple variables are specified, the variables should 
#' be sorted in hierarchical order
#' (e.g. body system class before adverse event term)
#' and are nested in the table.
#' @param rowVarInSepCol Character vector with \code{rowVar}
#' that should be included in separated columns.
#' By default (NULL), all row variables are nested in the first column 
#' of the table.\cr
#' To include the groups within a \code{var} variable in a separated column, 
#' set: \code{rowVarInSepCol == 'variableGroup'}.
#' @param rowVarLab Named character vector with 
#' label for the \code{rowVar} variable(s).
#' @param statsVar Character vector with columns of \code{summaryTable} with
#' statistic variables. For the export: if not specified, all columns of \code{data}
#' besides row, column variables, 'variable', 'variableGroup'
#' and 'isTotal' are considered.
#' @param rowVarTotalInclude Character vector with \code{rowVar}
#' for which the total should be reported.\cr
#' If the higher row variable is specified, the total across all rows
#' is reported. \cr
#' For the export, these variable(s) are formatted as factor with 
#' \strong{'Total' as the first level}.
#' @param rowVarTotalInSepRow Character vector with \code{rowVarTotalInclude}
#' (not in \code{rowVarInSepCol}) for which the total should be included in a separated row labelled 'Total'.
#' Otherwise (by default) the total is included in the header row of each category.
#' @param colVar Character vector with variable(s) to be included in columns.
#' If multiple variables are specified, the variables should 
#' be sorted in hierarchical order,
#' and are included in multi-columns layout.\cr
#' Use: 'variable' to include the variables to summarize: \code{var}
#'  (if multiple) in different columns.
#' @param colTotalInclude Logical, if TRUE (FALSE by default) include the summary 
#' statistics across columns in a separated column.
#' @param colTotalLab String, label for the total column  'Total' by default.\cr
#' @param subjectVar String, variable of \code{data} with subject ID,
#' 'USUBJID' by default.
#' @param statsLayout String with layout for the statistics names 
#' (in case more than one statistic is included), among:
#' \itemize{
#' \item{row (by default for 'flextable' output): \cr All statistics are
#'  included in different rows in the first column of the table (after the row variable(s))}
#' \item{'col' (by default for 'DT' output): \cr Statistics are included 
#' in separated columns (last row of the header).\cr
#' This option is not compatible with categorical variable(s).}
#' \item{'rowInSepCol': \cr Statistics are included in different rows, 
#' but in a separated column than the \code{rowVar} variable(s)}
#' }
#' @param statsValueLab String with label for the statistic value, 
#' 'StatisticValue' by default.\cr
#' This is only included in the table if the statistics provided in 
#' \code{stats} are not named and if no \code{colVar} is specified.
#' @param statsExtra (optional) Named list with functions for additional custom
#' statistics to be computed.\cr
#' Each function:
#' \itemize{
#' \item{has as parameter, either: 'x': the variable (\code{var}) to compute
#' the summary statistic on or 'data': the entire dataset}
#' \item{returns the corresponding summary statistic as a numeric vector}
#' }
#' For example, to additionally compute the coefficient of variation, this can be set to:
#' \code{list(statCVPerc = function(x) sd(x)/mean(x)*100)} (or \code{\link{cv}}).
#' @param type String with type of table: 
#' \itemize{
#' \item{'summaryTable': summary table with statistics for numeric variable}
#' \item{'countTable': count table}
#' \item{'auto' (by default): 'summaryTable' if the variable is numeric,
#' 'countTable' otherwise}
#' }
#' @param statsLabInclude Logical, if TRUE include the statistic label
#' in the table. \cr By default only included if more than
#' one statistic variables are available in the table.
#' @param title Character vector with title(s) for the table.
#' Set to NULL (by default) if no title should be included.
#' If multiple are specified, specified for each element of \code{byVar} (in order of the levels).
#' @param pageDim Numeric vector of length 2 with page width and height.\cr
#' Depending on \code{outputType}:
#' \itemize{
#' \item{'flextable': in inches}
#' \item{'DT': in number of rows in the table.\cr
#' Currently only the height is used (e.g. \code{c(NA, 4)})
#' }}
#' @param columnsWidth Column widths of the table, for \code{outputType}:
#' \itemize{
#' \item{'flextable': the widths can be specified either:
#'  \itemize{
#'  \item{in percentage: the sum of the widths should be 1}
#'  \item{in inch: the sum should be the page width (see \code{\link{getDimPage}})}
#' } If the widths are not specified for all columns, the width of the remaining last
#'  columns is set such as the width of the total table is the page width.}
#' \item{'DT': see \code{\link[clinUtils]{getClinDT}}}
#'}
#' @param labelVars (optional) Named character vector with label for
#' the row, column variable(s) or variable(s) to summarize. \cr
#' Labels specified via dedicated parameter: e.g. 
#' \code{rowVarLab}, \code{colVarLab}, \code{varLab}
#' have priority on this parameter.
#' @name inTextSummaryTable-common-args
#' @return No return value, used for the documentation of R functions
NULL

#' Common arguments for the functionalities of the inTextSummaryTable package
#' for flextable export.
#' @param style (flextable output) String with table style,
#'  either 'report' or 'presentation'.
#' This parameter affects the fontsize, font family, color of the text and background, 
#' and table dimensions of the table.
#' @param rowTotalLab (flextable output) string with label for the row with total.
#' @param rowAutoMerge (flextable output) Logical, if TRUE (by default) automatically merge rows,
#' e.g. in case there is only one sub-category (e.g. categorical variable with only one group)
#' or only one statistic per category.
#' @param rowVarFormat (flextable output) Named list with special formatting for the \code{rowVar}.
#' Currently, only possibility is to set the variable elements in bold, with:
#' list(var1 = "bold").
#' (Use 'variable' for \code{var} or 'variableGroup' for group within categorical variables.)
#' @param rowPadBase (flextable output) Base padding for row (in points), 
#' 14.4 by default (corresponds to 0.2 inches)
#' @param fontname (flextable output) String with font name, by default:
#' 'Times' if \code{style} is 'report' and 'Tahoma' if \code{style} is 'presentation'.
#' @param fontsize (flextable output) Integer with font size, by default:
#' 8 if \code{style} is 'report' and 10 if \code{style} is 'presentation'.
#' @param margin (flextable output) Margin in the document in inches
#' (1 by default).
#' This is used to specify the width of the table, from:
#' [\code{pageDim[1]} - 2 * margin].
#' @param colorTable (flextable output) Named character vector with color for the table background/body/text/line,
#' e.g. created with the \code{\link{getColorPaletteTable}} function.
#' @param landscape (flextable output) Logical, if TRUE the file is in landscape format.\cr
#' By default: FALSE if \code{style} is 'report' and TRUE if \code{style} is 'presentation'.
#' @param footer (flextable output) Character vector with footer(s) for the table.
#' Set to NULL (by default) if no footer should be included.
#' @param vline (flextable output) String mentioning how vertical lines 
#' should be included in the body of the table, either: 
#' \itemize{
#' \item{'none' (default): no vertical lines included}
#' \item{'auto': vertical lines included between sub-groups}
#' }
#' @param hline (flextable output) String mentioning how horizontal lines 
#' should be included in the body of the table, either: 
#' \itemize{
#' \item{'none': no horizontal lines included}
#' \item{'auto' (default): horizontal lines included between sub-groups}
#' }
#' @param file String with path of the file where the table should be exported.
#' The file should have the extension: '.html'.
#' If NULL, the summary table is not exported but only returned as output.
#' If \code{byVar} is specified, each table is exported to a separated
#' file with the suffix: 'file_[i].html' with i the index of the file.
#' @param colHeaderMerge (flextable output) Logical, if TRUE (FALSE by default) 
#' the column header is merged.
#' @name inTextSummaryTable-flextable-args
#' @return No return value, used for the documentation of 
#' R functions for 'flextable' output
NULL

#' Common arguments for the functionalities of the inTextSummaryTable package
#' for DT export.
#' @param expandVar (DT output) Character vector with variables of the summary table which
#' should be expanded in the data.
#' @param pageDim (DT output) Numeric vector of length 2 with page width and height,
#' in number of rows (currently only
#' the height is used (e.g. \code{c(NA, 4)})
#' @param noEscapeVar (DT output) Character vector with variables of \code{summaryTable}
#' which shouldn't be escaped in the table (e.g. containing URLs).
#' @param barVar (DT output) Character vector with variables of \code{summaryTable}
#' that should be represented as a bar.
#' @param file String with path of the file where the table should be exported.
#' The file should have the extension: '.docx'.
#' If NULL, the summary table is not exported but only returned as output.
#' If \code{byVar} is specified, each table is exported to a separated
#' file with the suffix: 'file_[i].docx' with i the index of the file.
#' @name inTextSummaryTable-DT-args
#' @return No return value, used for the documentation of 
#' R functions for 'DT' output
NULL

#' Plot subject summary profile.
#' 
#' The user can either specify a variable for the standard error
#' (\code{seVar}),
#' or directly the variables for the minimum and maximum values for the error
#' bars (\code{minVar}, \code{maxVar}).
#' @param data Data.frame with summary statistics to represent in the plot,
#' e.g. a \code{\link{summaryTable}} object.
#' @param xLab String with label for the \code{xVar}.
#' @param xGap (optional) Numeric vector of length 2 for which
#' a gap should be created in the x-axis.
#' Only available if \code{xVar} is specified and a numeric variable.
#' Records with \code{xVar} within \code{xGap} are filtered from the plot,
#' vertical lines are included at the min/max of the gap,
#' and the gap is represented as '//' in the x-axis of the plot.
#' @param xGapDiffNew Numeric vector of length 2 with new range
#' of the \code{xGap}. If not specified, the minimum difference between
#' consecutive x elements in the data is used.
#' @param meanVar String, variable of \code{data} with the mean variable.
#' @param seVar String, variable of \code{data} with the standard error.
#' @param minVar,maxVar String, variables of \code{data} with minimum and
#' maximum value for error bar. 
#' If both are specified, \code{seVar} is ignored.
#' @param yLab String with label for the y-axis.
#' If different labels should be used for different elements of
#' \code{byVar} variable, the vector should be named
#' with each corresponding element (collapsed with '.' if multiple).
#' @param facetVar Character vector, variable(s) of \code{data} for facetting.
#' @param facetScale String with type of scale used for facetting, 'free_y' by default
#' (fixed scale in the x-axis and free in the y-axis).
#' @param useLinetype Logical, if TRUE (FALSE by default) use also linetype
#' to differenciate the variable specified via \code{colorVar} in the mean line.
#' @param linetypePalette Vector with linetype(s), in case \code{useLinetype} is TRUE.
#' @param title String with title for the plot.
#' If different labels should be used for different elements of
#' \code{byVar} variable, the vector should be named
#' with each corresponding element (collapsed with '.' if multiple).
#' @param jitter Numeric with jitter for the x-axis, only used if \code{colorVar} specified.
#' @param label Logical or expression or list of expression.
#' Points are labelled with \code{meanVar} if set to TRUE,
#' or with the specified expression if \code{label} is an expression.
#' If a list is specified, 'textLabel' (required) 
#' should contain expression to extract label, 
#' and 'textHjust' and 'textVjust' (optional) may contain expression 
#' specifying horizontal and vertical adjustment of the label.
#' @param labelPadding Amount of padding (space) between each point
#' and its \code{label}, 1.5 lines by default.
#' See parameter \code{point.padding} of the
#' \code{\link[ggrepel]{geom_text_repel}} function.
#' @param sizePoint Size for the point.
#' @param sizeLine Size for the line linking means and error bars.
#' @param sizeLabel Size for the label, only used if \code{label} is not NULL.
#' @param xAxisLabs (optional) Named character vector with labels for the x-axis.
#' @param xAxisExpand Object passed to the 'expand' parameter of:
#' \code{\link[ggplot2]{scale_x_continuous}},
#' (\code{\link[ggplot2]{waiver}} by default).
#' @param yLim Vector of the length 2 with limits for the y-axis.
#' @param yAxisExpand Expansion constants for the limits for the y-axis.
#' See the documentation of the \code{expand} parameter of the 
#' \code{\link[ggplot2]{scale_y_continuous}} function
#' for the available values for this parameter.
#' @param yLimExpand This parameter is deprecated, use \code{yAxisExpand}
#' instead.
#' @param tableText (optional) Character vector with colname of \code{data}
#' or expression from colnames of \code{data} to be represented in
#' the table below the plot.
#' By default, no table is displayed.
#' @param tableTextFontface Font face for the text included
#' in the table.
#' @param tableYAxisLabs Logical, if TRUE (by default)
#' the labels of the \code{colorVar} are included
#' in the y-axis of the table.
#' @param tableHeight Numeric of length 1 with height for the table.
#' @param byVar Variable(s) of \code{data} for which separated plot(s)
#' should be created.
#' @param hLine (optional) numeric with y-intercept of line(s) to be added.
#' If different thresholds should be used for different elements of the 
#' \code{byVar} or \code{facetVar} variables, the vector should be named
#' with each corresponding element (collapsed with '.' if multiple).
#' @param hLineColor String with color for \code{hLine}, 'black' by default.
#' @param hLineLty String with linetype for \code{hLine}, 'solid' by default.
#' @param vLine (optional) numeric with x-intercept of line(s) to be added.
#' If different thresholds should be used for different elements of the 
#' \code{byVar} or \code{facetVar} variables, the vector should be named
#' with each corresponding element (collapsed with '.' if multiple).
#' @param vLineColor String with color for \code{vLine}, 'black' by default.
#' @param vLineLty String with linetype for \code{vLine}, 'solid' by defaul
#' @param useShape Logical, if TRUE (by default) \code{colorVar} is also used for the shape.
#' @param widthErrorBar Numeric vector of length 1 with width of error bar.
#' @param shapePalette Named vector with shape palette for \code{colorVar}.
#' @param themeIncludeVerticalGrid Logical, if TRUE (by default)
#' include theme vertical grid lines (if present in \code{themeFct}).
#' @param ggExtra Extra \code{ggplot} call of function generating such call(s)
#' to be added in main plot.\cr
#' Multiple calls are set via a function, e.g. 
#' \code{ggExtra = function(gg){gg + geom_vline(...) + geom_hline(...)}}.\cr
#' If different calls should be used for different elements of the 
#' \code{byVar} variable, the vector should be named
#' with each corresponding element (collapsed with '.' if multiple).
#' @param yTrans (optional) String with transformation for the y-axis.
#' Currently only 'log10' (or NULL, default) is available.
#' In case error bars go in the negative, their values are set to a 'small enough' value for plotting:
#' \code{min(data)/10} or \code{yLim[1]} if \code{yLim} is specified.
#' @param legendPosition String with legend position.
#' By default, 'bottom' of \code{tableText} is not specified,
#' 'none' otherwise.
#' @param tablePlotMargin Margin between the plot and the table,
#' expressed as \code{\link[grid]{unit}}, 0 by default.
#' @param ... Additional parameters for \code{\link[ggrepel]{geom_text_repel}} or
#' \code{\link[ggplot2]{geom_text}}
#' used for the \code{label}.
#' @inheritParams subjectProfileSummaryTable
#' @return \code{\link[ggplot2]{ggplot}} object or list of such
#' objects of \code{byVar} is specified.
#' @author Laure Cougnaud
#' @importFrom clinUtils getColorPalette getShapePalette getLinetypePalette
#' @importFrom clinUtils getLabelVar
#' @import ggplot2
#' @import cowplot
#' @importFrom ggrepel geom_text_repel
#' @importFrom scales trans_new
#' @importFrom plyr rbind.fill
#' @importFrom utils packageVersion
#' @export
subjectProfileSummaryPlot <- function(data,
    xVar = NULL, xLab = getLabelVar(xVar, labelVars = labelVars), 
    xAxisExpand = waiver(),
    xGap = NULL, xGapDiffNew = NULL,
    meanVar = "statMean", seVar = if("statSE" %in% colnames(data))	"statSE", 
    minVar = NULL, maxVar = NULL,
    yLab = paste(c(
            sub("^stat", "", meanVar), 
            if(!is.null(minVar) & !is.null(maxVar)){
                  paste0(
                      "(", sub("^stat", "", minVar), ", ", 
                      sub("^stat", "", maxVar), ")"
                  )
                }else	if(!is.null(seVar))	paste("+-", sub("^stat", "", seVar))
        ), collapse = " "),
    facetVar = NULL, facetScale = "free_y",
    colorVar = NULL, colorLab = getLabelVar(colorVar, labelVars = labelVars),
    colorPalette = NULL,
    labelVars = NULL,
    useLinetype = TRUE,
    linetypePalette = NULL,
    useShape = TRUE,
    shapePalette = NULL,
    jitter = NULL,
    title = NULL, caption = NULL,
    yTrans = NULL, yLim = NULL, xLim = NULL,
    yAxisExpand = c(0.05, 0.05),
    yLimExpand = NULL,
    xAxisLabs = NULL,
    sizePoint = 1.5,
    sizeLine = 0.5,
    sizeLabel = 3.87,
    widthErrorBar = 0.9,
    tableText = NULL, tableTextFontface = 1,
    tableHeight = 0.1, 
    tableYAxisLabs = !is.null(colorVar),
    tablePlotMargin = unit(0, "pt"),
    label = FALSE, labelPadding = unit(1, "lines"), 
    byVar = NULL,
    hLine = NULL, hLineColor = "black", hLineLty = "solid",
    vLine = NULL, vLineColor = "black", vLineLty = "solid",
    style = "report",
    fontname = switch(style, 'report' = "Times", 'presentation' = "Tahoma"),
    fontsize = switch(style, 'report' = 8, 'presentation' = 10),
    themeFct = switch(style, 'report' = theme_classic, 'presentation' = theme_bw),
    themeIncludeVerticalGrid = TRUE,
    ggExtra = NULL,
    legendPosition = ifelse(!is.null(tableText), "none", "bottom"),
    ...){
  
	if(class(data)[1] != "data.frame")
		data <- as.data.frame(data)
		
  if(!is.null(yLimExpand)){
    .Deprecated(new = "yAxisExpand", old = "yLimExpand")
    yAxisExpand <- yLimExpand
  }
  
  useMinMax <- !is.null(minVar) & !is.null(maxVar)
  if(!is.null(minVar) & is.null(maxVar))
    warning("'minVar' is not used because 'maxVar' is not specified.")
  if(!is.null(maxVar) & is.null(minVar))
    warning("'maxVar' is not used because 'minVar' is not specified.")
  
  if(!is.null(facetVar) & !is.null(tableText)){
    warning("Table cannot be used in combination with 'facetVar', no table is included.")
    tableText <- NULL
  }
  
  if(!is.null(byVar)){
    inputParams <- c(as.list(environment()), list(...))
    if(!all(byVar %in% colnames(data))){
      warning("'byVar' is not available in the 'data' so is not used.")
      byVar <- FALSE
    }else{
      
      # get parameter in case possibility to have it by 'byVar' element
      # only works if default is NULL
      getParamByEl <- function(x, el, default = x){
        if(!is.null(x) && !is.null(names(x))){
          if(el %in% names(x))	x[[el]]	else	NULL
        }else	default
      }
      
      res <- dlply(data, byVar, function(dataBy){
            byVarEl <- as.matrix(unique(dataBy[, byVar, drop = FALSE]))
            byElST <- paste0(byVarEl, collapse = " ")
            byEl <- paste0(byVarEl, collapse = ".")
            inputParamsBy <- inputParams
            inputParamsBy$data <- dataBy
            inputParamsBy$byVar <- NULL
            inputParamsBy$yLab <- getParamByEl(x = inputParamsBy$yLab, el = byEl, default = paste(inputParamsBy$yLab, byElST))
            inputParamsBy$hLine <- getParamByEl(x = inputParamsBy$hLine, el = byEl)
            inputParamsBy$vLine <- getParamByEl(x = inputParamsBy$vLine, el = byEl)
            inputParamsBy$title <- getParamByEl(x = inputParamsBy$title, el = byEl)
            inputParamsBy$ggExtra <- getParamByEl(x = inputParamsBy$ggExtra, el = byEl)
            do.call(subjectProfileSummaryPlot, inputParamsBy)		
          })	
      return(res)
    }
  }
  
  if(!is.null(yTrans) && !(is.character(yTrans) && yTrans == "log10")){
    warning("Currently 'yTrans' can only have value: 'log10', so this parameter is ignored.")
    yTrans <- NULL
  }
  
  if(!is.null(xGap)){
    if(!is.null(xVar)){
      if(!is.numeric(data[, xVar])){
        warning("'xGap' should only be specified for continuous x-variable, 'xGap' is ignored.")
        xGap <- NULL
      }else	if(length(xGap) != 2){
        warning("'xGap' should be of length 2, 'xGap' is ignored.")
        xGap <- NULL
      }
    }else{
      warning("'xGap' should only be specified if 'xVar' is specified, 'xGap' is ignored.")
      xGap <- NULL
    }
    
  }
  
  varNotInData <- setdiff(c(meanVar, seVar, minVar, maxVar), colnames(data))
  if(length(varNotInData) > 0)
    stop(paste("Variable(s):", toString(varNotInData), "are not in data."))
  
  if(!is.null(xVar) & is.null(jitter))
    jitter <- if(is.numeric(data[, xVar])){
          # only one element
          if(length(unique(data[, xVar])) == 1){
            0.1
            # multiple element, take 10% of the min diff
          }else{
            min(diff(sort(unique(data[, xVar]))))*0.1
          }
        }else	0.3
  
  pd <- position_dodge(jitter) # move them .05 to the left and right
  
  # compute minimum and maximum limits for the error bars
  includeEB <- !is.null(seVar) | useMinMax
  if(includeEB){
    
    if(useMinMax){
      dataYMinYMax <- data[, c(minVar, maxVar)]
    }else{
      dataYMinYMax <- data[, meanVar] + data[, seVar] %*% t(c(-1, 1))
    }
    
    # for log scale, set negative values (if any) to a small positive value
    # otherwise the entire error bar is not displayed in ggplot2
    if(!is.null(yTrans) && yTrans == "log10"){
      idxNeg <- which(dataYMinYMax < 0, arr.ind = TRUE)
      if(nrow(idxNeg) > 0){
        yMin <- if(!is.null(yLim))	yLim[1]
        if(!(!is.null(yMin) && !is.na(yMin)))
          yMin <- min(dataYMinYMax[dataYMinYMax > 0])/10
        warning(paste(nrow(idxNeg), "negative value(s) in the error bars, these values",
                "are set to:", prettyNum(yMin), "for plotting."
            ))
        dataYMinYMax[idxNeg] <- yMin
      }
    }
    
    data[, c("ymin", "ymax")] <- dataYMinYMax
    
  }
  
  # in case variable contain spaces or other character not parsed by ggplot2
  if(!is.null(xVar))
    data$xVar <- data[, xVar]
  if(!is.null(meanVar))
    data$meanVar <- data[, meanVar]
  if(!is.null(colorVar))
    data$colorVar <- data[, colorVar]
  
  if(!(is.logical(label) && !label)){
    if(is.list(label)){
      if(!all(sapply(label, is.language))){
        stop("If 'label' is a list, should be a list of expressions.")
      }else{
        if(!"textLabel" %in% names(label))
          stop("If 'label' is a list, should contain at least 'textLabel'.")
        for(labelI in names(label))
          data[, labelI] <- eval(expr = label[[labelI]], envir = data)
      }
    }else	if(is.language(label)){
      data$textLabel <- eval(expr = label, envir = data)
    }else	if(is.logical(label)){
      data$textLabel <- data[, meanVar]
    }
  }
  
  # base plot
  gg <- ggplot()
  
  ## horizontal line(s)
  setLines <- function(gg, inputLine, typeLine = c("hline", "vline"), color, linetype){
    
    typeLine <- match.arg(typeLine)
    paramName <- switch(typeLine, "hline" = "yintercept", "vline" = "xintercept")
    geomFct <- match.fun(paste0("geom_", typeLine))
    dataLine <- data.frame(line = inputLine, color = color, linetype = linetype, stringsAsFactors = FALSE)
    if(!is.null(facetVar) & !is.null(names(inputLine))){
      facetLine <- names(inputLine)
      if(is.factor(data[, facetVar])){
        facetLine <- factor(facetLine, levels = levels(data[, facetVar]))
      }
      dataLine <- cbind(
          dataLine,
          setNames(data.frame(facetLine, stringsAsFactors = FALSE), facetVar)
      )
      
    }
    for(i in seq_len(nrow(dataLine))){
      gg <- gg + do.call(geomFct, 
          list(
              data = dataLine[i, ], 
              mapping = do.call(aes, setNames(list(sym("line")), paramName)),
              color = dataLine[i, "color"],
              linetype = dataLine[i, "linetype"]
          )
      )
    }
    return(gg)
    
  }
  if(!is.null(hLine))
    gg <- setLines(
        gg = gg,
        inputLine = hLine, 
        typeLine = "hline", color = hLineColor,
        linetype = hLineLty
    )
  if(!is.null(vLine))
    gg <- setLines(
        gg = gg,
        inputLine = vLine, 
        typeLine = "vline", color = vLineColor,
        linetype = vLineLty
    )
  
  # if break in the x-axis: remove data within breaks +
  # create 'group' variable used in aesthetic of geom_line
  if(!is.null(xGap)){
    
    # remove data contained within the gaps:
    idxRetained <- which(data$xVar <= xGap[1] | data$xVar >= xGap[2])
    data <- data[idxRetained, ]
    
    # separated lines with the: 'group' aesthetic:
    lineGroup <- c(
        list(ifelse(data[, xVar] < xGap[2], "Group1", "Group2")),
        if(!is.null(colorVar))	list(data$colorVar)
    )
    data$lineGroup <- do.call(interaction, lineGroup)
    
    ## create a transformation on the x-axis 
    # to 'shift' the elements > xGap[2]
    if(is.null(xGapDiffNew)){
      xGapDiffNew <- min(diff(sort(unique(data$xVar))))
    }
    xShift <- xGap[2] - (xGap[1] + xGapDiffNew)
    if(xShift < 0)	xShift <- xGap[2]
    
    # create programmatically transform and inverse functions
    # 1) create empty functions 2) fill function body
    transInvFct <- transFct <- function(x){}
    body(transFct) <- bquote(ifelse(x >= .(xGap[2]), x - .(xShift), x))
    body(transInvFct) <- bquote(ifelse(x >= .(xGap[2]), x + .(xShift), x))
    xTrans <- trans_new(name = "xGap", transform = transFct, inverse = transInvFct)
    
    ## horizontal lines:
    gg <- gg + geom_vline(xintercept = xGap, linetype = 2, color = "grey")
    
    ## add symbol x-axis:
    # vjust not properly placed with: geom = 'text', so use geom_label
    xGapPos <- xGap[1] + xGapDiffNew/2
#		yGapPos <- ifelse(is.null(yLim), min(data$meanVar, na.rm = TRUE), yLim[1]) -
#			yLimExpand[1] * ifelse(is.null(yLim), diff(range(data$meanVar, na.rm = TRUE)), diff(yLim))
    dataGapSym <- data.frame(y = -Inf, x = xGapPos, label = "//")
    aesArgs <- list(x = sym("x"), y = sym("y"), label = sym("label"))
    gg <- gg + geom_text(
        data = dataGapSym, 
        mapping = do.call(aes, aesArgs),
        show.legend = FALSE, inherit.aes = FALSE,
        size = sizeLabel,
        hjust = "center", vjust = 1
    )	
#	gg <- gg + annotate(geom = "label", 
#			y = yGapPos, x = xGapPos, label = "//", 
#			hjust = 0.5, vjust = 1,
#			size = sizeLabel,
#			label.size = 0
#		)
    
  }else xTrans <- NULL
  
  # line + points
  # base plot
  aesBase <- c(
      if(!is.null(xVar))	list(x = sym("xVar")),
      if(!is.null(colorVar))	list(color = sym("colorVar"))
  )
  aesLine <- c(
      aesBase,
      list(y = sym("meanVar")),
      list(group = ifelse(
              !is.null(xGap), sym("lineGroup"),
              ifelse(!is.null(colorVar), sym("colorVar"), 1)
          )),
      if(!is.null(colorVar) & useLinetype)	list(linetype = sym("colorVar"))
  )
  
  # line
  aesLineSize <- ifelse(packageVersion("ggplot2") >= "3.4.0", "linewidth", "size")
  argsGeomLine <- list(
  	mapping = do.call(aes, aesLine), 
	  position = pd, data = data
  )
  argsGeomLine[[aesLineSize]] <- sizeLine
  gg <- gg + do.call(geom_line, argsGeomLine)
  
  # point
  gg <- gg +
      geom_point(
          mapping = do.call(aes, 
              c(
                  aesBase, 
                  list(y = sym("meanVar")),
                  if(!is.null(colorVar) & useShape)	list(shape = sym("colorVar"))
              )
          ), 
          position = pd, size = sizePoint, data = data
      )
  
  if(!(is.logical(label) && !label)){
    
    # aes parameters
    aesJust <- setNames(c(sym("textHjust"), sym("textVjust")), c("hjust", "vjust"))
    aesJust <- aesJust[c("textHjust", "textVjust") %in% names(label)]
    aesArgs <- c(aesBase, list(label = sym("textLabel"), y = sym("meanVar")))
    geomTextFct <- "geom_text_repel"
    if(length(aesJust) > 0){
      geomTextFct <- "geom_text"
      aesArgs <- c(aesArgs, aesJust)
    }
    
    # geom_text(_repel) parameters
    geomTextArgs <- c(
        list(
            mapping = do.call(aes, aesArgs), 
            data = data,
            position = pd, size = sizeLabel,
            show.legend = FALSE,
            ...
        ),
        if(geomTextFct == "geom_text_repel")
          list(
              seed = 123, 
              point.padding = labelPadding
          )
    )
    gg <- gg + do.call(geomTextFct, geomTextArgs)
  }
  
  if(includeEB){
	  argsGeomEB <- list(
	    mapping = do.call(aes, 
	      c(aesBase, list(ymin = sym("ymin"), ymax = sym("ymax")))
	    ), 
	    data = data,
      position = pd, width = widthErrorBar
    )
	  argsGeomEB[[aesLineSize]] <- sizeLine
    gg <- gg + do.call(geom_errorbar, argsGeomEB)
  }
  
  # facetting
  if(!is.null(facetVar))
    gg <- gg + facet_wrap(facetVar, scales = facetScale)
  
  # palettes
  if(!is.null(colorVar)){
    if(is.null(colorPalette)) 
      
      colorPalette <- getColorPalette(
          x = data[, colorVar],
          palette = getOption("inTextSummaryTable.colors.plot")
      )
    gg <- gg + scale_color_manual(name = colorLab, values = colorPalette)
  }
  
  if(useLinetype){
    if(is.null(linetypePalette)){     
      linetypePalette <- getLinetypePalette(
          x = data[, colorVar],
          palette = getOption("inTextSummaryTable.linetypes.plot")
      )
	  if(length(linetypePalette) == 0)
		  linetypePalette <- NULL
  	}
    gg <- gg + scale_linetype_manual(name = colorLab, values = linetypePalette)			
  }
  
  if(useShape){
    if(is.null(shapePalette)){
      shapePalette <- getShapePalette(
          x = data[, colorVar],
          palette = getOption("inTextSummaryTable.shapes.plot")
      )
	  if(length(shapePalette) == 0)
		  shapePalette <- NULL
  	}
    gg <- gg + scale_shape_manual(name = colorLab, values = shapePalette)			
  }
  
  # labels for the axes/title
  argsLab <- list(
      x = xLab, 
      y = yLab, 
      title = title, 
      caption = if(is.null(tableText))	caption
  )
  argsLab <- argsLab[!sapply(argsLab, is.null)]
  if(length(argsLab) > 0)
    gg <- gg + do.call(labs, argsLab)
  
  argsTheme <- c(
      list(
          legend.position = legendPosition,
          text = element_text(family = fontname, size = fontsize)
      ),
      if(!themeIncludeVerticalGrid)
        list(
            panel.grid.major.x = element_blank(),
            panel.grid.minor.x = element_blank()
        )
  )
  gg <- gg + themeFct() + do.call(theme, argsTheme)
  
  if(!is.null(ggExtra)){
    if(is.function(ggExtra)){
      gg <- ggExtra(gg)
    }else{
      gg <- gg + ggExtra
    }
  }
  
  # y-axis:
  argsYScale <- c(
      if(!is.null(yAxisExpand))	list(expand = yAxisExpand),
      if(!is.null(yTrans))	list(trans = yTrans)
  )
  if(length(argsYScale) > 0)
    gg <- gg + do.call(scale_y_continuous, argsYScale)
  
  # limits/clipping
  argsCoordCart <- c(
      if(!is.null(xLim))	list(xlim = xLim),
      if(!is.null(yLim))	list(ylim = yLim),
      if(!is.null(xGap))	list(clip = "off")
  )
  if(length(argsCoordCart) > 0)
    gg <- gg + do.call(coord_cartesian, argsCoordCart)
  
  # set limits in the x-axis
  # even if not specified, to have correct alignment with table
  if(is.null(xAxisLabs)){
    xAxisLabs <- if(is.factor(data[, xVar]))
      levels(data[, xVar])	else	unique(data[, xVar])
    names(xAxisLabs) <- xAxisLabs
  }
  fctScaleX <- if(is.numeric(data[, xVar])){
        
        # limits should take the jitter into account!
        scaleXLim <- max(jitter, GeomErrorbar$default_aes$width/2)
        
        limits <- range(xAxisLabs) + c(-1, 1)*scaleXLim
        
        # fix in case ggExtra contains data with a different range
        ggData <- do.call(plyr::rbind.fill, ggplot_build(gg)$data)
        ggDataRange <- range(ggData$x, na.rm = TRUE)
      
        limits <- range(c(limits, ggDataRange), na.rm = TRUE)
          
        argsScaleXCont <- c(
            list(
                breaks = unname(xAxisLabs), 
                limits = limits, 
                labels = names(xAxisLabs),
                expand = xAxisExpand
            ),
            if(!is.null(xTrans))	list(trans = xTrans)
        )
        do.call(scale_x_continuous, argsScaleXCont)
      }else{
        scale_x_discrete(
            breaks = unname(xAxisLabs), 
            labels = names(xAxisLabs), 
            drop = FALSE,
            expand = xAxisExpand
        )
      }
  gg <- gg + fctScaleX
  
  res <- if(!is.null(tableText)){
        
        # if xLab is specified only consider
        # the corresponding elements in xVar for the table
        dataTable <- if(!is.null(xAxisLabs)){
              data[data[, xVar] %in% xAxisLabs, ]
            }else	data
        
        # create plot with table
        ggTable <- subjectProfileSummaryTable(
            data = dataTable, xVar = xVar, 
            text = tableText, fontface = tableTextFontface,
            xLim = xLim,
            xLab = xLab,
            colorVar = colorVar, 
            colorLab = colorLab,
            colorPalette = colorPalette,
            legendPosition = legendPosition,
            yAxisLabs = tableYAxisLabs,
            caption = caption,
            style = style,
            fontname = fontname,
            fontsize = fontsize,
            themeFct = themeFct,
            xTrans = xTrans
        )
        
        ggTable <- ggTable + fctScaleX
        
        # remove legend and title x-axis for base plot (will be included in table plot)
        plotMargin <- calc_element("plot.margin", themeFct())
		plotMargin[3] <- tablePlotMargin # set bottom margin
		gg <- gg + theme(
			legend.position = "none", 
			axis.title.x = element_blank(),
			plot.margin = plotMargin
		)  
		
        # combine base and table plot
        if(tableHeight > 1 | tableHeight < 0)
          stop("Table height should be between 0 and 1.")
        res <- plot_grid(gg, ggTable, ncol = 1,
            align = "hv", axis = "lr",
            rel_heights = c(1-tableHeight, tableHeight)
        )
        
      }else gg
  
  return(res)
		

	
}

#' Plot a table with \code{ggplot} of a text variable of interest.
#' 
#' The labels extracted based on the \code{text} parameter 
#' and displayed at the x-position based on \code{xVar} and the y-position based on
#' \code{colorVar}.
#' Each group specified in the color variables are displayed in different
#' lines in the plot.
#' @param data Data.frame (in long format) with data for the table.
#' @param xVar String, variable of \code{data} with variable for the x-axis.
#' @param text Character vector with colnames of \code{data}
#' or expression based on colnames of \code{data} to extract
#' the text label.
#' @param xLim Vector of the length 2 with limits for the x-axis.
#' @param colorVar String, variable of \code{data} for coloring.
#' @param colorLab String, label for \code{colorVar}, used in the legend.
#' @param fontface Numeric, fontface for the text.
#' @param colorPalette (named) Vector with color palette.
#' @param xLab String with label for the x-axis.
#' @param textSize Size for the text.
#' @param labelVars Named string with variable labels (names are the variable code).
#' @param showLegend Logical, should the legend be displayed? TRUE by default.
#' @param yAxisLabs Logical, if TRUE include the labels in the y-axis.
#' @param xAxisLabs Vector with labels for the x-axis if \code{xVar}
#' is discrete or vector with limits if continuous.
#' @param style String with subject profile style.
#' This affects the parameters: \code{fontname}, \code{fontsize}
#' and \code{themeFct}.
#' @param fontname String with font name,
#' by default 'Times' if \code{style} is 'report' and
#' 'Tahoma' if \code{style} is 'presentation'.
#' @param fontsize Numeric vector of length 1 with font size,
#' by default 8 if \code{style} is 'report' and
#' 10 if \code{style} is 'presentation'
#' @param pointSize Numeric indicating the size of points in the legend, 1.5 by default
#' @param themeFct Function with ggplot2 theme,
#' by default \code{\link[ggplot2]{theme_classic}} if \code{style} is 'report' and
#' \code{\link[ggplot2]{theme_bw}} if \code{style} is 'presentation'.
#' @param xTrans (optional) ggplot2 transformation
#' for the x-axis.
#' @param legendPosition String with legendPosition,
#' 'right' by default.
#' @param caption String with caption for the plot,
#' NULL by default.
#' @return \code{\link[ggplot2]{ggplot}} object
#' @import ggplot2
#' @importFrom utils packageVersion 
#' @importFrom clinUtils getLabelVar getColorPalette
#' @author Laure Cougnaud and Michela Pasetto
#' @export
subjectProfileSummaryTable <- function(
    data, xVar, text, 
    xLim = NULL, 
    colorVar = NULL, colorPalette = NULL,
    colorLab = getLabelVar(colorVar, labelVars = labelVars),
    fontface = 1,
    xLab = NULL,
    labelVars = NULL,
    caption = NULL,
    showLegend = TRUE,
    legendPosition = ifelse(showLegend, "right", "none"),
    yAxisLabs = FALSE,
    xAxisLabs = NULL,
    style = "report",
    fontname = switch(style, 'report' = "Times", 'presentation' = "Tahoma"),
    fontsize = switch(style, 'report' = 8, 'presentation' = 10), # pt
    pointSize = 1.5,
    themeFct = switch(style, 'report' = theme_classic, 'presentation' = theme_bw),
    textSize = fontsize/ggplot2::.pt,
    xTrans = NULL
){
  
  data$tableTextLabel <- if(is.language(text)){
        eval(expr = text, envir = data)
      }else if(is.character(text)){
        if(!text %in% colnames(data)){
          stop(paste0("'", text, "' should be among the columns of 'data'."))
        }else{
          data[, text]
        }
      }else stop("'text' should be an expression of character.")
  
  if(yAxisLabs & is.null(colorVar))
    warning("Labels for the y-axis are not included because color variable is not specified.")
  
  if(!is.null(colorVar)){
    
    # convert variable to a factor to set correct order of elements in the y-axis
    if(!is.factor(data[, colorVar]))
      data[, colorVar] <- as.factor(data[, colorVar])
    
    # order y rows with first level on top (max y), last level on the bottom (min y)
    data$tableY <- factor(data[, colorVar], levels = rev(levels(data[, colorVar])))
    
  }
  
  # aesthetics
  aesTablePlot <- c(
      list(
          x = sym(xVar),
          y = ifelse(!is.null(colorVar), sym("tableY"), 1),
          label = sym("tableTextLabel"),
          fontface = fontface
      ),
      if(!is.null(colorVar)) list(color = sym(colorVar))
  )
  # arguments for geom_text
  argsGeomText <- list(
      mapping = do.call("aes", aesTablePlot),
      size = textSize,			
      show.legend = FALSE
  )
  # arguments for geom_point
  # geom_point is used with size = NA to avoid the 'a' in the legend
  # this workaround will show points in the legend
  idxFontface <- which(names(aesTablePlot) == "fontface")
  aesPoint <- aesTablePlot[- idxFontface]
  aesPoint[["label"]] <- NULL # remove 'label' not used for geom_point
  argsGeomPoint <- list(
      mapping = do.call(aes, aesPoint),
      size = NA
  )
  
  # base plot
  # returns warnings because size of points = NA
  ggTable <- ggplot(data = data) + 
      do.call("geom_text", argsGeomText) + 
      do.call("geom_point", argsGeomPoint) +
      guides(colour = guide_legend(override.aes = list(size = pointSize)))
  
  # axis limits
  if(!is.null(xLim))
    ggTable <- ggTable + coord_cartesian(xlim = xLim)
  
  # labels
  # if no x-label, set labs(x = NULL) to remove bottom margin
  ggTable <- ggTable + 
      labs(
          x = if(!is.null(xLab))	xLab,
          caption = caption
      )
  
  # color palette
  if(!is.null(colorVar)){
    if(is.null(colorPalette))
      colorPalette <- getColorPalette(
          x = data[, colorVar],
          palette = getOption("inTextSummaryTable.colors.plot")
      )
    ggTable <- ggTable + scale_color_manual(name = colorLab, values = colorPalette)
  }
  
  if(!is.null(xAxisLabs)){
    ggTable <- if(is.numeric(data[, xVar])){
          argsScaleXCont <- c(
              list(limits = xAxisLabs),
              if(!is.null(xTrans))	list(trans = xTrans)
          )
          ggTable + do.call(scale_x_continuous, argsScaleXCont)
        }else{
          ggTable + scale_x_discrete(breaks = xAxisLabs)
        }
  }
  
  # default: expand by 0.6 units on each side
  # cowplot cut labels if change expand_scale
  if(packageVersion("ggplot2") >= "3.3") {
    ggTable <- ggTable + scale_y_discrete(expand = expansion(add = 0.2))
  } else ggTable <- ggTable + scale_y_discrete(expand = expand_scale(add = 0.2))
  
  # theme
  argsTheme <- c(
      list(
          legend.position = legendPosition,
          axis.ticks.y = element_blank(),
          axis.title.y = element_blank(),
          axis.text.x = element_blank(),
          axis.ticks.x = element_blank(),
          text = element_text(family = fontname, size = fontsize),
          panel.grid.major = element_blank(),
          panel.grid.minor = element_blank(),
          panel.border = element_blank(),
          panel.background = element_blank(),
          plot.margin = unit(c(0,0,0,0), "mm")
      ),
      if(is.null(xLab))	list(axis.title.x = element_blank()),
      if(!yAxisLabs){
            list(axis.text.y = element_blank())
          }else if(!is.null(colorVar)){
            # consider levels of color variable factor to
            # set correct palette in case color palette specified for some elements
            # without data (not represented in the plot)
            paletteEl <- levels(droplevels(data[, "tableY"]))
            colorPaletteAxes <- unname(colorPalette[paletteEl])
            list(axis.text.y = element_text(colour = colorPaletteAxes))
          }
  )
  ggTable <- ggTable + themeFct() + do.call(theme, argsTheme)
  
  return(ggTable)
  
}

context("Create a subject profile summary plot")

library(ggplot2)
library(plyr)

test_that("An error is generated if the default variable for the plot is not available in the data", {
			
	expect_error(
		subjectProfileSummaryPlot(data = data.frame()),
		"Variable.* not in data"
	)
			
})

test_that("The plot is correctly facetted based on a variable", {
			
	summaryTable <- data.frame(
		PARAM = factor(
			c("AAA", "AAA", "ZZZ", "ZZZ"), 
			levels = c("ZZZ", "AAA")
		),
		visit = c(1, 2, 1, 2), 
		statMean = rnorm(4)
	)
	
	expect_silent(
		gg <- subjectProfileSummaryPlot(
			data = summaryTable,
			xVar = "visit", 
			facetVar = "PARAM"
		)
	)
	
	# check that the plots is facetted
	# and that facetted are ordered according to levels of factor
	ggData <- lapply(ggplot_build(gg)$data, `[`, c("x", "y", "PANEL"))
	ggData <- unique(do.call(rbind, ggData))
	facets <- levels(summaryTable$PARAM)
	for(i in seq_along(facets)){
		expect_equal(
			object = subset(ggData, PANEL == !!i)[, c("x", "y")],
			expected = setNames(
				subset(summaryTable, PARAM == facets[[!!i]])[, c("visit", "statMean")],
				c("x", "y")
			),
			check.attributes = FALSE
		)
	}
			
})

test_that("The scale of the facet is correctly set", {
			
	summaryTable <- data.frame(
		PARAM = c("AAA", "AAA", "ZZZ", "ZZZ"),
		visit = c(1, 2, 1, 2), 
		statMean = rnorm(4)
	)
			
	expect_true({
				
		gg <- subjectProfileSummaryPlot(
			data = summaryTable,
			xVar = "visit", 
			facetVar = "PARAM",
			facetScale = "free_y"
		)
		expect_true(
			with(ggplot_build(gg)$layout$facet$params, 
				free$y & !free$x
			)
		)
		
	})
			
})

test_that("Horizontal lines are correctly set", {
			
	hLine <- c(1, 3)
	hLineColor <- c("blue", "red")
	hLineLty <- c("dotted", "dashed")
	gg <- subjectProfileSummaryPlot(
		data = data.frame(
			visit = c(1, 2), 
			statMean = rnorm(2)
		),
		xVar = "visit", 
		hLine = hLine,
		hLineColor = hLineColor,
		hLineLty = hLineLty
	)
	
	# extract data behind the lines
	isGeomLine <- sapply(gg$layers, function(l) inherits(l$geom, "GeomHline"))
	ggDataLine <- do.call(rbind, ggplot_build(gg)$data[isGeomLine])
	
	expect_equal(ggDataLine$yintercept, hLine)
	expect_equal(ggDataLine$colour, hLineColor)
	expect_equal(ggDataLine$linetype, hLineLty)
	
})

test_that("Horizontal lines are correctly set by facet", {
			
	summaryTable <- data.frame(
		PARAM = factor(
			c("AAA", "AAA", "ZZZ", "ZZZ"),
			levels = c("ZZZ", "AAA")
		),
		visit = c(1, 2, 1, 2), 
		statMean = rnorm(4)
	)
			
	expect_equal({
				
		gg <- subjectProfileSummaryPlot(
			data = summaryTable,
			xVar = "visit", 
			facetVar = "PARAM",
			hLine = c(AAA = 3, ZZZ = 1)
		)
		
		# extract data behind the lines
		isGeomLine <- sapply(gg$layers, function(l) inherits(l$geom, "GeomHline"))
		ggDataLine <- do.call(rbind, ggplot_build(gg)$data[isGeomLine])
	
		ggDataLine[match(c(1, 2), ggDataLine$PANEL), "yintercept"]
		
		}, 
		expected = c(1, 3)
	)
	
})

test_that("Vertical lines are correctly set", {
			
	vLine <- c(1, 2)
	vLineColor <- c("green", "yellow")
	vLineLty <- c("dashed", "solid")
	gg <- subjectProfileSummaryPlot(
		data = data.frame(
			visit = c(1, 2), 
			statMean = rnorm(2)
		),
		xVar = "visit", 
		vLine = vLine,
		vLineColor = vLineColor,
		vLineLty = vLineLty
	)
	
	# extract data behind the lines
	isGeomLine <- sapply(gg$layers, function(l) inherits(l$geom, "GeomVline"))
	ggDataLine <- do.call(rbind, ggplot_build(gg)$data[isGeomLine])
	
	expect_equal(ggDataLine$xintercept, vLine)
	expect_equal(ggDataLine$colour, vLineColor)
	expect_equal(ggDataLine$linetype, vLineLty)
	
})

test_that("A table is successfully included in a plot", {
			
	summaryTable <- data.frame(
		visit = c(1, 2), 
		statMean = rnorm(2),
		n = c(10, 20),
		TRT = c("A", "B")
	)
			
	# ggplot2: a (expected) warning is created
	# because geom_point is used with size = NA to 
	# avoid the 'a' in the legend
	withCallingHandlers(
		expr = {
			gg <- subjectProfileSummaryPlot(
				data = summaryTable, 
				xVar = "visit",
				tableText = "n"
			)
		},
		warning = function(w){
			if(grepl("missing values.*geom_point", conditionMessage(w)))
				invokeRestart("muffleWarning")
		}
	)
	
	expect_s3_class(gg, "ggplot")
	ggData <- ggplot_build(gg)$data
	expect_length(ggData, 2) # plot + table
	
})

test_that("A table with specific height is correctly included in a plot", {
			
	summaryTable <- data.frame(
		visit = c(1, 2), 
		statMean = rnorm(2),
		n = c(10, 20)
	)

	tableHeight <- 0.45
	
	# ggplot2: a (expected) warning is created
	# because geom_point is used with size = NA to 
	# avoid the 'a' in the legend
	withCallingHandlers(
		expr = {
			gg <- subjectProfileSummaryPlot(
				data = summaryTable, 
				xVar = "visit",
				tableText = "n",
				tableHeight = tableHeight
			)
		},
		warning = function(w){
			if(grepl("missing values.*geom_point", conditionMessage(w)))
				invokeRestart("muffleWarning")
		}
	)
	
	ggData <- ggplot_build(gg)$data
	ggData <- do.call(plyr::rbind.fill, ggData)
	
	# 2 panels are created
	expect_equal(nrow(ggData), 2)
	
	# check that created panels have correct height:
	gDataYCoord <- as.list(as.data.frame(t(ggData[, c("ymin", "ymax")])))
	gDataYCoord <- gDataYCoord[order(sapply(gDataYCoord, min))]
	expect_setequal(
		object = gDataYCoord,
		expected = list(c(0, 0.45), c(0.45, 1))
	)
	
})

test_that("An error is generated if the height for the table is not correctly specified", {
			
	summaryTable <- data.frame(
		visit = c(1, 2), 
		statMean = rnorm(2),
		n = c(10, 20)
	)
			
	expect_error(
		subjectProfileSummaryPlot(
			data = summaryTable, 
			xVar = "visit",
			tableText = "n",
			tableHeight = 1.5
		),
		"Table height should be between 0 and 1."
	)
			
})

test_that("A warning is generated if the facet and text variables are specified", {
			
	summaryTable <- data.frame(
		visit = c(1, 2),
		PARAM = c("AAA", "ZZZ"),
		statMean = rnorm(2),
		n = c(1, 2)
	)	
			
	expect_warning(
		subjectProfileSummaryPlot(
			data = summaryTable,
			xVar = "visit",
			tableText = "n",
			facetVar = "PARAM"
		),
		"Table cannot be used in combination with 'facetVar', no table is included."
	)
	
})

test_that("Extra ggplot with a data point outside the plot range is correctly included", {
  
  summaryTable <- data.frame(
    visit = c(1, 2), 
    statMean = c(2, 3),
    statSE = c(0.1, 0.2)
  )
  
  dataExtra <- data.frame(visit = 4, y = 2)
  ggExtra <- ggplot2::geom_point(ggplot2::aes(x = visit, y = y), data = dataExtra)
  gg <- subjectProfileSummaryPlot(
    data = summaryTable,
    xVar = "visit", 
    ggExtra = ggExtra
  )
  xScales <- ggplot_build(gg)$layout$panel_scales_x[[1]]
  # test that the data point from ggExtra is included inside plot limits
  expect_gte(object = max(xScales$limits), expected = 4)
  
})

test_that("Extra ggplot specified as a function is correctly included", {
  
  summaryTable <- data.frame(
    visit = c(1, 2), 
    statMean = c(2, 3),
    statSE = c(0.1, 0.2)
  )
  
  ggExtra <- function(gg){
    gg <- gg + 
      ggplot2::geom_vline(xintercept = 1) +
      ggplot2::geom_hline(yintercept = 2)
    return(gg)
  }
  gg <- subjectProfileSummaryPlot(
    data = summaryTable,
    xVar = "visit", 
    ggExtra = ggExtra
  )
  
  getDataGeom <- function(geom){
    isGeom <- sapply(gg$layers, function(l) inherits(l$geom, geom))
    data <- do.call(rbind, ggplot_build(gg)$data[isGeom])
    return(data)
  }
  expect_equal(getDataGeom(geom = "GeomVline")$xintercept, 1)
  expect_equal(getDataGeom(geom = "GeomHline")$yintercept, 2)
  
})

context("Create a subject profile summary plot by a variable")

library(ggplot2)

test_that("A plot is correctly created by a variable", {
			
	summaryTable <- data.frame(
		visit = c(1, 2, 1, 2), 
		TRT = factor(
			c("A", "A", "B", "B"), 
			levels = c("B", "A")
		),
		statMean = rnorm(4)
	)
	
	res <- subjectProfileSummaryPlot(
		data = summaryTable,
		xVar = "visit", 
		byVar = "TRT"
	)
			
	groups <- levels(summaryTable$TRT)
	expect_named(res, groups)
			
	# compare the data behind the plot
	for(group in groups){
		
		expect_identical(
				
			object = ggplot_build(res[[!!group]])$data, 
				
			expected = {
				ggGroup <- subjectProfileSummaryPlot(
					data = subset(summaryTable, TRT == !!group),
					xVar = "visit",
					yLab = paste("Mean", !!group)
				)
				ggplot_build(ggGroup)$data
			}
		
		)
		
	}
	
})

test_that("A warning is generated if the 'by' variable is not available in the data", {
			
	summaryTable <- data.frame(
		visit = c(1, 2, 1, 2), 
		TRT = factor(
			c("A", "A", "B", "B"), 
			levels = c("B", "A")
		),
		statMean = rnorm(4)
	)
			
	expect_warning(
		subjectProfileSummaryPlot(
			data = summaryTable,
			xVar = "visit", 
			byVar = "TRT1"
		),
		"'byVar' is not available in the 'data'"
	)
			
})

test_that("Titles are correctly set by group of a variable", {
			
	summaryTable <- data.frame(
		visit = c(1, 2, 1, 2), 
		TRT = factor(
			c("A", "A", "B", "B"),
			levels = c("B", "A")
		),
		statMean = rnorm(4)
	)

	title <- c(A = "Treatment A", B = "")	
	res <- subjectProfileSummaryPlot(
		data = summaryTable,
		xVar = "visit", 
		byVar = "TRT",
		title = title
	)
	
	for(group in levels(summaryTable$TRT)){
		expect_equal(res[[!!group]]$labels$title, title[[!!group]])
	}
	
})

test_that("Labels for the y-axis are correctly set by group of a variable", {
			
	summaryTable <- data.frame(
		visit = c(1, 2, 1, 2), 
		TRT = factor(
			c("A", "A", "B", "B"),
			levels = c("B", "A")
		),
		statMean = rnorm(4)
	)
			
	yLab <- c(A = "Mean treat A", B = "Mean treat B")
			
	res <- subjectProfileSummaryPlot(
		data = summaryTable,
		xVar = "visit", 
		byVar = "TRT",
		yLab = yLab
	)
			
	for(group in levels(summaryTable$TRT)){
		expect_equal(res[[!!group]]$labels$y, yLab[[!!group]])
	}
	
})

test_that("Vertical and horizontal lines are correctly set by group of a variable", {
			
	summaryTable <- data.frame(
		visit = c(1, 2, 1, 2), 
		TRT = factor(c("A", "A", "B", "B"), levels = c("B", "A")),
		statMean = rnorm(4)
	)
			
	hLine <- list(A = 1, B = NULL)
	vLine <- list(A = 2, B = 1)
	res <- subjectProfileSummaryPlot(
		data = summaryTable,
		xVar = "visit", 
		byVar = "TRT",
		hLine = hLine, vLine = vLine
	)
	
	for(group in levels(summaryTable$TRT)){
			
		expect_equal({
						
			ggGroup <- res[[!!group]]
			isGeomHLine <- sapply(ggGroup$layers, function(l) inherits(l$geom, "GeomHline"))
			
			if(!is.null(hLine[[!!group]])){
				layer_data(ggGroup, which(isGeomHLine))$yintercept
			}
			
			}, 
			expected = hLine[[!!group]]
		)
		
		expect_equal({
					
			ggGroup <- res[[!!group]]
			isGeomVLine <- sapply(ggGroup$layers, function(l) inherits(l$geom, "GeomVline"))
					
			if(!is.null(vLine[[!!group]])){
				layer_data(ggGroup, which(isGeomVLine))$xintercept
			}
					
			}, 
			expected = vLine[[!!group]]
		)
	
	}
			
})

test_that("Extra ggplot specifications are correctly set by group of a variable", {
			
	summaryTable <- data.frame(
		visit = c(1, 2, 1, 2), 
		TRT = factor(
			c("A", "A", "B", "B"),
			levels = c("B", "A")
		),
		statMean = rnorm(4)
	)
		
	legPos <- c(A = "right", B = "left")
	ggExtra <- sapply(legPos, function(x)
		theme(legend.position = x),
		simplify = FALSE
	)
	res <- subjectProfileSummaryPlot(
		data = summaryTable,
		xVar = "visit", 
		byVar = "TRT",
		ggExtra = ggExtra
	)
	
	for(group in levels(summaryTable$TRT)){
		
		expect_equal(
			object = res[[!!group]]$theme$legend.position,
			expected = legPos[[!!group]]
		)
		
	}
	
})				

context("Create a subject profile summary plot with labels")

library(ggplot2)

test_that("A label is correctly specified for a x variable", {
			
	summaryTable <- data.frame(
		visit = c(1, 2), 
		statMean = rnorm(2)
	)
	xLab <- "Study visit"
	gg <- subjectProfileSummaryPlot(
		data = summaryTable, xVar = "visit",
		xLab = xLab
	)
	expect_identical(
		object = gg$labels$x,
		expected = xLab
	)
	
})

test_that("The label for the y variable is correctly set by default", {
			
	summaryTable <- data.frame(
		visit = c(1, 2), 
		statMean = rnorm(2)
	)
	# by default: X is used as a title from xVar = stat[X] 
	gg <- subjectProfileSummaryPlot(
		data = summaryTable, xVar = "visit"
	)
	expect_identical(
		object = gg$labels$y, 
		expected = "Mean"
	)

})

test_that("A label is correctly set for a y variable", {
			
	summaryTable <- data.frame(
		visit = c(1, 2), 
		statMean = rnorm(2)
	)
	yLab <- "Mean of the actual values"
	gg <- subjectProfileSummaryPlot(
		data = summaryTable, xVar = "visit",
		yLab = yLab
	)
	expect_identical(
		object = gg$labels$y, 
		expected = yLab
	)
	
})

test_that("A title is correctly set", {
			
	summaryTable <- data.frame(
		visit = c(1, 2), 
		statMean = rnorm(2)
	)	
			
	title <- "Example of summary plot"
	gg <- subjectProfileSummaryPlot(
		data = summaryTable, 
		xVar = "visit",
		title = title
	)
	expect_equal(
		object = gg$labels$title,
		expected = title
	)
			
})

test_that("A caption is correctly set", {
			
	summaryTable <- data.frame(
		visit = c(1, 2), 
		statMean = rnorm(2)
	)	
	
	caption <- "This plot has been created by the in-text package."
	gg <- subjectProfileSummaryPlot(
		data = summaryTable, 
		xVar = "visit",
		caption = caption
	)
	expect_equal(
		object = gg$labels$caption,
		expected = caption
	)
	
})

test_that("The variable labels are correctly extracted from the labels of all variables", {
			
	summaryTable <- data.frame(
		visit = c(1, 1, 2, 2), 
		TRT = c("A", "B", "A", "B"),
		statMean = rnorm(4)
	)
	
	labelVars <- c(visit = "Study visit", TRT = "Study treatment")
	
	gg <- subjectProfileSummaryPlot(
		data = summaryTable, 
		xVar = "visit",
		colorVar = "TRT",
		labelVars = labelVars
	)
	
	expect_equal(gg$labels$x, labelVars["visit"])
	
	ggScales <- gg$scales$scales
	isColorAes <- sapply(ggScales, function(x) 
		all(x[["aesthetics"]] == "colour")
	)
	expect_equal(sum(isColorAes), 1)
	expect_equal(ggScales[[which(isColorAes)]]$name, labelVars["TRT"])
	
})

context("Create a subject profile summary plot with specific theme")

library(ggplot2)

test_that("The point size is correctly set", {
			
	summaryTable <- data.frame(
		visit = c(1, 2), 
		statMean = rnorm(2)
	)
	sizePoint <- 4
	gg <- subjectProfileSummaryPlot(
		data = summaryTable, 
		xVar = "visit",
		sizePoint = sizePoint
	)
	# extract data behind the points
	isGeomPoint <- sapply(gg$layers, function(l) inherits(l$geom, "GeomPoint"))
	ggDataPoint <- layer_data(gg, which(isGeomPoint))
	expect_setequal(ggDataPoint$size, sizePoint)
			
})

test_that("The line size is correctly set", {
			
	summaryTable <- data.frame(
		visit = c(1, 2), 
		statMean = rnorm(2)
	)
	sizeLine <- 5
	gg <- subjectProfileSummaryPlot(
		data = summaryTable, 
		xVar = "visit",
		sizeLine = sizeLine
	)
	# extract data behind the lines
	isGeomLine <- sapply(gg$layers, function(l) inherits(l$geom, "GeomLine"))
	ggDataLine <- layer_data(gg, which(isGeomLine))
	aesLineSize <- ifelse(packageVersion("ggplot2") >= "3.4.0", "linewidth", "size")
	expect_setequal(ggDataLine[[aesLineSize]], sizeLine)
	
})

test_that("The label size is correctly set", {
			
	summaryTable <- data.frame(
		visit = c(1, 2), 
		statMean = rnorm(2)
	)	
			
	sizeLabel <- 6
	gg <- subjectProfileSummaryPlot(
		data = summaryTable, 
		xVar = "visit",
		label = TRUE, 
		sizeLabel = sizeLabel
	)
			
	# extract data behind the text
	isGeomText <- sapply(gg$layers, function(l) inherits(l$geom, "GeomTextRepel"))
	ggDataText <- layer_data(gg, which(isGeomText))
	expect_setequal(ggDataText$size, sizeLabel)
		
})

context("Create a subject profile summary plot with a x variable")

library(ggplot2)
library(plyr)

test_that("A plot is correctly created with a continuous x variable", {
			
	summaryTable <- data.frame(
		visit = c(1, 2), 
		statMean = rnorm(2)
	)
			
	gg <- subjectProfileSummaryPlot(
		data = summaryTable, 
		xVar = "visit"
	)
	
	ggData <- ggplot_build(gg)$data
	
	# combine across layers
	ggDataAll <- do.call(plyr::rbind.fill, ggData)
	ggDataAll <- unique(ggDataAll[, c("x", "y")])
	
	expect_equal(
		object = ggDataAll, 
		expected = summaryTable, 
		check.attributes = FALSE
	)
	
})

test_that("A plot is successfully created with a continuous x variable with only one element", {
			
	expect_s3_class(
		subjectProfileSummaryPlot(
			data = data.frame(
				visit = 1, 
				statMean = rnorm(1)
			), 
			xVar = "visit"
		),
		"ggplot"
	)
			
})

test_that("A plot is correctly created with a discrete x variable", {
			
	summaryTable <- data.frame(
		visit = factor(c("B", "A")), 
		statMean = rnorm(2)
	)
			
	gg <- subjectProfileSummaryPlot(
		data = summaryTable, 
		xVar = "visit"
	)
			
	ggData <- ggplot_build(gg)$data
			
	# combine across layers
	ggDataAll <- do.call(plyr::rbind.fill, ggData)
	ggDataAll <- unique(ggDataAll[, c("x", "y")])
			
	summaryTable$visitN <- as.numeric(summaryTable$visit)
	summaryTable <- summaryTable[order(summaryTable$visitN), ]
	ggDataAll$x <- as.numeric(ggDataAll$x) # x is also of type: 'mapped_discrete'
	expect_equal(
		object = ggDataAll, 
		expected = summaryTable[, c("visitN", "statMean")], 
		check.attributes = FALSE
	)
		
})

test_that("The x-axis labels are correctly set", {
			
	summaryTable <- data.frame(
		visit = c(1, 2), 
		statMean = rnorm(2)
	)
	xAxisLabs <- c("Visit 1" = 2, "Baseline" = 1)
	
	gg <- subjectProfileSummaryPlot(
		data = summaryTable, 
		xVar = "visit",
		xAxisLabs = xAxisLabs
	)
	
	# extract labels from the ggplot object
	ggScales <- gg$scales$scales
	isScaleX <- sapply(ggScales, function(x) 
		"x" %in% x[["aesthetics"]]
	)
	ggScaleX <- gg$scales$scales[[which(isScaleX)]]
	ggXAxisLabs <- setNames(ggScaleX$breaks, ggScaleX$labels)
	expect_equal(
		object = ggXAxisLabs, 
		expected = xAxisLabs
	)
	
})

test_that("A x-axis gap is correctly set", {

	summaryTable <- data.frame(
		visit = c(1, 2, 3), 
		statMean = rnorm(3)
	)
	xGap <- c(1, 3)
	gg <- subjectProfileSummaryPlot(
		data = summaryTable,
		xVar = "visit", xGap = xGap
	)
	ggData <- ggplot_build(gg)$data
	
	# a '//' symbol is included in the x-axis
	idxLabel <- which(
		sapply(ggData, function(x) 
			"label" %in% colnames(x) && x$label == "//")
	)
	expect_length(idxLabel, 1)
	expect_equal(ggData[[idxLabel]]$x, 2)
	expect_equal(ggData[[idxLabel]]$y, -Inf)
	
	# vertical lines are included
	idxVLine <- which(
		sapply(ggData, function(x) 
			"xintercept" %in% colnames(x)
		)
	)
	expect_length(idxVLine, 1)
	expect_equal(ggData[[idxVLine]]$xintercept, xGap)
	
	# check that data is correctly filtered:
	ggDataPlot <- ggData[-c(idxVLine, idxLabel)]
	ggDataPlot <- lapply(ggDataPlot, `[`, c("x", "y"))
	ggDataPlot <- unique(do.call(rbind, ggDataPlot))
	expect_equal(
		object = ggDataPlot,
		expected = setNames(summaryTable[-2, ], c("x", "y")),
		check.attributes = FALSE
	)
	
})

test_that("A warning is generated if a gap is requested for the x-axis but the x variable is not continuous", {

	summaryTable <- data.frame(
		visit = c("1", "2"), 
		statMean = rnorm(2)
	)			
	
	expect_warning(
		subjectProfileSummaryPlot(
			data = summaryTable,
			xVar = "visit",
			xGap = c(1, 2)
		),
		"'xGap' should only be specified for continuous x-variable"
	)
	
})

test_that("A warning is generated if a gap is requested for the x-axis but the x variable is not specified", {

	summaryTable <- data.frame(
		visit = c("1", "2"), 
		statMean = rnorm(2)
	)
	expect_warning(
		subjectProfileSummaryPlot(
			data = summaryTable,
			colorVar = "visit",
			xGap = c(1, 2)
		),
		"'xGap' should only be specified if 'xVar' is specified"
	)		

})

test_that("A warning is generated if a x-axis gap is not of length 2", {

	summaryTable <-  data.frame(
		visit = c(1, 2), 
		statMean = rnorm(2)
	)
	expect_warning(
		subjectProfileSummaryPlot(
			data = summaryTable,
			xVar = "visit",
			xGap = 1
		),
		"'xGap' should be of length 2"
	)	

})

test_that("The range of the x-axis gap is correctly set to a specified value", {
	
	summaryTable <- data.frame(
		visit = c(1, 2, 3), 
		statMean = rnorm(3)
	)
	xGap <- c(1, 2)
	gg <- subjectProfileSummaryPlot(
		data = summaryTable,
		xVar = "visit", 
		xGap = xGap, xGapDiffNew = 0.5
	)
	ggData <- ggplot_build(gg)$data
	
	# a '//' symbol is included in the x-axis
	idxLabel <- which(
		sapply(ggData, function(x) 
			"label" %in% colnames(x) && x$label == "//")
	)
	expect_length(idxLabel, 1)
	expect_equal(ggData[[idxLabel]]$x, 1.25)
	
	# check that data is correctly filtered:
	ggDataPlot <- ggData[-c(idxLabel)]
	ggDataX <- lapply(ggDataPlot, function(x){
		if("x" %in% colnames(x))
			x[, c("x")]
	})
	ggDataX <- unique(unlist(ggDataX))
	expect_equal(
		object = ggDataX,
		expected = c(1, 1.5, 2.5)
	)
	
})
		
test_that("A jitter is correctly set for the x-axis ", {
		
	summaryTable <- data.frame(
		visit = c(1, 1, 2, 2), 
		TRT = c("A", "B", "A", "B"),
		statMean = rnorm(4)
	)
		
	jitter <- 1
	gg <- subjectProfileSummaryPlot(
		data = summaryTable,
		xVar = "visit", 
		colorVar = "TRT",
		jitter = jitter
	)
		
	ggDataAll <- do.call(plyr::rbind.fill, ggplot_build(gg)$data)
	ggXJitter <- unique(with(ggDataAll, xmax-xmin)*2)
	expect_equal(
		object = ggXJitter, 
		expected = jitter
	)
		
})
		
test_that("The limits are correctly set for the x-axis", {
	
	summaryTable <- data.frame(
		visit = c(1, 2), 
		statMean = rnorm(2)
	)
	
	xLim <- c(1, 10)
	gg <- subjectProfileSummaryPlot(
		data = summaryTable, 
		xVar = "visit",
		xLim = xLim
	)
	expect_equal(
		object = ggplot_build(gg)$layout$coord$limits$x, 
		expected = xLim
	)		
	
})

test_that("The x-axis is correctly expanded", {
			
	summaryTable <- data.frame(
		visit = c(1, 2), 
		statMean = rnorm(2)
	)
			
	xAxisExpand <- expansion(mult = 4, add = 0)
	gg <- subjectProfileSummaryPlot(
		data = summaryTable, 
		xVar = "visit",
		xAxisExpand = xAxisExpand
	)
	
	# extract labels from the ggplot object
	ggScales <- gg$scales$scales
	isScaleX <- sapply(ggScales, function(x) 
		"x" %in% x[["aesthetics"]]
	)
	expect_equal(
		object = ggScales[[which(isScaleX)]]$expand, 
		expected = xAxisExpand
	)
	
})

context("Create a subject profile summary plot with a table")

library(ggplot2)
library(plyr)

test_that("A text variable is correctly set", {
		
	summaryTable <- data.frame(
		visit = c(1, 2),
		n = c(10, 20)
	)		
	
	gg <- subjectProfileSummaryTable(
		data = summaryTable,
		xVar = "visit",
		text = "n"
	)
	
	expect_s3_class(gg, "ggplot")
	
	# extract data behind the text
	isGeomText <- sapply(gg$layers, function(l) inherits(l$geom, "GeomText"))
	ggDataText <- layer_data(gg, which(isGeomText))
	
	expect_identical(
		object = unname(ggDataText[, c("x", "label")]),
		expected = unname(summaryTable)
	)
				
})

test_that("An error is generated if the text variable is not available", {
			
	summaryTable <- data.frame(
		visit = c(1, 2),
		n = c(10, 20)
	)		
			
	expect_error(
		subjectProfileSummaryTable(
			data = summaryTable,
			xVar = "visit",
			text = "n2"
		),
		"'n2' should be among the columns of 'data'"
	)
	
})

test_that("A text variable is correctly set as an expression", {
			
	summaryTable <- data.frame(
		visit = c(1, 2),
		n = c(10, 20),
		TRT = c("A", "B")
	)		
			
	textExpr <- bquote(paste("# patients:", n, "\ntreatment:", TRT))
	gg <- subjectProfileSummaryTable(
		data = summaryTable,
		xVar = "visit",
		text = textExpr
	)
	
	# extract data behind the text
	isGeomText <- sapply(gg$layers, function(l) inherits(l$geom, "GeomText"))
	ggDataText <- layer_data(gg, which(isGeomText))
			
	summaryTable$label <- with(summaryTable, eval(textExpr))
	expect_identical(
		object = unname(ggDataText[, c("x", "label")]),
		expected = unname(summaryTable[, c("visit", "label")])
	)
			
})

test_that("The size of the text is correctly set", {
			
	summaryTable <- data.frame(
		visit = c(1, 2),
		n = c(10, 20)
	)		
			
	textSize <- 67
	gg <- subjectProfileSummaryTable(
		data = summaryTable,
		xVar = "visit",
		text = "n", textSize = textSize
	)
	
	# extract data behind the text
	isGeomText <- sapply(gg$layers, function(l) inherits(l$geom, "GeomText"))
	ggDataText <- layer_data(gg, which(isGeomText))
	
	expect_setequal(
		object = ggDataText$size, 
		expected = textSize
	)
		
})

test_that("The label for the x variable is correctly set", {
			
	summaryTable <- data.frame(
		visit = c(1, 2), 
		n = c(10, 20)
	)
	xLab <- "Study visit"
	gg <- subjectProfileSummaryTable(
		data = summaryTable, 
		xVar = "visit",
		text = "n",
		xLab = xLab
	)
	expect_identical(
		object = gg$labels$x,
		expected = xLab
	)
			
})

test_that("The x-axis labels are correctly set for a continuous x variable", {
			
	summaryTable <- data.frame(
		visit = c(1, 2), 
		n = c(10, 20)
	)
	xAxisLabs <- c(1, 4)
	
	gg <- subjectProfileSummaryTable(
		data = summaryTable, 
		xVar = "visit",
		text = "n",
		xAxisLabs = xAxisLabs
	)
			
	# extract labels from the ggplot object
	ggScales <- gg$scales$scales
	isScaleX <- sapply(ggScales, function(x) 
		"x" %in% x[["aesthetics"]]
	)
	expect_equal(
		object = gg$scales$scales[[which(isScaleX)]]$limits, 
		expected = xAxisLabs
	)
	
})

test_that("The x-axis labels are correctly set for a categorical x variable", {
			
	# Note: these labels are not displayed by default
	# but still set in the ggplot object
	summaryTable <- data.frame(
		visit = c("Visit 0", "Visit 1"), 
		n = c(10, 20)
	)
	xAxisLabs <- c(
		`Visit 0` = "Baseline", 
		"Visit 1" = "First visit"
	)
			
	gg <- subjectProfileSummaryTable(
		data = summaryTable, 
		xVar = "visit",
		text = "n",
		xAxisLabs = xAxisLabs
	)
			
	# extract labels from the ggplot object
	ggScales <- gg$scales$scales
	isScaleX <- sapply(ggScales, function(x) 
		"x" %in% x[["aesthetics"]]
	)
	expect_equal(
		object = gg$scales$scales[[which(isScaleX)]]$breaks, 
		expected = xAxisLabs
	)
			
})


test_that("The limits are correctly set for the x-axis", {
			
	summaryTable <- data.frame(
		visit = c(1, 2), 
		n = c(10, 20)
	)
			
	xLim <- c(1, 10)
	gg <- subjectProfileSummaryTable(
		data = summaryTable, 
		xVar = "visit",
		text = "n",
		xLim = xLim
	)
	expect_equal(
		object = ggplot_build(gg)$layout$coord$limits$x,
		expected = xLim
	)		
			
})

test_that("The labels of the color variable are correctly displayed in the y-axis", {
		
	summaryTable <- data.frame(
		visit = c(1, 1, 2, 2),
		n = c("1", "2", "3", "4"),
		TRT = factor(c("A", "B", "A", "B"), levels = c("B", "A"))
	)	
	
	gg <- subjectProfileSummaryTable(
		data = summaryTable,
		xVar = "visit",
		text = "n",
		colorVar = "TRT"
	)
	
	## check if labels are based on color var
	
	# extract data behind the text
	isGeomText <- sapply(gg$layers, function(l) inherits(l$geom, "GeomText"))
	ggDataText <- layer_data(gg, which(isGeomText))
	ggDataText$y <- as.numeric(ggDataText$y) # ggplot set target to 'mapped_discrete' class
	ggDataText <- ggDataText[with(ggDataText, order(x, y)), ]
	
	dataPlotReference <- data.frame(
		x = c(1, 1, 2, 2),
		y = c(1, 2, 1, 2), # 1: bottom (last level), 2: top (first level)
		label = c("1", "2", "3", "4")
	)
	# check that correct data is displayed (and in the correct order)
	expect_equal(
		object = ggDataText[, c("x", "y", "label")],
		expected = dataPlotReference,
		check.attributes = FALSE
	)
	
})
	
test_that("The text and point are colored based on the specified color variable", {
			
	summaryTable <- data.frame(
		visit = c(1, 1, 2, 2),
		n = sample.int(4),
		TRT = factor(c("A", "B", "A", "B"), levels = c("B", "A"))
	)	
			
	gg <- subjectProfileSummaryTable(
		data = summaryTable,
		xVar = "visit",
		text = "n",
		colorVar = "TRT"
	)
	
	## color is different for the groups for the text and point (used for the legend)
	
	isGeomTextPoint <- sapply(gg$layers, function(l) inherits(l$geom, c("GeomText", "GeomPoint")))
	ggDataTextPoint <- do.call(plyr::rbind.fill, ggplot_build(gg)$data[isGeomTextPoint])
			
	colors <- with(ggDataTextPoint, tapply(colour, y, unique))
	expect_type(colors, "character")
	expect_length(colors, 2)
	expect_length(unique(colors), 2)
			
})

test_that("A color palette is correctly set", {
			
	summaryTable <- data.frame(
		visit = c(1, 1, 2, 2),
		n = sample.int(4),
		TRT = c("A", "B", "A", "B"),
		stringsAsFactors = TRUE
	)	
			
	colorPalette <- c(A = "red", B = "yellow")
	gg <- subjectProfileSummaryTable(
		data = summaryTable,
		xVar = "visit",
		text = "n",
		colorVar = "TRT", colorPalette = colorPalette
	)
			
	# levels of the color variable are sorted from the top (high y) to the bottom (low y)
	summaryTable$y <- as.numeric(factor(summaryTable$TRT, levels = rev(levels(summaryTable$TRT))))
	
	# extract data behind point and text:
	isGeomTextPoint <- sapply(gg$layers, function(l) inherits(l$geom, c("GeomText", "GeomPoint")))
	ggDataTextPoint <- do.call(plyr::rbind.fill, ggplot_build(gg)$data[isGeomTextPoint])
			
	ggDataTextPointWithInput <- merge(ggDataTextPoint, summaryTable,
		by.x = c("x", "y"),
		by.y = c("visit", "y"),
		all = TRUE
	)
			
	colors <- with(ggDataTextPointWithInput, tapply(colour, TRT, unique))
	expect_equal(
		object = as.vector(colors[names(colorPalette)]), 
		expected = unname(colorPalette)
	)
			
})

test_that("A label for the color variable is correctly set", {
			
	summaryTable <- data.frame(
		visit = c(1, 1, 2, 2),
		n = sample.int(4),
		TRT = c("A", "B", "A", "B")
	)	
	colorLab <- "Study Treatment"
			
	gg <- subjectProfileSummaryTable(
		data = summaryTable,
		xVar = "visit", 
		text = "n",
		colorVar = "TRT",
		colorLab = colorLab
	)
			
	# extract color scale
	ggScales <- gg$scales$scales
	isColorAes <- sapply(ggScales, function(x) 
		all(x[["aesthetics"]] == "colour")
	)
	expect_equal(
		object = sum(isColorAes), 
		expected = 1
	)
	expect_equal(
		object = ggScales[[which(isColorAes)]]$name, 
		expected = colorLab
	)
			
})

test_that("The size of the points (in the legend) is correctly set", {
			
	# Note: this affect the size of the points in the legend
	summaryTable <- data.frame(
		visit = c(1, 2),
		n = c(10, 20),
		TRT = c("a", "b")
	)
			
	pointSize <- 10
	gg <- subjectProfileSummaryTable(
		data = summaryTable,
		xVar = "visit",
		text = "n", 
		colorVar = "TRT",
		pointSize = pointSize
	)
	
	# for ggplot2 >= 3.5.0
	aesSize <- if (!inherits(ggplot2::guide_none(), "Guide")){
		gg$guides$colour$override.aes$size
	}else{
		gg$guides$guides$colour$params$override.aes$size
	}
	expect_equal(object = aesSize, expected = pointSize)
	
})

test_that("The variable labels are correctly extracted from the labels of all variables", {
			
	summaryTable <- data.frame(
		visit = c(1, 2), 
		n = c(10, 20),
		TRT = c("A", "B", "A", "B")
	)
	labelVars <- c(visit = "Study visit", TRT = "Study treatment")
	gg <- subjectProfileSummaryTable(
		data = summaryTable, 
		xVar = "visit",
		text = "n",
		colorVar = "TRT",
		labelVars = labelVars
	)
	
	# check label for coloring
	ggScales <- gg$scales$scales
	isColorAes <- sapply(ggScales, function(x) 
		all(x[["aesthetics"]] == "colour")
	)
	expect_equal(sum(isColorAes), 1)
	expect_equal(ggScales[[which(isColorAes)]]$name, labelVars["TRT"])
			
})

test_that("The labels of the y-axis are correctly included with a color variable as a factor", {
	
	summaryTable <- data.frame(
		visit = c(1, 2), 
		n = c(10, 20),
		TRT = factor(c("A", "B", "A", "B"), levels = c("B", "A", "C", "Z"))
	)
	
	colorPalette <- c(A = "red", B = "blue")
	
	# ggplot2: a (expected) warning is created
	# because of colors specified to text element
	withCallingHandlers(
		expr = {
			gg <- subjectProfileSummaryTable(
				data = summaryTable, 
				xVar = "visit",
				text = "n",
				colorVar = "TRT",
				colorPalette = colorPalette,
				yAxisLabs = TRUE
			)
		},
		warning = function(w){
			if(grepl("Vectorized input", conditionMessage(w)))
				invokeRestart("muffleWarning")
		}
	)
	
	expect_false(inherits(gg$theme$axis.text.y, "element_blank"))
	# check that color of labels are correct in the y-axis
	# Warning: labels are set from the lowest y (last level factor) to the highest y (first level factor)
	expect_equal(
		object = gg$theme$axis.text.y$colour,
		expected = c("red", "blue")
	)
	
})

test_that("A warning is generated if the labels for the y-axis are requested but no color variable is specified", {
			
	summaryTable <- data.frame(
		visit = c(1, 2), 
		n = c(10, 20),
		TRT = factor(c("A", "B", "A", "B"), levels = c("B", "A", "C", "Z"))
	)
			
	expect_warning(
		gg <- subjectProfileSummaryTable(
			data = summaryTable, 
			xVar = "visit",
			text = "n",
			yAxisLabs = TRUE
		),
		"Labels for the y-axis are not included because color variable is not specified."
	)
	
})

test_that("The labels of the y-axis are correctly included with a color variable as a character", {
			
	summaryTable <- data.frame(
		visit = c(1, 2), 
		n = c(10, 20),
		TRT = c("B", "A", "B", "A"),
		stringsAsFactors = FALSE
	)		
	colorPalette <- c(A = "red", B = "blue")
	
	# ggplot2: a (expected) warning is created
	# because of colors specified to text element
	withCallingHandlers(
		expr = {
			gg <- subjectProfileSummaryTable(
				data = summaryTable, 
				xVar = "visit",
				text = "n",
				colorVar = "TRT",
				colorPalette = colorPalette,
				yAxisLabs = TRUE
			)
		},
		warning = function(w){
			if(grepl("Vectorized input", conditionMessage(w)))
				invokeRestart("muffleWarning")
		}
	)

	# check that color of labels are correct in the y-axis
	# Warning: labels are set from the lowest y (last level factor) to the highest y (first level factor)
	expect_equal(
		object = gg$theme$axis.text.y$colour,
		expected = c("blue", "red")
	)
	
	# extract data behind the text
	isGeomText <- sapply(gg$layers, function(l) inherits(l$geom, "GeomText"))
	ggDataText <- layer_data(gg, which(isGeomText))
	expect_equal(
		unname(c(with(ggDataText, tapply(colour, y, unique)))),
		c("blue", "red")
	)
	
})

test_that("The labels of the y-axis are correctly not included", {

	summaryTable <- data.frame(
		visit = c(1, 2), 
		n = c(10, 20),
		TRT = c("A", "B", "A", "B")
	)
	
	gg <- subjectProfileSummaryTable(
		data = summaryTable, 
		xVar = "visit",
		text = "n",
		colorVar = "TRT",
		yAxisLabs = FALSE
	)
			
	# check if axis labels have been removed
	expect_s3_class(gg$theme$axis.text.y, "element_blank")
			
})

test_that("A color palette is correctly set for the labels of the y-axis ", {
			
	summaryTable <- data.frame(
		visit = c(1, 2), 
		n = c(10, 20),
		TRT = c("A", "B")
	)
	
	colorPalette <- c(A = "red", B = "blue")
	
	# ggplot2: a (expected) warning is created
	# because of colors specified to text element
	withCallingHandlers(
		expr = {
			gg <- subjectProfileSummaryTable(
				data = summaryTable, 
				xVar = "visit",
				text = "n",
				colorVar = "TRT",
				colorPalette = colorPalette,
				yAxisLabs = TRUE
			)
		},
		warning = function(w){
			if(grepl("Vectorized input", conditionMessage(w)))
				invokeRestart("muffleWarning")
		}
	)
	
	expect_equal(
		object = gg$theme$axis.text.y$colour,
		expected = c("blue", "red")
	)
			
})

test_that("The font size of the text is correctly set", {
			
	summaryTable <- data.frame(
		visit = c(1, 2),
		n = c(10, 20)
	)		
			
	fontsize <- 10
	gg <- subjectProfileSummaryTable(
		data = summaryTable,
		xVar = "visit",
		text = "n",
		fontsize = fontsize
	)
	expect_equal(
		object = gg$theme$text$size, 
		expected = fontsize
	)

})

test_that("The font face of the text is correctly set", {
      
	summaryTable <- data.frame(
		visit = c(1, 2),
		n = c(10, 20)
	)		
      
	fontface <- 3
	gg <- subjectProfileSummaryTable(
		data = summaryTable,
		xVar = "visit",
		text = "n",
		fontface = fontface
	)
	
	# extract data behind the text
	isGeomText <- sapply(gg$layers, function(l) inherits(l$geom, "GeomText"))
	ggDataText <- layer_data(gg, which(isGeomText))
	
	expect_setequal(
		object = ggDataText$fontface, 
		expected = fontface
	)
       
})

test_that("The font of the text is correctly set", {
			
	summaryTable <- data.frame(
		visit = c(1, 2),
		n = c(10, 20)
	)		
			
	fontname <- "Arial"
	gg <- subjectProfileSummaryTable(
		data = summaryTable,
		xVar = "visit",
		text = "n",
		fontname = fontname
	)
	expect_equal(
		object = gg$theme$text$family, 
		expected = fontname
	)
			
})

test_that("A theme is correctly set for the plot", {
			
	summaryTable <- data.frame(
		visit = c(1, 2),
		n = c(10, 20)
	)		
			
	gg <- subjectProfileSummaryTable(
		data = summaryTable,
		xVar = "visit",
		text = "n",
		themeFct = function()
			ggplot2::theme(aspect.ratio = 0.75)
	)
			
	expect_equal(
		object = gg$theme$aspect.ratio, 
		expected = 0.75
	)
			
})

test_that("A legend is correctly included", {
			
	summaryTable <- data.frame(
		visit = c(1, 1, 2, 2),
		n = sample.int(4),
		TRT = c("A", "B", "A", "B")
	)	
			
	gg <- subjectProfileSummaryTable(
		data = summaryTable,
		xVar = "visit", 
		text = "n",
		colorVar = "TRT",
		showLegend = TRUE
	)
	expect_false(gg$theme$legend.position == "none")
	
})

test_that("A legend is correctly not included", {
			
	summaryTable <- data.frame(
		visit = c(1, 1, 2, 2),
		n = sample.int(4),
		TRT = c("A", "B", "A", "B")
	)
	
	gg <- subjectProfileSummaryTable(
		data = summaryTable,
		xVar = "visit", 
		text = "n",
		colorVar = "TRT",
		showLegend = FALSE
	)
	expect_equal(
		object = gg$theme$legend.position,
		expected = "none"
	)

})

context("Create a subject profile summary plot with aesthetics")

library(ggplot2)
library(plyr)

test_that("A color variable is correctly set", {
			
	summaryTable <- data.frame(
		visit = c(1, 2, 1, 2), 
		TRT = c("A", "A", "B", "B"),
		statMean = rnorm(4),
		stringsAsFactors = TRUE
	)	
	
	gg <- subjectProfileSummaryPlot(
		data = summaryTable,
		xVar = "visit", 
		colorVar = "TRT"
	)
	ggData <- ggplot_build(gg)$data
	summaryTable$TRTN <- as.numeric(summaryTable$TRT)
	
	# combine across layers
	ggDataAll <- do.call(plyr::rbind.fill, ggData)
	
	ggDataWithInput <- merge(
		x = summaryTable, y = ggDataAll, 
		by.x = c("statMean", "TRTN"),
		by.y = c("y", "group"),
		all = TRUE
	)		
	colors <- with(ggDataWithInput, tapply(colour, TRT, unique))
	expect_type(colors, "character")
	expect_length(colors, 2)
	expect_length(unique(colors), 2)
	
})

test_that("A color palette is correctly set", {
			
	summaryTable <- data.frame(
		visit = c(1, 2, 1, 2), 
		TRT = factor(c("A", "A", "B", "B"), levels = c("B", "A")),
		statMean = rnorm(4)
	)	
	
	colorPalette <- c(A = "red", B = "yellow")
	gg <- subjectProfileSummaryPlot(
		data = summaryTable,
		xVar = "visit", 
		colorVar = "TRT",
		colorPalette = colorPalette
	)
	ggData <- ggplot_build(gg)$data
	summaryTable$TRTN <- as.numeric(summaryTable$TRT)
	
	# combine across layers
	ggDataAll <- do.call(plyr::rbind.fill, ggData)
	
	ggDataWithInput <- merge(
		x = summaryTable, y = ggDataAll, 
		by.x = c("statMean", "TRTN"),
		by.y = c("y", "group"),
		all = TRUE
	)
	
	colors <- with(ggDataWithInput, tapply(colour, TRT, unique))
	expect_type(colors, "character")
	expect_equal(as.vector(colors[names(colorPalette)]), unname(colorPalette))
	
})

test_that("A label for the color variable is correctly set", {
			
	summaryTable <- data.frame(
		visit = c(1, 2, 1, 2), 
		TRT = c("A", "A", "B", "B"), 
		statMean = rnorm(4)
	)	
	colorLab <- "Study Treatment"
		
	gg <- subjectProfileSummaryPlot(
		data = summaryTable,
		xVar = "visit", 
		colorVar = "TRT",
		colorLab = colorLab
	)
		
	# extract color scale
	ggScales <- gg$scales$scales
	isColorAes <- sapply(ggScales, function(x) 
		all(x[["aesthetics"]] == "colour")
	)
	expect_equal(sum(isColorAes), 1)
	expect_equal(ggScales[[which(isColorAes)]]$name, colorLab)
		
})

test_that("Line types are correctly used to differenciate the groups of the color variable", {
			
	summaryTable <- data.frame(
		visit = c(1, 2, 1, 2), 
		TRT = c("A", "A", "B", "B"),
		statMean = rnorm(4),
		stringsAsFactors = TRUE
	)	
			
	gg <- subjectProfileSummaryPlot(
		data = summaryTable,
		xVar = "visit", 
		colorVar = "TRT",
		useLinetype = TRUE
	)
	summaryTable$TRTN <- as.numeric(summaryTable$TRT)
	
	# extract data behind the lines
	isGeomLine <- sapply(gg$layers, function(l) inherits(l$geom, "GeomLine"))
	ggDataLine <- layer_data(gg, which(isGeomLine))
	
	ggDataLineWithInput <- merge(
		x = summaryTable, y = ggDataLine, 
		by.x = c("statMean", "TRTN"),
		by.y = c("y", "group"),
		all = TRUE
	)		

	ltys <- with(ggDataLineWithInput, tapply(linetype, TRT, unique, incomparable = NA_character_))
	expect_type(ltys, "character")
	expect_length(ltys, 2)
	expect_length(unique(ltys), 2)
	
})

test_that("A linetype palette is correctly set", {
			
	summaryTable <- data.frame(
		visit = c(1, 2, 1, 2), 
		TRT = factor(c("A", "A", "B", "B"), levels = c("B", "A")),
		statMean = rnorm(4)
	)	
	
	linetypePalette <- c(A = "dotted", B = "dashed")
	gg <- subjectProfileSummaryPlot(
		data = summaryTable,
		xVar = "visit", 
		colorVar = "TRT",
		useLinetype = TRUE,
		linetypePalette = linetypePalette
	)
	summaryTable$TRTN <- as.numeric(summaryTable$TRT)
	
	# extract data behind the lines
	isGeomLine <- sapply(gg$layers, function(l) inherits(l$geom, "GeomLine"))
	ggDataLine <- layer_data(gg, which(isGeomLine))
	
	ggDataLineWithInput <- merge(
		x = summaryTable, y = ggDataLine, 
		by.x = c("statMean", "TRTN"),
		by.y = c("y", "group"),
		all = TRUE
	)			
	
	ltys <- with(ggDataLineWithInput, tapply(linetype, TRT, unique))
	expect_type(ltys, "character")
	expect_equal(as.vector(ltys[names(linetypePalette)]), unname(linetypePalette))
	
})

test_that("Shapes are correctly used to differenciate the groups of the color variable", {
			
	summaryTable <- data.frame(
		visit = c(1, 2, 1, 2), 
		TRT = c("A", "A", "B", "B"),
		statMean = rnorm(4),
		stringsAsFactors = TRUE
	)	
	
	gg <- subjectProfileSummaryPlot(
		data = summaryTable,
		xVar = "visit", 
		colorVar = "TRT",
		useShape = TRUE
	)
	summaryTable$TRTN <- as.numeric(summaryTable$TRT)
	
	# extract data behind the points
	isGeomPoint <- sapply(gg$layers, function(l) inherits(l$geom, "GeomPoint"))
	ggDataPoint <- layer_data(gg, which(isGeomPoint))
	
	ggDataPointWithInput <- merge(
		x = summaryTable, y = ggDataPoint, 
		by.x = c("statMean", "TRTN"),
		by.y = c("y", "group"),
		all = TRUE
	)		
	
	shapes <- with(ggDataPointWithInput, tapply(shape, TRT, unique, incomparable = NA_character_))
	expect_type(shapes, "integer")
	expect_length(shapes, 2)
	expect_length(unique(shapes), 2)
	
})

test_that("A shape palette is correctly set", {
			
	summaryTable <- data.frame(
		visit = c(1, 2, 1, 2), 
		TRT = factor(c("A", "A", "B", "B"), levels = c("B", "A")),
		statMean = rnorm(4)
	)
	
	shapePalette <- c(A = 5, B = 9)
	gg <- subjectProfileSummaryPlot(
		data = summaryTable,
		xVar = "visit", 
		colorVar = "TRT",
		useShape = TRUE,
		shapePalette = shapePalette
	)
	summaryTable$TRTN <- as.numeric(summaryTable$TRT)
	
	# extract data behind the points
	isGeomPoint <- sapply(gg$layers, function(l) inherits(l$geom, "GeomPoint"))
	ggDataPoint <- layer_data(gg, which(isGeomPoint))
	
	ggDataPointWithInput <- merge(
		x = summaryTable, y = ggDataPoint, 
		by.x = c("statMean", "TRTN"),
		by.y = c("y", "group"),
		all = TRUE
	)	
	
	shapes <- with(ggDataPointWithInput, tapply(shape, TRT, unique, incomparable = NA_character_))
	expect_type(shapes, "double")
	expect_equal(as.vector(shapes[names(shapePalette)]), unname(shapePalette))
	
})

test_that("Points are correctly labelled with the y-variable values", {
			
	summaryTable <- data.frame(
		visit = c(1, 2), 
		statMean = rnorm(2)
	)	
	
	gg <- subjectProfileSummaryPlot(
		data = summaryTable, 
		xVar = "visit",
		label = TRUE
	)
	
	# extract data behind the text
	isGeomText <- sapply(gg$layers, function(l) inherits(l$geom, "GeomTextRepel"))
	ggDataText <- layer_data(gg, which(isGeomText))
	
	ggDataTextWithInput <- merge(
		x = summaryTable, y = ggDataText, 
		by.x = c("visit", "statMean"),
		by.y = c("x", "y"),
		all = TRUE
	)		
	# labels are set to 'statMean' by default
	with(ggDataTextWithInput, 
		expect_equal(object = label, expected = statMean)
	)
	
})

test_that("Points are correctly labelled with an expression of the data variables", {
			
	summaryTable <- data.frame(
		visit = c(1, 2), 
		statMean = rnorm(2)
	)	
	
	labelExpr <- bquote(paste(
		"Visit:", visit, "\n", "Mean:", 
		round(statMean, 2)
	))
	gg <- subjectProfileSummaryPlot(
		data = summaryTable, 
		xVar = "visit",
		label = labelExpr
	)
	
	# extract data behind the text
	isGeomText <- sapply(gg$layers, function(l) 
		inherits(l$geom, "GeomTextRepel"))
	ggDataText <- layer_data(gg, which(isGeomText))
	
	ggDataTextWithInput <- merge(
		x = summaryTable, y = ggDataText, 
		by.x = c("visit", "statMean"),
		by.y = c("x", "y"),
		all = TRUE
	)		
	
	with(ggDataTextWithInput, expect_equal(
		object = label, 
		expected = eval(labelExpr)
	))
	
})
test_that("An error is generated if the label for the points is not correctly specified", {
	
	summaryTable <- data.frame(
		visit = c(1, 2), 
		statMean = c(4, 5)
	)	
			
	expect_error(
		subjectProfileSummaryPlot(
			data = summaryTable, 
			xVar = "visit",
			label = list(a = bquote(statMean))
		),
		"label.*should contain at least 'textLabel'"
	)
			
})

test_that("An error is generated if the text for the label for the points is not an expression", {
	
	summaryTable <- data.frame(
		visit = c(1, 2), 
		statMean = c(4, 5)
	)
	expect_error(
		subjectProfileSummaryPlot(
			data = summaryTable, 
			xVar = "visit",
			label = list(textLabel = "blabla")
		),
		"label.*should be a list of expressions"
	)
	
})

test_that("Points are correctly labelled with a text justified based on data variables", {
			
	summaryTable <- data.frame(
		visit = c(1, 2), 
		statMean = c(4, 5)
	)
	
	labelExpr <- list(
		textLabel = bquote(paste("Mean:", round(statMean, 2))),
		textHjust = bquote(ifelse(visit == 1, -1, 1)),
		textVjust = bquote(ifelse(statMean == 4, 1, -1))
	)
	gg <- subjectProfileSummaryPlot(
		data = summaryTable, 
		xVar = "visit",
		label = labelExpr
	)
	
	# extract data behind the text
	isGeomText <- sapply(gg$layers, function(l) 
		inherits(l$geom, "GeomText"))
	ggDataText <- layer_data(gg, which(isGeomText))
	
	ggDataTextWithInput <- merge(
		x = summaryTable, y = ggDataText, 
		by.x = c("visit", "statMean"),
		by.y = c("x", "y"),
		all = TRUE
	)		
	
	# labels are correctly extracted
	with(ggDataTextWithInput, expect_equal(
		object = label, 
		expected = eval(labelExpr[["textLabel"]])
	))
	
	# horizontal justification is correct
	with(ggDataTextWithInput, expect_equal(
		object = hjust, 
		expected = eval(labelExpr[["textHjust"]])
	))
	
	# vertical justification is correct
	with(ggDataTextWithInput, expect_equal(
		object = vjust, 
		expected = eval(labelExpr[["textVjust"]])
	))
	
})

test_that("The padding between points and labels is correctly set when specified", {
			
	summaryTable <- data.frame(
		visit = c(1, 2), 
		statMean = rnorm(2)
	)	
			
	labelPadding <- unit(2, "cm")
	gg <- subjectProfileSummaryPlot(
		data = summaryTable, 
		xVar = "visit",
		label = TRUE,
		labelPadding = labelPadding
	)
			
	# extract data behind the text
	isGeomText <- sapply(gg$layers, function(l) inherits(l$geom, "GeomTextRepel"))
	ggDataText <- layer_data(gg, which(isGeomText))
	expect_identical(
		gg$layers[[which(isGeomText)]]$geom_params$point.padding,
		labelPadding
	)
	
})

context("Create a subject profile summary plot with a y variable")

library(ggplot2)
library(plyr)

test_that("The mean data is correctly set", {
			
	summaryTable <- data.frame(
		visit = c(1, 2), 
		statMean1 = rnorm(2)
	)	
	
	gg <- subjectProfileSummaryPlot(
		data = summaryTable, 
		xVar = "visit",
		meanVar = "statMean1"
	)
	expect_s3_class(gg, "ggplot")
	
	# check data is correctly retained
	ggData <- lapply(ggplot_build(gg)$data, `[`, c("x", "y"))
	ggData <- unique(do.call(rbind, ggData))
	expect_identical(
		object = ggData,
		expected = setNames(summaryTable, c("x", "y"))
	)
	
})


test_that("The standard error data is correctly set", {
			
	summaryTable <- data.frame(
		visit = c(1, 2), 
		statMean = rnorm(2),
		statSE = c(1, 2)
	)
			
	gg <- subjectProfileSummaryPlot(
		data = summaryTable, 
		xVar = "visit",
		meanVar = "statMean",
		seVar = "statSE"
	)
			
	expect_s3_class(gg, "ggplot")
			
	# extract data behind the error bars
	isGeomErrorBar <- sapply(gg$layers, function(l) inherits(l$geom, "GeomErrorbar"))
	expect_length(which(isGeomErrorBar), 1)
	
	ggErrorBar <- layer_data(gg, which(isGeomErrorBar))

	expect_identical(
		ggErrorBar[, c("ymin", "ymax")],
		setNames(
			with(summaryTable, cbind.data.frame(statMean-statSE, statMean+statSE)),
			c("ymin", "ymax")
		)
	)
			
})

test_that("The data with the minimum and maximum values are correctly set", {
		
	summaryTable <- data.frame(
		visit = c(1, 2), 
		statMean = c(1, 2),
		statMin = c(0, 1),
		statMax = c(2, 3)
	)		

	gg <- subjectProfileSummaryPlot(
		data = summaryTable, 
		xVar = "visit",
		minVar = "statMin", maxVar = "statMax"
	)
	isGeomErrorBar <- sapply(gg$layers, function(l) inherits(l$geom, "GeomErrorbar"))
	expect_length(which(isGeomErrorBar), 1)
	ggErrorBar <- layer_data(gg, which(isGeomErrorBar))

	expect_identical(
		object = ggErrorBar[, c("ymin", "ymax")],
		expected = setNames(
			summaryTable[, c("statMin", "statMax")],
			c("ymin", "ymax")
		)
	)

})

test_that("A warning is generated if a variable is specified for the minimum value but not for the maximum value", {
			
	summaryTable <- data.frame(
		visit = c(1, 2), 
		statMean = c(1, 2),
		statMin = c(0, 1)
	)
			
	expect_warning(
		gg <- subjectProfileSummaryPlot(
			data = summaryTable, 
			xVar = "visit",
			minVar = "statMin"
		),
		"'minVar' is not used because 'maxVar' is not specified."
	)
	isGeomErrorBar <- sapply(gg$layers, function(l) inherits(l$geom, "GeomErrorbar"))
	expect_length(which(isGeomErrorBar), 0)
	
})

test_that("A warning is generated if a variable is specified for the maximum value but not for the minimum value", {
			
	summaryTable <- data.frame(
		visit = c(1, 2), 
		statMean = c(1, 2),
		statMax = c(2, 3)
	)
			
	expect_warning(
		gg <- subjectProfileSummaryPlot(
			data = summaryTable, 
			xVar = "visit",
			maxVar = "statMax"
		),
		"'maxVar' is not used because 'minVar' is not specified."
	)
	isGeomErrorBar <- sapply(gg$layers, function(l) inherits(l$geom, "GeomErrorbar"))
	expect_length(which(isGeomErrorBar), 0)
	
})
			
test_that("A y-axis transformation is correctly set", {
			
	summaryTable <- data.frame(
		visit = c(1, 2), 
		statMean = rlnorm(2)
	)
			
	gg <- subjectProfileSummaryPlot(
		data = summaryTable, 
		xVar = "visit",
		yTrans = "log10"
	)
	ggDataAll <- do.call(rbind.fill, ggplot_build(gg)$data)
	ggDataAll <- unique(ggDataAll[, c("x", "y")])
	
	ggDataWithInput <- merge(
		x = summaryTable, y = ggDataAll, 
		by.x = "visit",
		by.y = "x",
		all = TRUE
	)	
	with(ggDataWithInput, 
		expect_equal(
			object = y, 
			expected = log10(statMean)
		)
	)		
	
})

test_that("A warning is generated if the y-axis transformation is not set as a character", {
			
	summaryTable <- data.frame(
		visit = c(1, 2), 
		statMean = rlnorm(2)
	)

	expect_warning(
		subjectProfileSummaryPlot(
			data = summaryTable, 
			xVar = "visit",
			yTrans = log
		)
	)
	
})

test_that("A warning is generated if the y-axis transformation is not available", {
	
	summaryTable <- data.frame(
		visit = c(1, 2), 
		statMean = rlnorm(2)
	)	
	
	expect_warning(
		subjectProfileSummaryPlot(
			data = summaryTable, 
			xVar = "visit",
			yTrans = 'sqrt'
		)
	)
	
})

test_that("Negative values are correctly set to a default minimum value with a log10 transformation with a warning", {
		
	summaryTable <- data.frame(
		visit = c(1, 2), 
		statMean = c(1, 2),
		statSE = c(0.5, 3)
	)
	
	expect_warning(
		gg <- subjectProfileSummaryPlot(
			data = summaryTable, 
			xVar = "visit",
			yTrans = "log10"
		),
		"1 negative value\\(s\\) in the error bars"
	)
	
	# extract data behind the error bars
	isGeomEB <- sapply(gg$layers, function(l) inherits(l$geom, "GeomErrorbar"))
	ggDataEB <- layer_data(gg, which(isGeomEB))

	expect_equal(
		object = subset(ggDataEB, x == 2)$ymin, 
		expected = log10(0.5/10)
	)
	
})

test_that("Negative values are correctly set to a specified minimum value with a log10 transformation with a warning", {
			
	summaryTable <- data.frame(
		visit = c(1, 2), 
		statMean = c(1, 2),
		statSE = c(0.5, 3)
	)
	
	## set negative values to lower limit of y if specified
	expect_warning(
		gg <- subjectProfileSummaryPlot(
			data = summaryTable, 
			xVar = "visit",
			yTrans = "log10",
			yLim = c(0.005, 20)
		),
		"1 negative value\\(s\\) in the error bars"
	)
	
	# extract data behind the lines
	isGeomEB <- sapply(gg$layers, function(l) inherits(l$geom, "GeomErrorbar"))
	ggDataEB <- layer_data(gg, which(isGeomEB))
	
	expect_equal(subset(ggDataEB, x == 2)$ymin, log10(0.005))
			
})

test_that("The limits are correctly set for the y-axis", {
			
	summaryTable <- data.frame(
		visit = c(1, 2), 
		statMean = rnorm(2)
	)
	
	yLim <- c(-1, 1)
	gg <- subjectProfileSummaryPlot(
		data = summaryTable, 
		xVar = "visit",
		yLim = yLim
	)
	expect_equal(
		object = ggplot_build(gg)$layout$coord$limits$y, 
		expected = yLim
	)		
	
})

test_that("The y-axis is correctly expanded", {
			
	summaryTable <- data.frame(
		visit = c(1, 2), 
		statMean = rnorm(2)
	)
	
	yAxisExpand <- expansion(mult = 0, add = 2)
	gg <- subjectProfileSummaryPlot(
		data = summaryTable, 
		xVar = "visit",
		yAxisExpand = yAxisExpand
	)
			
	# extract labels from the ggplot object
	ggScales <- gg$scales$scales
	isScaleY <- sapply(ggScales, function(x) 
		"y" %in% x[["aesthetics"]]
	)
	expect_equal(
		object = ggScales[[which(isScaleY)]]$expand, 
		expected = yAxisExpand
	)
			
})

test_that("A warning is generated if the old specification to expand the y-axis is used", {
			
	summaryTable <- data.frame(
		visit = c(1, 2), 
		statMean = rnorm(2)
	)
	yAxisExpand <- expansion(mult = 0, add = 2)
	expect_warning(
		subjectProfileSummaryPlot(
			data = summaryTable, 
			xVar = "visit",
			yLimExpand = yAxisExpand
			),
		"'yLimExpand' is deprecated."
	)
	
})

#' Format summary statistics table for export
#' @param colHeaderTotalInclude Logical, if TRUE include the total of number of patients
#' (\code{'statN'}) in the column header.
#' @param emptyValue String with placeholder used to fill the table for missing values, '-' by default.
#' This value is typically used e.g. if not all statistics are computed for all specified
#' row/col/var variables.
#' @inheritParams subjectProfileSummaryPlot
#' @inheritParams inTextSummaryTable-common-args
#' @return summaryTable reformatted to wide format
#' @author Laure Cougnaud
#' @importFrom clinUtils getLabelVar
#' @importFrom reshape2 melt dcast
#' @importFrom plyr colwise
#' @importFrom stats as.formula
#' @importFrom utils capture.output
formatSummaryStatisticsTable <- function(
	summaryTable,
	# row
	rowVar = getAttribute(summaryTable, "rowVar"),
	# column
	colVar = getAttribute(summaryTable, "colVar"),
	colTotalLab = getAttribute(summaryTable, "colTotalLab", default = "Total"),
	colHeaderTotalInclude = TRUE,
	# stats
	statsVar = getAttribute(summaryTable, "statsVar"),
	statsLabInclude = NULL,
	statsLayout = "row",
	statsValueLab = "StatisticValue",
	emptyValue = "-"){
		
	if(!is.data.frame(summaryTable)){
		
		inputParams <- as.list(environment())
		res <- sapply(summaryTable, function(summaryTableI){
			inputParamsBy <- inputParams
			inputParamsBy$summaryTable <- summaryTableI
			do.call(formatSummaryStatisticsTable, inputParamsBy)		
		}, simplify = FALSE)	
		return(res)
		
	}
		
	statsLayout <- match.arg(statsLayout, choices = c("row", "col", "rowInSepCol"))
		
	## format table
	if("isTotal" %in% colnames(summaryTable)){
		
		if(!is.null(colVar) & colHeaderTotalInclude){
			
			# add total in column header
			colVarWithCount <- colVar[length(colVar)] 
			
			# fix in case 'colVar' contains space(s)
			colVarQuote <- if(!is.null(colVar))	paste0("`", colVar, "`")
			
			dataWithTotal <- ddply(summaryTable, colVarQuote, function(x){
				idxTotal <- which(x$isTotal)
				if(length(idxTotal) == 1){
					# for the total column, include the N in all columns (to be merged afterwards)
					colToModif <- if(all(!is.na(x[, colVar])) && all(x[, colVar] == colTotalLab))	colVar	else	colVarWithCount
					for(col in colToModif){
						x[, col] <- factor(paste0(x[, col], "\n(N=",  x[idxTotal , "statN"], ")"))
					}
					x[-idxTotal, ]
				}else x
			})
		
			# ensure that order of columns with Total is as specified in levels of the factor originally
			for(col in colVar){
				colVarWithCountEl <- levels(dataWithTotal[, col])
				colVarInit <-  summaryTable[, col]
				colVarEl <- if(is.factor(colVarInit))	levels(colVarInit)	else	unique(colVarInit)	
				colVarWithCountElOrdered <- colVarWithCountEl[
					order(match(sub("(.+)\n\\(N=.+\\)", "\\1", colVarWithCountEl), colVarEl))
				]
				dataWithTotal[, col] <- factor(dataWithTotal[, col], levels = colVarWithCountElOrdered)
			}
	
		}else{
			idxTotal <- which(summaryTable$isTotal)
			nTotal <- summaryTable[idxTotal, "statN"]
			if(colHeaderTotalInclude && length(nTotal) > 1)
				stop("Multiple values for the header total but",
					" no column variables ('colVar') are specified.")
			if(length(idxTotal) > 0){
				dataWithTotal <- summaryTable[-idxTotal, ]
			}else{
				dataWithTotal <- summaryTable
			}
		}
		
	}else{
		dataWithTotal <- summaryTable
		nTotal <- NA
	}
	
	# Note: best way would be to make this function works for
	# empty data.frame but dcast returns: 
	# 'Error in dim(ordered) <- ns : dims [product 1] do not match the length of object [0]'
	if(nrow(dataWithTotal) == 0){
		warning("No data remain after filtering of total rows.")
		return(invisible())
	}

	# convert from wide to long format
	if(is.null(statsVar)){
		statsVar <- setdiff(colnames(dataWithTotal),  
			c(rowVar, colVar, "variable", "variableGroup", "isTotal")
		)
	}

	if(statsValueLab == "Statistic")
		stop("'statsValueLab' should be different than 'Statistic'.")
	
	# in case only one statistic is computed and 'stats' was named
	# retain the original name, otherwise use statsValueLab
	statsValueNewName <- if(length(statsVar) == 1 && statsVar != "Statistic"){
		statsVar
	}else	statsValueLab
	dataLong <- melt(
		data = dataWithTotal, 
		id.vars = c(rowVar, colVar),
		measure.vars = statsVar,
		value.name = statsValueNewName,
		variable.name = "Statistic"
	)
	
	# Is the label for the statistic required?
	isStatsLabRequired <- !(length(statsVar) == 1 && length(unique(dataLong$Statistic)) == 1)
	if(is.null(statsLabInclude)){
		statsLabInclude <- isStatsLabRequired
	}else	if(!statsLabInclude & isStatsLabRequired){
		warning(paste("Statistic label is included ('statsLabInclude' set to TRUE)",
			"because more than one statistic variable is available in the table."))
		statsLabInclude <- TRUE
	}
	
	if(!statsLabInclude)	dataLong$Statistic <- NULL
	
	emptyStats <- which(is.na(dataLong[, statsValueNewName]))
	if(length(emptyStats) > 0)
		dataLong <- dataLong[-emptyStats, ]
	
	# if only one 'stats' and no named, set 'Statistic' to NA 
	# e.g. in DM table: count per sub-group for categorical variable
	if(!is.null(rowVar) & "Statistic" %in% colnames(dataLong)){
		idxUniqueStatNotNamed <- which(!duplicated(dataLong[, c(rowVar, colVar)]) & dataLong$Statistic == "Statistic")
		if(length(idxUniqueStatNotNamed) > 0)
			dataLong[idxUniqueStatNotNamed, "Statistic"] <- NA
	}
	
	# format statistic value
	if(is.numeric(dataLong[, statsValueNewName]))
		dataLong[, statsValueNewName] <- formatC(dataLong[, statsValueNewName])
	
	# put elements in 'colVar' in different columns (long -> wide format)
	if(!is.null(colVar) | (statsLayout == "col" & length(statsVar) > 1)){
		rowVarFm <- c(
			if(!is.null(rowVar)) paste(paste0("`", rowVar, "`"), collapse = " + "), 
			if(statsLabInclude & statsLayout != "col")	"Statistic"
		)
		if(is.null(rowVarFm))	rowVarFm <- "."
		colVarFm <- c(colVar, if(statsLabInclude & statsLayout == "col")	"Statistic")
		colVarFm <- if(!is.null(colVarFm))	paste0("`", colVarFm, "`")
		formulaWithin <- as.formula(paste(
			paste(rowVarFm, collapse = "+"),
			"~", 
			paste(colVarFm, collapse = " + ")
		))
		varsFm <- all.vars(formulaWithin)
		varsFm <- setdiff(varsFm, ".")
		isDupl <- duplicated(dataLong[, varsFm])
		if(any(isDupl)){
			dataDupl <- merge(dataLong, dataLong[isDupl, varsFm, drop = FALSE])
			stop("Table formatting to multiple columns failed because of ",
				"duplicated records for each row/col:\n",
				paste(capture.output(print(dataDupl)), collapse = "\n")
			)
		}
		dataLong <- dcast(dataLong, formula = formulaWithin, 
			value.var = statsValueNewName, fill = emptyValue
		)
		if(all(rowVarFm == "."))	dataLong["."] <- NULL
	}else{
		if(colHeaderTotalInclude & length(nTotal) != 0)
			colnames(dataLong)[match(statsValueNewName, colnames(dataLong))] <- 
				paste0(statsValueNewName, "\n(N=",  nTotal, ")")
	}
	
	attributes(dataLong)$summaryTable <- attributes(summaryTable)$summaryTable
	attributes(dataLong)$summaryTable$statsLayout <- statsLayout
	attributes(dataLong)$summaryTable$statsLabInclude <- statsLabInclude
	
	return(dataLong)
	
}

#' Get default set of statistics for variables of interest and specific dataset.
#' 
#' This set of statistics can be passed directly to the \code{stats} parameter
#' of the package functions.\cr
#' By default, statistics are extracted based on the variable(s) type
#' and formatted with the default rules implemented in the package.
#' @param type Character vector with type of statistics to extract, among:
#' \itemize{
#' \item{'default': default sets of statistics, 
#' see types: 'summary-default' and 'count-default' in \code{\link{getStats}}}
#' \item{'all': all computed statistics, see types: 'summary' and 'count' in 
#' \code{\link{getStats}}}
#' \item{any formatted statistics as implemented in \code{\link{getStats}}, 
#' see section 'Formatted statistics' in
#' \code{\link[=inTextSummaryTable-stats]{in-text table statistics}}.
#' }
#' }
#' To specify statistics for a continuous (numeric) or categorical
#' variable separately, this vector can be named with: 'cont' or
#' 'cat' respectively (elements not named are used for both continuous 
#' and categorical variables).
#' @param var (optional, recommended for continuous variable)
#' Character vector with variable(s) of data, to
#' compute statistics on.\cr
#' If NULL (by default), counts of the entire dataset are computed.\cr
#' It is passed to the \code{x} parameter of \code{\link{getStats}}.
#' @param extra List with extra statistics to include, or function to apply on each
#' \code{var} (e.g. depending on the class of \code{var}) to get such statistic.
#' @param args (optional) Named list with extra arguments for 
#' \code{\link{getStats}} for continuous (name: 'cont') or 
#' categorical variable (name: 'cat') specifically. 
#' @param ... Extra parameters passed to the \code{\link{getStats}} function
#' (independent of the variable type).
#' @inheritParams inTextSummaryTable-common-args
#' @return List with statistics to compute, named by \code{var}
#' @seealso \link{getStats}
#' @examples 
#' # default set of statistics (depending if the variable is continuous or categorical)
#' exampleData <- data.frame(
#'   USUBJID = 1 : 4, 
#'   WEIGHT = c(67, 78, 83, 61), 
#'   SEX = c("F", "M", "M", "F"), 
#'   stringsAsFactors = FALSE
#' )
#' getStatsData(data = exampleData, var = c("WEIGHT", "SEX"))
#' # all set of statistics (depending if the variable is continuous or categorical)
#' getStatsData(data = exampleData, var = c("WEIGHT", "SEX"), type = "all")
#' # custom set of statistics for all variables
#' getStatsData(data = exampleData, var = c("WEIGHT", "SEX"), type = c("n", "%"))
#' # custom set of statistics, depending on the type of the variable
#' getStatsData(data = exampleData, var = c("WEIGHT", "SEX"), 
#'   type = c(cont = "median (range)", cont = "mean (se)", cat = "n (%)"), 
#'   args = list(cat = list(includeName = FALSE))
#' )
#' @author Laure Cougnaud
#' @export
getStatsData <- function(
	data, 
	var = NULL, 
	type = "default", 
	extra = NULL, 
	args = NULL,
	...){

	if(!missing(data) && class(data)[1] != "data.frame")
		data <- as.data.frame(data)
	
	argsGeneral <- list(...)
	getArgsGetStats <- function(var = NULL){
		
		isNumVar <- !is.null(var) && var != "all" && is.numeric(data[, var])
		
		# get type of the variable
		if(!is.null(names(type))){
			if(any(!names(type) %in% c("", "cont", "cat")))
				stop("Only: 'cont', 'cat' and '' (empty) are allowed for the names of the 'type' parameter.")
			if(isNumVar){
				type <- type[names(type) %in% c("", "cont")]
			}else	type <- type[names(type) %in% c("", "cat")]
		}
		if("all" %in% type){
			type[match("all", type)] <- ifelse(isNumVar, "summary", "count")
		}else	if("default" %in% type){
			type[match("default", type)] <- ifelse(isNumVar, "summary-default", "count-default")
		}
		type <- unname(type)
		
		# get extra variable
		argsVar <- if(isNumVar)	args[["cont"]]	else	args[["cat"]]
		argsGetStats <- c(
			list(type = type),
			argsVar,
			if(!is.null(var) &&	var != "all")	list(x = data[, var]),
			list(...)
		)
		stats <- do.call(getStats, argsGetStats)
		return(stats)
		
	}
	
	getExtra <- function(var = NULL){
		if(!is.null(var) && var == "all")	var <- NULL
		if(!is.null(extra)){
			listExtra <- lapply(extra, function(extraEl){
				if(is.function(extraEl))	extraEl(data[, var])	else	extraEl
			})
			listExtra <- listExtra[!sapply(listExtra, is.null)]
		}
	}
		
	if(is.null(var)){
		stats <- getArgsGetStats()
		stats <- c(stats, getExtra())
	}else{
		stats <- sapply(var, function(varI){
			statsVar <- getArgsGetStats(var = varI)
			c(statsVar, getExtra(var = varI))
		}, simplify = FALSE)
		
	}
	
	return(stats)

}

#'  Get default set of statistics for one particular variable.
#' 
#' This set of statistics can be passed directly to the \code{stats} parameter
#' of the package functions.
#' @param type Character vector with type of statistics (multiple are possible).
#' Available statistics are specified in the section 'Formatted statistics' and
#' formatting in 'Statistics formatting' in
#' \code{\link[=inTextSummaryTable-stats]{in-text table statistics}}.
#' @param includeName Logical, should the statistics name be included (TRUE by default)?
#' This is applied for the statistic names used in each for the set defined in \code{type};
#' and for the label of the list if \code{type} is of length 2.
#' If there are multiple \code{type} or statistics within a set, the names are retained (to avoid confusion).
#' @param x (optional, recommended for continuous variable) Numeric vector 
#' for which the statistics should be computed on.\cr
#' This is used to derive the number of decimals to include
#' for a continuous variable.\cr
#' If not specified, the values are rounded with \code{\link{formatC}}.
#' @param nDecCont Integer with base number of decimals 
#' for continuous variable, or function returning this number based on \code{x} 
#' (\code{\link{getNDecimals}} by default).
#' @param nDecN,nDecm Integer with number of decimals 
#' for number of subjects/records (0 by default).
#' @param formatPercentage Function used to format the percentages
#' (see \code{\link{formatPercentage}} for default behaviour).
#' @return Expression (or call object) containing
#' function to extract requested summary statistics.
#' If multiple \code{type} are specified, they are combined to a list.
#' Names of the list will be typically used to name the statistic
#' in the summary table.
#' @examples
#' ## default set of statistics are available for:
#' 
#' # for count table:
#' getStats("count")
#' getStats("n (%)")
#' getStats("n")
#' getStats("%")
#' getStats("m")
#' getStats("%m")
#' getStats("m (%)")
#' # for continuous variable:
#' getStats("summary")
#' getStats("mean (se)")
#' getStats("mean (sd)")
#' getStats("median (range)")
#' getStats("median\n(range)")
#' getStats(c("Mean", "SE"))
#' # for continuous variable with log-normal distribution:
#' getStats(c("Geometric Mean", "Geometric SD"))
#' getStats("geometric mean (geometric sd)")
#' 
#' ## to not include statistic name in the table
#' getStats("median\n(range)", includeName = FALSE)
#' getStats(c("summary", "median\n(range)"), includeName = FALSE)
#' 
#' ## to extract the number of decimals based on a continuous variable (see ?getMaxNDecimals) 
#' exampleData <- data.frame(
#'   USUBJID = 1 : 4, 
#'   WEIGHT = c(67, 78, 83, 61), 
#'   SEX = c("F", "M", "M", "F"), 
#'   stringsAsFactors = FALSE
#' )
#' getStats(type = c('median (range)', 'mean (se)'), x = exampleData$WEIGHT)
#' # compare with when 'x' is not specified:
#' getStats(type = c('median (range)', 'mean (se)'))
#' 
#' ## custom function to format the percentages:
#' getStats(type = "count", formatPercentage = function(x) round(x, 2))
#' @seealso \link{getStatsData}
#' @author Laure Cougnaud
#' @importFrom clinUtils roundHalfUpTextFormat
#' @export
getStats <- function(
	type = "summary",
	includeName = TRUE,
	x = NULL, 
	nDecCont = getMaxNDecimals,
	nDecN = 0, nDecm = nDecN,
	formatPercentage = inTextSummaryTable::formatPercentage
){
	
	# number of decimals for continuous variable
	nDecContBase <- if(is.function(nDecCont) & !is.null(x) & is.numeric(x)){
		nDecCont(x)
	}else	if(is.numeric(nDecCont))	nDecCont	
	
	statsBase <- c(
		# statistics for categorical variable
		list(
			# counts
			n = bquote(roundHalfUpTextFormat(statN, .(nDecN))),
			m = bquote(roundHalfUpTextFormat(statm, .(nDecm))),
			# percentage
			`%` = substitute(
				formatPercentage(statPercN), 
				list(formatPercentage = formatPercentage)
			),
			`%m` = substitute(
				formatPercentage(statPercm), 
				list(formatPercentage = formatPercentage)
			)
		),
		# statistics for continuous variable
		if(!is.null(nDecContBase)){
			statsBase <- c(
			  list(
  				Mean = bquote(roundHalfUpTextFormat(statMean, .(nDecContBase + 1))),
  				Median = bquote(roundHalfUpTextFormat(statMedian, .(nDecContBase + 1))),
  				SD = bquote(roundHalfUpTextFormat(statSD, .(nDecContBase + 1))),
  				SE = bquote(roundHalfUpTextFormat(statSE, .(nDecContBase + 2))),
  				Min = bquote(roundHalfUpTextFormat(statMin, .(nDecContBase))),
  				Max = bquote(roundHalfUpTextFormat(statMax, .(nDecContBase)))
  			),
  			if(any(grepl("Geometric Mean", type, ignore.case = TRUE)))
  				list(`Geometric Mean` = bquote(roundHalfUpTextFormat(statGeomMean, .(nDecContBase + 1)))),
			  if(any(grepl("Geometric SD", type, ignore.case = TRUE)))
			    list(`Geometric SD` = bquote(roundHalfUpTextFormat(statGeomSD, .(nDecContBase + 1))))
			)
		}else{
			c(
			  list(
  				Mean = quote(formatC(statMean)),
  				Median = quote(formatC(statMedian)),
  				SD = quote(formatC(statSD)),
  				SE = quote(formatC(statSE)),
  				Min = quote(formatC(statMin)),
  				Max = quote(formatC(statMax))
  			),
			  if(any(grepl("Geometric Mean", type, ignore.case = TRUE)))
				  list(`Geometric Mean` = quote(formatC(statGeomMean))),
			  if(any(grepl("Geometric SD", type, ignore.case = TRUE)))
				  list(`Geometric SD` = quote(formatC(statGeomSD)))
			)
		}
	)
	
	type <- match.arg(
		arg = type,
		choices = c(
			"summary-default", "count-default", 
			"summary", "count", 
			"n (%)", "n/N (%)",
			"m (%)",
			"median (range)", "median\n(range)",
			"mean (se)", "mean (sd)", "mean (range)",
			"geometric mean (geometric sd)",
			"(min, max)",
			names(statsBase)
		),
		several.ok = TRUE
	)
	
	statsList <- sapply(type, function(typeI){
		switch(typeI,
			`summary-default` = statsBase[c("n", "Mean", "SD", "SE", "Median", "Min", "Max")],
			`count-default` = statsBase[c("n", "%")],
			`summary` = statsBase[c("n", "Mean", "SD", "SE", "Median", "Min", "Max", "%", "m")],
			`count` = statsBase[c("n", "%", "m")],
			`n (%)` = list('n (%)' = 
				bquote(
					ifelse(
						is.na(statPercN), "-",
						ifelse(statN == 0, "0",
							paste0(.(statsBase$n), " (", .(statsBase$`%`), ")")
						)
					)
				)
			),
			`m (%)` = list('m (%)' = 
				bquote(
					ifelse(
						is.na(statPercm), "-",
						ifelse(statm == 0, "0",
							paste0(.(statsBase$m), " (", .(statsBase$`%m`), ")")
						)
					)
				)
			),
			`n/N (%)` = list('n/N (%)' = bquote(
					ifelse(
						is.na(statPercN), "-",
							ifelse(statN == 0, "0",
								paste0(.(statsBase$n), "/",  statPercTotalN, " (", .(statsBase$`%`), ")")
							)
					)
				)
			),
			`(min, max)` = list('(Min, Max)' =
				bquote(paste0(
					"(", .(statsBase$Min), ", ", .(statsBase$Max), ")"
				))
			),
			`median (range)` = list('Median (range)' = 
				bquote(paste0(
					.(statsBase$Median), 
					" (", .(statsBase$Min), ",", .(statsBase$Max), ")"
				))
			),
			`median\n(range)` = list('Median\n(range)' = 
				bquote(paste0(
					.(statsBase$Median), 
					"\n(", .(statsBase$Min), ",", .(statsBase$Max), ")"
				))
			),
			`mean (se)` = list('Mean (SE)' = 
				bquote(paste0(.(statsBase$Mean), " (", .(statsBase$SE),  ")"))
			),
			`mean (sd)` = list('Mean (SD)' = 
				bquote(paste0(.(statsBase$Mean), " (", .(statsBase$SD),  ")"))
			),
			`geometric mean (geometric sd)` = list('Geometric Mean (Geometric SD)' = 
        bquote(paste0(.(statsBase$`Geometric Mean`), " (", .(statsBase$`Geometric SD`),  ")"))
			),
			`mean (range)` = list('Mean (range)' = 
				bquote(paste0(
					.(statsBase$Mean), 
					" (", .(statsBase$Min), ",", .(statsBase$Max), ")"
				))
			),
			if(typeI %in% names(statsBase))	statsBase[typeI]
		)
	}, simplify = FALSE)
	stats <- unlist(unname(statsList), recursive = FALSE)
	
	if(any(duplicated(names(stats))))
		warning("Returned statistics have duplicated names (you probably included multiple 'type' statistics). ",
			"Make sure to provide only one statistic with the same name to the in-text table functions.")
	
	if(!includeName){
		if(length(stats) == 1){
			stats <- unname(stats)
		}else	warning(paste("The labels for the different types:", 
			toString(sQuote(type)), "are retained, to avoid confusion."))
	}
	
	return(stats)
	
}

context("Compute summary statistics table with column variable")

library(plyr)

test_that("A summary table is correctly computed for a continuous variable by a column variable", {
			
	data <- data.frame(
		USUBJID = seq.int(7),
		AGE = seq(20, 62, length.out = 7),
		TRT = rep(c("A", "B"), times = c(3, 4)),
		stringsAsFactors = FALSE
	)
	expect_silent(
		sumTable <- computeSummaryStatisticsTable(
			data,
			var = "AGE",
			colVar = "TRT"
		)
	)
	expect_s3_class(sumTable, "summaryTable")
	expect_true("TRT" %in% colnames(sumTable))
	expect_identical(levels(sumTable$TRT), unique(data$TRT))
	
	# table with colVar = cbind of table created for each element of colVar
	statsContName <- c("statMean", "statSD", "statSE", "statMedian", "statMin", "statMax")
	for(trt in unique(data$TRT)){
		expect_equal(
			object = subset(sumTable, TRT == !!trt, select = -TRT),
			expected = computeSummaryStatisticsTable(data = subset(data, TRT == !!trt), var = "AGE"),
			check.attributes = FALSE
		)
	}
	
	# counts:
	expect_true(all(sumTable$statPercN %in% 100))
	expect_equal(subset(sumTable, TRT == "A" & isTotal)$statN, 3)
	expect_equal(subset(sumTable, TRT == "A")$statPercTotalN, c(3, 3))
	expect_equal(subset(sumTable, TRT == "B" & isTotal)$statN, 4)
	expect_equal(subset(sumTable, TRT == "B")$statPercTotalN, c(4, 4))	

})

test_that("Column elements are ordered alphabetically if the column variable is a character", {
			
	data <- data.frame(
		USUBJID = seq.int(6),
		AGE = seq(20, 62, length.out = 6),
		TRT = rep(c("A", "B"), each = 3),
		stringsAsFactors = FALSE
	)
	expect_equal(
		levels(computeSummaryStatisticsTable(data, var = "AGE", colVar = "TRT")$TRT),
		c("A", "B")
	)
	
})

test_that("Column elements are ordered as specified if the column variable is a factor", {
	
	data <- data.frame(
		USUBJID = seq.int(6),
		AGE = seq(20, 62, length.out = 6),
		TRT = rep(c("A", "B"), each = 3),
		stringsAsFactors = FALSE
	)
	data$TRT <- factor(data, levels = c("B", "A"))
	summaryTable <- computeSummaryStatisticsTable(data, var = "AGE", colVar = "TRT")
	expect_equal(
		levels(summaryTable$TRT),
		c("B", "A")
	)
			
})

test_that("The column totals are correctly computed", {
			
	data <- data.frame(
		USUBJID = seq.int(6),
		AGE = seq(20, 62, length.out = 6),
		TRT = rep(c("A", "B"), each = 3),
		stringsAsFactors = FALSE
	)
	
	expect_silent(
		sumTableColTotal <- computeSummaryStatisticsTable(
			data,
			var = "AGE",
			colVar = "TRT",
			colTotalInclude = TRUE
		)
	)
	expect_identical(
		levels(sumTableColTotal$TRT),
		c(unique(data$TRT), "Total")
	)
	
	# total should be the same as stats computed on entire dataset:
	expect_equal(
		subset(sumTableColTotal, TRT == "Total", select = -TRT),	
		computeSummaryStatisticsTable(data, var = "AGE"),
		check.attributes = FALSE # row.names diff
	)
	
})
	
test_that("A label is correctly set to the column totals", {
				
	data <- data.frame(
		USUBJID = seq.int(6),
		AGE = seq(20, 62, length.out = 6),
		TRT = rep(c("A", "B"), each = 3),
		stringsAsFactors = FALSE
	)
	
	sumTableColTotal <- computeSummaryStatisticsTable(
		data,
		var = "AGE",
		colVar = "TRT",
		colTotalInclude = TRUE
	)
	expect_equal(
		object = {
			sumTableColTotalLab <- computeSummaryStatisticsTable(
				data,
				var = "AGE",
				colVar = "TRT",
				colTotalInclude = TRUE, colTotalLab = "All treatments"
			)	
			subset(sumTableColTotalLab, TRT == "All treatments", select = -TRT)
		},
		expected = subset(sumTableColTotal, TRT == "Total", select = -TRT)
	)
	
})

test_that("The column totals are correctly extracted from a different dataset", {
			
	data <- data.frame(
		USUBJID = c("1", "1", "2", "3", "2", "3", "3"),
		AEBODSYS = c("A", "A", "A", "A", "B", "B", "B"),
		AEDECOD = c("a1", "a2", "a1", "a1", "b1", "b1", "b2"),
		TRT = c("X1", "X1", "X1", "X2", "X1", "X2", "X2"),
		stringsAsFactors = FALSE
	)
	dataTotalCol <- do.call(
		rbind, 
		replicate(2, data, simplify = FALSE)
	)
	dataTotalCol$USUBJID <- as.character(sample.int(nrow(dataTotalCol)))
		
	# full summary table
	summaryTable <- computeSummaryStatisticsTable(
		data,
		rowVar = c("AEBODSYS", "AEDECOD"),
		colVar = "TRT",
		colTotalInclude = TRUE,
		dataTotalCol = dataTotalCol
	)
	
	# counts in total column should be the same
	# as computed for the full data specified in 'dataTotalCol'
	expect_equal(
		object = subset(summaryTable, TRT == "Total", select = -TRT),
		expected = computeSummaryStatisticsTable(
			data = dataTotalCol,
			rowVar = c("AEBODSYS", "AEDECOD")
		),
		check.attributes = FALSE
	)
	
})

test_that("The column totals for row totals are correctly computed", {
			
	data <- data.frame(
		USUBJID = c("1", "1", "2", "3", "2", "3", "3"),
		AEBODSYS = c("A", "A", "A", "A", "B", "B", "B"),
		AEDECOD = c("a1", "a2", "a1", "a1", "b1", "b1", "b2"),
		TRT = c("X1", "X1", "X1", "X2", "X1", "X2", "X2"),
		stringsAsFactors = FALSE
	)
				
	# full summary table
	rowVar <- c("AEBODSYS", "AEDECOD")
	summaryTable <- computeSummaryStatisticsTable(
		data,
		rowVar = rowVar,
		rowVarTotalInclude = rowVar,
		colVar = "TRT",
		colTotalInclude = TRUE
	)
	
	# row total across AEDECOD
	expect_equal(
		object = subset(summaryTable, 
			subset = TRT == "Total" & AEDECOD == "Total" & AEBODSYS != "Total", 
			select = -c(TRT, AEDECOD),
		),
		expected = subset(
			computeSummaryStatisticsTable(data = data, rowVar = c("AEBODSYS")),
			subset = !isTotal
		),
		check.attributes = FALSE
	)
	
	# row total across AEBODYS		
	expect_equal(
		object = subset(summaryTable, 
			subset = TRT == "Total" & AEDECOD == "Total" & AEBODSYS == "Total", 
			select = -c(TRT, AEDECOD, AEBODSYS),
		),
		expected = subset(
			computeSummaryStatisticsTable(data = data),
			subset = !isTotal
		),
		check.attributes = FALSE
	)
	
})

test_that("The column totals for row totals are correctly computed from a different dataset", {
			
	data <- data.frame(
		USUBJID = c("1", "1", "2", "3", "2", "3", "3"),
		AEBODSYS = c("A", "A", "A", "A", "B", "B", "B"),
		AEDECOD = c("a1", "a2", "a1", "a1", "b1", "b1", "b2"),
		TRT = c("X1", "X1", "X1", "X2", "X1", "X2", "X2"),
		stringsAsFactors = FALSE
	)
	dataTotalColDummy <- do.call(
		rbind, 
		replicate(2, data, simplify = FALSE)
	)
	dataTotalColDummy$USUBJID <- as.character(sample.int(nrow(dataTotalColDummy)))
	dataTotalCol <- list(
		total = dataTotalColDummy,
		AEBODSYS = dataTotalColDummy[sample(nrow(dataTotalColDummy), 9), ],
		AEDECOD = dataTotalColDummy[sample(nrow(dataTotalColDummy), 5), ]
	)
			
	# full summary table
	rowVar <- c("AEBODSYS", "AEDECOD")
	summaryTable <- computeSummaryStatisticsTable(
		data,
		rowVar = rowVar,
		rowVarTotalInclude = rowVar,
		colVar = "TRT",
		colTotalInclude = TRUE,
		dataTotalCol = dataTotalCol
	)
	
	# total used for the percentages are based on 'dataTotalCol' with 'total' label
	summaryTableColTotal <- subset(summaryTable, TRT == "Total")
	nSubjTotalCol <- length(unique(dataTotalCol[["total"]]$USUBJID))
	expect_setequal(
		summaryTableColTotal$statPercTotalN,
		nSubjTotalCol
	)
	# total used for the column header is based on 'dataTotalCol' with 'total' label
	expect_equal(
		subset(summaryTableColTotal, isTotal)$statN,
		nSubjTotalCol
	)
			
	# counts in total column should be the same
	# as computed for the full data specified in 'dataTotalCol'
			
	# col total for the general row total
	expect_equal(
		object = subset(summaryTable, 
			TRT == "Total" & AEDECOD == "Total" & AEBODSYS == "Total", 
			select = c(-AEBODSYS, -AEDECOD, -TRT)
		),
		expected = subset(
			computeSummaryStatisticsTable(data = dataTotalCol[["total"]]),
			subset = !isTotal
		),
		check.attributes = FALSE
	)
	
	# col total across AEDECOD
	expect_equal(
		object = subset(summaryTable, 
			subset = (TRT == "Total" & AEDECOD == "Total" & AEBODSYS != "Total"), 
			select = c("statN", "statm")
		),
		expected = subset(
			computeSummaryStatisticsTable(
				data = dataTotalCol[["AEBODSYS"]], 
				rowVar = "AEBODSYS"
			),
			subset = !isTotal,
			select = c("statN", "statm")
		),
		check.attributes = FALSE
	)
	
	# col total for most nested row
	expect_equal(
		object = subset(summaryTable, 
			subset = (TRT == "Total" & AEDECOD != "Total" & AEBODSYS != "Total"), 
			select = c("statN", "statm")
		),
		expected = subset(
			computeSummaryStatisticsTable(
				data = dataTotalCol[["AEDECOD"]], 
				rowVar = "AEDECOD"
			),
			subset = !isTotal,
			select = c("statN", "statm")
		),
		check.attributes = FALSE
	)
			
})

test_that("Only column elements of the data are included by default", {
			
	data <- data.frame(
		USUBJID = seq.int(6),
		AGE = seq(20, 62, length.out = 6),
		TRT = factor(rep(c("A", "B"), each = 3)),
		DOSE = factor(rep(c("100", "200"), each = 3)),
		stringsAsFactors = FALSE
	)
	
	# by default, only the columns appearing in the data are included
	expect_silent(
		sumTableBase <- computeSummaryStatisticsTable(
			data,
			var = "AGE",
			colVar = c("TRT", "DOSE")
		)
	)
	expect_equal(
		object = unique(sumTableBase[, c("TRT", "DOSE")]),
		expected = data.frame(
			TRT = factor(c("A", "B")), 
			DOSE = factor(c("100", "200"))
		),
		check.attributes = FALSE
	)
	
})

test_that("All combinations of column elements are correctly included when requested", {
			
	data <- data.frame(
		USUBJID = seq.int(6),
		AGE = seq(20, 62, length.out = 6),
		TRT = factor(rep(c("A", "B"), each = 3)),
		DOSE = factor(rep(c("100", "200"), each = 3)),
		stringsAsFactors = FALSE
	)
	
	# if colInclude0 is specified, all combinations of columns are included based on levels
	expect_silent(
		sumTableInclude0 <- computeSummaryStatisticsTable(
			data,
			var = "AGE",
			colVar = c("TRT", "DOSE"),
			colInclude0 = TRUE
		)
	)
	
	# all combinations are used for the 'total', in correct order:
	colAllComb <- expand.grid(unique(data[, c("TRT", "DOSE")]))
	colAllComb <- colAllComb[with(colAllComb, order(TRT, DOSE)), ]
	sumTableInclude0Total <- subset(sumTableInclude0, isTotal)
	expect_equal(
		sumTableInclude0Total[, c("TRT", "DOSE")],
		colAllComb,
		check.attributes = FALSE
	)
	
	# statistics for combinations appearing in the data are identical
	# than when 'include0' is set to FALSE:
	colVarInData <- unique(data[, c("DOSE", "TRT")])
	sumTableBase <- computeSummaryStatisticsTable(
		data,
		var = "AGE",
		colVar = c("TRT", "DOSE")
	)
	expect_equal(
		merge(colVarInData, sumTableInclude0)[, colnames(sumTableBase)],
		sumTableBase,
		check.attributes = FALSE
	)
	
	# statistics for combinations NOT appearing in the data are empty
  idxNotInData <- which(
    !do.call(interaction, sumTableInclude0[, colnames(colVarInData)]) %in% 
    do.call(interaction, colVarInData)
  ) # ~ anti-join
	sumTableNotInData <- sumTableInclude0[idxNotInData, ]
	statsCont <- c("statMean", "statSD", "statSE", "statMedian", "statMin", "statMax")
	expect_true(all(is.na(sumTableNotInData[, statsCont])))
	expect_true(all(sumTableNotInData[, c("statN", "statm", "statPercTotalN")] == 0))
	expect_true(all(is.nan(sumTableNotInData[, c("statPercN")])))
			
})

test_that("Specific combinations of column elements are included when requested", {
			
	data <- data.frame(
		USUBJID = seq.int(6),
		AGE = seq(20, 62, length.out = 6),
		TRT = factor(rep(c("A", "B"), each = 3)),
		DOSE = factor(rep(c("100", "200"), each = 3)),
		stringsAsFactors = FALSE
	)
	
	# sum table with combinations appearing in data
	sumTableBase <- computeSummaryStatisticsTable(
		data,
		var = "AGE",
		colVar = c("TRT", "DOSE")
	)
	
	## if other groups should be included, 'colVarDataLevels' can be used
			
	# different doses for all treatment and non alphabetical order:
	colVarDataLevels <- data.frame(
		TRT = factor(
			rep(c("A", "B", "C"), length.out = 5), 
			levels = c("C", "A", "B")
		), 
		DOSE = factor(
			c("100", "200", "600", "400", "100")
		)
	)
	
	expect_silent(
		sumTable <- computeSummaryStatisticsTable(
			data,
			var = "AGE",
			colVar = c("TRT", "DOSE"),
			colVarDataLevels = colVarDataLevels
		)
	)
	# all combinations are included for the 'total', in correct order:
	expect_equal(
		subset(sumTable, isTotal)[, c("TRT", "DOSE")],
		colVarDataLevels[with(colVarDataLevels, order(TRT, DOSE)), ],
		check.attributes = FALSE
	)
	
	# statistics for combinations appearing in the data are identical
	# than when 'colVarDataLevels' is not specified:
	colVarInData <- unique(data[, c("DOSE", "TRT")])
	expect_equal(
		merge(colVarInData, sumTable)[, colnames(sumTableBase)],
		sumTableBase,
		check.attributes = FALSE
	)
	
	# statistics for combinations NOT appearing in the data are empty
 idxNotInData <- which(
  !do.call(interaction, sumTable[, colnames(colVarInData)]) %in% 
  do.call(interaction, colVarInData)
 ) # ~ anti-join
 sumTableNotInData <- sumTable[idxNotInData, ]
	statsCont <- c("statMean", "statSD", "statSE", "statMedian", "statMin", "statMax")
	expect_true(all(is.na(sumTableNotInData[, statsCont])))
	expect_true(all(sumTableNotInData[, c("statN", "statm", "statPercTotalN")] == 0))
	expect_true(all(is.nan(sumTableNotInData[, c("statPercN")])))
	
})

test_that("A warning is generated if specified combinations of column elements are not available in the data", {
			
	data <- data.frame(
		USUBJID = seq.int(6),
		AGE = seq(20, 62, length.out = 6),
		TRT = factor(rep(c("A", "B"), each = 3)),
		DOSE = factor(rep(c("100", "200"), each = 3)),
		stringsAsFactors = FALSE
	)
			
	## missing groups in colVarDataLevels
	colVarDataLevels <- data.frame(
		TRT = factor(c("A", "B")),
		DOSE = c("0", "200")
	)
	expect_warning(
		sumTable <- computeSummaryStatisticsTable(
			data,
			var = "AGE",
			colVar = c("TRT", "DOSE"),
			colVarDataLevels = colVarDataLevels
		),
		"Some variable records are not present in the data used for variable levels."
	)
	
})

test_that("Columns in the row total data but not in the data are correctly included", {
			
	dataAll <- data.frame(
		USUBJID = rep(c(1:6), each = 2),
		TRT = rep(c("A", "B"), each = 6),
		COD = rep(c("Term1", "Term2", "Term3"), each = 4),
		stringsAsFactors = FALSE
	)
	data <- subset(dataAll, TRT == "A")
	dataTotalRow <- list(COD = dataAll)
	
	expect_silent(
		summaryTable <- computeSummaryStatisticsTable(
			data = data,
			colVar = "TRT",
			rowVar = "COD",
			rowVarTotalInclude = "COD",
			stats = getStats("n (%)"),
			dataTotalRow = dataTotalRow
		)
	)
	expect_s3_class(summaryTable, "summaryTable")
	expect_identical(levels(summaryTable$TRT), c("A", "B"))
	
	summaryTableGroupOnlyInTotal <- subset(summaryTable, TRT == "B")
	expect_equal(nrow(summaryTableGroupOnlyInTotal), 1)
	expect_equal(as.character(summaryTableGroupOnlyInTotal$COD), "Total")
	expect_equal(summaryTableGroupOnlyInTotal$statN, 3)
	expect_equal(summaryTableGroupOnlyInTotal$statm, 6)
	
})

test_that("The column totals are correctly computed for a subset of the columns", {
			
	data <- data.frame(
		USUBJID = seq.int(5),
		AGE = seq(20, 62, length.out = 5),
		TRT = c("A", "A", "A", "B", "B"),
		DOSE = c("100", "100", "200", "300", "400"),
		stringsAsFactors = FALSE
	)

	expect_silent(
		sumTable <- computeSummaryStatisticsTable(
			data,
			var = "AGE",
			colVar = c("TRT", "DOSE"),
			colVarTotal = "TRT"
		)
	)
	
	expect_equal(sum(sumTable$isTotal), 2)
	
	expect_equal(subset(sumTable, isTotal & TRT == "A")$statN, 3)
	expect_true(all(subset(sumTable, TRT == "A")$statPercTotalN == 3))
	
	expect_equal(subset(sumTable, isTotal & TRT == "B")$statN, 2)
	expect_true(all(subset(sumTable, TRT == "B")$statPercTotalN == 2))
	
	expect_equal(sumTable$statPercN, sumTable$statN/sumTable$statPercTotalN*100)
	
})

test_that("The column totals are correctly computed by the variable to summarize", {
			
	data <- data.frame(
		USUBJID = seq.int(5),
		AGE = seq(20, 62, length.out = 5),
		SEX = c(NA_character_, c("F", "M", "F", "M")),
		TRT = c("A", "A", "A", "B", "B"),
		DOSE = c("100", "100", "200", "300", "400"),
		stringsAsFactors = FALSE
	)
	
	# by variable
	expect_silent(
		sumTable <- computeSummaryStatisticsTable(
			data,
			var = c("AGE", "SEX"),
			colVar = c("TRT", "DOSE"),
			colVarTotal = "variable"
		)
	)
	expect_equal(sum(sumTable$isTotal), 2)
	
	expect_equal(subset(sumTable, isTotal & variable == "AGE")$statN, 5)
	expect_true(all(subset(sumTable, variable == "AGE")$statPercTotalN == 5))
	
	expect_equal(subset(sumTable, isTotal & variable == "SEX")$statN, 4)
	expect_true(all(subset(sumTable, variable == "SEX")$statPercTotalN == 4))
	
	expect_equal(sumTable$statPercN, sumTable$statN/sumTable$statPercTotalN*100)
	
})

test_that("A warning is generated if the column for the totals is not available in the data", {

	data <- data.frame(
		USUBJID = seq.int(5),
		AGE = seq(20, 62, length.out = 5),
		TRT = c("A", "A", "A", "B", "B"),
		stringsAsFactors = FALSE
	)
			
	expect_warning(
		computeSummaryStatisticsTable(
			data,
			var = "AGE",
			colVar = "TRT",
			colVarTotal = "TRT2"
		),
		"Variable.* in colVarTotal.* ignored because.*not available"
	)

})

test_that("Column percentages are correctly computed for a subset of the columns", {
			
	data <- data.frame(
		USUBJID = seq.int(5),
		AGE = seq(20, 62, length.out = 5),
		TRT = c("A", "A", "A", "B", "B"),
		DOSE = c("100", "100", "200", "300", "400"),
		stringsAsFactors = FALSE
	)
	
	expect_silent(
		sumTable <- computeSummaryStatisticsTable(
			data,
			var = "AGE",
			colVar = c("TRT", "DOSE"),
			colVarTotalPerc = "TRT"
		)
	)
	
	# column total still computed per TRT and DOSE
	expect_equal(sum(sumTable$isTotal), 4)
	sumTableTotal <- subset(sumTable, isTotal)
	expect_equal(
		sumTableTotal[match(c("100", "200", "300", "400"), sumTableTotal$DOSE), "statN"], 
		c(2, 1, 1, 1)
	)
	
	# but percentage are computed by treatment
	expect_true(all(subset(sumTable, TRT == "A")$statPercTotalN == 3))
	expect_true(all(subset(sumTable, TRT == "B")$statPercTotalN == 2))
	expect_equal(sumTable$statPercN, sumTable$statN/sumTable$statPercTotalN*100)
	
})	

test_that("Column percentages are correctly computed by the variable to summarize", {
		
	data <- data.frame(
		USUBJID = seq.int(5),
		AGE = seq(20, 62, length.out = 5),
		SEX = c(NA_character_, c("F", "M", "F", "M")),
		TRT = c("A", "A", "A", "B", "B"),
		DOSE = c("100", "100", "200", "300", "400"),
		stringsAsFactors = FALSE
	)		
	
	# by variable
	expect_silent(
		sumTable <- computeSummaryStatisticsTable(
			data,
			var = c("AGE", "SEX"),
			colVar = c("TRT", "DOSE"),
			colVarTotalPerc = "variable"
		)
	)
	
	# column total still computed per TRT and DOSE
	expect_equal(sum(sumTable$isTotal), 4)
	
	# but percentage are computed by treatment
	expect_true(all(subset(sumTable, variable == "AGE")$statPercTotalN == 5))
	expect_true(all(subset(sumTable, variable == "SEX")$statPercTotalN == 4))
	expect_equal(sumTable$statPercN, sumTable$statN/sumTable$statPercTotalN*100)
		
})

test_that("A warning is generated if the column for the percentages is not available in the data", {
			
	data <- data.frame(
		USUBJID = seq.int(5),
		AGE = seq(20, 62, length.out = 5),
		TRT = c("A", "A", "A", "B", "B"),
		stringsAsFactors = FALSE
	)	
			
	# wrong specification
	expect_warning(
		sumTable <- computeSummaryStatisticsTable(
			data,
			var = "AGE",
			colVar = "TRT",
			colVarTotalPerc = "TRT2"
		),
		"Variable.* in colVarTotalPerc.* ignored because.*not available"
	)
	
})

context("Get set of statistics from data")

library(tibble)

test_that("Count statistics are extracted by default if the data and type of statistics are not specified", {
			
	expect_identical(
		getStatsData(data.frame()),	
		getStatsData(type = "count-default")
	)
		
})

test_that("Count statistics are extracted by default if a categorical variable is specified", {
		
	data <- data.frame(group = LETTERS[seq.int(10)])
	stat <- getStatsData(data, var = "group")
	expect_type(stat, "list")
	expect_length(stat, 1)
	expect_named(stat, "group")
	expect_identical(
		stat$group,
		getStats(type = "count-default")
	)
			
})

test_that("All count statistics are extracted for a categorical variable if specified", {
			
	data <- data.frame(group = LETTERS[seq.int(10)])
	stat <- getStatsData(data, var = "group", type = "all")
	expect_type(stat, "list")
	expect_length(stat, 1)
	expect_named(stat, "group")
	expect_identical(
		stat$group,
		getStats(type = "count", x = data$value)
	)
	
})


test_that("Summary statistics are extracted by default if a continuous variable is specified", {
			
	data <- data.frame(value = rnorm(10))
	stat <- getStatsData(data, var = "value")
	expect_type(stat, "list")
	expect_length(stat, 1)
	expect_named(stat, "value")
	expect_identical(
		stat$value,
		getStats(type = "summary-default", x = data$value)
	)
			
})

test_that("All summary statistics are extracted for a continuous variable if specified", {
			
	data <- data.frame(value = rnorm(10))
	stat <- getStatsData(data, var = "value", type = "all")
	expect_type(stat, "list")
	expect_length(stat, 1)
	expect_named(stat, "value")
	expect_identical(
		stat$value,
		getStats(type = "summary", x = data$value)
	)
	
})

test_that("Default statistics are correctly extracted if categorical and continuous variables are specified", {
	
	data <- data.frame(
		value = rnorm(10),
		group = LETTERS[seq.int(10)]
	)
	stat <- getStatsData(data, var = c("value", "group"))
	
	expect_identical(
		stat,
		c(getStatsData(data, var = "value"), getStatsData(data, var = "group"))
	)
			
})

test_that("Statistics are correctly extracted when specified separately for categorical and continuous variables", {
			
	data <- data.frame(
		value = rnorm(10),
		group = LETTERS[seq.int(10)]
	)
	type <- c(cat = "%", cont = "Mean", cont = "Median", "n")
	stat <- getStatsData(data, var = c("value", "group"), type = type)
			
	statSep <- c(
		getStatsData(data, var = "value", type = c("Mean", "Median", "n")), 
		getStatsData(data, var = "group", type = c("%", "n"))
	)
	
	expect_identical(stat, statSep)
	
})

test_that("Parameters are correctly passed for categorical and continuous variables separately", {

	data <- data.frame(
		value = rnorm(10),
		group = LETTERS[seq.int(10)]
	)
	args <- list(
		cat = list(includeName = FALSE),
		cont = list(includeName = TRUE)
	)
			
	stat <- getStatsData(
		data, 
		var = c("value", "group"), 
		type = "n", 
		args = args
	)
			
	statSep <- c(
		getStatsData(data, var = "value", type = "n", includeName = TRUE), 
		getStatsData(data, var = "group", type = "n", includeName = FALSE)
	)
			
	expect_identical(stat, statSep)
			
})

test_that("An extra custom statistic as a function of the variable is correctly extracted", {
			
	data <- data.frame(
		value = rnorm(10)
	)
	myCustomStat <- function(x){
		bquote(mean(x)/.(length(x)))
	}
	stat <- getStatsData(
		data = data, 
		var = "value", 
		type = "Mean",
		extra = list(myCustomStat = myCustomStat)
	)
	expect_named(stat$value, c("Mean", "myCustomStat"))
	expect_equal(
		stat$value$myCustomStat, 
		myCustomStat(x = data$value)
	)
			
})

test_that("An extra custom statistic as an expression is correctly extracted", {
			
	data <- data.frame(value = rnorm(10))
	myCustomStat <- bquote(mean(x))
	stat <- getStatsData(
		data = data, 
		var = "value", 
		type = "Mean",
		extra = list(myCustomStat = myCustomStat)
	)
	expect_named(stat$value, c("Mean", "myCustomStat"))
	expect_equal(
		stat$value$myCustomStat, 
		myCustomStat
	)
			
})

test_that("General parameters are correctly passed to the generic statistic extraction function", {
			
	data <- data.frame(value = rnorm(10))
	stat <- getStatsData(
		data = data, 
		var = "value", 
		type = "Mean",
		nDecCont = 3
	)
	expect_equal(
		stat$value, 
		getStats(type = "Mean", nDecCont = 3)
	)
		
})

test_that("Statistics are correctly extracted for a numeric variable for a tibble", {
		
	data <- data.frame(value = rnorm(10))
	data <- tibble::as_tibble(data)
	typeStats <- c(cont = "Median", cat = "n")
	stats <- getStatsData(
		data = data, 
		var = "value",
		type = typeStats
	)
	expect_named(stats[["value"]], "Median")
			
})

#' @importFrom clinUtils clinColors clinShapes clinLinetypes
.onAttach <- function(libname, pkgname) {
  
  # options for tables in presentation mode
  options(inTextSummaryTable.pageDim.presentation = pageDimPresentation)
  options(inTextSummaryTable.colors.table.presentation = tableColorsPresentation)
  
  # palettes for plots
  options(inTextSummaryTable.colors.plot = clinUtils::clinColors)
  options(inTextSummaryTable.shapes.plot = clinUtils::clinShapes)
  options(inTextSummaryTable.linetypes.plot = clinUtils::clinLinetypes)
  
}

#' Compute summary statistics for a specific dataset and variables of interest
#' @param rowOrder Specify how the rows should be ordered in the final table, either a:
#' \itemize{
#' \item{String among:
#' \itemize{
#' \item{'auto' (by default): if the variable is a factor, keep its order, otherwise order alphabetically}
#' \item{'alphabetical': order alphabetically}
#' \item{'total': order rows in decreasing order of the total number of subjects
#' across all columns for this specific category.}
#' }}
#' \item{Function with input the summary table and output the ordered elements of the \code{rowVar}}
#' }
#' To specify different ordering methods for different \code{rowVar}, specify a list
#' of such elements, named with the \code{rowVar} variable.
#' For the table output of \code{\link{computeSummaryStatisticsTable}} (long format),
#' this order is also reflected in the \strong{\code{levels}} of the row factor variable.
#' @param rowOrderTotalFilterFct Function used to filter the data used to order the rows
#' based on total counts (in case \code{rowOrder} is 'total'),
#' To order rows based on one specific column category,
#' e.g. to order based on the counts in the treatment column:
#' function(x) subset(x, TRTP == "treatmentX")
#' @param rowOrderCatLast String with category to be printed in the last 
#' row of each \code{rowVar} (if any, set to NULL if none). 
#' @param rowVarTotalByVar Character vector with a row variable
#' used to categorize the row total.\cr
#' Note that this is only used if row total(s) is/are requested via \code{rowVarTotalInclude},
#' and this variable should also be included in \code{rowVar}.
#' This can be specified also for a specific row variable if the vector is named.\cr
#' For example: \code{c(ADECOD = "AESEV")} to compute total by severity 
#' for row adverse event term in a typical adverse event count table 
#' (by System Organ Class and Adverse Event Term).
#' @param varGeneralLab String with general label for variable specified in \code{var}.
#' In case of multiple variable in \code{var}, this will be included in the table header
#' (see 'rowVarLab' attribute of the output).
#' @param varSubgroupLab String with general label for sub-group of
#' categorical variable(s) for count table, 'Variable group' by default.
#' This will be included in the final table header (see 'rowVarLab' attribute of the output).
#' @param varIgnore Vector with elements to ignore in the \code{var} variable(s).
#' The \code{data} records with such elements in \code{var} are \strong{filtered} from the data 
#' at the start of the workflow.
#' @param dataTotal Data.frame used to extract the Total number of subject
#' per column in column header ('N = [X]').
#' It should contain the variables specified by \code{colVarTotal}.
#' If not specified, the total number of subjects is extracted from the \code{data}.
#' @param dataTotalPerc Data.frame used to extract the total counts per column 
#' for the computation of the percentage.\cr
#' By default, \code{dataTotal} is used.\cr
#' It should contain the variables specified by \code{colVarTotalPerc}.
#' @param dataTotalRow Data.frame used to extract the total count across all
#' elements of the row
#' variable, list of such data.frame for each \code{rowVar} variable.\cr
#' If the dataset is specified by row variable, the list should be named with:
#' variable X if the total across elements of variable X should be included.
#' By default, \code{data} is used.
#' @param dataTotalCol Data.frame from which the total across columns is 
#' extracted (in case \code{colTotalInclude} is TRUE)
#' or list of such data.frame for each \code{rowVar} variable.\cr
#' If the dataset is specified by row variable, the list should be named with:
#' with:
#' \itemize{
#' \item{last row variable: for the dataset used in the total column for 
#' the most nested row variable}
#' \item{higher row variable (X+1): for the dataset used for the total column
#' and row total of X}
#' \item{'total': for the dataset used for the total column and general row total}
#' }
#' If only a subset of the variables is specified in this list, 
#' \code{data} is used for the remaining variable(s) (or 'total') if needed.\cr
#' This dataset (the one for 'total' if a list) is also used for:
#' \itemize{
#' \item{the header of the total column in case \code{dataTotal} is
#' not specified}
#' \item{the denominator of the percentages in the total column
#' in case \code{dataTotalPerc} is not specified}
#' }
#' By default, \code{data} is used.
#' @param filterFct (optional) Function taking as input
#' the summary table with computed statistics and returning a subset 
#' of the summary table.\cr
#' Note: The filtering function should also handle records with :
#' \itemize{
#' \item{total for the column header: \code{isTotal} set to TRUE,
#' and \code{colVar}/\code{rowVar} is NA.\cr
#' For example: \code{filterFct = function(data) subset(data, isTotal & myColVar == "group 1")}
#' }
#' \item{\code{rowVar}/\code{colVar} set to 'Total'/\code{colTotalLab} 
#' if \code{rowVarTotalInclude}/\code{colTotalInclude} is specified}
#' }
#' @param colVarTotal String with column(s) considered to compute the total by,
#' reported in the header of the table, by default same as \code{colVar}.
#' Use: 'variable' to compute total by \code{var} (if multiple).
#' @param colVarTotalPerc String with column(s) considered to compute the total by,
#' used as denominator for the percentage computation, by default same as \code{colVarTotal}.
#' Use: 'variable' to compute total by \code{var} (if multiple).
#' @param rowVarTotalPerc Character vector with row variables by which the total
#' should be computed for the denominator for the percentage computation.
#' By default the total is only computed only by column (NULL by default).
#' If the total should be based on the total number of records per variable,
#' \code{rowVarTotalPerc} should be set to 'variable'.
#' @param byVar Variable(s) of \code{data} for which separated table(s)
#' should be created.
#' @param byVarLab String with label for \code{byVar}, used to set the names
#' of the output list of table(s).
#' @param statsPerc String with 'base statistical variable' used to compute the 
#' percentage, either: 
#' \itemize{
#' \item{'statN' (by default): the number of subjects}
#' \item{'statm': the number of records}
#' }
#' @param stats (optional) Statistic(s) of interest to compute, either:
#' \itemize{
#' \item{string with the name of a default set of statistics
#' available in the package, 
#' see section 'Formatted statistics' in
#' \code{\link[=inTextSummaryTable-stats]{in-text table statistics}}.\cr
#' See the corresponding \code{type} parameter of the
#' \code{\link{getStatsData}} for more information
#' on how the statistic is internally extracted.}
#' \item{(expert mode) named list of language object (see \code{\link{is.language}}) 
#' of base summary statistics of interest, see section:
#' 'Base statistics' in
#' \code{\link[=inTextSummaryTable-stats]{in-text table statistics}}.\cr
#' The names are reported in the header.\cr
#' If \code{stats} if of length 1, the name of the summary statistic is not included
#' in the table.\cr
#' The statistics can be specified separately:
#' \itemize{
#' \item{for each \code{var} (if multiple), 
#' by naming each element of the list:
#' \code{list(varName1 = list(...), varName2 = list())}
#' }
#' \item{and/or for each element in:
#' \code{statsVarBy}, by naming each sublist.}
#' }}
#' }
#' @param statsGeneralLab String with general label for statistics, 'Statistic' by default.
#' Only included if no \code{statsVar} if longer than 1.
#' @param statsVarBy String with variable in \code{rowVar}/\code{colVar}
#' which the statistics should be computed by.\cr
#' In this case, \code{stats} (nested list or not) should be additionally nested
#' to specify the statistics for each element in \code{statsVarBy}.
#' @param varIncludeTotal This argument is deprecated, please use: 'varTotalInclude' instead.
#' @param varTotalInSepRow Logical, should the total per variable be included in
#' a separated row (by default) or in the row containing the header of the variable?
#' @inheritParams inTextSummaryTable-common-args
#' @inheritParams computeSummaryStatisticsByRowColVar
#' @return An object \code{\link{summaryTable}}
#' or list of such objects if
#' \code{byVar} is specified.
#' @author Laure Cougnaud
#' @importFrom plyr ddply rbind.fill dlply
#' @importFrom clinUtils getLabelVar
#' @export
computeSummaryStatisticsTable <- function(
	data,  
	var = NULL, varFlag = NULL, varInclude0 = FALSE,
	varLab = NULL,
	varLabInclude = length(var) > 1,
	varGeneralLab = "Variable", varSubgroupLab = "Variable group",
	varIgnore = NULL,
	varIncludeTotal = FALSE,
	varTotalInclude = FALSE,
	varTotalInSepRow = FALSE,
	colVar = NULL, colVarDataLevels = NULL, 
	colVarTotal = colVar, 
	colVarTotalPerc = colVarTotal,
	colTotalInclude = FALSE, colTotalLab = "Total",
	colInclude0 = FALSE,
	rowVar = NULL, rowVarDataLevels = NULL, 
	rowVarLab = NULL,
	rowOrder = "auto", rowOrderTotalFilterFct = NULL, rowOrderCatLast = NULL,
	rowVarTotalInclude = NULL,
	rowVarTotalInSepRow = NULL,
	rowVarTotalByVar = NULL,
	rowVarTotalPerc = NULL,
	rowInclude0 = FALSE, 
	type = "auto",
	subjectVar = "USUBJID",	
	dataTotal = NULL, dataTotalPerc = dataTotal,
	dataTotalRow = NULL,
	dataTotalCol = NULL,
	stats = NULL, 
	statsVarBy = NULL,
	statsExtra = NULL,
	statsGeneralLab = "Statistic",
	statsPerc = c("statN", "statm"),
	filterFct = NULL,
	labelVars = NULL,
	byVar = NULL, byVarLab = NULL,
	checkVarDiffBySubj = "error"
){
	
	inputParams <- as.list(environment())
	
	statsPerc <- match.arg(statsPerc)
	
	if(!inherits(data, "data.frame"))
		stop("A data.frame should be specified in 'data'.")
	
	if(class(data)[1] != "data.frame")
		data <- as.data.frame(data)
	
	if(nrow(data) == 0){
		message("No data to report.")
		return(invisible())
	}

	## compute multiple statistics table based on 'byVar'
	byVar <- checkVar(var = byVar, varLabel = "byVar", data = data)
	if(!is.null(byVar)){
		
		byVarLab <- getLabelVar(byVar, data = data, labelVars = labelVars, label = byVarLab)
		
		res <- dlply(data, byVar, function(dataBy){
			inputParamsBy <- inputParams
			inputParamsBy$data <- dataBy
			inputParamsBy$byVar <- NULL
			do.call(computeSummaryStatisticsTable, inputParamsBy)		
		})	
		uniqueNameDf <- unique(data[, byVar, drop = FALSE])
		newName <- do.call(paste, 
			c(mapply(paste, byVarLab[byVar], uniqueNameDf, sep = ": ", SIMPLIFY = FALSE),
			sep = "\n")
		)
		initName <- do.call(paste, c(uniqueNameDf, sep = "."))
		names(res) <- newName[match(names(res), initName)]
			
		return(res)
			
	}
	
	# get default set of statistics
	if(is.character(stats)){
		stats <- getStatsData(data = data, var = var, type = stats)
		# if only one variable is specified, 'stats' should not be named:
		if(!is.null(var) && length(var) == 1)	stats <- stats[[var]]
	}
	
	# add default statistics if geometric mean/sd are requested
	statsCall <- sapply(unlist(stats, recursive = TRUE), deparse)
	statsExtra <- c(
	  statsExtra,
	  if(any(grepl("statGeomMean", statsCall, ignore.case = TRUE)) && 
	     !"statGeomMean" %in% names(statsExtra))
	    list(statGeomMean = inTextSummaryTable::geomMean),
	  if(any(grepl("statGeomSD", statsCall, ignore.case = TRUE)) &&
	    !"statGeomSD" %in% names(statsExtra))
	    list(statGeomSD = inTextSummaryTable::geomSD)
	)
	
	# check if 'varIncludeTotal' is not default
	if(!(is.logical(varIncludeTotal) && length(varIncludeTotal) == 1 && !varIncludeTotal)){
		warning("Argument: 'varIncludeTotal' is deprecated, please use 'varTotalInclude' instead.")
		varTotalInclude <- varIncludeTotal
	}
	
	# check var
	var <- checkVar(var = var, varLabel = "var", data = data, varUncheck = "all")
	varFlag <- checkVar(var = varFlag, varLabel = "varFlag", varRef = var, refLabel = "var")
	varInclude0 <- checkVar(
		var = varInclude0, varLabel = "varInclude0", 
		varRef = var, refLabel = "var",
		varUncheck = c(TRUE, FALSE)
	)
	varLabInclude <- checkVarLabInclude(var = var, varLabInclude = varLabInclude)
	varTotalInclude <- checkVar(var = varTotalInclude, varLabel = "varTotalInclude", varRef = var, refLabel = "var", varUncheck = c(TRUE, FALSE))
	
	# check row var parameters:
	rowVar <- checkVar(var = rowVar, varLabel = "rowVar", data = data)
	rowVarTotalInclude <- checkVar(var = rowVarTotalInclude, varLabel = "rowVarTotalInclude", varRef = rowVar, refLabel = "rowVar")
	rowVarTotalInSepRow <- checkVar(var = rowVarTotalInSepRow, varLabel = "rowVarTotalInSepRow", varRef = rowVarTotalInclude, refLabel = "rowVarTotalInclude")
	if(!varLabInclude & "variable" %in% rowVarTotalPerc){
		warning("Percentages cannot be computed by 'variable' because variable not included in the table (varLabInclude is FALSE).")
		rowVarTotalPerc <- setdiff(rowVarTotalPerc, "variable")
	}
	rowVarTotalPerc <- checkVar(var = rowVarTotalPerc, varLabel = "rowVarTotalPerc", varRef = rowVar, refLabel = "rowVar", varUncheck = "variable")
	
	# in case the variable should be in multiple columns, 'colVar' might include: 'variable'
	colVarInit <- colVar <- checkVar(var = colVar, varLabel = "colVar", data = data, varUncheck = "variable")
	if("variable" %in% colVar){
		if(!(!is.null(var) && varLabInclude))
			warning(paste(
				"'var' not included in columns because",
				"'varLabInclude' is FALSE.",
				"You might want to set: 'statsLayout' to 'col'."
			))
	}
	colVar <- setdiff(colVar, "variable")
	if(length(colVar) == 0)	colVar <- NULL
	colVarTotal <- checkVar(var = colVarTotal, varLabel = "colVarTotal", varRef = colVar, refLabel = "colVar", varUncheck = "variable")
	colVarTotalPerc <- checkVar(var = colVarTotalPerc, varLabel = "colVarTotalPerc", varRef = colVar, refLabel = "colVar", varUncheck = "variable")

	# check variable(s) label
	varLab <- getLabelVar(var, data = data, labelVars = labelVars, label = varLab)
	rowVarLab <- getLabelVar(rowVar,  data = data, labelVars = labelVars, label = rowVarLab)
	if(is.null(varGeneralLab) || !is.character(varGeneralLab) ||length(varGeneralLab) > 1){
		warning("'varGeneralLab' should be a character of length 1, it is set to 'Variable' by default.")
		varGeneralLab <- "Variable"
	}
	if(is.null(varSubgroupLab) || !is.character(varSubgroupLab) ||length(varSubgroupLab) > 1){
		warning("'varSubgroupLab' should be a character of length 1, it is set to 'Variable group' by default.")
		varSubgroupLab <- "Variable group"
	}
	
	# check stats params:
	statsVarBy <- checkVar(
		var = statsVarBy , varLabel = "statsVarBy",
		varRef = c(colVar, rowVar), refLabel = "row or column variables",
		msgType = "error"
	)
	
	# Compute the column total in case:
	# - the rows should be ordered based on the total category
	# - total should be extracted within a function specified in rowOrder
	# - excepted when no column variable is specified
	colTotalIncludeInit <- colTotalInclude
	if(is.null(colVar)){
		if(colTotalIncludeInit)	
			warning("Column 'total' is not included because no column variable is specified.")
		colTotalInclude <- FALSE
	}else{
		check <- sapply(rowOrder, function(x) !(is.character(x) && x != "total"))
		if(any(check))	colTotalInclude <- TRUE
	}
	
#	if(!is.null(colVar))
#	checkIfTotal <- function(x)	!is.function(x) && any(x == "total")
#	if(!colTotalInclude && any(sapply(rowOrder, checkIfTotal)))
#		colTotalInclude <- TRUE
	
	# ignore certain elements
	if(!is.null(var) && !is.null(varIgnore))
		for(varI in var){
			if(varI != "all")
				data <- data[!data[, varI] %in% varIgnore, ]
		}
	
	# for flag variable:
	if(!is.null(varFlag)){
		
		# convert them to a format to only retain flagged records
		data[, varFlag] <- colwise(convertVarFlag)(data[, varFlag, drop = FALSE])
		
		postProcessVarFlagTable <- function(summaryTable)
			postProcessVarFlag(summaryTable = summaryTable, varFlag = varFlag)
		filterFct <- c(filterFct, list(postProcessVarFlagTable))
		
	}
	
	# convert row/column variables to factor
	data <- convertVarRowVarColVarToFactor(
		data = data, rowVar = rowVar, colVar = colVar, var = var
	)
		
	# get general statistics for each combination of rowVar/colVar
	summaryTable <- computeSummaryStatisticsByRowColVar(
		data = data, 
		var = var, varLab = varLab, varTotalInclude = varTotalInclude, varInclude0 = varInclude0,
		varLabInclude = varLabInclude,
		statsExtra = statsExtra,
		type = type,
		rowVar = rowVar, rowInclude0 = rowInclude0, rowVarDataLevels = rowVarDataLevels,
		colVar = colVar, colInclude0 = colInclude0, colVarDataLevels = colVarDataLevels,
		subjectVar = subjectVar, labelVars = labelVars,
		checkVarDiffBySubj = checkVarDiffBySubj
	)
	
	if(nrow(summaryTable) == 0)
		stop("No data available (or all missing values).")
	
	# original levels of colVar in summaryTable
	colVarLevels <- sapply(
		summaryTable[, colVar, drop = FALSE], function(x)
			if(is.factor(x))	levels(x)	else	sort(unique(x)),
		simplify = FALSE
	)
	
	# get statistics across columns by row
	if(colTotalInclude){
		
		dataForColTotal <- if(!is.null(dataTotalCol)){
			# different datasets for the different row variables:
			if(is.list(dataTotalCol) && !is.data.frame(dataTotalCol)){ 
				if(!is.null(rowVar) && rowVar[length(rowVar)] %in% names(dataTotalCol)){
					dataTotalCol[[rowVar[length(rowVar)]]]
				}else	data
			# one unique df:
			}else	dataTotalCol
		}else	data

		# convert row/column variables to factor
		dataForColTotal <- convertVarRowVarColVarToFactor(
			data = dataForColTotal, 
			rowVar = rowVar, 
			colVar = NULL,
			var = var
		)
		
		summaryTableColTotal <- computeSummaryStatisticsByRowColVar(
			data = dataForColTotal, 
			var = var, varLab = varLab, varTotalInclude = varTotalInclude, varInclude0 = varInclude0,
			varLabInclude = varLabInclude,
			statsExtra = statsExtra,
			type = type,
			rowVar = rowVar, rowInclude0 = rowInclude0,	rowVarDataLevels = rowVarDataLevels,
			subjectVar = subjectVar, labelVars = labelVars,
			msgLabel = "total column",
			checkVarDiffBySubj = checkVarDiffBySubj
		)
		
		if(nrow(summaryTableColTotal) > 0)
			summaryTableColTotal[, colVar] <- colTotalLab
			
		summaryTable <- rbind.fill(summaryTable, summaryTableColTotal)
		summaryTable[, colVar] <- lapply(colVar, function(x)
			factor(summaryTable[, x], levels = unique(c(colVarLevels[[x]], colTotalLab)))
		)
		
		# extract the data used for the total header + percentage:
		dataForColTotalPercHeader <- if(!is.null(dataTotalCol)){
			# different datasets for the different row variables:
			if(is.list(dataTotalCol) && !is.data.frame(dataTotalCol)){ 
				if(!is.null(rowVar) && 'total' %in% names(dataTotalCol)){
					dataTotalCol[["total"]]
				}else	data
				# one unique df:
			}else	dataTotalCol
		}else	data
	
		# convert row/column variables to factor
		dataForColTotalPercHeader <- convertVarRowVarColVarToFactor(
			data = dataForColTotalPercHeader, 
			rowVar = rowVar, 
			colVar = NULL,
			var = var
		)
		
	}
	
	if(!is.null(rowVarTotalInclude)){
		
		# order specified variables as in rowVar
		rowVarTotalInclude <- intersect(rowVar, rowVarTotalInclude)
		
		# compute sub-total for each specified rowVar (excepted the last one)
		summaryTableRowSubtotal <- data.frame()
		for(rVST in rowVarTotalInclude){
			
			dataForSubTotal <- if(!is.null(dataTotalRow)){
				if(is.data.frame(dataTotalRow)){
					dataTotalRow
				}else{
					if(rVST %in% names(dataTotalRow)){
						dataTotalRow[[rVST]]
					}else	stop(paste0("Dataset for row total missing for: ", shQuote(rVST), "."))
				}
			}else	data
	
			# remove rows which have NA for the nested sub-variable
			# otherwise have summary statistics are duplicated (sub-total and initial)
			rowVarSubTotalOther <- rowVar[
				setdiff(seq_along(rowVar), seq_len(match(rVST, rowVar)))
			]
			filterRowNestedVar <- function(data){
				if(length(rowVarSubTotalOther) > 0){
					
					checkVar(var = rowVarSubTotalOther, "rowVar", data = data, 
						refLabel = "dataset for row total", msgType = "error")
					
					idxMissingSubVar <- which(
						rowSums(is.na(data[, rowVarSubTotalOther, drop = FALSE])) == length(rowVarSubTotalOther)
					)
					if(length(idxMissingSubVar) > 0)
						data <- data[-idxMissingSubVar, ]
				}
				data
			}
			dataForSubTotal <- filterRowNestedVar(data = dataForSubTotal)
			
			# compute statistics by higher level rowVar
			rowVarOther <- rowVar[seq_len(match(rVST, rowVar)-1)]
			if(length(rowVarOther) == 0) rowVarOther <- NULL
			
			# extra variable used to compute the total by
			rowVarTotalByVarI <- if(!is.null(rowVarTotalByVar)){
				if(!is.null(names(rowVarTotalByVar))){
					if(rVST %in% names(rowVarTotalByVar))
						rowVarTotalByVar[rVST]
				}else	rowVarTotalByVar
			}
			rowVarTotalByVarI <- checkVar(
				var = rowVarTotalByVarI, 
				varLabel = "rowVarTotalByVar",
				varRef = rowVar,
				refLabel = "rowVar"
			)
			rowVarOther <- c(rowVarOther, rowVarTotalByVarI)
			
			checkVar(
				var = c(rowVarOther, colVar, var), "variables", data = dataForSubTotal, 
				refLabel = "dataset for row total", msgType = "error",
				varUncheck = "all"
			)
			
			# convert row/column variables to factor
			dataForSubTotal <- convertVarRowVarColVarToFactor(
				data = dataForSubTotal, 
				rowVar = rowVarOther, 
				colVar = colVar,
				var = var
			)
			
			# compute statistics
			summaryTableRowSubtotalVar <- computeSummaryStatisticsByRowColVar(
				data = dataForSubTotal, 
				var = var, varTotalInclude = varTotalInclude, varInclude0 = varInclude0,
				varLabInclude = varLabInclude,
				statsExtra = statsExtra,
				type = type,
				rowVar = rowVarOther, rowInclude0 = rowInclude0, rowVarDataLevels = rowVarDataLevels,
				colVar = colVar, colInclude0 = colInclude0, colVarDataLevels = colVarDataLevels,
				subjectVar = subjectVar, varLab = varLab, labelVars = labelVars,
				msgLabel = paste("row total for", rVST),
				checkVarDiffBySubj = checkVarDiffBySubj
			)
			
			# include also the total across columns (if required)
			if(colTotalInclude){
				
				rowVarOtherCol <- if(is.null(rowVarOther))	'total'	else	rowVarOther
				dataForSubTotalForColTotal <- if(!is.null(dataTotalCol)){
					# different datasets for the different row variables:
					if(is.list(dataTotalCol) && !is.data.frame(dataTotalCol)){ 
						if(!is.null(rowVar) && rowVarOtherCol %in% names(dataTotalCol)){
							filterRowNestedVar(dataTotalCol[[rowVarOtherCol]])
						}else	dataForSubTotal
					# one unique df:
					}else	dataTotalCol
				}else	dataForSubTotal
		
				# convert row/column variables to factor
				dataForSubTotalForColTotal <- convertVarRowVarColVarToFactor(
					data = dataForSubTotalForColTotal, 
					rowVar = rowVarOther, 
					colVar = NULL,
					var = var
				)
				
				checkVar(var = c(rowVarOther, var), "variables", data = dataForSubTotalForColTotal, 
					refLabel = "dataset for row total across columns", msgType = "error")
				
				summaryTableRowSubtotalVarColTotal <- computeSummaryStatisticsByRowColVar(
					data = dataForSubTotalForColTotal, 
					var = var, varTotalInclude = varTotalInclude, varInclude0 = varInclude0,
					varLabInclude = varLabInclude,
					statsExtra = statsExtra,
					type = type,
					rowVar = rowVarOther, rowInclude0 = rowInclude0, rowVarDataLevels = rowVarDataLevels,
					subjectVar = subjectVar, varLab = varLab, labelVars = labelVars,
					msgLabel = paste("total column for the row total for", rVST),
					checkVarDiffBySubj = checkVarDiffBySubj
				)
				if(nrow(summaryTableRowSubtotalVarColTotal) > 0)
					summaryTableRowSubtotalVarColTotal[, colVar] <- colTotalLab
				summaryTableRowSubtotalVar <- rbind.fill(summaryTableRowSubtotalVar, summaryTableRowSubtotalVarColTotal)
				summaryTableRowSubtotalVar[, colVar] <- lapply(colVar, function(x)
					factor(summaryTableRowSubtotalVar[, x], levels = unique(c(colVarLevels[[x]], colTotalLab)))
				)
			}
			
			# set other row variables to 'Total'
			if(nrow(summaryTableRowSubtotalVar) > 0)
				summaryTableRowSubtotalVar[, setdiff(rowVar, rowVarOther)] <- "Total"
			
			# save results
			summaryTableRowSubtotal <- rbind.fill(summaryTableRowSubtotal, summaryTableRowSubtotalVar)
			
		}
	
		# bind the df with row total and subtotal to the summary table
		# ensure that the order of the levels of the variable are retained		
		rowVarLevels <- sapply(
			summaryTable[, rowVar, drop = FALSE], function(x)
				if(is.factor(x))	levels(x)	else	sort(unique(x)),
			simplify = FALSE
		)
		
		summaryTable <- rbind.fill(summaryTable, summaryTableRowSubtotal)
		
		summaryTable[, rowVar] <- lapply(rowVar, function(var){
			xVar <- summaryTable[, var]
			# only add Total if included in the table (case: missing nested var)
			levelsX <- unique(c("Total", rowVarLevels[[var]]))
			factor(xVar, levels = levelsX)
		})
		
	}
	
	## get counts for the entire dataset
	
	# data considered to compute the total

	isDataTotalSpec <- !is.null(dataTotal)
	
	# by default, if not specified, data is used
	if(!isDataTotalSpec){
		dataTotal <- data
		# in case no 'dataTotal' is included, consider 'dataTotalCol' for the header across columns
		dataTotalColTotalHeader <- if(colTotalInclude)	dataForColTotalPercHeader
	}
	# check if colVarTotal is in dataTotal:
	colVarTotal <- checkVar(var = colVarTotal, varLabel = "colVarTotal", 
		data = dataTotal, refLabel = "total dataset", varUncheck = "variable")

	# format columns as factor in correct order
	if(isDataTotalSpec){
		# to have specified order for colVar in case different order 'dataTotal'
		colVarTotalUsed <- setdiff(colVarTotal, "variable")
		if(!is.null(colVarTotalUsed)){
			dataTotal[, colVarTotalUsed] <- lapply(colVarTotalUsed, function(var)
				if(is.factor(summaryTable[, var])){
					factor(dataTotal[, var], levels = colVarLevels[[var]])
				}else dataTotal[, var]
			)
		}
		dataTotalColTotalHeader <- dataTotal
	}
	
	# get total for column headers:

	# convert row/column variables to factor
	dataTotal <- convertVarRowVarColVarToFactor(
		data = dataTotal, 
		rowVar = NULL, 
		colVar = colVar,
		var = var
	)
	summaryTableTotal <- computeSummaryStatisticsTableTotal(
		data = dataTotal, 
		colVar = colVar, colVarTotal = colVarTotal,
		colTotalLab = colTotalLab,
		colInclude0 = colInclude0, 
		colTotalInclude = colTotalInclude, dataTotalCol = dataTotalColTotalHeader,
		colVarDataLevels = colVarDataLevels, colVarLevels = colVarLevels,
		subjectVar = subjectVar,
		# not used:
		var = var, varLab = varLab, labelVars = labelVars,
		msgLabel = "header total",
		checkVarDiffBySubj = checkVarDiffBySubj
	)
	
	# save total or not in the 'isTotal' column
	summaryTableTotal$isTotal <- TRUE
	if(nrow(summaryTable) > 0)
		summaryTable$isTotal <- FALSE
	# bind to the summary table
	summaryTable <- rbind.fill(summaryTable, summaryTableTotal)
	
	## compute percentages
	
	# get counts (for percentage computation)
	if(!all(colVarTotalPerc %in% colnames(summaryTable)))
		stop("'colVarTotalPerc' are not in the computed summary statistics table.")
	computeTotalPerc <- 
		!setequal(colVarTotal, colVarTotalPerc) | 
		!is.null(rowVarTotalPerc) | 
		!is.null(dataTotalPerc)

	if(is.null(dataTotalPerc)){
		dataTotalPerc <- dataTotal
		dataTotalPercTotalHeader <- dataTotalColTotalHeader
	}else{
		# convert row/column variables to factor
		dataTotalPerc <- convertVarRowVarColVarToFactor(
			data = dataTotalPerc, 
			rowVar = NULL, 
			colVar = colVar,
			var = var
		)
		dataTotalPercTotalHeader <- dataTotalPerc
	}
	
	# check if colVarTotal is in dataTotal:
	colVarTotalPerc <- checkVar(
		var = colVarTotalPerc, 
		varLabel = "colVarTotalPerc", 
		data = dataTotalPerc, 
		refLabel = "total dataset for percentage", 
		varUncheck = "variable"
	)

	summaryTableTotalPerc <- if(computeTotalPerc){

		computeSummaryStatisticsTableTotal(
			data = dataTotalPerc, dataTotalCol = dataTotalPercTotalHeader,
			colVar = colVar, colVarTotal = colVarTotalPerc,
			colTotalLab = colTotalLab,
			colInclude0 = colInclude0, colTotalInclude = colTotalInclude,
			colVarDataLevels = colVarDataLevels, colVarLevels = colVarLevels,
			rowVarTotal = rowVarTotalPerc, 
			var = var, varLab = varLab, labelVars = labelVars,
			subjectVar = subjectVar,
			msgLabel = "total for percentage",
			checkVarDiffBySubj = checkVarDiffBySubj
		)
		
	}else	summaryTableTotal
	summaryTable <- rbind.fill(
		cbind(summaryTable, isTotalPerc = FALSE), 
		cbind(summaryTableTotalPerc, isTotalPerc = TRUE)
	)
	
	# compute percentages
	statPercCols <- c(
		total = sub("stat(.+)", "statPercTotal\\1", statsPerc),
		perc = sub("stat(.+)", "statPerc\\1", statsPerc)
	)
	summaryTable <- ddply(summaryTable, c(rowVarTotalPerc, colVarTotalPerc), function(x){
		idxTotalPerc <- which(x$isTotalPerc)
		if(length(idxTotalPerc) > 0){
			if(length(idxTotalPerc) != 1)
				stop("Multiple total records for the percentage computation.")
			statPercTotal <- x[idxTotalPerc, statsPerc]
			dataPercTotal <- list(statPercTotal, x[, statsPerc]/statPercTotal*100)
			names(dataPercTotal) <- c(statPercCols["total"], statPercCols["perc"])
			res <- cbind(x, dataPercTotal)
			res[-idxTotalPerc, ]
		}else{
			dataPercTotal <- setNames(
				list(NA_integer_, NA_real_), 
				statPercCols[c("total", "perc")]
			)
			cbind(x, dataPercTotal)
		}
	})
	summaryTable$isTotalPerc <- NULL

	# filter records if any 'filterFct' is specified
	if(!is.null(filterFct)){
		if(!is.list(filterFct))	filterFct <- list(filterFct)
		for(fct in filterFct)
			summaryTable <- fct(summaryTable)
	}
	
	# order the row variables as specified
	if(!is.null(rowVar) && !is.null(rowOrder)){
		summaryTable[, rowVar] <- lapply(rowVar, function(var){
			# (double bracket works for list and character vector)
			methodVar <- if(var %in% names(rowOrder))	rowOrder[[var]]	else rowOrder
			convertVarToFactorWithOrder(
				data = summaryTable, var = var, 
				method = methodVar,
				catLast = rowOrderCatLast,
				totalFilterFct = rowOrderTotalFilterFct,
				otherVars = setdiff(rowVar, var),
				colVar = colVar, colTotalLab = colTotalLab
			)
		})
	}
	
	# extract statistic names
	statsVarInit <- setdiff(
		colnames(summaryTable), 
		c(rowVar, colVar, ".id", "variable", "variableGroup", "isTotal", "variableInit")
	)
	
	# compute stats
	resStats <- getStatisticsSummaryStatisticsTable(
		summaryTable = summaryTable, 
		statsVarInit = statsVarInit, 
		var = var, 
		stats = stats,
		statsVarBy = statsVarBy
	)
	summaryTable <- resStats$summaryTable
	statsVar <- resStats$statsVar

	colsToRemove <- which(colnames(summaryTable) %in% c(".id", "variableInit"))
	if(length(colsToRemove) > 0)
		summaryTable <- summaryTable[, -colsToRemove]
	
	# remove the rows with total if should be ordered by total 
	# but the total not included
	if(!colTotalIncludeInit & colTotalInclude){
		
		idxColTotal <- which(rowSums(summaryTable[, colVar, drop = FALSE] == colTotalLab) > 0)
		if(length(idxColTotal) > 0)
			summaryTable <- summaryTable[-idxColTotal, ]
		removeTotalLevel <- function(x){
			if(is.factor(x) && colTotalLab %in% levels(x)){
				factor(x, levels = setdiff(levels(x), colTotalLab))
			}else	x
		}
		summaryTable[, colVar] <- colwise(removeTotalLevel)(summaryTable[, colVar, drop = FALSE])
	
	}
	
	# if only flag variables, remove 'variableGroup'
	# (otherwise empty line when not specifying 'stats')
	if("variableGroup" %in% colnames(summaryTable) && 
		length(var) > 0 &&
		length(setdiff(var, varFlag)) == 0){
		summaryTable[, which(colnames(summaryTable) == "variableGroup")] <- NULL
	}
	
	# sort columns
	colSorted <- c(rowVar, colVar, "variable", "variableGroup", "isTotal", statsVarInit, statsVar)
	colSorted <- intersect(colSorted, colnames(summaryTable)) 
	colSorted <- c(colSorted, setdiff(colnames(summaryTable), colSorted))
	if(length(colSorted) > 0)
		summaryTable <- summaryTable[, colSorted]
	
	# table attributes
	attrTable <- list(
			
		# attributes created from this function
		statsVar = statsVar,
		rowVar = c(rowVar, 
			if(varLabInclude & !"variable" %in% colVarInit)	"variable", 
			# in case only one variable, but still count
			if("variableGroup" %in% colnames(summaryTable))	"variableGroup"
		),
		rowVarLab = c(
			rowVarLab, 
			if(varLabInclude & !"variable" %in% colVarInit)	
				c("variable" = varGeneralLab),
			if("variableGroup" %in% colnames(summaryTable))	c('variableGroup' = varSubgroupLab),
			if(length(statsVar) > 1)	c("Statistic" = statsGeneralLab)
		),

		# attributes extracted from the input parameters,
		# to set similar defaults for the exportSummaryStatisticsTable
		rowVarTotalInclude = {
			c(
				rowVarTotalInclude,
				# add 'variableGroup' if the total should be computed by variable group
				if(!is.null(var) && (
					(is.logical(varTotalInclude) && varTotalInclude) ||
					any(var %in% varTotalInclude)
				))	"variableGroup"
			)
		},
		rowVarTotalInSepRow = c(rowVarTotalInSepRow, if(varTotalInSepRow)	"variableGroup"),
		colVar = colVarInit,
		colTotalLab = colTotalLab

	)
	
	attributes(summaryTable) <- c(attributes(summaryTable), list(summaryTable = attrTable))
	
	class(summaryTable) <- c("summaryTable", class(summaryTable))
	
	return(summaryTable)
	
}

#' Compute summary statistics total table.
#' @param colVarTotal Character vector with column(s) considered to compute the total.
#' This could also contain 'variable'.
#' @param rowVarTotal Character vector with row(s) considered to compute the total.
#' This could also contain 'variable'.
#' @param colVarLevels list with levels of each \code{colVar}
#' @param dataTotalCol Data.frame from which the total across columns is 
#' extracted (in case \code{colTotalInclude} is TRUE).
#' @param msgLabel (optional) String with label for the data ('total' by default), 
#' included in the message/warning for checks.
#' @inheritParams inTextSummaryTable-common-args
#' @return data.frame with total table.
#' @author Laure Cougnaud
#' @importFrom reshape2 melt
#' @keywords internal
computeSummaryStatisticsTableTotal <- function(
	data, 
	colVar = NULL, colVarTotal = colVar,
	rowVarTotal = NULL, 
	var = NULL, varLab = getLabelVar(var, data = data, labelVars = labelVars),
	colTotalLab = "Total",
	colInclude0 = FALSE,
	colTotalInclude = FALSE, dataTotalCol = NULL,
	colVarDataLevels = NULL, colVarLevels = NULL,
	subjectVar = "USUBJID",
	labelVars = NULL,
	msgLabel = "total",
	checkVarDiffBySubj = "error"){

	# in case total should be computed by 'var'
	# convert wide -> long format: one column with all variables
	formatDataTotalWithVar <- function(data){
		
		if("all" %in% var)	data[, "all"] <- "all"
		
		# to avoid warnings in 'melt': 
		# - 'attributes are not identical across measure variables; they will be dropped'
		# - cannot avoid coercion of factors when measure attributes not identical
		if(!is.null(var))
			data[, var] <- lapply(data[, var, drop = FALSE], as.character)
		
		data <- melt(
			data = data, 
			measure.vars = var, 
			variable.name = "variableInit",
			value.name = "value",
			factorsAsStrings = FALSE # reshape2 >= 1.4
		)	
		data <- data[!is.na(data$value), ]# remove records not in the variable
		data$variable <- factor(
			varLab[as.character(data$variableInit)],
			levels = varLab[var]
		)
		return(data)
		
	}

	# total is computed based on number of elements available in each 'var'
	# (and not # elements within each group of 'var')
	if("variable" %in% c(rowVarTotal, colVarTotal)){
		data <- formatDataTotalWithVar(data)
	}
	
	# counts by elements in colVar
	summaryTableTotal <- computeSummaryStatisticsByRowColVar(
		data = data, 
		type = "countTable", 
		colVar = colVarTotal, 
		rowVar = rowVarTotal,
		colInclude0 = colInclude0, 
		colVarDataLevels = colVarDataLevels,
		subjectVar = subjectVar,
		msgLabel = msgLabel,
		checkVarDiffBySubj = checkVarDiffBySubj
	)
	
	# counts across all elements of colVar
	if(colTotalInclude){
		colVarTotalTI <- setdiff(colVarTotal, colVar)
		if(length(colVarTotalTI) == 0)	colVarTotalTI <- NULL
		if(is.null(dataTotalCol))	dataTotalCol <- data
		
		if("variable" %in% c(rowVarTotal, colVarTotal))
			dataTotalCol <- formatDataTotalWithVar(dataTotalCol)
		
		summaryTableTotalCol <- computeSummaryStatisticsByRowColVar(
			data = dataTotalCol, 
			type = "countTable", 
			colVar = colVarTotalTI,
			rowVar = rowVarTotal,
			subjectVar = subjectVar,
			msgLabel = paste0("total column", 
				if(!is.null(msgLabel))	paste(" for the ", msgLabel)
			),
			checkVarDiffBySubj = checkVarDiffBySubj
		)
		if(nrow(summaryTableTotalCol) > 0)
			summaryTableTotalCol[, colVar] <- colTotalLab
		summaryTableTotal <- rbind.fill(summaryTableTotal, summaryTableTotalCol)
		if(!is.null(colVarLevels))
			summaryTableTotal[, colVar] <- lapply(colVar, function(x)
				factor(summaryTableTotal[, x], levels = unique(c(colVarLevels[[x]], colTotalLab)))
			)
	}
	
	return(summaryTableTotal)
	
}

#' Compute summary statistics by specified \code{rowVar} and \code{colVar}
#' @param rowInclude0 Logical, if TRUE (FALSE by default),
#' include rows with no records, based on all combinations 
#' of the \code{rowVar} (assuming nested variable(s)).
#' @param varInclude0 Logical, should rows with no counts 
#' for the count \code{var} or \code{varFlag} variable(s)
#' be included in the table?
#' Either:
#' \itemize{
#' \item{logical of length 1, if TRUE (FALSE by default) 
#' rows with no count are included for all \code{var}
#' }
#' \item{a character vector containing categorical \code{var} 
#' for which zero counts rows should be included}
#' }
#' @param colInclude0 Logical, if TRUE (FALSE by default),
#' include columns with no records, based on all combinations 
#' of the \code{columnVar} (assuming nested variable(s)).
#' If variable(s) are not nested, possible combinations
#' can be specified via \code{colVarDataLevels}.
#' @param varLab Named character vector with label for each variable 
#' specified in \code{var}.
#' By default, extracted from the \code{labelVars}.
#' if not available, \code{var} is used.
#' @param rowVarDataLevels Data.frame with unique combinations of \code{rowVar}
#' to be included in columns.
#' Each column should correspond to \code{colVar} and as factor
#' if the elements should be ordered in the final table.
#' @param colVarDataLevels Data.frame with unique combinations of \code{colVar}
#' to be included in columns.
#' Each column should correspond to \code{colVar} and as factor
#' if the elements should be ordered in the final table.
#' @param varTotalInclude Should the total across all categories of \code{var} 
#' be included for the count table?
#' Only used for categorical variables (and \code{var} not 'all').
#' Either:
#' \itemize{
#' \item{logical of length 1, if TRUE (FALSE by default) include the total for all categorical \code{var}}
#' \item{a character vector containing categorical \code{var} for which the total should be included}
#' }
#' @param checkVarDiffBySubj String, 'error' (default), 'warning',
#' or 'none'.  
#' Should an error, a warning, or nothing be produced
#' if a continuous variable (\code{var}) contains
#' different values for the same subject (by row/column)?
#' @inheritParams inTextSummaryTable-common-args
#' @return data.frame of class 'countTable' or 'summaryTable',
#' depending on the 'type' parameter; with statistics in columns,
#' either if \code{type} is:
#' \itemize{
#' \item{'summaryTable': 
#' \itemize{
#' \item{'N': number of subjects}
#' \item{'Mean': mean of \code{var}}
#' \item{'SD': standard deviation of \code{var}}
#' \item{'SE': standard error of \code{var}}
#' \item{'Median': median of \code{var}}
#' \item{'Min': minimum of \code{var}}
#' \item{'Max': maximum of \code{var}}
#' \item{'m': number of records}
#' }
#' }
#' \item{'countTable': 
#' \itemize{
#' \item{'N': number of subjects}
#' \item{'m': number of records}
#' }
#' }
#' }
#' @author Laure Cougnaud
#' @importFrom clinUtils getLabelVar
#' @keywords internal
computeSummaryStatisticsByRowColVar <- function(
	data, 
	var = NULL, varLab = getLabelVar(var = var, data = data, labelVars = labelVars), varInclude0 = FALSE,
	varLabInclude = length(var) > 1,
	varTotalInclude = FALSE,
	type = "auto",
	rowVar = NULL, rowInclude0 = FALSE, rowVarDataLevels = NULL,
	colVar = NULL, colInclude0 = FALSE, colVarDataLevels = NULL,
	subjectVar = "USUBJID",
	labelVars = NULL,
	statsExtra = NULL,
	msgLabel = NULL,
	checkVarDiffBySubj = "error"){

	if(is.logical(varTotalInclude) && length(varTotalInclude) > 1)
		stop("If 'varTotalInclude' if a logical, it should be of length 1.")
	
	varLabInclude <- checkVarLabInclude(var = var, varLabInclude = varLabInclude)

	computeSummaryStatisticsCustom <- function(...)
		computeSummaryStatistics(..., 
			subjectVar = subjectVar, 
			statsExtra = statsExtra,
			msgLabel = msgLabel,
			checkVarDiffBySubj = checkVarDiffBySubj
		)
		
	# build variables used for grouping:
	# 1) consider 'ddply(, .drop = FALSE)' to also include zeros
	# 2) build interaction of row variable(s), column variable(s) to only consider 
	# available combinations of row/column variables (if multiple variables in each direction)
	# otherwise ddply(, .drop = FALSE) with also include combinations of elements non present 
	# e.g. AE term in an AE group not present
	# (variables considered independently)
	groupVar <- c(
		if(!is.null(rowVar)){
			if((!rowInclude0) | (!is.null(rowVarDataLevels))){
				resICR <- interactionCustom(data = data, var = rowVar, varDataLevels = rowVarDataLevels)
				data$rowVariables <- resICR$x
				rowDataLevels <- resICR$dataLevels
				"rowVariables"
			}else	rowVar
		},
		if(!is.null(colVar)){
			if((!colInclude0) | (!is.null(colVarDataLevels))){
				resICC <- interactionCustom(data = data, var = colVar, varDataLevels = colVarDataLevels)
				data$colVariables <- resICC$x
				colDataLevels <- resICC$dataLevels
				"colVariables"
			}else colVar	
		}
	)
	
	# get general statistics (by group if specified)
	summaryTable <- ddply(data, groupVar, function(x){
		# compute statistics for each specified variable
		# (loop in case multiple are specified)
		if(is.null(var)){
			sumTable <- computeSummaryStatisticsCustom(
				data = x, 
				var = var, 
				type = type,
				varTotalInclude = varTotalInclude,
				msgVars = groupVar
			)
		}else{
			summaryTableVarList <- lapply(var, function(varI){
				varITotalInclude <- 
					(is.logical(varTotalInclude) && varTotalInclude) || 
					varI %in% varTotalInclude
				varIInclude0 <- 
					(is.logical(varInclude0) && varInclude0) || 
					varI %in% varInclude0
				sumTable <- computeSummaryStatisticsCustom(
					data = x, 
					var = varI, 
					type = type,
					varTotalInclude = varITotalInclude,
					filterEmptyVar = !varIInclude0,
					msgVars = groupVar
				)
				# only store the variable if more than one specified variable
				if(!is.null(sumTable) && nrow(sumTable) > 0 && varLabInclude){
					cbind.data.frame(variableInit = varI, sumTable, stringsAsFactors = FALSE)
				}else sumTable
				
			})
	
			summaryTable <- do.call(rbind.fill, summaryTableVarList)
			# if multiple variable(s), sort 'variable' in order specified in input
			if(!is.null(summaryTable) && varLabInclude){
				summaryTable$variable <- factor(
					varLab[summaryTable$variableInit],
					levels = varLab[var]
				)
			}
			summaryTable
		}
	}, .drop = FALSE)

	if(is.null(groupVar))
		summaryTable[, ".id"] <- NULL

	# include original rowVar/colVar
	if(!is.null(rowVar) & ((!rowInclude0) | (!is.null(rowVarDataLevels)))){
		summaryTable[, rowVar] <- rowDataLevels[match(summaryTable$rowVariables, rowDataLevels$factorLevel), rowVar]
		summaryTable$rowVariables <- NULL
	}
	if(!is.null(colVar) & ((!colInclude0) | (!is.null(colVarDataLevels)))){
		summaryTable[, colVar] <- colDataLevels[match(summaryTable$colVariables, colDataLevels$factorLevel), colVar]
		summaryTable$colVariables <- NULL
	}
	
	# include the total as the last level
	varTotalIsIncluded <- 
		(is.logical(varTotalInclude) && varTotalInclude) ||
		any(var %in% varTotalInclude)
	if("variableGroup" %in% colnames(summaryTable) && varTotalIsIncluded)
		summaryTable[, "variableGroup"] <- factor(summaryTable[, "variableGroup"], 
			levels = c("Total", setdiff(levels(summaryTable[, "variableGroup"]), "Total"))
		)
		
	return(summaryTable)
	
}


#' Convert a variable to a factor with levels in specified order.
#' @param var String with variable of \code{data} to sort.
#' @param otherVars Character vector with other variable(s) of \code{data}
#' considered in the specific dimension.
#' @param method Ordering method to use, either:
#' \itemize{
#' \item{String among:
#' \itemize{
#' \item{'auto': if \code{var} is a factor, keep its order, otherwise alphabetically}
#' \item{'alphabetical': \code{var} is order in alphabetical order}
#' \item{'total': \code{var} is ordered based on the \code{totalVar} variable, in decreasing order.
#' The total count is extracted from the rows with all \code{otherVars} equal to 'Total'.
#' If no \code{otherVars} is specified, consider all rows.}
#' }
#' }
#' \item{Function to be applied on each subset to get the order elements of the variable}
#' }
#' @param totalVar String with variable of \code{data} considered in case \code{type} is 'total',
#' 'statN' by default.
#' @param totalFilterFct (optional) Function which returns a subset of the data of interest,
#' to filter the total data considered for the ordering.
#' @param catLast String with last category (if any).
#' Set to NULL if no specific category should be included as last element.
#' @inheritParams inTextSummaryTable-common-args
#' @return Factor \code{var} variable of \code{data} with specified order.
#' @importFrom plyr daply
#' @author Laure Cougnaud
#' @keywords internal
convertVarToFactorWithOrder <- function(
	data, var, otherVars = NULL, 
	colVar = NULL, colTotalLab = "Total",
	method = c("auto", "alphabetical", "total"),
	totalFilterFct = NULL,
	totalVar = "statN",
	catLast = NULL){

	if(!is.function(method)){
		
		method <- match.arg(method)
	
		res <- switch(method,
			'auto' = if(is.factor(data[, var])){
				varLevels <- levels(data[, var])
				varLevels <- c(
					setdiff(varLevels, catLast),
					intersect(catLast, varLevels)
				)
				res <- factor(data[, var], levels = varLevels)
			}else{
				convertVarToFactorWithOrder(
					data = data, var = var, catLast = catLast,
					method = "alphabetical"
				)
			},
			'alphabetical' = {
				varLevels <- c(
					intersect("Total", data[, var]),
					sort(setdiff(unique(as.character(data[, var])), c(catLast, "Total")), decreasing = FALSE),
					intersect(catLast, data[, var])
				)
				factor(data[, var], levels = varLevels)
			},
			'total' = {
					
					# remove total for this variable
					dataForTotal <- data[which(data[, var] != "Total"), ]
					
					# filter records if any 'filterFct' is specified
					if(!is.null(totalFilterFct)){
						dataForTotal <- totalFilterFct(dataForTotal)
					}else{
						if(!is.null(colVar)){
							idxTotal <- which(rowSums(dataForTotal[, colVar, drop = FALSE] == colTotalLab) == length(colVar))
							if(length(idxTotal) != 0){
								dataForTotal <- dataForTotal[idxTotal, ]
							}else	warning("Total across columns not available to order the rows in the summary table.")
						}
					}
					
					if(!is.null(otherVars) && length(otherVars) > 0){
						# consider rows with subtotal for this variable (if any)
						idxRowTotal <- which(rowSums(dataForTotal[, otherVars, drop = FALSE] == "Total") == length(otherVars))
						if(length(idxRowTotal) > 0)
							dataForTotal <- dataForTotal[idxRowTotal, ]
					}
					
					totalPerVar <- daply(dataForTotal, var, function(x) sum(x[, totalVar], na.rm = TRUE))
					varLevels <- c(
						intersect("Total",  data[, var]), 
						setdiff(names(sort(totalPerVar, decreasing = TRUE)), c("Total", catLast)),
						intersect(catLast, data[, var])
					)
					varLevels <- c(varLevels, setdiff(as.character(unique(data[, var])), varLevels))
					factor(data[, var], levels = varLevels)
					
			}
		)
		
	}else{
		
		varLevels <- as.character(method(data))
		varLevels <- c(varLevels, setdiff(as.character(unique(data[, var])), varLevels))
		if("Total" %in% varLevels)	varLevels <- c("Total", setdiff(varLevels, "Total"))
		varLevels <- c(
			setdiff(varLevels, catLast),
			intersect(catLast, varLevels)
		)
		res <- factor(data[, var], levels = varLevels)
		
	}
	
	return(res)
		
}

#' Compute custom statistics specified by the user.
#' @param summaryTable Summary table.
#' @param statsVarInit Character vector with initial statistics names.
#' @param stats (Optionally) named list of expression or call object of summary statistics of interest.
#' The names are reported in the header.
#' The following variables are recognized, if the table is a: 
#' \itemize{
#' \item{'summaryTable': 
#' \itemize{
#' \item{'statN': number of subjects}
#' \item{'statMean': mean of \code{var}}
#' \item{'statSD': standard deviation of \code{var}}
#' \item{'statSE': standard error of \code{var}}
#' \item{'statMedian': median of \code{var}}
#' \item{'statMin': minimum of \code{var}}
#' \item{'statMax': maximum of \code{var}}
#' \item{'statPerc': percentage of subjects}
#' \item{'statm': number of records}
#' }
#' }
#' \item{'countTable': 
#' \itemize{
#' \item{'statN': number of subjects}
#' \item{'statPercN': percentage of subjects}
#' \item{'statm': number of records}
#' }
#' }
#' }
#' If \code{stats} if of length 1, the name of the summary statistic is not included
#' in the table.
#' The statistics can be specified for each \code{var} (if multiple), 
#' by naming each element of the list:
#' list(varName1 = list(...), varName2 = list()) and/or for each element in:
#' \code{statsVarBy}, by naming each sublist.
#' @param statsVarBy Character vector with colnames of
#' \code{data} by which the statistics should be computed.
#' In this case, \code{stats} (nested list or not) should be additionally nested
#' to specify the statistics for each element in \code{statsVarBy}.
#' @inheritParams inTextSummaryTable-common-args
#' @return List with two elements:
#' \itemize{
#' \item{'summaryTable': summary table updated with statistics specified in \code{stats}}
#' \item{'statsVar': Character vector with statistics names}
#' }
#' @author Laure Cougnaud
#' @keywords internal
getStatisticsSummaryStatisticsTable <- function(
	summaryTable, statsVarInit, 
	var = NULL, stats = NULL, statsVarBy = NULL){
	
	# compute specified metrics
	if(!is.null(stats)){
		
		if(length(stats) > 1 & is.null(names(stats)))
			stop("'statsFct' should be named.")
		
		# add specified custom statistics in summaryTable
		addStats <- function(sumTable, stats){
			
			# check if stats has the same name than default name
			statsName <- names(stats)
			if(!is.null(statsName)){
				statsNameConflict <- intersect(statsName, statsVarInit)
				if(length(statsNameConflict) > 0){
					for(stat in statsNameConflict){
						if(as.character(stats[[stat]]) == stat){
							stats <- stats[names(stats) != stat]
							if(length(stats) == 0){
								return(sumTable)
							}
						}else{
							stop("The statistic name: '", stat, "'",
								" is a default name used, please choose a different name.")
						}
					}
				}
			}
			
			runStats <- function(expr){
				if(is.language(expr)){
					eval(expr = expr, envir = sumTable)
				}else{
					stop(paste("Statistics specified in 'stats' should be an expression or name object,",
						"or a list named with variable, statistic name or statsVarBy elements."))
				}
			}
			statsDf <- if(is.list(stats)){
				sapply(stats, runStats, simplify = FALSE)
			}else	list(runStats(expr = stats))
			if(is.null(statsName))	names(statsDf) <- "Statistic"
			
			# save in summaryTable
			sumTable <- cbind(sumTable, statsDf, stringsAsFactors = FALSE)
			
			return(sumTable)
			
		}
		
		getStatColName <- function(stats)
			if(is.null(names(stats)))	"Statistic"	else	names(stats)

		# order stats across all nested list
		# to avoid wrong ordering in case some stats are ordered differently by variable	
		getOrderedStats <- function(statsByVar){
			statsByVarUnique <- unique(unlist(statsByVar))
			statsByVarPos <- lapply(statsByVar, match, x = statsByVarUnique)
			statsByVarPosMax <- do.call(pmax, c(statsByVarPos, list(na.rm = TRUE)))
			# order in case multiple elements have same position
			statsByVarOrdered <- unname(statsByVarUnique[order(statsByVarPosMax)]) 
			return(statsByVarOrdered)
		}
		
		# variable to compute the statistics by:
		if(any(names(stats) %in% var)){
			
			statsVarByUsed <- c(statsVarBy, "variableInit")
			
			if(!"variableInit" %in% colnames(summaryTable))
				stop("'stats' should not be named by variable if only one variable is specified.")
			
			# in case more stats are specified than specified var
			varsUsed <- setdiff(unique(as.character(summaryTable$variableInit)), NA)
			
			if(!all(varsUsed %in% names(stats))){
				stop(paste("If 'stats' is specified for each variable,",
					"it should be specified for all variables specified in 'var'."))
			}else stats <- stats[varsUsed]
			statsByVar <- lapply(stats, function(x)
				if(!is.null(statsVarBy)){
					getOrderedStats(lapply(x, getStatColName))
				}else	getStatColName(x)
			)
			statsVar <- getOrderedStats(statsByVar)
		}else{
			statsVarByUsed <- statsVarBy
			statsVar <- if(!is.null(statsVarBy)){
				statsByVar <- lapply(stats, getStatColName)
				getOrderedStats(statsByVar)
			}else	getStatColName(stats)
		}
#		statsVar <- unname(unique(unlist(lapply(unlist(stats, recursive = FALSE), getStatColName))))
		
		summaryTable <- ddply(summaryTable, statsVarByUsed, function(x){
			statsX <- if("variableInit" %in% statsVarByUsed){
				stats[[unique(x$variableInit)]]
			}else stats
			if(!is.null(statsVarBy))
				statsX <- statsX[[as.character(unique(x[, statsVarBy]))]]
			if(is.null(statsX)){
				if(!"isTotal" %in% colnames(x) && all(x$isTotal))
					warning("'stats' missing for:", toString(unique(x[, statsVarByUsed])))
				x
			}else	addStats(sumTable = x, stats = statsX)
		})

		if(".id" %in% colnames(summaryTable))
			summaryTable <- summaryTable[, -which(colnames(summaryTable) == ".id")]
		
	}else	statsVar <- statsVarInit
	
	res <- list(
		summaryTable = summaryTable,
		statsVar = statsVar	
	)

}

context("Compute summary statistics table with a variable to summarize")

test_that("A summary table is correctly computed for a continuous variable", {
      
	dataCont <- data.frame(x = c(NA, 1, 3, 6, 10), USUBJID = seq.int(5))
      
	sumTableCont <- computeSummaryStatisticsTable(data = dataCont, var = "x")
	expect_s3_class(sumTableCont, "summaryTable")
	varsCont <- c(
		"statN", "statm", "statMean", "statSD", "statSE", "statMedian",
		"statMin", "statMax", "statPercTotalN", "statPercN"
	)
	expect_named(sumTableCont, c("isTotal", varsCont), ignore.order = TRUE)
	expect_equal(object = sumTableCont$isTotal, expected = c(FALSE, TRUE))
      
	# stats computed via 'computeSummaryStatistics':
	sumTableContVar <- subset(sumTableCont, !isTotal)
	statsVar <- computeSummaryStatistics(dataCont, var = "x")
	expect_equal(
		object = sumTableContVar[, colnames(statsVar)], 
		expected = statsVar, 
		check.attributes = FALSE
	)
      
	# extra total statistics are added
	expect_equal(object = sumTableContVar$statPercTotalN, expected = 5)
	expect_equal(object = sumTableContVar$statPercN, expected = 4/5*100)
	  
})

test_that("A warning is generated if the variable to summarize is not available", {
			
	dataCont <- data.frame(x = c(NA, 1, 3, 6, 10), USUBJID = seq.int(5))
	expect_warning(
		computeSummaryStatisticsTable(data = dataCont, var = "y"),
		"y.* ignored"
	)
      
})

test_that("A summary table is correctly computed for a categorical variable", {
      
	dataCat <- data.frame(
		x = c(NA_character_, "B", "B", "B", "A"), 
		USUBJID = seq.int(5), 
		stringsAsFactors = TRUE
	)
      
	sumTableCat <- computeSummaryStatisticsTable(data = dataCat, var = "x")
	expect_s3_class(sumTableCat, "summaryTable")
	varsCat <- c("statN", "statm", "statPercTotalN", "statPercN")
	expect_named(sumTableCat, c("variableGroup", "isTotal", varsCat), ignore.order = TRUE)
	expect_equal(sumTableCat$isTotal, c(FALSE, FALSE, TRUE))
      
	# stats computed via 'computeSummaryStatistics':
	sumTableCatVar <- subset(sumTableCat, !isTotal)
	statsVar <- computeSummaryStatistics(dataCat, var = "x")
	expect_equal(
		object = sumTableCatVar[, colnames(statsVar)], 
		expected = statsVar, 
		check.attributes = FALSE
	)
      
	# extra total statistics are added
	expect_equal(
		object = sumTableCatVar$statPercTotalN, 
		expected = c(5, 5)
	)
	expect_equal(
		object = sumTableCatVar$statPercN, 
		expected = with(sumTableCatVar, statN/statPercTotalN*100)
	)
      
})

test_that("A summary table is correctly computed for a flag variable", {
      
	data <- data.frame(
		USUBJID = seq.int(7),
		x = rep(c("A", "B"), times = c(3, 4)),
		xFlag = rep(c("", "Y"), length.out = 7),
		stringsAsFactors = FALSE
	)
      
	res <- computeSummaryStatisticsTable(
		data = data, 
		var = c("x", "xFlag"), 
		varFlag = "xFlag"
	)
	expect_s3_class(res, "summaryTable")
	expect_true("variable" %in% colnames(res))
	expect_true("variableGroup" %in% colnames(res))
	  
	resAll <- computeSummaryStatisticsTable(
		data = data, 
		var = c("x", "xFlag")
	)
	  
	# variable not specified in varFlag is retained
	expect_equal(
		object = subset(res, variable == "x"),
		expected = subset(resAll, variable == "x"),
		check.attributes = FALSE
	)
	  
	# only flagged records are retained for varFlag
	expect_equal(
		object = subset(res, variable == "xFlag", -variableGroup),
		expected = subset(resAll, variable == "xFlag" & variableGroup == "Y", -variableGroup),
		check.attributes = FALSE
	)	
      
})

test_that("A warning is generated if the flag variable is not available in the variables to summarize", {
			
	data <- data.frame(
		USUBJID = seq.int(7),
		x = rep(c("A", "B"), times = c(3, 4)),
		xFlag = rep(c("", "Y"), length.out = 7),
		stringsAsFactors = FALSE
	)
			
	expect_warning(
		computeSummaryStatisticsTable(data = data, var = "x", varFlag = "xFlag"),
		"xFlag.* in varFlag.*ignored"
	)

})

test_that("Zero counts for a categorical variable are correctly included when requested", {
			
	data <- data.frame(
		USUBJID = seq.int(6), 
		x = factor(c("A", "B", "A"), levels = c("A", "B", "C"))
	)
	
	sumTable0 <- computeSummaryStatisticsTable(data = data, var = "x", varInclude0 = TRUE)
	expect_identical(as.character(na.omit(sumTable0$variableGroup)), c("A", "B", "C"))
	sumTable0Rows <- subset(sumTable0, variableGroup == "C")
	expect_equal(object = sumTable0Rows$statN, expected = 0)
	expect_equal(object = sumTable0Rows$statm, expected = 0)
	
})

test_that("Zero counts for a categorical variable are correctly not included when requested", {
			
	data <- data.frame(
		USUBJID = seq.int(6), 
		x = factor(c("A", "B", "A"), levels = c("A", "B", "C"))
	)
			
	sumTable <- computeSummaryStatisticsTable(data = data, var = "x", varInclude0 = FALSE)
	expect_identical(
		object = as.character(na.omit(sumTable$variableGroup)), 
		expected = c("A", "B")
	)
			
})

test_that("Zero counts for a categorical variable are not included by default", {
			
	data <- data.frame(
		USUBJID = seq.int(6), 
		x = factor(c("A", "B", "A"), levels = c("A", "B", "C"))
	)
	expect_identical(
		object = computeSummaryStatisticsTable(data = data, var = "x"),
		expected = computeSummaryStatisticsTable(data = data, var = "x", varInclude0 = FALSE)
	)
	
})

test_that("Zero counts for a categorical flag variable are correctly included when requested", {
	
	data <- data.frame(
		USUBJID = seq.int(6), 
		xFlag = factor("N", c("N", "Y"))
	)		

	sumTableVarFlag0 <- computeSummaryStatisticsTable(
		data = data,
		var = "xFlag", varFlag = "xFlag",
		varInclude0 = TRUE
	)
	sumTableVarFlag0Rows <- subset(sumTableVarFlag0, !isTotal)
	expect_equal(object = sumTableVarFlag0Rows$statN, expected = 0)
	expect_equal(object = sumTableVarFlag0Rows$statm, expected = 0)
			
})

test_that("Zero counts for a categorical flag variable are correctly included when requested", {
			
	data <- data.frame(
		USUBJID = seq.int(6), 
		xFlag = factor("N", c("N", "Y"))
	)		
			
	sumTable <- computeSummaryStatisticsTable(
		data = data,
		var = "xFlag", varFlag = "xFlag",
		varInclude0 = FALSE
	)
	expect_equal(nrow(subset(sumTable, !isTotal)), 0)
			
})

test_that("Zero counts for a categorical flag variable are not included by default", {
			
	data <- data.frame(
		USUBJID = seq.int(6), 
		xFlag = factor("N", c("N", "Y"))
	)
	
	expect_identical(
		object = computeSummaryStatisticsTable(data = data, var = "xFlag", varFlag = "xFlag", varInclude0 = FALSE),
		expected = computeSummaryStatisticsTable(data = data, var = "xFlag", varFlag = "xFlag")
	)
	
})

test_that("Zero counts for a categorical variable are correctly included for a subset of the variables to summarize", {
			
	data <- data.frame(
		USUBJID = seq.int(6), 
		x = factor(c("A", "B", "A"), levels = c("A", "B", "C")),
		xFlag = factor("N", c("N", "Y"))
	)
	sumTableVarFlag0 <- computeSummaryStatisticsTable(
		data = data,
		var = c("x", "xFlag"), varFlag = "xFlag",
		varInclude0 = "x"
	)
	expect_identical(
		object = as.character(na.omit(sumTableVarFlag0$variableGroup)), 
		expected = c("A", "B", "C")
	)
			
})	

test_that("A warning is generated if the variable for which zero counts are included is not specified in the variables to summarize", {
			
	data <- data.frame(
		USUBJID = seq.int(6), 
		x = factor(c("A", "B", "A"), levels = c("A", "B", "C"))
	)
	expect_warning(
		computeSummaryStatisticsTable(data = data, var = "x", varInclude0 = "y"),
		".*y.* in varInclude0 are ignored.*"
	)
	
})

test_that("Elements in a categorical variable to summarize are correctly ignored when requested", {
      
      data <- data.frame(
          USUBJID = seq.int(6),
          x = rep(c("A", "B"), times = 3),
          stringsAsFactors = FALSE
      )
      res <- computeSummaryStatisticsTable(
          data = data, var = "x", varIgnore = "A"
      )
      expect_s3_class(res, "summaryTable")
      expect_identical(levels(res$variableGroup), "B")
      
})

test_that("The total for the categorical variables to summarize is correctly computed", {
			
	data <- data.frame(
		x = c("A", "B", "A", "B"), 
		USUBJID = c(1, 2, 3, 1)
	)
			
	sumTableVarTotal <- computeSummaryStatisticsTable(
		data = data, var = "x", varTotalInclude = TRUE
	)
	
	expect_equal(
		as.character(na.omit(sumTableVarTotal$variableGroup)),
		c("A", "B", "Total")
	)
	sumTableVarTotalRows <- subset(sumTableVarTotal, variableGroup == "Total")
	expect_equal(sumTableVarTotalRows$statN, 3)
	expect_equal(sumTableVarTotalRows$statm, 4)
	expect_equal(sumTableVarTotalRows$statPercTotalN, 3)
	expect_equal(sumTableVarTotalRows$statPercN, 100)
	
	expect_identical(
		object = computeSummaryStatisticsTable(data = data, var = "x", varTotalInclude = "x"),
		expected = computeSummaryStatisticsTable(data = data, var = "x", varTotalInclude = TRUE)
	)
			
})

test_that("The total for the categorical variables to summarize is not included by default", {
			
	data <- data.frame(
		x = c("A", "B", "A", "B"), 
		USUBJID = c(1, 2, 3, 1)
	)

	expect_identical(
		object = computeSummaryStatisticsTable(data = data, var = "x", varTotalInclude = FALSE),
		expected = computeSummaryStatisticsTable(data = data, var = "x")
	)
	
})

test_that("The total for a subset of the categorical variables to summarize is correctly computed", {
			
	data <- data.frame(
		x = c("A", "B", "A", "B"), 
		y = c("a", "d", "g", "t"), 
		USUBJID = c(1, 2, 3, 1)
	)
	sumTableVarTotalChar <- computeSummaryStatisticsTable(
		data = data, 
		var = c("x", "y"), 
		varTotalInclude = "y"
	)
	sumTableVarTotalCharRows <- subset(sumTableVarTotalChar, variableGroup == "Total")
	expect_identical(as.character(sumTableVarTotalCharRows$variable), "y")

})

test_that("A warning is generated if the variable to compute totals on is not available", {
			
	data <- data.frame(
		x = c("A", "B", "A", "B"), 
		USUBJID = c(1, 2, 3, 1)
	)
	
	expect_warning(
		computeSummaryStatisticsTable(data = data, var = "x", varTotalInclude = "blabla"),
		".* in varTotalInclude.*ignored"
	)
	
})

test_that("The total for the variables to summarize is correctly defined in a separated row", {
			
	# the total is included in a separated row
	# during the export step
	# -> check the attribute of the output
	# to make sure that user specification is correctly stored
	data <- data.frame(
		x = c("A", "B", "A", "B"), 
		USUBJID = c(1, 2, 3, 1)
	)
	
	sumTable <- computeSummaryStatisticsTable(data = data, varTotalInSepRow = TRUE)
	expect_true("variableGroup" %in% attr(sumTable, "summaryTable")$rowVarTotalInSepRow)

})

test_that("The total for the variables to summarize is not defined in a separated row by default", {
			
	# the total is included in a separated row
	# during the export step
	# -> check the attribute of the output
	# to make sure that user specification is correctly stored
	data <- data.frame(
		x = c("A", "B", "A", "B"), 
		USUBJID = c(1, 2, 3, 1)
	)
		
	sumTable <- computeSummaryStatisticsTable(data = data, varTotalInSepRow = FALSE	)
	expect_false("variableGroup" %in% attr(sumTable, "summaryTable")$rowVarTotalInSepRow)	
			
})

test_that("A warning is generated if the deprecated parameter 'varIncludeTotal' is used", {
			
	dataCont <- data.frame(
		x = c(NA, 1, 3, 6, 10), 
		USUBJID = seq.int(5)
	)
	expect_warning(
		computeSummaryStatisticsTable(dataCont, varIncludeTotal = TRUE),
		"Argument: 'varIncludeTotal' is deprecated, please use 'varTotalInclude' instead."
	)
	
})

test_that("Elements of a categorical variable to summarize are ordered alphabetically if the variable is a character", {
      
	data <- data.frame(
		USUBJID = seq.int(2),
		SEX = c("F", "M"),
		stringsAsFactors = FALSE
	)
      
	# If variable is a character, levels should be sorted in alphabetical order
	tableCharac <- computeSummaryStatisticsTable(data = data, var = "SEX")
	expect_identical(
		object = levels(tableCharac$variableGroup),
		expected = sort(unique(data$SEX))
	)
	  
})

test_that("A categorical variable to summarize is converted to a factor if the variable is a character", {
			
	# edge-case: alphabetical order is used
	# all variables should already be converted as factor as input of the fct
	data <- data.frame(group = c("B", "Z", "A", "G"), stringsAsFactors = FALSE)
	varConverted <- inTextSummaryTable:::convertVarToFactorWithOrder(
		data = data, var = "group",
		method = "auto"
	)	
	expect_s3_class(varConverted, "factor")
	expect_equal(levels(varConverted), c("A", "B", "G", "Z"))
			
})

test_that("Elements of a categorical variable to summarize are ordered as specified if the variable is a factor", {
			
	data <- data.frame(
		USUBJID = seq.int(2),
		SEX = factor(c("F", "M"), levels = c("M", "F")),
		stringsAsFactors = FALSE
	)
      
	tableFactor <- computeSummaryStatisticsTable(data = data, var = "SEX")
	expect_identical(levels(tableFactor$variableGroup), levels(data$SEX))
	  
})
      
test_that("Elements of a categorical variable to summarize are ordered correctly even if they are not available for all columns", {
			
	# only last element of SEX for first TRT (will be computed first)
	# the first element of SEX for the other TRT
	# to check that order of SEX is computed upfront grouping by TRT
	data <- data.frame(
		USUBJID = seq.int(4),
		TRT = factor(c("A", "A", "B", "B"), levels = c("A", "B")),
		SEX = factor(c("F", "F", "M", "M"), levels = c("M", "F")),
		stringsAsFactors = FALSE
	)
	# even if not present for all column variables
	tableFactorNotComplete <- computeSummaryStatisticsTable(
		data = data, 
		var = "SEX", 
		colVar = "TRT"
	)
	expect_identical(
		levels(tableFactorNotComplete$variableGroup), 
		levels(data$SEX)
	)
      
	# even if the variable is character
	data$SEX <- as.character(data$SEX)
	tableCharacNotComplete <- computeSummaryStatisticsTable(
		data = data, 
		var = "SEX", 
		colVar = "TRT"
	)
	expect_identical(
		object = levels(tableCharacNotComplete$variableGroup), 
		expected = sort(unique(data$SEX))
	)
  
})

test_that("No variable name is included in the table when no variable is specified", {
      
	data <- data.frame(USUBJID = seq.int(2))
	descTableNoVar <- computeSummaryStatisticsTable(data = data)
	expect_false("variable" %in% colnames(descTableNoVar))	
	  
})

test_that("A warning is generated if the name of the variable is requested but no variable is specified", {
			
	data <- data.frame(USUBJID = seq.int(2))
	expect_warning(
		descTableNoVar <- computeSummaryStatisticsTable(
			data = data, 
			varLabInclude = TRUE
		),
		regexp = "Variable label is not included"
	)
	  
})

test_that("No variable name is included in the table when only one variable is specified", {
				  
	data <- data.frame(
		USUBJID = seq.int(2),
		SEX = c("F", "M")
	)
      
	descTableOneVar <- computeSummaryStatisticsTable(data = data, var = "SEX")
	expect_false("variable" %in% colnames(descTableOneVar))
	
})

test_that("The variable name is included in the table when requested for one variable", {
			
	data <- data.frame(
		USUBJID = seq.int(2),
		SEX = c("F", "M")
	)
	descTableOneVarWithLabel <- computeSummaryStatisticsTable(
		data = data, var = "SEX", varLabInclude = TRUE
	)
	expect_true("variable" %in% colnames(descTableOneVarWithLabel))	
	
})


test_that("A warning is generated if the variable should be included in columns but the variable name should not be included", {
			
	data <- data.frame(
		USUBJID = seq.int(2),
		SEX = c("F", "M")
	)
	expect_warning(
		computeSummaryStatisticsTable(
			data = data, 
			var = "SEX", colVar = "variable",
			varLabInclude = FALSE
	  	),
		"var' not included in columns because 'varLabInclude' is FALSE"
	)
	
})

test_that("The variable name is included if more than one variable is specified", {
			
	set.seed(123)
	data <- data.frame(
		USUBJID = seq.int(2),
		SEX = c("F", "M"),
		AGE = rnorm(2)
	)
      
	descTableMoreOneVar <- computeSummaryStatisticsTable(
		data = data, var = c("AGE", "SEX")
	)
	expect_true("variable" %in% colnames(descTableMoreOneVar))
	
})

test_that("A warning is generated if more than one variable is specified but the variable name should not be included", {
		
	set.seed(123)
	data <- data.frame(
		USUBJID = seq.int(2),
		SEX = c("F", "M"),
		AGE = rnorm(2)
	)
	expect_warning({
		descTableMoreOneVarWithLabel <- computeSummaryStatisticsTable(
			data = data, var = c("AGE", "SEX"), varLabInclude = FALSE
		)
		},
		regexp = "Variable label is included"
	)
	
	expect_true("variable" %in% colnames(descTableMoreOneVarWithLabel))
	
})

test_that("Variables to summarize are labelled with the variable name by default", {
									
	data <- data.frame(
		USUBJID = seq.int(6),
		SEX = rep(c("M", "F"), times = 3),
		AGE = seq(20, 62, length.out = 6),
		stringsAsFactors = FALSE
	)
	var <- c("SEX", "AGE")	
	sumTable <- computeSummaryStatisticsTable(data, var = var)
	
	for(varI in var){
		sumTableVarI <- computeSummaryStatisticsTable(data = data, var = varI)
		expect_equal(
			object = subset(sumTable, variable == !!varI, select = colnames(sumTableVarI)),
			expected = subset(sumTableVarI, !isTotal),
			check.attributes = FALSE
		)
	}
	
})

test_that("The labels of the variables to summarize are correctly extracted from a specified labels", {
		
	data <- data.frame(
		USUBJID = seq.int(6),
		SEX = rep(c("M", "F"), times = 3),
		AGE = seq(20, 62, length.out = 6),
		stringsAsFactors = FALSE
	)
	var <- c("SEX", "AGE")
	varLab <- c(SEX = "Gender", AGE = "Age in years")
	sumTable <- computeSummaryStatisticsTable(data, var = var, varLab = varLab)
	
	for(varI in var){
		sumTableVarI <- computeSummaryStatisticsTable(data = data, var = varI)
		expect_equal(
			object = subset(sumTable, variable == varLab[!!varI], select = colnames(sumTableVarI)),
			expected = subset(sumTableVarI, !isTotal),
			check.attributes = FALSE
		)
	}
	
})

test_that("The labels of the variables to summarize are correctly extracted from the labels of all variables", {
			
	data <- data.frame(
		USUBJID = seq.int(6),
		SEX = rep(c("M", "F"), times = 3),
		AGE = seq(20, 62, length.out = 6),
		stringsAsFactors = FALSE
	)
	var <- c("SEX", "AGE")
	labelVars <- c(SEX = "Gender", AGE = "Age in years")
	sumTable <- computeSummaryStatisticsTable(data, var = var, labelVars = labelVars)
	
	for(varI in var){
		sumTableVarI <- computeSummaryStatisticsTable(data = data, var = varI)
		expect_equal(
			object = subset(sumTable, variable == labelVars[!!varI], select = colnames(sumTableVarI)),
			expected = subset(sumTableVarI, !isTotal),
			check.attributes = FALSE
		)
	}
	
})
	

test_that("The labels of the variables to summarize are correctly extracted from the labels of a subset of the variables", {
			
	data <- data.frame(
		USUBJID = seq.int(6),
		SEX = rep(c("M", "F"), times = 3),
		AGE = seq(20, 62, length.out = 6),
		stringsAsFactors = FALSE
	)
	var <- c("SEX", "AGE")

	sumTableVarLabNotFull <- computeSummaryStatisticsTable(
		data = data, var = var, varLab = c(SEX = "Gender")
	)
	sumTable <- computeSummaryStatisticsTable(data, var = var)
	
	expect_identical(
		object = subset(sumTableVarLabNotFull, variable == "Gender", select = -variable),
		expected = subset(sumTable, variable == "SEX", select = -variable)
	)
	expect_identical(
		object = subset(sumTableVarLabNotFull, variable == "AGE", select = -variable),
		expected = subset(sumTable, variable == "AGE", select = -variable)
	)
	
})

test_that("The general label for the variables is correctly included by default", {
		
	# general label for var set during export step
	# currently only stored in the output			
			
	data <- data.frame(
		USUBJID = seq.int(6),
		SEX = rep(c("M", "F"), times = 3),
		AGE = seq(20, 62, length.out = 6),
		stringsAsFactors = FALSE
	)
	
	sumTable <- computeSummaryStatisticsTable(data, var = c("SEX", "AGE"))
	expect_equal(
		object = unname(attr(sumTable, "summaryTable")$rowVarLab["variable"]),
		expected = "Variable"
	)

})

test_that("The general label for the variables is correctly set when specified", {
			
	data <- data.frame(
		USUBJID = seq.int(6),
		SEX = rep(c("M", "F"), times = 3),
		AGE = seq(20, 62, length.out = 6),
		stringsAsFactors = FALSE
	)
	
	sumTable <- computeSummaryStatisticsTable(
		data = data, var = c("SEX", "AGE"), 
		varGeneralLab = "test"
	)
			
	expect_equal(
		object = unname(attr(sumTable, "summaryTable")$rowVarLab["variable"]),
		expected = "test"
	)
	
})

test_that("The general label is correctly set with a warning when specified as empty", {
			
	data <- data.frame(
		USUBJID = seq.int(6),
		SEX = rep(c("M", "F"), times = 3),
		AGE = seq(20, 62, length.out = 6),
		stringsAsFactors = FALSE
	)
	expect_warning(
		sumTableVarGenLabEmpty <- computeSummaryStatisticsTable(data, 
			var = c("SEX", "AGE"), varGeneralLab = NULL),
		".*varGeneralLab.* set to .* by default"
	)
	expect_equal(
		object = unname(attr(sumTableVarGenLabEmpty, "summaryTable")$rowVarLab["variable"]),
		expected = "Variable"
	)
			
})

test_that("The label for the variable subgroups is correctly included by default", {
			
	# label for var subgroup set during export step
	# currently only stored in the output			
	
	data <- data.frame(
		USUBJID = seq.int(6),
		SEX = rep(c("M", "F"), times = 3),
		stringsAsFactors = FALSE
	)
			
	sumTable <- computeSummaryStatisticsTable(data, var = "SEX")
	expect_equal(
		object = unname(attr(sumTable, "summaryTable")$rowVarLab["variableGroup"]),
		expected = "Variable group"
	)

})

test_that("The label for the variable subgroups is correctly set when specified", {
		
	data <- data.frame(
		USUBJID = seq.int(6),
		SEX = rep(c("M", "F"), times = 3),
		stringsAsFactors = FALSE
	)

	sumTable <- computeSummaryStatisticsTable(
		data = data, 
		var = "SEX", 
		varSubgroupLab = "test"
	)
	expect_equal(
		object = unname(attr(sumTable, "summaryTable")$rowVarLab["variableGroup"]),
		expected = "test"
	)
	
})

test_that("The label for the variable subgroups is correctly set with a warning when specified as empty", {
			
	data <- data.frame(
		USUBJID = seq.int(6),
		SEX = rep(c("M", "F"), times = 3),
		stringsAsFactors = FALSE
	)
			
	expect_warning(
		sumTableVarSubgroupLabEmpty <- computeSummaryStatisticsTable(
			data = data, 
			var = "SEX", 
			varSubgroupLab = NULL
		),		
		".*'varSubgroupLab.* set to .* by default"
	)
	expect_equal(
		object = unname(attr(sumTableVarSubgroupLabEmpty, "summaryTable")$rowVarLab["variableGroup"]),
		expected = "Variable group"
	)
			
})

context("Compute summary statistics table")

test_that("An error is generated if data is not a data frame", {
			
	# e.g. if we forget to create the 'data' object, 
	# 'data()' function is used instead:
	expect_error(
		computeSummaryStatisticsTable(data),
		"A data.frame should be specified in.*data.*."
	)
	
})

test_that("An message is generated if the data is empty", {
			
	data <- data.frame()
	expect_message(
		res <- computeSummaryStatisticsTable(data),
		"No data to report."
	)
	expect_null(res)
	
})

test_that("A count table for the entire data is created if no variable is specified", {
			
	dataCont <- data.frame(
		x = c(NA, 1, 3, 6, 10), 
		USUBJID = seq.int(5)
	)
	
	# no variable specified: a count table is created
	sumNoVar <- computeSummaryStatisticsTable(dataCont)
	expect_s3_class(sumNoVar, "summaryTable")
	expect_named(sumNoVar, c("isTotal", "statN", "statm", "statPercTotalN", "statPercN"), ignore.order = TRUE)
	expect_equal(
		unname(unlist(subset(sumNoVar, isTotal)[1, c("statN", "statm", "statPercTotalN", "statPercN")])),
		c(5, 5, 5, 100)
	)
	expect_equal(
		subset(sumNoVar, isTotal, select = -isTotal), 
		subset(sumNoVar, !isTotal, select = -isTotal), 
		check.attributes = FALSE
	)
	
})

test_that("An error is generated if the default subject ID variable is not available", {
			
	dataCont <- data.frame(
		x = c(NA, 1, 3, 6, 10),
		`subject identifier` = seq.int(5), 
		check.names = FALSE
	)
	
	expect_error(
		computeSummaryStatisticsTable(dataCont),
		"Subject variable.*not available for the computation of the number of subjects."
	)
	
})

test_that("Counts of subjects are correctly computed when a custom subject ID variable is specified", {
		
	dataCont <- data.frame(
		x = c(NA, 1, 3, 6, 10),
		`subject identifier` = seq.int(5), 
		check.names = FALSE
	)
			
	expect_silent(sumTable <- 
		computeSummaryStatisticsTable(
			data = dataCont, 
			subjectVar = "subject identifier"
		)			
	)
	expect_equal(sumTable$statN, c(5, 5))
	expect_equal(sumTable$statPercTotalN, c(5, 5))
	expect_equal(sumTable$statPercN, c(100, 100))
			
})

test_that("The summary table is correctly filtered", {
			
	data <- data.frame(
		USUBJID = seq.int(5),
		AEDECOD = rep(c("A", "B"), length.out = 5)
	)

	summaryTableFiltered <- computeSummaryStatisticsTable(
		data = data, var = "AEDECOD",
		filterFct = function(x) subset(x, variableGroup == "A")
	)
	expect_equal(
		summaryTableFiltered,
		subset(computeSummaryStatisticsTable(data = data, var = "AEDECOD"), variableGroup == "A"),
		check.attributes = FALSE
	)
	
})

test_that("An error is generated if the filtering function is not correct", {
			
	data <- data.frame(
		USUBJID = seq.int(5),
		AEDECOD = rep(c("A", "B"), length.out = 5)
	)

	expect_error(
		computeSummaryStatisticsTable(
			data = data, var = "AEDECOD",
			filterFct = function(x) subset(x, variableGroup2 == "A")
		)
	)
			
})

test_that("The summary table is correctly filtered when a flag variable is specified", {
	
	# internally, a filterFct is set when varFlag is specified
	# this test checks that if the user specifies additionally 
	# a filterFct, this one is also used.
			
	data <- data.frame(
		USUBJID = seq.int(5),
		AEDECOD = c("A", "A", "B", "B", "B"),
		ANFL = c("Y", "N", "Y", "N", "Y")
	)
	
	summaryTableAll <- computeSummaryStatisticsTable(
		data = data, 
		var = "ANFL", varFlag = "ANFL",
		rowVar = "AEDECOD",
		stats = "n"
	)
			
	summaryTableFiltered <- computeSummaryStatisticsTable(
		data = data, 
		var = "ANFL", varFlag = "ANFL",
		rowVar = "AEDECOD",
		stats = "n",
		filterFct = function(x) subset(x, AEDECOD == "A")
	)
	
	expect_equal(
		summaryTableFiltered,
		subset(summaryTableAll, AEDECOD == "A"),
		check.attributes = FALSE
	)
			
})

test_that("The summary table is correctly computed by a grouping variable", {
		
	data <- data.frame(
		USUBJID = seq.int(5),
		AEDECOD = factor(c("A", "A", "B", "B", "B"), levels = c("B", "A"))
	)	
	
	summaryTableByVar <- computeSummaryStatisticsTable(
		data = data,
		byVar = "AEDECOD"
	)
	
	expect_type(summaryTableByVar, "list")
	expect_length(summaryTableByVar, 2)
	expect_named(summaryTableByVar, paste("AEDECOD:", c("B", "A")))
	
	for(group in unique(data$AEDECOD)){
		expect_equal(
			object = summaryTableByVar[[paste("AEDECOD:", !!group)]],
			expected = computeSummaryStatisticsTable(data = subset(data, AEDECOD == !!group)),
			check.attributes = FALSE
		)
	}
			
})

test_that("The summary table is correctly computed by a grouping variable with specified label", {
			
	data <- data.frame(
		USUBJID = seq.int(5),
		AEDECOD = factor(c("A", "A", "B", "B", "B"), 
			levels = c("B", "A"))
	)	
	
	# correct spec
	summaryTableByVar <- computeSummaryStatisticsTable(
		data = data,
		byVar = "AEDECOD",
		byVarLab = c("AEDECOD" = "Adverse event")
	)
	expect_named(summaryTableByVar, paste("Adverse event:", c("B", "A")))
	
})
	
test_that("The summary table is correctly computed by a grouping variable with default variable name if label is missing", {
		
	data <- data.frame(
		USUBJID = seq.int(5),
		AEDECOD = factor(c("A", "A", "B", "B", "B"), 
			levels = c("B", "A"))
	)
	
	# in case byVarLab specified for other variables
	# var code is used:
	summaryTableByVar <- computeSummaryStatisticsTable(
		data = data,
		byVar = "AEDECOD",
		byVarLab = c("blabla" = "Adverse event")
	)
	expect_named(summaryTableByVar, paste("AEDECOD:", c("B", "A")))
			
})

context("Compute summary statistics table with row variable")

test_that("A summary table is correctly computed for a continuous variable by a row variable", {
			
	data <- data.frame(
		parent = c("A", "A", "A", "A", "B", "B"), 
		child = c("a", "a", "a", "b", "c", "c"),
		x = rnorm(n = 6),
		USUBJID = seq.int(6)
	)
	
	sumTableRowVar <- computeSummaryStatisticsTable(
		data,
		rowVar = c("parent", "child"),
		var = "x"
	)    
	expect_s3_class(sumTableRowVar, "summaryTable")
	stats <- c(
		"statN", "statm",
		"statMean",
		"statSD", "statSE",
		"statMedian", "statMin",
		"statMax",
		"statPercTotalN", "statPercN"
	)
	expect_identical(
		names(sumTableRowVar),
		c("parent", "child", "isTotal", stats)
	)
	lastRowIdx <- nrow(sumTableRowVar)
	expect_true(sumTableRowVar$isTotal[lastRowIdx])
	expect_identical(
		sumTableRowVar$statPercN[lastRowIdx],
		100
	)
	
	# check if summary statistics table with rowVar specification
	# corresponds to summary statistics computed for each unique rowVar
	rowVarUnique <- unique(data[, c("parent", "child")])
	statsToCompare <- setdiff(stats , c("statPercTotalN", "statPercN"))
	for(iRow in seq_len(nrow(rowVarUnique))){
		expect_equal(
			object = {
				dataIRow <- merge(rowVarUnique[!!iRow, ], data)
				sumTableIRow <- computeSummaryStatisticsTable(dataIRow, var = "x")
				subset(sumTableIRow, !isTotal)[, statsToCompare]
			}, 
			expected = merge(rowVarUnique[!!iRow, ], sumTableRowVar)[, statsToCompare],
			check.attributes = FALSE
		)
	}
	
})

test_that("A warning is generated if the row variable is not available", {
			
	data <- data.frame(
		parent = c("A", "A", "A", "A", "B", "B"), 
		child = c("a", "a", "a", "b", "c", "c"),
		USUBJID = seq.int(6)
	)
			
	expect_warning(
		computeSummaryStatisticsTable(data, rowVar = "Y"),
		"Y.* in rowVar.*ignored"
	)
	
})


test_that("Row variables are labelled with the variable name by default", {
			
	data <- data.frame(
		parent = c("A", "A", "A", "A", "B", "B"), 
		child = c("a", "a", "a", "b", "c", "c"),
		USUBJID = seq.int(6)
	)
	rowVar <- c("parent", "child")

	summaryTable <- computeSummaryStatisticsTable(data, rowVar = rowVar)
	expect_identical(
		object = attr(summaryTable, "summaryTable")$rowVarLab[rowVar],
		expected = setNames(rowVar, rowVar)
	)
	
})

test_that("The labels of the row variables are correctly extracted from a specified labels", {
			
	data <- data.frame(
		parent = c("A", "A", "A", "A", "B", "B"), 
		child = c("a", "a", "a", "b", "c", "c"),
		USUBJID = seq.int(6)
	)
	rowVar <- c("parent", "child")
	rowVarLab <- c(parent = "Parent variable", child = "Child variable")
			
	summaryTable <- computeSummaryStatisticsTable(
		data = data, 
		rowVar = rowVar, 
		rowVarLab = rowVarLab
	)
	expect_identical(
		object = attr(summaryTable, "summaryTable")$rowVarLab[rowVar], 
		expected = rowVarLab
	)
	
})

test_that("The labels of the row variables are correctly extracted from the labels of all variables", {
			
	data <- data.frame(
		parent = c("A", "A", "A", "A", "B", "B"), 
		child = c("a", "a", "a", "b", "c", "c"),
		USUBJID = seq.int(6)
	)
	rowVar <- c("parent", "child")

	labelVars <- c(parent = "Parent variable", child = "Child variable")
	summaryTable <- computeSummaryStatisticsTable(
		data = data, 
		rowVar = rowVar, 
		labelVars = labelVars
	)
	expect_identical(
		attr(summaryTable, "summaryTable")$rowVarLab[rowVar], 
		labelVars[rowVar]
	)
	
})

test_that("The labels of the row variables are correctly extracted from the labels of all variables", {
	
	data <- data.frame(
		parent = c("A", "A", "A", "A", "B", "B"), 
		child = c("a", "a", "a", "b", "c", "c"),
		USUBJID = seq.int(6)
	)
	rowVar <- c("parent", "child")
	summaryTable <- computeSummaryStatisticsTable(
		data = data, rowVar = rowVar, 
		rowVarLab = c(child = "Child variable")
	)
	
	# no errors if labels are not specified for all row variables
	expect_identical(
		object = attr(summaryTable, "summaryTable")$rowVarLab[rowVar], 
		expected = c(parent = "parent", child = "Child variable")
	)

})

test_that("Row elements are ordered alphabetically if the row variable is a character", {
			
	data <- data.frame(
		USUBJID = seq.int(6),
		SEX = rep(c("M", "F"), times = 3),
		stringsAsFactors = FALSE
	)
	
	expect_silent(
		resAuto <- computeSummaryStatisticsTable(
			data,
			rowVar = "SEX",
			rowOrder = "auto"
		)
	)
	expect_identical(
		levels(resAuto$SEX),
		c("F", "M")
	)
	
})

test_that("Row elements are ordered alphabetically when requested", {
			
	data <- data.frame(
		USUBJID = seq.int(6),
		SEX = factor(rep(c("M", "F"), times = 3), levels = c("M", "F"))
	)
	
	expect_silent(
		resAlphabet <- computeSummaryStatisticsTable(
			data,
			rowVar = "SEX",
			rowOrder = "alphabetical"
		)
	)
	expect_identical(
		levels(resAlphabet$SEX),
		c("F", "M")
	)
	
})
	
test_that("An error is generated if the order for the rows is not correctly specified", {
		
	data <- data.frame(
		USUBJID = seq.int(6),
		SEX = rep(c("M", "F"), times = 3),
		stringsAsFactors = FALSE
	)
			
	expect_error(
		resAuto <- computeSummaryStatisticsTable(
			data = data,
			rowVar = "SEX",
			rowOrder = "test"
		),
		'.*should be one of.*auto.*alphabetical.*total.*'
	)
	
})

test_that("Row elements are ordered as specified when the row variable is a factor", {
			
	data <- data.frame(
		USUBJID = seq.int(6),
		SEX = factor(rep(c("M", "F"), times = 3), levels = c("M", "F")),
		stringsAsFactors = FALSE
	)
	expect_silent(
		resAuto <- computeSummaryStatisticsTable(
			data,
			rowVar = "SEX",
			rowOrder = "auto"
		)
	)
	expect_identical(levels(resAuto$SEX), c("M", "F"))
	
})

test_that("Row elements are correctly ordered based on the total", {
			
	data <- data.frame(
		USUBJID = seq.int(6),
		TRT = c("B", "B", "B", "B", "A", "A"),
		stringsAsFactors = FALSE
	)
	
	# Total
	expect_silent(
		resTotal <- computeSummaryStatisticsTable(
			data,
			rowVar = "TRT",
			rowOrder = "total"
		)
	)
	expect_identical(
		levels(resTotal$TRT),
		c("B", "A")
	)
	
})

test_that("Row elements are correctly ordered based on the column totals", {
			
	data <- data.frame(
		USUBJID = seq.int(6),
		SEX = c("F", "F", "M", "M", "M", "M"),
		TRT = c("B", "B", "B", "B", "A", "A"),
		stringsAsFactors = FALSE
	)
			
	expect_silent(
		resTotal <- computeSummaryStatisticsTable(
			data,
			rowVar = "SEX", colVar = "TRT",
			rowOrder = "total", 
		)
	)
	expect_identical(
		levels(resTotal$SEX),
		c("M", "F")
	)
	# Total across columns included internally but not returned by the function
	expect_identical(
		levels(resTotal$TRT),
		c("A", "B")
	)
			
})

test_that("Row elements are correctly ordered when the order is specified for each row variable", {
			
	data <- data.frame(
		USUBJID = seq.int(6),
		SEX = c("F", "F", "M", "M", "M", "M"),
		TRT = c("B", "B", "B", "B", "A", "A"),
		stringsAsFactors = FALSE
	)
	
	# Different for each row variable
	expect_silent(
		resDoubleOrder <- computeSummaryStatisticsTable(
			data,
			rowVar = c("SEX", "TRT"),
			rowOrder = c(SEX = "auto", TRT = "total")
		)
	)      
	expect_identical(
		levels(resDoubleOrder$TRT),
		c("B", "A")
	)
	expect_identical(
		levels(resDoubleOrder$SEX),
		c("F", "M")
	)
	
})

test_that("Row elements are correctly ordered based on a specified function", {
			
	data <- data.frame(
		USUBJID = seq.int(6),
		TRT = c("B", "B", "B", "B", "A", "A"),
		stringsAsFactors = FALSE
	)
	resFct <- computeSummaryStatisticsTable(
		data,
		rowVar = "TRT",
		rowOrder = function(summaryTable){
			data <- subset(summaryTable, !isTotal)
			data[order(data$statN, decreasing = TRUE), "TRT"]
		}
	)
	expect_identical(levels(resFct$TRT), c("B", "A"))
	
})

test_that("Row elements are correctly ordered when not all elements are returned by a specified function", {
			
	data <- data.frame(
		USUBJID = seq.int(6),
		TRT = c("B", "B", "B", "B", "A", "A"),
		stringsAsFactors = FALSE
	)
			
	# if function is wrong, e.g. doesn't return all values in summaryTable
	# the remaining values are added anyway
	expect_silent(
		resFct <- computeSummaryStatisticsTable(
			data,
			rowVar = "TRT",
			rowOrderTotalFilterFct = function(summaryTable){
				c("")
			}
		)
	)
	expect_identical(levels(resFct$TRT), c("A", "B"))
	
})

test_that("Row elements are correctly ordered based on the total of a subset of the data", {
			
	# example order based on total would be different than based on specified subset data total
	data <- data.frame(
		USUBJID = seq.int(5),
		TRT = c("B", "B", "B", "A", "A"),
		SEX = c("M", "M", "F", "F", "F"),
		stringsAsFactors = FALSE
	)
			
	expect_silent(
		resTotal <- computeSummaryStatisticsTable(
			data = data,
			rowVar = c("TRT", "SEX"),
			rowOrder = "total",
			rowOrderTotalFilterFct = function(x) subset(x, TRT == "B")
		)
	)
	expect_identical(levels(resTotal$SEX), c("M", "F"))
			
})

test_that("The last element of the rows is correctly set when specified", {
			
	data <- data.frame(
		USUBJID = seq.int(5),
		TRT = c("Z", "B", "Other", "Y", "M"),
		catLast = "Other",
		stringsAsFactors = FALSE
	)
			
	expect_silent(
		res <- computeSummaryStatisticsTable(
			data = data,
			rowVar = "TRT",
			rowOrder = "auto",
			rowOrderCatLast = "Other"
		)
	)
	expect_identical(levels(res$TRT), c("B", "M", "Y", "Z", "Other"))
			
})

test_that("Specific combinations of row elements are included when requested", {
			
	# for example: table of lab abnormalities
	data <- data.frame(
		USUBJID = seq.int(7),
		PARAM = factor(c("A", "A", "A", "B", "B", "B", "B")),
		AVALC = factor(c(">10", ">10", ">10", "<=2", "]2, 10]", "]2, 10]", "]2, 10]")),
		stringsAsFactors = FALSE
	)
	
	avalcFact <- c("<=10", ">10", "<=2", "]2, 10]", "> 10")
	rowVarDataLevels <- data.frame(
		PARAM = c(rep("A", 2), rep("B", 3)),
		AVALC = factor(avalcFact, levels = avalcFact)
	)
	expect_silent(
		resSpecLevels <- computeSummaryStatisticsTable(
			data,
			rowVar = c("PARAM", "AVALC"),
			rowVarDataLevels = rowVarDataLevels
		)
	)
	# all levels specified in rowVarDataLevels are present:
	expect_identical(levels(resSpecLevels$AVALC), levels(rowVarDataLevels$AVALC))
	
	# categories not in the data are also present with a count of 0:
	avalcNotInData <- setdiff(avalcFact, data$AVALC)
	expect_true(all(avalcNotInData %in% resSpecLevels$AVALC))
	expect_identical(
		subset(resSpecLevels, AVALC %in% avalcNotInData)$statN,
		as.integer(rep(0, length(avalcNotInData)))
	)
	
})
	
test_that("The row totals are correctly computed", {
			
	data <- data.frame(
		USUBJID = c(1, 2, 3, 4, 5, 6, 6),
		SEX = c("M", "M", "M", "F", "F", "F", "F"),
		stringsAsFactors = FALSE
	)
			
	sumTableRowTotal <- computeSummaryStatisticsTable(
		data = data,
		rowVar = "SEX",
		rowVarTotalInclude = "SEX"
	)
	expect_true("Total" %in% sumTableRowTotal$SEX)
	sumTableTotal <- subset(sumTableRowTotal, SEX == "Total")
	expect_equal(sumTableTotal$statN, 6)
	expect_equal(sumTableTotal$statm, 7)
			
})


test_that("A warning is generated when the variable for row totals is not available", {
			
	data <- data.frame(
		USUBJID = c(1, 2, 3, 4, 5, 6, 6),
		SEX = c("M", "M", "M", "F", "F", "F", "F"),
		stringsAsFactors = FALSE
	)
			
	sumTableBase <- computeSummaryStatisticsTable(data = data, rowVar = "SEX")
	expect_warning(
		sumTableRowTotalWrong <- computeSummaryStatisticsTable(
			data = data,
			rowVar = "SEX",
			rowVarTotalInclude = TRUE
		),
		"Variable.*TRUE.* are not available"
	)
	expect_identical(sumTableRowTotalWrong, sumTableBase)
		
})

test_that("The row totals are correctly extracted in separated rows when requested", {
			
	data <- data.frame(
		USUBJID = seq.int(6),
		SEX = rep(c("M", "F"), times = 3),
		stringsAsFactors = FALSE
	)
	expect_silent(
		summaryTable <- computeSummaryStatisticsTable(
			data = data, 
			rowVar = "SEX",
			rowVarTotalInclude = "SEX",
			rowVarTotalInSepRow = "SEX"
		)
	)
	# inclusion of row var in separated row during export step
	# currently only stored in the output
	expect_true("SEX" %in% attr(summaryTable, "summaryTable")$rowVarTotalInSepRow)
		
})

test_that("A warning is generated if the variable for row totals to be included in separated rows is not among the row total variables", {
		
	data <- data.frame(
		USUBJID = seq.int(6),
		SEX = rep(c("M", "F"), times = 3),
		stringsAsFactors = FALSE
	)
			
	expect_warning(
		summaryTable <- computeSummaryStatisticsTable(
			data = data, 
			rowVar = "SEX",
			rowVarTotalInSepRow = "SEX"
		),
		"SEX.*in rowVarTotalInSepRow are ignored.*"
	)
			
})

test_that("The row totals are correctly computed by variable", {
			
	# example where a subject: 'a' has same severity for 2 adverse events
	# so should be counted only once for the 'Mild' category
	data <- data.frame(
		USUBJID = c("a", "a", "b", "c", "c", "a"),
		ABODSYS = rep("X", 6),
		AEDECOD = c("A", "A", "A", "A", "B", "B"),
		AESEV = c("Mild", "Moderate", "Mild", "Severe", "Moderate", "Mild"),
		stringsAsFactors = FALSE
	)
	
	sumTableRVBV <- computeSummaryStatisticsTable(
		data = data, 
		rowVar = c("AEDECOD", "AESEV"),
		rowVarTotalInclude = "AEDECOD",
		rowVarTotalByVar = "AESEV"
	)
	expect_true("Total" %in% sumTableRVBV$AEDECOD)
	sumTableRVBVRowTotal <- subset(sumTableRVBV, AEDECOD == "Total")
	expect_equal(as.character(sumTableRVBVRowTotal$AESEV), c("Mild", "Moderate", "Severe"))
	# subject 'a' is only counted once in the 'Mild' category
	expect_equal(subset(sumTableRVBVRowTotal, AESEV == "Mild")$statN, 2)
	# but 3 records are reported
	expect_equal(subset(sumTableRVBVRowTotal, AESEV == "Mild")$statm, 3)
	
})

test_that("The row totals are correctly computed by variable for only a subset of the variable to summarize", {
			
	# example where a subject: 'a' has same severity for 2 adverse events
	# so should be counted only once for the 'Mild' category
	data <- data.frame(
		USUBJID = c("a", "a", "b", "c", "c", "a"),
		ABODSYS = rep("X", 6),
		AEDECOD = c("A", "A", "A", "A", "B", "B"),
		AESEV = c("Mild", "Moderate", "Mild", "Severe", "Moderate", "Mild"),
		stringsAsFactors = FALSE
	)
	
	expect_silent(
		sumTableRVBVSpec <- computeSummaryStatisticsTable(
			data = data, 
			rowVar = c("ABODSYS", "AEDECOD", "AESEV"),
			rowVarTotalInclude = c("ABODSYS", "AEDECOD"),
			rowVarTotalByVar = c(AEDECOD = "AESEV")
		)
	)
	# if rowVarTotalByVar is not specified, only one total row is included
	sumTableTotalRowVar1 <- subset(sumTableRVBVSpec, ABODSYS == "Total")
	expect_length(nrow(sumTableTotalRowVar1), 1)
	
	# otherwise, total row(s) for each element in rowVarTotalByVar
	sumTableTotalRowVar2 <- subset(sumTableRVBVSpec, ABODSYS == "X" & AEDECOD == "Total")
	
	# should match stats based on table on ADECOD
	sumTableRVBV <- computeSummaryStatisticsTable(
		data = data, 
		rowVar = c("AEDECOD", "AESEV"),
		rowVarTotalInclude = "AEDECOD",
		rowVarTotalByVar = "AESEV"
	)
	sumTableRVBVRowTotal <- subset(sumTableRVBV, AEDECOD == "Total")
	expect_equal(
		subset(sumTableTotalRowVar2, select = -ABODSYS), 
		sumTableRVBVRowTotal,
		check.attributes = FALSE # row names might differ
	)

})

test_that("A warning is generated if the variable to compute row totals by is not available", {
			
	# example where a subject: 'a' has same severity for 2 adverse events
	# so should be counted only once for the 'Mild' category
	data <- data.frame(
		USUBJID = c("a", "a", "b", "c", "c", "a"),
		AEDECOD = c("A", "A", "A", "A", "B", "B"),
		stringsAsFactors = FALSE
	)
			
	expect_warning(
		summaryTable <- computeSummaryStatisticsTable(
			data = data, 
			rowVar = "AEDECOD",
			rowVarTotalInclude = "AEDECOD",
			rowVarTotalByVar = "a"
		),
		".*rowVarTotalByVar.*ignored"
	)
	
})

test_that("The row percentages are correctly computed by a row variable", {
	
	data <- data.frame(
		USUBJID = c("a", "b", "c", "d", "a", "b"),
		AEDECOD = c("A", "A", "A", "A", "B", "B"),
		stringsAsFactors = FALSE
	)
			
	sumTableRVTPerc <- computeSummaryStatisticsTable(
		data = data, 
		rowVar = "AEDECOD", 
		rowVarTotalPerc = "AEDECOD"
	)
	# sum percentage by rowVar == 100
	expect_equal(subset(sumTableRVTPerc, AEDECOD == "A")$statPercTotalN, 4)
	expect_equal(subset(sumTableRVTPerc, AEDECOD == "B")$statPercTotalN, 2)
	expect_equal(
		unique(as.numeric(na.omit(sumTableRVTPerc$statPercN))),
		100
	)
	
})

test_that("A warning is generated if the variable to compute row percentages is not available", {
			
	data <- data.frame(
		USUBJID = c("a", "b", "c", "d", "a", "b"),
		AEDECOD = c("A", "A", "A", "A", "B", "B"),
		stringsAsFactors = FALSE
	)
	
	# variable not in rowVar
	expect_warning(
		summaryTable <- computeSummaryStatisticsTable(
			data = data, 
			rowVar = "AEDECOD",
			rowVarTotalPerc = "a"
		),
		"rowVarTotalPerc.*ignored.*not available in: rowVar"
	)
	
})
	
test_that("The row percentages are correctly computed by the variable to summarize", {
				
	data <- data.frame(
		USUBJID = c("a", "b", "c", "d", "a", "b"),
		AEDECOD = c("A", "A", "A", "A", "B", "B"),
		AESEV = c("Mild", "Moderate", "Moderate", "Severe", "Moderate", "Mild"),
		AESEV2 = c("Mild", "Moderate", NA_character_, NA_character_, "Moderate", "Mild"),
		stringsAsFactors = FALSE
	)

	expect_silent(
		sumTableRVTPerc <- computeSummaryStatisticsTable(
			data = data, 
			rowVar = "AEDECOD",
			var = c("AESEV", "AESEV2"), 
			rowVarTotalPerc = "variable",
			varLabInclude = TRUE # because 'variable' not included if only one variable
		)
	)
	expect_equal(
		unique(subset(sumTableRVTPerc, variable == "AESEV")$statPercTotalN), 
		4
	)
	expect_equal(
		unique(subset(sumTableRVTPerc, variable == "AESEV2")$statPercTotalN), 
		2
	)
	
})

test_that("A warning is generated if row percentages should be computed by the variable to summarize but the variable is not included", {
		
	data <- data.frame(
		USUBJID = c("a", "b", "c", "d", "a", "b"),
		AEDECOD = c("A", "A", "A", "A", "B", "B"),
		AESEV = c("Mild", "Moderate", "Moderate", "Severe", "Moderate", "Mild"),
		AESEV2 = c("Mild", "Moderate", NA_character_, NA_character_, "Moderate", "Mild"),
		stringsAsFactors = FALSE
	)
	
	# variable not included in summary table:
	expect_warning(
		computeSummaryStatisticsTable(
			data = data, 
			rowVar = "AEDECOD",
			var = "AESEV", 
			rowVarTotalPerc = "variable"
		),
		"Percentages cannot be computed by.*variable.* because variable not included"
	)
	
})

test_that("The row percentages are correctly computed by the variable to summarize if only one variable is specified.", {
  
  data <- data.frame(
    USUBJID = c("a", "b", "c", "d", "a", "b"),
    AESEV = c("Mild", "Moderate", "Moderate", "Severe", "Moderate", "Mild"),
    stringsAsFactors = FALSE
  )
  
  # variable not included in summary table:
  expect_silent(
    summaryTable <- computeSummaryStatisticsTable(
      data = data, 
      var = "AESEV", varLabInclude = TRUE,
      rowVarTotalPerc = "variable"
    )
  )
  
  expect_setequal(
    object = na.omit(summaryTable$statPercTotalN),
    expect = length(unique(data$USUBJID))
  )
  
})

---
title: "Creation of in-text tables"
author: "Laure Cougnaud"
date: "`r format(Sys.Date(), '%B %d, %Y')`"
output: 
  rmarkdown::html_document:
    toc: true
    toc_float: true
    toc_depth: 5
    number_sections: true
vignette: >
  %\VignetteIndexEntry{02 - Creation of in-text tables}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

```{r options, echo = FALSE}
	
	library(knitr)
	opts_chunk$set(
		echo = TRUE, results = 'markup', warning = FALSE, 
		# stop document execution if error (not the default)
		error = FALSE, 
		message = FALSE, cache = FALSE,
		fig.width = 8, fig.height = 7,
		fig.path = "./figures_vignette/",
		fig.align = 'center')
	options(width = 170)
	# instead of warn = 0 by default
	# include warnings when they occur in the document
	options(warn = 1)
	
```


This vignette focuses on how to create in-text tables with the
`inTextSummaryTable` package.

In this vignette we assume you have ready the `data.frame`(s) to create the
tables. If you have doubts on the data format, please look the introductory
vignette at the section "data format".

We will use the example data available in the `clinUtils` package. Let's load
the packages and the data, and get started!


```{r loadPackages}

	library(inTextSummaryTable)
	library(tools) # toTitleCase

```

```{r loadData}	
	
	library(clinUtils)

	# load example data
    data(dataADaMCDISCP01)
	
	dataAll <- dataADaMCDISCP01
    labelVars <- attr(dataAll, "labelVars")
	
```


The **`getSummaryStatisticsTable`** creates an in-text table of summary
statistics for variable(s) of interest.

The _Demographic_ data (`ADSL` dataset) is used as example for the summary
statistics table.

```{r data-SL}

	dataSL <- dataAll$ADSL

```

# Variable(s) to summarize

Variable(s) to summarize in the table are specified via the **`var` parameter**.

Different set of statistics are reported depending on the type of variable:
[Categorical variable] or [Continuous variable].

See the documentation in section _Base statistics_ for more details on the
statistics included by default for each type, via:

```{r getHelp, eval = FALSE}

? `inTextSummaryTable-stats` 

```

## Categorical variable

For a **discrete/categorical variable**, the in-text table can display the
**counts/percentages of the number of subjects or records for each category** of
the variable.

### Counts of the entire dataset

If **no variable is specified** (via the `var` parameter), the counts are
displayed for the **entire dataset**.

```{r count-simple}

	getSummaryStatisticsTable(data = dataSL)

```

Please note that this is equivalent of setting (`var = 'all'`).

### Counts of categories

If a **variable is specified** (via the `var` parameter), the counts are
displayed **for each category**.

```{r count-categories}

	getSummaryStatisticsTable(data = dataSL, var = "SEX")

```

### Sort categories

The categories of the variable are sorted alphabetically by default. To sort the
categories in a specific order, the variable should be formatted as
**`factor`**, whose ordered categories are included in its **`levels`**.

```{r count-categories-order}

	# specify manually the order of the categories
	dataSL$SEX <- factor(dataSL$SEX, levels = c("M", "F"))
	getSummaryStatisticsTable(data = dataSL, var = "SEX")
	
	# order categories based on a numeric variable
    dataSL$SEXN <- ifelse(dataSL$SEX == "M", 2, 1)
	dataSL$SEX <- reorder(dataSL$SEX, dataSL$SEXN)
	getSummaryStatisticsTable(data = dataSL, var = "SEX")
	
```

### Inclusion of categories not available in the data

By default, the table only includes the categories present in the input data, to
ensure a compact table for CSR export.

```{r count-categories-empty-1}

	dataSLExample <- dataSL
	
	# 'SEX' formatted as character with only male
	dataSLExample$SEX <- "M" # only male
	getSummaryStatisticsTable(data = dataSLExample, var = "SEX")
	
```	

If extra categories should be represented in the table, the categorical variable
should be **formatted as a factor**, whose **levels contain all categories** to
be displayed in the table.

Furthermore, the parameter: `varInclude0` should be set to `TRUE` or to the
specific variable (in case multiple variables are specified) to indicate that
categories with 0 counts should be included.

```{r count-categories-empty-2}
	
	# 'SEX' formatted as factor, to include also female in the table
	# (even if not available in the data)
	dataSLExample$SEX <- factor("M", levels = c("F", "M"))
	getSummaryStatisticsTable(data = dataSLExample, var = "SEX", varInclude0 = TRUE)
	# or:
    getSummaryStatisticsTable(data = dataSLExample, var = "SEX", varInclude0 = "SEX")

```


### Count table for 'flag'-variables

A specific type of categorical variable is a **'flag variable'**, which indicates
if a record fulfills a specific criteria.

Such variable is typically formatted in the data as: 

* 'Y' if the criteria is met for the specific record
* 'N' if the criteria is not fulfilled for the specific record
* '' if the criteria is missing for this record

The name of such variable typically ends with **'FL'** in a CDISC-compliant _ADaM_
or _SDTM_ dataset.

For example, the subject-level dataset contains the following flag variables:

```{r flag-variables}

    labelVars[grep("FL$", colnames(dataSL), value = TRUE)]
    # has the subject discontinued from the study?
    dataSL$DISCONFL

```

If this variable is specified in `var`, the counts for each category is
reported:

```{r count-flag-var}

    getSummaryStatisticsTable(
	    data = dataSL,
	    var = "SAFFL"
    )

```

However, the interest is often to only reports the counts for the records
fulfilling the criteria (records with 'Y'). This is the case if the variable is
specified via the `varFlag` parameter too.

```{r count-flag-varFlag}

	getSummaryStatisticsTable(
		data = dataSL,
		var = "SAFFL",
		varFlag = "SAFFL"
	)

```

### Inclusion of total across categories

To include the total counts across categories, the `varTotalInclude` parameter
should be set to `TRUE` (or to the specific variable).

```{r count-varTotalInclude}

	getSummaryStatisticsTable(
		data = dataSL, 
		var = "SEX", 
		varTotalInclude = TRUE
	)

```

## Continuous variable

For a **continuous variable**, the in-text table displays **standard
distribution statistics** of the variable.
  
Please note that **missing records (NA) for the variable are filtered**, so the
**count statistics** (number of subjects, records, percentage) are based **only
on the non missing records**.
  
For a continuous variable, the presence of different values for the same subject 
(and across row/column variables) are checked and an appropriate error message
is returned if multiple different values are available.

```{r numeric}

	getSummaryStatisticsTable(data = dataSL, var = "AGE")

```

## Continuous and categorical variables in the table

The table can contain a mix of categorical and continuous variables.

```{r mixedTable}

	getSummaryStatisticsTable(
		data = dataSL, 
		var = c("AGE", "SEX")
	)

```

# Statistics of interest

Statistics of interest and their format are specified via the **`stats`
parameter**.

If an unique statistic expression is specified, the 'Statistic' column doesn't
appear in the table.  
In case multiple statistics are specified, these are
included as separated row.

## Standard statistic set

A standard set of statistics is specified via specific tags to be passed to the
`stats` function.

The list of available statistics is mentioned in the section '_Formatted
statistics_' in:

```{r getHelpStats, eval = FALSE}

    ? `inTextSummaryTable-stats` 

```

Please see below examples of commonly used statistics.

### Categorical table

```{r stats-count}

	# count: n, '%' and m
	getSummaryStatisticsTable(
		data = dataSL,
		var = "SEX",
		stats = "count"
	)

	# n (%)
	getSummaryStatisticsTable(
		data = dataSL,
		var = "SEX",
		stats = "n (%)"
	)
	
	# n/N (%)
	getSummaryStatisticsTable(
		data = dataSL,
		var = "SEX",
		stats = "n/N (%)"
	)
	
```

### Continuous variable

```{r stats-numeric}
	
	## continuous variable
	
	# all summary stats
	getSummaryStatisticsTable(
		data = dataSL,
		var = "AGE",
		stats = "summary"
	)
	
	# median (range)
	getSummaryStatisticsTable(
		data = dataSL,
		var = "AGE",
		stats = "median (range)"
	)
	
	# median and (range) in a different line:
	getSummaryStatisticsTable(
		data = dataSL,
		var = "AGE",
		stats = "median\n(range)"
	)
	
	# mean (se)
	getSummaryStatisticsTable(
		data = dataSL,
		var = "AGE",
		stats = "mean (se)"
	)
	
	# mean (sd)
	getSummaryStatisticsTable(
		data = dataSL,
		var = "AGE",
		stats = "mean (sd)"
	)
	
	# mean and sd in separated rows
	getSummaryStatisticsTable(
		data = dataSL,
		var = "AGE",
		stats = c("Mean", "SD")
	)
```

## Custom statistics formatting (Advanced)

To change the formatting of the statistics, the `stats` parameter should contain
a language object (e.g. `expression` or `call`) of the default base set of
statistics.

See the documentation in section '_Base statistics_' for more details on the base
statistics included by default, via:

```{r getHelpStats2, eval = FALSE}

? `inTextSummaryTable-stats` 

```

For example, the following count table is restricted to the number of subjects
per categories:

```{r stats-N}

	getSummaryStatisticsTable(
		data = dataSL,
		var = c("RACE", "SEX"),
		stats = list(N = expression(statN))
	)

```

The summary statistics table is restricted to the median and range:

```{r stats-meanSE}

	getSummaryStatisticsTable(
		data = dataSL,
		var = c("AGE", "HEIGHTBL", "WEIGHTBL", "BMIBL"),
		varGeneralLab = "Parameter", statsGeneralLab = "",
		colVar = "TRT01P",
		stats = list(
			`median` = expression(statMedian),
			`(min, max)` = expression(paste0("(", statMin, ",", statMax, ")"))
		)
	)

```

Note that the 'Standard statistics set' is formatted internally via the
`getStatsData` (and `getStats`) functions, which creates consistently a list of
`language` objects.

```{r getStatsData}

    # this count table:
    getSummaryStatisticsTable(
	    data = dataSL,
	    var = "SEX",
	    stats = "count"
    )
    # ... is equivalent to:
    getSummaryStatisticsTable(
	    data = dataSL,
	    var = "SEX",
	    stats = getStats(type = "count")
    )

    # this summary table...
    getSummaryStatisticsTable(
	    data = dataSL,
	    var = "AGE",
	    stats = "mean (se)"
    )
    # ... is equivalent to:
    getSummaryStatisticsTable(
	    data = dataSL,
	    var = "AGE",
	    stats = getStatsData(type = "mean (se)", var = "AGE", data = dataSL)[["AGE"]]
    )
```

## Statistics by variable/group

The statistics can also be provided for each variable separately, if `stats` is
named by variable:

```{r statExtra-EachVariable}
		
	getSummaryStatisticsTable(
		data = dataSL, 
		var = c("AGE", "RACE"),
		stats = list(
			AGE = getStats("median (range)"),
			RACE = getStats("n (%)")
		)
	)

```

## Extra statistics

Extra statistics (not available in the default set of statistics) should be
specified via the `statsExtra` parameter.

A set of extra utility functions to compute common extra statistics are also
available in the package:

* coefficient of variation with the `cv` function
* geometric mean with the `geomMean` function
* geometric standard deviation with the `geomSD` function
* geometric coefficient of variation with the `geomCV` function

```{r statsUtilityFct}

	getSummaryStatisticsTable(
		data = dataSL,
		var = "HEIGHTBL",
		# specify extra stats to compute
		statsExtra = list(
			statCV = cv,
			statGeomMean = geomMean,
			statGeomSD = geomSD,
			statsGeomCV = geomCV
		)
	)

```

Full customized statistics can also be provided. For example, if you would like
to specify your own formula for the coefficient of variation:

```{r statsExtra}

	# include the coefficient of variation via the 'statsExtra' parameter
	getSummaryStatisticsTable(
		data = dataSL,
		var = "HEIGHTBL",
		statsExtra = list(statCVPerc = function(x) sd(x)/mean(x)*100)
	)

```

These statistics are then available for customization via the `stats` parameter.

```{r statsExtra-stats}

	# format the statistics with the 'stats' parameter
	getSummaryStatisticsTable(
		data = dataSL,
		var = "HEIGHTBL",
		statsExtra = list(statCVPerc = function(x) sd(x)/mean(x)*100),
		stats = list(Mean = expression(statMean), 'CV%' = expression(statCVPerc))
	)


```

## Rounding strategy

Please note that all statistics are rounded by default 
in the package based on the 
**'rounding up' strategy for rounding off a 5**, 
which **differs from the default rounding strategy in R** (`round` function).

This was a deliberate choice to reproduce summarized statistics 
created with the SAS software.

Please find more explanations in the documentation of the 
`? roundHalfUp` and `? roundHalfUpTextFormat` functions.

## Number of decimals

The detailed rules for the number of decimals for the statistics
are described in the section _Statistics formatting_ in:

```{r getHelpStats3, eval = FALSE}

    ? `inTextSummaryTable-stats` 

```

To specify fixed amounts of digits for the statistics to be displayed in the
table, the statistics are formatted in the `stats` parameter.

### Default number of decimals

#### Categorical variable

The percentages are formatted by default as specified in the table below.

```{r nDecimals-catVar, echo = FALSE, fig.cap = "Standard Layout for Frequency Tabulations of Categorical Variables<br>"}

include_graphics("./images/nDecimals_catVar.png")

```

By default, the counts for a categorical variables are formatted as specified
above:

* the number of subjects is displayed with 0 digits (`nDecN` is set to 0)
* the frequency percentage is implemented in the `formatPercentage` function

```{r getStats-count}

	# Internal rule for the number of decimals for the percentage
	formatPercentage(c(NA, 0, 100, 99.95, 0.012, 34.768))
	
	# Used by default in the 'getStats' function
	getStats(type = "count")

```

#### Continuous variable {#numberDecimalsContinuousVariable}

The number of decimals for statistics based on a continuous variable is by default
as specified in the tables below.

```{r nDecimals-numVar, echo = FALSE, fig.cap = "Standard Layout for Descriptive Statistics of Continuous Variables<br>"}

include_graphics("./images/nDecimals_numVar.png")

```

In the package: 'Very small values' are considered values below 1.

When specifying the default set of available statistics with the `getStats`
function, and **only if the variable is specified** (`x` parameter), the number
of decimals for a continuous variable is determined by:

1. Extracting the number of decimals for individual values based on:
    + **pre-defined rules** based on the number of decimals of the individual
      values (`getNDecimalsRule` function)
    + the number of decimals **available in the input data** via the
      `getNDecimalsData` function
    + taking the **minimum of these two criterias** (`getNDecimals` function), 
    such as the number of decimals according  the rule won't be higher that the
      actual number of decimals available in the data
2. Taking the **maximum number of decimals** across all individual values via
   the `getMaxNDecimals` function, which is used as 'base' number of decimals
   considered for the summary statistics
3. The actual number of decimals for each statistic is extracted 
by adding to the 'base' number of decimals:
    + **0 extra decimal for the minimum, maximum**
    + **1 extra decimal for the mean, median, sd**
    + **2 extra decimals for SE**

Please note that if a different framework than implemented in steps 1 and 2
should be used for the extraction of the number of decimals for a specific
variable, the number of decimals of interest can be fixed via the `nDecCont`
parameter.

```{r getMaxNDecimals}

	# Duration of Disease (Months)
	print(dataSL$DURDIS)
	
	## Extract the number of decimals for each value:
	
	# based on pre-defined rule, this metric should be displayed with 1 decimal:
	getNDecimalsRule(x = dataSL$DURDIS)
	
	# but available in the data only with 0 decimals
	getNDecimalsData(x = dataSL$DURDIS)
	
	# The minimum of the #decimals based on the data and pre-defined rule is:
	getNDecimals(x = dataSL$DURDIS)
	
	## Take the maximum number of decimals 
	getMaxNDecimals(x = dataSL$DURDIS)
	
	## Custom set of statistics are extracted when x is specified:
	getStats(x = dataSL$DURDIS)
	
	# To fix the number of decimals:
	getStats(type = "summary", nDecCont = 1)
	
	## Create summary statistics table
	getSummaryStatisticsTable(
		data = dataSL,
		var = c("AGE", "DURDIS"),
		stats = list(
			AGE = getStats(type = "median (range)", x = dataSL$AGE),
            DURDIS = getStats(type = "median (range)", x = dataSL$DURDIS)
		)
	)

```

### Custom `stats` function (Advanced)

A  custom function can be created to create custom statistics with fixed number
of digits.

For example, the AGE is displayed with 1 digit and the height with two digits:

```{r stats-digits}

	getSummaryStatisticsTable(
		data = dataSL,
		var = c("AGE", "HEIGHTBL"),
		stats = list(
			AGE = list(Median = expression(roundHalfUpTextFormat(statMedian, 1))),
            HEIGHTBL = list(Median = expression(roundHalfUpTextFormat(statMedian, 2)))
		)
	)

```

To create the `stats` parameter for a specific number of digits, a custom
function can be created:

```{r stats-digits-complex}

	# wrapper function to include median with specific number of digits
	# and min/max with specified number of digits - 1
	statsDMNum <- function(digitsMin)
		list('Median (range)' = 
			bquote(paste0(
				roundHalfUpTextFormat(statMedian, .(digitsMin+1)), 
				" (", roundHalfUpTextFormat(statMin, .(digitsMin)), ",", 
				roundHalfUpTextFormat(statMax, .(digitsMin)),
				")"
			))
	)

	getSummaryStatisticsTable(
		data = dataSL,
		var = c("AGE", "HEIGHTBL", "WEIGHTBL", "BMIBL", "RACE", "SEX"),
		stats = list(
			AGE = statsDMNum(0),
			HEIGHTBL = statsDMNum(1),
			WEIGHTBL = statsDMNum(1),
			BMIBL = statsDMNum(1),
			RACE = getStats("n (%)"),
			SEX = getStats("n (%)")
		)
	)

```

## Statistics layout

The layout of the statistics is specified via the `statsLayout` parameter.  

By default, the statistics are included in rows within each variable.

```{r statsLayoutRow}

	# statsLayout = 'row'
	getSummaryStatisticsTable(
		data = dataSL,
		var = c("AGE", "HEIGHTBL"),
		stats = list(Mean = expression(statMean), 'SE' = expression(statSE))
	)
	
```

The statistics can also be included in columns.

```{r statsLayoutCol}

	getSummaryStatisticsTable(
		data = dataSL,
		var = c("AGE", "HEIGHTBL"),
		stats = list(Mean = expression(statMean), 'SE' = expression(statSE)),
		statsLayout = "col"
	)

```

The statistics can also be specified in different rows, but in a separated
column.

```{r summaryTable-PP-medianMinMax-statsLayoutRowVarInSepCol}

	getSummaryStatisticsTable(
		data = dataSL,
		var = c("AGE", "HEIGHTBL"),
		stats = list(Mean = expression(statMean), 'SE' = expression(statSE)),
		statsLayout = "rowInSepCol"
	)

```

By default, if only one statistic is available in the table, the name of the
statistic is not included in the rows/columns, as the statistic is generally
described in this case in the title of the table.

```{r summaryTable-statsLayout-onlyOneStat}

	getSummaryStatisticsTable(
		data = dataSL,
		var = c("AGE", "HEIGHTBL"),
		stats = list(Mean = expression(statMean))
	)

```

To include even in this case the name of the statistic, the parameter
`statsLabInclude` should be set to `TRUE`.

```{r summaryTable-statsLayout-onlyOneStat-statsLabInclude}

	getSummaryStatisticsTable(
		data = dataSL,
		var = c("AGE", "HEIGHTBL"),
		stats = list(Mean = expression(statMean)),
		statsLabInclude = TRUE
	)

```

# Table layout

The general table layout is driven by the specification of variables to be
displayed in rows (in the vertical direction) or in columns (in the horizontal
direction).

If no variables are specified in `var`, counts across row/column variable are
displayed.

The adverse events dataset is used for demonstration.

```{r countTable-AE-data}
				
	dataAE <-  subset(dataAll$ADAE, SAFFL == "Y" & TRTEMFL == "Y")
	
	# ensure that order of elements is the one specified in 
	# the corresponding numeric variable
	dataAE$TRTA <- with(dataAE, reorder(TRTA, TRTAN))
	dataAE$AESEV <- factor(
		dataAE$AESEV, 
		levels = c("MILD", "MODERATE", "SEVERE")
	)
	
	dataAEInterest <- subset(dataAE, AESOC %in% c(
		"INFECTIONS AND INFESTATIONS",
        "GENERAL DISORDERS AND ADMINISTRATION SITE CONDITIONS"
       )
    )

```

## Row and column variables

Specific grouping variable(s) for the columns can be specified via the
**`colVar`** parameter and for the rows via the **`rowVar`** parameter.

If multiple category variables are specified, they should be specified in
hierarchical order.

```{r rowVarColVar}

	# unique row variable
	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = "AEDECOD",
		stats = getStats("n (%)"),
		labelVars = labelVars
	)
	
	# multiple nested row variables
	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		stats = getStats("n (%)"),
		labelVars = labelVars
	)
	
	# unique column variable
	getSummaryStatisticsTable(
		data = dataAEInterest,
		colVar = "TRTA",
		stats = getStats("n (%)"),
		labelVars = labelVars
	)
	
	# combination of rows and columns
	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		stats = getStats("n (%)"),
		labelVars = labelVars,
		colHeaderTotalInclude = FALSE
	)

```

## Row variable

By default (when `outputType` is set to: 'flextable'), if multiple row variables
are specified, they are considered nested and displayed in the first column of
the final table. Each sub-category is indicated with a specific indent
(customizable with `rowVarPadBase`).

### Variable in separated column

**Row variables** that should be included as a **separated column** should be
specified via the `rowVarInSepCol` parameter.

```{r rowVarInSepCol}

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD", "AESEV"),
		rowVarInSepCol = "AESEV",
		colVar = "TRTA",
		stats = getStats("n (%)"),
		labelVars = labelVars
	)
	
```

### Row ordering

The **categories in the row variables can be ordered** based on the
**`rowOrder`** variable. 

This variable is either:

* a string with the name of an implemented method to order the rows, among:
     + `alphabetical`: categories are ordered **alphabetically**
     + `auto`: categories are ordered based on the **levels** if the input
       variable is a factor,    alphabetically otherwise
     + `total`: categories are ordered based on the 'total' column (see section
       \@ref(colTotal))    (if the total column is not included in the table)
* a custom ordering function to apply in the data to order the rows

#### Common order for all row variables

```{r rowOrder-common}

	# 'auto':

	# set order of SOC to reverse alphabetical order
	dataAEInterest$AESOC <- factor(
		dataAEInterest$AESOC, 
		levels = rev(sort(unique(as.character(dataAEInterest$AESOC))))
	)
	# AEDECOD is not a factor -> sort alphabetically by default
	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarLab = labelVars[c("AEDECOD")],
		rowVarTotalInclude = c("AESOC", "AEDECOD"),
		colVar = "TRTA", colTotalInclude = TRUE,
		stats = getStats("n (%)"),
		labelVars = labelVars
	)
	
	# total counts
	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarLab = labelVars[c("AEDECOD")],
		rowVarTotalInclude = c("AESOC", "AEDECOD"),
		colVar = "TRTA", colTotalInclude = TRUE, colTotalLab = "Number of subjects",
		rowOrder = "total",
		stats = getStats("n (%)"),
		labelVars = labelVars
	)
	# same order even if the 'total' column is not specified
	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarLab = labelVars[c("AEDECOD")],
		rowVarTotalInclude = c("AESOC", "AEDECOD"),
		colVar = "TRTA", 
		rowOrder = "total", 
		stats = getStats("n (%)"),
		labelVars = labelVars
	)

```

#### Different orders for each row variable

In case the order should be different for each row variable, a named list is
provided for the `rowVar` parameter.

```{r rowOrder-specific}

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarLab = labelVars[c("AEDECOD")],
		rowVarTotalInclude = c("AESOC", "AEDECOD"),
		colVar = "TRTA", #colTotalInclude = TRUE,
		rowOrder = c(AESOC = "alphabetical", AEDECOD = "total"),
		stats = getStats("n (%)"),
		labelVars = labelVars
	)

```

#### Row order based on the total of a column category

If the row categories should be **ordered by total counts for a specific
category of the column variable(s)**, a function **`rowOrderTotalFilterFct`** is
specified.

The adverse events are sorted based on the incidence in the treated group.

```{r rowOrder-rowOrderTotalFilterFct}

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarLab = labelVars[c("AEDECOD")],
		rowVarTotalInclude = c("AESOC", "AEDECOD"),
		colVar = "TRTA", colTotalInclude = TRUE,
		rowOrder = "total",
		stats = getStats("n (%)"),
		labelVars = labelVars,
		# consider only the counts of the treated patients to order the rows
		rowOrderTotalFilterFct = function(x) subset(x, TRTA == "Xanomeline High Dose")
	)

```

#### Row order based on a custom specified function

If the method to order the rows is more complex, the `rowOrder` parameter
specifies a function taking the summary table as input and returning the order
levels of the elements in the row variable.

For example, the adverse event table is sorted based on the counts of patient
presenting this event across all treatment classes, and in case of ties based on 
the counts of treated-patients presenting this event.

```{r rowOrder-functionExample1}

	library(plyr)
	getSummaryStatisticsTable(
		data = dataAEInterest,
		type = "count",
		rowVar = "AEHLT",
		rowOrder = function(x){
			x <- subset(x, !isTotal)
			totalAcrossTreatments <- subset(x, TRTA == "Total")
			# counts across treated patients
			totalForTreatmentOnly <- subset(x, TRTA == "Xanomeline High Dose")
			dataCounts <- merge(totalAcrossTreatments, totalForTreatmentOnly, by = "AEHLT", suffixes = c(".all", ".treat"))
			# sort first based on overall count, then counts of treated patients
			dataCounts[with(dataCounts, order(`statN.all`, `statN.treat`, decreasing = TRUE)), "AEHLT"]
		},
		colVar = "TRTA", colTotalInclude = TRUE,
		labelVars = labelVars,
		title = "Table: Adverse Events ordered based on total counts",
		stats = list(expression(paste0(statN, " (", round(statPercN, 1), ")"))),
		footer = "Statistics: n (%)"
	)

```

The adverse event table is now ordered based on the counts in the placebo, then
treated-patients column, for the organ class and the adverse event term
separately.

```{r rowOrder-functionExample2}

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarLab = labelVars[c("AEDECOD")],
		rowVarTotalInclude = c("AESOC", "AEDECOD"),
		colVar = "TRTA", colTotalInclude = TRUE,
		rowOrder = list(
			AESOC = function(table) {
				# records with total for each AESOC
				nAESOCPlacebo <- subset(table, !isTotal & grepl("placebo", TRTA) & AEDECOD == "Total")
				nAESOCTreat <- subset(table, !isTotal & grepl("High Dose", TRTA) & AEDECOD == "Total")
				nAESOCDf <- merge(nAESOCPlacebo, nAESOCTreat, by = "AESOC", suffixes = c(".placebo", ".treatment"))
				nAESOCDf[with(nAESOCDf, order(`statN.placebo`, `statN.treatment`, decreasing = TRUE)), "AESOC"]
			},
			AEDECOD = function(table) {
				# records with counts for each AEDECOD
				nAEDECODPlacebo <- subset(table, !isTotal & grepl("placebo", TRTA) & AEDECOD != "Total")
				nAEDECODTreat <- subset(table, !isTotal & grepl("High Dose", TRTA) & AEDECOD != "Total")
				nAEDECODDf <- merge(nAEDECODPlacebo, nAEDECODTreat, by = "AEDECOD", suffixes = c(".placebo", ".treatment"))
				nAEDECODDf[with(nAEDECODDf, order(`statN.placebo`, `statN.treatment`, decreasing = TRUE)), "AEDECOD"]
			}
		),
		stats = getStats("n (%)"),
		labelVars = labelVars
	)

```

### Row variable labels

#### Based on dataset

The **labels used for the variables parameter** (row variables) **are
automatically extracted from the labels** contained in the _SAS_ dataset, by
specifying the `labelVars` parameter.

```{r summaryTable-PP-rowVarWithLabel-labelVars}

	# combination of rows and columns
	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		stats = getStats("n (%)"),
		labelVars = labelVars
	)
	
```

#### Custom

The label can also be specified directly via the `rowVarLab` parameter, for each
variable in `rowVar`.

If an unique row label should be used (even if multiple row variables are
specified), `rowVarLab` is set to this unique label.

```{r summaryTable-PP-rowVarWithLabel-rowVarLab}

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		stats = getStats("n (%)"),
		rowVarLab = c(
			'AESOC' = "TEAE by SOC and Preferred Term\nn (%)"
		),
		labelVars = labelVars
	)
	
```

### Inclusion of row/column categories not available in the data

As for the variable to summarize, to include categories in the row or column
variables not available in the data, these variables should be formatted as a
factor with categories specified in its levels.

Furthermore, the parameters `rowInclude0` and `colInclude0` should be set to
TRUE to include counts for empty categories within the row/column. 

```{r countTable-emptyVars}

	## only consider a subset of adverse events
	dataAESubset <- subset(dataAE, AEHLT == "HLT_0617")
	
	## create dummy categories for:
	# treatment
	dataAESubset$TRTA <- with(dataAESubset, 
		factor(TRTA, levels = c(unique(as.character(TRTA)), "Treatment B"))
	)
	# low-level term category
	dataAESubset$AELLT <- with(dataAESubset, 
		factor(AELLT, levels = c(unique(as.character(AELLT)), "Lymphocyte percentage increased"))
	)
	
	# create summary statistics table
	getSummaryStatisticsTable(
		data = dataAESubset,
		type = "count",
		rowVar = c("AEHLT", "AELLT"),
		rowInclude0 = TRUE, colInclude0 = TRUE,
		colVar = "TRTA",
		labelVars = labelVars,
		title = "Table: Adverse Events: white blood cell analyses",
		stats = getStats("n (%)"),
		footer = "Statistics: n (%)"
	)

```


## Variable(s) to summarize

### Default

The **variable(s) used for the summary statistics** (`var`) are included **by
default in rows**.

```{r summaryTable-layout-var}

    dataDIABP <- subset(dataAll$ADVS, 
		SAFFL == "Y" & ANL01FL == "Y" &
		PARAMCD == "DIABP" & 
		AVISIT %in% c("Baseline", "Week 8") &
		ATPT == "AFTER LYING DOWN FOR 5 MINUTES"
	)
    dataDIABP$TRTA <- reorder(dataDIABP$TRTA, dataDIABP$TRTAN)
    dataDIABP$AVISIT <- reorder(dataDIABP$AVISIT, dataDIABP$AVISITN)
    
	getSummaryStatisticsTable(
		data = dataDIABP,
		var = c("AVAL", "CHG"),
		colVar = "TRTA",
		rowVar = "AVISIT",
		labelVars = labelVars,
		stats = getStats("summary-default")
	)

```

### Summary variable in columns

In case multiple variables are to be summarized, the different variables can be
included in different columns by including the specific label: 'variable' in
`colVar`. Beware that such layout only makes sense for variables with similar
types (e.g. all numeric variables).

```{r summaryTable-layout-varInColumn}
    
getSummaryStatisticsTable(
	data = dataDIABP,
	var = c("AVAL", "CHG"),
	colVar = c("variable", "TRTA"),
	rowVar = "AVISIT",
	labelVars = labelVars,
	stats = getStats("summary-default")
)

```

### Inclusion of summary variables in case one variable is specified

By default, the variable label is not included if only one summary statistic
variable is specified.

```{r summaryTable-layout-var-oneLabel-default}
		
	getSummaryStatisticsTable(data = dataSL, var = "AGE", colVar = "TRT01P")

```

To include the label in case only one summary statistic variable is specified,
the parameter `varLabInclude` should be set to TRUE.

```{r summaryTable-layout-var-oneLabel-varLabInclude}
		
	getSummaryStatisticsTable(
		data = dataSL, 
		var = "AGE", 
		varLabInclude = TRUE,
		colVar = "TRT01P"
	)

```

## Inclusion of the counts per group in case of missing values

It might be of interest to display the counts of all subjects per row/column
variable in association of the summary statistic of a variable of interest.

For example it could be of interest to report the total number of subjects per
group, which could differ from the total number of subjects for a variable of
interest if this variable contain missing values.

```{r summaryTable-layout-var-empty}

    dataAEInterest$AESEVN <- ifelse(dataAEInterest$AESEV == "MILD", 1, 2)
	dataAEInterestWC <- ddply(dataAEInterest, c("AEDECOD", "USUBJID", "TRTA"), function(x) {
		x[which.max(x$AESEVN), ]
	})
	dataAEInterestWC[1, "AESEV"] <- NA
    getSummaryStatisticsTable(
        data = dataAEInterestWC,
        colVar = "TRTA",
        rowVar = "AEBODSYS",
		stats = getStats("n (%)"),
        var = c("AESEV", "all"),
		labelVars = labelVars
    )

```

# Total

The summary table contains **different types of total**:

* total used for the **percentage computation** displayed in the table.  
For example: report percentage of subjects with specific adverse event.
* total reported in the **column header**  
For example: total number of subjects
  for a specific treatment arm.
* **total across rows**, reported in the row header   
For example: to report percentage of subjects with adverse events in a specific
  body system (across adverse events).
* **total across columns**, reported in a separated column  
For example: to report summary statistics across all treatments arms.

By default, the totals are extracted based on the input data, but separated
datasets can be specified for the header,  percentage computation, row or column
total.

## Summary

The different types of total of the summary table are summarized below:

| Type | Inclusion in the table |  Dataset: parameter name | Dataset: default | 
|---|---|---|---| 
| Total in the column header | Yes by default <br/> removed if `colHeaderTotalInclude = FALSE`| `dataTotal` | `data` for table content <br/>`dataTotalCol` for total column| 
| Total for the percentage | Only if percentage requested in `stats` | `dataTotalPerc` | `dataTotal` for table content <br/> `dataTotalCol` for total column <br/> (for 'total' if specified as a list)| 
|Total across rows | Not by default <br/> for specified row variable with `rowVarTotalInclude` | `dataTotalRow`  | `data` for table content <br/> `dataTotalCol` for total column <br/> (for 'total' if specified as a list) | 
|Total across columns | Not by default <br/> only if `colTotalInclude = TRUE` | `dataTotalCol` | `data` |

## Total for the column header

### Current datasset

By default, the total reported in the total header is extracted from the
**available number of subjects** in the input `data`.

For example, the total number of patients per treatment arm is extracted from
the subject-level (`ADSL`) dataset. 

```{r header-total-default}
	
	# by default, total number of subjects extracted from data
	getSummaryStatisticsTable(
		data = subset(dataAEInterest, AESOC == "INFECTIONS AND INFESTATIONS"),
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		stats = getStats("n (%)"),
		rowVarLab = c(
			'AESOC' = "TEAE by SOC and Preferred Term\nn (%)"
		),
		labelVars = labelVars
	)
	
```

### External dataset

 If the total should be extracted from a **different dataset**,  it should be
specified via the **`dataTotal`** variable.  Please note that by default
`dataTotal` is also used for  the computation of the percentage.

```{r header-total-dataTotal}	
	
	# dataset used to extract the 'Total'
	dataTotalAE <- subset(dataAll$ADSL, SAFFL == "Y")
	# should contain columns specified in 'colVar'
	dataTotalAE$TRTA <- dataTotalAE$TRT01A 

	getSummaryStatisticsTable(
		data = subset(dataAEInterest, AESOC == "INFECTIONS AND INFESTATIONS"),
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		stats = getStats("n (%)"),
		rowVarLab = c(
			'AESOC' = "TEAE by SOC and Preferred Term\nn (%)"
		),
		dataTotal = dataTotalAE,
		labelVars = labelVars
	)

```

## Remove total in column header

The total number of subjects in each column is by default included. This is not
displayed if `colHeaderTotalInclude` is set to FALSE.

```{r header-total-colHeaderTotalInclude}

	getSummaryStatisticsTable(
		data = subset(dataAEInterest, AESOC == "INFECTIONS AND INFESTATIONS"),
		rowVar = c("AESOC", "AEDECOD"),
		rowVarTotalInclude = "AEDECOD",
		rowVarTotalInSepRow = "AEDECOD",
		colVar = "TRTA",
		stats = getStats("n (%)"),
		rowVarLab = c(
			'AESOC' = "TEAE by SOC and Preferred Term\nn (%)"
		),
		dataTotal = dataTotalAE,
		labelVars = labelVars,
		colHeaderTotalInclude = FALSE
	)

```

## Percentage

### Dataset

A different dataset used for the computation of the percentage can be specified
via the **`dataTotalPerc`** parameter.  

```{r dataTotalPerc}

    getSummaryStatisticsTable(
	    data = subset(dataAEInterest, AESOC == "INFECTIONS AND INFESTATIONS"),
	    rowVar = c("AESOC", "AEDECOD"),
	    colVar = "TRTA",
	    stats = getStats("n (%)"),
	    rowVarLab = c(
		    'AESOC' = "TEAE by SOC and Preferred Term\nn (%)"
	    ),
	    dataTotalPerc = dataTotalAE,
	    labelVars = labelVars
    )

```

Please note that by default, if `dataTotalPerc` is specified, but not
`dataTotal`, counts reported in the column header are still extracted from
`data`.

### Variables to compute percentage by

If the total number of subjects differ between the components of the table, the
extra row/column(s) variable(s) are specified via
`colVarTotalPerc`/`rowVarTotalPerc`.

For example, in a table of laboratory measurements per reference range
(laboratory abnormalities): the
total number of subjects for the computation of the percentage are extracted
based on the number of subjects with available measurements per visit.

```{r rowVarTotalPerc}

	dataLB <- subset(dataAll$ADLBC, 
		SAFFL == "Y" & 
		PARAMCD %in% c("K", "CHOL") &
		grepl("(Baseline)|(Week 20)", AVISIT)
	)
	dataLB$AVISIT <- with(dataLB, reorder(trimws(AVISIT), AVISITN))
	
	# counts versus the total per actual treatment arm
	getSummaryStatisticsTable(
		data = dataLB,
		colVar = "TRTA", 
		rowVar = c("PARAM", "AVISIT"), 
		var = "LBNRIND",
		stats = getStats("n (%)"),
		rowAutoMerge = FALSE, emptyValue = "0",
	)
	
	# percentage based on total number of subjects with available
	# measurement at specific visit for each parameter
	getSummaryStatisticsTable(
		data = dataLB,
		colVar = "TRTA", 
		rowVar = c("PARAM", "AVISIT"), 
		rowVarTotalPerc = c("PARAM", "AVISIT"),
		var = "LBNRIND",
		stats = getStats("n (%)"),
		rowAutoMerge = FALSE, emptyValue = "0",
	)	
	
```

Please note the different percentage for the number of patients
with normal cholesterol measurements at week 20 between
the two tables.

### Percentage of the number of records

By default, the percentage is based on the number of subjects.  

If the percentage should be computed based on the number of records instead, the
parameter: `statsPerc` should be set to `statm` (`statN` by default).

For example, to extract the percentage of laboratory measurements by reference
range and parameter:

```{r statsPerc}

getSummaryStatisticsTable(
	data = dataLB,
	colVar = "TRTA", 
	rowVar = c("PARAM", "AVISIT"), 
	rowVarTotalPerc = c("PARAM", "AVISIT"),
	var = "LBNRIND", 
	stats = getStats("m (%)"),
	statsPerc = "statm",
	rowAutoMerge = FALSE, emptyValue = "0",
)

```

## Total across columns {#colTotal}

### Inclusion

The total **across all columns** is included if the `colTotalInclude` is set to
TRUE.

```{r colTotalInclude}

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		colTotalInclude = TRUE, 
		stats = getStats("n (%)"),
		rowVarLab = c(
			'AESOC' = "TEAE by SOC and Preferred Term\nn (%)"
		),
		dataTotal = dataTotalAE,
		labelVars = labelVars
	)

```

By default, the total number of subjects is extracted based on the input dataset
across columns: subjects presenting the **same event in multiple column(s) are
counted once** in the column total (e.g. for adverse event table in a context of
cross-over experiment).

### Label

This column is by default labelled 'Total', but this can be customized with the
`colTotalLab` parameter.

```{r colTotalLab}

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		colTotalInclude = TRUE, colTotalLab = "All subjects",
		stats = getStats("n (%)"),
		rowVarLab = c(
			'AESOC' = "TEAE by SOC and Preferred Term\nn (%)"
		),
		dataTotal = dataTotalAE,
		labelVars = labelVars
	)

```

### Dataset

A different dataset for the total column can also be specified via the
`dataTotalCol` parameter.

For example, the table is restricted to only the treatment arm, but both arms
are considered in the total column:

```{r dataTotalCol}

	getSummaryStatisticsTable(
		data = subset(dataAEInterest, grepl("High Dose", TRTA)),
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		colTotalInclude = TRUE, colTotalLab = "Placebo and treatment arm",
		dataTotalCol = dataAEInterest,
		stats = getStats("n (%)"), emptyValue = "0",
		rowVarLab = c(
			'AESOC' = "TEAE by SOC and Preferred Term\nn (%)"
		),
		dataTotal = dataTotalAE,
		labelVars = labelVars
	)

```

## Total across rows

### Inclusion

If the total should be included across elements of specific `rowVar`
variable(s), this(these) variable(s) should be included in `rowVarTotalInclude`.

```{r rowVarTotalInclude}

	# total reported across AESOC
	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarTotalInclude = "AESOC", 
		colVar = "TRTA",
		stats = getStats("n (%)"),
		labelVars = labelVars
	)
	
	# total reported across AESOC and across AEDECOD
	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarTotalInclude = c("AESOC", "AEDECOD"), 
		colVar = "TRTA",
		stats = getStats("n (%)"),
		labelVars = labelVars
	)

```

In case multiple row variables are specified, the total can also be included for
each of this variable. In this case, the total is by default included in the
header of each category of this variable.

### Label

For the first row variable, the total is included in the first row of the table,
with the label specified in `rowTotalLab`.

```{r rowTotalLab}

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarTotalInclude = "AESOC", rowTotalLab = "Any AE", 
		colVar = "TRTA",
		stats = getStats("n (%)"),
		labelVars = labelVars
	)

```

### Inclusion as separated category

The row total can also be included as a separated category ('Total') in the
table, if this variable is additionally specified in `rowVarTotalInSepRow`.

```{r rowVarTotalInclude-rowVarTotalInSepRow}

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarTotalInclude = "AEDECOD",
		rowVarTotalInSepRow = "AEDECOD",
		colVar = "TRTA",
		stats = getStats("n (%)"),
		labelVars = labelVars
	)

```

### Dataset

A different dataset considered for the row total is specified via the 
`dataTotalRow` parameter.

Different datasets can also be specified for each row variable separately (via a
named list).

For example, the worst-case severity per adverse event, per and across
system organ classes are displayed in the table below.

```{r dataTotalRow}

	dataAEInterest$AESEVN <- as.numeric(dataAEInterest$AESEV)
	
	# compute worst-case scenario per subject*AE term*treatment
	dataAEInterestWC <- ddply(dataAEInterest, c("AESOC", "AEDECOD", "USUBJID", "TRTA"), function(x){
		x[which.max(x$AESEVN), ]
	})

	## datasets used for the total: 
	# for total: compute worst-case across SOC and across AE term
	# (otherwise patient counted in multiple categories if present different categories for different AEs)
	dataTotalRow <- list(
		# within visit (across AEDECOD)
		'AEDECOD' = ddply(dataAEInterest, c("AESOC", "USUBJID", "TRTA"), function(x){	
			x[which.max(x$AESEVN), ]
		}),
		# across visits
		'AESOC' = ddply(dataAEInterest, c("USUBJID", "TRTA"), function(x){	
			x[which.max(x$AESEVN), ]
		})
	)

	getSummaryStatisticsTable(
		data = dataAEInterestWC,
		## row variables:
		rowVar = c("AESOC", "AEDECOD", "AESEV"), 
		rowVarInSepCol = "AESEV",
		# total for column header and denominator
		dataTotal = dataTotalAE, 
		# include total across SOC and across AEDECOD
		rowVarTotalInclude = c("AESOC", "AEDECOD"), 
		# data for total row
		dataTotalRow = dataTotalRow, 
		# count for each severity category for the total
		rowVarTotalByVar = "AESEV", 
		rowTotalLab = "Any TEAE", 
		rowVarLab = c(AESOC = "Subjects with, n(%):", AESEV = "Worst-case scenario"),
		# sort per total in the total column
		rowOrder = "total", 
		## column variables
		colVar = "TRTA", 
		stats = getStats("n (%)"),
		emptyValue = "0",
		labelVars = labelVars
	)

```

# Labels

If the data is loaded into R with the `read_haven` of the `haven` package,  or
the `loadDataADaMSDTM` function of the `clinUtils` package, the label  for each
variable is stored in the 'label' attribute of the corresponding  column.

However, if this label is lost (e.g. if the object is subsetted), labels can be
specified via the `labelVars` parameter for all variables at once, or via
specific `[parameter]Lab` parameter, as `rowVarLab`/`colVarLab`/`varLab` for the
row/column/variable to summarize respectively.

# Title and footnote

Title and footnote are specified via the corresponding `title` and `footer`
parameters. The convenient function `toTitleCase` from the `tools` package is
used to set title case for the title of the summary statistics table.

```{r titleAndFooter}

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		stats = getStats("n (%)"),
		dataTotal = dataTotalAE,
		labelVars = labelVars,
		title = toTitleCase("MOR106-CL-102: Adverse Events by System Organ Class and Preferred Term (Safety Analysis Set, Part 1)"),
		footer = c(
			"N=number of subjects with data; n=number of subjects with this observation",
			"Denominator for percentage calculations = the total number of subjects per treatment group in the safety population"
		)
	)

```

# Appendix

## Session information

```{r includeSessionInfo, echo = FALSE, results = "asis"}
print(sessionInfo())
```

---
title: "Creation of in-text tables"
author: "Laure Cougnaud"
date: "`r format(Sys.Date(), '%B %d, %Y')`"
output: 
  rmarkdown::html_document:
    toc: true
    toc_float: true
    toc_depth: 5
    number_sections: true
vignette: >
  %\VignetteIndexEntry{Create in-text tables}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

```{r options, echo = FALSE}
	
	library(knitr)
	opts_chunk$set(
		echo = TRUE, results = 'markup', warning = FALSE, 
		# stop document execution if error (not the default)
		error = FALSE, 
		message = FALSE, cache = FALSE,
		fig.width = 8, fig.height = 7,
		fig.path = "./figures_vignette/",
		fig.align = 'center')
	options(width = 170)
	# instead of warn = 0 by default
	# include warnings when they occur in the document
	options(warn = 1)
	
```


This vignette focuses on how to create in-text tables with the
`inTextSummaryTable` package.

In this vignette we assume you have ready the `data.frame`(s) to create the
tables. If you have doubts on the data format, please look the introductory
vignette at the section "data format".

We will use the example data available in the `clinUtils` package. Let's load
the packages and the data, and get started!


```{r loadPackages}

	library(inTextSummaryTable)
	library(tools) # toTitleCase

```

```{r loadData}	
	
	library(clinUtils)

	# load example data
    data(dataADaMCDISCP01)
	
	dataAll <- dataADaMCDISCP01
    labelVars <- attr(dataAll, "labelVars")
	
```


The **`getSummaryStatisticsTable`** creates an in-text table of summary
statistics for variable(s) of interest.

The _Demographic_ data (`ADSL` dataset) is used as example for the summary
statistics table.

```{r data-SL}

	dataSL <- dataAll$ADSL

```

# Variable(s) to summarize

Variable(s) to summarize in the table are specified via the **`var` parameter**.

Different set of statistics are reported depending on the type of variable:
[Categorical variable] or [Continuous variable].

See the documentation in section _Base statistics_ for more details on the
statistics included by default for each type, via:

```{r getHelp, eval = FALSE}

? `inTextSummaryTable-stats` 

```

## Categorical variable

For a **discrete/categorical variable**, the in-text table can display the
**counts/percentages of the number of subjects or records for each category** of
the variable.

### Counts of the entire dataset

If **no variable is specified** (via the `var` parameter), the counts are
displayed for the **entire dataset**.

```{r count-simple}

	getSummaryStatisticsTable(data = dataSL)

```

Please note that this is equivalent of setting (`var = 'all'`).

### Counts of categories

If a **variable is specified** (via the `var` parameter), the counts are
displayed **for each category**.

```{r count-categories}

	getSummaryStatisticsTable(data = dataSL, var = "SEX")

```

### Sort categories

The categories of the variable are sorted alphabetically by default. To sort the
categories in a specific order, the variable should be formatted as
**`factor`**, whose ordered categories are included in its **`levels`**.

```{r count-categories-order}

	# specify manually the order of the categories
	dataSL$SEX <- factor(dataSL$SEX, levels = c("M", "F"))
	getSummaryStatisticsTable(data = dataSL, var = "SEX")
	
	# order categories based on a numeric variable
    dataSL$SEXN <- ifelse(dataSL$SEX == "M", 2, 1)
	dataSL$SEX <- reorder(dataSL$SEX, dataSL$SEXN)
	getSummaryStatisticsTable(data = dataSL, var = "SEX")
	
```

### Inclusion of categories not available in the data

By default, the table only includes the categories present in the input data, to
ensure a compact table for CSR export.

```{r count-categories-empty-1}

	dataSLExample <- dataSL
	
	# 'SEX' formatted as character with only male
	dataSLExample$SEX <- "M" # only male
	getSummaryStatisticsTable(data = dataSLExample, var = "SEX")
	
```	

If extra categories should be represented in the table, the categorical variable
should be **formatted as a factor**, whose **levels contain all categories** to
be displayed in the table.

Furthermore, the parameter: `varInclude0` should be set to `TRUE` or to the
specific variable (in case multiple variables are specified) to indicate that
categories with 0 counts should be included.

```{r count-categories-empty-2}
	
	# 'SEX' formatted as factor, to include also female in the table
	# (even if not available in the data)
	dataSLExample$SEX <- factor("M", levels = c("F", "M"))
	getSummaryStatisticsTable(data = dataSLExample, var = "SEX", varInclude0 = TRUE)
	# or:
    getSummaryStatisticsTable(data = dataSLExample, var = "SEX", varInclude0 = "SEX")

```


### Count table for 'flag'-variables

A specific type of categorical variable is a **'flag variable'**, which indicates
if a record fulfills a specific criteria.

Such variable is typically formatted in the data as: 

* 'Y' if the criteria is met for the specific record
* 'N' if the criteria is not fulfilled for the specific record
* '' if the criteria is missing for this record

The name of such variable typically ends with **'FL'** in a CDISC-compliant _ADaM_
or _SDTM_ dataset.

For example, the subject-level dataset contains the following flag variables:

```{r flag-variables}

    labelVars[grep("FL$", colnames(dataSL), value = TRUE)]
    # has the subject discontinued from the study?
    dataSL$DISCONFL

```

If this variable is specified in `var`, the counts for each category is
reported:

```{r count-flag-var}

    getSummaryStatisticsTable(
	    data = dataSL,
	    var = "SAFFL"
    )

```

However, the interest is often to only reports the counts for the records
fulfilling the criteria (records with 'Y'). This is the case if the variable is
specified via the `varFlag` parameter too.

```{r count-flag-varFlag}

	getSummaryStatisticsTable(
		data = dataSL,
		var = "SAFFL",
		varFlag = "SAFFL"
	)

```

### Inclusion of total across categories

To include the total counts across categories, the `varTotalInclude` parameter
should be set to `TRUE` (or to the specific variable).

```{r count-varTotalInclude}

	getSummaryStatisticsTable(
		data = dataSL, 
		var = "SEX", 
		varTotalInclude = TRUE
	)

```

## Continuous variable

For a **continuous variable**, the in-text table displays **standard
distribution statistics** of the variable.
  
Please note that **missing records (NA) for the variable are filtered**, so the
**count statistics** (number of subjects, records, percentage) are based **only
on the non missing records**.
  
For a continuous variable, the presence of different values for the same subject 
(and across row/column variables) are checked and an appropriate error message
is returned if multiple different values are available.

```{r numeric}

	getSummaryStatisticsTable(data = dataSL, var = "AGE")

```

## Continuous and categorical variables in the table

The table can contain a mix of categorical and continuous variables.

```{r mixedTable}

	getSummaryStatisticsTable(
		data = dataSL, 
		var = c("AGE", "SEX")
	)

```

# Statistics of interest

Statistics of interest and their format are specified via the **`stats`
parameter**.

If an unique statistic expression is specified, the 'Statistic' column doesn't
appear in the table.  
In case multiple statistics are specified, these are
included as separated row.

## Standard statistic set

A standard set of statistics is specified via specific tags to be passed to the
`stats` function.

The list of available statistics is mentioned in the section '_Formatted
statistics_' in:

```{r getHelpStats, eval = FALSE}

    ? `inTextSummaryTable-stats` 

```

Please see below examples of commonly used statistics.

### Categorical table

```{r stats-count}

	# count: n, '%' and m
	getSummaryStatisticsTable(
		data = dataSL,
		var = "SEX",
		stats = "count"
	)

	# n (%)
	getSummaryStatisticsTable(
		data = dataSL,
		var = "SEX",
		stats = "n (%)"
	)
	
	# n/N (%)
	getSummaryStatisticsTable(
		data = dataSL,
		var = "SEX",
		stats = "n/N (%)"
	)
	
```

### Continuous variable

```{r stats-numeric}
	
	## continuous variable
	
	# all summary stats
	getSummaryStatisticsTable(
		data = dataSL,
		var = "AGE",
		stats = "summary"
	)
	
	# median (range)
	getSummaryStatisticsTable(
		data = dataSL,
		var = "AGE",
		stats = "median (range)"
	)
	
	# median and (range) in a different line:
	getSummaryStatisticsTable(
		data = dataSL,
		var = "AGE",
		stats = "median\n(range)"
	)
	
	# mean (se)
	getSummaryStatisticsTable(
		data = dataSL,
		var = "AGE",
		stats = "mean (se)"
	)
	
	# mean (sd)
	getSummaryStatisticsTable(
		data = dataSL,
		var = "AGE",
		stats = "mean (sd)"
	)

```

## Custom statistics formatting (Advanced)

To change the formatting of the statistics, the `stats` parameter should contain
a language object (e.g. `expression` or `call`) of the default base set of
statistics.

See the documentation in section '_Base statistics_' for more details on the base
statistics included by default, via:

```{r getHelpStats2, eval = FALSE}

? `inTextSummaryTable-stats` 

```

For example, the following count table is restricted to the number of subjects
per categories:

```{r stats-N}

	getSummaryStatisticsTable(
		data = dataSL,
		var = c("RACE", "SEX"),
		stats = list(N = expression(statN))
	)

```

The summary statistics table is restricted to the median and range:

```{r stats-meanSE}

	getSummaryStatisticsTable(
		data = dataSL,
		var = c("AGE", "HEIGHTBL", "WEIGHTBL", "BMIBL"),
		varGeneralLab = "Parameter", statsGeneralLab = "",
		colVar = "TRT01P",
		stats = list(
			`median` = expression(statMedian),
			`(min, max)` = expression(paste0("(", statMin, ",", statMax, ")"))
		)
	)

```

Note that the 'Standard statistics set' is formatted internally via the
`getStatsData` (and `getStats`) functions, which creates consistently a list of
`language` objects.

```{r getStatsData}

    # this count table:
    getSummaryStatisticsTable(
	    data = dataSL,
	    var = "SEX",
	    stats = "count"
    )
    # ... is equivalent to:
    getSummaryStatisticsTable(
	    data = dataSL,
	    var = "SEX",
	    stats = getStats(type = "count")
    )

    # this summary table...
    getSummaryStatisticsTable(
	    data = dataSL,
	    var = "AGE",
	    stats = "mean (se)"
    )
    # ... is equivalent to:
    getSummaryStatisticsTable(
	    data = dataSL,
	    var = "AGE",
	    stats = getStatsData(type = "mean (se)", var = "AGE", data = dataSL)[["AGE"]]
    )
```

## Statistics by variable/group

The statistics can also be provided for each variable separately, if `stats` is
named by variable:

```{r statExtra-EachVariable}
		
	getSummaryStatisticsTable(
		data = dataSL, 
		var = c("AGE", "RACE"),
		stats = list(
			AGE = getStats("median (range)"),
			RACE = getStats("n (%)")
		)
	)

```

## Extra statistics

Extra statistics (not available in the default set of statistics) should be
specified via the `statsExtra` parameter.

A set of extra utility functions to compute common extra statistics are also
available in the package:

* coefficient of variation with the `cv` function
* geometric mean with the `geomMean` function
* geometric standard deviation with the `geomSD` function
* geometric coefficient of variation with the `geomCV` function

```{r statsUtilityFct}

	getSummaryStatisticsTable(
		data = dataSL,
		var = "HEIGHTBL",
		# specify extra stats to compute
		statsExtra = list(
			statCV = cv,
			statGeomMean = geomMean,
			statGeomSD = geomSD,
			statsGeomCV = geomCV
		)
	)

```

Full customized statistics can also be provided. For example, if you would like
to specify your own formula for the coefficient of variation:

```{r statsExtra}

	# include the coefficient of variation via the 'statsExtra' parameter
	getSummaryStatisticsTable(
		data = dataSL,
		var = "HEIGHTBL",
		statsExtra = list(statCVPerc = function(x) sd(x)/mean(x)*100)
	)

```

These statistics are then available for customization via the `stats` parameter.

```{r statsExtra-stats}

	# format the statistics with the 'stats' parameter
	getSummaryStatisticsTable(
		data = dataSL,
		var = "HEIGHTBL",
		statsExtra = list(statCVPerc = function(x) sd(x)/mean(x)*100),
		stats = list(Mean = expression(statMean), 'CV%' = expression(statCVPerc))
	)


```

## Rounding strategy

Please note that all statistics are rounded by default 
in the package based on the 
**'rounding up' strategy for rounding off a 5**, 
which **differs from the default rounding strategy in R** (`round` function).

This was a deliberate choice to reproduce summarized statistics 
created with the SAS software.

Please find more explanations in the documentation of the 
`? roundHalfUp` and `? roundHalfUpTextFormat` functions.

## Number of decimals

The detailed rules for the number of decimals for the statistics
are described in the section _Statistics formatting_ in:

```{r getHelpStats3, eval = FALSE}

    ? `inTextSummaryTable-stats` 

```

To specify fixed amounts of digits for the statistics to be displayed in the
table, the statistics are formatted in the `stats` parameter.

### Default number of decimals

#### Categorical variable

The percentages are formatted by default as specified in the table below.

```{r nDecimals-catVar, echo = FALSE, fig.cap = "Standard Layout for Frequency Tabulations of Categorical Variables<br>"}

include_graphics("./images/nDecimals_catVar.png")

```

By default, the counts for a categorical variables are formatted as specified
above:

* the number of subjects is displayed with 0 digits (`nDecN` is set to 0)
* the frequency percentage is implemented in the `formatPercentage` function

```{r getStats-count}

	# Internal rule for the number of decimals for the percentage
	formatPercentage(c(NA, 0, 100, 99.95, 0.012, 34.768))
	
	# Used by default in the 'getStats' function
	getStats(type = "count")

```

#### Continuous variable {#numberDecimalsContinuousVariable}

The number of decimals for statistics based on a continuous variable is by default
as specified in the tables below.

```{r nDecimals-numVar, echo = FALSE, fig.cap = "Standard Layout for Descriptive Statistics of Continuous Variables<br>"}

include_graphics("./images/nDecimals_numVar.png")

```

In the package: 'Very small values' are considered values below 1.

When specifying the default set of available statistics with the `getStats`
function, and **only if the variable is specified** (`x` parameter), the number
of decimals for a continuous variable is determined by:

1. Extracting the number of decimals for individual values based on:
    + **pre-defined rules** based on the number of decimals of the individual
      values (`getNDecimalsRule` function)
    + the number of decimals **available in the input data** via the
      `getNDecimalsData` function
    + taking the **minimum of these two criterias** (`getNDecimals` function), 
    such as the number of decimals according  the rule won't be higher that the
      actual number of decimals available in the data
2. Taking the **maximum number of decimals** across all individual values via
   the `getMaxNDecimals` function, which is used as 'base' number of decimals
   considered for the summary statistics
3. The actual number of decimals for each statistic is extracted 
by adding to the 'base' number of decimals:
    + **0 extra decimal for the minimum, maximum**
    + **1 extra decimal for the mean, median, sd**
    + **2 extra decimals for SE**

Please note that if a different framework than implemented in steps 1 and 2
should be used for the extraction of the number of decimals for a specific
variable, the number of decimals of interest can be fixed via the `nDecCont`
parameter.

```{r getMaxNDecimals}

	# Duration of Disease (Months)
	print(dataSL$DURDIS)
	
	## Extract the number of decimals for each value:
	
	# based on pre-defined rule, this metric should be displayed with 1 decimal:
	getNDecimalsRule(x = dataSL$DURDIS)
	
	# but available in the data only with 0 decimals
	getNDecimalsData(x = dataSL$DURDIS)
	
	# The minimum of the #decimals based on the data and pre-defined rule is:
	getNDecimals(x = dataSL$DURDIS)
	
	## Take the maximum number of decimals 
	getMaxNDecimals(x = dataSL$DURDIS)
	
	## Custom set of statistics are extracted when x is specified:
	getStats(x = dataSL$DURDIS)
	
	# To fix the number of decimals:
	getStats(type = "summary", nDecCont = 1)
	
	## Create summary statistics table
	getSummaryStatisticsTable(
		data = dataSL,
		var = c("AGE", "DURDIS"),
		stats = list(
			AGE = getStats(type = "median (range)", x = dataSL$AGE),
            DURDIS = getStats(type = "median (range)", x = dataSL$DURDIS)
		)
	)

```

### Custom `stats` function (Advanced)

A  custom function can be created to create custom statistics with fixed number
of digits.

For example, the AGE is displayed with 1 digit and the height with two digits:

```{r stats-digits}

	getSummaryStatisticsTable(
		data = dataSL,
		var = c("AGE", "HEIGHTBL"),
		stats = list(
			AGE = list(Median = expression(roundHalfUpTextFormat(statMedian, 1))),
            HEIGHTBL = list(Median = expression(roundHalfUpTextFormat(statMedian, 2)))
		)
	)

```

To create the `stats` parameter for a specific number of digits, a custom
function can be created:

```{r stats-digits-complex}

	# wrapper function to include median with specific number of digits
	# and min/max with specified number of digits - 1
	statsDMNum <- function(digitsMin)
		list('Median (range)' = 
			bquote(paste0(
				roundHalfUpTextFormat(statMedian, .(digitsMin+1)), 
				" (", roundHalfUpTextFormat(statMin, .(digitsMin)), ",", 
				roundHalfUpTextFormat(statMax, .(digitsMin)),
				")"
			))
	)

	getSummaryStatisticsTable(
		data = dataSL,
		var = c("AGE", "HEIGHTBL", "WEIGHTBL", "BMIBL", "RACE", "SEX"),
		stats = list(
			AGE = statsDMNum(0),
			HEIGHTBL = statsDMNum(1),
			WEIGHTBL = statsDMNum(1),
			BMIBL = statsDMNum(1),
			RACE = getStats("n (%)"),
			SEX = getStats("n (%)")
		)
	)

```

## Statistics layout

The layout of the statistics is specified via the `statsLayout` parameter.  

By default, the statistics are included in rows within each variable.

```{r statsLayoutRow}

	# statsLayout = 'row'
	getSummaryStatisticsTable(
		data = dataSL,
		var = c("AGE", "HEIGHTBL"),
		stats = list(Mean = expression(statMean), 'SE' = expression(statSE))
	)
	
```

The statistics can also be included in columns.

```{r statsLayoutCol}

	getSummaryStatisticsTable(
		data = dataSL,
		var = c("AGE", "HEIGHTBL"),
		stats = list(Mean = expression(statMean), 'SE' = expression(statSE)),
		statsLayout = "col"
	)

```

The statistics can also be specified in different rows, but in a separated
column.

```{r summaryTable-PP-medianMinMax-statsLayoutRowVarInSepCol}

	getSummaryStatisticsTable(
		data = dataSL,
		var = c("AGE", "HEIGHTBL"),
		stats = list(Mean = expression(statMean), 'SE' = expression(statSE)),
		statsLayout = "rowInSepCol"
	)

```

By default, if only one statistic is available in the table, the name of the
statistic is not included in the rows/columns, as the statistic is generally
described in this case in the title of the table.

```{r summaryTable-statsLayout-onlyOneStat}

	getSummaryStatisticsTable(
		data = dataSL,
		var = c("AGE", "HEIGHTBL"),
		stats = list(Mean = expression(statMean))
	)

```

To include even in this case the name of the statistic, the parameter
`statsLabInclude` should be set to `TRUE`.

```{r summaryTable-statsLayout-onlyOneStat-statsLabInclude}

	getSummaryStatisticsTable(
		data = dataSL,
		var = c("AGE", "HEIGHTBL"),
		stats = list(Mean = expression(statMean)),
		statsLabInclude = TRUE
	)

```

# Table layout

The general table layout is driven by the specification of variables to be
displayed in rows (in the vertical direction) or in columns (in the horizontal
direction).

If no variables are specified in `var`, counts across row/column variable are
displayed.

The adverse events dataset is used for demonstration.

```{r countTable-AE-data}
				
	dataAE <-  subset(dataAll$ADAE, SAFFL == "Y" & TRTEMFL == "Y")
	
	# ensure that order of elements is the one specified in 
	# the corresponding numeric variable
	dataAE$TRTA <- with(dataAE, reorder(TRTA, TRTAN))
	dataAE$AESEV <- factor(
		dataAE$AESEV, 
		levels = c("MILD", "MODERATE", "SEVERE")
	)
	
	dataAEInterest <- subset(dataAE, AESOC %in% c(
		"INFECTIONS AND INFESTATIONS",
        "GENERAL DISORDERS AND ADMINISTRATION SITE CONDITIONS"
       )
    )

```

## Row and column variables

Specific grouping variable(s) for the columns can be specified via the
**`colVar`** parameter and for the rows via the **`rowVar`** parameter.

If multiple category variables are specified, they should be specified in
hierarchical order.

```{r rowVarColVar}

	# unique row variable
	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = "AEDECOD",
		stats = getStats("n (%)"),
		labelVars = labelVars
	)
	
	# multiple nested row variables
	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		stats = getStats("n (%)"),
		labelVars = labelVars
	)
	
	# unique column variable
	getSummaryStatisticsTable(
		data = dataAEInterest,
		colVar = "TRTA",
		stats = getStats("n (%)"),
		labelVars = labelVars
	)
	
	# combination of rows and columns
	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		stats = getStats("n (%)"),
		labelVars = labelVars,
		colHeaderTotalInclude = FALSE
	)

```

## Row variable

By default (when `outputType` is set to: 'flextable'), if multiple row variables
are specified, they are considered nested and displayed in the first column of
the final table. Each sub-category is indicated with a specific indent
(customizable with `rowVarPadBase`).

### Variable in separated column

**Row variables** that should be included as a **separated column** should be
specified via the `rowVarInSepCol` parameter.

```{r rowVarInSepCol}

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD", "AESEV"),
		rowVarInSepCol = "AESEV",
		colVar = "TRTA",
		stats = getStats("n (%)"),
		labelVars = labelVars
	)
	
```

### Row ordering

The **categories in the row variables can be ordered** based on the
**`rowOrder`** variable. 

This variable is either:

* a string with the name of an implemented method to order the rows, among:
     + `alphabetical`: categories are ordered **alphabetically**
     + `auto`: categories are ordered based on the **levels** if the input
       variable is a factor,    alphabetically otherwise
     + `total`: categories are ordered based on the 'total' column (see section
       \@ref(colTotal))    (if the total column is not included in the table)
* a custom ordering function to apply in the data to order the rows

#### Common order for all row variables

```{r rowOrder-common}

	# 'auto':

	# set order of SOC to reverse alphabetical order
	dataAEInterest$AESOC <- factor(
		dataAEInterest$AESOC, 
		levels = rev(sort(unique(as.character(dataAEInterest$AESOC))))
	)
	# AEDECOD is not a factor -> sort alphabetically by default
	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarLab = labelVars[c("AEDECOD")],
		rowVarTotalInclude = c("AESOC", "AEDECOD"),
		colVar = "TRTA", colTotalInclude = TRUE,
		stats = getStats("n (%)"),
		labelVars = labelVars
	)
	
	# total counts
	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarLab = labelVars[c("AEDECOD")],
		rowVarTotalInclude = c("AESOC", "AEDECOD"),
		colVar = "TRTA", colTotalInclude = TRUE, colTotalLab = "Number of subjects",
		rowOrder = "total",
		stats = getStats("n (%)"),
		labelVars = labelVars
	)
	# same order even if the 'total' column is not specified
	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarLab = labelVars[c("AEDECOD")],
		rowVarTotalInclude = c("AESOC", "AEDECOD"),
		colVar = "TRTA", 
		rowOrder = "total", 
		stats = getStats("n (%)"),
		labelVars = labelVars
	)

```

#### Different orders for each row variable

In case the order should be different for each row variable, a named list is
provided for the `rowVar` parameter.

```{r rowOrder-specific}

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarLab = labelVars[c("AEDECOD")],
		rowVarTotalInclude = c("AESOC", "AEDECOD"),
		colVar = "TRTA", #colTotalInclude = TRUE,
		rowOrder = c(AESOC = "alphabetical", AEDECOD = "total"),
		stats = getStats("n (%)"),
		labelVars = labelVars
	)

```

#### Row order based on the total of a column category

If the row categories should be **ordered by total counts for a specific
category of the column variable(s)**, a function **`rowOrderTotalFilterFct`** is
specified.

The adverse events are sorted based on the incidence in the treated group.

```{r rowOrder-rowOrderTotalFilterFct}

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarLab = labelVars[c("AEDECOD")],
		rowVarTotalInclude = c("AESOC", "AEDECOD"),
		colVar = "TRTA", colTotalInclude = TRUE,
		rowOrder = "total",
		stats = getStats("n (%)"),
		labelVars = labelVars,
		# consider only the counts of the treated patients to order the rows
		rowOrderTotalFilterFct = function(x) subset(x, TRTA == "Xanomeline High Dose")
	)

```

#### Row order based on a custom specified function

If the method to order the rows is more complex, the `rowOrder` parameter
specifies a function taking the summary table as input and returning the order
levels of the elements in the row variable.

For example, the adverse event table is sorted based on the counts of patient
presenting this event across all treatment classes, and in case of ties based on 
the counts of treated-patients presenting this event.

```{r rowOrder-functionExample1}

	library(plyr)
	getSummaryStatisticsTable(
		data = dataAEInterest,
		type = "count",
		rowVar = "AEHLT",
		rowOrder = function(x){
			x <- subset(x, !isTotal)
			totalAcrossTreatments <- subset(x, TRTA == "Total")
			# counts across treated patients
			totalForTreatmentOnly <- subset(x, TRTA == "Xanomeline High Dose")
			dataCounts <- merge(totalAcrossTreatments, totalForTreatmentOnly, by = "AEHLT", suffixes = c(".all", ".treat"))
			# sort first based on overall count, then counts of treated patients
			dataCounts[with(dataCounts, order(`statN.all`, `statN.treat`, decreasing = TRUE)), "AEHLT"]
		},
		colVar = "TRTA", colTotalInclude = TRUE,
		labelVars = labelVars,
		title = "Table: Adverse Events ordered based on total counts",
		stats = list(expression(paste0(statN, " (", round(statPercN, 1), ")"))),
		footer = "Statistics: n (%)"
	)

```

The adverse event table is now ordered based on the counts in the placebo, then
treated-patients column, for the organ class and the adverse event term
separately.

```{r rowOrder-functionExample2}

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarLab = labelVars[c("AEDECOD")],
		rowVarTotalInclude = c("AESOC", "AEDECOD"),
		colVar = "TRTA", colTotalInclude = TRUE,
		rowOrder = list(
			AESOC = function(table) {
				# records with total for each AESOC
				nAESOCPlacebo <- subset(table, !isTotal & grepl("placebo", TRTA) & AEDECOD == "Total")
				nAESOCTreat <- subset(table, !isTotal & grepl("High Dose", TRTA) & AEDECOD == "Total")
				nAESOCDf <- merge(nAESOCPlacebo, nAESOCTreat, by = "AESOC", suffixes = c(".placebo", ".treatment"))
				nAESOCDf[with(nAESOCDf, order(`statN.placebo`, `statN.treatment`, decreasing = TRUE)), "AESOC"]
			},
			AEDECOD = function(table) {
				# records with counts for each AEDECOD
				nAEDECODPlacebo <- subset(table, !isTotal & grepl("placebo", TRTA) & AEDECOD != "Total")
				nAEDECODTreat <- subset(table, !isTotal & grepl("High Dose", TRTA) & AEDECOD != "Total")
				nAEDECODDf <- merge(nAEDECODPlacebo, nAEDECODTreat, by = "AEDECOD", suffixes = c(".placebo", ".treatment"))
				nAEDECODDf[with(nAEDECODDf, order(`statN.placebo`, `statN.treatment`, decreasing = TRUE)), "AEDECOD"]
			}
		),
		stats = getStats("n (%)"),
		labelVars = labelVars
	)

```

### Row variable labels

#### Based on dataset

The **labels used for the variables parameter** (row variables) **are
automatically extracted from the labels** contained in the _SAS_ dataset, by
specifying the `labelVars` parameter.

```{r summaryTable-PP-rowVarWithLabel-labelVars}

	# combination of rows and columns
	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		stats = getStats("n (%)"),
		labelVars = labelVars
	)
	
```

#### Custom

The label can also be specified directly via the `rowVarLab` parameter, for each
variable in `rowVar`.

If an unique row label should be used (even if multiple row variables are
specified), `rowVarLab` is set to this unique label.

```{r summaryTable-PP-rowVarWithLabel-rowVarLab}

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		stats = getStats("n (%)"),
		rowVarLab = c(
			'AESOC' = "TEAE by SOC and Preferred Term\nn (%)"
		),
		labelVars = labelVars
	)
	
```

### Inclusion of row/column categories not available in the data

As for the variable to summarize, to include categories in the row or column
variables not available in the data, these variables should be formatted as a
factor with categories specified in its levels.

Furthermore, the parameters `rowInclude0` and `colInclude0` should be set to
TRUE to include counts for empty categories within the row/column. 

```{r countTable-emptyVars}

	## only consider a subset of adverse events
	dataAESubset <- subset(dataAE, AEHLT == "HLT_0617")
	
	## create dummy categories for:
	# treatment
	dataAESubset$TRTA <- with(dataAESubset, 
		factor(TRTA, levels = c(unique(as.character(TRTA)), "Treatment B"))
	)
	# low-level term category
	dataAESubset$AELLT <- with(dataAESubset, 
		factor(AELLT, levels = c(unique(as.character(AELLT)), "Lymphocyte percentage increased"))
	)
	
	# create summary statistics table
	getSummaryStatisticsTable(
		data = dataAESubset,
		type = "count",
		rowVar = c("AEHLT", "AELLT"),
		rowInclude0 = TRUE, colInclude0 = TRUE,
		colVar = "TRTA",
		labelVars = labelVars,
		title = "Table: Adverse Events: white blood cell analyses",
		stats = getStats("n (%)"),
		footer = "Statistics: n (%)"
	)

```


## Variable(s) to summarize

### Default

The **variable(s) used for the summary statistics** (`var`) are included **by
default in rows**.

```{r summaryTable-layout-var}

    dataDIABP <- subset(dataAll$ADVS, 
		SAFFL == "Y" & ANL01FL == "Y" &
		PARAMCD == "DIABP" & 
		AVISIT %in% c("Baseline", "Week 8") &
		ATPT == "AFTER LYING DOWN FOR 5 MINUTES"
	)
    dataDIABP$TRTA <- reorder(dataDIABP$TRTA, dataDIABP$TRTAN)
    dataDIABP$AVISIT <- reorder(dataDIABP$AVISIT, dataDIABP$AVISITN)
    
	getSummaryStatisticsTable(
		data = dataDIABP,
		var = c("AVAL", "CHG"),
		colVar = "TRTA",
		rowVar = "AVISIT",
		labelVars = labelVars,
		stats = getStats("summary-default")
	)

```

### Summary variable in columns

In case multiple variables are to be summarized, the different variables can be
included in different columns by including the specific label: 'variable' in
`colVar`. Beware that such layout only makes sense for variables with similar
types (e.g. all numeric variables).

```{r summaryTable-layout-varInColumn}
    
getSummaryStatisticsTable(
	data = dataDIABP,
	var = c("AVAL", "CHG"),
	colVar = c("variable", "TRTA"),
	rowVar = "AVISIT",
	labelVars = labelVars,
	stats = getStats("summary-default")
)

```

### Inclusion of summary variables in case one variable is specified

By default, the variable label is not included if only one summary statistic
variable is specified.

```{r summaryTable-layout-var-oneLabel-default}
		
	getSummaryStatisticsTable(data = dataSL, var = "AGE", colVar = "TRT01P")

```

To include the label in case only one summary statistic variable is specified,
the parameter `varLabInclude` should be set to TRUE.

```{r summaryTable-layout-var-oneLabel-varLabInclude}
		
	getSummaryStatisticsTable(
		data = dataSL, 
		var = "AGE", 
		varLabInclude = TRUE,
		colVar = "TRT01P"
	)

```

## Inclusion of the counts per group in case of missing values

It might be of interest to display the counts of all subjects per row/column
variable in association of the summary statistic of a variable of interest.

For example it could be of interest to report the total number of subjects per
group, which could differ from the total number of subjects for a variable of
interest if this variable contain missing values.

```{r summaryTable-layout-var-empty}

    dataAEInterest$AESEVN <- ifelse(dataAEInterest$AESEV == "MILD", 1, 2)
	dataAEInterestWC <- ddply(dataAEInterest, c("AEDECOD", "USUBJID", "TRTA"), function(x) {
		x[which.max(x$AESEVN), ]
	})
	dataAEInterestWC[1, "AESEV"] <- NA
    getSummaryStatisticsTable(
        data = dataAEInterestWC,
        colVar = "TRTA",
        rowVar = "AEBODSYS",
		stats = getStats("n (%)"),
        var = c("AESEV", "all"),
		labelVars = labelVars
    )

```

# Total

The summary table contains **different types of total**:

* total used for the **percentage computation** displayed in the table.  
For example: report percentage of subjects with specific adverse event.
* total reported in the **column header**  
For example: total number of subjects
  for a specific treatment arm.
* **total across rows**, reported in the row header   
For example: to report percentage of subjects with adverse events in a specific
  body system (across adverse events).
* **total across columns**, reported in a separated column  
For example: to report summary statistics across all treatments arms.

By default, the totals are extracted based on the input data, but separated
datasets can be specified for the header,  percentage computation, row or column
total.

## Summary

The different types of total of the summary table are summarized below:

| Type | Inclusion in the table |  Dataset: parameter name | Dataset: default | 
|---|---|---|---| 
| Total in the column header | Yes by default <br/> removed if `colHeaderTotalInclude = FALSE`| `dataTotal` | `data` for table content <br/>`dataTotalCol` for total column| 
| Total for the percentage | Only if percentage requested in `stats` | `dataTotalPerc` | `dataTotal` for table content <br/> `dataTotalCol` for total column <br/> (for 'total' if specified as a list)| 
|Total across rows | Not by default <br/> for specified row variable with `rowVarTotalInclude` | `dataTotalRow`  | `data` for table content <br/> `dataTotalCol` for total column <br/> (for 'total' if specified as a list) | 
|Total across columns | Not by default <br/> only if `colTotalInclude = TRUE` | `dataTotalCol` | `data` |

## Total for the column header

### Current datasset

By default, the total reported in the total header is extracted from the
**available number of subjects** in the input `data`.

For example, the total number of patients per treatment arm is extracted from
the subject-level (`ADSL`) dataset. 

```{r header-total-default}
	
	# by default, total number of subjects extracted from data
	getSummaryStatisticsTable(
		data = subset(dataAEInterest, AESOC == "INFECTIONS AND INFESTATIONS"),
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		stats = getStats("n (%)"),
		rowVarLab = c(
			'AESOC' = "TEAE by SOC and Preferred Term\nn (%)"
		),
		labelVars = labelVars
	)
	
```

### External dataset

 If the total should be extracted from a **different dataset**,  it should be
specified via the **`dataTotal`** variable.  Please note that by default
`dataTotal` is also used for  the computation of the percentage.

```{r header-total-dataTotal}	
	
	# dataset used to extract the 'Total'
	dataTotalAE <- subset(dataAll$ADSL, SAFFL == "Y")
	# should contain columns specified in 'colVar'
	dataTotalAE$TRTA <- dataTotalAE$TRT01A 

	getSummaryStatisticsTable(
		data = subset(dataAEInterest, AESOC == "INFECTIONS AND INFESTATIONS"),
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		stats = getStats("n (%)"),
		rowVarLab = c(
			'AESOC' = "TEAE by SOC and Preferred Term\nn (%)"
		),
		dataTotal = dataTotalAE,
		labelVars = labelVars
	)

```

## Remove total in column header

The total number of subjects in each column is by default included. This is not
displayed if `colHeaderTotalInclude` is set to FALSE.

```{r header-total-colHeaderTotalInclude}

	getSummaryStatisticsTable(
		data = subset(dataAEInterest, AESOC == "INFECTIONS AND INFESTATIONS"),
		rowVar = c("AESOC", "AEDECOD"),
		rowVarTotalInclude = "AEDECOD",
		rowVarTotalInSepRow = "AEDECOD",
		colVar = "TRTA",
		stats = getStats("n (%)"),
		rowVarLab = c(
			'AESOC' = "TEAE by SOC and Preferred Term\nn (%)"
		),
		dataTotal = dataTotalAE,
		labelVars = labelVars,
		colHeaderTotalInclude = FALSE
	)

```

## Percentage

### Dataset

A different dataset used for the computation of the percentage can be specified
via the **`dataTotalPerc`** parameter.  

```{r dataTotalPerc}

    getSummaryStatisticsTable(
	    data = subset(dataAEInterest, AESOC == "INFECTIONS AND INFESTATIONS"),
	    rowVar = c("AESOC", "AEDECOD"),
	    colVar = "TRTA",
	    stats = getStats("n (%)"),
	    rowVarLab = c(
		    'AESOC' = "TEAE by SOC and Preferred Term\nn (%)"
	    ),
	    dataTotalPerc = dataTotalAE,
	    labelVars = labelVars
    )

```

Please note that by default, if `dataTotalPerc` is specified, but not
`dataTotal`, counts reported in the column header are still extracted from
`data`.

### Variables to compute percentage by

If the total number of subjects differ between the components of the table, the
extra row/column(s) variable(s) are specified via
`colVarTotalPerc`/`rowVarTotalPerc`.

For example, in a table of laboratory measurements per reference range
(laboratory abnormalities): the
total number of subjects for the computation of the percentage are extracted
based on the number of subjects with available measurements per visit.

```{r rowVarTotalPerc}

	dataLB <- subset(dataAll$ADLBC, 
		SAFFL == "Y" & 
		PARAMCD %in% c("K", "CHOL") &
		grepl("(Baseline)|(Week 20)", AVISIT)
	)
	dataLB$AVISIT <- with(dataLB, reorder(trimws(AVISIT), AVISITN))
	
	# counts versus the total per actual treatment arm
	getSummaryStatisticsTable(
		data = dataLB,
		colVar = "TRTA", 
		rowVar = c("PARAM", "AVISIT"), 
		var = "LBNRIND",
		stats = getStats("n (%)"),
		rowAutoMerge = FALSE, emptyValue = "0",
	)
	
	# percentage based on total number of subjects with available
	# measurement at specific visit for each parameter
	getSummaryStatisticsTable(
		data = dataLB,
		colVar = "TRTA", 
		rowVar = c("PARAM", "AVISIT"), 
		rowVarTotalPerc = c("PARAM", "AVISIT"),
		var = "LBNRIND",
		stats = getStats("n (%)"),
		rowAutoMerge = FALSE, emptyValue = "0",
	)	
	
```

Please note the different percentage for the number of patients
with normal cholesterol measurements at week 20 between
the two tables.

### Percentage of the number of records

By default, the percentage is based on the number of subjects.  

If the percentage should be computed based on the number of records instead, the
parameter: `statsPerc` should be set to `statm` (`statN` by default).

For example, to extract the percentage of laboratory measurements by reference
range and parameter:

```{r statsPerc}

getSummaryStatisticsTable(
	data = dataLB,
	colVar = "TRTA", 
	rowVar = c("PARAM", "AVISIT"), 
	rowVarTotalPerc = c("PARAM", "AVISIT"),
	var = "LBNRIND", 
	stats = getStats("m (%)"),
	statsPerc = "statm",
	rowAutoMerge = FALSE, emptyValue = "0",
)

```

## Total across columns {#colTotal}

### Inclusion

The total **across all columns** is included if the `colTotalInclude` is set to
TRUE.

```{r colTotalInclude}

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		colTotalInclude = TRUE, 
		stats = getStats("n (%)"),
		rowVarLab = c(
			'AESOC' = "TEAE by SOC and Preferred Term\nn (%)"
		),
		dataTotal = dataTotalAE,
		labelVars = labelVars
	)

```

By default, the total number of subjects is extracted based on the input dataset
across columns: subjects presenting the **same event in multiple column(s) are
counted once** in the column total (e.g. for adverse event table in a context of
cross-over experiment).

### Label

This column is by default labelled 'Total', but this can be customized with the
`colTotalLab` parameter.

```{r colTotalLab}

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		colTotalInclude = TRUE, colTotalLab = "All subjects",
		stats = getStats("n (%)"),
		rowVarLab = c(
			'AESOC' = "TEAE by SOC and Preferred Term\nn (%)"
		),
		dataTotal = dataTotalAE,
		labelVars = labelVars
	)

```

### Dataset

A different dataset for the total column can also be specified via the
`dataTotalCol` parameter.

For example, the table is restricted to only the treatment arm, but both arms
are considered in the total column:

```{r dataTotalCol}

	getSummaryStatisticsTable(
		data = subset(dataAEInterest, grepl("High Dose", TRTA)),
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		colTotalInclude = TRUE, colTotalLab = "Placebo and treatment arm",
		dataTotalCol = dataAEInterest,
		stats = getStats("n (%)"), emptyValue = "0",
		rowVarLab = c(
			'AESOC' = "TEAE by SOC and Preferred Term\nn (%)"
		),
		dataTotal = dataTotalAE,
		labelVars = labelVars
	)

```

## Total across rows

### Inclusion

If the total should be included across elements of specific `rowVar`
variable(s), this(these) variable(s) should be included in `rowVarTotalInclude`.

```{r rowVarTotalInclude}

	# total reported across AESOC
	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarTotalInclude = "AESOC", 
		colVar = "TRTA",
		stats = getStats("n (%)"),
		labelVars = labelVars
	)
	
	# total reported across AESOC and across AEDECOD
	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarTotalInclude = c("AESOC", "AEDECOD"), 
		colVar = "TRTA",
		stats = getStats("n (%)"),
		labelVars = labelVars
	)

```

In case multiple row variables are specified, the total can also be included for
each of this variable. In this case, the total is by default included in the
header of each category of this variable.

### Label

For the first row variable, the total is included in the first row of the table,
with the label specified in `rowTotalLab`.

```{r rowTotalLab}

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarTotalInclude = "AESOC", rowTotalLab = "Any AE", 
		colVar = "TRTA",
		stats = getStats("n (%)"),
		labelVars = labelVars
	)

```

### Inclusion as separated category

The row total can also be included as a separated category ('Total') in the
table, if this variable is additionally specified in `rowVarTotalInSepRow`.

```{r rowVarTotalInclude-rowVarTotalInSepRow}

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		rowVarTotalInclude = "AEDECOD",
		rowVarTotalInSepRow = "AEDECOD",
		colVar = "TRTA",
		stats = getStats("n (%)"),
		labelVars = labelVars
	)

```

### Dataset

A different dataset considered for the row total is specified via the 
`dataTotalRow` parameter.

Different datasets can also be specified for each row variable separately (via a
named list).

For example, the worst-case severity per adverse event, per and across
system organ classes are displayed in the table below.

```{r dataTotalRow}

	dataAEInterest$AESEVN <- as.numeric(dataAEInterest$AESEV)
	
	# compute worst-case scenario per subject*AE term*treatment
	dataAEInterestWC <- ddply(dataAEInterest, c("AESOC", "AEDECOD", "USUBJID", "TRTA"), function(x){
		x[which.max(x$AESEVN), ]
	})

	## datasets used for the total: 
	# for total: compute worst-case across SOC and across AE term
	# (otherwise patient counted in multiple categories if present different categories for different AEs)
	dataTotalRow <- list(
		# within visit (across AEDECOD)
		'AEDECOD' = ddply(dataAEInterest, c("AESOC", "USUBJID", "TRTA"), function(x){	
			x[which.max(x$AESEVN), ]
		}),
		# across visits
		'AESOC' = ddply(dataAEInterest, c("USUBJID", "TRTA"), function(x){	
			x[which.max(x$AESEVN), ]
		})
	)

	getSummaryStatisticsTable(
		data = dataAEInterestWC,
		## row variables:
		rowVar = c("AESOC", "AEDECOD", "AESEV"), 
		rowVarInSepCol = "AESEV",
		# total for column header and denominator
		dataTotal = dataTotalAE, 
		# include total across SOC and across AEDECOD
		rowVarTotalInclude = c("AESOC", "AEDECOD"), 
		# data for total row
		dataTotalRow = dataTotalRow, 
		# count for each severity category for the total
		rowVarTotalByVar = "AESEV", 
		rowTotalLab = "Any TEAE", 
		rowVarLab = c(AESOC = "Subjects with, n(%):", AESEV = "Worst-case scenario"),
		# sort per total in the total column
		rowOrder = "total", 
		## column variables
		colVar = "TRTA", 
		stats = getStats("n (%)"),
		emptyValue = "0",
		labelVars = labelVars
	)

```

# Labels

If the data is loaded into R with the `read_haven` of the `haven` package,  or
the `loadDataADaMSDTM` function of the `clinUtils` package, the label  for each
variable is stored in the 'label' attribute of the corresponding  column.

However, if this label is lost (e.g. if the object is subsetted), labels can be
specified via the `labelVars` parameter for all variables at once, or via
specific `[parameter]Lab` parameter, as `rowVarLab`/`colVarLab`/`varLab` for the
row/column/variable to summarize respectively.

# Title and footnote

Title and footnote are specified via the corresponding `title` and `footer`
parameters. The convenient function `toTitleCase` from the `tools` package is
used to set title case for the title of the summary statistics table.

```{r titleAndFooter}

	getSummaryStatisticsTable(
		data = dataAEInterest,
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		stats = getStats("n (%)"),
		dataTotal = dataTotalAE,
		labelVars = labelVars,
		title = toTitleCase("MOR106-CL-102: Adverse Events by System Organ Class and Preferred Term (Safety Analysis Set, Part 1)"),
		footer = c(
			"N=number of subjects with data; n=number of subjects with this observation",
			"Denominator for percentage calculations = the total number of subjects per treatment group in the safety population"
		)
	)

```

# Appendix

## Session information

```{r includeSessionInfo, echo = FALSE, results = "asis"}
print(sessionInfo())
```

---
title: "Creation of standard in-text tables"
author: "Laure Cougnaud"
date: "`r format(Sys.Date(), '%B %d, %Y')`"
output: 
  rmarkdown::html_document:
    toc: true
    toc_float: true
    toc_depth: 5
    number_sections: true
vignette: >
  %\VignetteIndexEntry{03 - Creation of standard in-text tables}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

# Introduction

```{r options, echo = FALSE}
	
	library(knitr)
	opts_chunk$set(
		echo = TRUE, results = 'asis', warning = FALSE, 
		# stop document execution if error (not the default)
		error = FALSE, message = FALSE, cache = FALSE,
		fig.width = 8, fig.height = 7,
		fig.path = "./figures_vignette/",
		fig.align = 'center')
	# instead of warn = 0 by default
	# include warnings when they occur in the document
	options(width = 170)
	
```

```{r loadPackages, warning = FALSE}

	library(clinUtils)
	library(tools)# toTitleCase
	library(plyr) # for ddply, rbind.fill
	library(inTextSummaryTable)

```

## Data format

The package is demonstrated with a subset of the ADaM datasets 
from the CDISC Pilot 01 dataset, available in the `clinUtils` package.

```{r loadData}	

	# load example data
    library(clinUtils)

    # load example data
    data(dataADaMCDISCP01)

    dataAll <- dataADaMCDISCP01
    labelVars <- attr(dataAll, "labelVars")

```

Typical in-text table for the CSR are included in the following sections.

Please note that the **table content** e.g. variables, statistics of interest
**depends strongly on the study at hand and personal preferences**.

# Subject information

## Subject disposition

```{r table-subjectDisposition}
	
	# data of interest
	dataDM <- dataAll$ADSL
	
	varDMFL <- grep("FL$", colnames(dataDM), value = TRUE)
	varDMFLLabel <- sub(" Flag$", "", labelVars[varDMFL])
	
	getSummaryStatisticsTable(
		data = dataDM,
		var = varDMFL, varFlag = varDMFL, varGeneralLab = "Analysis Set, N", 
		varLab = varDMFLLabel,
		stats = getStats("n (%)"),
		colVar = "TRT01P",
		labelVars = labelVars,
		colTotalInclude = TRUE, colTotalLab = "All subjects",
		varInclude0 = TRUE,
		title = toTitleCase("Table: subject disposition"),
		file = file.path("tables_CSR", "Table_subjectDisposition.docx")
	)

```

## Demographics

```{r table-demography}

	# data of interest
	dataDM <- subset(dataAll$ADSL, SAFFL == "Y")
	
	# variables of interest
	# Note: if available: ethnicity is included
	varsDM <- c(
		"SEX", "AGE", "AGEGR1",
		"RACE", "ETHNIC",
		"HEIGHTBL", "WEIGHTBL", 
		"BMIBL", "BMIBLGR1"
	)

	# Sort variables according to corresponding numeric variable
	dataDM$AGEGR1 <- with(dataDM, reorder(AGEGR1, AGEGR1N))
	dataDM$RACE <- with(dataDM, reorder(RACE, RACEN))
	dataDM$TRT01P <- with(dataDM, reorder(TRT01P, TRT01PN))
	
	## Define set of statistics of interest:
	statsDM <- getStatsData(
		data = dataDM, var = varsDM,
		# different for continuous and categorical variable
		type = c(cont = "median (range)", cat = "n (%)"),
		# for categorical variable, statistic name (here: 'n (%)')
		# should not be included in the table
		args = list(cat = list(includeName = FALSE))
	)

	## create the table:
	
	getSummaryStatisticsTable(
		data = dataDM, 
		# variables to summarize
		var = varsDM, 
		varGeneralLab = "Parameter",
		# column
		colVar = "TRT01P", colTotalInclude = TRUE, colTotalLab = "All subjects",
		# statistics
		stats = statsDM,
		statsGeneralLab = "",
		labelVars = labelVars,
		# if only one category, should be included in separated row (e.g. RACE: White)
		rowAutoMerge = FALSE,
		rowInclude0 = FALSE, emptyValue = 0,
		title = toTitleCase("Table: Demographic Data (safety Analysis Set)"),
		file = file.path("tables_CSR", "Table_demographicData.docx")
	)
	
```

## Baseline disease characteristics

Please note that the content of the table strongly depends on the study.

```{r table-baselineDiseaseCharacteristics}

	# data of interest
	dataBDC <- subset(dataAll$ADSL, SAFFL == "Y")
	
	# create table
	getSummaryStatisticsTable(
		data = dataBDC,
		var = c("DURDIS", "EDUCLVL"), varGeneralLab = "Parameter", 
		colVar = "TRT01P", colTotalInclude = TRUE, colTotalLab = "All subjects",
		stats = getStats("median\n(range)"), statsGeneralLab = "",
		rowAutoMerge = FALSE,
		labelVars = labelVars,
		title = toTitleCase("Table: Baseline Disease Characteristics (safety analysis set)"),
		file = file.path("tables_CSR", "Table_BaselineCharacteristics.docx")
	)

```

## Medical History and Concomitant Diseases

```{r table-MH}

	dataCM <- subset(dataAll$ADCM, SAFFL == "Y")

	# sort variable according to corresponding numeric variables
	dataCM$TRTA <- with(dataCM, reorder(TRTA, TRTAN))
	
	# Terms should be in lower-case
	dataCM$CMDECOD <- simpleCap(tolower(dataCM$CMDECOD))
	dataCM$CMCLAS <- simpleCap(tolower(dataCM$CMCLAS))
			
	getSummaryStatisticsTable(
		data = dataCM,
		colVar = "TRTA", colTotalInclude = TRUE, colTotalLab = "All subjects",
		rowVar = c("CMCLAS", "CMDECOD"), 
		# include total across generic terms and across ATC4 classes
		rowVarTotalInclude = c("CMCLAS", "CMDECOD"), 
		rowTotalLab = "Any prior and concomitant medication",
		stats = getStats("n (%)"),
		# sort rows based on counts of subjects in the total column 
		rowOrder = "total",
		labelVars = labelVars,
		emptyValue = 0,
		title = toTitleCase(paste("Prior and concomitant therapies",
			"by medication class and generic term (safety analyis set)"
		)),
		file = file.path("tables_CSR", "Table_CM.docx")
	)
	
```

# Efficacy Analyses

The example dataset has has two primary endpoints:

* ADAS-Cog (11), a.k.a Alzheimer's Disease Assessment Scale - Cognitive Subscale
a metric containing 11 items, available in the `ADQSADAS` dataset
* CIBIC+ score a.k.a Video-referenced Clinician's Interview-based Impression of Change
 available in the `ADQSCIBC` dataset
 
```{r table-efficacy}

	dataAdasCog11 <- subset(dataAll$ADQSADAS, PARAMCD == "ACTOT")
	dataCIBIC <- subset(dataAll$ADQSCIBC, PARAMCD == "CIBICVAL")
	
	dataEfficacy <- plyr::rbind.fill(dataAdasCog11, dataCIBIC)
	
	dataEfficacy$TRTP <- with(dataEfficacy, reorder(TRTP, TRTPN))
	dataEfficacy$AVISIT <- with(dataEfficacy, reorder(AVISIT, AVISITN))
	
	stats <- getStatsData(
		data = dataEfficacy, 
		var = c("AVAL", "CHG"), 
		type = c("n", "mean (se)", "median (range)")
	)
	
	getSummaryStatisticsTable(
		data = dataEfficacy,
		rowVar = "PARAM",
		colVar = c("TRTP", "AVISIT"),
		var = c("AVAL", "CHG"), 
		stats = stats,
		labelVars = labelVars,
		title = paste("Table: efficacy endpoints", 
			toTitleCase("actual value and changes from baseline per time point"				
		)),
		file = file.path("tables_CSR", "Table_efficacy.docx")
	)
	
```

# Safety Analyses

## Adverse Events

### Treatment-emergent summary table

```{r table-summaryTable}

	## data of interest: safety analysis set and treatment-emergent
	dataTEAE <- subset(dataAll$ADAE, SAFFL == "Y" & TRTEMFL == "Y")
	
	# order treatment and severity categories
	dataTEAE$TRTA <- with(dataTEAE, reorder(TRTA, TRTAN))
	
	## data considered for the total
	dataTotalAE <- subset(dataAll$ADSL, SAFFL == "Y")
	dataTotalAE$TRTA <- with(dataTotalAE, reorder(TRT01A, TRT01AN))
	
	# TEAE with worst intensity
	# build worst-case scenario
	dataTEAE$AESEV <- factor(dataTEAE$AESEV, levels = c("MILD", "MODERATE", "SEVERE"))
	dataTEAE$AESEVN <- as.numeric(dataTEAE$AESEV)
	dataTEAE <- ddply(dataTEAE, c("USUBJID", "TRTA"), function(x)
		cbind.data.frame(x, 
			WORSTINT = with(x, ifelse(AESEVN == max(AESEVN), as.character(AESEV), NA_character_))
	))
	dataTEAE$WORSTINT <- factor(dataTEAE$WORSTINT, levels = levels(dataTEAE$AESEV))
	
	## specify labels for each variable:
	varsAE <- c("TRTEMFL", "AESER", "AESDTH", "AEREL")
	
	# create the table
	getSummaryStatisticsTable(
		data = dataTEAE,
		colVar = "TRTA",
		# define variables to compute statistics on
		var = c("TRTEMFL", "AESER", "WORSTINT", "AESDTH", "AEREL"), 
		varFlag = c("TRTEMFL", "AESER", "AESDTH"),
		varLab = c(TRTEMFL = "Treatment-Emergent", WORSTINT = "Worst-case severity:"),
		varGeneralLab = "Subjects with, n(%):",
		# force the inclusion of lines for variable without count:
		varInclude0 = TRUE,
		# include the total for the worst-case scenario
		varTotalInclude = "WORSTINT",
		# statistics:
		stats = getStats('n (%)'),
		emptyValue = "0",
		labelVars = labelVars,
		# dataset used for the total in the header column (and for percentage as default)
		dataTotal = dataTotalAE,
		# title/export
		title = toTitleCase("Table: Summary Table of Treatment-emergent Adverse Events (safety analysis set)"),
		file = file.path("tables_CSR", "Table_TEAE_summary.docx")
	)
	
```

### Treatment-emergent incidence table

#### Events occuring in at least one subject

```{r table-TEAE}

	dataTEAE <- subset(dataAll$ADAE, SAFFL == "Y" & TRTEMFL == "Y")
	
	# order treatment and severity categories
	dataTEAE$TRTA <- with(dataTEAE, reorder(TRTA, TRTAN))
	
	## data considered for the total
	dataTotalAE <- subset(dataAll$ADSL, SAFFL == "Y")
	dataTotalAE$TRTA <- with(dataTotalAE, reorder(TRT01A, TRT01AN))
	
	getSummaryStatisticsTable(
		data = dataTEAE,
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		## total
		# data
		dataTotal = dataTotalAE,
		# row total
		rowVarTotalInclude = c("AESOC", "AEDECOD"), rowTotalLab = "Any TEAE",
		stats = getStats("n (%)"),
		labelVars = labelVars,
		rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term,\nn (%)"),
		# sort rows based on the total column:
		rowOrder = "total", 
		rowOrderTotalFilterFct = function(x) subset(x, TRTA == "Total"),
		title = paste("Table: Treatment-emergent Adverse Events by System Organ Class",
			"and Preferred Term (Safety Analysis Set)"
		),
		file = file.path("tables_CSR", "Table_TEAE_SOCPT_atLeast1Subject.docx")
	)

```

#### Events occuring in at least 25% of all subjects

```{r table-TEAE-inAtLeast25Percent}

	getSummaryStatisticsTable(
		data = dataTEAE,
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		## total
		# data
		dataTotal = dataTotalAE, 
		# row total
		rowVarTotalInclude = c("AESOC", "AEDECOD"), rowTotalLab = "Any TEAE",
		stats = getStats("n (%)"),
		labelVars = labelVars,
		rowVarLab = c('AESOC' = "SOC and Preferred Term,\nn (%)"),
		# sort rows based on the total column:
		rowOrder = "total", 
		rowOrderTotalFilterFct = function(x) subset(x, TRTA == "Total"),
		title = paste("Table: Treatment-emergent Adverse Events by System Organ Class",
			"and Preferred Term reported in at least 25% of the subjects",
			"in any treatment group (Safety Analysis Set)"
		),
		file = file.path("tables_CSR", "Table_TEAE_SOCPT_atLeast25PercentsSubject.docx"),
		# include only events occuring in at least 25% for at least one preferred term:
		filterFct = function(x)
			ddply(x, "AESOC", function(x){ # per AESOC to include the total
				ddply(x, "AEDECOD", function(y){
					yTotal <- subset(y, grepl("Total", TRTA))
					if(any(yTotal$statPercN >= 25))	y
				})
			})
	)

```

### Treatment-emergent worst-case table

```{r tableTEAE-worstCase}

	dataTEAE <- subset(dataAll$ADAE, SAFFL == "Y" & TRTEMFL == "Y")
	
	# order treatment and severity categories
	dataTEAE$TRTA <- with(dataTEAE, reorder(TRTA, TRTAN))
	
	## data considered for the total
	dataTotalAE <- subset(dataAll$ADSL, SAFFL == "Y")
	dataTotalAE$TRTA <- with(dataTotalAE, reorder(TRT01A, TRT01AN))
	
	# TEAE with worst intensity
	dataTEAE$AESEV <- factor(dataTEAE$AESEV, levels = c("MILD", "MODERATE", "SEVERE"))
	dataTEAE$AESEVN <- as.numeric(dataTEAE$AESEV)
	
	# extract worst-case scenario data (only one record if multiple with same severity)
	dataAEWC <- ddply(dataTEAE, c("AESOC", "AEDECOD", "USUBJID", "TRTA"), function(x){
		x[which.max(x$AESEVN), ]
	})
	# worst-case scenario in lower case
	dataAEWC$WORSTINT <- simpleCap(tolower(dataAEWC$AESEV))
	labelVars["WORSTINT"] <- "Worst-case scenario"

	## datasets used for the total: 
	# for total: compute worst-case across SOC and across AE term
	# (otherwise patient counted in multiple categories if present different categories for different AEs)
	dataTotalRow <- list(
		# within SOC (across AEDECOD)
		'AEDECOD' = ddply(dataAEWC, c("AESOC", "USUBJID", "TRTA"), function(x){	
			x[which.max(x$AESEVN), ]
		}),
		# across SOC
		'AESOC' = ddply(dataAEWC, c("USUBJID", "TRTA"), function(x){	
			x[which.max(x$AESEVN), ]
		})
	)
	
	getSummaryStatisticsTable(
		data = dataAEWC,
		## row variables:
		rowVar = c("AESOC", "AEDECOD", "WORSTINT"), rowVarInSepCol = "WORSTINT",
		# include total across SOC and across AEDECOD
		rowVarTotalInclude = c("AESOC", "AEDECOD"), dataTotalRow = dataTotalRow, 
		rowVarTotalByVar = "WORSTINT", # count for each severity category for the total
		rowTotalLab = "Any TEAE", rowVarLab = c(AESOC = "Subjects with, n(%):", WORSTINT = "Worst-case scenario"),
		# sort per total in the total column
		rowOrder = "total", 
		## column variables
		colVar = "TRTA", 
		stats = getStats("n (%)"),
		emptyValue = "0",
		labelVars = labelVars,
		dataTotal = dataTotalAE,
		title = toTitleCase(paste("Table: Treatment-emergent Adverse",
			"Events by system organ",
			"and preferred term by worst-case (safety Analysis Set)"
		)),
		file = file.path("tables_CSR", "Table_TEAE_Severity.docx")
	)
	
```

## Laboratory safety

### Table of laboratory abnormalities

```{r tableLab}

	dataLBAbn <- subset(dataAll$ADLBC, SAFFL == "Y" & LBNRIND != "NORMAL")
	
	dataLBAbn$PARAM <- with(dataLBAbn, reorder(PARAM, PARAMN))
	dataLBAbn$TRTA <- with(dataLBAbn, reorder(TRTA, TRTAN))
	dataLBAbn$LBNRIND <- factor(dataLBAbn$LBNRIND, levels = c("LOW", "HIGH"))

	dataLBAbnTotal <- subset(dataAll$ADSL, SAFFL == "Y")
	dataLBAbnTotal$TRTA <- with(dataLBAbnTotal, reorder(TRT01A, TRT01AN))
	
	getSummaryStatisticsTable(
		data = dataLBAbn,
		rowVar = c("PARCAT1", "PARAM"), 
		rowVarTotalInclude = c("PARCAT1", "PARAM"),
		colVar = "TRTA", 
		var = "LBNRIND", 
		rowVarInSepCol = "variableGroup", varSubgroupLab = "Abnormality",
		rowVarLab = c('PARCAT1' = "Laboratory Parameter\nn (%)"),
		stats = getStats("n (%)"),
		labelVars = labelVars,
		rowOrder = c("PARCAT1" = "total", "PARAM" = "total", "variableGroup" = "auto"),
		dataTotal = dataLBAbnTotal, 
		title = toTitleCase(paste("Table: Treatment-emergent",
			"Worst-case Laboratory Abnormalities (safety analysis set)"
		)),
		emptyValue = "0",
		file = file.path("tables_CSR", "Table_Lab_Severity.docx")
	)

```

## Electrocardiogram

Please note that there is no ECG dataset in the CDISC Pilot dataset
used for the examples, so this table is not effectively created
in the vignette.

Nevertheless, an example code is provided below to create a standard
table of summary statistics for the ECG parameters.

```{r ECG-formatData, eval = FALSE}

	# data of interest
	paramsECG <- c("QT", "QTCF", "QRS", "PR", "RR", "EGHR")

	dataECG <- subset(dataAll$ADEG, SAFFL == "Y" & PARAMCD %in% paramsECG)
	dataECG$TRTA <- with(dataECG, reorder(TRTA, TRTAN))
	dataECG$PARAM <- with(dataECG, reorder(PARAM, PARAMN))
	
	# consider all non-missing post-baseline records
	dataECGPostBaseline <- subset(dataECG, 
		AVISIT %in% c("Screening", "Baseline", "Worst-case post-baseline")
	)
	
	# worst-case scenario:
	dataECGWC <- subset(dataECG, AVISIT == "Worst-case post-baseline")
	# treatment-emergent
	dataECGWC$TRTEMFL <- with(dataECGWC, ifelse(BASECAT1 != CHGCAT1, "Y", "N"))
	dataECGWCTE <- subset(dataECGWC, TRTEMFL == "Y")
	dataECGWC <- convertVarToFactor(dataECGWC, 
		var = c("AVALCAT1", "CHGCAT1"), 
		varNum = c("AVALCA1N", "CHGCAT1N")
	)
	
	# create the table
	getSummaryStatisticsTable(
		data = dataECGWC,
		# layout:
		colVar = "TRTA",
		rowVar = "PARAM", rowVarLab = c('PARAM' = "ECG Parameter"),
		# metrics to compute statistics on
		var = c("AVALCAT1", "CHGCAT1"),
		# in a separated column
		rowVarInSepCol = c("variable", "variableGroup"),
		# labels
		varGeneralLab = "Abnormality",
		varSubgroupLab = "Worst-Case Post-Baseline",
		stats = getStats("n (%)"),
		labelVars = labelVars,
		# total: all post-baseline
		dataTotal = dataECGPostBaseline, 
		emptyValue = "0",
		rowVarTotalPerc = "PARAM", # total per parameter
		# ensure that categories are below the type of abnormality
		rowAutoMerge = FALSE,
		# only retain abnormalities:
		filterFct = function(x){
			subset(x, !variableGroup %in% c("<= 450 msec", "<= 30 msec"))
		},
		title = toTitleCase(paste("Table: Treatment-emergent worst-case",
			"ECG abnormalities and change from baseline ECG abnormalities (safety analysis set)"
		)),
		file = file.path("tables_CSR", "Table_ECG.docx")
	)
	
```

## Vital signs

### Treatment-emergent vital signs abnormalities

```{r tableVitalSigns}

	# analyis set and parameters of interest
	dataVS <- subset(dataAll$ADVS, 
		SAFFL == "Y" & ANL01FL == "Y" & VISIT != "BASELINE"
	)
	
	dataVS$PARAM <- with(dataVS, reorder(PARAM, PARAMN))
	dataVS$ANRIND <- with(dataVS, reorder(PARAM, PARAMN))
	dataVS$TRTA <- with(dataVS, reorder(TRTA, TRTAN))
	dataVS$SHIFT1 <- with(dataVS, factor(ifelse(SHIFT1 == "", NA_character_, SHIFT1)))
			
	getSummaryStatisticsTable(
		data = dataVS,
		rowVar = "PARAM", 
		rowVarInSepCol = "variableGroup", 
		rowVarInclude0 = TRUE,
		colVar = "TRTA", 
		var = "SHIFT1", varTotalInclude = TRUE,
		emptyValue = 0,
		stats = getStats("n (%)"),
		rowVarTotalPerc = "PARAM",
		labelVars = labelVars,
		title = toTitleCase(paste("Table: Treatment-emergent Worst-case",
			"Vital Sign Abnormalities (Safety Analysis Set)"
		)),
		file = file.path("tables_CSR", "Table_VitalSigns_Severity.docx")
	)

```

# Pharmacokinetics analysis

Please note that this example pharmacodynamics dataset contains different
subjects than the other datasets used in the vignette.

```{r PK}

	paramcdPK <- c("AUCINFO", "CMAX", "TMAX")
	dataPK <- subset(dataAll$ADPP, PKFL == "Y" & PARAMCD %in% paramcdPK)
	
	dataPK$PARCAT1 <- with(dataPK, reorder(PARCAT1, PARCAT1N))
	dataPK$PARAMCD <- with(dataPK, reorder(PARAMCD, PARAMN))
	dataPK$TRTA <- with(dataPK, reorder(TRTA, TRTAN))
	dataPK$PARAMCD <- with(dataPK, reorder(PARAMCD, PARAMN))
	
	# build pretty labels
	labelsPK <- c(
		AUCINFO = "AUC_{Inf,obs}\n(h*ng/mL)",
		CMAX = "C_{max}\n(ng/mL)",
		TMAX = "t_{max}\n(h)"
	)
	dataPK$PARAM <- factor(dataPK$PARAMCD, 
		levels = levels(dataPK$PARAMCD), 
		labels = labelsPK[levels(dataPK$PARAMCD)]
	)
	
	statsPK <- dlply(dataPK, "PARAM", function(dataParam){
		getStatsData(
			data = dataParam,
			var = "AVAL",
			type = "median\n(range)",
			includeName = FALSE
		)[[1]]
	})
	
	getSummaryStatisticsTable(
		data = dataPK,
		rowVar = c("PARCAT1", "PARAM"), colVar = "TRTA",
		var = "AVAL",
#		rowVarLab = c('PARCAT1' = "PK parameters"),
		stats = statsPK, statsVarBy = "PARAM",
		emptyValue = "-",
		title = toTitleCase("Table: Summary of PK parameters (pharmacokinetics analysis set)"),
		file = file.path("tables_CSR", "Table_PK_Parameters.docx"),
		labelVars = labelVars
	)

```

# Appendix

## Session information

```{r includeSessionInfo, echo = FALSE, results = "asis"}
print(sessionInfo())
```

---
title: "Visualization functionalities"
author: "Laure Cougnaud"
date: "`r format(Sys.Date(), '%B %d, %Y')`"
output: 
  rmarkdown::html_document:
    toc: true
    toc_float: true
    toc_depth: 5
    number_sections: true
vignette: >
  %\VignetteIndexEntry{Visualization functionalities}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

<style>
.zoomDiv {
  opacity: 0;
  position:fixed;
  top: 50%;
  left: 50%;
  z-index: 50;
  transform: translate(-50%, -50%);
  box-shadow: 0px 0px 50px #888888;
  max-height:100%; 
  overflow: scroll;
}

.zoomImg {
  width: 100%;
}
</style>


<script type="text/javascript">
  $(document).ready(function() {
    $('body').prepend("<div class=\"zoomDiv\"><img src=\"\" class=\"zoomImg\"></div>");
    // onClick function for all plots (img's)
    $('img:not(.zoomImg)').click(function() {
      $('.zoomImg').attr('src', $(this).attr('src'));
      $('.zoomDiv').css({opacity: '1', width: '60%'});
    });
    // onClick function for zoomImg
    $('img.zoomImg').click(function() {
      $('.zoomDiv').css({opacity: '0', width: '0%'});
    });
  });
</script>

```{r options, echo = FALSE}

library(knitr)
opts_chunk$set(
    echo = TRUE, 
    results = 'markup', warning = FALSE, 
    # stop document execution if error (not the default)
    error = FALSE, 
    message = FALSE, cache = FALSE,
    fig.width = 8, fig.height = 7,
    fig.path = "./figures_vignette/",
    fig.align = 'center')
options(width = 170)
# instead of warn = 0 by default
# include warnings when they occur in the document
options(warn = 1)

```

This vignette focuses on the visualization functionalities available within the
`inTextSummaryTable` package.

If you need to see figures bigger, just click on a graphic, and it will pop up
more readable.

Summary statistics can be visualized by:

1. **extracting the summary statistics table** with the
  `computeSummaryStatisticsTable` function (used for `getSummaryStatistics`
  function)
2. **visualizing the summary statistics via the `subjectProfileSummaryPlot`**
  function
  
By default the **`subjectProfileSummaryPlot`** plots the mean values with
**error bars** based on standard errors.

Moreover, the `subjectProfileSummaryPlot` allows the possibility to add a table
of counts of the number of subject for a specific combination of variables.

This vignette will guide the user to

* default **visualizations for A4 and presentation documents**
* **customize palettes of colors, shape, linetypes**
* **visualization with and without table of counts and by facets**
* general customizable input

However, in order to get a full overview of the functionalities the
documentation is available in the console with `? subjectProfileSummaryPlot`.

We will first create example data sets to show how the exporting functionalities
work. The data sets used here are available in the `clinUtils` package.


# Load packages and data

```{r loadPackages}
library(inTextSummaryTable)
library(clinUtils)
```

```{r loadData}	

# load example data
data(dataADaMCDISCP01)

dataAll <- dataADaMCDISCP01
labelVars <- attr(dataAll, "labelVars")

```
  
# Compute summary statistics  
  
Below we compute summary statistics for the laboratory data set.
  
```{r visualization-extractData}

dataLB <- subset(dataAll$ADLBC, grepl("Baseline|Week", AVISIT))
dataLB$AVISIT <- with(dataLB, reorder(trimws(AVISIT), AVISITN))
dataLB$TRTA <- with(dataLB, reorder(TRTA, TRTAN))

summaryTableDf <- computeSummaryStatisticsTable(
    data = dataLB,
    var = "AVAL",
    rowVar = c("PARCAT1", "PARAM"),
    colVar = c("TRTA", "AVISIT")
)

```

# Visualization for A4 document

The code below shows the default visualization, which plots the **mean with
standard errors** **suitable for an A4 document**, e.g. Word/pdf.

Note that the table below the plot is based on the counts for treatment and
visit variables, as specificed in the `colVar` argument of the code chuck above
for `computeSummaryStatisticsTable`.

```{r visualization-a4document}

# create the plot
dataPlot <- subset(
    summaryTableDf, 
    !isTotal &
     PARAM == "Alanine Aminotransferase (U/L)"
)

subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    colorVar = "TRTA",
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)

```

# Visualization for presentation

The code below shows the default visualization **suitable for a
presentation**, either in Powerpoint or ioslides.

```{r visualization-presentation, eval = requireNamespace("extrafont", quietly = TRUE) && "Tahoma" %in% extrafont::fonts()}
# 'Tahoma' font should be registered upfront to create plots with: 'presentation' style
subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    colorVar = "TRTA",
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN",
    style = "presentation"
)

```

Please note that the default font for presentation is _Tahoma_. 

If you don't have this default font available, 
the R package [_extrafont_](https://cran.r-project.org/package=extrafont)
might be useful to register fonts with your R graphics device.

# Palettes

The `inTextSummaryTable` uses default palettes available in the
`clinUtils` package.

However, palettes are fully customizable. Currently, there are two alternative
approaches to change the defaults:

* set the custom palettes as arguments in the `subjectProfileSummaryPlot`
  via the dedicated parameters **`colorPalette`**, **`shapePalette`**,
  **`linetypePalette`**
* set the **global options** into the R session.

Passing input arguments to the function might be convenient when creating one
plot.

Instead, **passing the global options is very handy when creating multiple
graphics**, so that the palettes have to be set only once at the beginning of
the R script, without the need to copy-paste input arguments in the
different visualizations.

If you want to **know more about aesthetics options of the
`inTextSummaryTable`**, there is a dedicated vignette available
[here](../doc/inTextSummaryTable-aesthetics.html)) or with

```{r getVignetteAesthetics, eval = FALSE}

vignette("inTextSummaryTable-aesthetics", "inTextSummaryTable")

```

If you want to **know more about default options from the
`clinUtils`**, you may check out the vignette of the package:

```{r getVignetteClinUtils, eval = FALSE}

vignette("clinUtils-vignette", "clinUtils")

```

Sections below show below both alternative approaches for setting the palettes. 

## Custom palettes via arguments in the function

In this section we guide on how to set custom palettes via the
`subjectProfileSummaryPlot` function.

### Color palette

This section focuses on the possibility to set the `colorPalette` argument.

```{r visualization-customColorFromArgument}

# custom color palette by setting a named vector of colors
customColorPalette <- c(
    `Xanomeline Low Dose` = "green",
    `Xanomeline High Dose` = "red",
	`Placebo` = "blue"
)
subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    colorVar = "TRTA",
    colorPalette = customColorPalette,
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)

```

### Shape and linetype palettes

This section focuses on the possibility to set the `shapePalette` and
`linetypePalette` arguments.

```{r visualization-customShapeFromArgument}

# custom shape palette
customShapePalette <- c(15, 17, 19)
subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    colorVar = "TRTA",
    shapePalette = customShapePalette,
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)

# custom linetype palette
customLinetypePalette <- c("twodash", "dotted", "longdash")
subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    colorVar = "TRTA",
    linetypePalette = customLinetypePalette,
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)

```

## Custom palettes via setting global options

The `inTextSummaryTable` package allows the user to define global options in
the R session.

As mentioned above, setting the global options is convenient because the
palettes have to be set only once in the R script or Rmd document.

Please note that the global options have to be set **after** loading the
package. This because when loading the package, the default global options
for palettes will overwrite the custom palettes.

Here in practice:

```{r exampleLoadingPackageAndOptions, eval = FALSE}

# this is OK
library(inTextSummaryTable)
options(inTextSummaryTable.colors.plot = customColorPalette)

# this does NOT set your custom palette
options(inTextSummaryTable.colors.plot = customColorPalette)
library(inTextSummaryTable)

```

### Coloring palettes

Color palettes can be set up with the `inTextSummaryTable.colors.plot` option.

```{r visualization-customColorFromOptions}

# custom color palette by setting a simple vector of colors
customColorPalette <- c("blue", "green", "orange")
options(inTextSummaryTable.colors.plot = customColorPalette)

subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    colorVar = "TRTA",
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)

# custom color palette by setting a named vector of colors
customColorPalette <- c(
    `Xanomeline Low Dose` = "green",
    `Xanomeline High Dose` = "red",
	`Placebo` = "purple"
)
options(inTextSummaryTable.colors.plot = customColorPalette)

subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    colorVar = "TRTA",
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)

```

### Shape and linetype palettes

What shown for the color palette is consistent for shape and linetypes
palettes. In this case, the global options are called
`inTextSummaryTable.shapes.plot` and `inTextSummaryTable.linetypes.plot`.

```{r visualization-customShapeFromOptions}

# custom shape palette
customShapePalette <- c(17, 19, 22)
options(inTextSummaryTable.shapes.plot = customShapePalette)

subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    colorVar = "TRTA",
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)

# custom linetype palette
customLinetypePalette <- c("dotted", "longdash", "solid")
options(inTextSummaryTable.linetypes.plot = customLinetypePalette)

subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    colorVar = "TRTA",
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)

```

## Set back default palettes

There is always to possibility to switch back to the default palettes of the
package:

```{r visualization-backDefaultPalettes}

options(inTextSummaryTable.colors.plot = clinColors)
options(inTextSummaryTable.shapes.plot = clinShapes)
options(inTextSummaryTable.linetypes.plot = clinLinetypes)

```

# Visualization without count table

The user can decide to remove the table of counts by setting the parameter
**`tableText`** to `NULL`, as the code below shows.

```{r visualization-table}

subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    colorVar = "TRTA",
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = NULL
)

```


# Visualization by facets

It is possible to directly plot multiple parameters with the **`facetVar`**
argument.

Please note that setting the `facetVar` is not compatible with adding a table
of counts underneath the plot.

```{r visualization-facets, out.width = "100%", fig.height = 9, fig.width = 9}

# create the plot
dataPlotFacets <- subset(
    summaryTableDf, 
    !isTotal &
        PARAM %in% c(
            "Alanine Aminotransferase (U/L)",
            "Albumin (g/L)",
            "Bilirubin (umol/L)",
            "Calcium (mmol/L)"
        )
)

subjectProfileSummaryPlot(
    data = dataPlotFacets,
    xVar = "AVISIT",
    colorVar = "TRTA",
    labelVars = labelVars,
    facetVar = c("PARCAT1", "PARAM"),
    useLinetype = TRUE
)

```

# Customize your visualization

The `subjectProfileSummaryPlot` has many options! A non-exaustive demonstration
is presented below.

```{r visualization-otherInputs}

# plot the median instead of mean
# no error bars
subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    meanVar = "statMedian",
    seVar = NULL,
    colorVar = "TRTA",
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)

# plot the mean with standard deviation
subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    seVar = "statSD",
    colorVar = "TRTA",
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)

# change labels
subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    xLab = "Time points",
    yLab = "Mean and Standard Errors",
    title = "Title of the plot",
    colorVar = "TRTA",
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)

```

# Appendix

## Session information

```{r includeSessionInfo, echo = FALSE, results = "asis"}
print(sessionInfo())
```

---
title: "Customization of aesthetics of in-text tables"
author: "Michela Pasetto and Laure Cougnaud"
date: "`r format(Sys.Date(), '%B %d, %Y')`"
output: 
  rmarkdown::html_document:
    toc: true
    toc_float: true
    toc_depth: 5
    number_sections: true
vignette: >
  %\VignetteIndexEntry{06 - Customization of aesthetics of in-text tables}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

```{r options, echo = FALSE}

library(knitr)
opts_chunk$set(
    echo = TRUE, results = 'markup', warning = FALSE, 
    # stop document execution if error (not the default)
    error = FALSE, 
    message = FALSE, cache = FALSE,
    fig.width = 4, fig.height = 4,
    fig.path = "./figures_vignette/",
    fig.align = 'center')
options(width = 170)
# instead of warn = 0 by default
# include warnings when they occur in the document
options(warn = 1)

```

The `inTextSummaryTable` contains functionalities to set palettes up.

The **palettes for visualization are retrieved from the `clinUtils`**
package, whereas the **palettes for the tables are defined in the
`inTextSummaryTable`** itself.

Moreover, **palettes are set through global options**.

This has the main advantage that if you wish to change the default palettes, it
is possible to set your preferences only once at the beginning of the R script
or Rmd document.

When loading the package, the global options for palettes get
automatically defined into the R session. If you wish to change the default
color scheme, you can apply your preferences by changing the global
options. 

Below, we present what options are available and how to customize them.

# Available options

The `inTextSummaryTable` package has the following options:

* **`inTextSummaryTable.colors.table.presentation`**

* **`inTextSummaryTable.pageDim.presentation`**

* **`inTextSummaryTable.colors.plot`**

* **`inTextSummaryTable.shapes.plot`**

* **`inTextSummaryTable.linetypes.plot`**

The options with the wording _colors.table.presentation_ and
_pageDim.presentation_ define the color scheme for tables and
the dimensions of the page in a presentation, respectively.

Instead, the options with _colors.plot_, _shapes.plot_ and
_linetype.plot_ set up the palettes for the visualization functionalities.

Note that **the options have to be defined after loading the package**. This
because when loading the package, the default global options for palettes will
overwrite the custom palettes.

In this way, you can see how the options are visible after loading the package:

```{r aesthetics-showOptions}

library(inTextSummaryTable)

# options for color scheme in in presentations
options("inTextSummaryTable.colors.table.presentation")

# options for colors in plots: viridis palette
options("inTextSummaryTable.colors.plot")

# options for shapes in plots
options("inTextSummaryTable.shapes.plot")

# options for linetypes in plots
options("inTextSummaryTable.linetypes.plot")

```


# Options for reporting

In this section we discuss the options for reporting, namely the
`inTextSummaryTable.colors.table.presentation` and
`inTextSummaryTable.pageDim.presentation`.

## Colors

By default, the reporting format makes tables with a black text on a white
background.

On the contrary, the presentation format creates tables with a blue header and
white text, whereas the body is grey with black text.

Below a demonstration for a dummy data set.

```{r aesthetics-defaultsReporting}

# default report style
getSummaryStatisticsTable(
    data = data.frame(USUBJID = c(1, 2)),
    style = "report" 
)

# default presentation style
getSummaryStatisticsTable(
    data = data.frame(USUBJID = c(1, 2)),
    style = "presentation" 
)

```

If you wish to define a personal color scheme, a named vector can be created and
passed to the `options`, as shown below.

Colors can be provided in hexadecimals or in `rgb` specification. In the
example we use hexadecimals just for convenience.

Note that the `bodyBackground1` and `bodyBackground2` allow to have alternating
row colors.

```{r aesthetics-changeReportingOptions}

# create named vector
customColorTable <- c(
    # black text in the header
    'header' = "#000000",
    # green background in the header
    'headerBackground' = "#74D055FF",
    # black text in the body
    'body' = "#000000", 
    # yellow background for all rows
    'bodyBackground1' = "#FDE725FF",
    'bodyBackground2' = "#FDE725FF",
    # black footer
    'footer' = "#000000",
    # white footer background
    'footerBackground' = "#FFFFFF",
    # black line for footer
    'line' = "#000000"
)
# set options
options(inTextSummaryTable.colors.table.presentation = customColorTable)
# create the table
getSummaryStatisticsTable(
    data = data.frame(USUBJID = c(1, 2)),
    style = "presentation" 
)

```

## Dimension of the page

By default, the `inTextSummaryTable` ships with a default PowerPoint in the
standard 4:3 size (7.5 x 10 inches).

However, it is often common to create a PowerPoint template with **Widescreen
size** of 16:9 which consistts of 7.50 x 13.32 inches.

Therefore, it possible to accomodate such widescreen size by providing

```{r aesthetics-changeDimPageOptions}

# default page dimension of a powerpoint created with Rmd
getOption("inTextSummaryTable.pageDim.presentation")

# set custom dimension of page for presentation
# in this example, the dimension is the widescreen size
pageDimCustom <- c(7.5, 13.32)
options(inTextSummaryTable.pageDim.presentation = pageDimCustom)
getOption("inTextSummaryTable.pageDim.presentation")

```

# Options for visualization

The defaults palettes for visualization are retrieved from the `clinUtils`
package.

If you wish to know more about those palettes, we refer to the vignette of
`clinUtils` available with 

```{r getVignetteClinUtils, eval = FALSE}

vignette("clinUtils-vignette", "clinUtils")

```

Below a demonstration with a dummy data set.

```{r aesthetics-defaultsVisualization}

# default colors, shapes and linetypes
summaryTable <- data.frame(
    visit = c(1, 2, 1, 2), 
    TRT = c("A", "A", "B", "B"),
    statMean = rnorm(4)
)    
subjectProfileSummaryPlot(
    data = summaryTable,
    xVar = "visit", 
    colorVar = "TRT" 
)

```

For the visualization, the same spirit of the tables applies. The user can
specify a vector of colors, shapes or linetypes. The vector do not necessarly
have to be named.

```{r aesthetics-changeVisualizationOptions}

# specify colors
options(inTextSummaryTable.colors.plot = c("red", "green"))
subjectProfileSummaryPlot(
    data = summaryTable,
    xVar = "visit", 
    colorVar = "TRT" 
)

# specify shape
options(inTextSummaryTable.shapes.plot = c("circle", "square"))
subjectProfileSummaryPlot(
    data = summaryTable,
    xVar = "visit", 
    colorVar = "TRT" 
)

# specify linetypes
options(inTextSummaryTable.linetypes.plot = c("dotdash", "longdash"))
subjectProfileSummaryPlot(
    data = summaryTable,
    xVar = "visit", 
    colorVar = "TRT" 
)

```

# Set back default palettes

There is always to possibility to switch back to the default palettes of the
package:

```{r aesthetics-backDefaultPalettes}

options(inTextSummaryTable.colors.table.presentation = tableColorsPresentation)
options(inTextSummaryTable.colors.plot = clinUtils::clinColors)
options(inTextSummaryTable.shapes.plot = clinUtils::clinShapes)
options(inTextSummaryTable.linetypes.plot = clinUtils::clinLinetypes)

```

# Set up specific palettes

# Page dimensions in report/presentation 

The function **`getDimPage`** extracts dimensions available for a content in a
report or presentation, e.g. to specify the maximal width of a figure in A4
report or for a presentation.

```{r aesthetics-getDimPage}

# a4 format with one 1 inch margin
getDimPage(type = "width", style = "report")
getDimPage(type = "height", style = "report")

# Presentation format (according to template) with one 1 inch margin
getDimPage(type = "width", style = "presentation")
getDimPage(type = "height", style = "presentation")

```

# Appendix

## Session information

```{r includeSessionInfo, echo = FALSE, results = "asis"}
print(sessionInfo())
```

---
title: "Advanced user in-text tables"
author: "Laure Cougnaud"
date: "`r format(Sys.Date(), '%B %d, %Y')`"
output: 
  rmarkdown::html_document:
    toc: true
    toc_float: true
    toc_depth: 5
    number_sections: true
vignette: >
  %\VignetteIndexEntry{Advanced user of in-text tables}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

In this vignette we focus on providing more explanation on how
the `inTextSummaryTable` package actually works. We would describe some of the
functionalities less exposed to the users.

We assume you are already familiar on how to create and export tables, otherwise
we advise to first check out the dedicated vignettes for creating and
exporting tables. The vignettes are accessible with the commands below.

```{r getVignette, eval = FALSE}

vignette("inTextSummaryTable-createTables", "inTextSummaryTable")
vignette("inTextSummaryTable-exportTables", "inTextSummaryTable")

```

We will first create example data sets to show how the exporting functionalities
work. The data sets used are available in the `clinUtils` package.


```{r options, echo = FALSE}

library(knitr)
opts_chunk$set(
    echo = TRUE, results = 'markup', warning = FALSE, 
    # stop document execution if error (not the default)
    error = FALSE, 
    message = FALSE, cache = FALSE,
    fig.width = 8, fig.height = 7,
    fig.path = "./figures_vignette/",
    fig.align = 'center')
options(width = 170)
# instead of warn = 0 by default
# include warnings when they occur in the document
options(warn = 1)

```

```{r loadPackages}

library(inTextSummaryTable)
library(clinUtils)
library(tools) # toTitleCase

```

```{r loadData}	

# load example data
data(dataADaMCDISCP01)

dataAll <- dataADaMCDISCP01
labelVars <- attr(dataAll, "labelVars")

dataADSL <- dataADaMCDISCP01$ADSL

```


```{r formatExampleData}

dataAE <-  subset(dataAll$ADAE, SAFFL == "Y" & TRTEMFL == "Y")
dataAEInterest <- subset(dataAE, AESOC %in% c(
        "INFECTIONS AND INFESTATIONS",
        "GENERAL DISORDERS AND ADMINISTRATION SITE CONDITIONS"
    )
)

# ensure that order of elements is the one specified in 
# the corresponding numeric variable
dataAEInterest$TRTA <- reorder(dataAEInterest$TRTA, dataAEInterest$TRTAN)
dataAEInterest$AESEV <- factor(dataAEInterest$AESEV, levels = c("MILD", "MODERATE"))

dataTotalAE <- subset(dataAll$ADSL, TRT01A != "Placebo")
# should contain columns specified in 'colVar'
dataTotalAE$TRTA <- dataTotalAE$TRT01A 

```

# Detailed framework of the creation of the in-text table

The `getSummaryStatisticsTable` consists of the following framework:

* **computation of the summary statistics** table with the
  **`computeSummaryStatisticsTable`** function
* **export of the table** to the required format with the **`outputType`**
  parameter

## Computation of the summary statistics

The supporting data for the summary statistics table, is accessed via the
**`computeSummaryStatisticsTable`**. This includes the entire set of
statistics (as numeric) and combined statistic set.

The output from the `computeSummaryStatisticsTable` is equivalent of the table
output by the `getSummaryStatisticsTable` function when the `outputType` is set
to 'data.frame-base'.

```{r computeSummaryStatisticsTable}

summaryTable <- computeSummaryStatisticsTable(
    data = dataAEInterest,
    rowVar = c("AESOC", "AEDECOD"),
    rowVarTotalInclude = c("AESOC", "AEDECOD"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    dataTotal = dataTotalAE,
    labelVars = labelVars,
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)")
)

print(head(summaryTable, 3))
```	

Please note the presence of the **`isTotal` column**, which flags the records
containing the number of subjects reported in the table header.

```{r computeSummaryStatisticsTable-isTotal}
print(subset(summaryTable, isTotal))
```	

## Export table to the requested format

The summary table is exported to the format of interest with:

```{r export-flextable}

export(
	summaryTable = summaryTable,
	outputType = "flextable"
)

```	

Please see the vignette: `inTextSummaryTable-exportTables` for more
information on the different export types available.

# Combine summary statistics table

## Via the `combine` function

Summary statistics tables can be combined with the `combine` function.

```{r combine}
tableDemoCat <- computeSummaryStatisticsTable(
	data = dataADSL,
	var = c("SEX", "AGE"), varInclude0 = TRUE,
	colVar = "TRT01P",
	stats = getStats("n (%)", includeName = FALSE),
	labelVars = labelVars
)
tableDemoCont <- computeSummaryStatisticsTable(
	data = dataADSL,
	var = c("HEIGHTBL", "WEIGHTBL"),
	colVar = "TRT01P",
	stats = getStats(c("n", "Mean")),
	labelVars = labelVars
)
tableDemo <- combine(tableDemoCat, tableDemoCont)
export(tableDemo)
```

## Manually

The tables created via the `inTextSummaryTable` are simple R `data.frame` objects,
so these can be combined/update to include extra statistics of interest.

The general workflow is to:

* create a table of descriptive statistics with the package 
(via the `computeSummaryStatisticsTable` function)
* create a `data.frame` with your statistics of relevance - in a similar format 
* combine your table with a table created by the package
* export it (via the `exportSummaryStatisticsTable` function)

For example, we combine the descriptive statistics table created above
with a set of pre-computed statistics (e.g. p-values of the difference between the treatment
groups).

```{r combine-manually}
dataADSL$TRT01P <- with(dataADSL, reorder(TRT01P, TRT01PN))

# check format of table created with the package:
descTable <- tableDemoCont
descTable[, c("variable", "TRT01P", "isTotal", "n", "Mean")]
```

### Statistics in rows

```{r combine-manually-rows}
# add p-values in an extra row
infTable <- unique(subset(descTable, !isTotal)[, c("variable", "TRT01P"), drop = FALSE])
infTable[which(infTable$variable == "Baseline Height (cm)"), "pValue"] <- 1e-10
infTable[which(infTable$variable == "Baseline Weight (kg)"), "pValue"] <- 1e-9
summaryTable <- plyr::rbind.fill(descTable, infTable)

exportSummaryStatisticsTable(
	summaryTable = summaryTable, 
	rowVar = "variable", 
	colVar = "TRT01P", 
	statsVar = c("n", "Mean", "pValue") 
)
```

### Statistics in rows, in an extra column

```{r combine-manually-columns}
compLab <- "Comparison between treatments (p-value)"

# add p-values in a new column - in an extra row
infTable <- unique(subset(descTable, !isTotal)[, "variable", drop = FALSE])
infTable$TRT01P <- compLab
infTable[which(infTable$variable == "Baseline Height (cm)"), "pValue"] <- 1e-10
infTable[which(infTable$variable == "Baseline Weight (kg)"), "pValue"] <- 1e-9 
summaryTable <- plyr::rbind.fill(descTable, infTable) 

# order columns to have comparison column as last 
summaryTable$TRT01P <- factor(summaryTable$TRT01P, levels = c(levels(dataADSL$TRT01P), compLab))
exportSummaryStatisticsTable(
	summaryTable = summaryTable,  
	rowVar = "variable",  
	colVar = "TRT01P",  
	statsVar = c("n", "Mean", "pValue")
)
```

### Statistics in an extra column

```{r combine-manually-columns-rows}
infTable <- unique(subset(descTable, !isTotal)[, "variable", drop = FALSE])
infTable$TRT01P <- compLab
infTable[which(infTable$variable == "Baseline Height (cm)"), "Mean"] <- 1e-10
infTable[which(infTable$variable == "Baseline Weight (kg)"), "Mean"] <- 1e-9

summaryTable <- plyr::rbind.fill(descTable, infTable) 

# order columns to have comparison column as last 
summaryTable$TRT01P <- factor(summaryTable$TRT01P, levels = c(levels(dataADSL$TRT01P), compLab)) 

exportSummaryStatisticsTable(
	summaryTable = summaryTable,
	rowVar = "variable",
	colVar = "TRT01P",
	statsVar = c("n", "Mean")
)
```

# Data pre-processing

The variables used for the row and columns of the summary statistics tables
should be present in a long format in the input data for the
`getSummaryStatisticsTable` function.

In case the grouping of the rows/columns is more complex and no grouping
variable is yet available in the data, the function `combineVariables` offers
simpler functionalities to create the input data.

The label for the grouping is extracted from the SAS dataset labels if
`labelVars` is specified, or can be customized (`label` parameter).

For example, the adverse events are counted for different population set:
screened population, completer population, only events with high severity, or
related to the treatment and with high severity.

```{r combineVariables}

# prepare the data: create grouping of interest
dataAEGroup <- combineVariables(
    data = dataAEInterest,
    newVar = "AEGRP",
    paramsList = list(
        # for all screened patients
        list(var = "TRTA", value = "Xanomeline High Dose"),
        # for moderate severity
        list(var = "AESEV", value = "MODERATE", labelExtra = "Moderate"),
        list(var = "AENDY", label = paste("With adverse events ending date"))
    ),
    # include also counts for all records
    includeAll = TRUE,
    labelAll = "All Adverse events", 
    labelVars = labelVars
)
labelVars["AEGRP"] <- "Patient groups of interest"

# create the table
getSummaryStatisticsTable(
    data = dataAEGroup,
    colVar = "TRTA", 
    rowVar = "AEGRP", 
    labelVars = labelVars,
    dataTotal = dataTotalAE,
    stats = list(expression(paste0(statN, " (", round(statPercN, 1), ")"))),
    title = "Table: Adverse events: counts for groups of interest",
    footer = "Statistics: n (%)"
)

```

# Appendix

## Session information

```{r includeSessionInfo, echo = FALSE, results = "asis"}
print(sessionInfo())
```

---
title: "Exporting in-text tables"
author: "Laure Cougnaud"
date: "`r format(Sys.Date(), '%B %d, %Y')`"
output: 
  rmarkdown::html_document:
    toc: true
    toc_float: true
    toc_depth: 5
    number_sections: true
vignette: >
  %\VignetteIndexEntry{Exporting in-text tables}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

```{r options, echo = FALSE}

library(knitr)
opts_chunk$set(
    echo = TRUE, results = 'markup', warning = FALSE, 
    # stop document execution if error (not the default)
    error = FALSE, 
    message = FALSE, cache = FALSE,
    fig.width = 8, fig.height = 7,
    fig.path = "./figures_vignette/",
    fig.align = 'center')
options(width = 170)
# instead of warn = 0 by default
# include warnings when they occur in the document
options(warn = 1)

```

In this vignette we focus on exporting tables created with the
`inTextSummaryTable` package.

In particular, we will cover

* how to **export tables for specific formats as Word/Powerpoint/Html/R object**
* how to **make your custom aesthetics**
* how to **save tables into separated documents**
* how to set specific **text formatting** (e.g. superscripts/subscripts)
* how to **create multiple tables** at once based on a variable of interest
* how to **create a listing**
* how to **filter and split** exported tables

We assume you are already familiar on how to create tables, otherwise we advise
to first check out the dedicated vignette on how to make tables, available
[here](../doc/inTextSummaryTable-createTables.html)) or accessible with the
commands below.

```{r getVignette, eval = FALSE}

vignette("inTextSummaryTable-createTables", "inTextSummaryTable")

```

We will first create example data sets to show how the exporting functionalities
work. The data sets used are available in the `clinUtils` package.


# Load packages and data


```{r loadPackages}

library(inTextSummaryTable)
library(clinUtils)
library(tools) # toTitleCase

```

```{r loadData}	

# load example data
data(dataADaMCDISCP01)

dataAll <- dataADaMCDISCP01
labelVars <- attr(dataAll, "labelVars")

```

This section creates example data sets used below.

```{r createExampleData}

dataAE <-  subset(dataAll$ADAE, SAFFL == "Y" & TRTEMFL == "Y")
dataAEInterest <- subset(dataAE, AESOC %in% c(
        "INFECTIONS AND INFESTATIONS",
        "GENERAL DISORDERS AND ADMINISTRATION SITE CONDITIONS"
    )
)

# ensure that order of elements is the one specified in 
# the corresponding numeric variable
dataAEInterest$TRTA <- reorder(dataAEInterest$TRTA, dataAEInterest$TRTAN)
dataAEInterest$AESEV <- factor(dataAEInterest$AESEV, levels = c("MILD", "MODERATE"))

dataTotalAE <- subset(dataAll$ADSL, TRT01A != "Placebo")
# should contain columns specified in 'colVar'
dataTotalAE$TRTA <- dataTotalAE$TRT01A 


```

# Output formats {#outputFormat}

The default exported format is **`flextable`**, which is **suitable for Word/PowerPoint** documents.

Alternatively, a table can be exported as **datatable interactive table for html reports**, 
or as a **data frame in R**.

The `outputType` parameter specifies the output format of the summary statistics
table.

## Static table for Word/Powerpoint

### General format

To get the table in a format **suitable for _Word_/_Powerpoint_** documents, the
**outputType** parameter should be set to **'flextable'** (the default).

```{r outputType-flextable}

summaryTableFt <- getSummaryStatisticsTable(
    data = dataAEInterest,
    rowVar = c("AESOC", "AEDECOD"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    dataTotal = dataTotalAE,
    labelVars = labelVars,
    title = "Table: Adverse Events by System Organ Class and Preferred Term",
    footer = c(
        "N = number of subjects in the data; n = number of subjects with observation",
        paste("Denominator for percentage calculations is the total number of subjects",
            "per treatment group in the safety population"
        )
    ),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)"),
    outputType = "flextable"
)
class(summaryTableFt)
summaryTableFt

```

The object `summaryTableFt` is a **flextable object**.

This is available through the [`flextable` R
package](https://cran.r-project.org/package=flextable) (see `?flextable`).

When printed into the R console, a flextable object is displayed in the
internet browser. However, the
`summaryTableFt` object can be inserted within a `rmarkdown` document by
printing it in the specified document chunk, providing that the following
system requirement are satisfied:

* `flextable` version >= 0.4.7
* for a `docx` document (`output_type`):
  [pandoc](https://pandoc.org/installing.html) version >= 2.0 (see [`rmarkdown`
  documentation for
  Word](https://bookdown.org/yihui/rmarkdown/word-document.html))
* for `pptx` document: [pandoc](https://pandoc.org/installing.html) version >=
  2.4 (see [`rmarkdown` documentation for
  Powerpoint](https://bookdown.org/yihui/rmarkdown/powerpoint-presentation.html))

A `rmarkdown` chunk can contains only one `flextable` object. To include a list
of `flextable` in a `rmarkdown` document, the function `knitPrintListObjects` of
the `clinUtils` package can be used.

### Style: report or presentation

The table can be styled via the **`style`** parameter. This parameter affects
the fontsize, font family, color of the text and background, dimensions of the
table.

Other specific parameters can be used to export the table as landscape
(`landscape`), specify custom margins (`margin`), row indent (`rowVarPadBase`),
font size and family (`fontsize` and `font`).

By default, the table is styled for a **CSR report**.

```{r style-report}

getSummaryStatisticsTable(
    data = dataAEInterest,
    rowVar = c("AESOC", "AEDECOD"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    dataTotal = dataTotalAE,
    labelVars = labelVars,
    title = "Table: Adverse Events by System Organ Class and Preferred Term",
    footer = c(
        "N = number of subjects with data; n = number of subjects with this observation",
        paste("Denominator for percentage calculations is the total number of subjects",
            "per treatment group in the safety population"
        )
    ),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)")
)

```

The table can also be styled for a **presentation**.

Note that the default dimension of a table printed in a Powerpoint document
follows the standard 4:3 size (7.5 x 10 inches).

If you wish to set the dimension for a Widescreen document, it is possible to
set a global option for this. Please, check out the section "Custom aesthetics".

```{r style-presentation}

getSummaryStatisticsTable(
    data = dataAEInterest,
    rowVar = c("AESOC", "AEDECOD"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    dataTotal = dataTotalAE,
    labelVars = labelVars,
    title = "Table: Adverse Events by System Organ Class and Preferred Term",
    footer = c(
        "N = number of subjects with data; n = number of subjects with this observation",
        paste("Denominator for percentage calculations is the total number of subjects",
            "per treatment group in the safety population"
        )
    ),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)"),
    style = "presentation"
)

```


## Interactive table for html

The summary statistics table is exported as an interactive table by setting the
parameter: **`outputType`** to **'DT'**.

A `datatable` object, created with the
[DT](https://CRAN.R-project.org/package=DT) R package is created.

Interactivity includes the functionalities to:

* filter the table
* order columns
* export the table (or a subset of this)
* include custom interactivity e.g. the possibility to have expandable
  rows/columns, see patient profiles inclusion in the medical monitoring report

Such table can be included in a _html_ `rmarkdown` report.

The interactive table can be further customized 
by setting options specific to this output (see `clinUtils::getClinDT`).

Please note that the interactive table is only created if Pandoc is available.

```{r outputType-DT, eval = rmarkdown::pandoc_available()}

getSummaryStatisticsTable(
    data = dataAEInterest,
    rowVar = c("AESOC", "AEDECOD"),
    colVar = "TRTA",
    stats = getStats("n"),
    dataTotal = dataTotalAE,
    labelVars = labelVars,
    title = "Table: Adverse Events by System Organ Class and Preferred Term",
    footer = c(
        "N = number of subjects in the data; n = number of subjects with observation",
        paste("Denominator for percentage calculations is the total number of subjects",
            "per treatment group in the safety population"
        )
    ),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)"),
    outputType = "DT",
	## DT-specific options
	buttons = c() # remove all export buttons
)

```

## Data frames for R

The summary statistics table can be stored as a R object.

Note that the exported `data.frame`s contains _attributes_ that indicate which
statistics are used, which row variables/column variables are to be shown and
how etc.

### Final table

The summary statistics data displayed in the final table object can be exported
as an R `data.frame`.

In this case the **`outputType`** should be set to **`data.frame`**.

```{r outputType-dataframe}

summaryTable <- getSummaryStatisticsTable(
    data = dataAEInterest,
    rowVar = c("AESOC", "AEDECOD"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    dataTotal = dataTotalAE,
    labelVars = labelVars,
    title = "Table: Adverse Events by System Organ Class and Preferred Term",
    footer = c(
        "N = number of subjects in the data; n = number of subjects with observation",
        paste("Denominator for percentage calculations is the total number of subjects",
            "per treatment group in the safety population"
        )
    ),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)"),
    style = "presentation",
    outputType = "data.frame"
)
print(summaryTable, split.table = Inf)
```

### Full table in long format

The table with all the computed summary statistics, the specified set of
statistics of interest and all row/column variables stored in a long format is
available by using the **`outputType`** parameter to **'data.frame-base'**.

This format is typically of interest to run quality checks of the computed
summary statistics in comparison with e.g. Table/Listing/Figures, or to compare
the statistics computed between batches of the data.

```{r outputType-dataframe-base}

summaryTableAll <- getSummaryStatisticsTable(
    data = dataAEInterest,
    rowVar = c("AESOC", "AEDECOD"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    dataTotal = dataTotalAE,
    labelVars = labelVars,
    title = "Table: Adverse Events by System Organ Class and Preferred Term",
    footer = c(
        "N = number of subjects in the data; n = number of subjects with observation",
        paste("Denominator for percentage calculations is the total number of subjects",
            "per treatment group in the safety population"
        )
    ),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)"),
    style = "presentation",
    outputType = "data.frame-base"
)
print(summaryTableAll, split.table = Inf)
```

## Differences between the format

Most of the functionalities and default parameters are the same between the
`flextable` and `DT` format, at the exception of:
 
* for the `DT` format, by default the variables and statistics are included
      in different columns: `rowVarInSepCol` is `rowVar` and `statsLayout` is
      set to 'col'
* for the `flextable` format, by default the row variables and statistics
      are nested:   `rowVarInSepCol` is set to: 'NULL' and the `statsLayout` is
      set to 'row'

# Custom aesthetics

The `inTextSummaryTable` contains functionalities to set the aesthetics of the
tables.

Those **aesthetics are set through global options**.

This has the main advantage that if you wish to change the default color scheme,
it is possible to set your preferences only once at the beginning of the R
script or Rmd document.

When loading the package, the global options for palettes get automatically
defined into the R session. If you wish to change the default colors, you
can apply your preferences by changing the global options. 

The `inTextSummaryTable` package has the following options for table aesthetics:

* **`inTextSummaryTable.colors.table.presentation`**

* **`inTextSummaryTable.pageDim.presentation`**

The first option sets custom colors for a table in a presentation mode, the
last one defines the page dimension of a Powerpoint when exporting a table.

Note that **the options have to be defined after loading the package**. This
because when loading the package, the default global options for palettes will
overwrite the custom palettes.

## Colors

As you may have already noticed if you have read the vignette till here, by
default, the reporting format makes tables with a black text on a white
background. On the contrary, the presentation format creates tables with a blue
header and white text, whereas the body is grey with black text.

If you wish to define a personal color scheme, a named vector can be created and
passed to the `options`, as shown below.

Colors can be provided in hexadecimals or in `rgb` specification. In the example
we use hexadecimals just for convenience.

Note that the `bodyBackground1` and `bodyBackground2` allow to have alternating
row colors.

```{r aesthetics-changeReportingOptions}

# create named vector
customColorTable <- c(
    # black text in the header
    'header' = "#000000",
    # green background in the header
    'headerBackground' = "#74D055FF",
    # black text in the body
    'body' = "#000000", 
    # yellow background for all rows
    'bodyBackground1' = "#FDE725FF",
    'bodyBackground2' = "#FDE725FF",
    # black footer
    'footer' = "#000000",
    # white footer background
    'footerBackground' = "#FFFFFF",
    # black line for footer
    'line' = "#000000"
)
# set options
options(inTextSummaryTable.colors.table.presentation = customColorTable)
# create the table on a dummy data set
getSummaryStatisticsTable(
    data = data.frame(USUBJID = c(1, 2)),
    style = "presentation" 
)

```

## Dimension of the page

By default, the `inTextSummaryTable` ships with a default PowerPoint in the
standard 4:3 size (7.5 x 10 inches).

However, it is often common to create a PowerPoint template with **Widescreen
size** of 16:9 which consistts of 7.50 x 13.32 inches.

Therefore, it possible to accomodate such widescreen size by providing

```{r aesthetics-changeDimPageOptions}

# set custom dimension of page for presentation
# in this example, the dimension is the widescreen size
pageDimCustom <- c(7.5, 13.32)
options(inTextSummaryTable.pageDim.presentation = pageDimCustom)
getOption("inTextSummaryTable.pageDim.presentation")

```

## Further info on asthetics

For further information about the aesthetics settings of
the package, a dedicated vignette is available at 

```{r getVignetteAesthetics, eval = FALSE}

vignette("inTextSummaryTable-aesthetics", "inTextSummaryTable")

```

## Set back default palettes

There is always to possibility to switch back to the default palettes of the
package:

```{r aesthetics-backDefaultPalettes}

options(inTextSummaryTable.colors.table.presentation = tableColorsPresentation)

```


# Export table to a separated file

The table can be exported to a different file by specifying a file name in the
**`file`** parameter. 

Possible formats are:

* **txt**: the raw base table is exported to a text file. This format can be
  used e.g. to run quality checks with reference Table/Listings/Figures.
* **docx**: the formatted table is exported to a Word document. Such format is
  typically of interest to share the tables to medical writers, to be
  imported directly into a Statistical Analysis Plan (if `style = 'report'`).
* **html**: an interactive table is exported to the specified html file

```{r file, eval = FALSE}

# export table to a Word document
summaryTableFt <- getSummaryStatisticsTable(
    data = dataAEInterest,
    rowVar = c("AESOC", "AEDECOD"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    dataTotal = dataTotalAE,
    labelVars = labelVars,
    file = file.path("tables_CSR", "summaryTable-AEs.docx")
)

# export interactive table to a html document
summaryTableFt <- getSummaryStatisticsTable(
    data = dataAEInterest,
    rowVar = c("AESOC", "AEDECOD"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    dataTotal = dataTotalAE,
    labelVars = labelVars,
    file = file.path("tables_CSR", "summaryTable-AEs.html")
)

# export table in raw format to a text file
summaryTableFt <- getSummaryStatisticsTable(
    data = dataAEInterest,
    rowVar = c("AESOC", "AEDECOD"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    dataTotal = dataTotalAE,
    labelVars = labelVars,
    file = file.path("tables_CSR", "summaryTable-AEs.txt")
)

# export to multiple formats at once
summaryTableFt <- getSummaryStatisticsTable(
    data = dataAEInterest,
    rowVar = c("AESOC", "AEDECOD"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    dataTotal = dataTotalAE,
    labelVars = labelVars,
    file = file.path("tables_CSR",
        c("summaryTable.txt", "summaryTable.docx", "summaryTable.html")
    )
)

```

# Export table of pre-computed summary statistics

To create a table in a similar format as the in-text tables created by the
package, from pre-computed summary statistics (e.g. from CRO or from a different
software as _SAS_) the function **`exportSummaryStatisticsTable`** can be used.

This function takes as input a **`data.frame` in long format** that should
contain the summary statistics.

The long format should be such that the

* statistic value (`statsVar`)
* statistic type (if multiple), grouping row and columns (`rowVar` and `colVar`)

are both stored as different columns in the `data.frame`.

If the column headers should contain total number of subjects, the corresponding
counts should be stored in records with the column `isTotal` set to TRUE.

```{r export}

dataDIABP <- subset(
	dataAll$ADVS, 
	SAFFL == "Y" & ANL01FL == "Y" &
	PARAMCD == "DIABP" & 
	AVISIT %in% c("Baseline", "Week 6") &
	ATPT == "AFTER LYING DOWN FOR 5 MINUTES"
)

# create example of data.frame containing statistics of interest
statsEff <- sapply(c("AVAL", "CHG"), function(var) {
      
      getStats(
          type = c("n", "mean (se)", "median (range)"),
          x = dataDIABP[[var]]
      )
      
    }, simplify = FALSE)

summaryTable <- computeSummaryStatisticsTable(
    data = dataDIABP,
    colVar = c("TRTP", "AVISIT"),
    var = c("AVAL", "CHG"), varGeneralLab = "",
    stats = statsEff,
    labelVars = labelVars
)

# format df with statistics to in-text table format
export(
    summaryTable = summaryTable,
    statsVar = c("n", "Mean (SE)", "Median (range)"),
    rowVar = "variable", rowVarLab = "Statistic",
    colVar = c("TRTP", "AVISIT"),
    colHeaderTotalInclude = TRUE,
    labelVars = simpleCap(tolower(labelVars[c("AVAL", "CHG")])),
    title = toTitleCase(
		"Table: Diastolic Blood Pressure (mmHg) statistics"
    )
)

```

# Text formatting

Specific formatted text can be specified for the `flextable` output type only.

## Superscript/subscript

Superscript and subscript are specified via the special text pattern as `a^{b}`
and `a_{b}` respectively.

To use a superscript or a subscript in the table, the text should be formatted
as: `text^{superscript}` or `text_{subscript}`.

This is for example useful to reference additional informations in a specific
footnote or specify custom variable labels.

```{r summaryTable-PP-rowVarWithLabel-rowVarLab-superscript}

dataSL <- subset(dataAll$ADSL, SAFFL == "Y")

varsSL <- c("AGE", "WEIGHTBL", "BMIBL")
labelVars[varsSL] <- c(
    "Age",
    "Weight_{t}",
    "BMI (kg/m^{2})"
)

getSummaryStatisticsTable(
    data = dataSL, 
    var = varsSL,  
    stats = getStats("n (%"),
    labelVars = labelVars,
    fontsize = 16,
    title = toTitleCase("Demographic data (Safety Analysis Set)")
)

```

## Bold and greek letters

Specific cells of the table can be highlight in bold by using the syntax:
`bold{}` in the `stats` or `statsExtra` function.

This highlighting may depend:

* on a additional variable via the `statsExtra` parameter:  
for example:
  `statsExtra <- function(data)   with(data, ifelse(ANRIND != "N", paste0("bold{", toString(AVAL), "}"), toString(AVAL))`
* on the values of computed statistics, via the `stats` parameter

Greek letters are included as they are in the table.

```{r summaryTable-PP-rowVarWithLabel-rowVarLab-bold}

getSummaryStatisticsTable(
    data = dataSL, 
    var = varsSL,  
    stats = list(
		expression(paste0(
			ifelse(statMean > statMedian, 
				paste0("bold{", roundHalfUpTextFormat(statMean, 1), "}"), 
				roundHalfUpTextFormat(statMean, 1)
			), 
			"\n(", 
				roundHalfUpTextFormat(statSE, 2),")")
        )
    ),
    labelVars = labelVars,
    fontsize = 12,
    title = toTitleCase("Demographic data (Safety Analysis Set)")
)

```

# Create multiple tables by a variable

Multiple tables can be created for each element of a variable in the dataset  by
specifying the name of the variable in the argument **`byVar`**.  
 
A list of final summary table can be included into a `rmarkdown`  document with
the `knitPrintListObjects`. 

Please note that the following chunk option should be used:
**`results = 'asis'`**.

For example, AE tables are created for each treatment:

```{r byVar, results = "asis"}

summaryTableList <- getSummaryStatisticsTable(
    data = subset(dataAE, TRTEMFL == "Y"),
    rowVar = c("AESOC", "AEDECOD"),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)"),
    colVar = "TRTA",
    byVar = "TRTA",
    stats = getStats("n (%)"),
    labelVars = labelVars,
    title = "Table: Treatment-Emergent adverse events"
)

# print the list of tables in the rmarkdown document
clinUtils::knitPrintListObjects(summaryTableList) 

```


# Listing

Listing in a similar format as the in-text summary table is created via the `getListing` function.

```{r getListing}

varsListing <- c(
	"USUBJID", "AEBODSYS", "AEDECOD", "TRTA", 
	"AESEV", "AESER", "ASTDY", "AENDY"
)

dataListing <- subset(dataAE, TRTEMFL == "Y" & AESEV == "SEVERE")
dataListing <- dataListing[, varsListing]
colnames(dataListing) <- getLabelVar(var = varsListing, labelVars = labelVars)

getListing(
	data = dataListing, 
	title = "Listing of treatment-emergent severe adverse events",
	includeRownames = FALSE
)

```

# Filter records in a table

If only a subset of the records should be displayed in the final table, a custom
filtering functions is specified via the **`filterFct`** parameter.

```{r countTable-AE-filterFct}

library(plyr)

# SOC with AE terms with at least 2 subjects
getSummaryStatisticsTable(
    data = subset(dataAE, TRTEMFL == "Y"),
    rowVar = c("AESOC", "AEDECOD"),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    filterFct = function(x)
      ddply(x, "AESOC", function(y)
            if(any(y$statN >= 2))	y			
      ),
    labelVars = labelVars,
    title = paste(
        "Table: Adverse Events by System Organ Class and",
        "Preferred Term with at least 2 patients in System Organ Class"
    )
)

# AE term with at least one term with more than 70% in the treatment
getSummaryStatisticsTable(
    data = subset(dataAE, TRTEMFL == "Y"),
    rowVar = c("AESOC", "AEDECOD"),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    filterFct = function(x)
      ddply(x, "AEDECOD", function(xTerm) {
            if(any(xTerm$statPercN >= 70))	xTerm		
          }),
    labelVars = labelVars,
    title = "Table: Adverse Events by System Organ Class and Preferred Term with at least 70% patients"
)

```

Note: to help the creation of this filtering function, the displaying data
could be extracted first via the `computeSummaryStatisticsTable` function
(instead of creating directly the in-text table via the
`getSummaryStatisticsTable`), and then used to define the `filterFct` parameter.

```{r countTable-AE-filterFct- details}

x <- computeSummaryStatisticsTable(
    data = subset(dataAE, TRTEMFL == "Y"),
    rowVar = c("AESOC", "AEDECOD"),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    labelVars = labelVars
)
head(x)

# for specific AEDECOD
xTerm <- subset(x, AEDECOD == "MYOCARDIAL INFARCTION")
# identify the record for treated patient:
subset(xTerm, grepl("Placebo", TRTA))
# keep all records (placebo + treatment) if percentage if higher than 20%:
# if no 'else' condition, nothing (NULL) is returned:
if(subset(xTerm, grepl("Placebo", TRTA))$statPercN >= 20)	xTerm
# across all AE terms:
ddply(x, "AEDECOD", function(xTerm)
      if(subset(xTerm, grepl("Placebo", TRTA))$statPercN >= 20)	xTerm
)

# format it as a function and pass it to the 'filterFct' parameter
```

# Appendix

## Session information

```{r includeSessionInfo, echo = FALSE, results = "asis"}
print(sessionInfo())
```

---
title: "Introduction to the `inTextSummaryTable` package"
author: "Laure Cougnaud"
date: "`r format(Sys.Date(), '%B %d, %Y')`"
output: 
  rmarkdown::html_document:
    toc: true
    toc_float: true
    toc_depth: 5
    number_sections: true
vignette: >
  %\VignetteIndexEntry{01 - Introduction to the inTextSummaryTable package}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

# Introduction

```{r options, echo = FALSE}
	
	library(knitr)
	opts_chunk$set(
		echo = TRUE, results = 'markup', warning = FALSE, 
		# stop document execution if error (not the default)
		error = FALSE, 
		message = FALSE, cache = FALSE,
		fig.width = 8, fig.height = 7,
		fig.path = "./figures_vignette/",
		fig.align = 'center')
	options(width = 170)
	# instead of warn = 0 by default
	# include warnings when they occur in the document
	options(warn = 1)
	
```

The package `inTextSummaryTable` contains functionalities to **create
complex table of summary statistics or counts of variables of interest**.

This table can be formatted to static

* **in-text table** for a **Clinical Study Report**,
* a **topline presentation**
* an **interactive table**, e.g. to be used inside the **medical
  monitoring report**.

These tables can be included into a `rmarkdown` document which can be
converted into `html`, `docx` and `pptx`.
Each table can be separately exported into a `text`, `docx` or `html `document.

We start the guidance to the functionalities of the `inTextSummaryTable`, via
loading the following packages.

```{r loadPackages}

	library(inTextSummaryTable)
	library(tools) # toTitleCase

```

## Data format

The input data for the creation of summary table should be a **data.frame**,
usually loaded from a _SAS_ data file (`sas7bdat` format).
The label of the variables stored in the `SAS` datasets is also used
for the title or the caption. 

Note that the `loadDataADaMSDTM` function of the `clinUtils` package 
can be used to import your study-specific dataset(s) of interest into R.

For this vignette, ADaM datasets included in the `clinUtils` package are used.

```{r loadData}	
	
	library(clinUtils)

	# load example data
    data(dataADaMCDISCP01)
	
	dataAll <- dataADaMCDISCP01
    labelVars <- attr(dataAll, "labelVars")
	
	# dataAll contains a list
	class(dataAll)
	# ... of ADaM datasets
	names(dataAll)
	# access a specific dataset
	head(dataAll$ADSL, 1)
	
	# check label of a subset of the variable(s)
	head(labelVars)
	# or for a specific variable:
	labelVars["USUBJID"]
	# or:
	getLabelVar(var = "USUBJID", labelVars = labelVars)

```

# Create your tables

Extensive documentation on how to create your in-text table is available in the vignette:
[Creation of in-text tables](inTextSummaryTable-02-createTables.html).

# Create standard set of tables

The creation of standard T(LF)ables is described in the vignette: 
[Creation of standard in-text tables](inTextSummaryTable-03-standardTables.html)

# Table formatting/export

If you are familiar with the creation of the tables, but you are wondering how
to:

* **export tables for specific formats as Word/PowerPoint/Html/R object**
* **save tables into separated documents**
* set specific **text formatting** (e.g. superscripts/subscripts)
* **filter and split** exported tables

then check the vignette: [Exporting of in-text tables](inTextSummaryTable-04-exportTables.html).

# Advanced use

If you are familiar with the creation and export of the tables, and you want to
know more on the functionalities less exposed to the users, you can check out
the vignette: [Advanced use of in-text tables](inTextSummaryTable-05-advanced.html).

# Aesthetics

The `inTextSummaryTable` package allow the user to set global options for the
color schemes of tables and visualization. The main advantage is that if
you wish to change the default palettes, it is possible to set your preferences
only once at the beginning of the R script or Rmd document, see the vignette: [Customization of aesthetics of in-text tables](inTextSummaryTable-06-aesthetics.html).

# Visualization

The summary statistics displayed in the table can also be visualized, see the
vignette: [Visualization of in-text table summaries](inTextSummaryTable-07-visualization.html).

# Appendix

## Session information

```{r includeSessionInfo, echo = FALSE, results = "asis"}
print(sessionInfo())
```

---
title: "Advanced use of in-text tables"
author: "Laure Cougnaud"
date: "`r format(Sys.Date(), '%B %d, %Y')`"
output: 
  rmarkdown::html_document:
    toc: true
    toc_float: true
    toc_depth: 5
    number_sections: true
vignette: >
  %\VignetteIndexEntry{05 - Advanced use of in-text tables}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

In this vignette we focus on providing more explanation on how
the `inTextSummaryTable` package actually works. We would describe some of the
functionalities less exposed to the users.

We assume you are already familiar on how to create and export tables, otherwise
we advise to first check out the dedicated vignettes:

* [Creation of in-text tables](inTextSummaryTable-02-createTables.html)
* [Exporting of in-text tables](inTextSummaryTable-04-exportTables.html)

We will first create example data sets to show how the exporting functionalities
work. The data sets used are available in the `clinUtils` package.


```{r options, echo = FALSE}

library(knitr)
opts_chunk$set(
    echo = TRUE, results = 'markup', warning = FALSE, 
    # stop document execution if error (not the default)
    error = FALSE, 
    message = FALSE, cache = FALSE,
    fig.width = 8, fig.height = 7,
    fig.path = "./figures_vignette/",
    fig.align = 'center')
options(width = 170)
# instead of warn = 0 by default
# include warnings when they occur in the document
options(warn = 1)

```

```{r loadPackages}

library(inTextSummaryTable)
library(clinUtils)
library(tools) # toTitleCase

```

```{r loadData}	

# load example data
data(dataADaMCDISCP01)

dataAll <- dataADaMCDISCP01
labelVars <- attr(dataAll, "labelVars")

dataADSL <- dataADaMCDISCP01$ADSL

```


```{r formatExampleData}

dataAE <-  subset(dataAll$ADAE, SAFFL == "Y" & TRTEMFL == "Y")
dataAEInterest <- subset(dataAE, AESOC %in% c(
        "INFECTIONS AND INFESTATIONS",
        "GENERAL DISORDERS AND ADMINISTRATION SITE CONDITIONS"
    )
)

# ensure that order of elements is the one specified in 
# the corresponding numeric variable
dataAEInterest$TRTA <- reorder(dataAEInterest$TRTA, dataAEInterest$TRTAN)
dataAEInterest$AESEV <- factor(dataAEInterest$AESEV, levels = c("MILD", "MODERATE"))

dataTotalAE <- subset(dataAll$ADSL, TRT01A != "Placebo")
# should contain columns specified in 'colVar'
dataTotalAE$TRTA <- dataTotalAE$TRT01A 

```

# Detailed framework of the creation of the in-text table

The `getSummaryStatisticsTable` consists of the following framework:

* **computation of the summary statistics** table with the
  **`computeSummaryStatisticsTable`** function
* **export of the table** to the required format with the **`outputType`**
  parameter

## Computation of the summary statistics

The supporting data for the summary statistics table, is accessed via the
**`computeSummaryStatisticsTable`**. This includes the entire set of
statistics (as numeric) and combined statistic set.

The output from the `computeSummaryStatisticsTable` is equivalent of the table
output by the `getSummaryStatisticsTable` function when the `outputType` is set
to 'data.frame-base'.

```{r computeSummaryStatisticsTable}

summaryTable <- computeSummaryStatisticsTable(
    data = dataAEInterest,
    rowVar = c("AESOC", "AEDECOD"),
    rowVarTotalInclude = c("AESOC", "AEDECOD"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    dataTotal = dataTotalAE,
    labelVars = labelVars,
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)")
)

print(head(summaryTable, 3))
```	

Please note the presence of the **`isTotal` column**, which flags the records
containing the number of subjects reported in the table header.

```{r computeSummaryStatisticsTable-isTotal}
print(subset(summaryTable, isTotal))
```	

## Export table to the requested format

The summary table is exported to the format of interest with:

```{r export-flextable}

export(
	summaryTable = summaryTable,
	outputType = "flextable"
)

```	

Please see the vignette: [Exporting of in-text tables](inTextSummaryTable-04-exportTables.html)
for more information on the different export types available.

# Combine summary statistics table

## Via the `combine` function

Summary statistics tables can be combined with the `combine` function.

```{r combine}
tableDemoCat <- computeSummaryStatisticsTable(
	data = dataADSL,
	var = c("SEX", "AGE"), varInclude0 = TRUE,
	colVar = "TRT01P",
	stats = getStats("n (%)", includeName = FALSE),
	labelVars = labelVars
)
tableDemoCont <- computeSummaryStatisticsTable(
	data = dataADSL,
	var = c("HEIGHTBL", "WEIGHTBL"),
	colVar = "TRT01P",
	stats = getStats(c("n", "Mean")),
	labelVars = labelVars
)
tableDemo <- combine(tableDemoCat, tableDemoCont)
export(tableDemo)
```

## Manually

The tables created via the `inTextSummaryTable` are simple R `data.frame` objects,
so these can be combined/update to include extra statistics of interest.

The general workflow is to:

* create a table of descriptive statistics with the package 
(via the `computeSummaryStatisticsTable` function)
* create a `data.frame` with your statistics of relevance - in a similar format 
* combine your table with a table created by the package
* export it (via the `exportSummaryStatisticsTable` function)

For example, we combine the descriptive statistics table created above
with a set of pre-computed statistics (e.g. p-values of the difference between the treatment
groups).

```{r combine-manually}
dataADSL$TRT01P <- with(dataADSL, reorder(TRT01P, TRT01PN))

# check format of table created with the package:
descTable <- tableDemoCont
descTable[, c("variable", "TRT01P", "isTotal", "n", "Mean")]
```

### Statistics in rows

```{r combine-manually-rows}
# add p-values in an extra row
infTable <- unique(subset(descTable, !isTotal)[, c("variable", "TRT01P"), drop = FALSE])
infTable[which(infTable$variable == "Baseline Height (cm)"), "pValue"] <- 1e-10
infTable[which(infTable$variable == "Baseline Weight (kg)"), "pValue"] <- 1e-9
summaryTable <- plyr::rbind.fill(descTable, infTable)

exportSummaryStatisticsTable(
	summaryTable = summaryTable, 
	rowVar = "variable", 
	colVar = "TRT01P", 
	statsVar = c("n", "Mean", "pValue") 
)
```

### Statistics in rows, in an extra column

```{r combine-manually-columns}
compLab <- "Comparison between treatments (p-value)"

# add p-values in a new column - in an extra row
infTable <- unique(subset(descTable, !isTotal)[, "variable", drop = FALSE])
infTable$TRT01P <- compLab
infTable[which(infTable$variable == "Baseline Height (cm)"), "pValue"] <- 1e-10
infTable[which(infTable$variable == "Baseline Weight (kg)"), "pValue"] <- 1e-9 
summaryTable <- plyr::rbind.fill(descTable, infTable) 

# order columns to have comparison column as last 
summaryTable$TRT01P <- factor(summaryTable$TRT01P, levels = c(levels(dataADSL$TRT01P), compLab))
exportSummaryStatisticsTable(
	summaryTable = summaryTable,  
	rowVar = "variable",  
	colVar = "TRT01P",  
	statsVar = c("n", "Mean", "pValue")
)
```

### Statistics in an extra column

```{r combine-manually-columns-rows}
infTable <- unique(subset(descTable, !isTotal)[, "variable", drop = FALSE])
infTable$TRT01P <- compLab
infTable[which(infTable$variable == "Baseline Height (cm)"), "Mean"] <- 1e-10
infTable[which(infTable$variable == "Baseline Weight (kg)"), "Mean"] <- 1e-9

summaryTable <- plyr::rbind.fill(descTable, infTable) 

# order columns to have comparison column as last 
summaryTable$TRT01P <- factor(summaryTable$TRT01P, levels = c(levels(dataADSL$TRT01P), compLab)) 

exportSummaryStatisticsTable(
	summaryTable = summaryTable,
	rowVar = "variable",
	colVar = "TRT01P",
	statsVar = c("n", "Mean")
)
```

# Data pre-processing

The variables used for the row and columns of the summary statistics tables
should be present in a long format in the input data for the
`getSummaryStatisticsTable` function.

In case the grouping of the rows/columns is more complex and no grouping
variable is yet available in the data, the function `combineVariables` offers
simpler functionalities to create the input data.

The label for the grouping is extracted from the SAS dataset labels if
`labelVars` is specified, or can be customized (`label` parameter).

For example, the adverse events are counted for different population set:
screened population, completer population, only events with high severity, or
related to the treatment and with high severity.

```{r combineVariables}

# prepare the data: create grouping of interest
dataAEGroup <- combineVariables(
    data = dataAEInterest,
    newVar = "AEGRP",
    paramsList = list(
        # for all screened patients
        list(var = "TRTA", value = "Xanomeline High Dose"),
        # for moderate severity
        list(var = "AESEV", value = "MODERATE", labelExtra = "Moderate"),
        list(var = "AENDY", label = paste("With adverse events ending date"))
    ),
    # include also counts for all records
    includeAll = TRUE,
    labelAll = "All Adverse events", 
    labelVars = labelVars
)
labelVars["AEGRP"] <- "Patient groups of interest"

# create the table
getSummaryStatisticsTable(
    data = dataAEGroup,
    colVar = "TRTA", 
    rowVar = "AEGRP", 
    labelVars = labelVars,
    dataTotal = dataTotalAE,
    stats = list(expression(paste0(statN, " (", round(statPercN, 1), ")"))),
    title = "Table: Adverse events: counts for groups of interest",
    footer = "Statistics: n (%)"
)

```

# Appendix

## Session information

```{r includeSessionInfo, echo = FALSE, results = "asis"}
print(sessionInfo())
```

---
title: "Creation of standard in-text tables with the `inTextSummaryTable` package"
author: "Laure Cougnaud"
date: "`r format(Sys.Date(), '%B %d, %Y')`"
output: 
  rmarkdown::html_document:
    toc: true
    toc_float: true
    toc_depth: 5
    number_sections: true
vignette: >
  %\VignetteIndexEntry{Creation of standard in-text summary tables}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

# Introduction

```{r options, echo = FALSE}
	
	library(knitr)
	opts_chunk$set(
		echo = TRUE, results = 'asis', warning = FALSE, 
		# stop document execution if error (not the default)
		error = FALSE, message = FALSE, cache = FALSE,
		fig.width = 8, fig.height = 7,
		fig.path = "./figures_vignette/",
		fig.align = 'center')
	# instead of warn = 0 by default
	# include warnings when they occur in the document
	options(width = 170)
	
```

```{r loadPackages, warning = FALSE}

	library(clinUtils)
	library(tools)# toTitleCase
	library(plyr) # for ddply, rbind.fill
	library(inTextSummaryTable)

```

## Data format

The package is demonstrated with a subset of the ADaM datasets 
from the CDISC Pilot 01 dataset, available in the `clinUtils` package.

```{r loadData}	

	# load example data
    library(clinUtils)

    # load example data
    data(dataADaMCDISCP01)

    dataAll <- dataADaMCDISCP01
    labelVars <- attr(dataAll, "labelVars")

```

Typical in-text table for the CSR are included in the following sections.

Please note that the **table content** e.g. variables, statistics of interest
**depends strongly on the study at hand and personal preferences**.

# Subject information

## Subject disposition

```{r table-subjectDisposition}
	
	# data of interest
	dataDM <- dataAll$ADSL
	
	varDMFL <- grep("FL$", colnames(dataDM), value = TRUE)
	varDMFLLabel <- sub(" Flag$", "", labelVars[varDMFL])
	
	getSummaryStatisticsTable(
		data = dataDM,
		var = varDMFL, varFlag = varDMFL, varGeneralLab = "Analysis Set, N", 
		varLab = varDMFLLabel,
		stats = getStats("n (%)"),
		colVar = "TRT01P",
		labelVars = labelVars,
		colTotalInclude = TRUE, colTotalLab = "All subjects",
		varInclude0 = TRUE,
		title = toTitleCase("Table: subject disposition"),
		file = file.path("tables_CSR", "Table_subjectDisposition.docx")
	)

```

## Demographics

```{r table-demography}

	# data of interest
	dataDM <- subset(dataAll$ADSL, SAFFL == "Y")
	
	# variables of interest
	# Note: if available: ethnicity is included
	varsDM <- c(
		"SEX", "AGE", "AGEGR1",
		"RACE", "ETHNIC",
		"HEIGHTBL", "WEIGHTBL", 
		"BMIBL", "BMIBLGR1"
	)

	# Sort variables according to corresponding numeric variable
	dataDM$AGEGR1 <- with(dataDM, reorder(AGEGR1, AGEGR1N))
	dataDM$RACE <- with(dataDM, reorder(RACE, RACEN))
	dataDM$TRT01P <- with(dataDM, reorder(TRT01P, TRT01PN))
	
	## Define set of statistics of interest:
	statsDM <- getStatsData(
		data = dataDM, var = varsDM,
		# different for continuous and categorical variable
		type = c(cont = "median (range)", cat = "n (%)"),
		# for categorical variable, statistic name (here: 'n (%)')
		# should not be included in the table
		args = list(cat = list(includeName = FALSE))
	)

	## create the table:
	
	getSummaryStatisticsTable(
		data = dataDM, 
		# variables to summarize
		var = varsDM, 
		varGeneralLab = "Parameter",
		# column
		colVar = "TRT01P", colTotalInclude = TRUE, colTotalLab = "All subjects",
		# statistics
		stats = statsDM,
		statsGeneralLab = "",
		labelVars = labelVars,
		# if only one category, should be included in separated row (e.g. RACE: White)
		rowAutoMerge = FALSE,
		rowInclude0 = FALSE, emptyValue = 0,
		title = toTitleCase("Table: Demographic Data (safety Analysis Set)"),
		file = file.path("tables_CSR", "Table_demographicData.docx")
	)
	
```

## Baseline disease characteristics

Please note that the content of the table strongly depends on the study.

```{r table-baselineDiseaseCharacteristics}

	# data of interest
	dataBDC <- subset(dataAll$ADSL, SAFFL == "Y")
	
	# create table
	getSummaryStatisticsTable(
		data = dataBDC,
		var = c("DURDIS", "EDUCLVL"), varGeneralLab = "Parameter", 
		colVar = "TRT01P", colTotalInclude = TRUE, colTotalLab = "All subjects",
		stats = getStats("median\n(range)"), statsGeneralLab = "",
		rowAutoMerge = FALSE,
		labelVars = labelVars,
		title = toTitleCase("Table: Baseline Disease Characteristics (safety analysis set)"),
		file = file.path("tables_CSR", "Table_BaselineCharacteristics.docx")
	)

```

## Medical History and Concomitant Diseases

```{r table-MH}

	dataCM <- subset(dataAll$ADCM, SAFFL == "Y")

	# sort variable according to corresponding numeric variables
	dataCM$TRTA <- with(dataCM, reorder(TRTA, TRTAN))
	
	# Terms should be in lower-case
	dataCM$CMDECOD <- simpleCap(tolower(dataCM$CMDECOD))
	dataCM$CMCLAS <- simpleCap(tolower(dataCM$CMCLAS))
			
	getSummaryStatisticsTable(
		data = dataCM,
		colVar = "TRTA", colTotalInclude = TRUE, colTotalLab = "All subjects",
		rowVar = c("CMCLAS", "CMDECOD"), 
		# include total across generic terms and across ATC4 classes
		rowVarTotalInclude = c("CMCLAS", "CMDECOD"), 
		rowTotalLab = "Any prior and concomitant medication",
		stats = getStats("n (%)"),
		# sort rows based on counts of subjects in the total column 
		rowOrder = "total",
		labelVars = labelVars,
		emptyValue = 0,
		title = toTitleCase(paste("Prior and concomitant therapies",
			"by medication class and generic term (safety analyis set)"
		)),
		file = file.path("tables_CSR", "Table_CM.docx")
	)
	
```

# Efficacy Analyses

The example dataset has has two primary endpoints:

* ADAS-Cog (11), a.k.a Alzheimer's Disease Assessment Scale - Cognitive Subscale
a metric containing 11 items, available in the `ADQSADAS` dataset
* CIBIC+ score a.k.a Video-referenced Clinician's Interview-based Impression of Change
 available in the `ADQSCIBC` dataset
 
```{r table-efficacy}

	dataAdasCog11 <- subset(dataAll$ADQSADAS, PARAMCD == "ACTOT")
	dataCIBIC <- subset(dataAll$ADQSCIBC, PARAMCD == "CIBICVAL")
	
	dataEfficacy <- plyr::rbind.fill(dataAdasCog11, dataCIBIC)
	
	dataEfficacy$TRTP <- with(dataEfficacy, reorder(TRTP, TRTPN))
	dataEfficacy$AVISIT <- with(dataEfficacy, reorder(AVISIT, AVISITN))
	
	stats <- getStatsData(
		data = dataEfficacy, 
		var = c("AVAL", "CHG"), 
		type = c("n", "mean (se)", "median (range)")
	)
	
	getSummaryStatisticsTable(
		data = dataEfficacy,
		rowVar = "PARAM",
		colVar = c("TRTP", "AVISIT"),
		var = c("AVAL", "CHG"), 
		stats = stats,
		labelVars = labelVars,
		title = paste("Table: efficacy endpoints", 
			toTitleCase("actual value and changes from baseline per time point"				
		)),
		file = file.path("tables_CSR", "Table_efficacy.docx")
	)
	
```

# Safety Analyses

## Adverse Events

### Treatment-emergent summary table

```{r table-summaryTable}

	## data of interest: safety analysis set and treatment-emergent
	dataTEAE <- subset(dataAll$ADAE, SAFFL == "Y" & TRTEMFL == "Y")
	
	# order treatment and severity categories
	dataTEAE$TRTA <- with(dataTEAE, reorder(TRTA, TRTAN))
	
	## data considered for the total
	dataTotalAE <- subset(dataAll$ADSL, SAFFL == "Y")
	dataTotalAE$TRTA <- with(dataTotalAE, reorder(TRT01A, TRT01AN))
	
	# TEAE with worst intensity
	# build worst-case scenario
	dataTEAE$AESEV <- factor(dataTEAE$AESEV, levels = c("MILD", "MODERATE", "SEVERE"))
	dataTEAE$AESEVN <- as.numeric(dataTEAE$AESEV)
	dataTEAE <- ddply(dataTEAE, c("USUBJID", "TRTA"), function(x)
		cbind.data.frame(x, 
			WORSTINT = with(x, ifelse(AESEVN == max(AESEVN), as.character(AESEV), NA_character_))
	))
	dataTEAE$WORSTINT <- factor(dataTEAE$WORSTINT, levels = levels(dataTEAE$AESEV))
	
	## specify labels for each variable:
	varsAE <- c("TRTEMFL", "AESER", "AESDTH", "AEREL")
	
	# create the table
	getSummaryStatisticsTable(
		data = dataTEAE,
		colVar = "TRTA",
		# define variables to compute statistics on
		var = c("TRTEMFL", "AESER", "WORSTINT", "AESDTH", "AEREL"), 
		varFlag = c("TRTEMFL", "AESER", "AESDTH"),
		varLab = c(TRTEMFL = "Treatment-Emergent", WORSTINT = "Worst-case severity:"),
		varGeneralLab = "Subjects with, n(%):",
		# force the inclusion of lines for variable without count:
		varInclude0 = TRUE,
		# include the total for the worst-case scenario
		varTotalInclude = "WORSTINT",
		# statistics:
		stats = getStats('n (%)'),
		emptyValue = "0",
		labelVars = labelVars,
		# dataset used for the total in the header column (and for percentage as default)
		dataTotal = dataTotalAE,
		# title/export
		title = toTitleCase("Table: Summary Table of Treatment-emergent Adverse Events (safety analysis set)"),
		file = file.path("tables_CSR", "Table_TEAE_summary.docx")
	)
	
```

### Treatment-emergent incidence table

#### Events occuring in at least one subject

```{r table-TEAE}

	dataTEAE <- subset(dataAll$ADAE, SAFFL == "Y" & TRTEMFL == "Y")
	
	# order treatment and severity categories
	dataTEAE$TRTA <- with(dataTEAE, reorder(TRTA, TRTAN))
	
	## data considered for the total
	dataTotalAE <- subset(dataAll$ADSL, SAFFL == "Y")
	dataTotalAE$TRTA <- with(dataTotalAE, reorder(TRT01A, TRT01AN))
	
	getSummaryStatisticsTable(
		data = dataTEAE,
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		## total
		# data
		dataTotal = dataTotalAE,
		# row total
		rowVarTotalInclude = c("AESOC", "AEDECOD"), rowTotalLab = "Any TEAE",
		stats = getStats("n (%)"),
		labelVars = labelVars,
		rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term,\nn (%)"),
		# sort rows based on the total column:
		rowOrder = "total", 
		rowOrderTotalFilterFct = function(x) subset(x, TRTA == "Total"),
		title = paste("Table: Treatment-emergent Adverse Events by System Organ Class",
			"and Preferred Term (Safety Analysis Set)"
		),
		file = file.path("tables_CSR", "Table_TEAE_SOCPT_atLeast1Subject.docx")
	)

```

#### Events occuring in at least 25% of all subjects

```{r table-TEAE-inAtLeast25Percent}

	getSummaryStatisticsTable(
		data = dataTEAE,
		rowVar = c("AESOC", "AEDECOD"),
		colVar = "TRTA",
		## total
		# data
		dataTotal = dataTotalAE, 
		# row total
		rowVarTotalInclude = c("AESOC", "AEDECOD"), rowTotalLab = "Any TEAE",
		stats = getStats("n (%)"),
		labelVars = labelVars,
		rowVarLab = c('AESOC' = "SOC and Preferred Term,\nn (%)"),
		# sort rows based on the total column:
		rowOrder = "total", 
		rowOrderTotalFilterFct = function(x) subset(x, TRTA == "Total"),
		title = paste("Table: Treatment-emergent Adverse Events by System Organ Class",
			"and Preferred Term reported in at least 25% of the subjects",
			"in any treatment group (Safety Analysis Set)"
		),
		file = file.path("tables_CSR", "Table_TEAE_SOCPT_atLeast25PercentsSubject.docx"),
		# include only events occuring in at least 25% for at least one preferred term:
		filterFct = function(x)
			ddply(x, "AESOC", function(x){ # per AESOC to include the total
				ddply(x, "AEDECOD", function(y){
					yTotal <- subset(y, grepl("Total", TRTA))
					if(any(yTotal$statPercN >= 25))	y
				})
			})
	)

```

### Treatment-emergent worst-case table

```{r tableTEAE-worstCase}

	dataTEAE <- subset(dataAll$ADAE, SAFFL == "Y" & TRTEMFL == "Y")
	
	# order treatment and severity categories
	dataTEAE$TRTA <- with(dataTEAE, reorder(TRTA, TRTAN))
	
	## data considered for the total
	dataTotalAE <- subset(dataAll$ADSL, SAFFL == "Y")
	dataTotalAE$TRTA <- with(dataTotalAE, reorder(TRT01A, TRT01AN))
	
	# TEAE with worst intensity
	dataTEAE$AESEV <- factor(dataTEAE$AESEV, levels = c("MILD", "MODERATE", "SEVERE"))
	dataTEAE$AESEVN <- as.numeric(dataTEAE$AESEV)
	
	# extract worst-case scenario data (only one record if multiple with same severity)
	dataAEWC <- ddply(dataTEAE, c("AESOC", "AEDECOD", "USUBJID", "TRTA"), function(x){
		x[which.max(x$AESEVN), ]
	})
	# worst-case scenario in lower case
	dataAEWC$WORSTINT <- simpleCap(tolower(dataAEWC$AESEV))
	labelVars["WORSTINT"] <- "Worst-case scenario"

	## datasets used for the total: 
	# for total: compute worst-case across SOC and across AE term
	# (otherwise patient counted in multiple categories if present different categories for different AEs)
	dataTotalRow <- list(
		# within SOC (across AEDECOD)
		'AEDECOD' = ddply(dataAEWC, c("AESOC", "USUBJID", "TRTA"), function(x){	
			x[which.max(x$AESEVN), ]
		}),
		# across SOC
		'AESOC' = ddply(dataAEWC, c("USUBJID", "TRTA"), function(x){	
			x[which.max(x$AESEVN), ]
		})
	)
	
	getSummaryStatisticsTable(
		data = dataAEWC,
		## row variables:
		rowVar = c("AESOC", "AEDECOD", "WORSTINT"), rowVarInSepCol = "WORSTINT",
		# include total across SOC and across AEDECOD
		rowVarTotalInclude = c("AESOC", "AEDECOD"), dataTotalRow = dataTotalRow, 
		rowVarTotalByVar = "WORSTINT", # count for each severity category for the total
		rowTotalLab = "Any TEAE", rowVarLab = c(AESOC = "Subjects with, n(%):", WORSTINT = "Worst-case scenario"),
		# sort per total in the total column
		rowOrder = "total", 
		## column variables
		colVar = "TRTA", 
		stats = getStats("n (%)"),
		emptyValue = "0",
		labelVars = labelVars,
		dataTotal = dataTotalAE,
		title = toTitleCase(paste("Table: Treatment-emergent Adverse",
			"Events by system organ",
			"and preferred term by worst-case (safety Analysis Set)"
		)),
		file = file.path("tables_CSR", "Table_TEAE_Severity.docx")
	)
	
```

## Laboratory safety

### Table of laboratory abnormalities

```{r tableLab}

	dataLBAbn <- subset(dataAll$ADLBC, SAFFL == "Y" & LBNRIND != "NORMAL")
	
	dataLBAbn$PARAM <- with(dataLBAbn, reorder(PARAM, PARAMN))
	dataLBAbn$TRTA <- with(dataLBAbn, reorder(TRTA, TRTAN))
	dataLBAbn$LBNRIND <- factor(dataLBAbn$LBNRIND, levels = c("LOW", "HIGH"))

	dataLBAbnTotal <- subset(dataAll$ADSL, SAFFL == "Y")
	dataLBAbnTotal$TRTA <- with(dataLBAbnTotal, reorder(TRT01A, TRT01AN))
	
	getSummaryStatisticsTable(
		data = dataLBAbn,
		rowVar = c("PARCAT1", "PARAM"), 
		rowVarTotalInclude = c("PARCAT1", "PARAM"),
		colVar = "TRTA", 
		var = "LBNRIND", 
		rowVarInSepCol = "variableGroup", varSubgroupLab = "Abnormality",
		rowVarLab = c('PARCAT1' = "Laboratory Parameter\nn (%)"),
		stats = getStats("n (%)"),
		labelVars = labelVars,
		rowOrder = c("PARCAT1" = "total", "PARAM" = "total", "variableGroup" = "auto"),
		dataTotal = dataLBAbnTotal, 
		title = toTitleCase(paste("Table: Treatment-emergent",
			"Worst-case Laboratory Abnormalities (safety analysis set)"
		)),
		emptyValue = "0",
		file = file.path("tables_CSR", "Table_Lab_Severity.docx")
	)

```

## Electrocardiogram

Please note that there is no ECG dataset in the CDISC Pilot dataset
used for the examples, so this table is not effectively created
in the vignette.

Nevertheless, an example code is provided below to create a standard
table of summary statistics for the ECG parameters.

```{r ECG-formatData, eval = FALSE}

	# data of interest
	paramsECG <- c("QT", "QTCF", "QRS", "PR", "RR", "EGHR")

	dataECG <- subset(dataAll$ADEG, SAFFL == "Y" & PARAMCD %in% paramsECG)
	dataECG$TRTA <- with(dataECG, reorder(TRTA, TRTAN))
	dataECG$PARAM <- with(dataECG, reorder(PARAM, PARAMN))
	
	# consider all non-missing post-baseline records
	dataECGPostBaseline <- subset(dataECG, 
		AVISIT %in% c("Screening", "Baseline", "Worst-case post-baseline")
	)
	
	# worst-case scenario:
	dataECGWC <- subset(dataECG, AVISIT == "Worst-case post-baseline")
	# treatment-emergent
	dataECGWC$TRTEMFL <- with(dataECGWC, ifelse(BASECAT1 != CHGCAT1, "Y", "N"))
	dataECGWCTE <- subset(dataECGWC, TRTEMFL == "Y")
	dataECGWC <- convertVarToFactor(dataECGWC, 
		var = c("AVALCAT1", "CHGCAT1"), 
		varNum = c("AVALCA1N", "CHGCAT1N")
	)
	
	# create the table
	getSummaryStatisticsTable(
		data = dataECGWC,
		# layout:
		colVar = "TRTA",
		rowVar = "PARAM", rowVarLab = c('PARAM' = "ECG Parameter"),
		# metrics to compute statistics on
		var = c("AVALCAT1", "CHGCAT1"),
		# in a separated column
		rowVarInSepCol = c("variable", "variableGroup"),
		# labels
		varGeneralLab = "Abnormality",
		varSubgroupLab = "Worst-Case Post-Baseline",
		stats = getStats("n (%)"),
		labelVars = labelVars,
		# total: all post-baseline
		dataTotal = dataECGPostBaseline, 
		emptyValue = "0",
		rowVarTotalPerc = "PARAM", # total per parameter
		# ensure that categories are below the type of abnormality
		rowAutoMerge = FALSE,
		# only retain abnormalities:
		filterFct = function(x){
			subset(x, !variableGroup %in% c("<= 450 msec", "<= 30 msec"))
		},
		title = toTitleCase(paste("Table: Treatment-emergent worst-case",
			"ECG abnormalities and change from baseline ECG abnormalities (safety analysis set)"
		)),
		file = file.path("tables_CSR", "Table_ECG.docx")
	)
	
```

## Vital signs

### Treatment-emergent vital signs abnormalities

```{r tableVitalSigns}

	# analyis set and parameters of interest
	dataVS <- subset(dataAll$ADVS, 
		SAFFL == "Y" & ANL01FL == "Y" & VISIT != "BASELINE"
	)
	
	dataVS$PARAM <- with(dataVS, reorder(PARAM, PARAMN))
	dataVS$ANRIND <- with(dataVS, reorder(PARAM, PARAMN))
	dataVS$TRTA <- with(dataVS, reorder(TRTA, TRTAN))
	dataVS$SHIFT1 <- with(dataVS, factor(ifelse(SHIFT1 == "", NA_character_, SHIFT1)))
			
	getSummaryStatisticsTable(
		data = dataVS,
		rowVar = "PARAM", 
		rowVarInSepCol = "variableGroup", 
		rowVarInclude0 = TRUE,
		colVar = "TRTA", 
		var = "SHIFT1", varTotalInclude = TRUE,
		emptyValue = 0,
		stats = getStats("n (%)"),
		rowVarTotalPerc = "PARAM",
		labelVars = labelVars,
		title = toTitleCase(paste("Table: Treatment-emergent Worst-case",
			"Vital Sign Abnormalities (Safety Analysis Set)"
		)),
		file = file.path("tables_CSR", "Table_VitalSigns_Severity.docx")
	)

```

# Pharmacokinetics analysis

Please note that this example pharmacodynamics dataset contains different
subjects than the other datasets used in the vignette.

```{r PK}

	paramcdPK <- c("AUCINFO", "CMAX", "TMAX")
	dataPK <- subset(dataAll$ADPP, PKFL == "Y" & PARAMCD %in% paramcdPK)
	
	dataPK$PARCAT1 <- with(dataPK, reorder(PARCAT1, PARCAT1N))
	dataPK$PARAMCD <- with(dataPK, reorder(PARAMCD, PARAMN))
	dataPK$TRTA <- with(dataPK, reorder(TRTA, TRTAN))
	dataPK$PARAMCD <- with(dataPK, reorder(PARAMCD, PARAMN))
	
	# build pretty labels
	labelsPK <- c(
		AUCINFO = "AUC_{Inf,obs}\n(h*ng/mL)",
		CMAX = "C_{max}\n(ng/mL)",
		TMAX = "t_{max}\n(h)"
	)
	dataPK$PARAM <- factor(dataPK$PARAMCD, 
		levels = levels(dataPK$PARAMCD), 
		labels = labelsPK[levels(dataPK$PARAMCD)]
	)
	
	statsPK <- dlply(dataPK, "PARAM", function(dataParam){
		getStatsData(
			data = dataParam,
			var = "AVAL",
			type = "median\n(range)",
			includeName = FALSE
		)[[1]]
	})
	
	getSummaryStatisticsTable(
		data = dataPK,
		rowVar = c("PARCAT1", "PARAM"), colVar = "TRTA",
		var = "AVAL",
#		rowVarLab = c('PARCAT1' = "PK parameters"),
		stats = statsPK, statsVarBy = "PARAM",
		emptyValue = "-",
		title = toTitleCase("Table: Summary of PK parameters (pharmacokinetics analysis set)"),
		file = file.path("tables_CSR", "Table_PK_Parameters.docx"),
		labelVars = labelVars
	)

```

# Appendix

## Session information

```{r includeSessionInfo, echo = FALSE, results = "asis"}
print(sessionInfo())
```

---
title: "Introduction to the `inTextSummaryTable` package"
author: "Laure Cougnaud"
date: "`r format(Sys.Date(), '%B %d, %Y')`"
output: 
  rmarkdown::html_document:
    toc: true
    toc_float: true
    toc_depth: 5
    number_sections: true
vignette: >
  %\VignetteIndexEntry{Introduction to the inTextSummaryTable package}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

# Introduction

```{r options, echo = FALSE}
	
	library(knitr)
	opts_chunk$set(
		echo = TRUE, results = 'markup', warning = FALSE, 
		# stop document execution if error (not the default)
		error = FALSE, 
		message = FALSE, cache = FALSE,
		fig.width = 8, fig.height = 7,
		fig.path = "./figures_vignette/",
		fig.align = 'center')
	options(width = 170)
	# instead of warn = 0 by default
	# include warnings when they occur in the document
	options(warn = 1)
	
```

The package `inTextSummaryTable` contains functionalities to **create
complex table of summary statistics or counts of variables of interest**.

This table can be formatted to static

* **in-text table** for a **Clinical Study Report**,
* a **topline presentation**
* an **interactive table**, e.g. to be used inside the **medical
  monitoring report**.

These tables can be included into a `rmarkdown` document which can be
converted into `html`, `docx` and `pptx`.
Each table can be separately exported into a `text`, `docx` or `html `document.

We start the guidance to the functionalities of the `inTextSummaryTable`, via
loading the following packages.

```{r loadPackages}

	library(inTextSummaryTable)
	library(tools) # toTitleCase

```

## Data format

The input data for the creation of summary table should be a **data.frame**,
usually loaded from a _SAS_ data file (`sas7bdat` format).
The label of the variables stored in the `SAS` datasets is also used
for the title or the caption. 

Note that the `loadDataADaMSDTM` function of the `clinUtils` package 
can be used to import your study-specific dataset(s) of interest into R.

For this vignette, ADaM datasets included in the `clinUtils` package are used.

```{r loadData}	
	
	library(clinUtils)

	# load example data
    data(dataADaMCDISCP01)
	
	dataAll <- dataADaMCDISCP01
    labelVars <- attr(dataAll, "labelVars")
	
	# dataAll contains a list
	class(dataAll)
	# ... of ADaM datasets
	names(dataAll)
	# access a specific dataset
	head(dataAll$ADSL, 1)
	
	# check label of a subset of the variable(s)
	head(labelVars)
	# or for a specific variable:
	labelVars["USUBJID"]
	# or:
	getLabelVar(var = "USUBJID", labelVars = labelVars)

```

# Create your tables

A dedicated vignette on how to create your in-text table is available
[here](../doc/inTextSummaryTable-createTables.html)).

# Table formatting/export

If you are familiar with the creation of the tables, but you are wondering how
to:

* **export tables for specific formats as Word/PowerPoint/Html/R object**
* **save tables into separated documents**
* set specific **text formatting** (e.g. superscripts/subscripts)
* **filter and split** exported tables

then the the dedicated vignette is
available [here](../doc/inTextSummaryTable-exportTables.html)), or accessible
with

```{r getVignetteExport, eval = FALSE}

vignette("inTextSummaryTable-exportTables", "inTextSummaryTable")

```

# Advanced users

If you are familiar with the creation and export of the tables, and you want to
know more on the functionalities less exposed to the users, you can check out
the vignette for advanced users, which is
available [here](../doc/inTextSummaryTable-advanced.html)) and with

```{r getVignetteAdvanced, eval = FALSE}

vignette("inTextSummaryTable-advanced", "inTextSummaryTable")

```

# Visualization

The `inTextSummaryTable` has also visualization functionalities based on the
computation of summary statistics. The vignette for visualization is available
at [this](../doc/inTextSummaryTable-visualization.html)) link or through

```{r getVignetteVisualization, eval = FALSE}

vignette("inTextSummaryTable-visualization", "inTextSummaryTable")

``` 

# Aesthetics

The `inTextSummaryTable` package allow the user to set global options for the
color schemes of tables and visualization. The main advantage is that if
you wish to change the default palettes, it is possible to set your preferences
only once at the beginning of the R script or Rmd document.

The vignette is available
via [this](../doc/inTextSummaryTable-visualization.html)) link or

```{r getVignetteAesthetics, eval = FALSE}

vignette("inTextSummaryTable-aesthetics", "inTextSummaryTable")

```

# Appendix

## Session information

```{r includeSessionInfo, echo = FALSE, results = "asis"}
print(sessionInfo())
```

---
title: "Exporting of in-text tables"
author: "Laure Cougnaud"
date: "`r format(Sys.Date(), '%B %d, %Y')`"
output: 
  rmarkdown::html_document:
    toc: true
    toc_float: true
    toc_depth: 5
    number_sections: true
vignette: >
  %\VignetteIndexEntry{04 - Exporting of in-text tables}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

```{r options, echo = FALSE}

library(knitr)
opts_chunk$set(
    echo = TRUE, results = 'markup', warning = FALSE, 
    # stop document execution if error (not the default)
    error = FALSE, 
    message = FALSE, cache = FALSE,
    fig.width = 8, fig.height = 7,
    fig.path = "./figures_vignette/",
    fig.align = 'center')
options(width = 170)
# instead of warn = 0 by default
# include warnings when they occur in the document
options(warn = 1)

```

In this vignette we focus on exporting tables created with the
`inTextSummaryTable` package.

In particular, we will cover

* how to **export tables for specific formats as Word/Powerpoint/Html/R object**
* how to **make your custom aesthetics**
* how to **save tables into separated documents**
* how to set specific **text formatting** (e.g. superscripts/subscripts)
* how to **create multiple tables** at once based on a variable of interest
* how to **create a listing**
* how to **filter and split** exported tables

We assume you are already familiar on how to create tables, otherwise we advise
to first check out the dedicated vignette on how to make tables: 
[Creation of in-text tables](inTextSummaryTable-02-createTables.html)

We will first create example data sets to show how the exporting functionalities
work. The data sets used are available in the `clinUtils` package.

# Load packages and data


```{r loadPackages}

library(inTextSummaryTable)
library(clinUtils)
library(tools) # toTitleCase

```

```{r loadData}	

# load example data
data(dataADaMCDISCP01)

dataAll <- dataADaMCDISCP01
labelVars <- attr(dataAll, "labelVars")

```

This section creates example data sets used below.

```{r createExampleData}

dataAE <-  subset(dataAll$ADAE, SAFFL == "Y" & TRTEMFL == "Y")
dataAEInterest <- subset(dataAE, AESOC %in% c(
        "INFECTIONS AND INFESTATIONS",
        "GENERAL DISORDERS AND ADMINISTRATION SITE CONDITIONS"
    )
)

# ensure that order of elements is the one specified in 
# the corresponding numeric variable
dataAEInterest$TRTA <- reorder(dataAEInterest$TRTA, dataAEInterest$TRTAN)
dataAEInterest$AESEV <- factor(dataAEInterest$AESEV, levels = c("MILD", "MODERATE"))

dataTotalAE <- subset(dataAll$ADSL, TRT01A != "Placebo")
# should contain columns specified in 'colVar'
dataTotalAE$TRTA <- dataTotalAE$TRT01A 


```

# Output formats {#outputFormat}

The default exported format is **`flextable`**, which is **suitable for Word/PowerPoint** documents.

Alternatively, a table can be exported as **datatable interactive table for html reports**, 
or as a **data frame in R**.

The `outputType` parameter specifies the output format of the summary statistics
table.

## Static table for Word/Powerpoint

### General format

To get the table in a format **suitable for _Word_/_Powerpoint_** documents, the
**outputType** parameter should be set to **'flextable'** (the default).

```{r outputType-flextable}

summaryTableFt <- getSummaryStatisticsTable(
    data = dataAEInterest,
    rowVar = c("AESOC", "AEDECOD"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    dataTotal = dataTotalAE,
    labelVars = labelVars,
    title = "Table: Adverse Events by System Organ Class and Preferred Term",
    footer = c(
        "N = number of subjects in the data; n = number of subjects with observation",
        paste("Denominator for percentage calculations is the total number of subjects",
            "per treatment group in the safety population"
        )
    ),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)"),
    outputType = "flextable"
)
class(summaryTableFt)
summaryTableFt

```

The object `summaryTableFt` is a **flextable object**.

This is available through the [`flextable` R
package](https://cran.r-project.org/package=flextable) (see `?flextable`).

When printed into the R console, a flextable object is displayed in the
internet browser. However, the
`summaryTableFt` object can be inserted within a `rmarkdown` document by
printing it in the specified document chunk, providing that the following
system requirement are satisfied:

* `flextable` version >= 0.4.7
* for a `docx` document (`output_type`):
  [pandoc](https://pandoc.org/installing.html) version >= 2.0 (see [`rmarkdown`
  documentation for
  Word](https://yihui.org/rmarkdown/word-document))
* for `pptx` document: [pandoc](https://pandoc.org/installing.html) version >=
  2.4 (see [`rmarkdown` documentation for
  Powerpoint](https://yihui.org/rmarkdown/powerpoint-presentation))

A `rmarkdown` chunk can display only one `flextable` object. To include a list
of `flextable` in a `rmarkdown` document, the function `knitPrintListObjects` of
the `clinUtils` package can be used.

### Style: report or presentation

The table can be styled via the **`style`** parameter. This parameter affects
the fontsize, font family, color of the text and background, dimensions of the
table.

Other specific parameters can be used to export the table as landscape
(`landscape`), specify custom margins (`margin`), row indent (`rowVarPadBase`),
font size and family (`fontsize` and `font`).

By default, the table is styled for a **CSR report**.

```{r style-report}

getSummaryStatisticsTable(
    data = dataAEInterest,
    rowVar = c("AESOC", "AEDECOD"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    dataTotal = dataTotalAE,
    labelVars = labelVars,
    title = "Table: Adverse Events by System Organ Class and Preferred Term",
    footer = c(
        "N = number of subjects with data; n = number of subjects with this observation",
        paste("Denominator for percentage calculations is the total number of subjects",
            "per treatment group in the safety population"
        )
    ),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)")
)

```

The table can also be styled for a **presentation**.

Note that the default dimension of a table printed in a Powerpoint document
follows the standard 4:3 size (7.5 x 10 inches).

If you wish to set the dimension for a Widescreen document, it is possible to
set a global option for this. Please, check out the section "Custom aesthetics".

```{r style-presentation}

getSummaryStatisticsTable(
    data = dataAEInterest,
    rowVar = c("AESOC", "AEDECOD"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    dataTotal = dataTotalAE,
    labelVars = labelVars,
    title = "Table: Adverse Events by System Organ Class and Preferred Term",
    footer = c(
        "N = number of subjects with data; n = number of subjects with this observation",
        paste("Denominator for percentage calculations is the total number of subjects",
            "per treatment group in the safety population"
        )
    ),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)"),
    style = "presentation"
)

```


## Interactive table for html

The summary statistics table is exported as an interactive table by setting the
parameter: **`outputType`** to **'DT'**.

A `datatable` object, created with the
[DT](https://CRAN.R-project.org/package=DT) R package is created.

Interactivity includes the functionalities to:

* filter the table
* order columns
* export the table (or a subset of this)
* include custom interactivity e.g. the possibility to have expandable
  rows/columns, see patient profiles inclusion in the medical monitoring report

Such table can be included in a _html_ `rmarkdown` report.

The interactive table can be further customized 
by setting options specific to this output (see `clinUtils::getClinDT`).

Please note that the interactive table is only created if Pandoc is available.

```{r outputType-DT, eval = rmarkdown::pandoc_available()}

getSummaryStatisticsTable(
    data = dataAEInterest,
    rowVar = c("AESOC", "AEDECOD"),
    colVar = "TRTA",
    stats = getStats("n"),
    dataTotal = dataTotalAE,
    labelVars = labelVars,
    title = "Table: Adverse Events by System Organ Class and Preferred Term",
    footer = c(
        "N = number of subjects in the data; n = number of subjects with observation",
        paste("Denominator for percentage calculations is the total number of subjects",
            "per treatment group in the safety population"
        )
    ),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)"),
    outputType = "DT",
	## DT-specific options
	buttons = c() # remove all export buttons
)

```

## Data frames for R

The summary statistics table can be stored as a R object.

Note that the exported `data.frame`s contains _attributes_ that indicate which
statistics are used, which row variables/column variables are to be shown and
how etc.

### Final table

The summary statistics data displayed in the final table object can be exported
as an R `data.frame`.

In this case the **`outputType`** should be set to **`data.frame`**.

```{r outputType-dataframe}

summaryTable <- getSummaryStatisticsTable(
    data = dataAEInterest,
    rowVar = c("AESOC", "AEDECOD"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    dataTotal = dataTotalAE,
    labelVars = labelVars,
    title = "Table: Adverse Events by System Organ Class and Preferred Term",
    footer = c(
        "N = number of subjects in the data; n = number of subjects with observation",
        paste("Denominator for percentage calculations is the total number of subjects",
            "per treatment group in the safety population"
        )
    ),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)"),
    style = "presentation",
    outputType = "data.frame"
)
print(summaryTable, split.table = Inf)
```

### Full table in long format

The table with all the computed summary statistics, the specified set of
statistics of interest and all row/column variables stored in a long format is
available by using the **`outputType`** parameter to **'data.frame-base'**.

This format is typically of interest to run quality checks of the computed
summary statistics in comparison with e.g. Table/Listing/Figures, or to compare
the statistics computed between batches of the data.

```{r outputType-dataframe-base}

summaryTableAll <- getSummaryStatisticsTable(
    data = dataAEInterest,
    rowVar = c("AESOC", "AEDECOD"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    dataTotal = dataTotalAE,
    labelVars = labelVars,
    title = "Table: Adverse Events by System Organ Class and Preferred Term",
    footer = c(
        "N = number of subjects in the data; n = number of subjects with observation",
        paste("Denominator for percentage calculations is the total number of subjects",
            "per treatment group in the safety population"
        )
    ),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)"),
    style = "presentation",
    outputType = "data.frame-base"
)
print(summaryTableAll, split.table = Inf)
```

## Differences between the format

Most of the functionalities and default parameters are the same between the
`flextable` and `DT` format, at the exception of:
 
* for the `DT` format, by default the variables and statistics are included
      in different columns: `rowVarInSepCol` is `rowVar` and `statsLayout` is
      set to 'col'
* for the `flextable` format, by default the row variables and statistics
      are nested:   `rowVarInSepCol` is set to: 'NULL' and the `statsLayout` is
      set to 'row'

# Custom aesthetics

The `inTextSummaryTable` contains functionalities to set the aesthetics of the
tables.

Those **aesthetics are set through global options**.

This has the main advantage that if you wish to change the default color scheme,
it is possible to set your preferences only once at the beginning of the R
script or Rmd document.

When loading the package, the global options for palettes get automatically
defined into the R session. If you wish to change the default colors, you
can apply your preferences by changing the global options. 

The `inTextSummaryTable` package has the following options for table aesthetics:

* **`inTextSummaryTable.colors.table.presentation`**

* **`inTextSummaryTable.pageDim.presentation`**

The first option sets custom colors for a table in a presentation mode, the
last one defines the page dimension of a Powerpoint when exporting a table.

Note that **the options have to be defined after loading the package**. This
because when loading the package, the default global options for palettes will
overwrite the custom palettes.

## Colors

As you may have already noticed if you have read the vignette till here, by
default, the reporting format makes tables with a black text on a white
background. On the contrary, the presentation format creates tables with a blue
header and white text, whereas the body is grey with black text.

If you wish to define a personal color scheme, a named vector can be created and
passed to the `options`, as shown below.

Colors can be provided in hexadecimals or in `rgb` specification. In the example
we use hexadecimals just for convenience.

Note that the `bodyBackground1` and `bodyBackground2` allow to have alternating
row colors.

```{r aesthetics-changeReportingOptions}

# create named vector
customColorTable <- c(
    # black text in the header
    'header' = "#000000",
    # green background in the header
    'headerBackground' = "#74D055FF",
    # black text in the body
    'body' = "#000000", 
    # yellow background for all rows
    'bodyBackground1' = "#FDE725FF",
    'bodyBackground2' = "#FDE725FF",
    # black footer
    'footer' = "#000000",
    # white footer background
    'footerBackground' = "#FFFFFF",
    # black line for footer
    'line' = "#000000"
)
# set options
options(inTextSummaryTable.colors.table.presentation = customColorTable)
# create the table on a dummy data set
getSummaryStatisticsTable(
    data = data.frame(USUBJID = c(1, 2)),
    style = "presentation" 
)

```

## Dimension of the page

By default, the `inTextSummaryTable` ships with a default PowerPoint in the
standard 4:3 size (7.5 x 10 inches).

However, it is often common to create a PowerPoint template with **Widescreen
size** of 16:9 which consistts of 7.50 x 13.32 inches.

Therefore, it possible to accomodate such widescreen size by providing

```{r aesthetics-changeDimPageOptions}

# set custom dimension of page for presentation
# in this example, the dimension is the widescreen size
pageDimCustom <- c(7.5, 13.32)
options(inTextSummaryTable.pageDim.presentation = pageDimCustom)
getOption("inTextSummaryTable.pageDim.presentation")

```

## Further info on asthetics

For further information about the aesthetics settings of
the package, see the vignette: [Customization of aesthetics of in-text tables](inTextSummaryTable-06-aesthetics.html)

## Set back default palettes

There is always to possibility to switch back to the default palettes of the
package:

```{r aesthetics-backDefaultPalettes}

options(inTextSummaryTable.colors.table.presentation = tableColorsPresentation)

```


# Export table to a separated file

The table can be exported to a different file by specifying a file name in the
**`file`** parameter. 

Possible formats are:

* **txt**: the raw base table is exported to a text file. This format can be
  used e.g. to run quality checks with reference Table/Listings/Figures.
* **docx**: the formatted table is exported to a Word document. Such format is
  typically of interest to share the tables to medical writers, to be
  imported directly into a Statistical Analysis Plan (if `style = 'report'`).
* **html**: an interactive table is exported to the specified html file

```{r file, eval = FALSE}

# export table to a Word document
summaryTableFt <- getSummaryStatisticsTable(
    data = dataAEInterest,
    rowVar = c("AESOC", "AEDECOD"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    dataTotal = dataTotalAE,
    labelVars = labelVars,
    file = file.path("tables_CSR", "summaryTable-AEs.docx")
)

# export interactive table to a html document
summaryTableFt <- getSummaryStatisticsTable(
    data = dataAEInterest,
    rowVar = c("AESOC", "AEDECOD"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    dataTotal = dataTotalAE,
    labelVars = labelVars,
    file = file.path("tables_CSR", "summaryTable-AEs.html")
)

# export table in raw format to a text file
summaryTableFt <- getSummaryStatisticsTable(
    data = dataAEInterest,
    rowVar = c("AESOC", "AEDECOD"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    dataTotal = dataTotalAE,
    labelVars = labelVars,
    file = file.path("tables_CSR", "summaryTable-AEs.txt")
)

# export to multiple formats at once
summaryTableFt <- getSummaryStatisticsTable(
    data = dataAEInterest,
    rowVar = c("AESOC", "AEDECOD"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    dataTotal = dataTotalAE,
    labelVars = labelVars,
    file = file.path("tables_CSR",
        c("summaryTable.txt", "summaryTable.docx", "summaryTable.html")
    )
)

```

# Export table of pre-computed summary statistics

To create a table in a similar format as the in-text tables created by the
package, from pre-computed summary statistics (e.g. from CRO or from a different
software as _SAS_) the function **`exportSummaryStatisticsTable`** can be used.

This function takes as input a **`data.frame` in long format** that should
contain the summary statistics.

The long format should be such that the

* statistic value (`statsVar`)
* statistic type (if multiple), grouping row and columns (`rowVar` and `colVar`)

are both stored as different columns in the `data.frame`.

If the column headers should contain total number of subjects, the corresponding
counts should be stored in records with the column `isTotal` set to TRUE.

```{r export}

dataDIABP <- subset(
	dataAll$ADVS, 
	SAFFL == "Y" & ANL01FL == "Y" &
	PARAMCD == "DIABP" & 
	AVISIT %in% c("Baseline", "Week 6") &
	ATPT == "AFTER LYING DOWN FOR 5 MINUTES"
)

# create example of data.frame containing statistics of interest
statsEff <- sapply(c("AVAL", "CHG"), function(var) {
      
      getStats(
          type = c("n", "mean (se)", "median (range)"),
          x = dataDIABP[[var]]
      )
      
    }, simplify = FALSE)

summaryTable <- computeSummaryStatisticsTable(
    data = dataDIABP,
    colVar = c("TRTP", "AVISIT"),
    var = c("AVAL", "CHG"), varGeneralLab = "",
    stats = statsEff,
    labelVars = labelVars
)

# format df with statistics to in-text table format
export(
    summaryTable = summaryTable,
    statsVar = c("n", "Mean (SE)", "Median (range)"),
    rowVar = "variable", rowVarLab = "Statistic",
    colVar = c("TRTP", "AVISIT"),
    colHeaderTotalInclude = TRUE,
    labelVars = simpleCap(tolower(labelVars[c("AVAL", "CHG")])),
    title = toTitleCase(
		"Table: Diastolic Blood Pressure (mmHg) statistics"
    )
)

```

# Text formatting

Specific formatted text can be specified for the `flextable` output type only.

## Superscript/subscript

Superscript and subscript are specified via the special text pattern as `a^{b}`
and `a_{b}` respectively.

To use a superscript or a subscript in the table, the text should be formatted
as: `text^{superscript}` or `text_{subscript}`.

This is for example useful to reference additional informations in a specific
footnote or specify custom variable labels.

```{r summaryTable-PP-rowVarWithLabel-rowVarLab-superscript}

dataSL <- subset(dataAll$ADSL, SAFFL == "Y")

varsSL <- c("AGE", "WEIGHTBL", "BMIBL")
labelVars[varsSL] <- c(
    "Age",
    "Weight_{t}",
    "BMI (kg/m^{2})"
)

getSummaryStatisticsTable(
    data = dataSL, 
    var = varsSL,  
    stats = getStats("n (%"),
    labelVars = labelVars,
    fontsize = 16,
    title = toTitleCase("Demographic data (Safety Analysis Set)")
)

```

## Bold and greek letters

Specific cells of the table can be highlight in bold by using the syntax:
`bold{}` in the `stats` or `statsExtra` function.

This highlighting may depend:

* on a additional variable via the `statsExtra` parameter:  
for example:
  `statsExtra <- function(data)   with(data, ifelse(ANRIND != "N", paste0("bold{", toString(AVAL), "}"), toString(AVAL))`
* on the values of computed statistics, via the `stats` parameter

Greek letters are included as they are in the table.

```{r summaryTable-PP-rowVarWithLabel-rowVarLab-bold}

getSummaryStatisticsTable(
    data = dataSL, 
    var = varsSL,  
    stats = list(
		expression(paste0(
			ifelse(statMean > statMedian, 
				paste0("bold{", roundHalfUpTextFormat(statMean, 1), "}"), 
				roundHalfUpTextFormat(statMean, 1)
			), 
			"\n(", 
				roundHalfUpTextFormat(statSE, 2),")")
        )
    ),
    labelVars = labelVars,
    fontsize = 12,
    title = toTitleCase("Demographic data (Safety Analysis Set)")
)

```

# Create multiple tables by a variable

Multiple tables can be created for each element of a variable in the dataset  by
specifying the name of the variable in the argument **`byVar`**.  
 
A list of final summary table can be included into a `rmarkdown`  document with
the `knitPrintListObjects`. 

Please note that the following chunk option should be used:
**`results = 'asis'`**.

For example, AE tables are created for each treatment:

```{r byVar, results = "asis"}

summaryTableList <- getSummaryStatisticsTable(
    data = subset(dataAE, TRTEMFL == "Y"),
    rowVar = c("AESOC", "AEDECOD"),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)"),
    colVar = "TRTA",
    byVar = "TRTA",
    stats = getStats("n (%)"),
    labelVars = labelVars,
    title = "Table: Treatment-Emergent adverse events"
)

# print the list of tables in the rmarkdown document
clinUtils::knitPrintListObjects(summaryTableList) 

```


# Listing

Listing in a similar format as the in-text summary table is created via the `getListing` function.

```{r getListing}

varsListing <- c(
	"USUBJID", "AEBODSYS", "AEDECOD", "TRTA", 
	"AESEV", "AESER", "ASTDY", "AENDY"
)

dataListing <- subset(dataAE, TRTEMFL == "Y" & AESEV == "SEVERE")
dataListing <- dataListing[, varsListing]
colnames(dataListing) <- getLabelVar(var = varsListing, labelVars = labelVars)

getListing(
	data = dataListing, 
	title = "Listing of treatment-emergent severe adverse events",
	includeRownames = FALSE
)

```

# Filter records in a table

If only a subset of the records should be displayed in the final table, a custom
filtering functions is specified via the **`filterFct`** parameter.

```{r countTable-AE-filterFct}

library(plyr)

# SOC with AE terms with at least 2 subjects
getSummaryStatisticsTable(
    data = subset(dataAE, TRTEMFL == "Y"),
    rowVar = c("AESOC", "AEDECOD"),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    filterFct = function(x)
      ddply(x, "AESOC", function(y)
            if(any(y$statN >= 2))	y			
      ),
    labelVars = labelVars,
    title = paste(
        "Table: Adverse Events by System Organ Class and",
        "Preferred Term with at least 2 patients in System Organ Class"
    )
)

# AE term with at least one term with more than 70% in the treatment
getSummaryStatisticsTable(
    data = subset(dataAE, TRTEMFL == "Y"),
    rowVar = c("AESOC", "AEDECOD"),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    filterFct = function(x)
      ddply(x, "AEDECOD", function(xTerm) {
            if(any(xTerm$statPercN >= 70))	xTerm		
          }),
    labelVars = labelVars,
    title = "Table: Adverse Events by System Organ Class and Preferred Term with at least 70% patients"
)

```

Note: to help the creation of this filtering function, the displaying data
could be extracted first via the `computeSummaryStatisticsTable` function
(instead of creating directly the in-text table via the
`getSummaryStatisticsTable`), and then used to define the `filterFct` parameter.

```{r countTable-AE-filterFct- details}

x <- computeSummaryStatisticsTable(
    data = subset(dataAE, TRTEMFL == "Y"),
    rowVar = c("AESOC", "AEDECOD"),
    rowVarLab = c('AESOC' = "TEAE by SOC and Preferred Term\nn (%)"),
    colVar = "TRTA",
    stats = getStats("n (%)"),
    labelVars = labelVars
)
head(x)

# for specific AEDECOD
xTerm <- subset(x, AEDECOD == "MYOCARDIAL INFARCTION")
# identify the record for treated patient:
subset(xTerm, grepl("Placebo", TRTA))
# keep all records (placebo + treatment) if percentage if higher than 20%:
# if no 'else' condition, nothing (NULL) is returned:
if(subset(xTerm, grepl("Placebo", TRTA))$statPercN >= 20)	xTerm
# across all AE terms:
ddply(x, "AEDECOD", function(xTerm)
      if(subset(xTerm, grepl("Placebo", TRTA))$statPercN >= 20)	xTerm
)

# format it as a function and pass it to the 'filterFct' parameter
```

# Appendix

## Session information

```{r includeSessionInfo, echo = FALSE, results = "asis"}
print(sessionInfo())
```

---
title: "Visualization of in-text table summaries"
author: "Laure Cougnaud"
date: "`r format(Sys.Date(), '%B %d, %Y')`"
output: 
  rmarkdown::html_document:
    toc: true
    toc_float: true
    toc_depth: 5
    number_sections: true
vignette: >
  %\VignetteIndexEntry{07 - Visualization of in-text table summaries}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

<style>
.zoomDiv {
  opacity: 0;
  position:fixed;
  top: 50%;
  left: 50%;
  z-index: 50;
  transform: translate(-50%, -50%);
  box-shadow: 0px 0px 50px #888888;
  max-height:100%; 
  overflow: scroll;
}

.zoomImg {
  width: 100%;
}
</style>


<script type="text/javascript">
  $(document).ready(function() {
    $('body').prepend("<div class=\"zoomDiv\"><img src=\"\" class=\"zoomImg\"></div>");
    // onClick function for all plots (img's)
    $('img:not(.zoomImg)').click(function() {
      $('.zoomImg').attr('src', $(this).attr('src'));
      $('.zoomDiv').css({opacity: '1', width: '60%'});
    });
    // onClick function for zoomImg
    $('img.zoomImg').click(function() {
      $('.zoomDiv').css({opacity: '0', width: '0%'});
    });
  });
</script>

```{r options, echo = FALSE}

library(knitr)
opts_chunk$set(
    echo = TRUE, 
    results = 'markup', warning = FALSE, 
    # stop document execution if error (not the default)
    error = FALSE, 
    message = FALSE, cache = FALSE,
    fig.width = 8, fig.height = 7,
    fig.path = "./figures_vignette/",
    fig.align = 'center')
options(width = 170)
# instead of warn = 0 by default
# include warnings when they occur in the document
options(warn = 1)

```

This vignette focuses on the visualization functionalities available within the
`inTextSummaryTable` package.

If you need to see figures bigger, just click on a graphic, and it will pop up
more readable.

Summary statistics can be visualized by:

1. **extracting the summary statistics table** with the
  `computeSummaryStatisticsTable` function (used for `getSummaryStatistics`
  function)
2. **visualizing the summary statistics via the `subjectProfileSummaryPlot`**
  function
  
By default the **`subjectProfileSummaryPlot`** plots the mean values with
**error bars** based on standard errors.

Moreover, the `subjectProfileSummaryPlot` allows the possibility to add a table
of counts of the number of subject for a specific combination of variables.

This vignette will guide the user to

* default **visualizations for A4 and presentation documents**
* **customize palettes of colors, shape, linetypes**
* **visualization with and without table of counts and by facets**
* general customizable input

However, in order to get a full overview of the functionalities the
documentation is available in the console with `? subjectProfileSummaryPlot`.

We will first create example data sets to show how the exporting functionalities
work. The data sets used here are available in the `clinUtils` package.


# Load packages and data

```{r loadPackages}
library(inTextSummaryTable)
library(clinUtils)
```

```{r loadData}	

# load example data
data(dataADaMCDISCP01)

dataAll <- dataADaMCDISCP01
labelVars <- attr(dataAll, "labelVars")

```
  
# Compute summary statistics  
  
Below we compute summary statistics for the laboratory data set.
  
```{r visualization-extractData}

dataLB <- subset(dataAll$ADLBC, grepl("Baseline|Week", AVISIT))
dataLB$AVISIT <- with(dataLB, reorder(trimws(AVISIT), AVISITN))
dataLB$TRTA <- with(dataLB, reorder(TRTA, TRTAN))

summaryTableDf <- computeSummaryStatisticsTable(
    data = dataLB,
    var = "AVAL",
    rowVar = c("PARCAT1", "PARAM"),
    colVar = c("TRTA", "AVISIT")
)

```

# Visualization for A4 document

The code below shows the default visualization, which plots the **mean with
standard errors** **suitable for an A4 document**, e.g. Word/pdf.

Note that the table below the plot is based on the counts for treatment and
visit variables, as specificed in the `colVar` argument of the code chuck above
for `computeSummaryStatisticsTable`.

```{r visualization-a4document}

# create the plot
dataPlot <- subset(
    summaryTableDf, 
    !isTotal &
     PARAM == "Alanine Aminotransferase (U/L)"
)

subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    colorVar = "TRTA",
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)

```

# Visualization for presentation

The code below shows the default visualization **suitable for a
presentation**, either in Powerpoint or ioslides.

```{r visualization-presentation, eval = requireNamespace("extrafont", quietly = TRUE) && "Tahoma" %in% extrafont::fonts()}
# 'Tahoma' font should be registered upfront to create plots with: 'presentation' style
subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    colorVar = "TRTA",
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN",
    style = "presentation"
)

```

Please note that the default font for presentation is _Tahoma_. 

If you don't have this default font available, 
the R package [_extrafont_](https://cran.r-project.org/package=extrafont)
might be useful to register fonts with your R graphics device.

# Palettes

The `inTextSummaryTable` uses default palettes available in the
`clinUtils` package.

However, palettes are fully customizable. Currently, there are two alternative
approaches to change the defaults:

* set the custom palettes as arguments in the `subjectProfileSummaryPlot`
  via the dedicated parameters **`colorPalette`**, **`shapePalette`**,
  **`linetypePalette`**
* set the **global options** into the R session.

Passing input arguments to the function might be convenient when creating one
plot.

Instead, **passing the global options is very handy when creating multiple
graphics**, so that the palettes have to be set only once at the beginning of
the R script, without the need to copy-paste input arguments in the
different visualizations.

If you want to **know more about aesthetics options of the
`inTextSummaryTable`**, see the vignette: 
[Customization of aesthetics of in-text tables](inTextSummaryTable-06-aesthetics.html)

If you want to **know more about default options from the
`clinUtils`**, you may check out the vignette of the package:

```{r getVignetteClinUtils, eval = FALSE}

vignette("clinUtils-vignette", "clinUtils")

```

Sections below show below both alternative approaches for setting the palettes. 

## Custom palettes via arguments in the function

In this section we guide on how to set custom palettes via the
`subjectProfileSummaryPlot` function.

### Color palette

This section focuses on the possibility to set the `colorPalette` argument.

```{r visualization-customColorFromArgument}

# custom color palette by setting a named vector of colors
customColorPalette <- c(
    `Xanomeline Low Dose` = "green",
    `Xanomeline High Dose` = "red",
	`Placebo` = "blue"
)
subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    colorVar = "TRTA",
    colorPalette = customColorPalette,
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)

```

### Shape and linetype palettes

This section focuses on the possibility to set the `shapePalette` and
`linetypePalette` arguments.

```{r visualization-customShapeFromArgument}

# custom shape palette
customShapePalette <- c(15, 17, 19)
subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    colorVar = "TRTA",
    shapePalette = customShapePalette,
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)

# custom linetype palette
customLinetypePalette <- c("twodash", "dotted", "longdash")
subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    colorVar = "TRTA",
    linetypePalette = customLinetypePalette,
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)

```

## Custom palettes via setting global options

The `inTextSummaryTable` package allows the user to define global options in
the R session.

As mentioned above, setting the global options is convenient because the
palettes have to be set only once in the R script or Rmd document.

Please note that the global options have to be set **after** loading the
package. This because when loading the package, the default global options
for palettes will overwrite the custom palettes.

Here in practice:

```{r exampleLoadingPackageAndOptions, eval = FALSE}

# this is OK
library(inTextSummaryTable)
options(inTextSummaryTable.colors.plot = customColorPalette)

# this does NOT set your custom palette
options(inTextSummaryTable.colors.plot = customColorPalette)
library(inTextSummaryTable)

```

### Coloring palettes

Color palettes can be set up with the `inTextSummaryTable.colors.plot` option.

```{r visualization-customColorFromOptions}

# custom color palette by setting a simple vector of colors
customColorPalette <- c("blue", "green", "orange")
options(inTextSummaryTable.colors.plot = customColorPalette)

subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    colorVar = "TRTA",
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)

# custom color palette by setting a named vector of colors
customColorPalette <- c(
    `Xanomeline Low Dose` = "green",
    `Xanomeline High Dose` = "red",
	`Placebo` = "purple"
)
options(inTextSummaryTable.colors.plot = customColorPalette)

subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    colorVar = "TRTA",
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)

```

### Shape and linetype palettes

What shown for the color palette is consistent for shape and linetypes
palettes. In this case, the global options are called
`inTextSummaryTable.shapes.plot` and `inTextSummaryTable.linetypes.plot`.

```{r visualization-customShapeFromOptions}

# custom shape palette
customShapePalette <- c(17, 19, 22)
options(inTextSummaryTable.shapes.plot = customShapePalette)

subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    colorVar = "TRTA",
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)

# custom linetype palette
customLinetypePalette <- c("dotted", "longdash", "solid")
options(inTextSummaryTable.linetypes.plot = customLinetypePalette)

subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    colorVar = "TRTA",
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)

```

## Set back default palettes

There is always to possibility to switch back to the default palettes of the
package:

```{r visualization-backDefaultPalettes}

options(inTextSummaryTable.colors.plot = clinColors)
options(inTextSummaryTable.shapes.plot = clinShapes)
options(inTextSummaryTable.linetypes.plot = clinLinetypes)

```

# Visualization without count table

The user can decide to remove the table of counts by setting the parameter
**`tableText`** to `NULL`, as the code below shows.

```{r visualization-table}

subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    colorVar = "TRTA",
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = NULL
)

```


# Visualization by facets

It is possible to directly plot multiple parameters with the **`facetVar`**
argument.

Please note that setting the `facetVar` is not compatible with adding a table
of counts underneath the plot.

```{r visualization-facets, out.width = "100%", fig.height = 9, fig.width = 9}

# create the plot
dataPlotFacets <- subset(
    summaryTableDf, 
    !isTotal &
        PARAM %in% c(
            "Alanine Aminotransferase (U/L)",
            "Albumin (g/L)",
            "Bilirubin (umol/L)",
            "Calcium (mmol/L)"
        )
)

subjectProfileSummaryPlot(
    data = dataPlotFacets,
    xVar = "AVISIT",
    colorVar = "TRTA",
    labelVars = labelVars,
    facetVar = c("PARCAT1", "PARAM"),
    useLinetype = TRUE
)

```

# Customize your visualization

The `subjectProfileSummaryPlot` has many options! A non-exaustive demonstration
is presented below.

```{r visualization-otherInputs}

# plot the median instead of mean
# no error bars
subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    meanVar = "statMedian",
    seVar = NULL,
    colorVar = "TRTA",
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)

# plot the mean with standard deviation
subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    seVar = "statSD",
    colorVar = "TRTA",
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)

# change labels
subjectProfileSummaryPlot(
    data = dataPlot,
    xVar = "AVISIT",
    xLab = "Time points",
    yLab = "Mean and Standard Errors",
    title = "Title of the plot",
    colorVar = "TRTA",
    labelVars = labelVars,
    useLinetype = TRUE,
    tableText = "statN"
)

```

# Appendix

## Session information

```{r includeSessionInfo, echo = FALSE, results = "asis"}
print(sessionInfo())
```

---
title: "Aesthetics"
author: "Michela Pasetto and Laure Cougnaud"
date: "`r format(Sys.Date(), '%B %d, %Y')`"
output: 
  rmarkdown::html_document:
    toc: true
    toc_float: true
    toc_depth: 5
    number_sections: true
vignette: >
  %\VignetteIndexEntry{Aesthetics}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

```{r options, echo = FALSE}

library(knitr)
opts_chunk$set(
    echo = TRUE, results = 'markup', warning = FALSE, 
    # stop document execution if error (not the default)
    error = FALSE, 
    message = FALSE, cache = FALSE,
    fig.width = 4, fig.height = 4,
    fig.path = "./figures_vignette/",
    fig.align = 'center')
options(width = 170)
# instead of warn = 0 by default
# include warnings when they occur in the document
options(warn = 1)

```

The `inTextSummaryTable` contains functionalities to set palettes up.

The **palettes for visualization are retrieved from the `clinUtils`**
package, whereas the **palettes for the tables are defined in the
`inTextSummaryTable`** itself.

Moreover, **palettes are set through global options**.

This has the main advantage that if you wish to change the default palettes, it
is possible to set your preferences only once at the beginning of the R script
or Rmd document.

When loading the package, the global options for palettes get
automatically defined into the R session. If you wish to change the default
color scheme, you can apply your preferences by changing the global
options. 

Below, we present what options are available and how to customize them.

# Available options

The `inTextSummaryTable` package has the following options:

* **`inTextSummaryTable.colors.table.presentation`**

* **`inTextSummaryTable.pageDim.presentation`**

* **`inTextSummaryTable.colors.plot`**

* **`inTextSummaryTable.shapes.plot`**

* **`inTextSummaryTable.linetypes.plot`**

The options with the wording _colors.table.presentation_ and
_pageDim.presentation_ define the color scheme for tables and
the dimensions of the page in a presentation, respectively.

Instead, the options with _colors.plot_, _shapes.plot_ and
_linetype.plot_ set up the palettes for the visualization functionalities.

Note that **the options have to be defined after loading the package**. This
because when loading the package, the default global options for palettes will
overwrite the custom palettes.

In this way, you can see how the options are visible after loading the package:

```{r aesthetics-showOptions}

library(inTextSummaryTable)

# options for color scheme in in presentations
options("inTextSummaryTable.colors.table.presentation")

# options for colors in plots: viridis palette
options("inTextSummaryTable.colors.plot")

# options for shapes in plots
options("inTextSummaryTable.shapes.plot")

# options for linetypes in plots
options("inTextSummaryTable.linetypes.plot")

```


# Options for reporting

In this section we discuss the options for reporting, namely the
`inTextSummaryTable.colors.table.presentation` and
`inTextSummaryTable.pageDim.presentation`.

## Colors

By default, the reporting format makes tables with a black text on a white
background.

On the contrary, the presentation format creates tables with a blue header and
white text, whereas the body is grey with black text.

Below a demonstration for a dummy data set.

```{r aesthetics-defaultsReporting}

# default report style
getSummaryStatisticsTable(
    data = data.frame(USUBJID = c(1, 2)),
    style = "report" 
)

# default presentation style
getSummaryStatisticsTable(
    data = data.frame(USUBJID = c(1, 2)),
    style = "presentation" 
)

```

If you wish to define a personal color scheme, a named vector can be created and
passed to the `options`, as shown below.

Colors can be provided in hexadecimals or in `rgb` specification. In the
example we use hexadecimals just for convenience.

Note that the `bodyBackground1` and `bodyBackground2` allow to have alternating
row colors.

```{r aesthetics-changeReportingOptions}

# create named vector
customColorTable <- c(
    # black text in the header
    'header' = "#000000",
    # green background in the header
    'headerBackground' = "#74D055FF",
    # black text in the body
    'body' = "#000000", 
    # yellow background for all rows
    'bodyBackground1' = "#FDE725FF",
    'bodyBackground2' = "#FDE725FF",
    # black footer
    'footer' = "#000000",
    # white footer background
    'footerBackground' = "#FFFFFF",
    # black line for footer
    'line' = "#000000"
)
# set options
options(inTextSummaryTable.colors.table.presentation = customColorTable)
# create the table
getSummaryStatisticsTable(
    data = data.frame(USUBJID = c(1, 2)),
    style = "presentation" 
)

```

## Dimension of the page

By default, the `inTextSummaryTable` ships with a default PowerPoint in the
standard 4:3 size (7.5 x 10 inches).

However, it is often common to create a PowerPoint template with **Widescreen
size** of 16:9 which consistts of 7.50 x 13.32 inches.

Therefore, it possible to accomodate such widescreen size by providing

```{r aesthetics-changeDimPageOptions}

# default page dimension of a powerpoint created with Rmd
getOption("inTextSummaryTable.pageDim.presentation")

# set custom dimension of page for presentation
# in this example, the dimension is the widescreen size
pageDimCustom <- c(7.5, 13.32)
options(inTextSummaryTable.pageDim.presentation = pageDimCustom)
getOption("inTextSummaryTable.pageDim.presentation")

```

# Options for visualization

The defaults palettes for visualization are retrieved from the `clinUtils`
package.

If you wish to know more about those palettes, we refer to the vignette of
`clinUtils` available with 

```{r getVignetteClinUtils, eval = FALSE}

vignette("clinUtils-vignette", "clinUtils")

```

Below a demonstration with a dummy data set.

```{r aesthetics-defaultsVisualization}

# default colors, shapes and linetypes
summaryTable <- data.frame(
    visit = c(1, 2, 1, 2), 
    TRT = c("A", "A", "B", "B"),
    statMean = rnorm(4)
)    
subjectProfileSummaryPlot(
    data = summaryTable,
    xVar = "visit", 
    colorVar = "TRT" 
)

```

For the visualization, the same spirit of the tables applies. The user can
specify a vector of colors, shapes or linetypes. The vector do not necessarly
have to be named.

```{r aesthetics-changeVisualizationOptions}

# specify colors
options(inTextSummaryTable.colors.plot = c("red", "green"))
subjectProfileSummaryPlot(
    data = summaryTable,
    xVar = "visit", 
    colorVar = "TRT" 
)

# specify shape
options(inTextSummaryTable.shapes.plot = c("circle", "square"))
subjectProfileSummaryPlot(
    data = summaryTable,
    xVar = "visit", 
    colorVar = "TRT" 
)

# specify linetypes
options(inTextSummaryTable.linetypes.plot = c("dotdash", "longdash"))
subjectProfileSummaryPlot(
    data = summaryTable,
    xVar = "visit", 
    colorVar = "TRT" 
)

```

# Set back default palettes

There is always to possibility to switch back to the default palettes of the
package:

```{r aesthetics-backDefaultPalettes}

options(inTextSummaryTable.colors.table.presentation = tableColorsPresentation)
options(inTextSummaryTable.colors.plot = clinUtils::clinColors)
options(inTextSummaryTable.shapes.plot = clinUtils::clinShapes)
options(inTextSummaryTable.linetypes.plot = clinUtils::clinLinetypes)

```

# Set up specific palettes

# Page dimensions in report/presentation 

The function **`getDimPage`** extracts dimensions available for a content in a
report or presentation, e.g. to specify the maximal width of a figure in A4
report or for a presentation.

```{r aesthetics-getDimPage}

# a4 format with one 1 inch margin
getDimPage(type = "width", style = "report")
getDimPage(type = "height", style = "report")

# Presentation format (according to template) with one 1 inch margin
getDimPage(type = "width", style = "presentation")
getDimPage(type = "height", style = "presentation")

```

# Appendix

## Session information

```{r includeSessionInfo, echo = FALSE, results = "asis"}
print(sessionInfo())
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/miscellaneous.R
\name{postProcessVarFlag}
\alias{postProcessVarFlag}
\title{Post-process the summary statistics table with variable flag.}
\usage{
postProcessVarFlag(summaryTable, varFlag)
}
\arguments{
\item{summaryTable}{Summary table as created internally in
\code{\link{computeSummaryStatisticsTable}}.}

\item{varFlag}{Character vector, subset of \code{var} with variable(s) 
of type 'flag' (with 'Y', 'N' or '' for empty/non specified value).
Only the counts for records flagged (with 'Y') are retained.}
}
\value{
Summary table with
}
\description{
This function is for internal use within
the \code{\link{computeSummaryStatisticsTable}}
function.
}
\details{
This includes:
\itemize{
\item{converting the records from a flag variable
for the 'variableGroup' variable 
from 'Y' to \code{NA_character_}
}
\item{filter records from a flag variable 
with variableGroup set as 'N'}
}
}
\author{
Laure Cougnaud
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tableColors.R
\name{tableColorsPresentation}
\alias{tableColorsPresentation}
\title{Colors for tables in a presentation style}
\usage{
tableColorsPresentation
}
\description{
Default colors are
\itemize{
\item{header: white text on a blue background}
\item{body: black text on a grey background}
\item{footer: black text on a white background.}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/computeSummaryStatisticsTable.R
\name{computeSummaryStatisticsTableTotal}
\alias{computeSummaryStatisticsTableTotal}
\title{Compute summary statistics total table.}
\usage{
computeSummaryStatisticsTableTotal(
  data,
  colVar = NULL,
  colVarTotal = colVar,
  rowVarTotal = NULL,
  var = NULL,
  varLab = getLabelVar(var, data = data, labelVars = labelVars),
  colTotalLab = "Total",
  colInclude0 = FALSE,
  colTotalInclude = FALSE,
  dataTotalCol = NULL,
  colVarDataLevels = NULL,
  colVarLevels = NULL,
  subjectVar = "USUBJID",
  labelVars = NULL,
  msgLabel = "total",
  checkVarDiffBySubj = "error"
)
}
\arguments{
\item{data}{Data.frame with dataset to consider for the summary table.}

\item{colVar}{Character vector with variable(s) to be included in columns.
If multiple variables are specified, the variables should 
be sorted in hierarchical order,
and are included in multi-columns layout.\cr
Use: 'variable' to include the variables to summarize: \code{var}
 (if multiple) in different columns.}

\item{colVarTotal}{Character vector with column(s) considered to compute the total.
This could also contain 'variable'.}

\item{rowVarTotal}{Character vector with row(s) considered to compute the total.
This could also contain 'variable'.}

\item{var}{Character vector with variable(s) of \code{data}, 
to compute statistics on.\cr
If NULL (by default), counts by row/column variable(s) are computed.\cr
To also return counts of the \code{rowVar} in case other \code{var}
are specified, you can include: 'all' in the \code{var}.\cr
Missing values, if present, are filtered 
(also for the report of number of subjects/records).}

\item{colTotalLab}{String, label for the total column  'Total' by default.\cr}

\item{colTotalInclude}{Logical, if TRUE (FALSE by default) include the summary 
statistics across columns in a separated column.}

\item{dataTotalCol}{Data.frame from which the total across columns is 
extracted (in case \code{colTotalInclude} is TRUE).}

\item{colVarLevels}{list with levels of each \code{colVar}}

\item{subjectVar}{String, variable of \code{data} with subject ID,
'USUBJID' by default.}

\item{labelVars}{(optional) Named character vector with label for
the row, column variable(s) or variable(s) to summarize. \cr
Labels specified via dedicated parameter: e.g. 
\code{rowVarLab}, \code{colVarLab}, \code{varLab}
have priority on this parameter.}

\item{msgLabel}{(optional) String with label for the data ('total' by default), 
included in the message/warning for checks.}
}
\value{
data.frame with total table.
}
\description{
Compute summary statistics total table.
}
\author{
Laure Cougnaud
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/exportSummaryStatisticsTable.R
\name{export}
\alias{export}
\title{Export an object}
\usage{
export(...)
}
\arguments{
\item{...}{Extra parameters for the corresponding
method.}
}
\description{
Export an object
}
\seealso{
\code{\link{export.summaryTable}}
to export \code{\link{summaryTable}} objects.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in
%   R/flextable-exportSummaryStatisticsTableToFlextable.R
\name{exportSummaryStatisticsTableToFlextable}
\alias{exportSummaryStatisticsTableToFlextable}
\title{Export summary table to a flextable object,
e.g. to be exported in Word or PowerPoint.}
\usage{
exportSummaryStatisticsTableToFlextable(
  summaryTable,
  rowVar = getAttribute(summaryTable, "rowVar"),
  rowVarInSepCol = NULL,
  rowVarTotalInclude = getAttribute(summaryTable, "rowVarTotalInclude"),
  statsLayout = getAttribute(summaryTable, "statsLayout", default = "row"),
  statsVar = getAttribute(summaryTable, "statsVar"),
  statsLabInclude = getAttribute(summaryTable, "statsLabInclude", default =
    length(statsVar) > 1),
  rowVarLab = getAttribute(summaryTable, "rowVarLab", default = getLabelVar(rowVar,
    labelVars = labelVars)),
  rowVarTotalInSepRow = NULL,
  vline = c("none", "auto"),
  hline = c("auto", "none"),
  rowAutoMerge = TRUE,
  rowVarFormat = NULL,
  rowTotalLab = NULL,
  colHeaderMerge = TRUE,
  landscape = (style == "presentation"),
  margin = 1,
  rowPadBase = 14.4,
  title = NULL,
  footer = NULL,
  style = "report",
  colorTable = getColorPaletteTable(style = style),
  fontname = switch(style, report = "Times", presentation = "Tahoma"),
  fontsize = switch(style, report = 8, presentation = 10),
  file = NULL,
  pageDim = NULL,
  columnsWidth = NULL,
  labelVars = NULL
)
}
\arguments{
\item{summaryTable}{Summary table as provided by the 
\code{\link{formatSummaryStatisticsTable}}}

\item{rowVar}{Character vector with variable(s)
to be included in the rows.
If multiple variables are specified, the variables should 
be sorted in hierarchical order
(e.g. body system class before adverse event term)
and are nested in the table.}

\item{rowVarInSepCol}{Character vector with \code{rowVar}
that should be included in separated columns.
By default (NULL), all row variables are nested in the first column 
of the table.\cr
To include the groups within a \code{var} variable in a separated column, 
set: \code{rowVarInSepCol == 'variableGroup'}.}

\item{rowVarTotalInclude}{Character vector with \code{rowVar}
for which the total should be reported.\cr
If the higher row variable is specified, the total across all rows
is reported. \cr
For the export, these variable(s) are formatted as factor with 
\strong{'Total' as the first level}.}

\item{statsLayout}{String with layout for the statistics names 
(in case more than one statistic is included), among:
\itemize{
\item{row (by default for 'flextable' output): \cr All statistics are
 included in different rows in the first column of the table (after the row variable(s))}
\item{'col' (by default for 'DT' output): \cr Statistics are included 
in separated columns (last row of the header).\cr
This option is not compatible with categorical variable(s).}
\item{'rowInSepCol': \cr Statistics are included in different rows, 
but in a separated column than the \code{rowVar} variable(s)}
}}

\item{statsVar}{Character vector with columns of \code{summaryTable} with
statistic variables. For the export: if not specified, all columns of \code{data}
besides row, column variables, 'variable', 'variableGroup'
and 'isTotal' are considered.}

\item{statsLabInclude}{Logical, if TRUE include the statistic label
in the table. \cr By default only included if more than
one statistic variables are available in the table.}

\item{rowVarLab}{Named character vector with 
label for the \code{rowVar} variable(s).}

\item{rowVarTotalInSepRow}{Character vector with \code{rowVarTotalInclude}
(not in \code{rowVarInSepCol}) for which the total should be included in a separated row labelled 'Total'.
Otherwise (by default) the total is included in the header row of each category.}

\item{vline}{(flextable output) String mentioning how vertical lines 
should be included in the body of the table, either: 
\itemize{
\item{'none' (default): no vertical lines included}
\item{'auto': vertical lines included between sub-groups}
}}

\item{hline}{(flextable output) String mentioning how horizontal lines 
should be included in the body of the table, either: 
\itemize{
\item{'none': no horizontal lines included}
\item{'auto' (default): horizontal lines included between sub-groups}
}}

\item{rowAutoMerge}{(flextable output) Logical, if TRUE (by default) automatically merge rows,
e.g. in case there is only one sub-category (e.g. categorical variable with only one group)
or only one statistic per category.}

\item{rowVarFormat}{(flextable output) Named list with special formatting for the \code{rowVar}.
Currently, only possibility is to set the variable elements in bold, with:
list(var1 = "bold").
(Use 'variable' for \code{var} or 'variableGroup' for group within categorical variables.)}

\item{rowTotalLab}{(flextable output) string with label for the row with total.}

\item{colHeaderMerge}{(flextable output) Logical, if TRUE (FALSE by default) 
the column header is merged.}

\item{landscape}{(flextable output) Logical, if TRUE the file is in landscape format.\cr
By default: FALSE if \code{style} is 'report' and TRUE if \code{style} is 'presentation'.}

\item{margin}{(flextable output) Margin in the document in inches
(1 by default).
This is used to specify the width of the table, from:
[\code{pageDim[1]} - 2 * margin].}

\item{rowPadBase}{(flextable output) Base padding for row (in points), 
14.4 by default (corresponds to 0.2 inches)}

\item{title}{Character vector with title(s) for the table.
Set to NULL (by default) if no title should be included.
If multiple are specified, specified for each element of \code{byVar} (in order of the levels).}

\item{footer}{(flextable output) Character vector with footer(s) for the table.
Set to NULL (by default) if no footer should be included.}

\item{style}{(flextable output) String with table style,
 either 'report' or 'presentation'.
This parameter affects the fontsize, font family, color of the text and background, 
and table dimensions of the table.}

\item{colorTable}{(flextable output) Named character vector with color for the table background/body/text/line,
e.g. created with the \code{\link{getColorPaletteTable}} function.}

\item{fontname}{(flextable output) String with font name, by default:
'Times' if \code{style} is 'report' and 'Tahoma' if \code{style} is 'presentation'.}

\item{fontsize}{(flextable output) Integer with font size, by default:
8 if \code{style} is 'report' and 10 if \code{style} is 'presentation'.}

\item{file}{String with path of the file where the table should be exported.
The file should have the extension: '.html'.
If NULL, the summary table is not exported but only returned as output.
If \code{byVar} is specified, each table is exported to a separated
file with the suffix: 'file_[i].html' with i the index of the file.}

\item{pageDim}{Numeric vector of length 2 with page width and height.\cr
Depending on \code{outputType}:
\itemize{
\item{'flextable': in inches}
\item{'DT': in number of rows in the table.\cr
Currently only the height is used (e.g. \code{c(NA, 4)})
}}}

\item{columnsWidth}{Column widths of the table, for \code{outputType}:
\itemize{
\item{'flextable': the widths can be specified either:
 \itemize{
 \item{in percentage: the sum of the widths should be 1}
 \item{in inch: the sum should be the page width (see \code{\link{getDimPage}})}
} If the widths are not specified for all columns, the width of the remaining last
 columns is set such as the width of the total table is the page width.}
\item{'DT': see \code{\link[clinUtils]{getClinDT}}}
}}

\item{labelVars}{(optional) Named character vector with label for
the row, column variable(s) or variable(s) to summarize. \cr
Labels specified via dedicated parameter: e.g. 
\code{rowVarLab}, \code{colVarLab}, \code{varLab}
have priority on this parameter.}
}
\value{
\code{\link[flextable]{flextable}} object with summary table
If \code{summaryTable} is a list of summary tables,
returns a list of \code{\link[flextable]{flextable}}.
}
\description{
Export summary table to a flextable object,
e.g. to be exported in Word or PowerPoint.
}
\author{
Laure Cougnaud
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/decimals.R
\name{getMaxNDecimals}
\alias{getMaxNDecimals}
\title{Get maximum number of decimals in a variable,
based on pre-defined rule and/or data.}
\usage{
getMaxNDecimals(x, ...)
}
\arguments{
\item{x}{Numeric vector.}

\item{...}{Any parameters for the \code{\link{getNDecimals}} function.}
}
\value{
Integer with maximum number of decimals in a character vector.
}
\description{
The function \code{\link{getNDecimals}}) extracts the number
of decimals in a specific variable.
}
\examples{
x <- c(0.99, 5.679, 50.45, 1450)
# extract max number of decimals based on data:
getMaxNDecimals(x, useRule = FALSE, useData = TRUE)
# extract max number of decimals based on pre-defined rule:
getMaxNDecimals(x, useRule = TRUE, useData = FALSE)
# extract max number of decimals based on both rules
# minimum of both is used (by default)
getMaxNDecimals(x, useRule = TRUE, useData = TRUE)
}
\seealso{
Other decimals:
\code{\link[=formatPercentage]{formatPercentage()}},
\code{\link[=getMaxNDecimalsData]{getMaxNDecimalsData()}},
\code{\link[=getNDecimals]{getNDecimals()}},
\code{\link[=getNDecimalsData]{getNDecimalsData()}}
}
\author{
Laure Cougnaud
}
\concept{decimals}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summaryTable-combine.R
\name{uniqueVarWithOrder}
\alias{uniqueVarWithOrder}
\title{Get unique variables with meaningful order.}
\usage{
uniqueVarWithOrder(...)
}
\arguments{
\item{...}{Lists}
}
\value{
Vector with unique and ordered elements.
}
\description{
The following framework is followed:
\enumerate{
\item{get the unique elements in the vectors}
\item{for each of this element: get the average order across
the different vectors}
\item{put \code{variable}, if present as second to last element}
\item{put \code{variableGroup}, if present, as last element}
\item{order the unique elements based on the extracted order}
}
}
\author{
Laure Cougnaud
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stats-utility.R
\name{inTextSummaryTable-stats-utility}
\alias{inTextSummaryTable-stats-utility}
\title{Common arguments for the for the statistics utility functions
of the inTextSummaryTable package.}
\arguments{
\item{x}{Numeric vector.}

\item{na.rm}{Logical, should NA value(s) be removed (FALSE by default)?}
}
\value{
No return value, used for the documentation of 
stat utility R functions
}
\description{
Common arguments for the for the statistics utility functions
of the inTextSummaryTable package.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/miscellaneous.R
\name{getDimPage}
\alias{getDimPage}
\title{Get dimension of the page available for content 
for standard Word report or PowerPoint presentation.}
\usage{
getDimPage(
  type = c("width", "height"),
  landscape = (style == "presentation"),
  margin = 1,
  pageDim = NULL,
  style = "report"
)
}
\arguments{
\item{type}{Character vector with dimension of interest, among: 
'width', 'height', multiple are possible.
By default: \code{c("width", "height")}}

\item{landscape}{Logical, if TRUE the table is presented in landscape
format. \cr
By default: TRUE for \code{style}: 'report',
FALSE for \code{style}: 'presentation'.}

\item{margin}{Margin in the document in inches, 1 by default.}

\item{pageDim}{(optional) Numeric vector of length 2 with 
page width and height in inches in portrait format,
in case page dimensions differ from the default
implemented report/presentation.
These dimensions should include the margins.}

\item{style}{String with table style, either 'report' (by default, a4 format) 
or 'presentation'}
}
\value{
numeric vector with dimension of interest,
in the same order as specified via the \code{type}
parameter.
}
\description{
Report is in A4 and presentation dimensions extracted from
PowerPoint.
The returned dimensions are the page dimensions without the margins.
}
\examples{
## get part of the page available for content
# report A4 portrait format:
getDimPage(type = "width")
getDimPage(type = "height")
# report A4 landscape format:
getDimPage(type = "width", landscape = TRUE)
getDimPage(type = "height", landscape = TRUE)
# Note that the layout is by default set to 'landscape'
getDimPage(type = "width", style = "presentation")
getDimPage(type = "height", style = "presentation")
# custom dimensions: A3 format
getDimPage(type = "width", pageDim = c(11.7, 16.5))
# increase margin
getDimPage(type = "width", margin = 1.5)
# get both dimensions at once
getDimPage(type = c("width", "height"))
# get dimensions of the full page (including margins)
getDimPage(type = c("width", "height"), style = "report", margin = 0)
getDimPage(type = c("width", "height"), style = "presentation", margin = 0)
}
\author{
Laure Cougnaud
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/computeSummaryStatisticsTable.R
\name{computeSummaryStatisticsTable}
\alias{computeSummaryStatisticsTable}
\title{Compute summary statistics for a specific dataset and variables of interest}
\usage{
computeSummaryStatisticsTable(
  data,
  var = NULL,
  varFlag = NULL,
  varInclude0 = FALSE,
  varLab = NULL,
  varLabInclude = length(var) > 1,
  varGeneralLab = "Variable",
  varSubgroupLab = "Variable group",
  varIgnore = NULL,
  varIncludeTotal = FALSE,
  varTotalInclude = FALSE,
  varTotalInSepRow = FALSE,
  colVar = NULL,
  colVarDataLevels = NULL,
  colVarTotal = colVar,
  colVarTotalPerc = colVarTotal,
  colTotalInclude = FALSE,
  colTotalLab = "Total",
  colInclude0 = FALSE,
  rowVar = NULL,
  rowVarDataLevels = NULL,
  rowVarLab = NULL,
  rowOrder = "auto",
  rowOrderTotalFilterFct = NULL,
  rowOrderCatLast = NULL,
  rowVarTotalInclude = NULL,
  rowVarTotalInSepRow = NULL,
  rowVarTotalByVar = NULL,
  rowVarTotalPerc = NULL,
  rowInclude0 = FALSE,
  type = "auto",
  subjectVar = "USUBJID",
  dataTotal = NULL,
  dataTotalPerc = dataTotal,
  dataTotalRow = NULL,
  dataTotalCol = NULL,
  stats = NULL,
  statsVarBy = NULL,
  statsExtra = NULL,
  statsGeneralLab = "Statistic",
  statsPerc = c("statN", "statm"),
  filterFct = NULL,
  labelVars = NULL,
  byVar = NULL,
  byVarLab = NULL,
  checkVarDiffBySubj = "error"
)
}
\arguments{
\item{data}{Data.frame with dataset to consider for the summary table.}

\item{var}{Character vector with variable(s) of \code{data}, 
to compute statistics on.\cr
If NULL (by default), counts by row/column variable(s) are computed.\cr
To also return counts of the \code{rowVar} in case other \code{var}
are specified, you can include: 'all' in the \code{var}.\cr
Missing values, if present, are filtered 
(also for the report of number of subjects/records).}

\item{varFlag}{Character vector, subset of \code{var} with variable(s) 
of type 'flag' (with 'Y', 'N' or '' for empty/non specified value).
Only the counts for records flagged (with 'Y') are retained.}

\item{varInclude0}{Logical, should rows with no counts 
for the count \code{var} or \code{varFlag} variable(s)
be included in the table?
Either:
\itemize{
\item{logical of length 1, if TRUE (FALSE by default) 
rows with no count are included for all \code{var}
}
\item{a character vector containing categorical \code{var} 
for which zero counts rows should be included}
}}

\item{varLab}{Named character vector with label for each variable 
specified in \code{var}.
By default, extracted from the \code{labelVars}.
if not available, \code{var} is used.}

\item{varLabInclude}{Logical, if TRUE
the name of the summary statistic variable(s) (\code{var})
are included in the table.
This is automatically set to TRUE if more than one variable(s) 
and is specified, and FALSE if only one variable is specified.}

\item{varGeneralLab}{String with general label for variable specified in \code{var}.
In case of multiple variable in \code{var}, this will be included in the table header
(see 'rowVarLab' attribute of the output).}

\item{varSubgroupLab}{String with general label for sub-group of
categorical variable(s) for count table, 'Variable group' by default.
This will be included in the final table header (see 'rowVarLab' attribute of the output).}

\item{varIgnore}{Vector with elements to ignore in the \code{var} variable(s).
The \code{data} records with such elements in \code{var} are \strong{filtered} from the data 
at the start of the workflow.}

\item{varIncludeTotal}{This argument is deprecated, please use: 'varTotalInclude' instead.}

\item{varTotalInclude}{Should the total across all categories of \code{var} 
be included for the count table?
Only used for categorical variables (and \code{var} not 'all').
Either:
\itemize{
\item{logical of length 1, if TRUE (FALSE by default) include the total for all categorical \code{var}}
\item{a character vector containing categorical \code{var} for which the total should be included}
}}

\item{varTotalInSepRow}{Logical, should the total per variable be included in
a separated row (by default) or in the row containing the header of the variable?}

\item{colVar}{Character vector with variable(s) to be included in columns.
If multiple variables are specified, the variables should 
be sorted in hierarchical order,
and are included in multi-columns layout.\cr
Use: 'variable' to include the variables to summarize: \code{var}
 (if multiple) in different columns.}

\item{colVarDataLevels}{Data.frame with unique combinations of \code{colVar}
to be included in columns.
Each column should correspond to \code{colVar} and as factor
if the elements should be ordered in the final table.}

\item{colVarTotal}{String with column(s) considered to compute the total by,
reported in the header of the table, by default same as \code{colVar}.
Use: 'variable' to compute total by \code{var} (if multiple).}

\item{colVarTotalPerc}{String with column(s) considered to compute the total by,
used as denominator for the percentage computation, by default same as \code{colVarTotal}.
Use: 'variable' to compute total by \code{var} (if multiple).}

\item{colTotalInclude}{Logical, if TRUE (FALSE by default) include the summary 
statistics across columns in a separated column.}

\item{colTotalLab}{String, label for the total column  'Total' by default.\cr}

\item{colInclude0}{Logical, if TRUE (FALSE by default),
include columns with no records, based on all combinations 
of the \code{columnVar} (assuming nested variable(s)).
If variable(s) are not nested, possible combinations
can be specified via \code{colVarDataLevels}.}

\item{rowVar}{Character vector with variable(s)
to be included in the rows.
If multiple variables are specified, the variables should 
be sorted in hierarchical order
(e.g. body system class before adverse event term)
and are nested in the table.}

\item{rowVarDataLevels}{Data.frame with unique combinations of \code{rowVar}
to be included in columns.
Each column should correspond to \code{colVar} and as factor
if the elements should be ordered in the final table.}

\item{rowVarLab}{Named character vector with 
label for the \code{rowVar} variable(s).}

\item{rowOrder}{Specify how the rows should be ordered in the final table, either a:
\itemize{
\item{String among:
\itemize{
\item{'auto' (by default): if the variable is a factor, keep its order, otherwise order alphabetically}
\item{'alphabetical': order alphabetically}
\item{'total': order rows in decreasing order of the total number of subjects
across all columns for this specific category.}
}}
\item{Function with input the summary table and output the ordered elements of the \code{rowVar}}
}
To specify different ordering methods for different \code{rowVar}, specify a list
of such elements, named with the \code{rowVar} variable.
For the table output of \code{\link{computeSummaryStatisticsTable}} (long format),
this order is also reflected in the \strong{\code{levels}} of the row factor variable.}

\item{rowOrderTotalFilterFct}{Function used to filter the data used to order the rows
based on total counts (in case \code{rowOrder} is 'total'),
To order rows based on one specific column category,
e.g. to order based on the counts in the treatment column:
function(x) subset(x, TRTP == "treatmentX")}

\item{rowOrderCatLast}{String with category to be printed in the last 
row of each \code{rowVar} (if any, set to NULL if none).}

\item{rowVarTotalInclude}{Character vector with \code{rowVar}
for which the total should be reported.\cr
If the higher row variable is specified, the total across all rows
is reported. \cr
For the export, these variable(s) are formatted as factor with 
\strong{'Total' as the first level}.}

\item{rowVarTotalInSepRow}{Character vector with \code{rowVarTotalInclude}
(not in \code{rowVarInSepCol}) for which the total should be included in a separated row labelled 'Total'.
Otherwise (by default) the total is included in the header row of each category.}

\item{rowVarTotalByVar}{Character vector with a row variable
used to categorize the row total.\cr
Note that this is only used if row total(s) is/are requested via \code{rowVarTotalInclude},
and this variable should also be included in \code{rowVar}.
This can be specified also for a specific row variable if the vector is named.\cr
For example: \code{c(ADECOD = "AESEV")} to compute total by severity 
for row adverse event term in a typical adverse event count table 
(by System Organ Class and Adverse Event Term).}

\item{rowVarTotalPerc}{Character vector with row variables by which the total
should be computed for the denominator for the percentage computation.
By default the total is only computed only by column (NULL by default).
If the total should be based on the total number of records per variable,
\code{rowVarTotalPerc} should be set to 'variable'.}

\item{rowInclude0}{Logical, if TRUE (FALSE by default),
include rows with no records, based on all combinations 
of the \code{rowVar} (assuming nested variable(s)).}

\item{type}{String with type of table: 
\itemize{
\item{'summaryTable': summary table with statistics for numeric variable}
\item{'countTable': count table}
\item{'auto' (by default): 'summaryTable' if the variable is numeric,
'countTable' otherwise}
}}

\item{subjectVar}{String, variable of \code{data} with subject ID,
'USUBJID' by default.}

\item{dataTotal}{Data.frame used to extract the Total number of subject
per column in column header ('N = [X]').
It should contain the variables specified by \code{colVarTotal}.
If not specified, the total number of subjects is extracted from the \code{data}.}

\item{dataTotalPerc}{Data.frame used to extract the total counts per column 
for the computation of the percentage.\cr
By default, \code{dataTotal} is used.\cr
It should contain the variables specified by \code{colVarTotalPerc}.}

\item{dataTotalRow}{Data.frame used to extract the total count across all
elements of the row
variable, list of such data.frame for each \code{rowVar} variable.\cr
If the dataset is specified by row variable, the list should be named with:
variable X if the total across elements of variable X should be included.
By default, \code{data} is used.}

\item{dataTotalCol}{Data.frame from which the total across columns is 
extracted (in case \code{colTotalInclude} is TRUE)
or list of such data.frame for each \code{rowVar} variable.\cr
If the dataset is specified by row variable, the list should be named with:
with:
\itemize{
\item{last row variable: for the dataset used in the total column for 
the most nested row variable}
\item{higher row variable (X+1): for the dataset used for the total column
and row total of X}
\item{'total': for the dataset used for the total column and general row total}
}
If only a subset of the variables is specified in this list, 
\code{data} is used for the remaining variable(s) (or 'total') if needed.\cr
This dataset (the one for 'total' if a list) is also used for:
\itemize{
\item{the header of the total column in case \code{dataTotal} is
not specified}
\item{the denominator of the percentages in the total column
in case \code{dataTotalPerc} is not specified}
}
By default, \code{data} is used.}

\item{stats}{(optional) Statistic(s) of interest to compute, either:
\itemize{
\item{string with the name of a default set of statistics
available in the package, 
see section 'Formatted statistics' in
\code{\link[=inTextSummaryTable-stats]{in-text table statistics}}.\cr
See the corresponding \code{type} parameter of the
\code{\link{getStatsData}} for more information
on how the statistic is internally extracted.}
\item{(expert mode) named list of language object (see \code{\link{is.language}}) 
of base summary statistics of interest, see section:
'Base statistics' in
\code{\link[=inTextSummaryTable-stats]{in-text table statistics}}.\cr
The names are reported in the header.\cr
If \code{stats} if of length 1, the name of the summary statistic is not included
in the table.\cr
The statistics can be specified separately:
\itemize{
\item{for each \code{var} (if multiple), 
by naming each element of the list:
\code{list(varName1 = list(...), varName2 = list())}
}
\item{and/or for each element in:
\code{statsVarBy}, by naming each sublist.}
}}
}}

\item{statsVarBy}{String with variable in \code{rowVar}/\code{colVar}
which the statistics should be computed by.\cr
In this case, \code{stats} (nested list or not) should be additionally nested
to specify the statistics for each element in \code{statsVarBy}.}

\item{statsExtra}{(optional) Named list with functions for additional custom
statistics to be computed.\cr
Each function:
\itemize{
\item{has as parameter, either: 'x': the variable (\code{var}) to compute
the summary statistic on or 'data': the entire dataset}
\item{returns the corresponding summary statistic as a numeric vector}
}
For example, to additionally compute the coefficient of variation, this can be set to:
\code{list(statCVPerc = function(x) sd(x)/mean(x)*100)} (or \code{\link{cv}}).}

\item{statsGeneralLab}{String with general label for statistics, 'Statistic' by default.
Only included if no \code{statsVar} if longer than 1.}

\item{statsPerc}{String with 'base statistical variable' used to compute the 
percentage, either: 
\itemize{
\item{'statN' (by default): the number of subjects}
\item{'statm': the number of records}
}}

\item{filterFct}{(optional) Function taking as input
the summary table with computed statistics and returning a subset 
of the summary table.\cr
Note: The filtering function should also handle records with :
\itemize{
\item{total for the column header: \code{isTotal} set to TRUE,
and \code{colVar}/\code{rowVar} is NA.\cr
For example: \code{filterFct = function(data) subset(data, isTotal & myColVar == "group 1")}
}
\item{\code{rowVar}/\code{colVar} set to 'Total'/\code{colTotalLab} 
if \code{rowVarTotalInclude}/\code{colTotalInclude} is specified}
}}

\item{labelVars}{(optional) Named character vector with label for
the row, column variable(s) or variable(s) to summarize. \cr
Labels specified via dedicated parameter: e.g. 
\code{rowVarLab}, \code{colVarLab}, \code{varLab}
have priority on this parameter.}

\item{byVar}{Variable(s) of \code{data} for which separated table(s)
should be created.}

\item{byVarLab}{String with label for \code{byVar}, used to set the names
of the output list of table(s).}

\item{checkVarDiffBySubj}{String, 'error' (default), 'warning',
or 'none'.  
Should an error, a warning, or nothing be produced
if a continuous variable (\code{var}) contains
different values for the same subject (by row/column)?}
}
\value{
An object \code{\link{summaryTable}}
or list of such objects if
\code{byVar} is specified.
}
\description{
Compute summary statistics for a specific dataset and variables of interest
}
\author{
Laure Cougnaud
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/flextable-getListing.R
\name{mergeHeaderCols}
\alias{mergeHeaderCols}
\title{Merge columns and rows in the header of a flextable object.}
\usage{
mergeHeaderCols(x, mapping)
}
\arguments{
\item{x}{a 'flextable' object, see \link[flextable]{flextable-package} to learn how to create
'flextable' object.}

\item{mapping}{a \code{data.frame} specyfing for each colname
content of the column.}
}
\value{
Updated flextable object with merged cells in the header.
}
\description{
This:
\itemize{
\item{(1) for each row: checks that subsequent columns are identical (and corresponding
sub-header overlap), so should be merged}
\item{(2) for each column: checks that subsequent rows are identical, so should be
merged}
\item{filters cells planned to be merged across rows (2) if they are already
merged across columns (1)}
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/miscellaneous.R
\name{convertVarRowVarColVarToFactor}
\alias{convertVarRowVarColVarToFactor}
\title{Convert \code{rowVar}, \code{colVar} and character \code{var} in \code{data} to factor}
\usage{
convertVarRowVarColVarToFactor(data, rowVar = NULL, colVar = NULL, var = NULL)
}
\arguments{
\item{data}{Data.frame with dataset to consider for the summary table.}

\item{rowVar}{Character vector with variable(s)
to be included in the rows.
If multiple variables are specified, the variables should 
be sorted in hierarchical order
(e.g. body system class before adverse event term)
and are nested in the table.}

\item{colVar}{Character vector with variable(s) to be included in columns.
If multiple variables are specified, the variables should 
be sorted in hierarchical order,
and are included in multi-columns layout.\cr
Use: 'variable' to include the variables to summarize: \code{var}
 (if multiple) in different columns.}

\item{var}{Character vector with variable(s) of \code{data}, 
to compute statistics on.\cr
If NULL (by default), counts by row/column variable(s) are computed.\cr
To also return counts of the \code{rowVar} in case other \code{var}
are specified, you can include: 'all' in the \code{var}.\cr
Missing values, if present, are filtered 
(also for the report of number of subjects/records).}
}
\value{
Updated \code{data}
}
\description{
Convert \code{rowVar}, \code{colVar} and character \code{var} in \code{data} to factor
}
\author{
Laure Cougnaud
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formatSummaryStatisticsTable.R
\name{formatSummaryStatisticsTable}
\alias{formatSummaryStatisticsTable}
\title{Format summary statistics table for export}
\usage{
formatSummaryStatisticsTable(
  summaryTable,
  rowVar = getAttribute(summaryTable, "rowVar"),
  colVar = getAttribute(summaryTable, "colVar"),
  colTotalLab = getAttribute(summaryTable, "colTotalLab", default = "Total"),
  colHeaderTotalInclude = TRUE,
  statsVar = getAttribute(summaryTable, "statsVar"),
  statsLabInclude = NULL,
  statsLayout = "row",
  statsValueLab = "StatisticValue",
  emptyValue = "-"
)
}
\arguments{
\item{summaryTable}{A \code{\link{summaryTable}} object.}

\item{rowVar}{Character vector with variable(s)
to be included in the rows.
If multiple variables are specified, the variables should 
be sorted in hierarchical order
(e.g. body system class before adverse event term)
and are nested in the table.}

\item{colVar}{Character vector with variable(s) to be included in columns.
If multiple variables are specified, the variables should 
be sorted in hierarchical order,
and are included in multi-columns layout.\cr
Use: 'variable' to include the variables to summarize: \code{var}
 (if multiple) in different columns.}

\item{colTotalLab}{String, label for the total column  'Total' by default.\cr}

\item{colHeaderTotalInclude}{Logical, if TRUE include the total of number of patients
(\code{'statN'}) in the column header.}

\item{statsVar}{Character vector with columns of \code{summaryTable} with
statistic variables. For the export: if not specified, all columns of \code{data}
besides row, column variables, 'variable', 'variableGroup'
and 'isTotal' are considered.}

\item{statsLabInclude}{Logical, if TRUE include the statistic label
in the table. \cr By default only included if more than
one statistic variables are available in the table.}

\item{statsLayout}{String with layout for the statistics names 
(in case more than one statistic is included), among:
\itemize{
\item{row (by default for 'flextable' output): \cr All statistics are
 included in different rows in the first column of the table (after the row variable(s))}
\item{'col' (by default for 'DT' output): \cr Statistics are included 
in separated columns (last row of the header).\cr
This option is not compatible with categorical variable(s).}
\item{'rowInSepCol': \cr Statistics are included in different rows, 
but in a separated column than the \code{rowVar} variable(s)}
}}

\item{statsValueLab}{String with label for the statistic value, 
'StatisticValue' by default.\cr
This is only included in the table if the statistics provided in 
\code{stats} are not named and if no \code{colVar} is specified.}

\item{emptyValue}{String with placeholder used to fill the table for missing values, '-' by default.
This value is typically used e.g. if not all statistics are computed for all specified
row/col/var variables.}
}
\value{
summaryTable reformatted to wide format
}
\description{
Format summary statistics table for export
}
\author{
Laure Cougnaud
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stats-utility.R
\name{geomSD}
\alias{geomSD}
\title{Compute geometric standard deviation}
\usage{
geomSD(x, na.rm = FALSE)
}
\arguments{
\item{x}{Numeric vector.}

\item{na.rm}{Logical, should NA value(s) be removed (FALSE by default)?}
}
\value{
Numeric vector of length 1 with geometric mean.
}
\description{
The geometric standard deviation is computed as:
\eqn{\exp(\sigma(log(x)))}, with:
\itemize{
\item{log: natural logarithm}
\item{\eqn{\sigma}: standard deviation}
}
}
\examples{
# geometric standard deviation of a sample from a log normal distribution:
geomSD(rlnorm(n = 1000, meanlog = 0, sdlog = 1))
}
\seealso{
Other stats utility functions:
\code{\link[=cv]{cv()}},
\code{\link[=geomCV]{geomCV()}},
\code{\link[=geomMean]{geomMean()}},
\code{\link[=geomSE]{geomSE()}},
\code{\link[=se]{se()}}
}
\author{
Laure Cougnaud
}
\concept{stats utility functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summaryTable-doc.R
\name{summaryTable}
\alias{summaryTable}
\title{\code{summaryTable}}
\value{
Not relevant
}
\description{
\enumerate{
\item{The summary statistics are computed in
\code{\link{computeSummaryStatisticsTable}},
which creates a \code{summaryTable} object.}
\item{This object is exported to diverse formats via 
\code{\link{export}}}
}
Multiple \code{summaryTable} objects are combined
together with \code{\link{combine.summaryTable}}.
}
\details{
The \code{summaryTable} is an intermediary 
object of the package. 
This contains the summary statistics as data.frame.
This object contains:
\itemize{
\item{the row and column variable(s)}
\item{the computed statistic(s): \cr
If \code{type} is:
\itemize{
\item{'summaryTable': 
\itemize{
\item{'statN': number of subjects}
\item{'statMean': mean of \code{var}}
\item{'statSD': standard deviation of \code{var}}
\item{'statSE': standard error of \code{var}}
\item{'statMedian': median of \code{var}}
\item{'statMin': minimum of \code{var}}
\item{'statMax': maximum of \code{var}}
\item{'statPerc': percentage of subjects}
\item{'statPercTotalN': total number of subjects based on \code{dataTotalPerc},
denominator of \code{statPerc}}
\item{'statm': number of records}
}
}
\item{'countTable': 
\itemize{
\item{'statN': number of subjects}
\item{'statPercN' (or 'statPercm'): percentage of subjects
(or records depending on \code{statsPerc})}
\item{'statPercTotalN' (or 'statPercTotalm'): total number of 
subjects (or records) based on \code{dataTotalPerc}, and used as
denominator of \code{statPercN} (or 'statPercm')}
\item{'statm': number of records}
}}}
}
\item{computed statistics.\cr
The statistics are stored in columns corresponding
to names of the \code{statsVar}.\cr
If the specified statistics are not named and of length
1, the statistics are stored in a column called: 'Statistic'.}
\item{variables: 
\itemize{
\item{'variable': variable name in case \code{var} is of length > 1}
\item{'variableGroup': in case \code{var} is of length > 1 and for 
variable(s) used for count: elements of the variable}
}
}
\item{'isTotal': variable with logical flag, TRUE if the record contain 
the total by column}
}
Additionally, the output contains an extra attribute
'summaryTable', which is a list composed of:
\itemize{
\item{'statsVar': column name(s) of summary table with computed statistics
included in the final table}
\item{'rowVar': column name(s) of summary table with row variable
included in the final table. This parameter should be mainly used for qualitative variables and 'nests' together different rows in the final output table.}
\item{'rowVarLab': labels corresponding to the 'rowVar' attribute}
\item{'rowVarTotalInclude': row variables whose total will be included:
\code{rowVarTotalInclude} and 'variableGroup' if the variable total should be included}
\item{'rowVarTotalInSepRow': row variables whose total will be included in a separated row:
\code{rowVarTotalInSepRow} and 'variableGroup' if \code{varTotalInSepRow}}
\item{'colVar': column name(s) of summary table with column variable
included in the final table}
\item{'colTotalLab': label for the total}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in
%   R/flextable-convertSummaryStatisticsTableToFlextable.R
\name{exportFlextableToDocx}
\alias{exportFlextableToDocx}
\title{Export flextable to docx file}
\usage{
exportFlextableToDocx(
  object,
  file,
  landscape = FALSE,
  breaksAfter = if (!inherits(object, "flextable")) seq_along(object) else 1
)
}
\arguments{
\item{object}{\code{\link[flextable]{flextable}} object, or list of such objects}

\item{file}{String with path of the file where the table should be exported.
The file should have the extension: '.html'.
If NULL, the summary table is not exported but only returned as output.
If \code{byVar} is specified, each table is exported to a separated
file with the suffix: 'file_[i].html' with i the index of the file.}

\item{landscape}{(flextable output) Logical, if TRUE the file is in landscape format.\cr
By default: FALSE if \code{style} is 'report' and TRUE if \code{style} is 'presentation'.}

\item{breaksAfter}{In case \code{object} is list: 
integer vector with indices of list item after which a page break should 
be included in the final document.}
}
\value{
no returned value, the \code{object} is exported to a docx file.
}
\description{
Export flextable to docx file
}
\author{
Laure Cougnaud
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inTextSummaryTable-common-args-doc.R
\name{inTextSummaryTable-common-args}
\alias{inTextSummaryTable-common-args}
\title{Arguments used across the functions of the inTextSummaryTable package.}
\arguments{
\item{data}{Data.frame with dataset to consider for the summary table.}

\item{summaryTable}{A \code{\link{summaryTable}} object.}

\item{var}{Character vector with variable(s) of \code{data}, 
to compute statistics on.\cr
If NULL (by default), counts by row/column variable(s) are computed.\cr
To also return counts of the \code{rowVar} in case other \code{var}
are specified, you can include: 'all' in the \code{var}.\cr
Missing values, if present, are filtered 
(also for the report of number of subjects/records).}

\item{varFlag}{Character vector, subset of \code{var} with variable(s) 
of type 'flag' (with 'Y', 'N' or '' for empty/non specified value).
Only the counts for records flagged (with 'Y') are retained.}

\item{varLabInclude}{Logical, if TRUE
the name of the summary statistic variable(s) (\code{var})
are included in the table.
This is automatically set to TRUE if more than one variable(s) 
and is specified, and FALSE if only one variable is specified.}

\item{rowVar}{Character vector with variable(s)
to be included in the rows.
If multiple variables are specified, the variables should 
be sorted in hierarchical order
(e.g. body system class before adverse event term)
and are nested in the table.}

\item{rowVarInSepCol}{Character vector with \code{rowVar}
that should be included in separated columns.
By default (NULL), all row variables are nested in the first column 
of the table.\cr
To include the groups within a \code{var} variable in a separated column, 
set: \code{rowVarInSepCol == 'variableGroup'}.}

\item{rowVarLab}{Named character vector with 
label for the \code{rowVar} variable(s).}

\item{statsVar}{Character vector with columns of \code{summaryTable} with
statistic variables. For the export: if not specified, all columns of \code{data}
besides row, column variables, 'variable', 'variableGroup'
and 'isTotal' are considered.}

\item{rowVarTotalInclude}{Character vector with \code{rowVar}
for which the total should be reported.\cr
If the higher row variable is specified, the total across all rows
is reported. \cr
For the export, these variable(s) are formatted as factor with 
\strong{'Total' as the first level}.}

\item{rowVarTotalInSepRow}{Character vector with \code{rowVarTotalInclude}
(not in \code{rowVarInSepCol}) for which the total should be included in a separated row labelled 'Total'.
Otherwise (by default) the total is included in the header row of each category.}

\item{colVar}{Character vector with variable(s) to be included in columns.
If multiple variables are specified, the variables should 
be sorted in hierarchical order,
and are included in multi-columns layout.\cr
Use: 'variable' to include the variables to summarize: \code{var}
 (if multiple) in different columns.}

\item{colTotalInclude}{Logical, if TRUE (FALSE by default) include the summary 
statistics across columns in a separated column.}

\item{colTotalLab}{String, label for the total column  'Total' by default.\cr}

\item{subjectVar}{String, variable of \code{data} with subject ID,
'USUBJID' by default.}

\item{statsLayout}{String with layout for the statistics names 
(in case more than one statistic is included), among:
\itemize{
\item{row (by default for 'flextable' output): \cr All statistics are
 included in different rows in the first column of the table (after the row variable(s))}
\item{'col' (by default for 'DT' output): \cr Statistics are included 
in separated columns (last row of the header).\cr
This option is not compatible with categorical variable(s).}
\item{'rowInSepCol': \cr Statistics are included in different rows, 
but in a separated column than the \code{rowVar} variable(s)}
}}

\item{statsValueLab}{String with label for the statistic value, 
'StatisticValue' by default.\cr
This is only included in the table if the statistics provided in 
\code{stats} are not named and if no \code{colVar} is specified.}

\item{statsExtra}{(optional) Named list with functions for additional custom
statistics to be computed.\cr
Each function:
\itemize{
\item{has as parameter, either: 'x': the variable (\code{var}) to compute
the summary statistic on or 'data': the entire dataset}
\item{returns the corresponding summary statistic as a numeric vector}
}
For example, to additionally compute the coefficient of variation, this can be set to:
\code{list(statCVPerc = function(x) sd(x)/mean(x)*100)} (or \code{\link{cv}}).}

\item{type}{String with type of table: 
\itemize{
\item{'summaryTable': summary table with statistics for numeric variable}
\item{'countTable': count table}
\item{'auto' (by default): 'summaryTable' if the variable is numeric,
'countTable' otherwise}
}}

\item{statsLabInclude}{Logical, if TRUE include the statistic label
in the table. \cr By default only included if more than
one statistic variables are available in the table.}

\item{title}{Character vector with title(s) for the table.
Set to NULL (by default) if no title should be included.
If multiple are specified, specified for each element of \code{byVar} (in order of the levels).}

\item{pageDim}{Numeric vector of length 2 with page width and height.\cr
Depending on \code{outputType}:
\itemize{
\item{'flextable': in inches}
\item{'DT': in number of rows in the table.\cr
Currently only the height is used (e.g. \code{c(NA, 4)})
}}}

\item{columnsWidth}{Column widths of the table, for \code{outputType}:
\itemize{
\item{'flextable': the widths can be specified either:
 \itemize{
 \item{in percentage: the sum of the widths should be 1}
 \item{in inch: the sum should be the page width (see \code{\link{getDimPage}})}
} If the widths are not specified for all columns, the width of the remaining last
 columns is set such as the width of the total table is the page width.}
\item{'DT': see \code{\link[clinUtils]{getClinDT}}}
}}

\item{labelVars}{(optional) Named character vector with label for
the row, column variable(s) or variable(s) to summarize. \cr
Labels specified via dedicated parameter: e.g. 
\code{rowVarLab}, \code{colVarLab}, \code{varLab}
have priority on this parameter.}
}
\value{
No return value, used for the documentation of R functions
}
\description{
Arguments used across the functions of the inTextSummaryTable package.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/decimals.R
\name{formatPercentage}
\alias{formatPercentage}
\title{Format a percentage.}
\usage{
formatPercentage(x, nDec = 1)
}
\arguments{
\item{x}{Numeric vector with percentage(s)}

\item{nDec}{Integer of length 1, number
of decimals used to round the percentage,
1 by default.}
}
\value{
String with formatted percentage
}
\description{
The following rules are used:
\itemize{
\item{percentage = 0\%: '0'}
\item{ 0\% < percentage < 0.1\%: '<0.1'}
\item{99.9\% < percentage < 100\%: '>99.9'}
\item{percentage = 100\%: '100'}
\item{missing value (NA) (class without valid data): '-'}
\item{other: 'x.x' (1 decimal)}
}
}
\examples{
xPerc <- c(NA, 0, 100, 99.95, 0.012, 34.768)
formatPercentage(x = xPerc)
}
\seealso{
Other decimals:
\code{\link[=getMaxNDecimals]{getMaxNDecimals()}},
\code{\link[=getMaxNDecimalsData]{getMaxNDecimalsData()}},
\code{\link[=getNDecimals]{getNDecimals()}},
\code{\link[=getNDecimalsData]{getNDecimalsData()}}
}
\author{
Laure Cougnaud
}
\concept{decimals}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/decimals.R
\name{getNDecimals}
\alias{getNDecimals}
\title{Get number of decimals for a specific vector.}
\usage{
getNDecimals(x, useRule = TRUE, rule = "1", useData = TRUE)
}
\arguments{
\item{x}{Numeric vector.}

\item{useRule}{Logical (TRUE by default), should the \code{rule} be applied?}

\item{rule}{Character vector with rule to use to derive 
the number of parameters.
Currently only: '1' is implemented.
\itemize{
\item{'1': standard rule for the number of 
decimals for individual values for a 
continuous variable:
\itemize{
\item{value < 1 ('very small values'): 3}
\item{value < 10: 2}
\item{value in [10, 1000[: 1}
\item{value >= 1000: 0}
}
}
}}

\item{useData}{Logical (TRUE by default), should the number of decimals
be extracted based on the input data \code{x}?}
}
\value{
Numeric vector of same length than \code{x}
with the number of decimals.
}
\description{
The number of decimals is extracted either:
\itemize{
\item{from specific implemented rule :
see \code{\link{getNDecimalsRule}} for further details}
\item{from the data itself: 
see \code{\link{getNDecimalsData}} for further details
}
\item{both criterias: in this case the minimum of 
the number of decimals for both criterias is used}
}
}
\examples{
x <- c(0.99, 5.679, 50.45, 1450)
# extract number of decimals based on data:
getNDecimals(x, useRule = FALSE, useData = TRUE)
# extract number of decimals based on pre-defined rule:
getNDecimals(x, useRule = TRUE, useData = FALSE)
# extract number of decimals based on both rules
# minimum of both is used (by default)
getNDecimals(x, useRule = TRUE, useData = TRUE)
}
\seealso{
Other decimals:
\code{\link[=formatPercentage]{formatPercentage()}},
\code{\link[=getMaxNDecimals]{getMaxNDecimals()}},
\code{\link[=getMaxNDecimalsData]{getMaxNDecimalsData()}},
\code{\link[=getNDecimalsData]{getNDecimalsData()}}
}
\author{
Laure Cougnaud
}
\concept{decimals}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inTextSummaryTable.R
\docType{package}
\name{inTextSummaryTable}
\alias{inTextSummaryTable}
\alias{inTextSummaryTable-package}
\title{inTextSummaryTable: creation of in-text summary table}
\description{
Tables of summary statistics or count tables are created. 
These tables can be exported as in-text table to a Clinical Study Report 
(Word format), a topline presentation (PowerPoint format),
 or as interactive table to an html document.
}
\details{
\itemize{
\item{To get started with the package, see:\cr
\code{vignette("inTextSummaryTable-01-introduction", package = "inTextSummaryTable")}}
\item{To get example code for standard in-text tables created with the package, 
see:\cr
\code{vignette("inTextSummaryTable-03-standardTables", package = "inTextSummaryTable")}}
\item{The main function: \code{\link{getSummaryStatisticsTable}} enables
to create ready in-text table.}
}
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/openanalytics/inTextSummaryTable}
  \item Report bugs at \url{https://github.com/openanalytics/inTextSummaryTable/issues}
}

}
\author{
\strong{Maintainer}: Laure Cougnaud \email{laure.cougnaud@openanalytics.eu}

Authors:
\itemize{
  \item Laure Cougnaud \email{laure.cougnaud@openanalytics.eu}
  \item Michela Pasetto \email{michela.pasetto@openanalytics.eu}
}

Other contributors:
\itemize{
  \item Margaux Faes (tests) [reviewer]
  \item Open Analytics [copyright holder]
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/miscellaneous.R
\name{interactionCustom}
\alias{interactionCustom}
\title{Compute the interaction between variable(s),
without propagating the missing values (if present in one of the variable)
unlike the behaviour with \code{\link{interaction}}.
This also ensure that the levels of the final interaction variables
are ordered similarly as the levels of the input \code{var} (if present).}
\usage{
interactionCustom(data, var, varDataLevels = NULL)
}
\arguments{
\item{data}{Data.frame with data.}

\item{var}{Character vector with variable(s) to consider.}

\item{varDataLevels}{(optional) Data.frame with data to consider
to define the levels of the variable.
If not specified, only the combinations of variable(s) available in the data are retained.}
}
\value{
list with:
\itemize{
\item{'x': factor with interaction between the input \code{var}}
\item{'dataLevels': data.frame with mapping between the \code{var} 
variable(s) and the new factor levels}
}
}
\description{
Compute the interaction between variable(s),
without propagating the missing values (if present in one of the variable)
unlike the behaviour with \code{\link{interaction}}.
This also ensure that the levels of the final interaction variables
are ordered similarly as the levels of the input \code{var} (if present).
}
\author{
Laure Cougnaud
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summaryTable-combine.R
\name{combine}
\alias{combine}
\title{Combine objects}
\usage{
combine(...)
}
\arguments{
\item{...}{Extra parameters for the corresponding
method.}
}
\description{
Combine objects
}
\seealso{
\code{\link{combine.summaryTable}}
to combine \code{\link{summaryTable}} objects.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subjectProfileSummaryPlot.R
\name{subjectProfileSummaryTable}
\alias{subjectProfileSummaryTable}
\title{Plot a table with \code{ggplot} of a text variable of interest.}
\usage{
subjectProfileSummaryTable(
  data,
  xVar,
  text,
  xLim = NULL,
  colorVar = NULL,
  colorPalette = NULL,
  colorLab = getLabelVar(colorVar, labelVars = labelVars),
  fontface = 1,
  xLab = NULL,
  labelVars = NULL,
  caption = NULL,
  showLegend = TRUE,
  legendPosition = ifelse(showLegend, "right", "none"),
  yAxisLabs = FALSE,
  xAxisLabs = NULL,
  style = "report",
  fontname = switch(style, report = "Times", presentation = "Tahoma"),
  fontsize = switch(style, report = 8, presentation = 10),
  pointSize = 1.5,
  themeFct = switch(style, report = theme_classic, presentation = theme_bw),
  textSize = fontsize/ggplot2::.pt,
  xTrans = NULL
)
}
\arguments{
\item{data}{Data.frame (in long format) with data for the table.}

\item{xVar}{String, variable of \code{data} with variable for the x-axis.}

\item{text}{Character vector with colnames of \code{data}
or expression based on colnames of \code{data} to extract
the text label.}

\item{xLim}{Vector of the length 2 with limits for the x-axis.}

\item{colorVar}{String, variable of \code{data} for coloring.}

\item{colorPalette}{(named) Vector with color palette.}

\item{colorLab}{String, label for \code{colorVar}, used in the legend.}

\item{fontface}{Numeric, fontface for the text.}

\item{xLab}{String with label for the x-axis.}

\item{labelVars}{Named string with variable labels (names are the variable code).}

\item{caption}{String with caption for the plot,
NULL by default.}

\item{showLegend}{Logical, should the legend be displayed? TRUE by default.}

\item{legendPosition}{String with legendPosition,
'right' by default.}

\item{yAxisLabs}{Logical, if TRUE include the labels in the y-axis.}

\item{xAxisLabs}{Vector with labels for the x-axis if \code{xVar}
is discrete or vector with limits if continuous.}

\item{style}{String with subject profile style.
This affects the parameters: \code{fontname}, \code{fontsize}
and \code{themeFct}.}

\item{fontname}{String with font name,
by default 'Times' if \code{style} is 'report' and
'Tahoma' if \code{style} is 'presentation'.}

\item{fontsize}{Numeric vector of length 1 with font size,
by default 8 if \code{style} is 'report' and
10 if \code{style} is 'presentation'}

\item{pointSize}{Numeric indicating the size of points in the legend, 1.5 by default}

\item{themeFct}{Function with ggplot2 theme,
by default \code{\link[ggplot2]{theme_classic}} if \code{style} is 'report' and
\code{\link[ggplot2]{theme_bw}} if \code{style} is 'presentation'.}

\item{textSize}{Size for the text.}

\item{xTrans}{(optional) ggplot2 transformation
for the x-axis.}
}
\value{
\code{\link[ggplot2]{ggplot}} object
}
\description{
The labels extracted based on the \code{text} parameter 
and displayed at the x-position based on \code{xVar} and the y-position based on
\code{colorVar}.
Each group specified in the color variables are displayed in different
lines in the plot.
}
\author{
Laure Cougnaud and Michela Pasetto
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in
%   R/flextable-convertSummaryStatisticsTableToFlextable.R
\name{formatCustomFlextable}
\alias{formatCustomFlextable}
\title{Format superscript/subscripts/bold cells in a flextable.
Superscript should be indicated as: '\code{a^{b}}',
subscript as '\code{a_{b}}' and bold as '\code{bold{a}}' in the
 the input summary table.}
\usage{
formatCustomFlextable(
  dataTable,
  ft,
  fontname = "Times",
  part = "body",
  fontsize = 8,
  iBase = 0,
  bold = FALSE
)
}
\arguments{
\item{dataTable}{data.frame with data used in table,
summary table for body or header data.frame for the header.}

\item{ft}{Corresponding \code{\link[flextable]{flextable}}.}

\item{fontname}{String with font name, 'Times' by default.}

\item{part}{string with part of the table to consider, 
see \code{\link[flextable]{compose}}.}

\item{fontsize}{Integer with font size, 8 by default.}

\item{iBase}{Integer with base row index (if different than 0).}

\item{bold}{Logical (FALSE by default) should the superscript/subscript 
be indicated in bold?}
}
\value{
\code{\link[flextable]{flextable}} with superscript/subscript.
}
\description{
Format superscript/subscripts/bold cells in a flextable.
Superscript should be indicated as: '\code{a^{b}}',
subscript as '\code{a_{b}}' and bold as '\code{bold{a}}' in the
 the input summary table.
}
\author{
Laure Cougnaud
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/computeSummaryStatisticsTable.R
\name{computeSummaryStatisticsByRowColVar}
\alias{computeSummaryStatisticsByRowColVar}
\title{Compute summary statistics by specified \code{rowVar} and \code{colVar}}
\usage{
computeSummaryStatisticsByRowColVar(
  data,
  var = NULL,
  varLab = getLabelVar(var = var, data = data, labelVars = labelVars),
  varInclude0 = FALSE,
  varLabInclude = length(var) > 1,
  varTotalInclude = FALSE,
  type = "auto",
  rowVar = NULL,
  rowInclude0 = FALSE,
  rowVarDataLevels = NULL,
  colVar = NULL,
  colInclude0 = FALSE,
  colVarDataLevels = NULL,
  subjectVar = "USUBJID",
  labelVars = NULL,
  statsExtra = NULL,
  msgLabel = NULL,
  checkVarDiffBySubj = "error"
)
}
\arguments{
\item{data}{Data.frame with dataset to consider for the summary table.}

\item{var}{Character vector with variable(s) of \code{data}, 
to compute statistics on.\cr
If NULL (by default), counts by row/column variable(s) are computed.\cr
To also return counts of the \code{rowVar} in case other \code{var}
are specified, you can include: 'all' in the \code{var}.\cr
Missing values, if present, are filtered 
(also for the report of number of subjects/records).}

\item{varLab}{Named character vector with label for each variable 
specified in \code{var}.
By default, extracted from the \code{labelVars}.
if not available, \code{var} is used.}

\item{varInclude0}{Logical, should rows with no counts 
for the count \code{var} or \code{varFlag} variable(s)
be included in the table?
Either:
\itemize{
\item{logical of length 1, if TRUE (FALSE by default) 
rows with no count are included for all \code{var}
}
\item{a character vector containing categorical \code{var} 
for which zero counts rows should be included}
}}

\item{varLabInclude}{Logical, if TRUE
the name of the summary statistic variable(s) (\code{var})
are included in the table.
This is automatically set to TRUE if more than one variable(s) 
and is specified, and FALSE if only one variable is specified.}

\item{varTotalInclude}{Should the total across all categories of \code{var} 
be included for the count table?
Only used for categorical variables (and \code{var} not 'all').
Either:
\itemize{
\item{logical of length 1, if TRUE (FALSE by default) include the total for all categorical \code{var}}
\item{a character vector containing categorical \code{var} for which the total should be included}
}}

\item{type}{String with type of table: 
\itemize{
\item{'summaryTable': summary table with statistics for numeric variable}
\item{'countTable': count table}
\item{'auto' (by default): 'summaryTable' if the variable is numeric,
'countTable' otherwise}
}}

\item{rowVar}{Character vector with variable(s)
to be included in the rows.
If multiple variables are specified, the variables should 
be sorted in hierarchical order
(e.g. body system class before adverse event term)
and are nested in the table.}

\item{rowInclude0}{Logical, if TRUE (FALSE by default),
include rows with no records, based on all combinations 
of the \code{rowVar} (assuming nested variable(s)).}

\item{rowVarDataLevels}{Data.frame with unique combinations of \code{rowVar}
to be included in columns.
Each column should correspond to \code{colVar} and as factor
if the elements should be ordered in the final table.}

\item{colVar}{Character vector with variable(s) to be included in columns.
If multiple variables are specified, the variables should 
be sorted in hierarchical order,
and are included in multi-columns layout.\cr
Use: 'variable' to include the variables to summarize: \code{var}
 (if multiple) in different columns.}

\item{colInclude0}{Logical, if TRUE (FALSE by default),
include columns with no records, based on all combinations 
of the \code{columnVar} (assuming nested variable(s)).
If variable(s) are not nested, possible combinations
can be specified via \code{colVarDataLevels}.}

\item{colVarDataLevels}{Data.frame with unique combinations of \code{colVar}
to be included in columns.
Each column should correspond to \code{colVar} and as factor
if the elements should be ordered in the final table.}

\item{subjectVar}{String, variable of \code{data} with subject ID,
'USUBJID' by default.}

\item{labelVars}{(optional) Named character vector with label for
the row, column variable(s) or variable(s) to summarize. \cr
Labels specified via dedicated parameter: e.g. 
\code{rowVarLab}, \code{colVarLab}, \code{varLab}
have priority on this parameter.}

\item{statsExtra}{(optional) Named list with functions for additional custom
statistics to be computed.\cr
Each function:
\itemize{
\item{has as parameter, either: 'x': the variable (\code{var}) to compute
the summary statistic on or 'data': the entire dataset}
\item{returns the corresponding summary statistic as a numeric vector}
}
For example, to additionally compute the coefficient of variation, this can be set to:
\code{list(statCVPerc = function(x) sd(x)/mean(x)*100)} (or \code{\link{cv}}).}

\item{checkVarDiffBySubj}{String, 'error' (default), 'warning',
or 'none'.  
Should an error, a warning, or nothing be produced
if a continuous variable (\code{var}) contains
different values for the same subject (by row/column)?}
}
\value{
data.frame of class 'countTable' or 'summaryTable',
depending on the 'type' parameter; with statistics in columns,
either if \code{type} is:
\itemize{
\item{'summaryTable': 
\itemize{
\item{'N': number of subjects}
\item{'Mean': mean of \code{var}}
\item{'SD': standard deviation of \code{var}}
\item{'SE': standard error of \code{var}}
\item{'Median': median of \code{var}}
\item{'Min': minimum of \code{var}}
\item{'Max': maximum of \code{var}}
\item{'m': number of records}
}
}
\item{'countTable': 
\itemize{
\item{'N': number of subjects}
\item{'m': number of records}
}
}
}
}
\description{
Compute summary statistics by specified \code{rowVar} and \code{colVar}
}
\author{
Laure Cougnaud
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/miscellaneous.R
\name{getAttribute}
\alias{getAttribute}
\title{Get specific attribute from a summaryTable or a list of summaryTables}
\usage{
getAttribute(summaryTable, name, default = NULL)
}
\arguments{
\item{name}{String with attribute name.}

\item{default}{Object with default value for attribute, 
NULL by default.}
}
\value{
Object with attribute, or \code{default} is doesn't
exists in \code{summaryTable}
}
\description{
Get specific attribute from a summaryTable or a list of summaryTables
}
\author{
Laure Cougnaud
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/computeSummaryStatisticsTable.R
\name{getStatisticsSummaryStatisticsTable}
\alias{getStatisticsSummaryStatisticsTable}
\title{Compute custom statistics specified by the user.}
\usage{
getStatisticsSummaryStatisticsTable(
  summaryTable,
  statsVarInit,
  var = NULL,
  stats = NULL,
  statsVarBy = NULL
)
}
\arguments{
\item{summaryTable}{Summary table.}

\item{statsVarInit}{Character vector with initial statistics names.}

\item{var}{Character vector with variable(s) of \code{data}, 
to compute statistics on.\cr
If NULL (by default), counts by row/column variable(s) are computed.\cr
To also return counts of the \code{rowVar} in case other \code{var}
are specified, you can include: 'all' in the \code{var}.\cr
Missing values, if present, are filtered 
(also for the report of number of subjects/records).}

\item{stats}{(Optionally) named list of expression or call object of summary statistics of interest.
The names are reported in the header.
The following variables are recognized, if the table is a: 
\itemize{
\item{'summaryTable': 
\itemize{
\item{'statN': number of subjects}
\item{'statMean': mean of \code{var}}
\item{'statSD': standard deviation of \code{var}}
\item{'statSE': standard error of \code{var}}
\item{'statMedian': median of \code{var}}
\item{'statMin': minimum of \code{var}}
\item{'statMax': maximum of \code{var}}
\item{'statPerc': percentage of subjects}
\item{'statm': number of records}
}
}
\item{'countTable': 
\itemize{
\item{'statN': number of subjects}
\item{'statPercN': percentage of subjects}
\item{'statm': number of records}
}
}
}
If \code{stats} if of length 1, the name of the summary statistic is not included
in the table.
The statistics can be specified for each \code{var} (if multiple), 
by naming each element of the list:
list(varName1 = list(...), varName2 = list()) and/or for each element in:
\code{statsVarBy}, by naming each sublist.}

\item{statsVarBy}{Character vector with colnames of
\code{data} by which the statistics should be computed.
In this case, \code{stats} (nested list or not) should be additionally nested
to specify the statistics for each element in \code{statsVarBy}.}
}
\value{
List with two elements:
\itemize{
\item{'summaryTable': summary table updated with statistics specified in \code{stats}}
\item{'statsVar': Character vector with statistics names}
}
}
\description{
Compute custom statistics specified by the user.
}
\author{
Laure Cougnaud
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/miscellaneous.R
\name{writeTable}
\alias{writeTable}
\title{Custom function to write table to a text file}
\usage{
writeTable(x, file, ...)
}
\arguments{
\item{x}{Data.frame to export to the table, or list of such tables.}

\item{file}{String with text file to export to.}

\item{...}{Any parameters passed to the \code{\link[utils]{write.table}} function.}
}
\value{
No returned value, the object \code{x} is exported to the specified \code{file}.
}
\description{
This function is mainly a wrapper on \code{\link[utils]{write.table}},
with the specific options:
\itemize{
\item{no rownames}
\item{no quoting}
\item{tab separator}
}
}
\author{
Laure Cougnaud
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stats-utility.R
\name{geomSE}
\alias{geomSE}
\title{Compute geometric standard error of the mean.}
\usage{
geomSE(x, na.rm = FALSE)
}
\arguments{
\item{x}{Numeric vector.}

\item{na.rm}{Logical, should NA value(s) be removed (FALSE by default)?}
}
\value{
Numeric vector of length 1 with 
geometric standard error of the mean.
}
\description{
The geometric standard error of the mean is computed as:
\eqn{\exp(se(log(x)}, with:
\itemize{
\item{log: natural logarithm}
\item{\eqn{se}: standard error of the mean, as computed with \code{\link{se}}}
}
}
\examples{
# Geometric standard error of the mean of a sample from a log normal distribution:
geomSE(rlnorm(n = 1000, meanlog = 0, sdlog = 1))
}
\seealso{
Other stats utility functions:
\code{\link[=cv]{cv()}},
\code{\link[=geomCV]{geomCV()}},
\code{\link[=geomMean]{geomMean()}},
\code{\link[=geomSD]{geomSD()}},
\code{\link[=se]{se()}}
}
\author{
Laure Cougnaud
}
\concept{stats utility functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/miscellaneous.R
\name{checkVarLabInclude}
\alias{checkVarLabInclude}
\title{Check the \code{varLabInclude} variable.}
\usage{
checkVarLabInclude(var, varLabInclude = length(var) > 1)
}
\arguments{
\item{var}{String with variable to check.}

\item{varLabInclude}{Logical, if TRUE
the name of the summary statistic variable(s) (\code{var})
are included in the table.
This is automatically set to TRUE if more than one variable(s) 
and is specified, and FALSE if only one variable is specified.}
}
\value{
(Updated) \code{varLabInclude}
}
\description{
This function ensures that:
\itemize{
\item{variable name is included if more than one variable
are specified
}
\item{variable name is not included if no variable is specified}
}
}
\author{
Laure Cougnaud
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/miscellaneous.R
\name{checkVar}
\alias{checkVar}
\title{Check if variable(s) are present in reference: either in columns in 
a dataset or in reference set.}
\usage{
checkVar(
  var,
  varLabel,
  varUncheck = NULL,
  varRef,
  refLabel = ifelse(!missing(varRef), "reference variable", "data"),
  data,
  msgType = c("warning", "error")
)
}
\arguments{
\item{var}{String with variable to check.}

\item{varLabel}{String with label for \code{var}, e.g.
name of associated parameter.}

\item{varUncheck}{(Named) character vector with extra variables 
in \code{var} which shouldn't be checked.}

\item{varRef}{(Named) character vector with set of reference variables.}

\item{refLabel}{String with label for the reference}

\item{data}{Data.frame with data.}

\item{msgType}{String with type of message returned, either a 'warning' (default)
or an error.}
}
\value{
Depending on \code{msgType}:
\itemize{
\item{\code{warning}: warning is printed in the console, and a 
\code{var} filtered with element not in \code{data}
or in \code{refSet} is returned.
If filtered \code{var} is empty, NULL is returned.}
\item{\code{error}: an error is triggered.}
}
}
\description{
Filter variables not present in the data or in reference set with a warning,
and only returned filtered vector, or NULL if empty.
}
\author{
Laure Cougnaud
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summaryTable-combine.R
\name{combineColTotal}
\alias{combineColTotal}
\title{Combine the column total for a combined summary table}
\usage{
combineColTotal(summaryTable, attrs)
}
\arguments{
\item{summaryTable}{Combined summary table}

\item{attrs}{Nested list with attributes of each
summary table.}
}
\value{
summary table, with combined total columns
}
\description{
Combine the column total for a combined summary table
}
\author{
Laure Cougnaud
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in
%   R/flextable-convertSummaryStatisticsTableToFlextable.R
\name{getPatternPosition}
\alias{getPatternPosition}
\title{Get position(s) (start, end) of a pattern in a string.}
\usage{
getPatternPosition(x, startPattern, endPattern = "\\\\}", format)
}
\arguments{
\item{x}{String.}

\item{startPattern}{String with start pattern.}

\item{endPattern}{String with end pattern.}

\item{format}{String with type of formatting}
}
\value{
Matrix with columns: 'start' and 'end' with
start and end position(x) of the pattern, 'format' with
the \code{format} and 'pattern' with the regex pattern for the full match. 
NULL if no match.
}
\description{
Get position(s) (start, end) of a pattern in a string.
}
\author{
Laure Cougnaud
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inTextSummaryTable-common-args-doc.R
\name{inTextSummaryTable-DT-args}
\alias{inTextSummaryTable-DT-args}
\title{Common arguments for the functionalities of the inTextSummaryTable package
for DT export.}
\arguments{
\item{expandVar}{(DT output) Character vector with variables of the summary table which
should be expanded in the data.}

\item{pageDim}{(DT output) Numeric vector of length 2 with page width and height,
in number of rows (currently only
the height is used (e.g. \code{c(NA, 4)})}

\item{noEscapeVar}{(DT output) Character vector with variables of \code{summaryTable}
which shouldn't be escaped in the table (e.g. containing URLs).}

\item{barVar}{(DT output) Character vector with variables of \code{summaryTable}
that should be represented as a bar.}

\item{file}{String with path of the file where the table should be exported.
The file should have the extension: '.docx'.
If NULL, the summary table is not exported but only returned as output.
If \code{byVar} is specified, each table is exported to a separated
file with the suffix: 'file_[i].docx' with i the index of the file.}
}
\value{
No return value, used for the documentation of 
R functions for 'DT' output
}
\description{
Common arguments for the functionalities of the inTextSummaryTable package
for DT export.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/decimals.R
\name{getMaxNDecimalsData}
\alias{getMaxNDecimalsData}
\title{Get maximum number of decimals in a variable based on the data
(\code{\link{getNDecimalsData}})}
\usage{
getMaxNDecimalsData(x)
}
\arguments{
\item{x}{Numeric vector.}
}
\value{
Integer with maximum number of decimals in a character vector.
}
\description{
Get maximum number of decimals in a variable based on the data
(\code{\link{getNDecimalsData}})
}
\seealso{
Other decimals:
\code{\link[=formatPercentage]{formatPercentage()}},
\code{\link[=getMaxNDecimals]{getMaxNDecimals()}},
\code{\link[=getNDecimals]{getNDecimals()}},
\code{\link[=getNDecimalsData]{getNDecimalsData()}}
}
\author{
Laure Cougnaud
x <- c(0.99, 5.679, 50.45, 1450)
# extract max number of decimals based on data:
getMaxNDecimalsData(x)
}
\concept{decimals}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stats-getStats.R
\name{getStatsData}
\alias{getStatsData}
\title{Get default set of statistics for variables of interest and specific dataset.}
\usage{
getStatsData(
  data,
  var = NULL,
  type = "default",
  extra = NULL,
  args = NULL,
  ...
)
}
\arguments{
\item{data}{Data.frame with dataset to consider for the summary table.}

\item{var}{(optional, recommended for continuous variable)
Character vector with variable(s) of data, to
compute statistics on.\cr
If NULL (by default), counts of the entire dataset are computed.\cr
It is passed to the \code{x} parameter of \code{\link{getStats}}.}

\item{type}{Character vector with type of statistics to extract, among:
\itemize{
\item{'default': default sets of statistics, 
see types: 'summary-default' and 'count-default' in \code{\link{getStats}}}
\item{'all': all computed statistics, see types: 'summary' and 'count' in 
\code{\link{getStats}}}
\item{any formatted statistics as implemented in \code{\link{getStats}}, 
see section 'Formatted statistics' in
\code{\link[=inTextSummaryTable-stats]{in-text table statistics}}.
}
}
To specify statistics for a continuous (numeric) or categorical
variable separately, this vector can be named with: 'cont' or
'cat' respectively (elements not named are used for both continuous 
and categorical variables).}

\item{extra}{List with extra statistics to include, or function to apply on each
\code{var} (e.g. depending on the class of \code{var}) to get such statistic.}

\item{args}{(optional) Named list with extra arguments for 
\code{\link{getStats}} for continuous (name: 'cont') or 
categorical variable (name: 'cat') specifically.}

\item{...}{Extra parameters passed to the \code{\link{getStats}} function
(independent of the variable type).}
}
\value{
List with statistics to compute, named by \code{var}
}
\description{
This set of statistics can be passed directly to the \code{stats} parameter
of the package functions.\cr
By default, statistics are extracted based on the variable(s) type
and formatted with the default rules implemented in the package.
}
\examples{
# default set of statistics (depending if the variable is continuous or categorical)
exampleData <- data.frame(
  USUBJID = 1 : 4, 
  WEIGHT = c(67, 78, 83, 61), 
  SEX = c("F", "M", "M", "F"), 
  stringsAsFactors = FALSE
)
getStatsData(data = exampleData, var = c("WEIGHT", "SEX"))
# all set of statistics (depending if the variable is continuous or categorical)
getStatsData(data = exampleData, var = c("WEIGHT", "SEX"), type = "all")
# custom set of statistics for all variables
getStatsData(data = exampleData, var = c("WEIGHT", "SEX"), type = c("n", "\%"))
# custom set of statistics, depending on the type of the variable
getStatsData(data = exampleData, var = c("WEIGHT", "SEX"), 
  type = c(cont = "median (range)", cont = "mean (se)", cat = "n (\%)"), 
  args = list(cat = list(includeName = FALSE))
)
}
\seealso{
\link{getStats}
}
\author{
Laure Cougnaud
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tableColors.R
\name{tableColorsReport}
\alias{tableColorsReport}
\title{Colors for tables in a report style}
\usage{
tableColorsReport
}
\description{
Default colors are black text on a white background.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tableColors.R
\name{getColorPaletteTable}
\alias{getColorPaletteTable}
\title{Get color palette for the tables}
\usage{
getColorPaletteTable(style = c("report", "presentation"))
}
\arguments{
\item{style}{String with style of report. Either 'report' or
'presentation'. By default, the style is 'report'.}
}
\value{
A named vector with hex colors.
}
\description{
This function gets the color palettes for the tables 
specified as global options.
}
\details{
By default, the function returns the palette of the package. 
The user can specify a custom palette by setting the global options.
}
\examples{
# report style (the default)
getColorPaletteTable()
# presentation style
getColorPaletteTable(style = "presentation")
# custom palette
customColorTable <- c('header' = "#FFFFFF",'headerBackground' = "#3F4788FF", 
'body' = "#000000", 'bodyBackground1' = "#D9D9D9", 'bodyBackground2' = "#D9D9D9", 
'footer' = "#000000", 'footerBackground' = "#FFFFFF",'line' = "#FFFFFF")
options(inTextSummaryTable.colors.table.presentation = customColorTable)
getColorPaletteTable("presentation")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/flextable-getListing.R
\name{getListing}
\alias{getListing}
\title{Format or create flextable for listings.}
\usage{
getListing(
  data,
  ft,
  border = TRUE,
  highlight = integer(),
  bgVar = NULL,
  fontname = switch(style, report = "Times", presentation = "Tahoma"),
  fontsize = switch(style, report = 8, presentation = 10),
  landscape = (style == "presentation"),
  style = "report",
  margin = 1,
  adjustWidth = TRUE,
  colorTable = getColorPaletteTable(style = style),
  align = TRUE,
  title = NULL,
  pageDim = NULL,
  includeRownames = TRUE
)
}
\arguments{
\item{data}{data.frame with data used in table.}

\item{ft}{Corresponding \code{\link[flextable]{flextable}}.}

\item{border}{Logical, if TRUE add a border.}

\item{highlight}{Integer vector with index(ices) of column(s) to highlight
(only applies for \code{style}: 'presentation').
0 for rownames (if present).
Colors for:
\itemize{
\item{highlighted columns is specified in \code{colorTable["headerBackgroundHighlight"]}}
\item{non highlighted columns is specified in \code{colorTable["headerBackground"]}}
}}

\item{bgVar}{String with the column of the \code{data} used for
alternating the body background colors of the table.}

\item{fontname}{String with font name, 'Times' by default.}

\item{fontsize}{Integer with font size, 8 by default.}

\item{landscape}{Logical, if TRUE the table is presented in landscape
format. \cr
By default: TRUE for \code{style}: 'report',
FALSE for \code{style}: 'presentation'.}

\item{style}{String with table style, either 'report' (by default, a4 format) 
or 'presentation'}

\item{margin}{Margin in the document in inches, 1 by default.}

\item{adjustWidth}{Logical, if TRUE adjust column widths,
to comply to specification of \code{landscape},
\code{margin} and \code{pageDim}
(only set to \code{FALSE} if e.g. table dimensions 
are pre-set with the specified \code{ft}).}

\item{colorTable}{Named character vector with color for the table,
see output of \code{\link{getColorPaletteTable}} for required elements.}

\item{align}{Logical, if TRUE (by default), 
default alignment is set ('center' in all table).}

\item{title}{Character vector with title(s) for the table.
Set to NULL (by default) if no title should be included.
Only available if \code{ft} is not specified.}

\item{pageDim}{(optional) Numeric vector of length 2 with 
page width and height in inches in portrait format,
in case page dimensions differ from the default
implemented report/presentation.
These dimensions should include the margins.}

\item{includeRownames}{Logical, if TRUE (by default)
rownames are included in the \code{\link[flextable]{flextable}} object.}
}
\value{
\code{\link[flextable]{flextable}} with style.
}
\description{
Flextable version >= 0.4.7 and pandoc >= 2.4
is required to included such table in a Rmarkdown document.
}
\examples{
# style: report or presentation
getListing(data = head(mtcars), style = "report")
getListing(data = head(mtcars), style = "presentation")
# remove rownames (included by default)
getListing(data = head(mtcars), style = "presentation", includeRownames = FALSE)
# highlight:
#  all columns
getListing(data = head(mtcars), style = "presentation", highlight = seq_along(mtcars))
# rownames
getListing(data = head(mtcars), style = "presentation", highlight = 0)
# specific columns
getListing(data = head(mtcars), style = "presentation", highlight = c(2, 4))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/combineVariables.R
\name{combineVariables}
\alias{combineVariables}
\title{Create a data.frame combining a set of records from same or multiple variables.}
\usage{
combineVariables(
  data,
  paramsList,
  newVar,
  labelVars = NULL,
  fctTest = "==",
  includeAll = FALSE,
  labelAll = "Any"
)
}
\arguments{
\item{data}{Data.frame with dataset to consider for the summary table.}

\item{paramsList}{nested list of parameters, 
specifying how the records of interest should be selected.\cr
There are two ways to select a subset of interest:
\itemize{
\item{by specifying one unique variable of interest with: 
\itemize{
\item{\code{var}: string with column of \code{data} of interest}
\item{\code{value}: value of \code{var} of interest (only used if \code{var} is specified).\cr
If not specified only the values different than NA and '' are considered.}
\item{\code{fctTest}: string with name or directly comparison function
to apply on \code{var} to select subset of interest versus \code{value}.\cr
The function should take \code{var} as first parameter and \code{value} to compare to
as second parameter and returns a logical vector with TRUE or FALSE (of length \code{var})
if the condition is fullfilled.\cr
If not specified, the records with \code{var} equal to \code{value} are retained 
(\code{fctTest} is set to '==').} 
\item{label specification: 
\itemize{
\item{\code{label}: string with label for the condition,
includde in the new 'variable' column.\cr
If not specified and:
\itemize{
\item{\code{var} is specified: label is extracted from 
\code{labelVars} if available or set to \code{var} otherwise.}
\item{\code{var} is not specified: label should be specified.}
}}
\item{\code{labelExtra}: string with extra label, will be concatenated with \code{label}}
}}
}}

\item{by specifying a combination of variable of interest with: 
\itemize{
\item{\code{exprs}: string with expression of columns of \code{data} to select subset of interest}
\item{\code{label}: string with complete label for the group}
}}

}}

\item{newVar}{String with name of new variable to construct.}

\item{labelVars}{(optional) Named character vector with label for
the row, column variable(s) or variable(s) to summarize. \cr
Labels specified via dedicated parameter: e.g. 
\code{rowVarLab}, \code{colVarLab}, \code{varLab}
have priority on this parameter.}

\item{fctTest}{Default function to use to compare \code{var} and \code{value} specified
in each sublist of \code{paramsList}.\cr
This is only used if \code{fctTest} is not specified in each sublist.}

\item{includeAll}{Logical, if TRUE (FALSE by default) include also the entire data as an additional subgroup.}

\item{labelAll}{String of group label for the entire data in case \code{includeAll} is TRUE.}
}
\value{
Data.frame with records from \code{data} extracted based on
the different conditions specified in \code{paramsList}.\cr
This data.frame contains an additional variable 
(labelled based on \code{newVar}) mentioning the
specific condition for which the record was extracted
(based \code{label}, \code{labelExtra}, \code{labelVars}).\cr
This variable is a factor whose levels are ordered based on the order
of the condition specified in \code{paramsList}.
}
\description{
This typically converts the data from a wide to a long format.
For each variable, a subset of interest based on a condition can be specified.
}
\author{
Laure Cougnaud
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/exportSummaryStatisticsTable.R
\name{export.summaryTable}
\alias{export.summaryTable}
\alias{exportSummaryStatisticsTable}
\title{Export a summary table to \code{docx}, \code{pptx} 
or \code{html} format (interactive table)}
\usage{
\method{export}{summaryTable}(...)

exportSummaryStatisticsTable(
  summaryTable,
  rowVar = getAttribute(summaryTable, "rowVar"),
  rowVarLab = getAttribute(summaryTable, "rowVarLab", default = getLabelVar(rowVar,
    labelVars = labelVars)),
  rowVarInSepCol = NULL,
  rowVarFormat = NULL,
  rowVarTotalInclude = getAttribute(summaryTable, "rowVarTotalInclude"),
  rowTotalLab = NULL,
  rowVarTotalInSepRow = getAttribute(summaryTable, "rowVarTotalInSepRow"),
  rowAutoMerge = TRUE,
  colVar = getAttribute(summaryTable, "colVar"),
  colTotalLab = getAttribute(summaryTable, "colTotalLab", default = "Total"),
  colHeaderTotalInclude = TRUE,
  colHeaderMerge = TRUE,
  statsVar = getAttribute(summaryTable, "statsVar"),
  statsLayout = getAttribute(summaryTable, "statsLayout", default = ifelse("DT" \%in\%
    outputType, "col", "row")),
  statsValueLab = "StatisticValue",
  statsLabInclude = NULL,
  emptyValue = "-",
  labelVars = NULL,
  file = NULL,
  title = NULL,
  outputType = "flextable",
  pageDim = NULL,
  columnsWidth = NULL,
  landscape = (style == "presentation"),
  margin = 1,
  rowPadBase = 14.4,
  footer = NULL,
  style = "report",
  colorTable = getColorPaletteTable(style = style),
  fontsize = switch(style, report = 8, presentation = 10),
  fontname = switch(style, report = "Times", presentation = "Tahoma"),
  vline = "none",
  hline = "auto",
  expandVar = NULL,
  noEscapeVar = NULL,
  barVar = NULL,
  ...
)
}
\arguments{
\item{...}{(DT output) Extra parameters passed to the 
\code{\link[clinUtils]{getClinDT}}}

\item{summaryTable}{A \code{\link{summaryTable}} object.}

\item{rowVar}{Character vector with variable(s)
to be included in the rows.
If multiple variables are specified, the variables should 
be sorted in hierarchical order
(e.g. body system class before adverse event term)
and are nested in the table.}

\item{rowVarLab}{Named character vector with 
label for the \code{rowVar} variable(s).}

\item{rowVarInSepCol}{Character vector with \code{rowVar}
that should be included in separated columns.
By default (NULL), all row variables are nested in the first column 
of the table.\cr
To include the groups within a \code{var} variable in a separated column, 
set: \code{rowVarInSepCol == 'variableGroup'}.}

\item{rowVarFormat}{(flextable output) Named list with special formatting for the \code{rowVar}.
Currently, only possibility is to set the variable elements in bold, with:
list(var1 = "bold").
(Use 'variable' for \code{var} or 'variableGroup' for group within categorical variables.)}

\item{rowVarTotalInclude}{Character vector with \code{rowVar}
for which the total should be reported.\cr
If the higher row variable is specified, the total across all rows
is reported. \cr
For the export, these variable(s) are formatted as factor with 
\strong{'Total' as the first level}.}

\item{rowTotalLab}{(flextable output) string with label for the row with total.}

\item{rowVarTotalInSepRow}{Character vector with \code{rowVarTotalInclude}
(not in \code{rowVarInSepCol}) for which the total should be included in a separated row labelled 'Total'.
Otherwise (by default) the total is included in the header row of each category.}

\item{rowAutoMerge}{(flextable output) Logical, if TRUE (by default) automatically merge rows,
e.g. in case there is only one sub-category (e.g. categorical variable with only one group)
or only one statistic per category.}

\item{colVar}{Character vector with variable(s) to be included in columns.
If multiple variables are specified, the variables should 
be sorted in hierarchical order,
and are included in multi-columns layout.\cr
Use: 'variable' to include the variables to summarize: \code{var}
 (if multiple) in different columns.}

\item{colTotalLab}{String, label for the total column  'Total' by default.\cr}

\item{colHeaderTotalInclude}{Logical, if TRUE include the total of number of patients
(\code{'statN'}) in the column header.}

\item{colHeaderMerge}{(flextable output) Logical, if TRUE (FALSE by default) 
the column header is merged.}

\item{statsVar}{Character vector with columns of \code{summaryTable} with
statistic variables. For the export: if not specified, all columns of \code{data}
besides row, column variables, 'variable', 'variableGroup'
and 'isTotal' are considered.}

\item{statsLayout}{String with layout for the statistics names 
(in case more than one statistic is included), among:
\itemize{
\item{row (by default for 'flextable' output): \cr All statistics are
 included in different rows in the first column of the table (after the row variable(s))}
\item{'col' (by default for 'DT' output): \cr Statistics are included 
in separated columns (last row of the header).\cr
This option is not compatible with categorical variable(s).}
\item{'rowInSepCol': \cr Statistics are included in different rows, 
but in a separated column than the \code{rowVar} variable(s)}
}}

\item{statsValueLab}{String with label for the statistic value, 
'StatisticValue' by default.\cr
This is only included in the table if the statistics provided in 
\code{stats} are not named and if no \code{colVar} is specified.}

\item{statsLabInclude}{Logical, if TRUE include the statistic label
in the table. \cr By default only included if more than
one statistic variables are available in the table.}

\item{emptyValue}{String with placeholder used to fill the table for missing values, '-' by default.
This value is typically used e.g. if not all statistics are computed for all specified
row/col/var variables.}

\item{labelVars}{(optional) Named character vector with label for
the row, column variable(s) or variable(s) to summarize. \cr
Labels specified via dedicated parameter: e.g. 
\code{rowVarLab}, \code{colVarLab}, \code{varLab}
have priority on this parameter.}

\item{file}{(Optional) Name of the file the table should be exported to, 
either:
\itemize{
\item{string (of length 1). In this case, depending on the
file extension, the following is exported: 
\itemize{
\item{'txt': summary table in long format ('data.frame-base' \code{outputType})}
\item{'docx': summary table in final format is exported ('flextable' \code{outputType})}
\item{'html': interactive summary table is exported ('DT' \code{outputType})}
}}
\item{named character vector in case of multiple exports.
The names should correspond to the options in \code{outputType}:
\itemize{
\item{for 'data.frame-base' and 'data.frame': filename with 'txt' extension}
\item{for 'flextable': filename with 'docx' extension}
\item{for 'DT': filename with 'html' extension}
}}
}
If NULL (by default), the summary table is not exported but only returned as output.
If \code{byVar} is specified, each table is exported to a separated
file with the suffix: 'file_[i].[ext]' with i the index of the file
(and [ext] the file extension).}

\item{title}{Character vector with title(s) for the table.
Set to NULL (by default) if no title should be included.
If multiple are specified, specified for each element of \code{byVar} (in order of the levels).}

\item{outputType}{String with output type:
\itemize{
\item{'flextable' (by default): \code{\link[flextable]{flextable}} object, with format for
CSR, compatible with Word/PowerPoint export}
\item{'DT': \code{\link[DT]{datatable}} interactive table,
compatible with html export}
\item{'data.frame': data.frame in wide format (with elements in 
\code{colVar} in different columns)}
\item{'data.frame-base': data.frame in long format (with elements in 
\code{colVar} in different rows), useful for QC}
}}

\item{pageDim}{Numeric vector of length 2 with page width and height.\cr
Depending on \code{outputType}:
\itemize{
\item{'flextable': in inches}
\item{'DT': in number of rows in the table.\cr
Currently only the height is used (e.g. \code{c(NA, 4)})
}}}

\item{columnsWidth}{Column widths of the table, for \code{outputType}:
\itemize{
\item{'flextable': the widths can be specified either:
 \itemize{
 \item{in percentage: the sum of the widths should be 1}
 \item{in inch: the sum should be the page width (see \code{\link{getDimPage}})}
} If the widths are not specified for all columns, the width of the remaining last
 columns is set such as the width of the total table is the page width.}
\item{'DT': see \code{\link[clinUtils]{getClinDT}}}
}}

\item{landscape}{(flextable output) Logical, if TRUE the file is in landscape format.\cr
By default: FALSE if \code{style} is 'report' and TRUE if \code{style} is 'presentation'.}

\item{margin}{(flextable output) Margin in the document in inches
(1 by default).
This is used to specify the width of the table, from:
[\code{pageDim[1]} - 2 * margin].}

\item{rowPadBase}{(flextable output) Base padding for row (in points), 
14.4 by default (corresponds to 0.2 inches)}

\item{footer}{(flextable output) Character vector with footer(s) for the table.
Set to NULL (by default) if no footer should be included.}

\item{style}{(flextable output) String with table style,
 either 'report' or 'presentation'.
This parameter affects the fontsize, font family, color of the text and background, 
and table dimensions of the table.}

\item{colorTable}{(flextable output) Named character vector with color for the table background/body/text/line,
e.g. created with the \code{\link{getColorPaletteTable}} function.}

\item{fontsize}{(flextable output) Integer with font size, by default:
8 if \code{style} is 'report' and 10 if \code{style} is 'presentation'.}

\item{fontname}{(flextable output) String with font name, by default:
'Times' if \code{style} is 'report' and 'Tahoma' if \code{style} is 'presentation'.}

\item{vline}{(flextable output) String mentioning how vertical lines 
should be included in the body of the table, either: 
\itemize{
\item{'none' (default): no vertical lines included}
\item{'auto': vertical lines included between sub-groups}
}}

\item{hline}{(flextable output) String mentioning how horizontal lines 
should be included in the body of the table, either: 
\itemize{
\item{'none': no horizontal lines included}
\item{'auto' (default): horizontal lines included between sub-groups}
}}

\item{expandVar}{(DT output) Character vector with variables of the summary table which
should be expanded in the data.}

\item{noEscapeVar}{(DT output) Character vector with variables of \code{summaryTable}
which shouldn't be escaped in the table (e.g. containing URLs).}

\item{barVar}{(DT output) Character vector with variables of \code{summaryTable}
that should be represented as a bar.}
}
\value{
Depending on the \code{outputType}:
\itemize{
\item{'data.frame-base': input summary table in a long format with
all computed statistics}
\item{'data.frame': summary table in a wide format (
different columns for each \code{colVar}), with specified labels}
\item{'flextable' (by default): \code{\link[flextable]{flextable}}
 object with summary table}
\item{'DT': \code{\link[DT]{datatable}} object with summary table}
}
If multiple \code{outputType} are specified, a list of those objects, named
by \code{outputType}.\cr
If \code{byVar} is specified, each object consists of a list of tables,
one for each element in \code{byVar}.
}
\description{
The use of \code{export} is recommended.\cr
\code{exportSummaryStatisticsTable} is retained for
back-compatibility.
}
\section{Functions}{
\itemize{
\item \code{export(summaryTable)}: export \code{\link{summaryTable}} object

}}
\author{
Laure Cougnaud
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stats-utility.R
\name{cv}
\alias{cv}
\title{Compute the percentage coefficient of variation,
(in a scale from 0 to 100).}
\usage{
cv(x, na.rm = FALSE)
}
\arguments{
\item{x}{Numeric vector.}

\item{na.rm}{Logical, should NA value(s) be removed (FALSE by default)?}
}
\value{
Numeric vector of length 1 with coefficient of variation.
}
\description{
The coefficient of variation is computed as:
\eqn{\frac{\sigma(x)}{\bar{x}}*100}, with:
\itemize{
\item{\eqn{\sigma(x)}: standard deviation of \code{x}}
\item{\eqn{\bar{x}}: arithmetic mean of \code{x}}
}
}
\examples{
# coefficient of variation of normal distribution tends to 100\%
cv(rnorm(n = 1000, mean = 1, sd = 1))
}
\seealso{
Other stats utility functions:
\code{\link[=geomCV]{geomCV()}},
\code{\link[=geomMean]{geomMean()}},
\code{\link[=geomSD]{geomSD()}},
\code{\link[=geomSE]{geomSE()}},
\code{\link[=se]{se()}}
}
\author{
Laure Cougnaud
}
\concept{stats utility functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getSummaryStatisticsTable.R
\name{getSummaryStatisticsTable}
\alias{getSummaryStatisticsTable}
\title{Get summary statistics table}
\usage{
getSummaryStatisticsTable(
  data,
  var = NULL,
  varFlag = NULL,
  varLab = NULL,
  varLabInclude = length(var) > 1,
  varInclude0 = FALSE,
  varIgnore = NULL,
  varGeneralLab = "Variable",
  varSubgroupLab = "Variable group",
  varIncludeTotal = FALSE,
  varTotalInclude = FALSE,
  varTotalInSepRow = FALSE,
  rowVar = NULL,
  rowVarLab = NULL,
  rowVarDataLevels = NULL,
  rowOrder = "auto",
  rowOrderTotalFilterFct = NULL,
  rowOrderCatLast = NULL,
  rowVarInSepCol = NULL,
  rowVarFormat = NULL,
  rowVarTotalInclude = NULL,
  rowVarTotalByVar = NULL,
  rowVarTotalInSepRow = NULL,
  rowTotalLab = NULL,
  rowInclude0 = FALSE,
  rowAutoMerge = TRUE,
  emptyValue = "-",
  rowVarTotalPerc = NULL,
  colVar = NULL,
  colVarTotal = colVar,
  colVarTotalPerc = colVarTotal,
  colInclude0 = FALSE,
  colVarDataLevels = NULL,
  colTotalInclude = FALSE,
  colTotalLab = "Total",
  stats = NULL,
  statsExtra = NULL,
  statsVarBy = NULL,
  statsPerc = c("statN", "statm"),
  statsGeneralLab = "Statistic",
  statsValueLab = "StatisticValue",
  statsLabInclude = NULL,
  subjectVar = "USUBJID",
  filterFct = NULL,
  dataTotal = NULL,
  dataTotalPerc = dataTotal,
  dataTotalRow = NULL,
  dataTotalCol = NULL,
  type = "auto",
  byVar = NULL,
  byVarLab = NULL,
  checkVarDiffBySubj = "error",
  labelVars = NULL,
  outputType = "flextable",
  statsLayout = ifelse("DT" \%in\% outputType, "col", "row"),
  landscape = (style == "presentation"),
  margin = 1,
  rowPadBase = 14.4,
  title = NULL,
  footer = NULL,
  file = NULL,
  style = "report",
  colorTable = getColorPaletteTable(style = style),
  colHeaderTotalInclude = TRUE,
  colHeaderMerge = TRUE,
  fontsize = switch(style, report = 8, presentation = 10),
  fontname = switch(style, report = "Times", presentation = "Tahoma"),
  vline = "none",
  hline = "auto",
  pageDim = NULL,
  columnsWidth = NULL,
  expandVar = NULL,
  noEscapeVar = NULL,
  barVar = NULL,
  ...
)
}
\arguments{
\item{data}{Data.frame with dataset to consider for the summary table.}

\item{var}{Character vector with variable(s) of \code{data}, 
to compute statistics on.\cr
If NULL (by default), counts by row/column variable(s) are computed.\cr
To also return counts of the \code{rowVar} in case other \code{var}
are specified, you can include: 'all' in the \code{var}.\cr
Missing values, if present, are filtered 
(also for the report of number of subjects/records).}

\item{varFlag}{Character vector, subset of \code{var} with variable(s) 
of type 'flag' (with 'Y', 'N' or '' for empty/non specified value).
Only the counts for records flagged (with 'Y') are retained.}

\item{varLab}{Named character vector with label for each variable 
specified in \code{var}.
By default, extracted from the \code{labelVars}.
if not available, \code{var} is used.}

\item{varLabInclude}{Logical, if TRUE
the name of the summary statistic variable(s) (\code{var})
are included in the table.
This is automatically set to TRUE if more than one variable(s) 
and is specified, and FALSE if only one variable is specified.}

\item{varInclude0}{Logical, should rows with no counts 
for the count \code{var} or \code{varFlag} variable(s)
be included in the table?
Either:
\itemize{
\item{logical of length 1, if TRUE (FALSE by default) 
rows with no count are included for all \code{var}
}
\item{a character vector containing categorical \code{var} 
for which zero counts rows should be included}
}}

\item{varIgnore}{Vector with elements to ignore in the \code{var} variable(s).
The \code{data} records with such elements in \code{var} are \strong{filtered} from the data 
at the start of the workflow.}

\item{varGeneralLab}{String with general label for variable specified in \code{var}.
In case of multiple variable in \code{var}, this will be included in the table header
(see 'rowVarLab' attribute of the output).}

\item{varSubgroupLab}{String with general label for sub-group of
categorical variable(s) for count table, 'Variable group' by default.
This will be included in the final table header (see 'rowVarLab' attribute of the output).}

\item{varIncludeTotal}{This argument is deprecated, please use: 'varTotalInclude' instead.}

\item{varTotalInclude}{Should the total across all categories of \code{var} 
be included for the count table?
Only used for categorical variables (and \code{var} not 'all').
Either:
\itemize{
\item{logical of length 1, if TRUE (FALSE by default) include the total for all categorical \code{var}}
\item{a character vector containing categorical \code{var} for which the total should be included}
}}

\item{varTotalInSepRow}{Logical, should the total per variable be included in
a separated row (by default) or in the row containing the header of the variable?}

\item{rowVar}{Character vector with variable(s)
to be included in the rows.
If multiple variables are specified, the variables should 
be sorted in hierarchical order
(e.g. body system class before adverse event term)
and are nested in the table.}

\item{rowVarLab}{Named character vector with 
label for the \code{rowVar} variable(s).}

\item{rowVarDataLevels}{Data.frame with unique combinations of \code{rowVar}
to be included in columns.
Each column should correspond to \code{colVar} and as factor
if the elements should be ordered in the final table.}

\item{rowOrder}{Specify how the rows should be ordered in the final table, either a:
\itemize{
\item{String among:
\itemize{
\item{'auto' (by default): if the variable is a factor, keep its order, otherwise order alphabetically}
\item{'alphabetical': order alphabetically}
\item{'total': order rows in decreasing order of the total number of subjects
across all columns for this specific category.}
}}
\item{Function with input the summary table and output the ordered elements of the \code{rowVar}}
}
To specify different ordering methods for different \code{rowVar}, specify a list
of such elements, named with the \code{rowVar} variable.
For the table output of \code{\link{computeSummaryStatisticsTable}} (long format),
this order is also reflected in the \strong{\code{levels}} of the row factor variable.}

\item{rowOrderTotalFilterFct}{Function used to filter the data used to order the rows
based on total counts (in case \code{rowOrder} is 'total'),
To order rows based on one specific column category,
e.g. to order based on the counts in the treatment column:
function(x) subset(x, TRTP == "treatmentX")}

\item{rowOrderCatLast}{String with category to be printed in the last 
row of each \code{rowVar} (if any, set to NULL if none).}

\item{rowVarInSepCol}{Character vector with \code{rowVar}
that should be included in separated columns.
By default (NULL), all row variables are nested in the first column 
of the table.\cr
To include the groups within a \code{var} variable in a separated column, 
set: \code{rowVarInSepCol == 'variableGroup'}.}

\item{rowVarFormat}{(flextable output) Named list with special formatting for the \code{rowVar}.
Currently, only possibility is to set the variable elements in bold, with:
list(var1 = "bold").
(Use 'variable' for \code{var} or 'variableGroup' for group within categorical variables.)}

\item{rowVarTotalInclude}{Character vector with \code{rowVar}
for which the total should be reported.\cr
If the higher row variable is specified, the total across all rows
is reported. \cr
For the export, these variable(s) are formatted as factor with 
\strong{'Total' as the first level}.}

\item{rowVarTotalByVar}{Character vector with a row variable
used to categorize the row total.\cr
Note that this is only used if row total(s) is/are requested via \code{rowVarTotalInclude},
and this variable should also be included in \code{rowVar}.
This can be specified also for a specific row variable if the vector is named.\cr
For example: \code{c(ADECOD = "AESEV")} to compute total by severity 
for row adverse event term in a typical adverse event count table 
(by System Organ Class and Adverse Event Term).}

\item{rowVarTotalInSepRow}{Character vector with \code{rowVarTotalInclude}
(not in \code{rowVarInSepCol}) for which the total should be included in a separated row labelled 'Total'.
Otherwise (by default) the total is included in the header row of each category.}

\item{rowTotalLab}{(flextable output) string with label for the row with total.}

\item{rowInclude0}{Logical, if TRUE (FALSE by default),
include rows with no records, based on all combinations 
of the \code{rowVar} (assuming nested variable(s)).}

\item{rowAutoMerge}{(flextable output) Logical, if TRUE (by default) automatically merge rows,
e.g. in case there is only one sub-category (e.g. categorical variable with only one group)
or only one statistic per category.}

\item{emptyValue}{String with placeholder used to fill the table for missing values, '-' by default.
This value is typically used e.g. if not all statistics are computed for all specified
row/col/var variables.}

\item{rowVarTotalPerc}{Character vector with row variables by which the total
should be computed for the denominator for the percentage computation.
By default the total is only computed only by column (NULL by default).
If the total should be based on the total number of records per variable,
\code{rowVarTotalPerc} should be set to 'variable'.}

\item{colVar}{Character vector with variable(s) to be included in columns.
If multiple variables are specified, the variables should 
be sorted in hierarchical order,
and are included in multi-columns layout.\cr
Use: 'variable' to include the variables to summarize: \code{var}
 (if multiple) in different columns.}

\item{colVarTotal}{String with column(s) considered to compute the total by,
reported in the header of the table, by default same as \code{colVar}.
Use: 'variable' to compute total by \code{var} (if multiple).}

\item{colVarTotalPerc}{String with column(s) considered to compute the total by,
used as denominator for the percentage computation, by default same as \code{colVarTotal}.
Use: 'variable' to compute total by \code{var} (if multiple).}

\item{colInclude0}{Logical, if TRUE (FALSE by default),
include columns with no records, based on all combinations 
of the \code{columnVar} (assuming nested variable(s)).
If variable(s) are not nested, possible combinations
can be specified via \code{colVarDataLevels}.}

\item{colVarDataLevels}{Data.frame with unique combinations of \code{colVar}
to be included in columns.
Each column should correspond to \code{colVar} and as factor
if the elements should be ordered in the final table.}

\item{colTotalInclude}{Logical, if TRUE (FALSE by default) include the summary 
statistics across columns in a separated column.}

\item{colTotalLab}{String, label for the total column  'Total' by default.\cr}

\item{stats}{(optional) Statistic(s) of interest to compute, either:
\itemize{
\item{string with the name of a default set of statistics
available in the package, 
see section 'Formatted statistics' in
\code{\link[=inTextSummaryTable-stats]{in-text table statistics}}.\cr
See the corresponding \code{type} parameter of the
\code{\link{getStatsData}} for more information
on how the statistic is internally extracted.}
\item{(expert mode) named list of language object (see \code{\link{is.language}}) 
of base summary statistics of interest, see section:
'Base statistics' in
\code{\link[=inTextSummaryTable-stats]{in-text table statistics}}.\cr
The names are reported in the header.\cr
If \code{stats} if of length 1, the name of the summary statistic is not included
in the table.\cr
The statistics can be specified separately:
\itemize{
\item{for each \code{var} (if multiple), 
by naming each element of the list:
\code{list(varName1 = list(...), varName2 = list())}
}
\item{and/or for each element in:
\code{statsVarBy}, by naming each sublist.}
}}
}}

\item{statsExtra}{(optional) Named list with functions for additional custom
statistics to be computed.\cr
Each function:
\itemize{
\item{has as parameter, either: 'x': the variable (\code{var}) to compute
the summary statistic on or 'data': the entire dataset}
\item{returns the corresponding summary statistic as a numeric vector}
}
For example, to additionally compute the coefficient of variation, this can be set to:
\code{list(statCVPerc = function(x) sd(x)/mean(x)*100)} (or \code{\link{cv}}).}

\item{statsVarBy}{String with variable in \code{rowVar}/\code{colVar}
which the statistics should be computed by.\cr
In this case, \code{stats} (nested list or not) should be additionally nested
to specify the statistics for each element in \code{statsVarBy}.}

\item{statsPerc}{String with 'base statistical variable' used to compute the 
percentage, either: 
\itemize{
\item{'statN' (by default): the number of subjects}
\item{'statm': the number of records}
}}

\item{statsGeneralLab}{String with general label for statistics, 'Statistic' by default.
Only included if no \code{statsVar} if longer than 1.}

\item{statsValueLab}{String with label for the statistic value, 
'StatisticValue' by default.\cr
This is only included in the table if the statistics provided in 
\code{stats} are not named and if no \code{colVar} is specified.}

\item{statsLabInclude}{Logical, if TRUE include the statistic label
in the table. \cr By default only included if more than
one statistic variables are available in the table.}

\item{subjectVar}{String, variable of \code{data} with subject ID,
'USUBJID' by default.}

\item{filterFct}{(optional) Function taking as input
the summary table with computed statistics and returning a subset 
of the summary table.\cr
Note: The filtering function should also handle records with :
\itemize{
\item{total for the column header: \code{isTotal} set to TRUE,
and \code{colVar}/\code{rowVar} is NA.\cr
For example: \code{filterFct = function(data) subset(data, isTotal & myColVar == "group 1")}
}
\item{\code{rowVar}/\code{colVar} set to 'Total'/\code{colTotalLab} 
if \code{rowVarTotalInclude}/\code{colTotalInclude} is specified}
}}

\item{dataTotal}{Data.frame used to extract the Total number of subject
per column in column header ('N = [X]').
It should contain the variables specified by \code{colVarTotal}.
If not specified, the total number of subjects is extracted from the \code{data}.}

\item{dataTotalPerc}{Data.frame used to extract the total counts per column 
for the computation of the percentage.\cr
By default, \code{dataTotal} is used.\cr
It should contain the variables specified by \code{colVarTotalPerc}.}

\item{dataTotalRow}{Data.frame used to extract the total count across all
elements of the row
variable, list of such data.frame for each \code{rowVar} variable.\cr
If the dataset is specified by row variable, the list should be named with:
variable X if the total across elements of variable X should be included.
By default, \code{data} is used.}

\item{dataTotalCol}{Data.frame from which the total across columns is 
extracted (in case \code{colTotalInclude} is TRUE)
or list of such data.frame for each \code{rowVar} variable.\cr
If the dataset is specified by row variable, the list should be named with:
with:
\itemize{
\item{last row variable: for the dataset used in the total column for 
the most nested row variable}
\item{higher row variable (X+1): for the dataset used for the total column
and row total of X}
\item{'total': for the dataset used for the total column and general row total}
}
If only a subset of the variables is specified in this list, 
\code{data} is used for the remaining variable(s) (or 'total') if needed.\cr
This dataset (the one for 'total' if a list) is also used for:
\itemize{
\item{the header of the total column in case \code{dataTotal} is
not specified}
\item{the denominator of the percentages in the total column
in case \code{dataTotalPerc} is not specified}
}
By default, \code{data} is used.}

\item{type}{String with type of table: 
\itemize{
\item{'summaryTable': summary table with statistics for numeric variable}
\item{'countTable': count table}
\item{'auto' (by default): 'summaryTable' if the variable is numeric,
'countTable' otherwise}
}}

\item{byVar}{Variable(s) of \code{data} for which separated table(s)
should be created.}

\item{byVarLab}{String with label for \code{byVar}, used to set the names
of the output list of table(s).}

\item{checkVarDiffBySubj}{String, 'error' (default), 'warning',
or 'none'.  
Should an error, a warning, or nothing be produced
if a continuous variable (\code{var}) contains
different values for the same subject (by row/column)?}

\item{labelVars}{(optional) Named character vector with label for
the row, column variable(s) or variable(s) to summarize. \cr
Labels specified via dedicated parameter: e.g. 
\code{rowVarLab}, \code{colVarLab}, \code{varLab}
have priority on this parameter.}

\item{outputType}{String with output type:
\itemize{
\item{'flextable' (by default): \code{\link[flextable]{flextable}} object, with format for
CSR, compatible with Word/PowerPoint export}
\item{'DT': \code{\link[DT]{datatable}} interactive table,
compatible with html export}
\item{'data.frame': data.frame in wide format (with elements in 
\code{colVar} in different columns)}
\item{'data.frame-base': data.frame in long format (with elements in 
\code{colVar} in different rows), useful for QC}
}}

\item{statsLayout}{String with layout for the statistics names 
(in case more than one statistic is included), among:
\itemize{
\item{row (by default for 'flextable' output): \cr All statistics are
 included in different rows in the first column of the table (after the row variable(s))}
\item{'col' (by default for 'DT' output): \cr Statistics are included 
in separated columns (last row of the header).\cr
This option is not compatible with categorical variable(s).}
\item{'rowInSepCol': \cr Statistics are included in different rows, 
but in a separated column than the \code{rowVar} variable(s)}
}}

\item{landscape}{(flextable output) Logical, if TRUE the file is in landscape format.\cr
By default: FALSE if \code{style} is 'report' and TRUE if \code{style} is 'presentation'.}

\item{margin}{(flextable output) Margin in the document in inches
(1 by default).
This is used to specify the width of the table, from:
[\code{pageDim[1]} - 2 * margin].}

\item{rowPadBase}{(flextable output) Base padding for row (in points), 
14.4 by default (corresponds to 0.2 inches)}

\item{title}{Character vector with title(s) for the table.
Set to NULL (by default) if no title should be included.
If multiple are specified, specified for each element of \code{byVar} (in order of the levels).}

\item{footer}{(flextable output) Character vector with footer(s) for the table.
Set to NULL (by default) if no footer should be included.}

\item{file}{(Optional) Name of the file the table should be exported to, 
either:
\itemize{
\item{string (of length 1). In this case, depending on the
file extension, the following is exported: 
\itemize{
\item{'txt': summary table in long format ('data.frame-base' \code{outputType})}
\item{'docx': summary table in final format is exported ('flextable' \code{outputType})}
\item{'html': interactive summary table is exported ('DT' \code{outputType})}
}}
\item{named character vector in case of multiple exports.
The names should correspond to the options in \code{outputType}:
\itemize{
\item{for 'data.frame-base' and 'data.frame': filename with 'txt' extension}
\item{for 'flextable': filename with 'docx' extension}
\item{for 'DT': filename with 'html' extension}
}}
}
If NULL (by default), the summary table is not exported but only returned as output.
If \code{byVar} is specified, each table is exported to a separated
file with the suffix: 'file_[i].[ext]' with i the index of the file
(and [ext] the file extension).}

\item{style}{(flextable output) String with table style,
 either 'report' or 'presentation'.
This parameter affects the fontsize, font family, color of the text and background, 
and table dimensions of the table.}

\item{colorTable}{(flextable output) Named character vector with color for the table background/body/text/line,
e.g. created with the \code{\link{getColorPaletteTable}} function.}

\item{colHeaderTotalInclude}{Logical, if TRUE include the total of number of patients
(\code{'statN'}) in the column header.}

\item{colHeaderMerge}{(flextable output) Logical, if TRUE (FALSE by default) 
the column header is merged.}

\item{fontsize}{(flextable output) Integer with font size, by default:
8 if \code{style} is 'report' and 10 if \code{style} is 'presentation'.}

\item{fontname}{(flextable output) String with font name, by default:
'Times' if \code{style} is 'report' and 'Tahoma' if \code{style} is 'presentation'.}

\item{vline}{(flextable output) String mentioning how vertical lines 
should be included in the body of the table, either: 
\itemize{
\item{'none' (default): no vertical lines included}
\item{'auto': vertical lines included between sub-groups}
}}

\item{hline}{(flextable output) String mentioning how horizontal lines 
should be included in the body of the table, either: 
\itemize{
\item{'none': no horizontal lines included}
\item{'auto' (default): horizontal lines included between sub-groups}
}}

\item{pageDim}{Numeric vector of length 2 with page width and height.\cr
Depending on \code{outputType}:
\itemize{
\item{'flextable': in inches}
\item{'DT': in number of rows in the table.\cr
Currently only the height is used (e.g. \code{c(NA, 4)})
}}}

\item{columnsWidth}{Column widths of the table, for \code{outputType}:
\itemize{
\item{'flextable': the widths can be specified either:
 \itemize{
 \item{in percentage: the sum of the widths should be 1}
 \item{in inch: the sum should be the page width (see \code{\link{getDimPage}})}
} If the widths are not specified for all columns, the width of the remaining last
 columns is set such as the width of the total table is the page width.}
\item{'DT': see \code{\link[clinUtils]{getClinDT}}}
}}

\item{expandVar}{(DT output) Character vector with variables of the summary table which
should be expanded in the data.}

\item{noEscapeVar}{(DT output) Character vector with variables of \code{summaryTable}
which shouldn't be escaped in the table (e.g. containing URLs).}

\item{barVar}{(DT output) Character vector with variables of \code{summaryTable}
that should be represented as a bar.}

\item{...}{(DT output) Extra parameters passed to the 
\code{\link[clinUtils]{getClinDT}}}
}
\value{
Depending on the \code{outputType}:
\itemize{
\item{'data.frame-base': input summary table in a long format with
all computed statistics}
\item{'data.frame': summary table in a wide format (
different columns for each \code{colVar}), with specified labels}
\item{'flextable' (by default): \code{\link[flextable]{flextable}}
 object with summary table}
\item{'DT': \code{\link[DT]{datatable}} object with summary table}
}
If multiple \code{outputType} are specified, a list of those objects, named
by \code{outputType}.\cr
If \code{byVar} is specified, each object consists of a list of tables,
one for each element in \code{byVar}.
}
\description{
Get summary statistics table
}
\author{
Laure Cougnaud
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inTextSummaryTable-common-args-doc.R
\name{inTextSummaryTable-flextable-args}
\alias{inTextSummaryTable-flextable-args}
\title{Common arguments for the functionalities of the inTextSummaryTable package
for flextable export.}
\arguments{
\item{style}{(flextable output) String with table style,
 either 'report' or 'presentation'.
This parameter affects the fontsize, font family, color of the text and background, 
and table dimensions of the table.}

\item{rowTotalLab}{(flextable output) string with label for the row with total.}

\item{rowAutoMerge}{(flextable output) Logical, if TRUE (by default) automatically merge rows,
e.g. in case there is only one sub-category (e.g. categorical variable with only one group)
or only one statistic per category.}

\item{rowVarFormat}{(flextable output) Named list with special formatting for the \code{rowVar}.
Currently, only possibility is to set the variable elements in bold, with:
list(var1 = "bold").
(Use 'variable' for \code{var} or 'variableGroup' for group within categorical variables.)}

\item{rowPadBase}{(flextable output) Base padding for row (in points), 
14.4 by default (corresponds to 0.2 inches)}

\item{fontname}{(flextable output) String with font name, by default:
'Times' if \code{style} is 'report' and 'Tahoma' if \code{style} is 'presentation'.}

\item{fontsize}{(flextable output) Integer with font size, by default:
8 if \code{style} is 'report' and 10 if \code{style} is 'presentation'.}

\item{margin}{(flextable output) Margin in the document in inches
(1 by default).
This is used to specify the width of the table, from:
[\code{pageDim[1]} - 2 * margin].}

\item{colorTable}{(flextable output) Named character vector with color for the table background/body/text/line,
e.g. created with the \code{\link{getColorPaletteTable}} function.}

\item{landscape}{(flextable output) Logical, if TRUE the file is in landscape format.\cr
By default: FALSE if \code{style} is 'report' and TRUE if \code{style} is 'presentation'.}

\item{footer}{(flextable output) Character vector with footer(s) for the table.
Set to NULL (by default) if no footer should be included.}

\item{vline}{(flextable output) String mentioning how vertical lines 
should be included in the body of the table, either: 
\itemize{
\item{'none' (default): no vertical lines included}
\item{'auto': vertical lines included between sub-groups}
}}

\item{hline}{(flextable output) String mentioning how horizontal lines 
should be included in the body of the table, either: 
\itemize{
\item{'none': no horizontal lines included}
\item{'auto' (default): horizontal lines included between sub-groups}
}}

\item{file}{String with path of the file where the table should be exported.
The file should have the extension: '.html'.
If NULL, the summary table is not exported but only returned as output.
If \code{byVar} is specified, each table is exported to a separated
file with the suffix: 'file_[i].html' with i the index of the file.}

\item{colHeaderMerge}{(flextable output) Logical, if TRUE (FALSE by default) 
the column header is merged.}
}
\value{
No return value, used for the documentation of 
R functions for 'flextable' output
}
\description{
Common arguments for the functionalities of the inTextSummaryTable package
for flextable export.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/decimals.R
\name{getNDecimalsData}
\alias{getNDecimalsData}
\title{Get number of decimals based on the data in a numeric vector.
Note: NA is returned if the element is missing (NA).}
\usage{
getNDecimalsData(x)
}
\arguments{
\item{x}{Numeric vector.}
}
\value{
Numeric vector of same length than \code{x}
with the number of decimals.
}
\description{
Get number of decimals based on the data in a numeric vector.
Note: NA is returned if the element is missing (NA).
}
\examples{
x <- c(0.99, 5.679, 50.45, 1450)
getNDecimalsData(x)
}
\seealso{
Other decimals:
\code{\link[=formatPercentage]{formatPercentage()}},
\code{\link[=getMaxNDecimals]{getMaxNDecimals()}},
\code{\link[=getMaxNDecimalsData]{getMaxNDecimalsData()}},
\code{\link[=getNDecimals]{getNDecimals()}}
}
\author{
Laure Cougnaud
}
\concept{decimals}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stats-doc.R
\name{inTextSummaryTable-stats}
\alias{inTextSummaryTable-stats}
\title{Statistics in the in-text table package.}
\value{
No return value, used for the documentation of 
the \code{stats} parameter
}
\description{
In the in-text package, different set of statistics are available.
}
\details{
The statistics are first computed as numeric
('Base statistics' section below),
then formatted to be displayed in the table
('Formatted statistics' section below).
}
\section{Base statistics}{


In the in-text package, the following 'base
statistics' are reported in the summary table:
\itemize{
\item{for a continuous variable: 
\itemize{
\item{'statMean': variable mean}
\item{'statSD': variable standard deviation}
\item{'statSE': variable standard error}
\item{'statMedian': variable median}
\item{'statMin': variable minimum}
\item{'statMax': variable maximum}
\item{'statGeomMean' (only if requested): variable 
geometric mean (with \code{\link[inTextSummaryTable]{geomMean}})}
\item{'statGeomSD' (only if requested): variable geometric 
standard deviation (with \code{\link[inTextSummaryTable]{geomSD}})}
}
During the computation of the statistics,
if multiple and different values are available for a specific variable
and subject ID (by row/column): an error is triggered.
}
\item{for a categorical and continuous variable (or the full table): 
\itemize{
\item{'statN': number of subjects}
\item{'statm': number of records}
\item{'statPercN' (or 'statPercm'): percentage of subjects
(or records) for the specific group}
\item{'statPercTotalN' (or 'statPercTotalm'): number of subjects 
(or records) considered for the total (denominator) of the percentage}
}
The percentage and denominator of the percentage are based
on the number of subjects or records depending on
the \code{statsPerc} parameter.
}
}
These statistics are reported as numeric and
non rounded in the summary table, and are
typically used as input for the formatted statistics,
or for plots.
}

\section{Formatted statistics}{


In the in-text package, the following formatted
statistics can be reported in the final output table.
\itemize{

\item{for a continuous variable: 
\itemize{

\item{base statistics: 
\itemize{
\item{'Mean': formatted mean}
\item{'Median': formatted median}
\item{'SE': formatted standard error}
\item{'SD': formatted standard deviation}
\item{'Min': formatted minimum}
\item{'Max': formatted maximum}
\item{'Geometric Mean' (only if requested): formatted geometric mean}
\item{'Geometric SD' (only if requested): formatted geometric standard 
deviation}
}}

\item{multiple: 
\itemize{
\item{'summary-default': default set of statistics for a continuous variable:
 'n', 'Mean', 'SD', 'SE', 'Median', 'Min', 'Max'}
\item{'summary': all statistics available for a continuous variable:
'n', 'Mean', 'SD', 'SE', 'Median', 'Min', 'Max', '\%', 'm'}
}}

\item{combined statistics: 
\itemize{
\item{'median (range)': median (minimum,maximum)}
\item{'median\\n(range)': median and (minimum, maximum) below (linebreak)}
\item{'mean (sd)': mean and standard deviation}
\item{'geometric mean (geometric sd)': geometric mean and geometric standard 
deviation}
\item{'mean (se)': mean and standard error}
\item{'mean (range)': mean and (minimum, maximum)}
\item{'(min, max)': (minimum, maximum)}
}}

}}

\item{for a categorical or continuous variable (or the full table): 
\itemize{

\item{base statistics: 
\itemize{
\item{'n': formatted number of subjects}
\item{'m': formatted number of records}
\item{'\%': formatted percentage of subjects}
\item{'\%m': formatted percentage of records.\cr
Note: this is only available if the percentage of records is reported
(\code{statsPerc} set to 'statm').}
}}

\item{multiple: 
\itemize{
\item{'count-default': default set of statistics for a categorical variable:
 'n', '\%'}
\item{'count': all statistics available for a categorical variable: 'n', '\%', 'm'}
}}

\item{combined statistics: 
\itemize{
\item{'n (\%)': number of subjects (and associated percentage)}
\item{'n/N (\%)': number of subjects/total number of subjects (percentage)}
\item{'m (\%)': number of records (and associated percentage).\cr
Note: this is only available if the percentage of records is reported
(\code{statsPerc} set to 'statm').}
}}

}}
#' These statistics are specified via the \code{stats} parameter 
(or \code{type} parameter of \code{\link{getStats}}).\cr
These statistics are reported as text variables
in the summary table (as data.frame),
and typically displayed inside the final table.
}
}

\section{Statistics formatting}{

\itemize{

\item{In general, all rounding is handled with \code{\link[clinUtils]{roundHalfUpTextFormat}}.}

\item{statistics for continuous variable: 
\itemize{
\item{if the number of decimals (\code{nDecCont}) is specified: 
\cr statistics are rounded with the following number of decimals,
based on:
\itemize{
\item{'Min', 'Max': \code{nDecCont}}
\item{'Mean', 'SD', 'Median', 'Geometric Mean', 'Geometric SD': \code{nDecCont} + 1}
\item{'SE': \code{nDecCont} + 2}
}
Note that the number of decimals is extracted
from standard rules/data is the variable of interest 
is specified (e.g. via \code{var} in \code{\link{getStatsData}}).
}
\item{if the number of decimals is not specified: 
\cr a default format is set via the \code{\link{formatC}} function.}
}}

\item{statistics for counts:
\itemize{

\item{number of subjects, records are rounded with the number of decimals
specified via \code{nDecN} or \code{nDecm} (0 by default)}

\item{percentages are formatted by default with \code{\link{formatPercentage}}.}

\item{'n (\%)' and 'm (\%)':  
\itemize{
\item{if the percentage of subjects/records is missing, '-' is reported}
\item{if the number of subjects/records is 0, '0' is reported instead of '0 (0\%)'}
\item{otherwise the number and percentage of subjects/records are formatted as specified}
}}

\item{'n/N (\%)': 
\itemize{
\item{if the percentage of subjects is missing, '-' is reported}
\item{if the number of subjects is 0, '0' is reported instead of '0/... (0\%)'}
\item{otherwise the number and percentage of subjects and total are formatted as specified}
}}

}}

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DT-exportSummaryStatisticsTableToDT.R
\name{exportSummaryStatisticsTableToDT}
\alias{exportSummaryStatisticsTableToDT}
\title{Export summary table to an interactive DT table,
e.g. to be exported into an html document.}
\usage{
exportSummaryStatisticsTableToDT(
  summaryTable,
  rowVar = getAttribute(summaryTable, "rowVar"),
  rowVarLab = getAttribute(summaryTable, "rowVarLab", default = getLabelVar(rowVar,
    labelVars = labelVars)),
  rowVarInSepCol = NULL,
  statsVar = getAttribute(summaryTable, "statsVar"),
  statsLayout = getAttribute(summaryTable, "statsLayout", default = "col"),
  statsValueLab = "StatisticValue",
  title = NULL,
  expandVar = NULL,
  noEscapeVar = NULL,
  barVar = NULL,
  pageDim = NULL,
  labelVars = NULL,
  file = NULL,
  ...
)
}
\arguments{
\item{summaryTable}{Summary table as provided by the 
\code{\link{formatSummaryStatisticsTable}}.}

\item{rowVar}{Character vector with variable(s)
to be included in the rows.
If multiple variables are specified, the variables should 
be sorted in hierarchical order
(e.g. body system class before adverse event term)
and are nested in the table.}

\item{rowVarLab}{Named character vector with 
label for the \code{rowVar} variable(s).}

\item{rowVarInSepCol}{Character vector with \code{rowVar}
that should be included in separated columns.
By default (NULL), all row variables are nested in the first column 
of the table.\cr
To include the groups within a \code{var} variable in a separated column, 
set: \code{rowVarInSepCol == 'variableGroup'}.}

\item{statsVar}{Character vector with columns of \code{summaryTable} with
statistic variables. For the export: if not specified, all columns of \code{data}
besides row, column variables, 'variable', 'variableGroup'
and 'isTotal' are considered.}

\item{statsLayout}{String with layout for the statistics names 
(in case more than one statistic is included), among:
\itemize{
\item{row (by default for 'flextable' output): \cr All statistics are
 included in different rows in the first column of the table (after the row variable(s))}
\item{'col' (by default for 'DT' output): \cr Statistics are included 
in separated columns (last row of the header).\cr
This option is not compatible with categorical variable(s).}
\item{'rowInSepCol': \cr Statistics are included in different rows, 
but in a separated column than the \code{rowVar} variable(s)}
}}

\item{statsValueLab}{String with label for the statistic value, 
'StatisticValue' by default.\cr
This is only included in the table if the statistics provided in 
\code{stats} are not named and if no \code{colVar} is specified.}

\item{title}{Character vector with title(s) for the table.
Set to NULL (by default) if no title should be included.
If multiple are specified, specified for each element of \code{byVar} (in order of the levels).}

\item{expandVar}{(DT output) Character vector with variables of the summary table which
should be expanded in the data.}

\item{noEscapeVar}{(DT output) Character vector with variables of \code{summaryTable}
which shouldn't be escaped in the table (e.g. containing URLs).}

\item{barVar}{(DT output) Character vector with variables of \code{summaryTable}
that should be represented as a bar.}

\item{pageDim}{Numeric vector of length 2 with page width and height.\cr
Depending on \code{outputType}:
\itemize{
\item{'flextable': in inches}
\item{'DT': in number of rows in the table.\cr
Currently only the height is used (e.g. \code{c(NA, 4)})
}}}

\item{labelVars}{(optional) Named character vector with label for
the row, column variable(s) or variable(s) to summarize. \cr
Labels specified via dedicated parameter: e.g. 
\code{rowVarLab}, \code{colVarLab}, \code{varLab}
have priority on this parameter.}

\item{file}{String with path of the file where the table should be exported.
The file should have the extension: '.docx'.
If NULL, the summary table is not exported but only returned as output.
If \code{byVar} is specified, each table is exported to a separated
file with the suffix: 'file_[i].docx' with i the index of the file.}

\item{...}{(DT output) Extra parameters passed to the 
\code{\link[clinUtils]{getClinDT}}}
}
\value{
A \code{\link[DT]{datatable}} object.
}
\description{
Export summary table to an interactive DT table,
e.g. to be exported into an html document.
}
\author{
Laure Cougnaud
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/miscellaneous.R
\name{pageDimPresentation}
\alias{pageDimPresentation}
\title{Page dimension for powerpoints}
\usage{
pageDimPresentation
}
\description{
Page dimension for powerpoints
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/decimals.R
\name{getNDecimalsRule}
\alias{getNDecimalsRule}
\title{Get number of decimals based pre-defined rule(s).}
\usage{
getNDecimalsRule(x, rule = c("1"))
}
\arguments{
\item{x}{Numeric vector.}

\item{rule}{Character vector with rule to use to derive 
the number of parameters.
Currently only: '1' is implemented.
\itemize{
\item{'1': standard rule for the number of 
decimals for individual values for a 
continuous variable:
\itemize{
\item{value < 1 ('very small values'): 3}
\item{value < 10: 2}
\item{value in [10, 1000[: 1}
\item{value >= 1000: 0}
}
}
}}
}
\value{
Numeric vector of same length than \code{x}
with the number of decimals.
}
\description{
Note: NA is returned if the element is missing (NA).
}
\examples{
x <- c(0.99, 5.679, 50.45, 1450)
getNDecimalsRule(x = x)
}
\author{
Laure Cougnaud
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stats-utility.R
\name{geomMean}
\alias{geomMean}
\title{Compute geometric mean.}
\usage{
geomMean(x, na.rm = FALSE)
}
\arguments{
\item{x}{Numeric vector.}

\item{na.rm}{Logical, should NA value(s) be removed (FALSE by default)?}
}
\value{
Numeric vector of length 1 with geometric mean.
}
\description{
The geometric mean is computed as:
\eqn{\exp(\bar{log(x)})}, with:
\itemize{
\item{log: natural logarithm}
\item{\eqn{\bar{log(x)}}: arithmetic mean of log(x)}
}
}
\examples{
# geometric mean of a big sample from log normal distribution
# tends to the mean of the distribution:
geomMean(rlnorm(n = 1000, meanlog = 0, sdlog = 1))
}
\seealso{
Other stats utility functions:
\code{\link[=cv]{cv()}},
\code{\link[=geomCV]{geomCV()}},
\code{\link[=geomSD]{geomSD()}},
\code{\link[=geomSE]{geomSE()}},
\code{\link[=se]{se()}}
}
\author{
Laure Cougnaud
}
\concept{stats utility functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stats-utility.R
\name{se}
\alias{se}
\title{Compute standard error of the mean.}
\usage{
se(x, na.rm = FALSE)
}
\arguments{
\item{x}{Numeric vector.}

\item{na.rm}{Logical, should NA value(s) be removed (FALSE by default)?}
}
\value{
Numeric vector with standard error of the mean
}
\description{
The standard error of the mean is computed as:
\eqn{\frac{\sigma(x)}{\sqrt{length(x)}}}, with:\cr
\eqn{\sigma(x)}: standard deviation of \code{x}
}
\examples{
se(rnorm(1000))
}
\seealso{
Other stats utility functions:
\code{\link[=cv]{cv()}},
\code{\link[=geomCV]{geomCV()}},
\code{\link[=geomMean]{geomMean()}},
\code{\link[=geomSD]{geomSD()}},
\code{\link[=geomSE]{geomSE()}}
}
\author{
Laure Cougnaud
}
\concept{stats utility functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summaryTable-combine.R
\name{combine.summaryTable}
\alias{combine.summaryTable}
\title{Combine summary statistics table}
\usage{
\method{combine}{summaryTable}(..., summaryTables, combineVar = NULL, combineDir = c("row", "col"))
}
\arguments{
\item{...}{\code{\link{summaryTable}} objects.}

\item{summaryTables}{List of \code{\link{summaryTable}} objects.}

\item{combineVar}{(optional) String with name
of a new variable tracking from which table
each record originally come from.\cr
The label for each table is extracted from the names
of the \code{summaryTables} list, or 1:length(tables)
if the list is not named.\cr
If not specified (by default), the tables will be combined
but the information on which input table:
each record from the combined table belongs to will
not be retained.}

\item{combineDir}{String indicating
on which direction: 'row' or 'col' (a.k.a column)
the information on the table appartenance 
(\code{combineVar}) will
be displayed when the table is exported.}
}
\value{
A combined \code{\link{summaryTable}}.
}
\description{
\itemize{
\item{A new table is created, combining the tables by rows.}
\item{The attributes of the combined summary table are 
obtained by combining the attributes of all summary tables
(and removing duplicates).}
}
}
\details{
\itemize{
\item{In case only a set of tables contain categorical variable,
in nested rows, so the \code{variable} and \code{variableGroup}
variables, these variables are included as last in 
the 'row variables' attribute of the combined table.}
\item{Only one row is retained for the columns totals
per column variable (the first one in order of appearance).\cr
The column totals are not included if the column
variable(s) are not the same across tables, or
if the totals differ between tables.}
}
}
\author{
Laure Cougnaud
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/flextable-getListing.R
\name{createFlextableWithHeader}
\alias{createFlextableWithHeader}
\title{Create a flextable, setting the column names to syntactic names
if it is not the case.}
\usage{
createFlextableWithHeader(
  data,
  headerDf = NULL,
  title = NULL,
  includeRownames = TRUE,
  colHeaderMerge = TRUE
)
}
\arguments{
\item{data}{Data.frame with data.}

\item{headerDf}{(optional) Data.frame with header.
This should contain the same number of columns than \code{data}
(+ if \code{includeRownames} is TRUE) and optionally multiple rows. 
Neighbouring cells with same content
will be represented merged in the output.}

\item{title}{Character vector with title(s) for the table.
Set to NULL (by default) if no title should be included.}

\item{includeRownames}{Logical, if TRUE (by default)
rownames are included in the \code{\link[flextable]{flextable}} object.}

\item{colHeaderMerge}{(flextable output) Logical, if TRUE (FALSE by default) 
the column header is merged.}
}
\value{
list with:
\itemize{
\item{'ft': \code{\link[flextable]{flextable}}}
\item{'colsData': Named vector with original column names,
with names set to new syntactic names.}
}
}
\description{
Create a flextable, setting the column names to syntactic names
if it is not the case.
}
\author{
Laure Cougnaud
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/miscellaneous.R
\name{convertVarFlag}
\alias{convertVarFlag}
\title{Convert flag variable to a format such as only the flagged records are counted
in the summary table.}
\usage{
convertVarFlag(x)
}
\arguments{
\item{x}{Character or factor variable with flag variable,
should contain elements: 'Y' and 'N', or '' (for missing value).}
}
\value{
Formatted factor variable with levels: 'Y' & 'N'.
Empty strings have been converted to NA.
}
\description{
Convert flag variable to a format such as only the flagged records are counted
in the summary table.
}
\author{
Laure Cougnaud
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/flextable-getListing.R
\name{convertVectToBinary}
\alias{convertVectToBinary}
\title{Convert vector to a bincode of 0/1
based on consecutive values in the vector.}
\usage{
convertVectToBinary(x)
}
\arguments{
\item{x}{Vector.}
}
\value{
Integer vector of same length than \code{x}.
}
\description{
Convert vector to a bincode of 0/1
based on consecutive values in the vector.
}
\examples{
x <- c("group1", "group1", "group1", "group2", "group2", "group3", "group4", "group4")
convertVectToBinary(x = x)
}
\author{
Laure Cougnaud
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in
%   R/flextable-formatSummaryStatisticsTableFlextable.R
\name{formatSummaryStatisticsTableFlextable}
\alias{formatSummaryStatisticsTableFlextable}
\title{Merge nested rows of a summary table
for a format compatible with \code{flextable}}
\usage{
formatSummaryStatisticsTableFlextable(
  summaryTable,
  rowVar = getAttribute(summaryTable, "rowVar"),
  rowVarInSepCol = NULL,
  rowVarTotalInclude = getAttribute(summaryTable, "rowVarTotalInclude"),
  statsLayout = "row",
  statsVar = getAttribute(summaryTable, "statsVar"),
  statsLabInclude = getAttribute(summaryTable, "statsLabInclude", default =
    length(statsVar) > 1),
  rowVarLab = getAttribute(summaryTable, "rowVarLab", default = getLabelVar(rowVar,
    labelVars = labelVars)),
  rowVarTotalInSepRow = NULL,
  vline = c("none", "auto"),
  hline = c("none", "auto"),
  rowAutoMerge = TRUE,
  rowVarFormat = NULL,
  rowTotalLab = NULL,
  labelVars = NULL
)
}
\arguments{
\item{summaryTable}{A \code{\link{summaryTable}} object.}

\item{rowVar}{Character vector with variable(s)
to be included in the rows.
If multiple variables are specified, the variables should 
be sorted in hierarchical order
(e.g. body system class before adverse event term)
and are nested in the table.}

\item{rowVarInSepCol}{Character vector with \code{rowVar}
that should be included in separated columns.
By default (NULL), all row variables are nested in the first column 
of the table.\cr
To include the groups within a \code{var} variable in a separated column, 
set: \code{rowVarInSepCol == 'variableGroup'}.}

\item{rowVarTotalInclude}{Character vector with \code{rowVar}
for which the total should be reported.\cr
If the higher row variable is specified, the total across all rows
is reported. \cr
For the export, these variable(s) are formatted as factor with 
\strong{'Total' as the first level}.}

\item{statsLayout}{String with layout for the statistics names 
(in case more than one statistic is included), among:
\itemize{
\item{row (by default for 'flextable' output): \cr All statistics are
 included in different rows in the first column of the table (after the row variable(s))}
\item{'col' (by default for 'DT' output): \cr Statistics are included 
in separated columns (last row of the header).\cr
This option is not compatible with categorical variable(s).}
\item{'rowInSepCol': \cr Statistics are included in different rows, 
but in a separated column than the \code{rowVar} variable(s)}
}}

\item{statsVar}{Character vector with columns of \code{summaryTable} with
statistic variables. For the export: if not specified, all columns of \code{data}
besides row, column variables, 'variable', 'variableGroup'
and 'isTotal' are considered.}

\item{statsLabInclude}{Logical, if TRUE include the statistic label
in the table. \cr By default only included if more than
one statistic variables are available in the table.}

\item{rowVarLab}{Named character vector with 
label for the \code{rowVar} variable(s).}

\item{rowVarTotalInSepRow}{Character vector with \code{rowVarTotalInclude}
(not in \code{rowVarInSepCol}) for which the total should be included in a separated row labelled 'Total'.
Otherwise (by default) the total is included in the header row of each category.}

\item{vline}{(flextable output) String mentioning how vertical lines 
should be included in the body of the table, either: 
\itemize{
\item{'none' (default): no vertical lines included}
\item{'auto': vertical lines included between sub-groups}
}}

\item{hline}{(flextable output) String mentioning how horizontal lines 
should be included in the body of the table, either: 
\itemize{
\item{'none': no horizontal lines included}
\item{'auto' (default): horizontal lines included between sub-groups}
}}

\item{rowAutoMerge}{(flextable output) Logical, if TRUE (by default) automatically merge rows,
e.g. in case there is only one sub-category (e.g. categorical variable with only one group)
or only one statistic per category.}

\item{rowVarFormat}{(flextable output) Named list with special formatting for the \code{rowVar}.
Currently, only possibility is to set the variable elements in bold, with:
list(var1 = "bold").
(Use 'variable' for \code{var} or 'variableGroup' for group within categorical variables.)}

\item{rowTotalLab}{(flextable output) string with label for the row with total.}

\item{labelVars}{(optional) Named character vector with label for
the row, column variable(s) or variable(s) to summarize. \cr
Labels specified via dedicated parameter: e.g. 
\code{rowVarLab}, \code{colVarLab}, \code{varLab}
have priority on this parameter.}
}
\value{
summaryTable reformatted in long format, with extra attributes:
\itemize{
\item{'header': data.frame with header for each column}
\item{'padParams': list of list of parameters to be passed to the 
\code{\link[flextable]{padding}} function}
\item{'rowVar': column of output with row variable}
\item{'rowVarInSepCol': column(s) of output with row variable in separated column(s)}
\item{'vlineParams' and 'hlineParams': 
list of list with correspondingly parameters for
vertical and horizontal lines
}
\item{'vline': \code{vline} parameter}
\item{'formatParams': list of list with special formatting for the table,
currently only used if \code{rowVarFormat} if specified.}
}
If \code{summaryTable} is a list of summary tables,
returns a list of corresponding summary tables in long format.
}
\description{
Merge nested rows of a summary table
for a format compatible with \code{flextable}
}
\author{
Laure Cougnaud
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/computeSummaryStatisticsTable.R
\name{convertVarToFactorWithOrder}
\alias{convertVarToFactorWithOrder}
\title{Convert a variable to a factor with levels in specified order.}
\usage{
convertVarToFactorWithOrder(
  data,
  var,
  otherVars = NULL,
  colVar = NULL,
  colTotalLab = "Total",
  method = c("auto", "alphabetical", "total"),
  totalFilterFct = NULL,
  totalVar = "statN",
  catLast = NULL
)
}
\arguments{
\item{data}{Data.frame with dataset to consider for the summary table.}

\item{var}{String with variable of \code{data} to sort.}

\item{otherVars}{Character vector with other variable(s) of \code{data}
considered in the specific dimension.}

\item{colVar}{Character vector with variable(s) to be included in columns.
If multiple variables are specified, the variables should 
be sorted in hierarchical order,
and are included in multi-columns layout.\cr
Use: 'variable' to include the variables to summarize: \code{var}
 (if multiple) in different columns.}

\item{colTotalLab}{String, label for the total column  'Total' by default.\cr}

\item{method}{Ordering method to use, either:
\itemize{
\item{String among:
\itemize{
\item{'auto': if \code{var} is a factor, keep its order, otherwise alphabetically}
\item{'alphabetical': \code{var} is order in alphabetical order}
\item{'total': \code{var} is ordered based on the \code{totalVar} variable, in decreasing order.
The total count is extracted from the rows with all \code{otherVars} equal to 'Total'.
If no \code{otherVars} is specified, consider all rows.}
}
}
\item{Function to be applied on each subset to get the order elements of the variable}
}}

\item{totalFilterFct}{(optional) Function which returns a subset of the data of interest,
to filter the total data considered for the ordering.}

\item{totalVar}{String with variable of \code{data} considered in case \code{type} is 'total',
'statN' by default.}

\item{catLast}{String with last category (if any).
Set to NULL if no specific category should be included as last element.}
}
\value{
Factor \code{var} variable of \code{data} with specified order.
}
\description{
Convert a variable to a factor with levels in specified order.
}
\author{
Laure Cougnaud
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in
%   R/flextable-formatSummaryStatisticsTableFlextable.R
\name{smoothPadding}
\alias{smoothPadding}
\title{Smooth padding, e.g. remove padding bigger than 1}
\usage{
smoothPadding(pad)
}
\arguments{
\item{pad}{Integer vector with padding.}
}
\value{
Integer vector with 'smooth' padding.
}
\description{
Smooth padding, e.g. remove padding bigger than 1
}
\author{
Laure Cougnaud
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/computeSummaryStatistics.R
\name{computeSummaryStatistics}
\alias{computeSummaryStatistics}
\title{Compute summary statistics of interest of an unique variable of interest.}
\usage{
computeSummaryStatistics(
  data,
  var = NULL,
  varTotalInclude = FALSE,
  statsExtra = NULL,
  subjectVar = "USUBJID",
  filterEmptyVar = TRUE,
  type = "auto",
  checkVarDiffBySubj = c("error", "warning", "none"),
  msgLabel = NULL,
  msgVars = NULL
)
}
\arguments{
\item{data}{Data.frame with dataset to consider for the summary table.}

\item{var}{Character vector with variable(s) of \code{data}, 
to compute statistics on.\cr
If NULL (by default), counts by row/column variable(s) are computed.\cr
To also return counts of the \code{rowVar} in case other \code{var}
are specified, you can include: 'all' in the \code{var}.\cr
Missing values, if present, are filtered 
(also for the report of number of subjects/records).}

\item{varTotalInclude}{Logical (FALSE by default)
Should the total across all categories of \code{var} 
be included for the count table?
Only used if \code{var} is a categorical variable.}

\item{statsExtra}{(optional) Named list with functions for additional custom
statistics to be computed.\cr
Each function:
\itemize{
\item{has as parameter, either: 'x': the variable (\code{var}) to compute
the summary statistic on or 'data': the entire dataset}
\item{returns the corresponding summary statistic as a numeric vector}
}
For example, to additionally compute the coefficient of variation, this can be set to:
\code{list(statCVPerc = function(x) sd(x)/mean(x)*100)} (or \code{\link{cv}}).}

\item{subjectVar}{String, variable of \code{data} with subject ID,
'USUBJID' by default.}

\item{filterEmptyVar}{Logical, if TRUE doesn't return any results
if the variable is empty, otherwise return 0 for the counts and NA for summary 
statistics.
Criterias to consider a variable empty are:
\itemize{
\item{for a continuous variable: all missing (NA)}
\item{for a categorical variable: all missing or **category is included in the 
factor levels but not available in \code{data}**}
}
By default, an empty variable are filtered.}

\item{type}{String with type of table: 
\itemize{
\item{'summaryTable': summary table with statistics for numeric variable}
\item{'countTable': count table}
\item{'auto' (by default): 'summaryTable' if the variable is numeric,
'countTable' otherwise}
}}

\item{checkVarDiffBySubj}{String, 'error' (default), 'warning',
or 'none'.  
Should an error, a warning, or nothing be produced
if a continuous variable (\code{var}) contains
different values for the same subject?}

\item{msgLabel}{(optional) String with label for the data (NULL by default), 
included in the message/warning for checks.}

\item{msgVars}{(optional) Character vector with columns of \code{data}
containing extra variables (besides \code{var} and \code{subjectVar})
that should be included in the message/warning for checks.}
}
\value{
Data.frame with summary statistics in columns,
depending if \code{type} is:
\itemize{
\item{'summary': 
\itemize{
\item{'statN': number of subjects}
\item{'statm': number of records}
\item{'statMean': mean of \code{var}}
\item{'statSD': standard deviation of \code{var}}
\item{'statSE': standard error the mean of \code{var}}
\item{'statMedian': median of \code{var}}
\item{'statMin': minimum of \code{var}}
\item{'statMax': maximum of \code{var}}
}
}
\item{'count': 
\itemize{
\item{'variableGroup': factor with groups of \code{var} for which counts are reported}
\item{'statN': number of subjects}
\item{'statm': number of records}
}
}
}
}
\description{
Additionally, this function run extra checks on the data:
\itemize{
\item{an error message is triggered if any subject (identified by \code{subjectVar}) 
have different values in a continuous \code{var}
}
\item{an indicative message is triggered if multiple but identical records are available
for \code{subjectVar} and a continuous \code{var}}
}
}
\author{
Laure Cougnaud
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in
%   R/flextable-convertSummaryStatisticsTableToFlextable.R
\name{convertSummaryStatisticsTableToFlextable}
\alias{convertSummaryStatisticsTableToFlextable}
\title{Convert summary statistics table to flextable}
\usage{
convertSummaryStatisticsTableToFlextable(
  summaryTable,
  colHeaderMerge = TRUE,
  landscape = (style == "presentation"),
  margin = 1,
  rowPadBase = 14.4,
  title = NULL,
  footer = NULL,
  style = "report",
  colorTable = getColorPaletteTable(style = style),
  fontname = switch(style, report = "Times", presentation = "Tahoma"),
  fontsize = switch(style, report = 8, presentation = 10),
  file = NULL,
  pageDim = NULL,
  columnsWidth = NULL
)
}
\arguments{
\item{summaryTable}{A \code{\link{summaryTable}} object.}

\item{colHeaderMerge}{(flextable output) Logical, if TRUE (FALSE by default) 
the column header is merged.}

\item{landscape}{(flextable output) Logical, if TRUE the file is in landscape format.\cr
By default: FALSE if \code{style} is 'report' and TRUE if \code{style} is 'presentation'.}

\item{margin}{(flextable output) Margin in the document in inches
(1 by default).
This is used to specify the width of the table, from:
[\code{pageDim[1]} - 2 * margin].}

\item{rowPadBase}{(flextable output) Base padding for row (in points), 
14.4 by default (corresponds to 0.2 inches)}

\item{title}{Character vector with title(s) for the table.
Set to NULL (by default) if no title should be included.
If multiple are specified, specified for each element of \code{byVar} (in order of the levels).}

\item{footer}{(flextable output) Character vector with footer(s) for the table.
Set to NULL (by default) if no footer should be included.}

\item{style}{(flextable output) String with table style,
 either 'report' or 'presentation'.
This parameter affects the fontsize, font family, color of the text and background, 
and table dimensions of the table.}

\item{colorTable}{(flextable output) Named character vector with color for the table background/body/text/line,
e.g. created with the \code{\link{getColorPaletteTable}} function.}

\item{fontname}{(flextable output) String with font name, by default:
'Times' if \code{style} is 'report' and 'Tahoma' if \code{style} is 'presentation'.}

\item{fontsize}{(flextable output) Integer with font size, by default:
8 if \code{style} is 'report' and 10 if \code{style} is 'presentation'.}

\item{file}{String with path of the file where the table should be exported.
The file should have the extension: '.html'.
If NULL, the summary table is not exported but only returned as output.
If \code{byVar} is specified, each table is exported to a separated
file with the suffix: 'file_[i].html' with i the index of the file.}

\item{pageDim}{Numeric vector of length 2 with page width and height.\cr
Depending on \code{outputType}:
\itemize{
\item{'flextable': in inches}
\item{'DT': in number of rows in the table.\cr
Currently only the height is used (e.g. \code{c(NA, 4)})
}}}

\item{columnsWidth}{Column widths of the table, for \code{outputType}:
\itemize{
\item{'flextable': the widths can be specified either:
 \itemize{
 \item{in percentage: the sum of the widths should be 1}
 \item{in inch: the sum should be the page width (see \code{\link{getDimPage}})}
} If the widths are not specified for all columns, the width of the remaining last
 columns is set such as the width of the total table is the page width.}
\item{'DT': see \code{\link[clinUtils]{getClinDT}}}
}}
}
\value{
\code{\link[flextable]{flextable}} object with summary table
If \code{summaryTable} is a list of summary tables,
returns a list of \code{\link[flextable]{flextable}}.
}
\description{
Convert summary statistics table to flextable
}
\author{
Laure Cougnaud
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subjectProfileSummaryPlot.R
\name{subjectProfileSummaryPlot}
\alias{subjectProfileSummaryPlot}
\title{Plot subject summary profile.}
\usage{
subjectProfileSummaryPlot(
  data,
  xVar = NULL,
  xLab = getLabelVar(xVar, labelVars = labelVars),
  xAxisExpand = waiver(),
  xGap = NULL,
  xGapDiffNew = NULL,
  meanVar = "statMean",
  seVar = if ("statSE" \%in\% colnames(data)) "statSE",
  minVar = NULL,
  maxVar = NULL,
  yLab = paste(c(sub("^stat", "", meanVar), if (!is.null(minVar) & !is.null(maxVar)) {
  
      paste0("(", sub("^stat", "", minVar), ", ", sub("^stat", "", maxVar), ")")
 } else
    if (!is.null(seVar)) paste("+-", sub("^stat", "", seVar))), collapse = " "),
  facetVar = NULL,
  facetScale = "free_y",
  colorVar = NULL,
  colorLab = getLabelVar(colorVar, labelVars = labelVars),
  colorPalette = NULL,
  labelVars = NULL,
  useLinetype = TRUE,
  linetypePalette = NULL,
  useShape = TRUE,
  shapePalette = NULL,
  jitter = NULL,
  title = NULL,
  caption = NULL,
  yTrans = NULL,
  yLim = NULL,
  xLim = NULL,
  yAxisExpand = c(0.05, 0.05),
  yLimExpand = NULL,
  xAxisLabs = NULL,
  sizePoint = 1.5,
  sizeLine = 0.5,
  sizeLabel = 3.87,
  widthErrorBar = 0.9,
  tableText = NULL,
  tableTextFontface = 1,
  tableHeight = 0.1,
  tableYAxisLabs = !is.null(colorVar),
  tablePlotMargin = unit(0, "pt"),
  label = FALSE,
  labelPadding = unit(1, "lines"),
  byVar = NULL,
  hLine = NULL,
  hLineColor = "black",
  hLineLty = "solid",
  vLine = NULL,
  vLineColor = "black",
  vLineLty = "solid",
  style = "report",
  fontname = switch(style, report = "Times", presentation = "Tahoma"),
  fontsize = switch(style, report = 8, presentation = 10),
  themeFct = switch(style, report = theme_classic, presentation = theme_bw),
  themeIncludeVerticalGrid = TRUE,
  ggExtra = NULL,
  legendPosition = ifelse(!is.null(tableText), "none", "bottom"),
  ...
)
}
\arguments{
\item{data}{Data.frame with summary statistics to represent in the plot,
e.g. a \code{\link{summaryTable}} object.}

\item{xVar}{String, variable of \code{data} with variable for the x-axis.}

\item{xLab}{String with label for the \code{xVar}.}

\item{xAxisExpand}{Object passed to the 'expand' parameter of:
\code{\link[ggplot2]{scale_x_continuous}},
(\code{\link[ggplot2]{waiver}} by default).}

\item{xGap}{(optional) Numeric vector of length 2 for which
a gap should be created in the x-axis.
Only available if \code{xVar} is specified and a numeric variable.
Records with \code{xVar} within \code{xGap} are filtered from the plot,
vertical lines are included at the min/max of the gap,
and the gap is represented as '//' in the x-axis of the plot.}

\item{xGapDiffNew}{Numeric vector of length 2 with new range
of the \code{xGap}. If not specified, the minimum difference between
consecutive x elements in the data is used.}

\item{meanVar}{String, variable of \code{data} with the mean variable.}

\item{seVar}{String, variable of \code{data} with the standard error.}

\item{minVar, maxVar}{String, variables of \code{data} with minimum and
maximum value for error bar. 
If both are specified, \code{seVar} is ignored.}

\item{yLab}{String with label for the y-axis.
If different labels should be used for different elements of
\code{byVar} variable, the vector should be named
with each corresponding element (collapsed with '.' if multiple).}

\item{facetVar}{Character vector, variable(s) of \code{data} for facetting.}

\item{facetScale}{String with type of scale used for facetting, 'free_y' by default
(fixed scale in the x-axis and free in the y-axis).}

\item{colorVar}{String, variable of \code{data} for coloring.}

\item{colorLab}{String, label for \code{colorVar}, used in the legend.}

\item{colorPalette}{(named) Vector with color palette.}

\item{labelVars}{Named string with variable labels (names are the variable code).}

\item{useLinetype}{Logical, if TRUE (FALSE by default) use also linetype
to differenciate the variable specified via \code{colorVar} in the mean line.}

\item{linetypePalette}{Vector with linetype(s), in case \code{useLinetype} is TRUE.}

\item{useShape}{Logical, if TRUE (by default) \code{colorVar} is also used for the shape.}

\item{shapePalette}{Named vector with shape palette for \code{colorVar}.}

\item{jitter}{Numeric with jitter for the x-axis, only used if \code{colorVar} specified.}

\item{title}{String with title for the plot.
If different labels should be used for different elements of
\code{byVar} variable, the vector should be named
with each corresponding element (collapsed with '.' if multiple).}

\item{caption}{String with caption for the plot,
NULL by default.}

\item{yTrans}{(optional) String with transformation for the y-axis.
Currently only 'log10' (or NULL, default) is available.
In case error bars go in the negative, their values are set to a 'small enough' value for plotting:
\code{min(data)/10} or \code{yLim[1]} if \code{yLim} is specified.}

\item{yLim}{Vector of the length 2 with limits for the y-axis.}

\item{xLim}{Vector of the length 2 with limits for the x-axis.}

\item{yAxisExpand}{Expansion constants for the limits for the y-axis.
See the documentation of the \code{expand} parameter of the 
\code{\link[ggplot2]{scale_y_continuous}} function
for the available values for this parameter.}

\item{yLimExpand}{This parameter is deprecated, use \code{yAxisExpand}
instead.}

\item{xAxisLabs}{(optional) Named character vector with labels for the x-axis.}

\item{sizePoint}{Size for the point.}

\item{sizeLine}{Size for the line linking means and error bars.}

\item{sizeLabel}{Size for the label, only used if \code{label} is not NULL.}

\item{widthErrorBar}{Numeric vector of length 1 with width of error bar.}

\item{tableText}{(optional) Character vector with colname of \code{data}
or expression from colnames of \code{data} to be represented in
the table below the plot.
By default, no table is displayed.}

\item{tableTextFontface}{Font face for the text included
in the table.}

\item{tableHeight}{Numeric of length 1 with height for the table.}

\item{tableYAxisLabs}{Logical, if TRUE (by default)
the labels of the \code{colorVar} are included
in the y-axis of the table.}

\item{tablePlotMargin}{Margin between the plot and the table,
expressed as \code{\link[grid]{unit}}, 0 by default.}

\item{label}{Logical or expression or list of expression.
Points are labelled with \code{meanVar} if set to TRUE,
or with the specified expression if \code{label} is an expression.
If a list is specified, 'textLabel' (required) 
should contain expression to extract label, 
and 'textHjust' and 'textVjust' (optional) may contain expression 
specifying horizontal and vertical adjustment of the label.}

\item{labelPadding}{Amount of padding (space) between each point
and its \code{label}, 1.5 lines by default.
See parameter \code{point.padding} of the
\code{\link[ggrepel]{geom_text_repel}} function.}

\item{byVar}{Variable(s) of \code{data} for which separated plot(s)
should be created.}

\item{hLine}{(optional) numeric with y-intercept of line(s) to be added.
If different thresholds should be used for different elements of the 
\code{byVar} or \code{facetVar} variables, the vector should be named
with each corresponding element (collapsed with '.' if multiple).}

\item{hLineColor}{String with color for \code{hLine}, 'black' by default.}

\item{hLineLty}{String with linetype for \code{hLine}, 'solid' by default.}

\item{vLine}{(optional) numeric with x-intercept of line(s) to be added.
If different thresholds should be used for different elements of the 
\code{byVar} or \code{facetVar} variables, the vector should be named
with each corresponding element (collapsed with '.' if multiple).}

\item{vLineColor}{String with color for \code{vLine}, 'black' by default.}

\item{vLineLty}{String with linetype for \code{vLine}, 'solid' by defaul}

\item{style}{String with subject profile style.
This affects the parameters: \code{fontname}, \code{fontsize}
and \code{themeFct}.}

\item{fontname}{String with font name,
by default 'Times' if \code{style} is 'report' and
'Tahoma' if \code{style} is 'presentation'.}

\item{fontsize}{Numeric vector of length 1 with font size,
by default 8 if \code{style} is 'report' and
10 if \code{style} is 'presentation'}

\item{themeFct}{Function with ggplot2 theme,
by default \code{\link[ggplot2]{theme_classic}} if \code{style} is 'report' and
\code{\link[ggplot2]{theme_bw}} if \code{style} is 'presentation'.}

\item{themeIncludeVerticalGrid}{Logical, if TRUE (by default)
include theme vertical grid lines (if present in \code{themeFct}).}

\item{ggExtra}{Extra \code{ggplot} call of function generating such call(s)
to be added in main plot.\cr
Multiple calls are set via a function, e.g. 
\code{ggExtra = function(gg){gg + geom_vline(...) + geom_hline(...)}}.\cr
If different calls should be used for different elements of the 
\code{byVar} variable, the vector should be named
with each corresponding element (collapsed with '.' if multiple).}

\item{legendPosition}{String with legend position.
By default, 'bottom' of \code{tableText} is not specified,
'none' otherwise.}

\item{...}{Additional parameters for \code{\link[ggrepel]{geom_text_repel}} or
\code{\link[ggplot2]{geom_text}}
used for the \code{label}.}
}
\value{
\code{\link[ggplot2]{ggplot}} object or list of such
objects of \code{byVar} is specified.
}
\description{
The user can either specify a variable for the standard error
(\code{seVar}),
or directly the variables for the minimum and maximum values for the error
bars (\code{minVar}, \code{maxVar}).
}
\author{
Laure Cougnaud
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stats-getStats.R
\name{getStats}
\alias{getStats}
\title{Get default set of statistics for one particular variable.}
\usage{
getStats(
  type = "summary",
  includeName = TRUE,
  x = NULL,
  nDecCont = getMaxNDecimals,
  nDecN = 0,
  nDecm = nDecN,
  formatPercentage = inTextSummaryTable::formatPercentage
)
}
\arguments{
\item{type}{Character vector with type of statistics (multiple are possible).
Available statistics are specified in the section 'Formatted statistics' and
formatting in 'Statistics formatting' in
\code{\link[=inTextSummaryTable-stats]{in-text table statistics}}.}

\item{includeName}{Logical, should the statistics name be included (TRUE by default)?
This is applied for the statistic names used in each for the set defined in \code{type};
and for the label of the list if \code{type} is of length 2.
If there are multiple \code{type} or statistics within a set, the names are retained (to avoid confusion).}

\item{x}{(optional, recommended for continuous variable) Numeric vector 
for which the statistics should be computed on.\cr
This is used to derive the number of decimals to include
for a continuous variable.\cr
If not specified, the values are rounded with \code{\link{formatC}}.}

\item{nDecCont}{Integer with base number of decimals 
for continuous variable, or function returning this number based on \code{x} 
(\code{\link{getNDecimals}} by default).}

\item{nDecN, nDecm}{Integer with number of decimals 
for number of subjects/records (0 by default).}

\item{formatPercentage}{Function used to format the percentages
(see \code{\link{formatPercentage}} for default behaviour).}
}
\value{
Expression (or call object) containing
function to extract requested summary statistics.
If multiple \code{type} are specified, they are combined to a list.
Names of the list will be typically used to name the statistic
in the summary table.
}
\description{
This set of statistics can be passed directly to the \code{stats} parameter
of the package functions.
}
\examples{
## default set of statistics are available for:

# for count table:
getStats("count")
getStats("n (\%)")
getStats("n")
getStats("\%")
getStats("m")
getStats("\%m")
getStats("m (\%)")
# for continuous variable:
getStats("summary")
getStats("mean (se)")
getStats("mean (sd)")
getStats("median (range)")
getStats("median\n(range)")
getStats(c("Mean", "SE"))
# for continuous variable with log-normal distribution:
getStats(c("Geometric Mean", "Geometric SD"))
getStats("geometric mean (geometric sd)")

## to not include statistic name in the table
getStats("median\n(range)", includeName = FALSE)
getStats(c("summary", "median\n(range)"), includeName = FALSE)

## to extract the number of decimals based on a continuous variable (see ?getMaxNDecimals) 
exampleData <- data.frame(
  USUBJID = 1 : 4, 
  WEIGHT = c(67, 78, 83, 61), 
  SEX = c("F", "M", "M", "F"), 
  stringsAsFactors = FALSE
)
getStats(type = c('median (range)', 'mean (se)'), x = exampleData$WEIGHT)
# compare with when 'x' is not specified:
getStats(type = c('median (range)', 'mean (se)'))

## custom function to format the percentages:
getStats(type = "count", formatPercentage = function(x) round(x, 2))
}
\seealso{
\link{getStatsData}
}
\author{
Laure Cougnaud
}
